/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2018/01/11 $
 * $RCSfile: dmx_debug_filter_log.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file dmx_debug_utils.h
 *  Demux driver - dmx_debug_utils.h
 */

#ifndef DMX_DEBUG_UTILS_H
#define DMX_DEBUG_UTILS_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#ifndef CC_DISABLE_DMX_FILTER_LOG

#include "dmx_debug.h"

typedef enum
{
    LOG_CATEGORY_INIT        = (1U << 0),
    LOG_CATEGORY_PID         = (1U << 1),
    LOG_CATEGORY_BUFFER      = (1U << 2),
    LOG_CATEGORY_PSI         = (1U << 3),
    LOG_CATEGORY_PES         = (1U << 4),
    LOG_CATEGORY_CALLBACK    = (1U << 5),
    LOG_CATEGORY_UP          = (1U << 6),
    LOG_CATEGORY_PATTERN     = (1U << 7),
    LOG_CATEGORY_PCR         = (1U << 8),
    LOG_CATEGORY_RECORD      = (1U << 9),
    LOG_CATEGORY_PVRPLAY     = (1U << 10),
    LOG_CATEGORY_SCRAMBLE    = (1U << 11),
    LOG_CATEGORY_DDI         = (1U << 12),
    LOG_CATEGORY_MUL         = (1U << 13),
    LOG_CATEGORY_AV          = (1U << 14),
    LOG_CATEGORY_AUD_PREBUF  = (1U << 15),
    LOG_CATEGORY_PID_FLOW    = (1U << 16),
    LOG_CATEGORY_ISR         = (1U << 17),
    LOG_CATEGORY_FUNC_TRACE  = (1U << 18),
    LOG_CATEGORY_DDI_MOVE    = (1U << 19),
    LOG_CATEGORY_MAX,
}DMX_LOG_CATEGORY_T;

typedef enum
{
    LOG_MODULE_GLOBAL   = 0x0,
    LOG_MODULE_DMX_INST = 0x1,
    LOG_MODULE_PIDX     = 0x2,
    LOG_MODULE_SECIDX   = 0x3,
    LOG_MODULE_MAX_NUM,
}DMX_LOG_MODULE_T;

EXTERN VOID DMX_Utils_SetLogFilterSingleCategory(const DMX_LOG_MODULE_T rModule,
                                                 const UINT8 u1SigFilter,
                                                 const BOOL fgEnable);
EXTERN VOID DMX_Utils_SetLogFilterAllMask(const DMX_LOG_MODULE_T rModule, const UINT64 u8Mask);
EXTERN VOID DMX_Utils_PrintFilterSettingsStatus(VOID);
EXTERN const UINT64 DMX_Utils_GetLogFilterMaskVal(const DMX_LOG_MODULE_T rModule);

#define DMX_LOG_FILTER_LSHIFT(u1bitsCnt)  (u1bitsCnt < 64 ? (((UINT64)1)<<(UINT8)(u1bitsCnt)) : 0)
#define DMX_LOG_FILTER_TEST_BIT(u8Mask, u8BitVal) (((u8Mask) & (u8BitVal)) != 0)

#define DFLOG(category, fmt, ...)                                                   \
do {                                                                                \
    if (DMX_LOG_FILTER_TEST_BIT(DMX_Utils_GetLogFilterMaskVal(LOG_MODULE_GLOBAL),   \
                                ((UINT64)category)))                                \
    {                                                                               \
        LOG(1, (fmt), ##__VA_ARGS__);                                               \
    }                                                                               \
} while (0)

#define DPIDLOG(category, pidx, fmt, ...)                                          \
do {                                                                               \
    if (DMX_LOG_FILTER_TEST_BIT(DMX_Utils_GetLogFilterMaskVal(LOG_MODULE_PIDX),    \
                                (DMX_LOG_FILTER_LSHIFT(pidx))))                    \
    {                                                                              \
        DFLOG((category), ("[Pidx-%d] "fmt), pidx, ##__VA_ARGS__);                 \
    }                                                                              \
}while (0)

#define DINSTLOG(category, InstId, fmt, ...)                                            \
do {                                                                                    \
    if (DMX_LOG_FILTER_TEST_BIT(DMX_Utils_GetLogFilterMaskVal(LOG_MODULE_DMX_INST),     \
                                (DMX_LOG_FILTER_LSHIFT(InstId))))                       \
    {                                                                                   \
        DFLOG((category), ("[Inst-%d] "fmt), InstId, ##__VA_ARGS__);                    \
    }                                                                                   \
}while (0)

#define DMXGFLOG(filter, fmt, ...)                                                \
do {                                                                              \
    if (DMX_LOG_FILTER_TEST_BIT(DMX_Utils_GetLogFilterMaskVal(LOG_MODULE_GLOBAL), \
                                ((UINT64)filter)))                                \
    {                                                                             \
        LOG(2, (fmt), ##__VA_ARGS__);                                             \
    }                                                                             \
} while (0)

#else

#define DFLOG(category, fmt, ...)
#define DPIDLOG(category, pidx, fmt, ...)
#define DINSTLOG(category, InstId, fmt, ...)
#define DMXGFLOG(filter, fmt, ...)

#endif /* #ifndef CC_DISABLE_DMX_FILTER_LOG */

#endif	// DMX_DEBUG_UTILS_H

