#/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
#/******************************************************************************
#*
#* This file is provided under a dual license. When you use or
#* distribute this software, you may choose to be licensed under
#* version 2.1 of the GNU Lesser General Public License
#* ("LGPLv2.1 License") or BSD License.
#*
#* LGPLv2.1 License
#*
#* Copyright(C) 2019 MediaTek Inc.
#*
#* This program is free software; you can redistribute it and/or modify
#* it under the terms of version 2.1 of the GNU Lesser General Public
#* License as published by the Free Software Foundation.
#*
#* This program is distributed in the hope that it will be useful, but
#* WITHOUT ANY WARRANTY; without even the implied warranty of
#* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#* See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
#* details.
#*
#* BSD LICENSE
#*
#* Copyright(C) 2019 MediaTek Inc.
#* All rights reserved.
#*
#* Redistribution and use in source and binary forms, with or without
#* modification, are permitted provided that the following conditions
#* are met:
#*
#* * Redistributions of source code must retain the above copyright
#* notice, this list of conditions and the following disclaimer.
#* * Redistributions in binary form must reproduce the above copyright
#* notice, this list of conditions and the following disclaimer in
#* the documentation and/or other materials provided with the
#* distribution.
#* * Neither the name of the copyright holder nor the names of its
#* contributors may be used to endorse or promote products derived
#* from this software without specific prior written permission.
#*
#* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#* HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#*
#*****************************************************************************/
#

ifndef UBOOT_ROOT
export UBOOT_ROOT=$(word 1, $(subst /uboot/,/uboot /, $(shell bash -c pwd -L)))
endif

ifndef CROSS_COMPILE
export CROSS_COMPILE = arm11_mtk_le-
endif

ifndef COMPANY
export COMPANY = mediatek
endif

ifndef MODEL
export MODEL = common
endif

ifndef DRV_CUSTOM
DRV_CUSTOM = $(MODEL)
endif

#ifndef BOOT_TYPE
#BOOT_TYPE = ROM2NAND
#endif

ifndef ROM_CODE
ROM_CODE = n
endif

ifneq ($(strip $(wildcard $(UBOOT_ROOT)/drv_lib/include/$(MODEL).def)),)
-include $(UBOOT_ROOT)/drv_lib/include/$(MODEL).def
endif

ifeq "$(BOOT_TYPE)" "ROM2NAND"
ROM_CODE = y
BOOT = nand
CPPFLAGS += -DCC_NAND_BOOT
CFLAGS += -DCC_NAND_BOOT
AFLAGS += -DCC_NAND_BOOT
endif

ifeq "$(BOOT_TYPE)" "NAND"
BOOT = nand
CPPFLAGS += -DCC_NAND_BOOT
CFLAGS += -DCC_NAND_BOOT
AFLAGS += -DCC_NAND_BOOT
endif


ifeq "$(BOOT_TYPE)" "NOR"
BOOT = nor
endif

ifeq "$(BOOT_TYPE)" "ROM2NOR"
ROM_CODE = y
BOOT = nor
endif

ifeq "$(BOOT_TYPE)" "ROM2EMMC"
ROM_CODE = y
BOOT = emmc
endif

ifeq "$(SECURE_BOOT)" "y"
export CC_SECURE_ROM_DEF = -DCC_SECURE_ROM_BOOT
endif

ifeq "$(SUPPORT_COMBINE_4K_2K)" "true"
CFLAGS += -DCC_COMBINE_4K_2K
endif

export CPPFLAGS CFLAGS AFLAGS 
export ROM_CODE BOOT
export UBOOT_LIBRARY = y

ifeq "$(BOARD)" "mt5392b"
export CC_LOADER_DEF = CC_5391_LOADER -DCC_5392B_LOADER -DCC_53xx_LOADER
export CC_PRELOADER_DEF = CC_5391_PRELOADER -DCC_5392B_PRELOADER
endif

ifeq "$(BOARD)" "mt5363"
export CC_LOADER_DEF = CC_MTK_LOADER
export CC_PRELOADER_DEF = CC_MTK_PRELOADER
endif

COMMON_INC =  -I$(ROOTDIR)/drv_lib/include \
              -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/drv_inc \
              -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/inc \
	             -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/drv_cust

ifdef UBOOT_INC_DIR
COMMON_INC += -I$(UBOOT_INC_DIR)
endif
COMMON_INC += -I$(INC_ROOT)/x_inc

OSAI_INC =     -I. \
               -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/nptv/inc \
               -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/nptv/inc/hw \
               -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/nptv/inc/drv \
               -I$(ROOTDIR)/drv_lib/$(TARGET_IC)/nptv/inc/sys \
               -D_CPU_LITTLE_ENDIAN_ -DCC_UBOOT \
               -DPANEL_TABLE_CUST_FILE=\"$(COMPANY)/panel_table_cust.c\" 

ifeq ($(UBOOT_VERSION), 1.3.4)
 OSAI_INC +=  -DCUSTOM_CFG_FILE=\"$(COMPANY)/config/$(DRV_CUSTOM).h\" 
endif
ifneq ($(strip $(wildcard $(UBOOT_ROOT)/drv_lib/include/$(MODEL).def)),)
OSAI_INC += $(DEFINES)
endif

ifeq "$(BOARD)" "mt5392b"
OSAI_INC += -DCC_MT5391 -DCC_MT5392B
endif

ifeq "$(BOARD)" "mt5363"
OSAI_INC += -DCC_MT5363
endif

export COMMON_INC OSAI_INC

AS	?= $(CROSS_COMPILE)as
LD	?= $(CROSS_COMPILE)ld
CC	?= $(CROSS_COMPILE)gcc
AR	?= $(CROSS_COMPILE)ar
OBJCOPY ?= $(CROSS_COMPILE)objcopy

export AS LD CC AR OBJCOPY
