/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date  $
 * $RCSfile: cec_drv_if.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

#ifndef CEC_DRV_IF
#define CEC_DRV_IF

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#include "x_lint.h"

LINT_EXT_HEADER_BEGIN

#include "hw_vdoin.h"
#ifdef CC_MT5895
#include "cec2_if.h"
#else
#include "cec_if.h"
#endif

#include "hw_ckgen.h"
#include "api_eep.h"
#include "eepdef.h"
#include "eeprom_if.h"
#include "drvcust_if.h"
#include "pdwnc_if.h"

#include "x_os.h"
#include "x_printf.h"
#include "x_assert.h"
#include "x_pinmux.h"
#include "x_pdwnc.h"
#include "x_bim.h"
#include "x_timer.h"

/// define CEC module debug log function.

#include "x_debug.h"

LINT_EXT_HEADER_END
//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------

#ifdef CC_DRIVER_PROGRAM
#define CTS_TEST
#else
#define NOTIFY_CECMW
#endif

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------
#define RETX_MAX_CNT 2

#define CEC_LOG_BROADCAST CEC_LOG_UNREGISTERED

#define CEC_SUCCESS (0)

#define CEC_ERROR (1)


#define CEC_ACK 0
#define CEC_NACK 1
#define CEC_BUSY 2


typedef enum {
    HDMI_CONNECT_PORT_INIT = 0,
    HDMI_CONNECT_PORT_1,
    HDMI_CONNECT_PORT_2,
    HDMI_CONNECT_PORT_3,
    HDMI_CONNECT_PORT_4,
    HDMI_CONNECT_PORT_MAX
}HDMI_CONNECT_PORT_NUM;



#define DISABLE_ALL_TX_INT() \
{ \
    ENABLE_INT_FAIL(0); \
    ENABLE_INT_RB(0); \
    ENABLE_INT_LOW(0); \
    ENABLE_INT_UN(0); \
    ENABLE_INT_BS(0); \
}

#define SetCECStatus(arg) { \
    CRIT_STATE_T rCritState; \
    rCritState = x_crit_start(); \
    (_CEC_Status |= ((UINT16)arg)); \
    x_crit_end(rCritState); }

#define ClrCECStatus(arg) { \
    CRIT_STATE_T rCritState; \
    rCritState = x_crit_start(); \
    (_CEC_Status &= (~((UINT16)arg))); \
    x_crit_end(rCritState); }

#define IsCECStatus(arg)     ((_CEC_Status & ((UINT16)arg)) > 0)

#define CLR_CEC_BUSY_STATUS() (_u2CecBusyStatus = 0)
#define CEC_BUSY_STATUS() _u2CecBusyStatus

#define CEC_IS_BUSY() (_u2CecBusyStatus)

#define CEC_RX_IS_BUSY() (_u2CecBusyStatus & CEC_RX_BUSY)

#define SET_CECRX_BUSY()\
     do {\
         _u2CecBusyStatus |= CEC_RX_BUSY;\
         LOG(7, "RX busy ----->>\n");\
        }while(0)

#define CLR_CECRX_BUSY()\
     do {\
         _u2CecBusyStatus &= (~CEC_RX_BUSY);\
         LOG(7, "Clr RX busy <<-----\n");\
        }while(0)

#define CEC_TX_IS_BUSY() (_u2CecBusyStatus & CEC_TX_BUSY)

#define SET_CECTX_BUSY()\
     do {\
         _u2CecBusyStatus |= CEC_TX_BUSY;\
         LOG(7, "TX busy ----->>\n");\
        }while(0)

#define CLR_CECTX_BUSY()\
     do {\
         if (CEC_TX_IS_BUSY())\
         LOG(7, "Clr TX busy <<-----\n");\
         _u2CecBusyStatus &= (~CEC_TX_BUSY);\
        }while(0)

#define CEC_TX_FRAME (TRUE)
#define CEC_RX_FRAME (FALSE)
//---------------------------------------------------------------------------
// Inter-file variable
//---------------------------------------------------------------------------
EXTERN UINT16 _CEC_Status;

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------
EXTERN void CEC_PrintFrame(const CEC_FRAME_DESCRIPTION *pFrame, BOOL fgTxFrame);

void vCEC_Mainloop(void);
//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

#endif /* CEC_DRV_IF */

