/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


//  MTK Notice: Max Liao, 2006/12/04.
//  add CDC header file.

/*
 * MUSBStack-S Communication Device Class (CDC) definitions.
 * $Revision$
 */

#ifndef __MUSB_CDC_IF_H__
#define __MUSB_CDC_IF_H__

/**
 * Transfer completion callback.
 * Read and write are asynchronous requests, so the implementation
 * supplies this callback to be notified on their completion.
 */
typedef void (*MUSB_pfCdcTransferComplete)(UINT32 dwActualLength);

//-------------------------------------------------------------------------
/** MUSB_CdcInit
 *  USB device CDC mode initialization function.
 *  @param  void                    user's buffer.
 *  @retval  void.
 */
//-------------------------------------------------------------------------
extern void MUSB_CdcInit(void);

//-------------------------------------------------------------------------
/** MUSB_CdcConfigOK
 *  USB device CDC mode configuration function.
 *  @param  void                    user's buffer.
 *  @retval  0  Success.
 *  @retval  -1	FAIL. 
 */
//-------------------------------------------------------------------------
extern INT32 MUSB_CdcConfigOK(void);

//-------------------------------------------------------------------------
/** MUSB_CdcSyncTx
 *  USB device CDC mode sync tx function.
 *  @param  pBuf                    user's buffer.
 *  @param  dwCount             user's buffer size.
 *  @retval  >0 Actual Tx data length.
 *  @retval  -1	FAIL. 
 */
//-------------------------------------------------------------------------
extern INT32 MUSB_CdcSyncTx(UINT8 *pBuf, UINT32 dwCount);

//-------------------------------------------------------------------------
/** MUSB_CdcAsyncTx
 *  USB device CDC mode async tx function.
 *  @param  pBuf                    user's buffer.
 *  @param  dwCount             user's buffer size.
 *  @param  pfXferComplete     user's call back function. 
 *  @retval  0 	SUCCESS.
 *  @retval  -1	FAIL. 
 */
//-------------------------------------------------------------------------
extern INT32 MUSB_CdcAsyncTx(UINT8 *pBuf, UINT32 dwCount, 
    MUSB_pfCdcTransferComplete pfXferComplete);

//-------------------------------------------------------------------------
/** MUSB_CdcSyncRx
 *  USB device CDC mode sync rx function.
 *  @param  pBuf                    user's buffer.
 *  @param  dwCount             user's buffer size.
 *  @retval  >0 Actual Rx data length.
 *  @retval  -1	FAIL. 
 */
//-------------------------------------------------------------------------
extern INT32 MUSB_CdcSyncRx(UINT8 *pBuf, UINT32 dwCount);

//-------------------------------------------------------------------------
/** MUSB_CdcAsyncRx
 *  USB device CDC mode async rx function.
 *  @param  pBuf                    user's buffer.
 *  @param  dwCount             user's buffer size.
 *  @param  pfXferComplete     user's call back function.
 *  @retval  0 	SUCCESS.
 *  @retval  -1	FAIL. 
 */
//-------------------------------------------------------------------------
extern INT32 MUSB_CdcAsyncRx(UINT8 *pBuf, UINT32 dwCount, 
    MUSB_pfCdcTransferComplete pfXferComplete);

#endif	/* multiple inclusion protection */
