/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: sif_sw_if.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file spi_comm_if.h
 *  This file declare the public function prototype in sif_sw_if.c.
 */

#ifndef SPI_COMM_IF_H
#define SPI_COMM_IF_H


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#include "x_os.h"
#include "x_typedef.h"
//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

//#define SPI_PERFORMACE_TEST_BY_HWTIMER


#define GSPI_CPHA		(0x01)
#define GSPI_CPOL		(0x02)
#define GSPI_MODE_0		(0|0)
#define GSPI_MODE_1		(0|GSPI_CPHA)
#define GSPI_MODE_2		(GSPI_CPOL|0)
#define GSPI_MODE_3		(GSPI_CPOL|GSPI_CPHA)
#define GSPI_LSB_FIRST	(0x08)


#define SPI_WRITE 		(1U << 1)
#define SPI_READ 		(1U << 0)
#define GSPI_WRITE		SPI_WRITE
#define GSPI_READ		SPI_READ
#define ONE_PACK        0x01
#define FIRST_PACK      0x11
#define MID_PACK        0x12
#define LAST_PACK       0x02

//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------
typedef struct
{
    UINT32 u4Mode;
	UINT32 uBits;
	UINT32 u4Freq;
    UINT32 u4CsLead; //The latency between CS low and valid DATA output.
	UINT32 u4CsLag;  //The latency between valid DATA output and CS high.
	UINT32 u4CsHold; //The output CS hold time.
} PDWNC_GSPI_MODE_T;


typedef struct
{
    UINT32 WrRd;
	UINT32 DataLen;
	UINT32 *RdBuf;
	UINT32 *WrBuf;
} SPIPACK_T;

typedef struct{
	UINT32 u4MorL;
	UINT32 u4CS;
	UINT32 u4Mod;
	UINT32 u4EorL;
	UINT32 u4Clk;
}PREPH_SPI_CONF; 



//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

EXTERN INT32 PDWNC_GspiInit(PDWNC_GSPI_MODE_T *prGspiMode);
EXTERN INT32 PDWNC_GspiReadWrite(UINT32 fgReadWrite, UINT32 u4Nob,UINT8 *u1RdBuf, UINT8 *u1WrBuf);

EXTERN void SpiClkInit(UINT32 u4Clk);
EXTERN void SpiInit(PREPH_SPI_CONF *SpiInitCon);
EXTERN INT32 SpiReadWriteMultiPack(UINT32 u4PackLen, SPIPACK_T *PackBuf, PREPH_SPI_CONF *SpiInitCon);

#endif /* SPI_COMM_IF_H */
