/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


//lint -emacro(534,LOG)

//lint -save
//lint --e{950} No ANSI reserved word (__attribute__)
//lint -e960
//lint -e961

#ifndef X_DRV_CLI_H
#define X_DRV_CLI_H

#include "x_typedef.h"
#include "drv_common.h"

/******************************************************************************
* cli command respond value
******************************************************************************/
#define CLI_COMMAND_OK					0
#define CLI_UNKNOWN_CMD					-2147483647

#define CLI_MAIN_COMMAND_ITEM(x)                \
    extern const CLI_EXEC_T DATACLI_##x;        \
    CLI_EXEC_T *CLI_##x##_function(void)        \
    {                                           \
        return (CLI_EXEC_T *) &DATACLI_##x;     \
    }                                           \
    const CLI_EXEC_T DATACLI_##x __attribute__ ((section("data.cli"))) =

#define CLI_MAIN_COMMAND                __attribute__ ((section("data.cli")))
#define CLI_LAST_COMMAND                __attribute__ ((section("end.cli")))

#define CLI_NOT_IN_MINI

/******************************************************************************
* cli command access right
******************************************************************************/
typedef enum
{
	CLI_SUPERVISOR = 0,
	CLI_ADMIN,
	CLI_GUEST,
	CLI_HIDDEN
} CLI_ACCESS_RIGHT_T;


/******************************************************************************
* cli command structure
******************************************************************************/
typedef struct _CLI_EXEC
{
	CHAR*				pszCmdStr;													// command string
	CHAR*				pszCmdAbbrStr;												// command abbreviation
	INT32				(*pfExecFun) (INT32 i4Argc, const CHAR ** szArgv);			// execution function
	struct _CLI_EXEC	*prCmdNextLevel;											// next level command table
	CHAR*				pszCmdHelpStr;												// command description string
	CLI_ACCESS_RIGHT_T	eAccessRight;												// command access right
} CLI_EXEC_T;


/******************************************************************************
* cli command table get function
******************************************************************************/
typedef CLI_EXEC_T* (*CLI_GET_CMD_TBL_FUNC)(void);
typedef void (* VER_FUNC)(void);

/******************************************************************************
* cli console API
******************************************************************************/
extern void CLI_Init(void);
extern void CLI_Input(void* pvArg);
#ifdef CC_MINI_DRIVER
#define CLI_SetPrompt(szPrompt)
#else
extern void CLI_SetPrompt(const CHAR* szPrompt);
#endif /* CC_MINI_DRIVER */
extern INT32 CLI_CmdTblAttach(CLI_EXEC_T* pTbl);
extern INT32 CLI_CmdTblDeattach(CLI_EXEC_T* pTbl);
extern INT32 CLI_AliasAttach(const CHAR* szAlias, const CHAR* szString);
extern CLI_EXEC_T* CLI_GetDefaultCmdTbl(void);
extern INT32 CLI_PromptParser(const CHAR *pcCmdBuf);
extern INT32 CLI_Parser(const CHAR *pcCmdBuf);
extern INT32 CLI_Parser_Arg(const CHAR* szCmd, ...);
extern INT32 CLI_SetCliMode(CLI_ACCESS_RIGHT_T eCliAccess);
extern VER_FUNC DRVCLI_SetVerFunc(VER_FUNC pfnVer);


#endif /* X_DRV_CLI_H */

//lint -restore

