/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/

#ifndef X_HAL_REG_H
#define X_HAL_REG_H

//===========================================================================
// Memory mapping

#define DRAM_B_BASE                 0x0
#define PBI_B_BASE                  0x00000000
#define IO_BASE                     0x10000000
#define IO_PHYS                     0x10000000

/*
 * IO_VIRT is a virtual address to bus address view instead of cpu virt.
 */
#define IO_VIRT                     0x10000000
#define IO_VIRT_SRE                 0x12000000
// 0x8000 SRAM reserve for 8032 at 0x20176000-0x2018000
#define IO_VIRT_PDWNC_RESERVED      0xA000
#define IO_VIRT_PDWNC_SRAM          0x20000000
#define IO_VIRT_PDWNC               0x20180000

#define IO_VIRT_MAX                 (IO_VIRT_PDWNC_SRAM + 0x10000000)

#define PBI_A_BASE                  0x07000000
#define DRAM_A_BASE                 0x40000000

#ifdef CC_DRAM_START_ADDR
#define DRAM_START (CC_DRAM_START_ADDR)
#else
#define DRAM_START (0x00)
#endif
#define BROM_BASE                   (0x06000000)

#define SECURE_IO_START             (BROM_BASE)
#define SECURE_IO_END               (0x14000000)

#define SRAM_DMX_BASE               (0x04100000UL)
#define SRAM_DMX_SIZE               (0x00016000UL)                      //DMX=88KB
#define SRAM_DMX_END                (SRAM_DMX_BASE + SRAM_DMX_SIZE)
#define SRAM_OSD_BASE               (SRAM_DMX_END)
#define SRAM_OSD_SIZE               (0x00010000UL)                      //OSD=64KB
#define SRAM_LZHS_BASE              (SRAM_OSD_BASE + SRAM_OSD_SIZE)     //0x04126000 = SRAM_LZHS_VIRT
#define SRAM_LZHS_SIZE              (0x00001000UL)                      //LZHS=4KB
#define SRAM_PDWNC_BASE             (SRAM_LZHS_BASE + SRAM_LZHS_SIZE)   //0x04127000
#define SRAM_PDWNC_SIZE             (0x00004000UL)                      //PDWNC=16KB
#define SRAM_HOTBOOT_BASE           (0x20176000)
#define SRAM_HOTBOOT_SIZE           (0x0000a000UL)                      //HOTBOOT=40KB(Share cm4 sram)
#define SRAM_HOTBOOT_LOAD_DATA_BASE 0x2017f800
#define SRAM_HOTBOOT_LINK_DATA_BASE 0x0412a800
#define SRAM_HOTBOOT_DATA_SIZE      0x800
#define SRAM_SECURE_BASE            (SRAM_LZHS_BASE + SRAM_LZHS_SIZE + 0x00004000UL)
#define SRAM_SECURE_SIZE            (0x00000200UL)                      //SECURE=0.5KB

#define SRAM_ITCM_BASE              SRAM_LZHS_BASE //itcm=0k
#define SRAM_LZHS_VIRT              (0x04126000)

#define SRAM_START                  SRAM_DMX_BASE
#define SRAM_END                    (SRAM_SECURE_BASE + SRAM_SECURE_SIZE)
#ifdef CC_64BIT_SUPPORT
#define SRAM_CORE1_SIZE             0x200
#else
#define SRAM_CORE1_SIZE             0x100
#endif // CC_64BIT_SUPPORT
#define SRAM_CORE1_BASE             (0x04114000)

#ifdef CC_FPGA_SDR64B
#define FPGA_DRAM_SIZE_MB           128
#else
#define FPGA_DRAM_SIZE_MB           256
#endif

#define IO_ADDR(base)               (IO_VIRT + base)

//===========================================================================
// IO register definitions
#ifndef CC_MT5391_AUD_SUPPORT
#define PARSER0_BASE                (IO_VIRT + 0x00000)
#endif
#define GDMA_BASE                   (IO_VIRT + 0x01000)
#if defined(CC_MT5890) || defined(CC_MT5891)
#define VLD0_BASE                   (IO_VIRT + 0x80000)
#define VLD1_BASE                   (IO_VIRT + 0x90000)
#define MC0_BASE                    (IO_VIRT + 0x81000)
#define MC1_BASE                    (IO_VIRT + 0x91000)
#define DV_BASE                     (IO_VIRT + 0x85000)
#else
#define VLD0_BASE                   (IO_VIRT + 0x02000)
#define VLD1_BASE                   (IO_VIRT + 0x03000)
#define MC0_BASE                    (IO_VIRT + 0x0a000)
#define MC1_BASE                    (IO_VIRT + 0x0b000)
#define DV_BASE                     (IO_VIRT + 0x02200)
#endif
#define GRAPH_BASE                  (IO_VIRT + 0x04000)
#define AUDIO_BASE                  (IO_VIRT + 0x05000)

#if defined(CC_MT5398) || defined(CC_MT5399) || defined(CC_MT5890) || defined(CC_MT5891) || defined(CC_MT5893) || defined(CC_MT5895)
#define VGRAPH_BASE                 (IO_VIRT + 0x5c000)
#define VGRAPH_CMDQUE_BASE          (IO_VIRT + 0x5000)

#define GRAPH_MMU_BASE              (IO_VIRT + 0x68000)   //graphic mmu
#define GRAPH_MMU_CMD_BASE          (IO_VIRT + 0x4180)   //graphic mmu cmdque mode,  risc 0x68000  = cmdque 0x4180

//IMGRZ_HW_V4
#define IMGRZ_MMU_BASE			 	(IO_VIRT + 0x680c0)
#define IMGRZ_MMU_BASE_2	 	 	(IO_VIRT + 0x68140)
#define JPEG_MMU_BASE			 	(IO_VIRT + 0x68080)
#endif
#if defined(CC_MT5886)
#define GDMA_MMU_BASE			 	(IO_VIRT + 0x68100)
#elif defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
#define GDMA_MMU_BASE			 	(IO_VIRT + 0x68200)

#endif

#ifdef  CC_MT5391_AUD_SUPPORT
#define PARSER0_BASE                (AUDIO_BASE + 0x00700) //added by ling
#define PARSER0_BASE2               (AUDIO_BASE + 0x00600) //added by ling
#endif
#if defined(CC_MT5893) || defined(CC_MT5895)
#define DRAM_BASE                   (IO_VIRT + 0x202000)
#define DRAM_CHB_BASE               (IO_VIRT + 0x20A000)
#define DRAM_CHC_BASE               (IO_VIRT + 0x212000)
#define DRAM_CHD_BASE               (IO_VIRT + 0x21A000)

#define DRAM_DDRPHY_CHA_BANK        (IO_VIRT + 0x200000)
#define DRAM_DDRPHY_CHB_BANK        (IO_VIRT + 0x208000)
#define DRAM_DDRPHY_CHC_BANK        (IO_VIRT + 0x210000)
#define DRAM_DDRPHY_CHD_BANK        (IO_VIRT + 0x218000)

#define DRAM_DMARB_BASE             (IO_VIRT + 0x06000)
#define DRAM_DMARB_TZ_BASE          (IO_VIRT + 0x37000)

#define DRAMC0_NAO_BASE             (IO_VIRT + 0x204000)
#define DRAMC1_NAO_BASE             (IO_VIRT + 0x20C000)
#define DRAMC2_NAO_BASE             (IO_VIRT + 0x214000)
#define DRAMC3_NAO_BASE             (IO_VIRT + 0x21C000)
#else
#define DRAM_DMARB_BASE             (IO_VIRT + 0x06000)     // obsolete     // obsolete
#define DRAM_BASE                   (IO_VIRT + 0x07000)
#define DRAM_CHB_BASE               (IO_VIRT + 0x0F000)
#define DRAM_CHC_BASE               (IO_VIRT + 0x15000)
#define DRAM_DDRPHY_CHA_BANK        (IO_VIRT + 0x7A000)
#define DRAM_DDRPHY_CHB_BANK        (IO_VIRT + 0x7B000)
#define DRAM_DDRPHY_CHC_BANK        (IO_VIRT + 0x7C000)
#endif
#define DRAM_DMARB_CHB_BASE			(IO_VIRT + 0x10000)
#define DRAM_DMARB_CHC_BASE         (IO_VIRT + 0x11000)
#define BIM_BASE                    (IO_VIRT + 0x08000)
#define BIM_PHY_BASE                (IO_BASE + 0x08000)
#define I1394_BASE                  (IO_VIRT + 0x09000)     // obsolete
#define RS232_BASE                  (IO_VIRT + 0x0c000)
#define CKGEN_BASE                  (IO_VIRT + 0x0d000)
#define MBIST_BASE                  (IO_VIRT + 0x0e000)
#define FLASH_CARD_BASE             (IO_VIRT + 0x12000)
#define SMART_CARD_BASE0            (IO_VIRT + 0x13000)
#define JPG_VLD_BASE                (IO_VIRT + 0x14000)     // obsolete
#define PCMCIA_BASE                 (IO_VIRT + 0x1F000)
#define AES_BASE                    (IO_VIRT + 0x16000)
#define SHA1_BASE                   (IO_VIRT + 0x16400)
#define DFAST_BASE                  (IO_VIRT + 0x16800)
#define DEMUX0_BASE                 (IO_VIRT + 0x17000)    //DEMUX0
#define DEMUX1_BASE                 (IO_VIRT + 0x18000)    //DEMUX1
#define DEMUX2_BASE                 (IO_VIRT + 0x19000)    //DEMUX2
#define DEMUX3_BASE                 (IO_VIRT + 0x1a000)    //DEMUX3
#define DEMUX4_BASE                 (IO_VIRT + 0x1b000)    //DEMUX4
#define DEMUX5_BASE                 (IO_VIRT + 0x1c000)    //DEMUX5
#define DEMUX6_BASE                 (IO_VIRT + 0x1d000)    //DEMUX6
#define DEMUX7_BASE                 (IO_VIRT + 0x54000)    //DEMUX7
#define DEMUX8_BASE                 (IO_VIRT + 0x55000)    //DEMUX8
#define DEMUX9_BASE                 (IO_VIRT + 0x6A000)    //DEMUX9
#define DEMUX10_BASE                (IO_VIRT + 0x3D000)    //DEMUX10
#define DEMUX11_BASE                (IO_VIRT + 0x57000)    //DEMUX11
#define DEMUX12_BASE                (IO_VIRT + 0x58000)    //DEMUX12
#define DEMUX13_BASE                (IO_VIRT + 0x5A000)    //DEMUX13
#define DEMUX14_BASE                (IO_VIRT + 0x5B000)    //DEMUX14
#define DEMUX15_BASE                (IO_VIRT + 0x5C000)    //DEMUX15
#define DEMUX16_BASE                (IO_VIRT + 0x56000)    //DEMUX16
#define DEMUX17_BASE                (IO_VIRT + 0x69000)    //DEMUX17


#define PIDSWAP_BASE                (IO_VIRT + 0x1e000)
#define IDE_TEST_BASE               (IO_VIRT + 0x1f000)
#define GFX_SCALAR_BASE             (IO_VIRT + 0x20000)
#define IMGRZ_BASE                  (IO_VIRT + 0x7b000)
#if defined(CC_MT5895)
#define OSD_BASE                    (IO_VIRT_SRE + 0x5000)
#else
#define OSD_BASE                    (IO_VIRT + 0x21000)
#endif
#define VIDEO_IN0_BASE              (IO_VIRT + 0x22000)
#define VIDEO_IN1_BASE              (IO_VIRT + 0x23000)
#define PSCAN_BASE                  (IO_VIRT + 0x24000)
#define SCPOS_BASE                  (IO_VIRT + 0x25000)
#define BLK2RS_BASE                 (IO_VIRT_SRE + 0x21000)
#define POST_PROC_BASE              (IO_VIRT + 0x27000)
#define CM4_CONFIG_BASE             (IO_VIRT_PDWNC)
#define PDWNC_BASE                  (IO_VIRT_PDWNC + 0x4000)
#define PDWNC_SECURE_BASE           (IO_VIRT_PDWNC + 0x8000)
#define VAD_BASE                    (IO_VIRT_PDWNC + 0x18000)
#define HPDS_BASE                   (IO_VIRT + 0x29000)
#define MUSB_BASE                   (IO_VIRT + 0x50000)
#define MUSB1_BASE                  (IO_VIRT + 0x51000)
#define MUSB2_BASE                  (IO_VIRT + 0x52000)
#if defined(CC_MT5891)
#define MUSB3_BASE                  (IO_VIRT + 0x52000)
#else
#define MUSB3_BASE                  (IO_VIRT + 0x53000)
#endif
#define MUSB_COREBASE            (0)
#define MUSB_DMABASE             (0x200)
#define MUSB_PHYBASE             (0x9800)
#define MUSB_PORT0_PHYBASE       (0)
#define MUSB_PORT1_PHYBASE       (0x100)
#if defined(CC_MT5891)
#define MUSB_PORT2_PHYBASE       (-0x6000)
#else
#define MUSB_PORT2_PHYBASE       (0x200)
#endif
#if defined(CC_MT5890)
#define MUSB_PORT3_PHYBASE       (0x200)
#elif defined(CC_MT5891)
#define MUSB_PORT3_PHYBASE       (-0x6000)
#else
#define MUSB_PORT3_PHYBASE       (0x300)
#endif
#define BUSCFG_BASE                 (IO_VIRT + 0x76000)
#define ETHERNET_BASE               (IO_VIRT + 0x32000)
#define DTCP_BASE                   (IO_VIRT + 0x33000)
#define MJC_BASE                    (IO_VIRT + 0x34000)     // obsolete
#define ANA_PLL_GROUP_BASE			(IO_VIRT + 0x61000)
#define TRNG_BASE                   (IO_VIRT + 0x67000)
#define EMI_REG_BASE                (IO_VIRT + 0x71000)
#define MPU_REG_BASE                (IO_VIRT + 0x72000)
#define CHN_EMI0_BASE               (IO_VIRT + 0x205000)
#define CHN_EMI1_BASE               (IO_VIRT + 0x20D000)
#define CHN_EMI2_BASE               (IO_VIRT + 0x215000)
#define CHN_EMI3_BASE               (IO_VIRT + 0x21D000)
#define EMI_REG_2_BASE              (IO_VIRT + 0x204a000)
#define MPU_REG_2_BASE              (IO_VIRT + 0x204b000)
#define CHN_EMI4_BASE               (IO_VIRT + 0x1F5000)
#define CHN_EMI5_BASE               (IO_VIRT + 0x1FD000)
#define MCUSYS_BASE                 (IO_VIRT + 0xe0000)
#define MCUSYS_CONFIG_BASE          (IO_VIRT + 0xe2000)
#define IMGRZ_BASE2                 (IO_VIRT + 0x7c000)
#define IMGRZ_BASE3                 (IO_VIRT + 0x236000)
#define IMGRZ_BASE4                 (IO_VIRT + 0x237000)
#define IMGRZ_BASE5                 (IO_VIRT + 0x20000)
#define SMI_LARB0_BASE              (IO_VIRT + 0x207e000)
#define SMI_LARB1_BASE              (IO_VIRT + 0x208e000)
#define SMI_LARB2_BASE              (IO_VIRT + 0x2017000)
#define SMI_LARB3_BASE              (IO_VIRT + 0x205e000)
#define SMI_LARB4_BASE              (IO_VIRT + 0x202b000)
#define SMI_LARB5_BASE              (IO_VIRT + 0x2030000)
#define SMI_LARB6_BASE              (IO_VIRT + 0x226000)
#define SMI_LARB7_BASE              (IO_VIRT + 0x227000)
#define SMI_LARB8_BASE              (IO_VIRT + 0x2047000)
#define SMI_LARB9_BASE              (IO_VIRT + 0x201d000)
#define SMI_LARB10_BASE             (IO_VIRT + 0x22a000)
#define SMI_LARB11_BASE             (IO_VIRT + 0x22b000)
#define SMI_LARB12_BASE             (IO_VIRT + 0x22c000)
#define SMI_LARB13_BASE             (IO_VIRT + 0x205d000)
#define SMI_LARB14_BASE             (IO_VIRT + 0x205c000)
#define SMI_LARB15_BASE             (IO_VIRT + 0xc8000)
#define SMI_LARB16_BASE             (IO_VIRT + 0x2032000)
#define SMI_LARB17_BASE             (IO_VIRT + 0x2033000)
#define SMI_LARB18_BASE             (IO_VIRT + 0x79000)
#define SMI_LARB19_BASE             (IO_VIRT + 0x45000)
#define SMI_LARB20_BASE             (IO_VIRT + 0x46000)
#define SMI_LARB21_BASE             (IO_VIRT + 0x47000)
#define SMI_LARB22_BASE             (IO_VIRT + 0x48000)
#define SMI_LARB23_BASE             (IO_VIRT + 0x49000)
#define SMI_LARB24_BASE             (IO_VIRT + 0x209e000)
#define SMI0_COMMON_BASE            (IO_VIRT + 0x22D000)
#define SMI1_COMMON_BASE            (IO_VIRT + 0x22E000)
#define SMI2_COMMON_BASE            (IO_VIRT + 0x21000)
#define L2C_BASE                    (IO_VIRT + 0x2000000)
#define SMART_CARD_BASE1            (IO_VIRT + 0x6f000)
#define GPU_BASE                    (IO_VIRT + 0x3000000)
#define GIC_DIST_BASE               (0x0c000000)
#define GIC_RDIST_BASE              (0x0c080000)
#define GIC_CPU_BASE                (0x0c020000)
#define VDEC_BASE                   (IO_VIRT_SRE)

#endif
