/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: drv_upscaler.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *
 *
 * Author:
 * -------
 *
 *
 * Last changed:
 * -------------
 * $Author: brianpc.huang $
 *
 * $Modtime: 04/05/28 9:33a $
 *
 * $Revision: #1 $
*************************************************************************/

#ifndef _DRV_UPSCALER_H_
#define _DRV_UPSCALER_H_

#include "hw_scpip_pq.h"

enum SCPQ_COEF_TYPE
{
	COEFTYP_HOR_Y=0,
	COEFTYP_HOR_C,
	COEFTYP_VER_Y,
	COEFTYP_VER_C,
	COEFTYP_MAX
};

enum SCPQ_DIR_TYPE
{
	SCPQ_DIR_H=0,
	SCPQ_DIR_V,
	SCPQ_DIR_MAX
};

enum SCPQ_INPUTBIT_NUM
{
	SCIN_422=0,
	SCIN_444,
	COEFMODE_MAX
};


#define SCPQ_PHASE_NUM 129
#define SCPQ_COEF_LEN 3
#define SCPQ_COEF_TBLSIZE (SCPQ_COEF_LEN*SCPQ_PHASE_NUM)
#define SCPQ_COEFF_TYPENUM 4
#define SCPQ_COEF_TOTAL_SIEZ (SCPQ_COEFF_TYPENUM*SCPQ_COEF_TBLSIZE)


void vDrvSCPQInit(void);
void vDrvSCPQSramCoeffOnOff(UINT8 bOnOff);
UINT8 bDrvSCPQCoeffValidate(UINT8 bType, UINT32 dwCoeffTblR[SCPQ_COEF_TBLSIZE], UINT32 dwCoeffTblW[SCPQ_COEF_TBLSIZE]);
void vDrvSCPQSetSramCoeff(UINT8 bType, UINT32 dwCoeff[SCPQ_COEF_TBLSIZE]);
UINT8 vDrvSCPQSetVCoeff(UINT8 bPath);
UINT8 vDrvSCPQSetHCoeff(UINT8 bPath);
void vDrvDeRingInit(void);
void vDrvDeRingOnOff(UINT32 u4Path, UINT32 u4Dir, UINT32 u4OnOff);
void vDrvSCPQCoeffRead(UINT8 bType, UINT32 dwResult[SCPQ_COEF_TBLSIZE]);
void vDrvSCPQTapAdapInit(void);
void vDrvSCPQTapAdapOnOff(UINT32 u4Path, UINT32 u4Dir, UINT32 u4OnOff);
void vDrvSCPQVCTIInit(void);
void vDrvSCPQVCTIOnOff(UINT8 bOnOff);
void vDrvSCPQIRRInit(void);
void vDrvSCPQIRROnOff(UINT8 bOnOff);
void vDrvUsPeiInit(void);
void vDrvUsPeiOnOff(UINT32 u4Path, UINT32 u4OnOff);

/*****************************************************************************************/
/*********************   SC FA(Scaler Frequency Analyzer)   **********************************/
/*****************************************************************************************/
#ifdef CC_FA_SUPPORT
void vDrvSCPQFAInit(void);
void vDrvSetSCPQFAInputWnd(UINT16 wHSize, UINT16 wVSize);
void vDrvSetSCPQFAWgt(UINT8 u1H1Wgt, UINT8 u1H2Wgt, UINT8 u1H3Wgt, UINT8 u1V1Wgt, UINT8 u1V2Wgt, UINT8 u1V3Wgt);
void vDrvSCPQFAOnOff(UINT8 bOnOff);
void vDrvSCPQFALoadParam(void);
UINT8 vDrvGetSCFAEn(void);
void vDrvSetSCFAHdRoiHor(UINT8 u1RoiH);
UINT8 vDrvGetSCFAHdRoiHor(void);
void vDrvSetSCFAHdRoiVer(UINT8 u1RoiV);
UINT8 vDrvGetSCFAHdRoiVer(void);
void vDrvSetSCFACorTh(UINT8 u1HCorTh, UINT8 u1VCorTh) ;
UINT32 vDrvGetSCFAH1Sum(void);
UINT32 vDrvGetSCFAH2Sum(void);
UINT32 vDrvGetSCFAH1Cnt(void);
void vDrvGetSCPQFAStatus(void);
UINT8 u1DrvGetFAScore(void);
#endif

/*****************************************************************************************/
/**************************************  IBSE  ****************************************/
/*****************************************************************************************/
void vDrvSCPQIbseInit(void);
void vDrvSCPQIbseOnOff(UINT8 bOnOff);
void vDrvSCPQIbseParam(void);
#ifdef CC_MAPLE_CUST_DRV
void vDrvSCPQIbseParam_Cust(void);
#endif
void vDrvIbseGainCurveCoringCalc(void);
void vDrvIbseYlevInit(void);
void vDrvSetIbseYLevTable(UINT8* pIbseYLev);
void vDrvLoadIbseYLevTable(void);
void vDrvCalIbseYLevTable(void);
void vDrvAdapIbseYLev(void);
void vDrvIRRLoadParam(void);

#endif

