/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef _DRV_VBI_IF_H_
#define _DRV_VBI_IF_H_

/**************************************************************************
 * Header Files
 *************************************************************************/ 

#include "x_typedef.h"
#include "api_notify.h"


/**************************************************************************
 * Export Functions
 *************************************************************************/ 
/**
 * @brief Enable or disable CC notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnCCnotify: Future reserved
 */
EXTERN void VBI_StartNotifyCCOnOff(UINT8 u1Path, UINT8 u1OnOff,
                            NPTV_PFN_CC_NFY pfnCCnotify);

/**
 * @brief Enable or disable CC2X notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnCCnotify: Future reserved
 */

EXTERN void VBI_StartNotifyCC2XOnOff(UINT8 u1Path, UINT8 u1OnOff,
                            NPTV_PFN_CC2X_NFY pfnCC2Xnotify);

/**
 * @brief Enable or disable TTX notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnTTXnotify: Future reserved
 */
EXTERN void VBI_StartNotifyTTXOnOff(UINT8 u1Path, UINT8 u1OnOff,
                             NPTV_PFN_TTX_NFY pfnTTXnotify);

/**
 * @brief Enable or disable WSS notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnWSSnotify: Future reserved
 */
EXTERN void VBI_StartNotifyWSSOnOff(UINT8 u1Path, UINT8 u1OnOff,
                             NPTV_PFN_WSS_NFY pfnWSSnotify);

/**
 * @brief Enable or disable US WSS525 notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnWSSnotify: Future reserved
 */
EXTERN void VBI_StartNotifyUSWSSOnOff(UINT8 u1Path, UINT8 u1OnOff, NPTV_PFN_WSS_NFY pfnWSSnotify);


/**
 * @brief Enable or disable US WSS525 notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnWSSnotify: Future reserved
 */
EXTERN void VBI_StartNotifyTypeBWSSOnOff(UINT8 u1Path, UINT8 u1OnOff, NPTV_PFN_TYPE_B_WSS_NFY pfnWSSnotify);


/**
 * @brief Enable or disable VPS notification
 * @param u1OnOff: SV_ON, SV_OFF
 * @param pfnVPSnotify: Future reserved
 */
EXTERN void VBI_StartNotifyVPSOnOff(UINT8 u1OnOff, NPTV_PFN_VPS_NFY pfnVPSnotify);


/**
 * @brief Set TV Encoder Closed Caption Nofity 
 * @param pfnCCnotify: CC notify CallBack
 * @Note Only for Component Path
 */
EXTERN void VBI_SetTVECCNfy(NPTV_PFN_CC_NFY pfnCCnotify);


/**
 * @brief Set TV Encoder WSS625 Nofity 
 * @param pfnWSS625notify: WSS625 notify CallBack
 * @Note Only for Component Path
 */
EXTERN void VBI_SetTVEWSS625Nfy(NPTV_PFN_WSS_NFY pfnWSS625notify);

/**
 * @brief Set TV Encoder WSS525 Nofity 
 * @param pfnWSS525notify: WSS525 notify CallBack
 * @Note Only for Component Path
 */
EXTERN void VBI_SetTVEWSS525Nfy(NPTV_PFN_WSS_NFY pfnWSS525notify);


#endif

