/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: vdp_display.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file vdp_display.h
 *  Brief of file vdp_display.h.
 *  Details of file vdp_display.h (optional).
 */

#ifndef VDP_DISPLAY_H
#define VDP_DISPLAY_H


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_typedef.h"


//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
#ifdef CC_MAPLE_CUST_DRV
#if (defined(CC_MT5890) || defined(CC_MT5895)) && defined(CC_SUPPORT_4K2K)
#define VDP_OutPutWindSrcRegenCtlByBE
#endif
#endif

///-----------------------------------------------------------------------------
/// Public functions
///-----------------------------------------------------------------------------


/**
 * VDP Update source region with current decoder status.
 *
 * @param bPath specify the video plane id.
 * @return SV_SUCCESS or SV_FAIL
 */
UINT8 bVideoUpdateSrcRegion(UINT8 bPath);


/**
 * @brief bApiVideoSetDispRegion
 *
 * Set display position and size of the video window
 * 
 * @param  bPath: SV_VP_MAIN/SV_VP_PIP
 *         wXOff: the horizontal position of the display window 
 *         wYOff: the vertical position of the display window
 *         wWidth: the width of the display window 
 *         wHeight: the height of the display window
 * @retval SV_SUCCESS/SV_FAIL
 */
UINT8 bApiVideoSetDispRegion(UINT8 bPath, UINT16 wXOff, UINT16 wYOff, UINT16 wWidth, UINT16 wHeight);


/**
 * @brief vApiVideoSrcChangeNotify
 * Input Change Notify, ReCalculate Source Region
 */
//void vApiVideoInputChangeNotify(UINT8 bPath);


/**
 * @brief bApiVideoSetSrcOverScan
 * Set overscan of the source video window
 */
UINT8 bApiVideoSetSrcOverScan(UINT8 bPath, UINT16 wTop, UINT16 wBottom, UINT16 wLeft, UINT16 wRight); 


/**
 * @brief bApiVideoSetSrcRegion
 *
 * Set position and size of the source video window
 *
 * @param  bPath: SV_VP_MAIN/SV_VP_PIP
 *         wXOff: the horizontal position of the source video window 
 *         wYOff: the vertical position of the source video window
 *         wWidth: the width of the source video window 
 *         wHeight: the height of the source video window
 * @retval SV_SUCCESS/SV_FAIL
 */
UINT8 bApiVideoSetSrcRegion(UINT8 bPath, UINT16 wXOff, UINT16 wYOff, UINT16 wWidth, UINT16 wHeight); 

EXTERN void vFscSetOutWidthLimit(UINT16 u2MaxWidth);
EXTERN UINT16 u2FscGetMaxOutWidth(void);

void VDP_SetPlaneSize(UINT8 u1Plane, UINT32 u4Width, UINT32 u4Height);
void vSetPlaneWidth(UINT32 u4Plane, UINT32 u4Width);
UINT32 u4GetPlaneWidth(UINT32 u4Plane);
void vSetPlaneHeight(UINT32 u4Plane, UINT32 u4Height);
UINT32 u4GetPlaneHeight(UINT32 u4Plane);
void vSetPlaneEnable(UINT32 u4Plane, UINT16 bEnable);
UINT8 bGetPlaneEnable(UINT32 u4Plane);
void vSetOutputPlane(UINT32 u4VdpId, UINT32 u4Plane);
UINT8 u1GetOutputPlane(UINT32 u4VdpId);
#ifdef VDP_OutPutWindSrcRegenCtlByBE
BOOL VDP_GetOutRegionControlByBE(UINT8 bPath);
BOOL VDP_GetSrcRegionControlByBE(UINT8 bPath);
#endif
#endif  //VDP_DISPLAY_H

