/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date  $
 * $RCSfile: vendor.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/
#ifndef _VENDOR_H
#define _VENDOR_H

#include "panel.h"

/*
 * This file contains prototype customization settings for vendor's system.
   Description, value range and note are also attached.

 * There're following setting levels for customization:
   + Basic
     1. Can be customized freely if there isn't special request.
     2. All comments are outlined with '********'.

   + Advanced
     1. Should be customized carefully after proper understanding comments.
     2. All comments are outlined with '--------'.

   + Constraint
     1. Needn't be customized usually to meet some limitation or regulation.
     2. All comments are outlined with '========'.

   + Notes:
     1. The //(X) sign after #define means not customizable.
     2. The //(NO USE NOW) sign after #define means it's no use now.
 */

////////////////////////////////////////////////////////////////////////////////
// Module On/Off Setting
////////////////////////////////////////////////////////////////////////////////

	#define SUPPORT_DISPLAY 1
	#define SUPPORT_BACKLT 1// 5371 !CONFIG_MT8211
	#define SUPPORT_VIDEO 1
	#define SUPPORT_DSP 0// 5371!CONFIG_MT8211
	#define SUPPORT_EEP 0//1
	#define SUPPORT_TUNER 1
	#define SUPPORT_RADIO 0	
	#define SUPPORT_OSD 0//1
	#define SUPPORT_HW_INIT 1	
	#define SUPPORT_POWER 0//1
	
////////////////////////////////////////////////////////////////////////////////
// Tuner and correspoding UI Part
////////////////////////////////////////////////////////////////////////////////

	/****************************************************************************
	SUPPORT_MULTI_SYSTEM: Define if support multi-system is supported.
		1 - Indicates multi-system is supported, uni-system is not supported
		0 - Indicates multi-system is not supported, uni-system is supported

	SUPPORT_NTSC_TV:
		1 - Indicates TV signal is in NTSC mode
		0 - Indicates TV signal is in PAL  mode
	It's constraint definition that Multi System doen't support NTSTC TV signal
	and Uni System doen't support PAL TV signal.

	SUPPORT_JAPAN_TV:
		Japan TV interface can be active only under NTSC_TV
		Use (1 &&SUPPORT_NTSC_TV) to enable Japan TV functions when SUPPORT_NTSC_TV is 1
	****************************************************************************/
	#ifndef SUPPORT_MULTI_SYSTEM
	#define SUPPORT_MULTI_SYSTEM 0
	#else
	#undef SUPPORT_MULTI_SYSTEM
	#define SUPPORT_MULTI_SYSTEM 1
	#endif

	#define SUPPORT_NTSC_TV	(!SUPPORT_MULTI_SYSTEM)
	#define SUPPORT_JAPAN_TV 	(0 &&SUPPORT_NTSC_TV)	


	/****************************************************************************
	Define if PAL tuner is used and set the TVD AUTO detection default to 50Hz.
	****************************************************************************/
	#define TVD_DFT_PAL SUPPORT_MULTI_SYSTEM


	
////////////////////////////////////////////////////////////////////////////////
// Teletext/V-Chip/Closed-Caption/Auto-Naming/WSS Part
////////////////////////////////////////////////////////////////////////////////

	/****************************************************************************
	Define whether Teletext Decoder or Close Caption Decoder is supported.
	Teletext Decoder and Close Caption Decoder can't be supported simultaneously.
	****************************************************************************/
	#define SUPPORT_CLOSE_CAPTION_VCHIP 1 /* Close Caption Decoder */
	#define SUPPORT_TELETEXT 1 /* Teletext Decoder */		
	#define SUPPORT_AUTO_NAMING 1 /* VPS */	
	#define SUPPORT_WSS 1                 /* WSS */

	#define SUPPORT_VBI (SUPPORT_CLOSE_CAPTION_VCHIP|SUPPORT_WSS|SUPPORT_TELETEXT|SUPPORT_AUTO_NAMING)

////////////////////////////////////////////////////////////////////////////////
// Input Source Part
////////////////////////////////////////////////////////////////////////////////

	#include "source_table.h"

	/*---------------------------------------------------------------------------
	Scart auto source switch support (depend on pin 8 and pin 16 change).
	---------------------------------------------------------------------------*/
	#define SUPPORT_SCART  (SUPPORT_SCART1_INPUT || SUPPORT_SCART2_INPUT || SUPPORT_SCART3_INPUT)
	#define SUPPORT_PIP_NOSCARTOUT 0
	/*---------------------------------------------------------------------------
	Disable or Enable scart pin8 functions (scart plug detection & picture size auto)
	---------------------------------------------------------------------------*/
	#define ENABLE_SCART_PIN8	SUPPORT_SCART     
     
	//efine SUPPORT_TTX25                            0

#endif /* _VENDOR_H */
