/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: fbm_cs.c,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file fbm_cs.c
 */


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "fbm_cs.h"
#include "fbm_drvif.h"
#include "fbm_debug.h"
#include "x_assert.h"
#include "x_os.h"


//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------

#ifdef FBM_CS_LOG
extern UINT32 _u4EQHandle;
#endif


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


/******************************************************************************
* Local Function, Counting Semaphore
******************************************************************************/
#ifdef FBM_CS_LOG
//-----------------------------------------------------------------------------
/** Brief of _FbmFbLog.
 */
//-----------------------------------------------------------------------------
void _FbmCsLog(UINT32 u4Log, INT32 i4Count, UINT32 u4handle)
{
#define FBM_FB_LOG_NS 64
    static CHAR szDebug1[FBM_FB_LOG_NS];
    static CHAR szDebug2[FBM_FB_LOG_NS];
    static UINT32 u4DebugIdx = 0;
    static HANDLE_T hMutex = NULL;

    if (hMutex == NULL)
    {
        // create semaphore
        VERIFY(x_sema_create(&hMutex, X_SEMA_TYPE_MUTEX, X_SEMA_STATE_UNLOCK) == OSR_OK);

        for (u4DebugIdx = 0; u4DebugIdx < FBM_FB_LOG_NS; u4DebugIdx++)
        {
            szDebug1[u4DebugIdx] = 0;
            szDebug2[u4DebugIdx] = 0;
        }

        u4DebugIdx = 0;
    }

    if (u4handle != _u4EQHandle)
    {
        return;
    }

    VERIFY(x_sema_lock(hMutex, X_SEMA_OPTION_WAIT) == OSR_OK);

    szDebug1[u4DebugIdx] = '0' + u4Log;
    szDebug2[u4DebugIdx] = '0' + i4Count;

    if ((++u4DebugIdx >= (FBM_FB_LOG_NS - 1)) || (u4Log > 20))
    {
        LOG(1, "%s\n", szDebug1);
        LOG(1, "%s\n", szDebug2);
        LOG(1, "u4DebugIdx = %d\n", u4DebugIdx);
        u4DebugIdx = 0;
    }

    VERIFY(x_sema_unlock(hMutex) == OSR_OK);
}
#endif


#ifndef FBM_OSAI_CS
//-------------------------------------------------------------------------
/** _FBM_csema_create
 *  Create counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  hMutex   mutex.
 *  @param  i4InitCount   count.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_create(COUNTING_SEMAPHORE_T* prSemaphore, HANDLE_T hMutex, INT32 i4InitCount)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    prSemaphore->i4Count = i4InitCount;
    prSemaphore->hMutex = hMutex;        // special design, external mutex

    // create semaphore
    VERIFY(x_sema_create(&(prSemaphore->hBinary),
                         X_SEMA_TYPE_BINARY,
                         X_SEMA_STATE_LOCK) == OSR_OK);

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_reset
 *  Reset counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  hMutex   mutex.
 *  @param  i4InitCount   count.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_reset(COUNTING_SEMAPHORE_T* prSemaphore, HANDLE_T hMutex, INT32 i4InitCount)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    prSemaphore->i4Count = i4InitCount;
    prSemaphore->hMutex = hMutex;        // special design, external mutex

    // reset prSemaphore->hBinary to X_SEMA_STATE_LOCK
    if (x_sema_lock(prSemaphore->hBinary, X_SEMA_OPTION_NOWAIT) == OSR_OK)
    {
    }
    else
    {
    }

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_delete
 *  Delete counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_delete(const COUNTING_SEMAPHORE_T* prSemaphore)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    // delete semaphore
    VERIFY(x_sema_delete(prSemaphore->hBinary) == OSR_OK);

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_unlock
 *  Unlock counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_unlock(COUNTING_SEMAPHORE_T* prSemaphore)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    prSemaphore->i4Count++;

    if (prSemaphore->i4Count <= 0)
    {
        VERIFY(x_sema_unlock(prSemaphore->hBinary) == OSR_OK);  // wake
    }

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_lock
 *  Lock counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  e_options   wait/no_wait option.
 *  @param  u4Time   wait time.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_lock(COUNTING_SEMAPHORE_T* prSemaphore, SEMA_OPTION_T e_options, UINT32 u4Time)
{
    INT32 i4ReturnValue;

    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    if ((e_options == X_SEMA_OPTION_WAIT) && (u4Time != 0))
    {
        prSemaphore->i4Count--;

        if (prSemaphore->i4Count < 0)
        {
#ifdef CC_VDP_FULL_ISR
            x_crit_end(*prSemaphore->prState);
#else
            VERIFY(x_sema_unlock(prSemaphore->hMutex) == OSR_OK);
#endif

            i4ReturnValue = x_sema_lock_timeout(prSemaphore->hBinary, u4Time);

#ifdef CC_VDP_FULL_ISR
            *prSemaphore->prState = x_crit_start();
#else
            VERIFY(x_sema_lock(prSemaphore->hMutex,
                               X_SEMA_OPTION_WAIT) == OSR_OK);
#endif

            if (i4ReturnValue == OSR_TIMEOUT)
            {
                if (prSemaphore->i4Count >= 0)
                {
                    // 20041229 mutex is NOT well protected!
                    LOG(3, "{CS}\n");

                    VERIFY(x_sema_lock(prSemaphore->hBinary,
                                       X_SEMA_OPTION_NOWAIT) == OSR_OK);

                    i4ReturnValue = OSR_OK;
                }
                else
                {
                    // Get semaphore timeout, put it back
                    prSemaphore->i4Count++;

                    LOG(3, "{CS} Timeout (%d)\n", prSemaphore->i4Count);

                    return OSR_TIMEOUT;
                }
            }

            ASSERT(i4ReturnValue == OSR_OK);
        }

    }
    else
    {
        if (prSemaphore->i4Count <= 0)
        {
            return OSR_WOULD_BLOCK;
        }

        prSemaphore->i4Count--;
    }
    ASSERT(prSemaphore->i4Count >= 0);

    return OSR_OK;
}

#else

//-------------------------------------------------------------------------
/** _FBM_csema_create
 *  Create counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  hMutex   mutex.
 *  @param  i4InitCount   count.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_create(COUNTING_SEMAPHORE_T* prSemaphore, HANDLE_T hMutex, INT32 i4InitCount)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    prSemaphore->i4Count = i4InitCount;
    prSemaphore->hMutex = hMutex;        // special design, external mutex

    // create semaphore
    VERIFY(x_sema_create(&(prSemaphore->hBinary),
                         X_SEMA_TYPE_COUNTING,
                         i4InitCount) == OSR_OK);

#ifdef FBM_CS_LOG
    _FbmCsLog(0, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_reset
 *  Reset counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  hMutex   mutex.
 *  @param  i4InitCount   count.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_reset(COUNTING_SEMAPHORE_T* prSemaphore, HANDLE_T hMutex, INT32 i4InitCount)
{
    UINT32 u4Idx;
    INT32 i4Ret;
    UINT32 u4Count;

    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

#if 1
    prSemaphore->i4Count = i4InitCount;
    prSemaphore->hMutex = hMutex;        // special design, external mutex

    do
    {
        i4Ret =  x_sema_lock(prSemaphore->hBinary, X_SEMA_OPTION_NOWAIT);
    } while (i4Ret == OSR_OK);

    if (i4Ret != OSR_WOULD_BLOCK)
    {
        ASSERT(0);
    }

#ifdef FBM_CS_LOG
    _FbmCsLog(1, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

    for (u4Idx = 0; u4Idx < i4InitCount; u4Idx++)
    {
        VERIFY(x_sema_unlock(prSemaphore->hBinary) == OSR_OK);
    }

#else

u4Count = (i4InitCount - prSemaphore->i4Count);
prSemaphore->hMutex = hMutex;        // special design, external mutex

#ifdef FBM_CS_LOG
_FbmCsLog(1, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

for (u4Idx = 0; u4Idx < u4Count; u4Idx++)
{
    VERIFY(x_sema_unlock(prSemaphore->hBinary) == OSR_OK);
}

prSemaphore->i4Count = prSemaphore->i4Count + u4Count;
#endif

#ifdef FBM_CS_LOG
    _FbmCsLog(2, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_delete
 *  Delete counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_delete(const COUNTING_SEMAPHORE_T* prSemaphore)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    // delete semaphore
    VERIFY(x_sema_delete(prSemaphore->hBinary) == OSR_OK);

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_unlock
 *  Unlock counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_unlock(COUNTING_SEMAPHORE_T* prSemaphore)
{
    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

    VERIFY(x_sema_unlock(prSemaphore->hBinary) == OSR_OK);

    prSemaphore->i4Count++;
    ASSERT(prSemaphore->i4Count > 0);

#ifdef FBM_CS_LOG
    _FbmCsLog(4, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

    return OSR_OK;
}


//-------------------------------------------------------------------------
/** _FBM_csema_lock
 *  Lock counting semaphore
 *  @param  prSemaphore   semaphore pointer.
 *  @param  e_options   wait/no_wait option.
 *  @param  u4Time   wait time.
 *  @return OSR_OK.
 *          If fail, return != OSR_OK.
 */
//-------------------------------------------------------------------------
INT32 _FBM_csema_lock(COUNTING_SEMAPHORE_T* prSemaphore, SEMA_OPTION_T e_options, UINT32 u4Time)
{
    INT32 i4ReturnValue;

    if (prSemaphore == NULL)
    {
        return OSR_INV_ARG;
    }

#ifdef FBM_CS_LOG
    _FbmCsLog(5, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

    if ((e_options == X_SEMA_OPTION_WAIT) && (u4Time != 0))
    {
        VERIFY(x_sema_unlock(prSemaphore->hMutex) == OSR_OK);

        i4ReturnValue = x_sema_lock_timeout(prSemaphore->hBinary, u4Time);

#ifdef FBM_CS_LOG
        _FbmCsLog(6, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

        VERIFY(x_sema_lock(prSemaphore->hMutex,
                           X_SEMA_OPTION_WAIT) == OSR_OK);

        if (i4ReturnValue == OSR_OK)
        {
            prSemaphore->i4Count--;

#ifdef FBM_CS_LOG
            _FbmCsLog(7, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

            ASSERT(prSemaphore->i4Count >= 0);
        }
        else if (i4ReturnValue == OSR_TIMEOUT)
        {
            if (prSemaphore->i4Count == 0)
            {
                LOG(3, "{CS} Timeout (%d)\n", prSemaphore->i4Count);

#ifdef FBM_CS_LOG
                _FbmCsLog(8, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

            }
            else if (prSemaphore->i4Count > 0)
            {
                LOG(3, "{CS}\n");

                LOG(1," count(%d)\n", prSemaphore->i4Count);
#ifdef FBM_CS_LOG
                _FbmCsLog(23, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

                ASSERT(0);
            }
        }
    }
    else
    {

        i4ReturnValue = x_sema_lock(prSemaphore->hBinary, X_SEMA_OPTION_NOWAIT);

        if (i4ReturnValue == OSR_OK)
        {
            prSemaphore->i4Count--;

#ifdef FBM_CS_LOG
            _FbmCsLog(9, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

            if (prSemaphore->i4Count < 0)
            {
#ifdef FBM_CS_LOG
                _FbmCsLog(21, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

                ASSERT(prSemaphore->i4Count >= 0);
            }

        }
        else if (i4ReturnValue == OSR_WOULD_BLOCK)
        {
            if (prSemaphore->i4Count > 0)
            {

#ifdef FBM_CS_LOG
                _FbmCsLog(22, prSemaphore->i4Count, prSemaphore->hBinary);
#endif

                ASSERT(0);
            }
        }
        else
        {
            ASSERT(0);
        }
    }

    ASSERT(prSemaphore->i4Count >= 0);

    return i4ReturnValue;
}


#endif

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------


