/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: fbm_fb.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

/** @file sample.h
 *  Brief of file sample.h.
 *  Details of file sample.h (optional).
 */

#ifndef FBM_FB_H
#define FBM_FB_H


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "fbm_drvif.h"
#include "fbm_cs.h"
#include "x_typedef.h"


//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

#ifdef CC_FBM_4K_SEAMLESS_NOT_SUPPORT
#define FBM_SEAMLESS_GROUP_COUNT 2
#endif


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

/** Brief of FBM_FBQ_T
 *  Frame Buffer Queue
 */
typedef struct
{
    UCHAR ucReadIdx;
    UCHAR ucCount;
    BOOL fgTrueDispQ;
    UCHAR aucQueue[FBM_MAX_FB_NS_PER_GROUP];
} FBM_FBQ_T;


typedef struct
{
    UPTR u4Addr;
    UINT32 u4Size;
    BOOL fgInUse;
} FBM_SEAMLESS_POOL_T;


typedef enum
{
    FBM_RESOLUTION_CHANGE_MODE_NONE,
    FBM_RESOLUTION_CHANGE_MODE_SEAMLESS,
    FBM_RESOLUTION_CHANGE_MODE_NS,
} FBM_RESOLUTION_CHANGE_MODE_T;


/** Brief of FBM_FBG_T
 *  Frame Buffer Group
 */
typedef struct
{
    BOOL  fgEnableNewSeamless;
    BOOL  fgAutoSeamless;
    UCHAR ucFbgType;                        // Frame Buffer Group Type
    UCHAR ucFbgId;                          // Frame Buffer Group ID
    UCHAR ucFbgCm;                          // Frame Buffer Color Mode
    UCHAR ucFbNs;                           // Frame Buffer Number
    UCHAR ucFbNsBase;                       // Frame Buffer Number
    UCHAR ucReserveFbNsBase;                // Frame Buffer Number
    UCHAR ucFbNsOld;                        // Frame Buffer Number
    UCHAR ucFbNsOldBase;                    // Frame Buffer Number

    UCHAR ucFbNsMajor;                        // Frame Buffer Number
    UCHAR ucFbNsMajorBase;                    // Frame Buffer Number
    UCHAR ucFbNsMinor;                        // Frame Buffer Number
    UCHAR ucFbNsMinorBase;                    // Frame Buffer Number
    BOOL  fgFirstCreateFbg;                  //
    BOOL  fgMoreCreateFbg;                  //
    UCHAR ucMvBufNs;                        // MV Buffer Number
    UCHAR ucMajorMvBufNs;                        // MV Buffer Number
    UCHAR ucMinorMvBufNs;                        // MV Buffer Number
    UCHAR ucSeamlessFbNs;
    UCHAR ucSeamless2FbNs;
    UCHAR ucCurSeamlessTaskId;
	UCHAR ucPictureId;	
    UCHAR ucSeamlessRzId;
    UCHAR ucSeamlessRzId2;
    UCHAR ucSeamlessRzMask;
    UCHAR ucFbDecode;                       // Current Decoding FB
    UCHAR ucFbFRef;                         // Forward Reference FB
    UCHAR ucFbBRef;                         // Backward Reference FB
    UCHAR ucFbResize;                       // Current Resizing FB
    UCHAR ucFbResize2;                       // Current Resizing FB
    UCHAR ucFbFRefValid;
    UCHAR ucFbBRefValid;
    /*  Fix CTS suspend->resume video black screen */
    UINT64 u4LastB2rAcquireDispTag;       // Frame Buffer Group ID
    UCHAR ucLastB2rAcquireFbId;          // Frame Buffer Group ID
    /* */
    UCHAR ucStcSrc;                         // AV Sync Source
    UCHAR ucAvSyncMode;                     // AV Sync Mode

    UCHAR ucPlayMode;                       // DTV or MM or ...
    UCHAR ucNewFbId;
    UCHAR ucSeqChg;
    UINT32 u4SeqChgFlag;
	UINT32 u4CrtFbgCnt;
    BOOL  fgGotFirstSeqChg;
	BOOL  fgRlsDispQue;  // TRUE:call FBM_ReleaseDispQ 	
	BOOL  fgSeqNoEqual;  // TRUE: no equal about W&H,defaul FALSE 
    BOOL  fgOMXGetFirstFb;   //  VB1 seamless
    UCHAR ucOMXPrevFb;       //  VB1 seamless
    UCHAR ucLastDecodeReference;            // Last Decode Picture is Reference
    UCHAR aucResizeBlockFb;     //Sema lock block FB for wait empty RzFb
    UCHAR aucResize2BlockFb;     //Sema lock block FB for wait empty RzFb
    BOOL  fgIsSeamlessCft;       // using flag when checking  conflict
    UCHAR aucFbStatus[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResizeFbStatus[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResizeTaskId[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResizeSrcFbId[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResize2FbStatus[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResize2TaskId[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucResize2SrcFbId[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucPlayBackNs[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucChangeFieldNs[FBM_MAX_FB_NS_PER_GROUP];
    UCHAR aucChgFldPlayBackNs[FBM_MAX_FB_NS_PER_GROUP];
    BOOL   afgRefList[FBM_MAX_FB_NS_PER_GROUP];    //for H264 to maintain reference list
    BOOL   afgIsOwner[FBM_MAX_FB_NS_PER_GROUP]; // Is fb owned by FBM or by others, like OMX
    BOOL   afgWaitUse[FBM_MAX_FB_NS_PER_GROUP];
    BOOL   fgCreateFromInst;                 // Init fbg from external FBM inst
    BOOL   fgSendEOS;
    
    UINT8 u1DecoderSrcId;
    UINT8 u1FbgAppMode;
    UINT32 u4VDecFmt;                       //for mt539x various video decoder
    UINT32 u4FbgFlag;                       // Flag
    UINT32 u4FbWidth;                       // Frame Buffer Width, According to Color Mode
    UINT32 u4FbHeight;                      // Frame Buffer Height, According to Color Mode
    UINT32 u4WfdFlag;
    UINT32 u4_es_cnt;
    UINT32 u4ResidualFb;                    // Indicate whether residual FB is used
    UINT32 u4ResidualFree;                  // Indicate whether residual FB is pending to free
    UINT32 u4WorkBufSize;                   // The size for working buffer

    UPTR u4FbMemoryPool;                  //Y Addr
    UPTR u4FbMemoryPoolC;                 //C Addr support two dram channel.
    UINT32 u4FbMemoryPoolSize;
    UPTR u4FbMemoryPool2;                 //2nd part for SBS/TNB pool type.
    UPTR u4FbMemoryPool2C;                //2nd part for SBS/TNB pool type and two dram channel.
    UINT32 u4FbMemoryPool2Size;             //2nd part for SBS/TNB pool type and two dram channel.
    UPTR u4FbResizeMemoryPool;
    UINT32 u4FbResizeMemoryPoolSize;
    UPTR u4FbResizeExtraMemoryPool;
    UINT32 u4FbResizeExtraMemoryPoolSize;
    UPTR u4FbResizeMemoryPool2;
    UINT32 u4FbResizeMemoryPool2Size;
    /* 4K Seamless use*/
    BOOL fgMvAllocate;
    FBM_RESOLUTION_CHANGE_MODE_T eResolutionChangeMode;
    /* ----------------------------*/
    /* check Frame buffer addr overlap */
    /* --------------------------*/    
    UPTR u4FbStartPoolYAddr;
    UPTR u4FbEndPoolYAddr;
    UPTR u4FbStartPool2YAddr;
    UPTR u4FbEndPool2YAddr;

    UPTR u4FbStartPoolCAddr;
    UPTR u4FbEndPoolCAddr;
    UPTR u4FbStartPool2CAddr;
    UPTR u4FbEndPool2CAddr;
    
    UINT32 u4AvSyncStatus;                  // AV sync status, repeat or drop
#ifdef CC_B2R_RES_SUPPORT
    B2R_HW_RES_T* pt_res;
#endif
#ifdef FBM_VIRTUAL_MPEG
    UINT32 u4VmStatus;
#endif

    void* pMallocAddr;                      // Memory allocation addr
    void* pWorkBufVirAddr;                  // WorkBuf Virtual Address
    BOOL fg10Bit;
    BOOL fgUFO;
    BOOL fgIsGameMode;
    UPTR u4MvStartAddr; // seamless playback mv start addr to fix conflict with FB
    UPTR u4CabacStartAddr; // Fixed lae buffer
    UPTR au4AddrY[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4AddrY_Ext[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4AddrC[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4AddrC_Ext[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4AddrMv[FBM_MAX_FB_NS_PER_GROUP-1];
    UINT32 au4GfxMemsetId[FBM_MAX_FB_NS_PER_GROUP*2];  //0 means memset done. other means memset ing
    UINT32 au4GdmaMemsetYId[FBM_MAX_FB_NS_PER_GROUP];  //0 means memset done. other means memset ing
    UINT32 au4GdmaMemsetCId[FBM_MAX_FB_NS_PER_GROUP];  //0 means memset done. other means memset ing
    UPTR au4ResizeAddrY[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4ResizeAddrC[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4Resize2AddrY[FBM_MAX_FB_NS_PER_GROUP];
    UPTR au4Resize2AddrC[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 au4DbkOfstCnt[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 au4Dbk4pCnt[FBM_MAX_FB_NS_PER_GROUP];
    UINT64 u4DispTag[FBM_MAX_FB_NS_PER_GROUP];
    UINT64 u4DropTag[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 u4ResizeBufferFlag[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 u4Resize2BufferFlag[FBM_MAX_FB_NS_PER_GROUP];
    HAL_TIME_T arRlResizeFb[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 eSeamlessMode;
    UPTR au4MiscTblStartAddr; //VP9 just use,segld tife count prob total size
    UPTR u4ExtraYBuffer;
    UPTR u4ExtraCBuffer;
    UPTR u4RPRSeamlessBuffer;
	UINT32 u4PauseType;
	UINT32 u4StepCmd;
	UINT32 u4StepCnt;
	UINT32 u4StepThreshold;
	BOOL   fgSeqChgPause;	
    FBM_SEQ_HDR_T rSeqHdr;
    FBM_PIC_HDR_T* prPicHdr;
    FBM_POOL_T*    pFbmSmlPool;
    FBM_POOL_T*    pFbmSmlPool2;
    FBM_POOL_T*    pFbmSmlPoolC;
    FBM_POOL_T*    pFbmSmlPool2C;    
    FBM_POOL_T*    pFbmMpegPool;
    FBM_POOL_T*    pFbmMpegPool2;
    FBM_FBQ_T rEmptyQ;
    FBM_FBQ_T rDisplayQ;
    FBM_FBQ_T rResizeEmptyQ;
    FBM_FBQ_T rResize2EmptyQ;   //two image:rz0 using ResizeEmptyQ,Rz1 using ResizeEmptyQ2 ,in order to fix two prescale trhead!!
    FBM_FBQ_T rTagQ;
    FBM_SEQ_CHG_QUE_T  rSeqChgPendingQ; //using omx and nptv path	
    ENUM_CODEC_INFO_T eCodecType;
    FBM_FBID_TYPE_T   eFbIdType;

    COUNTING_SEMAPHORE_T hEmptyQSemaphore;  // Empty Queue Counting Semaphore
    COUNTING_SEMAPHORE_T hEmptyBQSemaphore; // Empty B Queue Counting Semaphore
    COUNTING_SEMAPHORE_T hEmptyResizeQSemaphore;
    COUNTING_SEMAPHORE_T hEmptyResize2QSemaphore;
    HANDLE_T hChgFieldSema;    

    HANDLE_T hMutex;                        // Thread safe protection
    CRIT_STATE_T* prState;                  // Thread/Isr safe protection

    B2R_NPTV_PENDING_Q  t_pending_q;
    BOOL fgRefNotEnough;
#ifdef CC_FBM_4K_SEAMLESS_NOT_SUPPORT
    UPTR u4SeamlessAddr[FBM_SEAMLESS_GROUP_COUNT];
#endif
    UCHAR ucSeqChgFbId;
	UINT8 u1Fps;
	BOOL bByPassMJC;
    FBM_FBG_DOLBY_TYPE eDolbyType;
    UPTR u4RpuPhysicalAddr;
    UINT32 u4TargetSize;
    BOOL fg8KMpeg;
    BOOL fgNonBuf;
    BOOL fgTunnelMode;
} FBM_FBG_T;

//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

extern void _FBM_FbInit(FBM_FBG_T* prFbg);

extern void _FBM_PutFrameBufferToEmptyQ(UCHAR ucFbgId, UCHAR ucFbId);
extern void _FBM_PutFrameBufferToEmptyResizeQ(UCHAR ucFbgId, UCHAR ucFbId);
extern void _FBM_PutFrameBufferToEmptyResize2Q(UCHAR ucFbgId, UCHAR ucFbId);
extern UCHAR FBM_GetLatestEmptyFrameBuffer(UCHAR ucFbgId, UINT32 u4Delay);

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------


#endif  // FBM_FB_H

