/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: fbm_inst_drvif.h,v $
 * $Revision: #1 $

 *
 *---------------------------------------------------------------------------*/

/** @file fbm_inst_drvif.h
 *  API of FBM (Frame Buffer Manager) instance
 *  Details of file fbm_inst_drvif.h (optional).
 */

#ifndef FBM_INST_IMPL_H
#define FBM_INST_IMPL_H


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "x_typedef.h"
#include "fbm_drvif.h"
#include "fbm_fb.h"

//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

#define FBM_INST_MAX_NS 10
#define FBM_INST_TYPE_ALLOC 0
#define FBM_INST_TYPE_SET 1

//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

typedef struct
{
    UCHAR  ucInstCm;
    UCHAR  ucInstMode;
    UCHAR  ucInstId;
    UCHAR  ucFbgId;
    UCHAR  ucFbNs;
    BOOL   fgOccupied; //For FBM_INST_TYPE_SET, FBs occupied by a given vdec.
    
    UINT32 u4VDecFmt;
    UINT32 u4FbWidth;
    UINT32 u4FbHeight;

    UCHAR aucFbStatus[FBM_MAX_FB_NS_PER_GROUP];

    UINT32 au4AddrY[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 au4AddrC[FBM_MAX_FB_NS_PER_GROUP];
    UINT32 au4AddrMv[FBM_MAX_FB_NS_PER_GROUP-1];

    HANDLE_T hMutex;
} FBM_INST_T;

//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------

#define FBM_INST_MUTEX_LOCK(id)                              \
    (VERIFY(x_sema_lock(_arFBMInst[id].hMutex, X_SEMA_OPTION_WAIT) == OSR_OK))

#define FBM_INST_MUTEX_UNLOCK(id)                            \
    (VERIFY(x_sema_unlock(_arFBMInst[id].hMutex) == OSR_OK))

#define VERIFY_INST(id)                                  \
    ((id >= FBM_INST_MAX_NS) ||                              \
    (_arFBMInst[id].ucInstId == FBM_INST_ID_UNKNOWN) ||   \
    (_arFBMInst[id].ucInstId != id))

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

void FBM_Inst_Init(void);
void FBM_GetInstBufAddr(UCHAR ucInstId, UCHAR ucFbId, UINT32 *pu4AddrY, UINT32 *pu4AddrC);

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------


#endif    // FBM_INST_IMPL_H


