/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: fbm_inst_if.h,v $
 * $Revision: #1 $

 *
 *---------------------------------------------------------------------------*/

/** @file fbm_inst_if.h
 *  API of FBM (Frame Buffer Manager) instance
 *  Details of file fbm_inst_drvif.h (optional).
 */

#ifndef FBM_INST_IF_H
#define FBM_INST_IF_H


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

//#include "x_typedef.h"

//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

#define FBM_INST_ID_UNKNOWN  0xFF

#define FBM_STATUS_OK 0
#define FBM_UNEXPECTED_ERROR 1
#define FBM_INVALID_ARGUMENT 2

//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

#if !defined (_TYPEDEF_FBM_UCHAR_)
typedef unsigned char  UCHAR_T;
#define _TYPEDEF_FBM_UCHAR_
#endif

#if !defined (_TYPEDEF_FBM_UINT32_)
typedef unsigned long    UINT32_T;
#define _TYPEDEF_FBM_UINT32_
#endif

#if !defined (_TYPEDEF_FBM_BOOL_)
typedef unsigned char  BOOL_T;
#define _TYPEDEF_FBM_BOOL_

#ifdef TRUE
#undef TRUE
#endif
#ifdef FALSE
#undef FALSE
#endif
#define TRUE ((BOOL_T) 1)
#define FALSE ((BOOL_T) 0)
#endif


typedef enum
{
    FBM_CFMT_420,
    FBM_CFMT_422,
    FBM_CFMT_444,
    FBM_CFMT_ARGB
} FBM_ColorFMT_T;

typedef enum
{
    FBM_H264,
    FBM_MPEG2,
    FBM_MPEG4,
    FBM_VC1,
    FBM_JPEG,
    FBM_RV,
    FBM_AVS,
    FBM_RAW,
    FBM_VP6,
    FBM_VP8,
    FBM_H265,
    FBM_AVS2
} FBM_VDEC_FORMAT;

typedef enum
{
    FBM_FRAME_SIZE,
    FBM_FRAME_COUNT
} FBM_INST_INFO_TYPE;

//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

UCHAR_T FBM_GetInstance(FBM_VDEC_FORMAT eVDecFormat, UINT32_T u4Width, UINT32_T u4Height, FBM_ColorFMT_T eColorMode, UINT32_T u4FrameNum, BOOL_T fgAlloc, UINT32_T u4ExtOption);
UINT32_T FBM_FreeInstance(UCHAR_T ucInstId);
UINT32_T FBM_FreeBuf(UCHAR_T ucInstId, UINT32_T u4Addr);
UINT32_T FBM_AllocBuf(UCHAR_T ucInstId, UINT32_T *pu4Addr);
UINT32_T FBM_Alloc_YCBuf(UCHAR_T ucInstId, UINT32_T *pu4AddrY,UINT32_T *pu4AddrC);
UINT32_T FBM_QueryBufNum(FBM_VDEC_FORMAT eVDecFormat, UINT32_T u4Width,
    UINT32_T u4Height, FBM_ColorFMT_T eColorMode, UINT32_T *pu4BufNum,
    UINT32_T *pu4Stride, UINT32_T *pu4SliceHeight,
    UINT32_T *pu4UVAddrOffset, UINT32_T u4ExtOption);
UINT32_T FBM_GetInstInfo(UCHAR_T ucInstId, FBM_INST_INFO_TYPE eInfoType, UINT32_T *pInfoResult);
#endif    // FBM_INST_IF_H


