/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_bsl.h
 *  $File: //DTV/PROD_BR/DTV_X_IDTV1501/apollo/middleware/public/bsl/c_bsl.h $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *    This header file contains Satellite list (BSL) database
 *    API definitions, which are exported to other Middleware components
 *    and applications.
 *         
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_BSL_H_
#define _C_BSL_H_
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_BSL
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/

#include "u_bsl.h"

/*-------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief  Create a BSL object and return a handle to this BSL object.
 *  @param  [in]  ps_bsl_name      -  Name of the BSL object.
 *  @param  [in]  ui2_bsl_id       -  Unique id for the BSL object.
 *                                    The limit for BSL ID is between 1 and
 *                                    65535.
 *  @param  [in]  e_mem_policy      - Memory policy: BSL_MEM_STATIC or
 *                                    BSL_MEM_DYNAMIC
 *  @param  [in]  ui2_max_nb_rec    - Specifies the maximum number of BSL record
 *                                    (for BSL_MEM_STATIC), or it specifies
 *                                    number of BSL records per memory chunk.
 *  @param  [in]  pv_tag            - Pointer to a private tag value provided
 *                                    by the caller.  This tag value is passed
 *                                    to the caller's notification function.
 *  @param  [in]  pf_nfy            - Notification function provided by the
 *                                    caller.  This function is called to notify
 *                                    the owner of this BSL handle when the BSL
 *                                    object has been modified.  If NULL, then
 *                                    no notification function will be called.
 *  @param  [out] ph_bsl           -  Handle to the BSL object.
 *  @see    c_bsl_open
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL object is created successfully.
 *  @retval BSLR_INV_ARG       - 'ui2_bsl_id' is 0, or ph_bsl is NULL, or
 *                                ps_bsl_name is 0.
 *  @retval BSLR_OUT_OF_HANDLE - Cannot get a handle.
 *  @retval BSLR_FAIL          - The specified 'ps_bsl_name' or 'ui2_bsl_id'
 *                                value is not unique (e.g., it is currently
 *                                been used), or limit of the number of BSL
 *                                objects that can be created is reached.
 *  @retval BSLR_OUT_OF_MEM    - Not enough memory to create BSL object.
 *  @retval BSLR_INV_ARG       - 'ps_bsl_name' or 'ph_bsl' is NULL.
 *  @retval BSLR_AEE_OUT_OF_RESOURCES  - Can not allocate AEE resources.
 *  @retval BSLR_AEE_NO_RIGHTS - No right for the AEE resources.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_create
(
    const CHAR*         ps_bsl_name,
    UINT16              ui2_bsl_id,
    BSL_MEM_POLICY_T    e_mem_policy,
    UINT16              ui2_max_nb_rec,
    VOID*               pv_tag,
    x_bsl_nfy_fct       pf_nfy,
    HANDLE_T*           ph_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This API deletes the specified BSL object associated
 *          with 'h_bsl' handle.    Other clients with a handle to
 *          the deleted BSL object will receive a notification of
 *          the delete event.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @return Return the error code
 *  @retval BSLR_OK            - Specified BSL object is deleted.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_delete
(
    HANDLE_T      h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This API removes all the satellite records from the
 *          specified BSL object pointed to by the 'h_bsl' handle.
 *          All other handles to this BSL object will receive a
 *          notification.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @return Return the error code
 *  @retval BSLR_OK            - All records are successfully removed.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_clean
(
    HANDLE_T      h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This API queries the range (FIRST, LAST, or NEXT) of
 *          valid bisskey record id values used in the BSL
 *          object.
 *  @param  [in]  h_bsl             - Handle to the BSL object.
 *  @param  [in]  e_bsl_qry_option  - Flag to specifies the record id value to
 *                                    be returned. The values are:
 *                                    BSL_FIRST_REC_ID, BSL_LAST_REC_ID,
 *                                    BSL_NEXT_REC_ID.
 *  @param  [out] pui2_rec_id       - Contain the bisskey record id
 *  value.
 *  @return Return the error code
 *  @retval BSLR_OK                 - BSL record id return.
 *  @retval BSLR_INV_HANDLE         - Bad BSL handle.
 *  @retval BSLR_INV_ARG            - 'pui2_rec_id' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_qry_rec_id
(
    HANDLE_T            h_bsl,
    BSL_REC_ID_QRY_T    e_bsl_qry_option,
    UINT16*             pui2_rec_id	
);


/*------------------------------------------------------------------*/
/*! @brief  This function adds a new service record to the BSL
 *          database object.  The BSL library will make copy of
 *          the record content from the address location pointed to
 *          by the 'pt_bsl_rec'. The 'ui2_bsl_rec_id' field in
 *          the 'pt_bsl_rec' structure specifies the key field of the
 *          service record to be added to the database.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @param  [in]  pt_bsl_rec   - Pointer to the service record structure
 *                                    to be added.
 *  @return Return the error code
 *  @retval BSLR_OK            - Record is successfully added.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 *  @retval BSLR_OUT_OF_MEM    - Can not allocate memory, add record failed.
 *  @retval BSLR_INV_ARG       - 'pt_bsl_rec' is null.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_add_rec
(
    HANDLE_T            h_bsl,
    const BSL_REC_T*    pt_bsl_rec
);

/*------------------------------------------------------------------*/
/*! @brief  This function deletes a specified service record from
 *          the BSL. The record to be deleted is specified by the
 *          satellite record id.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [in]  ui2_bsl_rec_id   - Specifies the satellite record to
 *                                    be deleted.
 *  @return Return the error code
 *  @retval BSLR_OK            - Specified record is deleted.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 *  @retval BSLR_REC_NOT_FOUND - The specified record to be deleted is not in
 *                                the BSL object.
 *  @retval BSLR_INV_ARG       - 'ui2_bsl_rec_id' is not a valid record id.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_del_rec
(
    HANDLE_T            h_bsl,
    UINT16              ui2_bsl_rec_id
);

/*------------------------------------------------------------------*/
/*! @brief  This function updates a Satellite record to the BSL
 *          object. If the key field value (e.g., 'ui2_bsl_rec_id' field
 *          in the 'pt_bsl_rec' structure) of the input service record
 *          matches an existing record in the BSL object, then the
 *          existing record is over-wrote with the new information,
 *          otherwise a new record is created in the BSL database.
 *
 *          If the 'b_must_exist' flag is TRUE, then there must be an
 *          existing record with the same key field value, else a status
 *          of 'BSLR_REC_NOT_FOUND' is returned to the caller.
 *          If 'b_must_exist' flag is FALSE, then a new service record
 *          is created if no existing record matching the key field value
 *          is found in the BSL database.
 *
 *          Call to this function must be protected, e.g., the BSL
 *          database object needs to be locked before the call to this
 *          function and unlock after returning from call to this function.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [in]  pt_bsl_rec       - Pointer to service record structure.
 *  @param  [in]  b_must_exist     - Boolean flag specifying if the service
 *                                   record to be updated should exist in the
 *                                   BSL database.
 *  @return Return the error code
 *  @retval BSLR_OK            - Record is successfully added/updated.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 *  @retval BSLR_OUT_OF_MEM    - Not enough memory to create a new BSL record.
 *  @retval BSLR_REC_NOT_FOUND - The specified record is not found
 *                               ('b_must_exist' == TRUE)
 *  @retval BSLR_INV_ARG       - 'pt_bsl_rec' is NULL or the pt_bsl_rec
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_update_rec
(
    HANDLE_T            h_bsl,
    const BSL_REC_T*    pt_bsl_rec,
    BOOL                b_must_exist  
);


/*------------------------------------------------------------------*/
/*! @brief  This function locks the BSL object.   The function blocks
 *          until the lock is acquired.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @see    c_bsl_unlock, c_bsl_read_lock
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL is locked.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_lock
(
    HANDLE_T            h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function unlocks the BSL database.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @see    c_bsl_lock
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL database is unlocked.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_unlock
(
    HANDLE_T            h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function acquires the read-lock on the BSL object.
 *          The function blocks until the lock is acquired.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @see    c_bsl_read_unlock, c_bsl_lock
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL is locked.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 */
/*------------------------------------------------------------------*/
extern INT32 c_bsl_read_lock
(
    HANDLE_T            h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function releases the read-lock on the BSL database.
 *  @param  [in]  h_bsl        - Handle to the BSL object.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL database is unlocked.
 *  @retval BSLR_INV_HANDLE    - Bad BSL handle.
 */
/*------------------------------------------------------------------*/
extern INT32 c_bsl_read_unlock
(
    HANDLE_T            h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This API loads a specified BSL object from file system
 *          into memory, and returns a handle to the newly created
 *          BSL object.
 *  @param  [in]  h_dir             - Handle to the parent directory of the file
 *                                    specified by the path 'ps_bsl_path'. If
 *                                    'ps_bsl_path' is an absolute path, then
 *                                    this argument should be set to
 *                                    'FM_ROOT_HANDLE'.
 *  @param  [in]  ps_bsl_path      - Specifies the path to the file where the
 *                                    BSL objects are stored in the file system.
 *  @param  [in]  ui2_bsl_id       - Specifies the BSL id to load from the file
 *                                    system.  The limit for BSL ID is between 1
 *                                    and 65535.
 *  @param  [in]  ps_bsl_name      - Specifies the name for the BSL object.  If
 *                                    null string is specified, then the default
 *                                    name stored in the file system is used to
 *                                    create the BSL object.
 *  @param  [in]  pv_tag            - Pointer to a private tag value provided by
 *                                    the caller.  This tag value is passed to
 *                                    the caller's notification function.
 *  @param  [in]  pf_nfy            - Notification function provided by the
 *                                    caller.  This function is called to notify
 *                                    the owner of this BSL handle when the BSL
 *                                    object has been modified by another
 *                                    client.  If NULL, then no notification
 *                                    function will be called.
 *  @param  [out] ph_bsl           - On return, contain the handle to the BSL
 *                                    object created from file system.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL object is created.
 *  @retval BSLR_NOT_FOUND     - 'ui2_bsl_id' is not found or 'ps_bsl_path'
 *                                does not exist.
 *  @retval BSLR_OUT_OF_HANDLE - Cannot get a handle.
 *  @retval BSLR_OUT_OF_MEM    - Not enough memory to create BSL object.
 *  @retval BSLR_FAIL          - The specified 'ps_bsl_name' or 'ui2_bsl_id'
 *                                value is not unique.
 *  @retval BSLR_INV_ARG       - 'ph_bsl' or 'ps_bsl_path' is NULL.
 *  @retval BSLR_AEE_OUT_OF_RESOURCES  - Can not allocate AEE resources.
 *  @retval BSLR_AEE_NO_RIGHTS - No right for the AEE resources.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_fs_load
(
    HANDLE_T            h_dir,
    const CHAR*         ps_bsl_path,
    UINT16              ui2_bsl_id,
    const CHAR*         ps_bsl_name,
    VOID*               pv_tag,
    x_bsl_nfy_fct       pf_nfy,
    HANDLE_T*           ph_bsl
);


/*------------------------------------------------------------------*/
/*! @brief  This API stores the specified BSL object to the file system.
 *  @param  [in]  h_dir             - Handle to the parent directory of the file
 *                                    specified by the path 'ps_bsl_path'. If
 *                                    'ps_bsl_path' is an absolute path, then
 *                                    this argument should be set to
 *                                    'FM_ROOT_HANDLE'.
 *  @param  [in]  ps_bsl_path      - Specifies the file path to the file where
 *                                    the BSL objects are stored in the file
 *                                    system.
 *  @param  [in]  ui2_bsl_id       - Specifies the  ID of the BSL object to be
 *                                    stored to the file system.  The limit for
 *                                    BSL ID is between 1 and 65535.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL object is stored to the file system.
 *  @retval BSLR_NOT_FOUND     - The BSL object specified by the 'ui2_bsl_id'
 *                                does not exist.
 *  @retval BSLR_INV_ARG       - 'ps_bsl_path' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_fs_store
(
    HANDLE_T            h_dir,
    const CHAR*         ps_bsl_path,
    UINT16              ui2_bsl_id
);


/*------------------------------------------------------------------*/
/*! @brief  This API deletes a specified BSL object on the file system.
 *  @param  [in]  h_dir             - Handle to the parent directory of the file
 *                                    specified by the path 'ps_bsl_path'. If
 *                                    'ps_bsl_path' is an absolute path, then
 *                                    this argument should be set to
 *                                    'FM_ROOT_HANDLE'.
 *  @param  [in]  ps_bsl_path      - Specifies the file path to the file where
 *                                    the BSL objects are stored in the file
 *                                    system.
 *  @param  [in]  ui2_bsl_id       - Specifies the ID of the BSL object to be
 *                                    deleted from the file system.  The limit
 *                                    for BSL ID is between 1 and 65535.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL object is deleted from the file system.
 *  @retval BSLR_NOT_FOUND     - 'ui2_bsl_id' is not found or 'ps_bsl_path'
 *                                does not exist.
 *  @retval BSLR_INV_ARG       - 'ps_bsl_path' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_fs_del
(
    HANDLE_T            h_dir,
    const CHAR*         ps_bsl_path,
    UINT16              ui2_bsl_id
);

/*------------------------------------------------------------------*/
/*! @brief  This API queries the BSL objects stored in file system.
 *  @param  [in]  h_dir             - Handle to the parent directory of the file
 *                                    specified by the path 'ps_bsl_path'. If
 *                                    'ps_bsl_path' is an absolute path, then
 *                                    this argument should be set to
 *                                    'FM_ROOT_HANDLE'.
 *  @param  [in]  ps_bsl_path      - Specifies the file path to the file where
 *                                    the BSL objects are stored in the file
 *                                    system..
 *  @param  [in]  ui2_idx           - Index value to get the next BSL object. 0
 *                                    means the first BSL, 1 is the second BSL,
 *                                    etc.
 *  @param  [out] pui2_bsl_id      - Pointer to a UINT16 variable. On return,
 *                                    it contains BSL ID.
 *  @param  [out] ps_bsl_name      - Pointer to a string buffer (minimum 17
 *                                    bytes) .  On return, it contains the
 *                                    default BSL name stored on the file
 *                                    system.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL object found.
 *  @retval BSLR_NOT_FOUND     - No BSL object found.
 *  @retval BSLR_INV_ARG       - 'ps_bsl_name' or 'pui2_bsl_id' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_fs_qry
(
    HANDLE_T            h_dir,
    const CHAR*         ps_bsl_path,
    UINT16              ui2_idx,
    UINT16*             pui2_bsl_id,
    CHAR*               ps_bsl_name
);

/*------------------------------------------------------------------*/
/*! @brief  This API sync a specified BSL handle with BSL records
 *          from persistent file system.  The existing BSL records
 *          are replaced with new records read from file.
 *  @param  [in]  h_bsl            - Specify the handle to the BSL object.
 *  @param  [in]  h_dir            - The handle to the directory.  The
 *                                    'ps_bsl_path' will be relative to the
 *                                    specified 'h_dir'.
 *  @param  [in]  ps_bsl_path      - Specifies the directory path of the BSL
 *                                    object in the file system. See File Path
 *                                    Specification, 10.1, for the types of file
 *                                    path that can be specified.
 *  @param  [in]  ui2_bsl_id       - Specifies the BSL id to load from the file
 *                                    system.  The limit for BSL ID is between 1
 *                                    and 65535.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL records are load from the persistent file
 *                                into the memory of BSL object.
 *  @retval BSLR_NOT_FOUND     - 'ui2_bsl_id' is not found or 'ps_bsl_path'
 *                                does not exist.
 *  @retval BSLR_FAIL          - internal error
 *  @retval BSLR_INV_ARG       - 'ps_bsl_path' is NULL, or 'h_bsl' is
 *                                NULL_HANDLE, or ui2_bsl_id is 0.
 *  @retval BSLR_INV_FILE_PATH - Specified file path is not a directory or a
 *                                RAW character file.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_fs_sync
(
    HANDLE_T      h_bsl,
    HANDLE_T      h_dir,
    const CHAR*   ps_bsl_path,
    UINT16        ui2_bsl_id
);

/*------------------------------------------------------------------*/
/*! @brief  This function gets a handle to the specified BSL database
 *          object by its ID value.
 *  @param  [in]  ui2_bsl_id       - Specifies the BSL object identifier value.
 *                                    The limit for BSL ID is between 1 and
 *                                    65535.
 *  @param  [in]  pv_tag            - A private tag value associated with the
 *                                    calling client.
 *  @param  [in]  pf_nfy            - Pointer to the caller provided
 *                                    notification function.  The notification
 *                                    function will be called in the context of
 *                                    the thread that modifies (update, add,
 *                                    delete) the BSL database.  If NULL, then
 *                                    no notification function will be called.
 *  @param  [out] ph_bsl           - Pointer to the handle created.
 *  @see    c_bsl_open_by_name
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL handle is returned.
 *  @retval BSLR_OUT_OF_HANDLE - No more handle available.
 *  @retval BSLR_OUT_OF_MEM    - Not enough memory to create the BSL control
 *                                structure.
 *  @retval BSLR_NOT_FOUND     - The specified BSL database object id (e.g.
 *                                'ui2_bsl_id') is not found.
 *  @retval BSLR_INV_ARG       - 'ph_bsl' is NULL.
 *  @retval BSLR_AEE_OUT_OF_RESOURCES  - Can not allocate AEE resources.
 *  @retval BSLR_AEE_NO_RIGHTS - No right for the AEE resources.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_open
(
    UINT16              ui2_bsl_id,
    VOID*               pv_tag,
    x_bsl_nfy_fct       pf_nfy,
    HANDLE_T*           ph_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function gets a handle to the specified BSL database
 *          object by its name.
 *  @param  [in]  ps_bsl_name      - Specifies the BSL object name (16 char
 *                                    max).
 *  @param  [in]  pv_tag           - A private tag value associated with the
 *                                    calling client.
 *  @param  [in]  pf_nfy           - Pointer to a caller provided notification
 *                                    function.  The notification function is
 *                                    called in the context of the thread that
 *                                    modifies (update, add, delete) the BSL
 *                                    database.  If NULL, then no notification
 *                                    function will be called.
 *  @param  [out] ph_bsl           - Pointer to the handle created.
 *  @see    c_bsl_open
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL handle is returned.
 *  @retval BSLR_OUT_OF_HANDLE - No more handle available.
 *  @retval BSLR_OUT_OF_MEM    - Not enough memory to create the BSL
 *                                handle structure.
 *  @retval BSLR_NOT_FOUND     - The specified BSL database object name (e.g.,
 *                                ps_bsl_name) is not found.
 *  @retval BSLR_INV_ARG       - 'ph_bsl' or 'ps_bsl_name' is NULL.
 *  @retval BSLR_AEE_OUT_OF_RESOURCES  - Can not allocate AEE resources.
 *  @retval BSLR_AEE_NO_RIGHTS - No right for the AEE resources.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_open_by_name
(
    const CHAR*         ps_bsl_name,
    VOID*               pv_tag,
    x_bsl_nfy_fct       pf_nfy,
    HANDLE_T*           ph_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function frees the specified handle to the BSL
 *          database object.
 *  @param  [in]  h_bsl        - Handle to BSL database object.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL handle is freed.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_close
(
    HANDLE_T    h_bsl
);

/*------------------------------------------------------------------*/
/*! @brief  This function gets the BSL ID and BSL name for the
 *          specified handle.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [out] pui2_bsl_id      - Pointer to BSL ID variable.  On return it
 *                                    contains the BSL ID value for this handle.
 *  @param  [out] ps_bsl_name      - Pointer to a char array variable (the size
 *                                    of this char array should be at least
 *                                    MAX_BSL_NAME_LEN+1). On return, this
 *                                    array contains the name of the BSL object.
 *  @return Return the error code
 *  @retval BSLR_OK            - Information about this handle (bsl id and bsl
 *                                name) is returned.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 *  @retval BSLR_INV_ARG       - 'pui2_bsl_id' or 'ps_bsl_name' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_get_handle_info
(
    HANDLE_T            h_bsl,
    UINT16*             pui2_bsl_id,
    CHAR*               ps_bsl_name
);

/*------------------------------------------------------------------*/
/*! @brief  This API gets a specific record stored in the BSL database
 *          object.    The search parameter is specified by the
 *          'ui2_bsl_rec_id' value.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [in]  ui2_bsl_rec_id   - Specifies the service record id.
 *  @param  [out] pt_bsl_rec       - Pointer to a service record structure.
 *                                    This structure must be allocated by the
 *                                    caller.  On return, it is filled with
 *                                    service record information.
 *  @param  [out] pui4_ver_id      - Pointer to a variable holding version id
 *                                    of the database object.
 *  @note   Version id of the BSL database object is incremented each time when
 *          a modification takes place.
 *  @return Return the error code
 *  @retval BSLR_OK            - Data from the specified record is returned.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 *  @retval BSLR_REC_NOT_FOUND - No record matching the search parameter
 *                                ('ui2_bsl_rec_id') is found.
 *  @retval BSLR_INV_ARG       - 'pui4_ver_id' or 'pt_bsl_rec' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_get_rec
(
    HANDLE_T            h_bsl,
    UINT16              ui2_bsl_rec_id,
    BSL_REC_T*          pt_bsl_rec,
    UINT32*             pui4_ver_id
);

/*------------------------------------------------------------------*/
/*! @brief  This function returns the number of records for the
 *          specified BSL.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [out] pui2_num_recs    - Pointer to a variable for the number of
 *                                    records in the BSL.
 *  @param  [out] pui4_ver_id      - Pointer to a variable holding version id
 *                                    of the BSL database object.
 *  @return Return the error code
 *  @retval BSLR_OK            - Number of records in the specified BSL is
 *                                returned.
 *  @retval BSLR_INV_HANDLE    - Bad 'h_bsl' handle.
 *  @retval BSLR_INV_ARG       - 'pui2_num_recs' or 'pui4_ver_id' is a NULL
 *                                pointer.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_get_num_recs
(
    HANDLE_T            h_bsl,
    UINT16*             pui2_num_recs,
    UINT32*             pui4_ver_id
);

/*------------------------------------------------------------------*/
/*! @brief  This API returns satellite list record entry from the
 *          BSL database object by its index. Maximum index is obtained
 *          by calling c_bsl_get_num_recs.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [in]  ui2_idx          - An idx to get the next BSL record.
 *                                    For example: 0 will the first BSL record,
 *                                    and 1 will be second, and so on.
 *  @param  [out] pt_bsl_rec       - Pointer to a service record structure.
 *                                    This structure must be allocated by the
 *                                    caller.  On return, it is filled with
 *                                    service record information.
 *  @param  [out] pui4_ver_id      - Pointer to a variable holding version id
 *                                    of the database object.
 *  @note   Version id of the BSL database object is incremented each time when
 *          a modification takes place.
 *  @return Return the error code
 *  @retval BSLR_OK            - Data from the specified record is returned.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 *  @retval BSLR_REC_NOT_FOUND - No record matching the search index
 *                                ('ui2_idx') is found.
 *  @retval BSLR_INV_ARG       - 'pui4_ver_id' or 'pt_bsl_rec' is NULL.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_get_rec_by_rec_idx
(
    HANDLE_T            h_bsl,
    UINT16              ui2_idx,
    BSL_REC_T*          pt_bsl_rec,
    UINT32*             pui4_ver_id
);


extern
INT32 c_bsl_get_num_rec_by_header
(
    HANDLE_T            h_bsl,
    BSL_REC_HEADER_T*   pt_rec_hdr,
    UINT16*             pui2_num_recs,
    UINT32*             pui4_ver_id
);

extern
INT32 c_bsl_get_rec_by_header
(
    HANDLE_T            h_bsl,
    BSL_REC_HEADER_T*   pt_rec_hdr,
    BSL_REC_T*          pt_bsl_rec,
    UINT16              ui2_idx,
    UINT32*             pui4_ver_id
);

/*------------------------------------------------------------------*/
/*! @brief  This API pre-sets the BSL_REC_T structure to a
 *          pre-defined invalid value (e.g., 0xFF).   The pre-defined invalid
 *          value can be used to detect changes in the BSL_REC_T structure for
 *          backward compatibility in the future.
 *
 *          For example, if a new field is added to the BSL record structure.
 *          At the runtime, if the new field in the BSL_REC_T structure passed
 *          into BSL library contain 0xFF, then we know the caller has not been
 *          re-compile with the new BSL module.
 *  @param  [out] pt_bsl_rec   - Pointer to a BSL record structure.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL library is initialized successfully.
 *  @retval BSLR_INV_ARG       - 'pt_bsl_rec' is a NULL pointer.
 */
/*------------------------------------------------------------------*/
extern
INT32 c_bsl_rec_init
(
    BSL_REC_T*    pt_bsl_rec
);

/*---------------------------------------------------------------a---*/
/*! @brief  This API specifies a entry position in BSL records,
 *          which will read when API c_bsl_read_cursor() is called.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [in]  e_pos            - Position of the BSL record to read.
 *  @see    c_bsl_read_cursor
 *  @return Return the error code
 *  @retval BSLR_OK            - Data from the specified record is returned.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 *  @retval BSLR_INV_ARG       - 'e_pos' is invalid.
 */
/*------------------------------------------------------------------*/
extern INT32 c_bsl_set_cursor
(
    HANDLE_T            h_bsl,
    BSL_ITERATE_T       e_pos
);

/*------------------------------------------------------------------*/
/*! @brief  This API read the content of a specific record
 *          stored in the BSL database.  The specific record is read
 *          is the record currently pointed by the cursor.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [out] pt_bsl_rec       - Pointer to a service record structure.
 *                                    This structure must be allocated by the
 *                                    caller.  On return, it is filled with
 *                                    service record information.
 *  @param  [out] pui4_ver_id      - Pointer to a variable holding version id
 *                                    of the database object.
 *  @note   Version id of the BSL database object is incremented each time when
 *          a modification takes place.
 *  @see    c_bsl_set_cursor
 *  @return Return the error code
 *  @retval BSLR_OK            - Data from the specified record is returned.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 *  @retval BSLR_INV_ARG       - 'pui4_ver_id' or 'pt_bsl_rec' is NULL.
 */
/*------------------------------------------------------------------*/
extern INT32 c_bsl_read_cursor
(
    HANDLE_T            h_bsl,
    BSL_REC_T*          pt_bsl_rec,
    UINT32*             pui4_ver_id
);

/*------------------------------------------------------------------*/
/*! @brief  This API applies the sort operation (using the
 *          per-handle compare fucntion, or the global compare
 *          function, or default compare function, to the BSL
 *          records.
 *  @param  [in]  h_bsl            - Handle to the BSL object.
 *  @param  [out] pui4_ver_id      - Contains the version id of the BSL
 *                                    database object.
 *  @return Return the error code
 *  @retval BSLR_OK            - BSL record are sorted and internal sort list
 *                                is updated.
 *  @retval BSLR_INV_HANDLE    - Bad handle.
 */
/*------------------------------------------------------------------*/
extern INT32 x_bsl_sort_rec
(
    HANDLE_T      h_bsl,
    UINT32*       pui4_ver_id 
);

/*----------------------------------------------------------------------------*/
/*! @} */
/*----------------------------------------------------------------------------*/
#endif /* _C_BSL_H_ */
