/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: c_cfg_parser.h,v $
 * $Revision: #1 $
 * $Date: 2016/9/26 $
 * $Author: yulong.jin $
 *
 * Description:
 *         This file contains public config parser function for AP query all configs
 *-----------------------------------------------------------------*/

#ifndef _C_CFG_PARSER_H_
#define _C_CFG_PARSER_H_
/*-----------------------------------------------------------------------------
 *  include files
 *---------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_cfg_parser.h"
#include "u_os.h"

/*-----------------------------------------------------------------------------
* get macro_name already configed or not
* input:
*   macro_name: the name of macro, such as MW_TV_AV_BYPASS_SUPPORT
* return value:
*   TRUE: macro_name is defined
*   FALSE: macro_name not defined
-----------------------------------------------------------------------------*/
extern BOOL c_cfg_parser_feature_is_enable(const CHAR* macro_name );

/*-----------------------------------------------------------------------------
* get macro_name value
* input:
*   macro_name: the name of macro, such as MW_TV_AV_BYPASS_SUPPORT
* return value:
*   the value of macro_name
-----------------------------------------------------------------------------*/
extern const CHAR* c_cfg_parser_get_config_value(const CHAR* macro_name);

/*-----------------------------------------------------------------------------
* get define_string value for type "int"
* input:
*   macro_name: the name of macro, such as MW_TV_AV_BYPASS_SUPPORT
*   macro_value: An INT32 pointer for macro_value, only valid when return is TRUE
* return value:
*   TRUE: the value of macro_value is valid
*    FALSE: the value of macro_value is invalid, macro not defined or macro has no value
-----------------------------------------------------------------------------*/
extern BOOL c_cfg_parser_get_config_value_int32(const CHAR* macro_name,  INT32* macro_value);

/*-----------------------------------------------------------------------------
* set macro_name and value
* return value:
*   TRUE: set sucess
*   FALSE: set fail
-----------------------------------------------------------------------------*/
extern INT32 c_cfg_parser_set_config_value(const CHAR* macro_name, const CHAR* macro_value);

/*-----------------------------------------------------------------------------
* delete macro_name and value
* return value:
*   TRUE: del sucess
*   FALSE: del fail
-----------------------------------------------------------------------------*/
extern INT32 c_cfg_parser_del_config(const CHAR* macro_name);
#endif