/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_chip_spec.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Chip Specification Library related
 *         definitions, which are exported.
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_CHIP_SPEC_H_
#define _C_CHIP_SPEC_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "x_common.h"
#include "u_chip_spec.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief This API gets QianJiaYue bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_QianJiaYue_bond (BOOL* pb_is_bond_enable);

/*------------------------------------------------------------------*/
/*! @brief This API gets IPTV bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_IPTV_bond (BOOL* pb_is_bond_enable);

/*------------------------------------------------------------------*/
/*! @brief This API gets PlayRDY bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_PlayRDY_bond (BOOL* pb_is_bond_enable);

/*------------------------------------------------------------------*/
/*! @brief This API gets MSS bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_MSS_bond (BOOL* pb_is_bond_enable);
/*------------------------------------------------------------------*/
/*! @brief This API gets specific chip bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_OPERA_bond (BOOL* pb_is_bond_enable);
/*------------------------------------------------------------------*/
/*! @brief This API gets specific chip bonding status.
 *
 *  @param [in] none
 *
 *  @param [out] pb_is_bond_enable - References the chip bonding status.
 *
 *  @note
 *  @see
 *
 *  @return
 *  @retval  CHIPSPECR_OK                - The routine was successful.
 *  @retval  CHIPSPECR_NOT_INIT          - The Chip Specification Library is not initialized.
 *  @retval  CHIPSPECR_HW_NOT_SUPPORT    - The hardware is not supported.
 *  @retval  CHIPSPECR_DEV_GET_FAILED    - Device get operation is failed.
 *  @retval  CHIPSPECR_INV_ARG            One of input parameter is invalid.
 *  @retval  CHIPSPECR_UNKNOWN            Bonding status is unknow.
 */
/*------------------------------------------------------------------*/
extern INT32 c_chip_spec_get_IPTV_DLNA_bond (BOOL* pb_is_bond_enable);


#ifdef __cplusplus
}
#endif

#endif /* _C_CHIP_SPEC_H_ */

