/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_dsmcc_oc.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains DSM-CC OC specific definitions, which
 *         are exported.
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_DSMCC_OC_H_
#define _C_DSMCC_OC_H_


/*---------------------------------------------------------------------------
                    include files
---------------------------------------------------------------------------*/

#include "c_handle_grp.h"
#include "c_common.h"
#include "c_tm.h"
#include "u_dsmcc_oc.h"


/*---------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
/*! @brief  Open a SRG given a PID and TS descriptor for the DSI object.
 *  @param  [in]  - References the root table object.
 *  @param  [in]  - Contains the PID value for the DSI.
 *  @param  [in]  - References the TS decsriptior.
 *  @param  [in]  - Specifies the source type for loading the requested
 *                  table.
 *  @param  [in]  - Contains additional information describing the source.
 *  @param  [in]  - Contains a notify tag value.
 *  @param  [in]  - References the notify function.
 *  @param  [out] - Contains a handle which references the SRG object.
 *  @param  [out] - Contains the condition of the SRG object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_open_srg_by_pid_ts_descr (HANDLE_T       h_root,
                                                  MPEG_2_PID_T   t_pid,
                                                  TS_DESCR_T*    pt_ts_descr,
                                                  TM_SRC_TYPE_T  e_src_type,
                                                  VOID*          pv_src_info,
                                                  VOID*          pv_nfy_tag,
                                                  x_tm_nfy_fct   pf_nfy,
                                                  HANDLE_T*      ph_srg,
                                                  TM_COND_T*     pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Open a SRG given a PID and TS id for the DSI object.
 *  @param  [in]  h_root      - References the root table object.
 *  @param  [in]  t_pid       - Contains the PID value for the DSI.
 *  @param  [in]  ui2_ts_id   - Contains the TS id.
 *  @param  [in]  e_src_type  - Specifies the source type for loading the requested
 *                              table.
 *  @param  [in]  pv_src_info - Contains additional information describing the source. 
 *  @param  [in]  pv_nfy_tag  - Contains a notify tag value.
 *  @param  [in]  pf_nfy      - References the notify function.
 *  @param  [out] ph_srg      - Contains a handle which references the SRG object.
 *  @param  [out] pe_cond     - Contains the condition of the SRG object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_open_srg_by_pid_ts_id (HANDLE_T       h_root,
                                               MPEG_2_PID_T   t_pid,
                                               UINT16         ui2_ts_id,
                                               TM_SRC_TYPE_T  e_src_type,
                                               VOID*          pv_src_info,
                                               VOID*          pv_nfy_tag,
                                               x_tm_nfy_fct   pf_nfy,
                                               HANDLE_T*      ph_srg,
                                               TM_COND_T*     pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API allows a client to parse a SRG's private data.
 *  @param  [in]  h_dsi      - Contains a handle referencing an SRG or DSI object.
 *  @param  [in]  pv_data    - References private data which is passed to the parsing
 *                             function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced by
 *                             argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                   data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_parse_srg_private_data (HANDLE_T        h_dsi,
                                                VOID*           pv_data,
                                                SIZE_T          z_data_len,
                                                x_tm_parse_fct  pf_parse,
                                                TM_COND_T*      pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Get subentry information given a handler referencing a SRG
 *          or DIR object.
 *  @param  [in]  h_obj            - Contains a handle referencing a SRG or DIR object.
 *  @param  [in]  ui2_idx          - Contains an index. A value of '0' will return
 *                                   information of the first sub-entry, a value of '1'
 *                                   information of the second sub-entry and so on.
 *  @param  [out] pt_subentry_info - Contains the subentry information.
 *  @param  [out] pe_cond          - Contains the SRG handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_get_subentry_info (HANDLE_T                   h_obj,
                                           UINT16                     ui2_idx,
                                           DSMCC_OC_SUBENTRY_INFO_T*  pt_subentry_info,
                                           TM_COND_T*                 pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Get number of subentries given a handle referencing a SRG or
 *          DIR object.
 *  @param  [in]  h_obj    - Contains a handle referencing a SRG or DIR object.
 *  @param  [out] pui2_cnt - Contains the number of subentries.
 *  @param  [out] pe_cond  - Contains the SRG handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_get_subentry_cnt (HANDLE_T    h_obj,
                                          UINT16*     pui2_cnt,
                                          TM_COND_T*  pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Open a DIR by name.
 *  @param  [in]  h_obj      - Contains the handle referencing a SRG or DIR object.
 *  @param  [in]  ps_name    - References a zero terminated string containing the
 *                             DIR name.
 *  @param  [in]  pv_nfy_tag - Tag to attach to this DIR request
 *  @param  [in]  pf_nfy     - Notification function to call
 *  @param  [out] ph_dir     - Contains a handle referencing the DIR object.
 *  @param  [out] pe_cond    - Contains the DIR handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_open_dir_by_name (HANDLE_T      h_obj,
                                          const CHAR*   ps_name,
                                          VOID*         pv_nfy_tag,
                                          x_tm_nfy_fct  pf_nfy,
                                          HANDLE_T*     ph_dir,
                                          TM_COND_T*    pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Open a FILE by name.
 *  @param  [in]  h_obj        - Contains the handle referencing a SRG or DIR
 *                               object.
 *  @param  [in]  ps_name      - References a zero terminated string containing the
 *                               DIR name.
 *  @param  [in]  b_file_cache - Will pre-load the file in memory if set to TRUE.
 *  @param  [in]  pv_nfy_tag   - Tag to attach to this FILE request
 *  @param  [in]  pf_nfy       - Notification function to call
 *  @param  [out] ph_file      - Contains a handle referencing the FILE object.
 *  @param  [out] pe_cond      - Contains the FILE handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_open_file_by_name (HANDLE_T      h_obj,
                                           const CHAR*   ps_name,
                                           BOOL          b_file_cache,
                                           VOID*         pv_nfy_tag,
                                           x_tm_nfy_fct  pf_nfy,
                                           HANDLE_T*     ph_file,
                                           TM_COND_T*    pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Reads a next chunk of data from a FILE object. This function
 *          is synchronous which means that if the DSMCC OC engine needs
 *          to load modules this API will wait.
 *  @param  [in]  - Contains the handle referencing the FILE object.
 *  @param  [in]  - The offset within the file.
 *  @param  [in]  - Contains the length of the buffer in argument
 *                  'pui1_buff'.
 *  @param  [out] - Contains the read data.
 *  @param  [out] - Contains the number of read Bytes.
 *  @param  [out] - Contains the FILE handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_read_file (HANDLE_T    h_file,
                                   UINT64      ui8_offs,
                                   SIZE_T      z_buff_len,
                                   UINT8*      pui1_buff,
                                   SIZE_T*     pz_buff_read,
                                   TM_COND_T*  pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  Reads a next chunk of data from a FILE object. This function
 *          is synchronous which means that if the DSMCC OC engine needs
 *          to load modules this API will wait. In case the data could
 *          be loaded within a specific time specified in argument
 *          'ui4_timeout' this API will return an error.
 *  @param  [in]  h_file       - Contains the handle referencing the FILE object.
 *  @param  [in]  ui8_offs     - The offset within the file.
 *  @param  [in]  z_buff_len   - Contains the length of the buffer in argument
 *                               'pui1_buff'.
 *  @param  [in]  pui1_buff    - Contains the read data.
 *  @param  [out] pz_buff_read - Contains the number of read Bytes.
 *  @param  [out] ui4_timeout  - Timeout in msec.
 *  @param  [out] pe_cond      -Contains the FILE handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_read_file_timeout (HANDLE_T    h_file,
                                           UINT64      ui8_offs,
                                           SIZE_T      z_buff_len,
                                           UINT8*      pui1_buff,
                                           SIZE_T*     pz_buff_read,
                                           UINT32      ui4_timeout,
                                           TM_COND_T*  pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  Reads a file in asynchronous mode. This API assumes that the
 *          client keeps the memory buffer available until the read
 *          operation has completed or the asynchronous read handle is
 *          deleted.
 *  @param  [in]  h_file        - Contains the handle referencing the FILE object.
 *  @param  [in]  ui8_offs      - The offset within the file.
 *  @param  [in]  z_buff_len    - Contains the length of the buffer in argument
 *                                'pui1_buff'.
 *  @param  [out] pui1_buff     -  Contains the read data.
 *  @param  [in]  pv_nfy_tag    - Tag to attach to this asynchronous read request
 *  @param  [in]  pf_nfy        - Notification function to call
 *  @param  [out] ph_async_read - Contains a handle referencing the asynchronous
 *                                read request.
 *  @param  [out] pe_cond       - Contains the asynchronous read condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_read_file_async (HANDLE_T      h_file,
                                         UINT64        ui8_offs,
                                         SIZE_T        z_buff_len,
                                         UINT8*        pui1_buff,
                                         VOID*         pv_nfy_tag,
                                         x_tm_nfy_fct  pf_nfy,
                                         HANDLE_T*     ph_async_read,
                                         TM_COND_T*    pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Get file information given a handle referencing a FILE
 *          object.
 *  @param  [in]  - h_file       - Contains a handle referencing a FILE object.
 *  @param  [out] - pt_file_info - Contains the file information.
 *  @param  [out] - pe_cond      - Contains the FILE handle condition.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid
 *                                  data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_oc_get_file_info (HANDLE_T               h_file,
                                       DSMCC_OC_FILE_INFO_T*  pt_file_info,
                                       TM_COND_T*             pe_cond);


#endif /* _C_DSMCC_OC_H_ */

