/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------*/
/*! @file c_evctx.h 
 * $RCSfile: c_evctx.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/1 $
 * $SWAuthor: Justin Deng $
 * $MD5HEX: 17cb15bcc75e0c0b8477b3ca4edeb294 $
 *
 *  @par Description:
 *         This contains Event Context library macros, definitons, structures,
 *         and API's which are exported to applications.
 *---------------------------------------------------------------------------*/

#ifndef _C_EVCTX_H_
#define _C_EVCTX_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_EVCTX
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_evctx.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @brief  load current event function

 *  @param  [in] pt_svc_cmd          - evctx command
 *  @param  [in] pv_nfy_tag          - notify tag
 *  @param  [in] pf_nfy              - notify function
 *  @param  [out] ph_event           - pointer of event object handle
 *  @param  [out] pe_cond            - pointer of event condition
 *  
 *  @note   
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_load_current_event (
                    EVCTX_COMMAND_T*    pt_svc_cmd,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_load_current_event_64 (
                    EVCTX_COMMAND_EX_T*    pt_svc_cmd,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  load next event function

 *  @param  [in] pt_svc_cmd          - evctx command
 *  @param  [in] pv_nfy_tag          - notify tag
 *  @param  [in] pf_nfy              - notify function
 *  @param  [out] ph_event           - pointer of event object handle
 *  @param  [out] pe_cond            - pointer of event condition
 *  
 *  @note   
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_load_next_event (
                    EVCTX_COMMAND_T*    pt_svc_cmd,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);


extern INT32 c_evctx_load_next_event_64 (
                    EVCTX_COMMAND_EX_T*    pt_svc_cmd,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  load schedule event function

 *  @param  [in] pt_svc_cmd          - evctx command
 *  @param  [in] pt_offset           - evctx offset
 *  @param  [in] pv_nfy_tag          - notify tag
 *  @param  [in] pf_nfy              - notify function
 *  @param  [out] ph_event           - pointer of event object handle
 *  @param  [out] pe_cond            - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_load_schedule_event (
                    EVCTX_COMMAND_T*    pt_svc_cmd,
                    EVCTX_OFFSET_T*     pt_offset,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_load_schedule_event_64 (
                    EVCTX_COMMAND_EX_T*    pt_svc_cmd,
                    EVCTX_OFFSET_T*     pt_offset,
                    VOID*               pv_nfy_tag,
                    x_evctx_nfy_fct     pf_nfy,
                    HANDLE_T*           ph_event,
                    EVCTX_COND_T*       pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  delete event function

 *  @param  [in] h_event          - handle of evctx object
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_delete_event (
                    HANDLE_T            h_event);


/*----------------------------------------------------------------------------*/
/*! @brief  get event condition function

 *  @param  [in] h_event                - handle of evctx object
 *  @param  [out] pe_cond               - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_get_cond (
                    HANDLE_T            h_event,
                    EVCTX_COND_T*       pe_cond);


/*----------------------------------------------------------------------------*/
/*! @brief  get event info length function

 *  @param  [in] h_event                - handle of evctx object
 *  @param  [in] e_key_type             - event key type
 *  @param  [in] pv_key_info            - event key information
 *  @param  [out] pz_event_info_len     - pointer of event information length
 *  @param  [out] pe_cond               - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_get_event_info_len (
                    HANDLE_T            h_event,
                    EVCTX_KEY_TYPE_T    e_key_type,
                    VOID*               pv_key_info,
                    SIZE_T*             pz_event_info_len,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_get_event_info_len_64 (
                    HANDLE_T            h_event,
                    EVCTX_KEY_TYPE_T    e_key_type,
                    VOID*               pv_key_info,
                    UINT64*             pz_event_info_len,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  get event info function

 *  @param  [in] h_event                - handle of evctx object
 *  @param  [in] e_key_type             - event key type
 *  @param  [in] pv_key_info            - event key information
 *  @param  [out] pz_event_info_len     - pointer of event information length
 *  @param  [out] pz_event_info_len     - pointer of event information length
 *  @param  [out] pv_event_info         - pointer of event information
 *  @param  [out] pe_cond               - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_get_event_info (
                    HANDLE_T            h_event,
                    EVCTX_KEY_TYPE_T    e_key_type,
                    VOID*               pv_key_info,
                    SIZE_T*             pz_event_info_len,
                    VOID*               pv_event_info,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_get_event_info_64 (
                    HANDLE_T            h_event,
                    EVCTX_KEY_TYPE_T    e_key_type,
                    VOID*               pv_key_info,
                    UINT64*             pz_event_info_len,
                    VOID*               pv_event_info,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  get event brief info length function

 *  @param  [in] h_event                - handle of evctx object
 *  @param  [in] pt_iso_639_lang        - pointer of iso 639 language structure
 *  @param  [out] pz_brief_info_len     - pointer of event information length
 *  @param  [out] pe_cond               - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_get_brief_info_len (
                    HANDLE_T            h_event,
                    ISO_639_LANG_T*     pt_iso_639_lang,
                    SIZE_T*             pz_brief_info_len,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_get_brief_info_len_64 (
                    HANDLE_T            h_event,
                    ISO_639_LANG_T*     pt_iso_639_lang,
                    UINT64*             pz_brief_info_len,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  get event brief info function

 *  @param  [in] h_event                - handle of evctx object
 *  @param  [in] pt_iso_639_lang        - pointer of iso 639 language structure
 *  @param  [in] pt_iso_639_lang        - pointer of iso 639 language structure
 *  @param  [out] pz_brief_info_len     - pointer of event information length
 *  @param  [out] pv_brief_info         - pointer of brief event information
 *  @param  [out] pe_cond               - pointer of event condition
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_get_brief_info (
                    HANDLE_T            h_event,
                    ISO_639_LANG_T*     pt_iso_639_lang,
                    SIZE_T*             pz_brief_info_len,
                    VOID*               pv_brief_info,
                    EVCTX_COND_T*       pe_cond);

extern INT32 c_evctx_get_brief_info_64 (
                    HANDLE_T            h_event,
                    ISO_639_LANG_T*     pt_iso_639_lang,
                    UINT64*             pz_brief_info_len,
                    VOID*               pv_brief_info,
                    EVCTX_COND_T*       pe_cond);
/*----------------------------------------------------------------------------*/
/*! @brief  set evctx country code function

 *  @param  [in] e_brdcst_type          - broadcast type
 *  @param  [in] pt_country_code        - pointer of country code
 *  @param  [in] ui2_region_id          - region id
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_evctx_set_country_code (
                    BRDCST_TYPE_T       e_brdcst_type,
                    ISO_3166_COUNT_T*   pt_country_code,
                    UINT16              ui2_region_id);

/*----------------------------------------------------------------------------*/
/*! @} */                                             /* end of groupMW_EVCTX */
/*----------------------------------------------------------------------------*/

#endif /* _C_EVCTX_H_ */
