/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_img_tfx.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file contains all the transition effect APIs interface
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_IMG
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_IMG_TFX_H_
#define _C_IMG_TFX_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_img_tfx.h"
#include "u_handle.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
                    extern data declarations
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    data declarations
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief  c_img_tfx_create
 *               This function creates a TFX instance.
 *  @param [in] pf_nfy_cb   the callback function to be registered
 *  @param [in] pv_tag      tag passed to the callback function
 *  @param [out] ph_tfx      the tfx handle
 *  @return  IMGR_OK     if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_create(
    HANDLE_T            *ph_tfx,
    IMG_TFX_NFY_FCT_T   pf_nfy_cb,
    VOID                *pv_tag);

/*------------------------------------------------------------------*/
/*! @brief    c_img_tfx_create
 *                This function deletes a TFX instance.
 *  @param [in] h_tfx       the TFX handle to be deleted
 *  @return IMGR_OK     if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_delete(
    HANDLE_T            h_tfx);

/*------------------------------------------------------------------*/
/*! @brief   c_img_tfx_play
 *               This function sends the message to execute the transition effect.
 *  @param [in] h_tfx           the TFX handle
 *  @param [in] h_dst           destination surface handle      (current picture)
 *  @param [in] h_src           source surface handle           (next picture)
 *  @param [in] pt_aux          the auxiliary data              (can be NULL)
 *  @param [in] e_effect        the effect to be played
 *  @param [in] e_scheme        the playing scheme
 *  @return IMGR_OK         if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_play(
    HANDLE_T            h_tfx,
    GL_HSURFACE_T       h_dst,
    GL_HSURFACE_T       h_src,
    TFX_AUX_DATA_T      *pt_aux,
    IMG_TFX_EFFECT_T    e_effect,
    IMG_TFX_SCHEME_T    e_scheme);

/*------------------------------------------------------------------*/
/*! @brief  c_img_tfx_pause
 *               This function pauses one tfx play.
 *  @param [in]  h_tfx           the TFX handle
 *  @return IMGR_OK         if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_pause(
    HANDLE_T            h_tfx);

/*------------------------------------------------------------------*/
/*! @brief   c_img_tfx_stop
 *              This function stops one tfx play.
 *  @param [in] h_tfx           the TFX handle
 *  @return IMGR_OK         if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_stop(
    HANDLE_T            h_tfx);

/*------------------------------------------------------------------*/
/*! @brief   c_img_tfx_get_effect_info
 *               This function gets the information of a specific effect.
 *  @param [in] e_effect        the queried effect
 *  @param [out] pt_frame_info   the info retrieved
 *  @return IMGR_OK if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_get_effect_info(
    IMG_TFX_EFFECT_T    e_effect,
    TFX_FRAME_INFO_T    *pt_frame_info);

/*------------------------------------------------------------------*/
/*! @brief  c_img_tfx_set_effect_info
 *              This function sets the information of a specific effect.
 *  @param [in] e_effect        the queried effect
 *  @param [in] pt_frame_info   the info set
 *  @return  IMGR_OK if succeeded
 */
/*------------------------------------------------------------------*/
extern INT32 c_img_tfx_set_effect_info(
    IMG_TFX_EFFECT_T    e_effect,
    TFX_FRAME_INFO_T    *pt_frame_info);

extern INT32 c_img_playback_tfx_play(HANDLE_T h_img, IMG_TFX_EFFECT_T    e_effect);
extern INT32 c_img_playback_tfx_stop(VOID);

#ifdef __cplusplus
}
#endif

#endif /* _C_IMG_TFX_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_IMG*/
/*----------------------------------------------------------------------------*/
