/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_mfmtrecg.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file exports all media format recognizer API and constant for 
 *         other module using.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_MUTIL_MFMTRECG Multi Media Format Recognizer
 *
 *  @ingroup groupMW
 *  @brief Provides external function TV Application
 *
 *  %MW Application API would be used by TV Application, lot of these APIs would
 *   be wrapped from Exported API
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
#ifndef _C_MFMTRECG_H_
#define _C_MFMTRECG_H_


/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#ifndef __KERNEL__

#include "u_common.h"
#include "u_handle.h"
#include "u_mfmtrecg.h"

#else

#include "u_common.h"
#include "u_handle.h"
#include "u_mfmtrecg.h"

#endif
/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
 
/* This API retrieves the format of the media file. */                        
/*------------------------------------------------------------------*/
/*! @brief          get the file format specified by ps_name
 *  @param [in]     ps_name Specified the file path name
 *  @param [out]    pt_format Returned file format
 *  @note
 *  @see
 *  @return
 *  @retval      -MFMTRECGR_OK                get format successfully
 *  @retval      -MFMTRECGR_INV_ARG           input param invalid
 *  @retval      -MFMTRECGR_HANDLER_NOT_FOUND recognize handle not found
 */
/*------------------------------------------------------------------*/
extern INT32 c_mfmtrecg_get_file_fmt (
                        const CHAR*       ps_name,       /* (I) specify the path name of the media file which want to retrieve the format and it is not case sensitive. */
                        MEDIA_FORMAT_T*   pt_format);    /* (O) retrieve the format of the current media file. */ 

/* This API retrieves the format of the media memory buffer. */                        
/*------------------------------------------------------------------*/
/*! @brief          get the file format specified by ps_name
 *  @param [in]     pv_mem_data Specified the mem address
 *  @param [in]     z_mem_leng Tthe mem length
 *  @param [out]    pt_format Returned file format
 *  @note
 *  @see
 *  @return  
 *  @retval      -MFMTRECGR_OK                get format successfully
 *  @retval      -MFMTRECGR_INV_ARG           input param invalid
 *  @retval      -MFMTRECGR_HANDLER_NOT_FOUND recognize handle not found
 */
/*------------------------------------------------------------------*/
extern INT32 c_mfmtrecg_get_mem_fmt ( 
                        const VOID*       pv_mem_data,   /* (I) specify the address of memory buffer which want to retrieve the format. */ 
                        SIZE_T            z_mem_leng,    /* (I) specify the length of memory buffer which want to retrieve the format.  */ 
                        MEDIA_FORMAT_T*   pt_format);    /* (O) retrieve the format of the media memory buffer. */ 

#ifdef __cplusplus
}
#endif

#endif /* _C_MFMTRECG_H_ */


/*----------------------------------------------------------------------------*/
/*! @} */                                              /* end of groupMW_MUTIL_MFMTRECG */
/*----------------------------------------------------------------------------*/

