/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_chip_spec.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains the exported constants and type
 *         definitions of the Chip Specification Library.
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_CHIP_SPEC_H_
#define _U_CHIP_SPEC_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"

/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_CHIP_SPEC  Chip Specification Library
 *
 *  @ingroup groupMW
 *  @brief Set and get the some information from chip specification
 *
 *  %MW Application API would be used by Application, lot of these APIs would
 *   be wrapped from Exported API. Using these APIs, application could set and get
 *   the some information from chip specificaiton.
 *   
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* Chip Specification library API return values */
#define CHIPSPECR_OK                         ((INT32)    0)    /**<  The routine was successful.   */
#define CHIPSPECR_NOT_INIT                   ((INT32)   -1)    /**<  The Chip Specification Library is not initialized.      */
#define CHIPSPECR_ALREADY_INIT               ((INT32)   -2)    /**<  The Chip Specification Library has initialized already.      */
#define CHIPSPECR_INV_ARG                    ((INT32)   -3)    /**<  One or more of the arguments contain invalid data.      */
#define CHIPSPECR_HW_NOT_SUPPORT             ((INT32)   -4)    /**<  The hardware is not supported      */
#define CHIPSPECR_DEV_OPEN_FAILED            ((INT32)   -5)    /**<  Device open operation is failed     */
#define CHIPSPECR_DEV_GET_FAILED             ((INT32)   -6)    /**<  Device get operation is failed      */
#define CHIPSPECR_DEV_SET_FAILED             ((INT32)   -7)    /**<  Device set operation is failed      */
#define CHIPSPECR_MEM_ALLOC_FAILED           ((INT32)   -8)    /**<  Memory allocation is failed.      */
#define CHIPSPECR_TIMER_ERROR                ((INT32)   -9)    /**<  A timer error occurred.      */
#define CHIPSPECR_HANDLE_LIB_ERROR           ((INT32)  -10)    /**<  A handle library error occurred.      */
#define CHIPSPECR_ERROR                      ((INT32)  -11)    /**<  Operation error.      */
#define CHIPSPECR_CLI_FAILED                 ((INT32)  -12)    /**<  CLI command execute unsuccessfully.    */
#define CHIPSPECR_UNKNOWN                    ((INT32)  -13)    /**<  The unknown status      */

/*------------------------------------------------------------------*/
/*! @enum   CHIPSPEC_STATUS_T
 *  @brief  the structure describe the status of chip bonding component
 *  @code
 *  typedef enum
 *  {
 *      CHIPSPEC_STATUS_UNKNOWN = 0,
 *      CHIPSPEC_STATUS_FAIL,
 *      CHIPSPEC_STATUS_SUCCESS
 *  } CH_SRC_TYPE_T;
 *  @endcode
 *  @li@c  CHIPSPEC_STATUS_UNKNOWN                              unknown- unknown status of the component
 *  @li@c  CHIPSPEC_STATUS_FAIL                                 the chip spec component initialization fail
 *  @li@c  CHIPSPEC_STATUS_SUCCESS                              the chip spec component initialization successfully
 */
/*------------------------------------------------------------------*/
typedef enum
{
    CHIPSPEC_STATUS_UNKNOWN = 0,
    CHIPSPEC_STATUS_FAIL,
    CHIPSPEC_STATUS_SUCCESS,
} CHIPSPEC_STATUS_T;

#endif /* _U_CHIP_SPEC_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                               /* end of groupMW_CHIP_SPEC */
/*----------------------------------------------------------------------------*/

