/*----------------------------------------------------------------------------*
 * No Warranty                                                                *
 * Except as may be otherwise agreed to in writing, no warranties of any      *
 * kind, whether express or implied, are given by MTK with respect to any MTK *
 * Deliverables or any use thereof, and MTK Deliverables are provided on an   *
 * "AS IS" basis.  MTK hereby expressly disclaims all such warranties,        *
 * including any implied warranties of merchantability, non-infringement and  *
 * fitness for a particular purpose and any warranties arising out of course  *
 * of performance, course of dealing or usage of trade.  Parties further      *
 * acknowledge that Company may, either presently and/or in the future,       *
 * instruct MTK to assist it in the development and the implementation, in    *
 * accordance with Company's designs, of certain softwares relating to        *
 * Company's product(s) (the "Services").  Except as may be otherwise agreed  *
 * to in writing, no warranties of any kind, whether express or implied, are  *
 * given by MTK with respect to the Services provided, and the Services are   *
 * provided on an "AS IS" basis.  Company further acknowledges that the       *
 * Services may contain errors, that testing is important and Company is      *
 * solely responsible for fully testing the Services and/or derivatives       *
 * thereof before they are used, sublicensed or distributed.  Should there be *
 * any third party action brought against MTK, arising out of or relating to  *
 * the Services, Company agree to fully indemnify and hold MTK harmless.      *
 * If the parties mutually agree to enter into or continue a business         *
 * relationship or other arrangement, the terms and conditions set forth      *
 * hereunder shall remain effective and, unless explicitly stated otherwise,  *
 * shall prevail in the event of a conflict in the terms in any agreements    *
 * entered into between the parties.                                          *
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2004, MediaTek Inc.
 * All rights reserved.
 *
 * Unauthorized use, practice, perform, copy, distribution, reproduction,
 * or disclosure of this information in whole or in part is prohibited.
 *-----------------------------------------------------------------------------
 * $RCSfile: u_mhp_api.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X/DTV_X_HQ_int/DTV_X_ATSC/3 $
 * $SWAuthor: Weider Chang $
 * $MD5HEX: 72ec35f463230d3f1747ea7bd18764a5 $
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/
 
#ifndef _U_MHP_H_
#define _U_MHP_H_

#include "u_common.h"
#include "u_scdb.h"
#include "u_wgl.h"
#include "u_svctx.h"


#define MHPR_OK                   ((INT32)    0)    /**<OK        */
#define MHPR_INV_ARG              ((INT32) -  1)    /**<Invalid arguments        */
#define MHPR_INV_HANDLE           ((INT32) -  2)    /**<Invalid handle        */
#define MHPR_OUT_OF_MEMORY        ((INT32) -  3)    /**<Out of memory        */
#define MHPR_OUT_OF_HANDLE        ((INT32) -  4)    /**<Out of handle        */
#define MHPR_INTERNAL_ERROR       ((INT32) -  5)    /**<Internal error        */
#define MHPR_NOT_INIT             ((INT32) -  6)    /**<Not initialized       */
#define MHPR_ALREADY_INIT         ((INT32) -  7)    /**<Already initialized        */
#define MHPR_INIT_FAIL            ((INT32) -  8)    /**<Failed to initialized        */
#define MHPR_INV_STATE            ((INT32) -  9)    /**<Invalid state        */
#define MHPR_SVC_NOT_FOUND        ((INT32) - 10)    /**<Service is not found       */
#define MHPR_ENG_NOT_FOUND        ((INT32) - 11)    /**<Engine is not found        */
#define MHPR_CONNECT_FAIL         ((INT32) - 12)    /**<Failed to connect        */
#define MHPR_LOAD_NIT_FAIL        ((INT32) - 13)    /**<Failed to load NIT        */
#define MHPR_LOAD_SDT_FAIL        ((INT32) - 14)    /**<Failed to load SDT        */
#define MHPR_LOAD_PAT_FAIL        ((INT32) - 15)    /**<Failed to load PAT        */
#define MHPR_LOAD_PMT_FAIL        ((INT32) - 16)    /**<Failed to LOAD PMT        */
#define MHPR_CREATE_TIMER_FAIL    ((INT32) - 17)    /**<Failed to create a timer        */
#define MHPR_START_TIMER_FAIL     ((INT32) - 18)    /**<Failed to start a timer        */
#define MHPR_MISMATCHED_RUN       ((INT32) - 19)    /**<Mismatched run        */
#define MHPR_OUT_OF_RESOURCES     ((INT32) - 20)    /**<Out of semaphores        */
#define MHPR_PARSE_PMT_FAIL       ((INT32) - 21)    /**<Failed to parse PMT        */
#define MHPR_SVC_UNAVAILABLE      ((INT32) - 22)    /**<Service is unavailable        */
#define MHPR_CONFIG_OSD_FAIL      ((INT32) - 23)    /**<Failed to config OSD        */
#define MHPR_COMP_NOT_OPEN        ((INT32) - 24)    /**<The MHP component has not been opened by a call to tmMHEG5Open()        */
#define MHPR_COMP_ALREADY_OPEN    ((INT32) - 25)    /**<The MHP component has already been opened by a call to tmMHEG5Open()        */
#define MHPR_COMP_NOT_STARTED     ((INT32) - 26)    /**<The MHP component has not been started by a call to tmMHEG5Start()         */
#define MHPR_COMP_ALREADY_STARTED ((INT32) - 27)    /**<The MHP component has already been started by a call to tmMHEG5Start()        */
#define MHPR_REG_CI_NFY_FAIL      ((INT32) - 28)    /**<Failed to register CI notify function        */
#define MHPR_CI_START_APP_BUSY    ((INT32) - 29)    /**<A CI application has already been started        */
#define MHPR_CI_NO_APP_STARTED    ((INT32) - 29)    /**<No CI applications has been started        */
#define MHPR_NO_SUPPORTED_PROFILE ((INT32) - 30)    /**<No supported profile        */


typedef UINT32 MHP_SET_TYPE_T;    /**< common set type. */
typedef UINT32 MHP_GET_TYPE_T;    /**< common get type. */


/**
 * Some languages have 2 different ISO 639.2 codes like ISO 639.2/B and ISO 639.2/T.
 * This structure is to deliver the language codes appropriately.
 */
typedef struct {
    INT16   i2_num; /**< The available number of language codes. It must be 1 or 2. */
    /**< If only one lang is available, this field is supposed to be used.
    It must be a null-terminated 3-letter language code. */
    CHAR    lang1[4];
    /**< The secondary lang code. It must be a null-terminated 3-letter language code. */
    CHAR    lang2[4];
} ACI_UP_LANG;

/**
 * Both ISO 639.2/B and ISO 639.2/T codes for the current language setting.
 * type = ACI_UP_LANG, size = sizeof(ACI_UP_LANG)
 OSD language: ACI_UP_LANG
  i2_num=1
  lang1
 */
#define MHP_SET_TYPE_LANG       ((MHP_SET_TYPE_T)1)

/**
 * The age information for the current parental rating setting.
 * type = INT32, size = sizeof(INT32)
 */
#define MHP_SET_TYPE_RATING     ((MHP_SET_TYPE_T)2)

/**
 * ISO 3166-1 code for the current country setting.
 * It must be a null-terminated 2-letter country code.
 * type = CHAR[3], size = sizeof(CHAR[3])
 */
#define MHP_SET_TYPE_COUNTRY    ((MHP_SET_TYPE_T)3)

/**
 * The timezone information for the current timezone setting.
 * type = INT32, size = sizeof(INT32)
 */
#define MHP_SET_TYPE_TIMEZONE   ((MHP_SET_TYPE_T)4)

/**
 * A boolean value for the MHP Auto Start setting.
 * type = BOOL, size = sizeof(BOOL)
 */
#define MHP_SET_TYPE_AUTO_START ((MHP_SET_TYPE_T)5)

/**
 * Required by x_svctx_select_svc() at ac_channel.c!
 * type = UINT16, size = sizeof(UINT16)
 */
#define MHP_SET_TYPE_SVC_LST_ID ((MHP_SET_TYPE_T)6)

/**
 * Required by x_svctx_select_svc() at ac_tuner.c!
 * It must be a null-terminated string constant.
 * type = CHAR[16], size = sizeof(CHAR[16])
 */
#define MHP_SET_TYPE_CONN_SRC_NAME ((MHP_SET_TYPE_T)7)

/**
 * A boolean value for the current subtitling on/off setting.
 * type = BOOL, size = sizeof(BOOL)
 */
#define MHP_SET_TYPE_SUBTITLING ((MHP_SET_TYPE_T)8)


/**
 *  
 * factory reset notification
 */
#define MHP_SET_TYPE_FACTORY_RESET ((MHP_SET_TYPE_T)9)


/**
 *  
 * channel change notification so that MHP engine can know if http player is still running
 */
#define MHP_SET_TYPE_CHANNEL_CHANGE ((MHP_SET_TYPE_T)10)


/**
 * Both ISO 639.2/B and ISO 639.2/T codes for the current language setting.
 * type = ACI_UP_LANG, size = sizeof(ACI_UP_LANG)
 audio language: ACI_UP_LANG
 i2_num=2
  lang1: primary audio language
  lang2: second audio language
 */
#define MHP_SET_TYPE_AUDIO_LANGUAGE ((MHP_SET_TYPE_T)11)



#define MHP_MSG_GRD_RENDERED      ((UINT32) 0)    /**<????        */
#define MHP_MSG_GRD_KEY_DOWN      ((UINT32) 1)    /**<????        */
#define MHP_MSG_GRD_KEY_UP        ((UINT32) 2)    /**<????        */
#define MHP_MSG_GRD_DRAWN         ((UINT32) 3)    /**<????        */
#define MHP_MSG_GRD_TUNING_BEFORE ((UINT32) 4)    /**<????        */
#define MHP_MSG_GRD_TUNING_AFTER  ((UINT32) 5)    /**<????        */
#define MHP_MSG_GRD_REG_KEY       ((UINT32) 6)    /**<????        */

/*------------------------------------------------------------------*/
/*! @brief Callback to notify the GUI states
 *  @param [in] ui4_msg - MHP_MSG_GRD_RENDERED,
                          MHP_MSG_GRD_KEY_DOWN, 
 *                        MHP_MSG_GRD_DRAWN 
 *  @param [in] pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*mhp_eng_state_nfy_fct)(
    UINT32                      ui4_msg, 
    VOID*                       pv_param,
    VOID*                       pv_tag
    );

typedef struct _MHP_PLAY_SBTL_INFO
{
	UINT32	ui4_tunerId;
	INT32	i4_pid;
	CHAR	a_lang[ISO_639_LANG_LEN];
	INT32	i4_c_pageId;
	INT32	i4_a_pageId;

} MHP_PLAY_SBTL_INFO;

typedef struct _MHP_PLAY_TTX_INFO
{
	UINT32	ui4_tunerId;
	UINT32	ui4_pid;
	CHAR	a_lang[ISO_639_LANG_LEN];
	UINT32  ui4_type;
	UINT32	ui4_m_number;
	UINT32	ui4_p_number;


} MHP_PLAY_TTX_INFO;

typedef enum _MHP_DO_SBTL_TTX_TYPE
{
	MHP_DO_SBTL_TTX_TYPE_UNKNOWN = 0,
	MHP_DO_TYPE_PLAY_SBTL,
	MHP_DO_TYPE_STOP_SBTL,
	MHP_DO_TYPE_PLAY_TTX,
	MHP_DO_TYPE_STOP_TTX

} MHP_DO_SBTL_TTX_TYPE;


/* Struct for notifying MHP subtitle or teletext information */
typedef struct _MHP_SBTL_TTX_NFY_INFO
{
	MHP_DO_SBTL_TTX_TYPE t_do_type;
	union{
		
		MHP_PLAY_SBTL_INFO	t_sbtl_play_info; 
		MHP_PLAY_TTX_INFO	t_ttx_play_info;

	}u_do_info;
	
} MHP_SBTL_TTX_NFY_INFO;


/* Definition of SBTL and TTX notify function */
typedef INT32 ( *mhp_sbtl_ttx_nfy_fct )(
    const MHP_SBTL_TTX_NFY_INFO* pt_info,
    VOID*                    	 pv_tag );


/******************For channel change******************************************/
typedef UINT32  MHP_REQUEST_CMD_T;
#define MHP_REQUEST_CMD_SERVICE_SELECTION     (( MHP_REQUEST_CMD_T ) 0 )
#define MHP_REQUEST_CMD_PLAY_STREAM           (( MHP_REQUEST_CMD_T ) 1 )
#define MHP_REQUEST_CMD_PLAY_BROADCAST        (( MHP_REQUEST_CMD_T ) 2 )
#define MHP_REQUEST_CMD_TUNE                  (( MHP_REQUEST_CMD_T ) 3 )
#define MHP_REQUEST_CMD_TUNE_WITH_TS_INFO     (( MHP_REQUEST_CMD_T ) 4 )
#define MHP_REQUEST_CMD_STOP_BROADCAST        (( MHP_REQUEST_CMD_T ) 5 )


typedef INT32 ( *mhp_request_op_nfy )(
                    MHP_REQUEST_CMD_T     e_request_cmd, 
                    VOID*                 pv_param,
                    VOID*                 pv_tag,
                    VOID*                 pv_data
                    );

/*for MHP_REQUEST_CMD_TUNE & MHP_REQUEST_CMD_TUNE_WITH_TS_INFO*/
typedef struct _MHP_TUNE_TO_INFO
{
	SRC_DESC_T                  t_src_desc;  /*c_svctx_select_svc   source description, can use its frequency to tune to a frequency*/
	x_svctx_select_nfy_fct      pf_nfy;      /*c_svctx_select_svc   notify*/
    VOID*                       pv_nfy_tag;   /*c_svctx_select_svc   tag*/
    BOOL                        b_by_frequency;
}MHP_TUNE_TO_INFO;

/*for MHP_REQUEST_CMD_SERVICE_SELECTION*/
typedef struct _MHP_SERVICE_SEL_INFO
{
	SRC_DESC_T                  t_src_desc;  /*c_svctx_select_svc   source description, can use its frequency to tune to a frequency*/
	UINT32                      ui4_stream_mode;
}MHP_SERVICE_SEL_INFO;

/*for MHP_REQUEST_CMD_PLAY_STREAM*/
typedef struct _MHP_STRM_SEL_BY_PID
{
    /*for select stream by pid*/
	SVCTX_STREAM_INFO_T     streamInfo;
	UINT16                  ui2_svc_id;
	MPEG_2_PID_T            t_pcr_pid;

	x_svctx_select_nfy_fct	pf_nfy;
	VOID*					pv_nfy_tag;
	BOOL                    b_video_freeze;

}MHP_STRM_SEL_BY_PID;

/*for MHP_REQUEST_CMD_PLAY_BROADCAST*/
typedef struct _MHP_STRM_SEL_BY_ID
{
	STREAM_COMP_ID_T       t_strmCompid;

	x_svctx_select_nfy_fct	pf_nfy;
	VOID*					pv_nfy_tag;
	BOOL                    b_video_freeze;
	
}MHP_STRM_SEL_BY_ID;

/*for MHP_REQUEST_CMD_PLAY_BROADCAST*/
typedef struct _MHP_STRM_STOP_BY_TYPE
{
	BOOL                    b_video_freeze;
    STREAM_TYPE_T           e_strm_type;
    UINT32                  ui4_time;	
}MHP_STRM_STOP_BY_TYPE;


typedef struct _MHP_GLUE_REQUEST_T
{
    MHP_REQUEST_CMD_T  t_command;
	union
	{
	    MHP_TUNE_TO_INFO       t_tune;
	    MHP_SERVICE_SEL_INFO   t_svc_sel_ino;
	    MHP_STRM_SEL_BY_PID    t_strm_sel_pid;
	    MHP_STRM_SEL_BY_ID     t_strm_sel_id;
	    MHP_STRM_STOP_BY_TYPE  t_strm_stop_type;
	}u_data;
    
} MHP_GLUE_REQUEST_T;
/*****************************Sample: how to implementation ac_mhp_reg_request_op_nfy*****************************/
#if 0 //def __DEMO_CODE__

#define __MTK_CHANNEL_CHANGE_V2__

static mhp_request_op_nfy    _pf_request_op_nfy = NULL;
static VOID*                 _pv_req_tag = NULL;


VOID x_ac_mhp_reg_request_op_nfy (
                mhp_request_op_nfy    pf_request_op_nfy,
                VOID*                 pv_tag )
{
    _pf_request_op_nfy = pf_request_op_nfy;
	_pv_req_tag = _pv_req_tag;
}

static VOID _tn_svctx_select_nfy_fct( HANDLE_T h_svctx, SVCTX_COND_T e_nfy_cond,
        SVCTX_NTFY_CODE_T e_code, STREAM_TYPE_T e_stream_type, VOID *pv_nfy_tag )
{
    if (( e_nfy_cond != SVCTX_COND_PRESENTING ) && ( e_nfy_cond != SVCTX_COND_PENDING )) 
	{
        return;
    }

    switch ( e_code ) 
	{
	    case SVCTX_NTFY_CODE_SIGNAL_LOSS:
	    case SVCTX_NTFY_CODE_NORMAL:
	    case SVCTX_NTFY_CODE_SERVICE_BLOCKED: 
			{
		        if ( e_code == SVCTX_NTFY_CODE_SIGNAL_LOSS ) 
				{
		            aci_tnCB( 0, AC_TN_EV_UNLOCKED );
		        } else 
		        {
		            aci_tnCB( 0, AC_TN_EV_TUNED );
		            aci_tnCB( 0, AC_TN_EV_LOCKED );
		        }
		        break;
			}

	    default:
	        /* do nothing */
	        break;
    }
}



/****************************************************************************************

tune to a tsl known to MTK platform
 
******************************************************************************************/
AC_BOOL ac_tn_tune( AC_UINT32 tnID, AC_TN_TS_HANDLE h ) 
{
	TSInfo*  tsl = tsListCached, *ts;   /* TS List, TS */
	AC_BOOL  succ;

	ACI_LOCK( aci_tnLock, AC_MU_WAIT, 0 );
	if ( tsl == NULL ) 
	{
		succ = AC_FALSE;
		goto finally;
	}
	for ( ts = tsl; ts->ts.next != NULL; ts++ ) 
	{
		if ( ts->ts.ts == h )
		{
			#ifndef __MTK_CHANNEL_CHANGE_V2__
			SRC_DESC_T          t_src_desc;
			INT32               i4_ret;
			memset( &t_src_desc, 0, sizeof( SRC_DESC_T ));

			t_src_desc.e_type = SRC_TYPE_BRDCST_FREQ;
			t_src_desc.ps_conn_src_type = BRDCST_HANDLER_DEFAULT_NAME; 
			t_src_desc.ps_conn_src_name = aci_s_conn_src_name;

			t_src_desc.u_info.t_freq.e_type = FREQ_INFO_TYPE_DIG_TER;
			t_src_desc.u_info.t_freq.e_app_scene = FREQ_APP_SCENE_DIG_MHP;
			t_src_desc.u_info.t_freq.u_data.t_dig_ter.ui4_freq = ts->ui4_freq;
			t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = ts->e_bandwidth;
			t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = ts->e_mod;
			t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_hierarchy_priority = ts->e_hierarchy_priority;
			i4_ret = x_svctx_select_svc( aci_svctx, 0, &t_src_desc, NULL, _svctx_select_nfy_fct, NULL );
			#else
			INT32               i4_ret;
			MHP_TUNE_TO_INFO    t_tune;

			memset( &t_tune, 0, sizeof( MHP_TUNE_TO_INFO ));

			t_tune.t_src_desc.e_type = SRC_TYPE_BRDCST_FREQ;
			t_tune.t_src_desc.ps_conn_src_name = aci_s_conn_src_name;

			t_tune.t_src_desc.u_info.t_freq.e_type = FREQ_INFO_TYPE_DIG_TER;
			t_tune.t_src_desc.u_info.t_freq.e_app_scene = FREQ_APP_SCENE_DIG_MHP;
			t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.ui4_freq = ts->ui4_freq;
			t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = ts->e_bandwidth;
			t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = ts->e_mod;
			t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_hierarchy_priority = ts->e_hierarchy_priority;

			t_tune.pf_nfy = _tn_svctx_select_nfy_fct;
			t_tune.pv_nfy_tag = NULL;

			if ( _pf_request_op_nfy )
			{
			    i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_TUNE, ( VOID* )&t_tune, _pv_req_tag, 0 );
			}
			else
			{
			    aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			#endif
			CHK_FAIL( i4_ret );

			succ = ( i4_ret == 0 );
		}
	}
	
finally:
	ACI_UNLOCK( aci_tnLock );
	return succ;

}

/**************** ************************************************************************************

tune to a frequency unknow to MTK platform

****************************************************************************************************/
AC_BOOL ac_tn_tuneToWithTSInfo( AC_UINT32 tunerId, Ac_TransportStreamInfo* tsInfo ) 
{
    ac_io_putString("glue: ac_tn_tuneToWithTSInfo()\n");

#ifndef __MTK_CHANNEL_CHANGE_V2__

    SRC_DESC_T    t_src_desc;
    INT32         i4_ret;
    memset( &t_src_desc, 0, sizeof( SRC_DESC_T ));

    t_src_desc.e_type = SRC_TYPE_BRDCST_FREQ;
    t_src_desc.ps_conn_src_name = aci_s_conn_src_name;

    aci_dbg(( "[TNGLUE] aci_s_conn_src_name: %s\n", aci_s_conn_src_name ));

    t_src_desc.u_info.t_freq.e_type = FREQ_INFO_TYPE_DIG_TER;
	t_src_desc.u_info.t_freq.e_app_scene = FREQ_APP_SCENE_DIG_MHP;

    t_src_desc.u_info.t_freq.u_data.t_dig_ter.ui4_freq
             = tsInfo->terrestrial.centre_frequency * 100;

    switch ( tsInfo->terrestrial.bandwidth ) 
	{
	    case AC_TN_BW_8_MHz:
	        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_8_MHz;
	        break;
	    case AC_TN_BW_7_MHz:
	        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_7_MHz;
	        break;
	    case AC_TN_BW_6_MHz:
	        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_6_MHz;
	        break;
	    default:
	        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_UNKNOWN;
	        break;
    }

    switch ( tsInfo->terrestrial.constellation ) {
    case AC_TN_CONSTEL_QPSK:
        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QPSK;
        break;
    case AC_TN_CONSTEL_16_QAM:
        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QAM_16;
        break;
    case AC_TN_CONSTEL_64_QAM:
        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QAM_64;
        break;
    default:
        t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_UNKNOWN;
    }
	t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_hierarchy_priority = /*e_hierarchy_priority*/
	

    i4_ret = x_svctx_select_svc( aci_svctx, 0, &t_src_desc, NULL, _svctx_select_nfy_fct, NULL );
#else
	INT32				i4_ret;
	MHP_TUNE_TO_INFO	t_tune;

	memset( &t_tune, 0, sizeof( MHP_TUNE_TO_INFO ));

    t_tune.t_src_desc.e_type = SRC_TYPE_BRDCST_FREQ;
    t_tune.t_src_desc.ps_conn_src_name = aci_s_conn_src_name;

    aci_dbg(( "[TNGLUE] aci_s_conn_src_name: %s\n", aci_s_conn_src_name ));

    t_tune.t_src_desc.u_info.t_freq.e_type = FREQ_INFO_TYPE_DIG_TER;
    t_tune.t_src_desc.u_info.t_freq.e_app_scene = FREQ_APP_SCENE_DIG_MHP;
	
    t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.ui4_freq
             = tsInfo->terrestrial.centre_frequency * 100;

    switch ( tsInfo->terrestrial.bandwidth ) 
	{
	    case AC_TN_BW_8_MHz:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_8_MHz;
	        break;
	    case AC_TN_BW_7_MHz:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_7_MHz;
	        break;
	    case AC_TN_BW_6_MHz:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_6_MHz;
	        break;
	    default:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_bandwidth = BW_UNKNOWN;
	        break;
    }

    switch ( tsInfo->terrestrial.constellation ) 
	{
	    case AC_TN_CONSTEL_QPSK:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QPSK;
	        break;
	    case AC_TN_CONSTEL_16_QAM:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QAM_16;
	        break;
	    case AC_TN_CONSTEL_64_QAM:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_QAM_64;
	        break;
	    default:
	        t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_mod = MOD_UNKNOWN;
    }
	t_tune.t_src_desc.u_info.t_freq.u_data.t_dig_ter.e_hierarchy_priority = /*e_hierarchy_priority*/
	
	t_tune.b_by_frequency = AC_TRUE;
	t_tune.pf_nfy = _tn_svctx_select_nfy_fct;
	t_tune.pv_nfy_tag = NULL;
	
	if ( _pf_request_op_nfy )
	{
		i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_TUNE_WITH_TS_INFO, ( VOID* )&t_tune, _pv_req_tag, 0 );
	}
	else
	{
		aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
		i4_ret = -1;
	}
#endif
    CHK_FAIL( i4_ret );

    return ( i4_ret == 0 );
}

/****** **************************************************************************************************

Note:
 1. you can call ac_c_requestServiceSelection to select some program after tune to some tsl or frequency.
   it is also succefull even if does not do any tuning action!

******************************************************************************************* ***************/
void ac_c_requestServiceSelection( Ac_ServiceInformation* si ) 
{
#ifndef __MTK_CHANNEL_CHANGE_V2__

    UINT32           ui4_stream_mode = ST_AUDIO | ST_VIDEO;
    SRC_DESC_T       t_src_desc;
    SRC_SVL_INFO_T*  pt_svl_info = &t_src_desc.u_info.t_svl;
    INT32            i4_ret;
    ACI_C_ServiceID *sid = NULL;

    ACI_C_FIND_CHANNEL_INFO( &sid, e, ( e->ts == si->ts ) && ( e->sid == si->serviceId ));
    if ( sid == NULL ) 
	{
        return;
    }

    pt_svl_info->ui2_svc_lst_id = aci_ui2_svc_lst_id;
    pt_svl_info->ui2_svl_rec_id = sid->ui2_svl_rec_id;

    t_src_desc.e_type = SRC_TYPE_BRDCST_SVL;
    t_src_desc.ps_conn_src_type = BRDCST_HANDLER_DEFAULT_NAME;
    t_src_desc.ps_conn_src_name = SN_MAIN_TUNER_TER_DIG;
    i4_ret = x_svctx_select_svc(aci_svctx, ui4_stream_mode, &t_src_desc, NULL, NULL, NULL);
#else
    MHP_SERVICE_SEL_INFO  t_service_sel_info;
	INT32                 i4_ret;
    SRC_SVL_INFO_T*       pt_svl_info = &t_service_sel_info.t_src_desc.u_info.t_svl;
	
	memset( &t_service_sel_info, 0 ,sizeof( MHP_SERVICE_SEL_INFO ));

	ACI_C_FIND_CHANNEL_INFO( &sid, e, ( e->ts == si->ts ) && ( e->sid == si->serviceId ));
    if ( sid == NULL ) 
	{
        return;
    }

    pt_svl_info->ui2_svc_lst_id = aci_ui2_svc_lst_id;
    pt_svl_info->ui2_svl_rec_id = sid->ui2_svl_rec_id;

    t_service_sel_info.t_src_desc.e_type = SRC_TYPE_BRDCST_SVL;
    t_service_sel_info.t_src_desc.ps_conn_src_type = BRDCST_HANDLER_DEFAULT_NAME;
    t_service_sel_info.t_src_desc.ps_conn_src_name = SN_MAIN_TUNER_TER_DIG;
	t_service_sel_info.ui4_stream_mode = ST_AUDIO | ST_VIDEO;

	if ( _pf_request_op_nfy )
	{
		i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_SERVICE_SELECTION, ( VOID* )&t_service_sel_info, _pv_req_tag, 0 );
	}
	else
	{
		aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
		i4_ret = -1;
	}
	 
#endif
    CHK_FAIL(i4_ret);
}


static INT32 _ac_m_SelectStreamBySvcidPid( 
            	STREAM_TYPE_T           e_strm_type, 
            	MPEG_2_PID_T			t_mpeg_2_pid,
            	UINT8                   ui1_codec,
            	MPEG_2_PID_T            t_pcr_id,
            	UINT16                  ui2_svc_id )
{
	INT32                  i4_ret        = SVCTXR_OK;
    BOOL                   b_already_stm = FALSE;
    UINT16                 ui2_pmt_svc_id ;
    
    SVCTX_STREAM_INFO_T    t_stream_info;
    STREAM_COMP_ID_T       t_comp_id;
    SCDB_REC_T             t_scdb;
    
    SM_SESS_GNRC_AV_SYNC_INFO_T     t_av_sync;
	
    memset( &t_stream_info, 0, sizeof( SVCTX_STREAM_INFO_T ));
    memset( &t_comp_id, 0, sizeof( STREAM_COMP_ID_T ));
    memset( &t_scdb, 0, sizeof( SCDB_REC_T ));

	if ( e_strm_type == ST_AUDIO )
	{
		t_stream_info.e_strm_type = ST_AUDIO;
		/*t_stream_info.u.t_aud_stream.e_aud_enc = AUD_ENC_MPEG_2;*/
		t_stream_info.u.t_aud_stream.e_aud_enc = ui1_codec;
	}
	else if ( e_strm_type == ST_VIDEO )
	{
		t_stream_info.e_strm_type = ST_VIDEO;
		/*t_stream_info.u.t_vid_stream.e_vid_enc = VID_ENC_MPEG_2;*/
		t_stream_info.u.t_vid_stream.e_vid_enc = ui1_codec;
	}
	else
	{
	    /*not support */
		return -1;
	}

    b_already_stm = ( x_svctx_get_using_strm_data(
 		              aci_svctx, 
 					  t_stream_info.e_strm_type, 
 					  &t_comp_id, &t_scdb ) == SVCTXR_OK );

    if ( b_already_stm ) /*stop current running stream*/
    {
        i4_ret = x_svctx_sync_stop_stream( aci_svctx, t_stream_info.e_strm_type, 0 );
        
        if( SVCTXR_OK == i4_ret )
        {
            aci_dbg(( "{glue:} x_svctx_sync_stop_stream e_strm_type (%d) success\n", t_stream_info.e_strm_type ));
        }
        else
        {
            aci_dbg(( "{glue:} x_svctx_sync_stop_stream e_strm_type (%d) failed return (%d) \n",
				       t_stream_info.e_strm_type, i4_ret ));
        }
    }
    else
    {
        aci_dbg(( "{glue:} x_svctx_get_using_strm_data e_strm_type (%d) return false \n", t_stream_info.e_strm_type ));
    }
	
    /*set svc_id */
    i4_ret = x_svctx_set_conn_attr( aci_svctx,
                                     ST_UNKNOWN,
                                     BRDCST_CTRL_SET_TYPE_SVC_ID,
                                     ( VOID* )(( UINT32 )ui2_pmt_svc_id ) );

    if ( SVCTXR_OK == i4_ret )
    {
        aci_dbg(( "{glue:} x_svctx_set_conn_attr svc_id (%d) success\n", ui2_pmt_svc_id ));
    }
    else
    {
        aci_dbg(( "{glue:} x_svctx_set_conn_attr svc_id (%d) failed return (%d) \n", ui2_pmt_svc_id, i4_ret ));
    }


    t_av_sync.e_av_sync_type    = SM_SESS_GNRC_AV_SYNC_FORCE_PCR_PID;
    t_av_sync.u.t_force_pcr_pid = t_pcr_id;

    /*set A/V sync*/
    x_svctx_set_session_attr(
        aci_svctx,
        e_strm_type,
        SM_SESS_GNRC_SET_TYPE_AV_SYNC,
        (VOID *)&t_av_sync,
        sizeof( SM_SESS_GNRC_AV_SYNC_INFO_T ));
	
    /*play stream by pid*/
    i4_ret = x_svctx_select_mpeg_stream_by_pid_ex(
                             aci_svctx,
                             &t_stream_info,
                             NULL,
                             NULL );

    if ( SVCTXR_OK == i4_ret )
    {
        aci_dbg(( "{glue:} x_svctx_select_mpeg_stream_by_pid_ex pid (%d) success\n", t_stream_info.t_mpeg_2_pid ));
    }
    else
    {
        aci_dbg(( "{glue:} x_svctx_select_mpeg_stream_by_pid_ex pid (%d) failed return (%d) \n",
			       t_stream_info.t_mpeg_2_pid, i4_ret ));
    }


    return i4_ret;
}

/***********************************************************************************************************

Note: 
  1. this should be called afer ac_c_requestServiceSelection 
  2  if witch to Http player  then resume TV braodcast please call ac_c_requestServiceSelection firstly before changes component! 
  3. should pass svc_id to MTK platform now  herein assume audioMode as  svc_id
 ************************************************************************************************************/
AC_BOOL ac_m_playStream( AC_UINT32 tunerId, AC_UINT32 deviceId, AC_M_DEC_SELECT_MODE mode,
        AC_UINT32 pcrPID, AC_UINT32 videoPID, AC_UINT32 vType,
        AC_UINT32 audioPID, AC_UINT32 aType, AC_M_AUDIO_MODE audioMode ) 
{
#ifndef __MTK_CHANNEL_CHANGE_V2__

    AC_INT32            i4_ret;
    AC_CV_ERROR         cvRet = 0;
    SVCTX_STREAM_INFO_T streamInfo;
	AC_INT32            svcId = audioMode; /*assume audioMode as ui2_svc_id*/

    ACI_LOCK( pidLock, AC_MU_WAIT, 0 );
    aci_dbg(( "glue: ac_m_playStream() 1\n" ));

    memset( &streamInfo, 0, sizeof( SVCTX_STREAM_INFO_T ));

    if ( aci_getAudioClipStatus() == 1 ) 
	{
        aci_setAudioStatus(AC_FALSE);
        ac_m_stopAudioClip();
    }

    switch ( mode )
	{
	    case AC_M_AUDIO_AND_VIDEO:
	        if ( videoFreeze ) 
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL(i4_ret);
	            videoFreeze = AC_FALSE;
	        }
	        i4_ret = _ac_m_SelectStreamBySvcidPid( ST_VIDEO, videoPID, vType, pcrPID, svcId );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }

	        i4_ret = _ac_m_SelectStreamBySvcidPid( ST_AUDIO, audioPID, aType, pcrPID, svcId );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }
	        aci_setAudioStatus( AC_TRUE );
	        break;

	    case AC_M_ONLY_VIDEO:
	        if ( videoFreeze ) 
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL( i4_ret );
	            videoFreeze = AC_FALSE;
	        }

	        i4_ret = _ac_m_SelectStreamBySvcidPid( ST_VIDEO, videoPID, vType, pcrPID, svcId );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n "));
	            goto finally;
	        }
	        break;

	    case AC_M_ONLY_AUDIO:
	        i4_ret = _ac_m_SelectStreamBySvcidPid( ST_AUDIO, audioPID, aType, pcrPID, svcId );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL(i4_ret);
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }
	        aci_setAudioStatus( AC_TRUE );

	        break;
    }
    ACI_UNLOCK( pidLock );
    return AC_TRUE;
#else
	AC_INT32			i4_ret;
	AC_CV_ERROR 		cvRet = 0;
	MHP_STRM_SEL_BY_PID t_strm_sel_info;
	AC_INT32            svcId = audioMode; /*assume audioMode as ui2_svc_id*/

	ACI_LOCK( pidLock, AC_MU_WAIT, 0 );
	aci_dbg(( "glue: ac_m_playStream() 1\n" ));

	memset( &t_strm_sel_info, 0, sizeof( MHP_STRM_SEL_BY_PID ));

	if ( aci_getAudioClipStatus() == 1 ) 
	{
		aci_setAudioStatus(AC_FALSE);
		ac_m_stopAudioClip();
	}

	switch ( mode )
	{
		case AC_M_AUDIO_AND_VIDEO:
			/*play video */
			if ( videoFreeze ) 
			{
			    t_strm_sel_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			t_strm_sel_info.streamInfo.e_strm_type = ST_VIDEO;
			t_strm_sel_info.streamInfo.t_mpeg_2_pid = videoPID;
			t_strm_sel_info.streamInfo.u.t_vid_stream.e_vid_enc = vType;

		    t_strm_sel_info.pf_nfy = x_svctx_select_nfy_video;
		    t_strm_sel_info.pv_nfy_tag = NULL;

			t_strm_sel_info.ui2_svc_id = svcId;  
			t_strm_sel_info.t_pcr_pid = ( MPEG_2_PID_T )pcrPID;

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_STREAM, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
          
			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL( i4_ret );
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
            /*play audio */
			memset( &t_strm_sel_info, 0, sizeof( MHP_STRM_SEL_BY_PID ));
			t_strm_sel_info.streamInfo.e_strm_type = ST_AUDIO;
			t_strm_sel_info.streamInfo.t_mpeg_2_pid = audioPID;
			t_strm_sel_info.streamInfo.u.t_aud_stream.e_aud_enc = aType;
		    t_strm_sel_info.pf_nfy = x_svctx_select_nfy_audio;
		    t_strm_sel_info.pv_nfy_tag = NULL;
			
			t_strm_sel_info.ui2_svc_id = svcId;  
			t_strm_sel_info.t_pcr_pid = ( MPEG_2_PID_T )pcrPID;

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_STREAM, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			
			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL( i4_ret );
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
			aci_setAudioStatus( AC_TRUE );
			break;

		case AC_M_ONLY_VIDEO:
			if ( videoFreeze ) 
			{
			    t_strm_sel_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			t_strm_sel_info.streamInfo.e_strm_type = ST_VIDEO;
			t_strm_sel_info.streamInfo.t_mpeg_2_pid = videoPID;
			t_strm_sel_info.streamInfo.u.t_vid_stream.e_vid_enc = vType;

		    t_strm_sel_info.pf_nfy = x_svctx_select_nfy_video;
		    t_strm_sel_info.pv_nfy_tag = NULL;
			
			t_strm_sel_info.ui2_svc_id = svcId;  
			t_strm_sel_info.t_pcr_pid = ( MPEG_2_PID_T )pcrPID;

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_STREAM, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
          
			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL( i4_ret );
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}

			break;

		case AC_M_ONLY_AUDIO:
			t_strm_sel_info.streamInfo.e_strm_type = ST_AUDIO;
			t_strm_sel_info.streamInfo.t_mpeg_2_pid = audioPID;
			t_strm_sel_info.streamInfo.u.t_aud_stream.e_aud_enc = aType;
		    t_strm_sel_info.pf_nfy = x_svctx_select_nfy_audio;
		    t_strm_sel_info.pv_nfy_tag = NULL;
			
			t_strm_sel_info.ui2_svc_id = svcId;  
			t_strm_sel_info.t_pcr_pid = ( MPEG_2_PID_T )pcrPID;

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_STREAM, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			
			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL( i4_ret );
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
			aci_setAudioStatus( AC_TRUE );

			break;
	}
	ACI_UNLOCK( pidLock );
	return AC_TRUE;

#endif

finally:
    ACI_UNLOCK( pidLock );
    aci_dbg(( "glue: ac_m_playStream() 2\n" ));
    return AC_FALSE;
}

/********************************************************************************************************

Play a stream:
Note: after switch to http if want to resume TV broadcast should call ac_c_requestServiceSelection before this function

********************************************************************************************************/
AC_BOOL ac_m_playBroadcast(AC_UINT32 deviceId, AC_M_DEC_SELECT_MODE mode) 
{
#ifndef __MTK_CHANNEL_CHANGE_V2__

    AC_INT32    i4_ret;
    AC_CV_ERROR cvRet = 0;

    ACI_LOCK(pidLock, AC_MU_WAIT, 0);
    aci_dbg(("glue: ac_m_playBroadcast() 1\n"));

    switch ( mode ) 
	{
	    case AC_M_AUDIO_AND_VIDEO:

	        if ( videoFreeze )
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL( i4_ret );
	            videoFreeze = AC_FALSE;
	        }
	        g_videoCompId.e_type = ST_VIDEO;
	        i4_ret = x_svctx_select_stream( aci_svctx, &g_videoCompId, x_svctx_select_nfy_video, NULL );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }

	        g_audioCompId.e_type = ST_AUDIO;
	        i4_ret = x_svctx_select_stream( aci_svctx, &g_audioCompId, x_svctx_select_nfy_audio, NULL );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet )
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }
	        aci_setAudioStatus( AC_TRUE );
	        break;

	    case AC_M_ONLY_VIDEO:
	        if ( videoFreeze ) 
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL( i4_ret );
	            videoFreeze = AC_FALSE;
	        }
	        
	        g_videoCompId.e_type = ST_VIDEO;
	        i4_ret = x_svctx_select_stream( aci_svctx, &g_videoCompId, x_svctx_select_nfy_video, NULL );
	        ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }
	        break;

	    case AC_M_ONLY_AUDIO:
	        g_audioCompId.e_type = ST_AUDIO;
	        i4_ret = x_svctx_select_stream( aci_svctx, &g_audioCompId, x_svctx_select_nfy_audio, NULL );
	        ACI_WAIT(cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT);
	        if ( AC_CV_SUCCESS != cvRet ) 
			{
	            CHK_FAIL( i4_ret );
	            aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
	            goto finally;
	        }
	        aci_setAudioStatus( AC_TRUE );
	        break;
    }
    aci_dbg(( "glue: ac_m_playBroadcast() 2\n" ));
    ACI_UNLOCK( pidLock );
    return AC_TRUE;
#else
	AC_INT32             i4_ret;
	AC_CV_ERROR          cvRet = 0;
	MHP_STRM_SEL_BY_ID   t_strm_sel_info;
	
	ACI_LOCK( pidLock, AC_MU_WAIT, 0 );
	aci_dbg(("glue: ac_m_playBroadcast() 1\n"));
    memset( &t_strm_sel_info, 0, sizeof( MHP_STRM_SEL_BY_ID ));

	switch ( mode ) 
	{
		case AC_M_AUDIO_AND_VIDEO:
            /*play video*/
			if ( videoFreeze )
			{
			    t_strm_sel_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			g_videoCompId.e_type = ST_VIDEO;
			memcpy( &t_strm_sel_info.t_strmCompid, &g_videoCompId, sizeof( STREAM_COMP_ID_T ));

			t_strm_sel_info.pf_nfy = x_svctx_select_nfy_video;
			t_strm_sel_info.pv_nfy_tag = NULL;
			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_BROADCAST, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}

			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL(i4_ret);
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
            /*play audio*/
			memset( &t_strm_sel_info, 0, sizeof( MHP_STRM_SEL_BY_ID ));


			g_audioCompId.e_type = ST_AUDIO;
			memcpy( &t_strm_sel_info.t_strmCompid, &g_audioCompId, sizeof( STREAM_COMP_ID_T ));
			t_strm_sel_info.pf_nfy = x_svctx_select_nfy_audio;
			t_strm_sel_info.pv_nfy_tag = NULL;
			
			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_BROADCAST, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}

			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet )
			{
				CHK_FAIL(i4_ret);
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
			aci_setAudioStatus( AC_TRUE );
			break;

		case AC_M_ONLY_VIDEO:
			if ( videoFreeze )
			{
			    t_strm_sel_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			
			g_videoCompId.e_type = ST_VIDEO;
			memcpy( &t_strm_sel_info.t_strmCompid, &g_videoCompId, sizeof( STREAM_COMP_ID_T ));
			
			t_strm_sel_info.pf_nfy = x_svctx_select_nfy_video;
			t_strm_sel_info.pv_nfy_tag = NULL;
			
			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_BROADCAST, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}

			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet ) 
			{
				CHK_FAIL(i4_ret);
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
			break;

		case AC_M_ONLY_AUDIO:
			g_audioCompId.e_type = ST_AUDIO;
			memcpy( &t_strm_sel_info.t_strmCompid, &g_audioCompId, sizeof( STREAM_COMP_ID_T ));
			t_strm_sel_info.pf_nfy = x_svctx_select_nfy_audio;
			t_strm_sel_info.pv_nfy_tag = NULL;
			
			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_PLAY_BROADCAST, ( VOID* )&t_strm_sel_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}

			ACI_WAIT( cvRet, pidCondVar, pidLock, MEDIA_TIMEOUT );
			if ( AC_CV_SUCCESS != cvRet )
			{
				CHK_FAIL(i4_ret);
				aci_dbg(( "[MDGLUE] ac_cv_wait is timeout \n" ));
				goto finally;
			}
			aci_setAudioStatus( AC_TRUE );
			break;
		}
	aci_dbg(( "glue: ac_m_playBroadcast() 2\n" ));
	ACI_UNLOCK( pidLock );
	return AC_TRUE;

#endif
finally:
    ACI_UNLOCK(pidLock);
    aci_err(("glue: ac_m_playBroadcast() faild\n"));
    return AC_FALSE;
}

/********************************************************************************************************

Play a stream:
Note: this fucntion shoould be called after ac_c_requestServiceSelection

********************************************************************************************************/

void ac_m_stopBroadcast(AC_UINT32 deviceId, AC_M_DEC_SELECT_MODE mode) 
{
#ifndef __MTK_CHANNEL_CHANGE_V2__
    AC_INT32    i4_ret;
    SCDB_REC_T  scdb_rec;

    ACI_LOCK( pidLock, AC_MU_WAIT, 0 );
    aci_dbg(( "glue: ac_m_stopBroadcast() 1\n" ));
 
    switch ( mode ) 
	{
	    case AC_M_AUDIO_AND_VIDEO:
	        if ( videoFreeze ) 
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL( i4_ret );
	            videoFreeze = AC_FALSE;
	        }

	        i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_VIDEO, &g_videoCompId, &scdb_rec );
	        CHK_FAIL(i4_ret );

	        i4_ret = x_svctx_sync_stop_stream( aci_svctx, ST_VIDEO, DEFAULT_STOP_SVC_TIMEOUT);
	        CHK_FAIL( i4_ret );

	        i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_AUDIO, &g_audioCompId, &scdb_rec );
	        CHK_FAIL( i4_ret );

	        i4_ret = x_svctx_sync_stop_stream( aci_svctx, ST_AUDIO, DEFAULT_STOP_SVC_TIMEOUT );
	        CHK_FAIL( i4_ret );
	        aci_setAudioStatus( AC_FALSE );
	        break;

	    case AC_M_ONLY_VIDEO:
	        if ( videoFreeze )
			{
	            i4_ret = x_svctx_set_video_plane( aci_svctx, SCC_VID_DEINT );
	            CHK_FAIL( i4_ret );
	            videoFreeze = AC_FALSE;
	        }
	        i4_ret= x_svctx_get_using_strm_data( aci_svctx, ST_VIDEO, &g_videoCompId, &scdb_rec );
	        CHK_FAIL( i4_ret );

	        i4_ret = x_svctx_sync_stop_stream( aci_svctx, ST_VIDEO, DEFAULT_STOP_SVC_TIMEOUT );
	        CHK_FAIL( i4_ret );
	        break;
	        
	    case AC_M_ONLY_AUDIO:
	        i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_AUDIO, &g_audioCompId, &scdb_rec );
	        CHK_FAIL( i4_ret );

	        i4_ret = x_svctx_sync_stop_stream( aci_svctx, ST_AUDIO, DEFAULT_STOP_SVC_TIMEOUT );
	        CHK_FAIL( i4_ret );
	        aci_setAudioStatus( AC_FALSE );
	        break;
    }
#else
	AC_INT32	           i4_ret;
	MHP_STRM_STOP_BY_TYPE   t_strm_stop_info;

	ACI_LOCK( pidLock, AC_MU_WAIT, 0 );
	aci_dbg(( "glue: ac_m_stopBroadcast() 1\n" ));
	
    memset( &t_strm_stop_info, 0, sizeof( MHP_STRM_STOP_BY_TYPE ) );
	switch ( mode ) 
	{
		case AC_M_AUDIO_AND_VIDEO:
			if ( videoFreeze ) 
			{
				t_strm_stop_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			t_strm_stop_info.e_strm_type = ST_VIDEO;
			t_strm_stop_info.ui4_time = DEFAULT_STOP_SVC_TIMEOUT;

			i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_VIDEO, &g_videoCompId, &scdb_rec );
			CHK_FAIL(i4_ret );

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_STOP_BROADCAST, ( VOID* )&t_strm_stop_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			CHK_FAIL( i4_ret );

			memset( &t_strm_stop_info, 0, sizeof( MHP_STRM_STOP_BY_TYPE ) );
			t_strm_stop_info.e_strm_type = ST_AUDIO;
			t_strm_stop_info.ui4_time = DEFAULT_STOP_SVC_TIMEOUT;
			
			i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_AUDIO, &g_audioCompId, &scdb_rec );
			CHK_FAIL( i4_ret );

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_STOP_BROADCAST, ( VOID* )&t_strm_stop_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			
			CHK_FAIL( i4_ret );
			aci_setAudioStatus( AC_FALSE );
			break;

		case AC_M_ONLY_VIDEO:
			if ( videoFreeze ) 
			{
				t_strm_stop_info.b_video_freeze = videoFreeze;
				videoFreeze = AC_FALSE;
			}
			t_strm_stop_info.e_strm_type = ST_VIDEO;
			t_strm_stop_info.ui4_time = DEFAULT_STOP_SVC_TIMEOUT;

			i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_VIDEO, &g_videoCompId, &scdb_rec );
			CHK_FAIL(i4_ret );

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_STOP_BROADCAST, ( VOID* )&t_strm_stop_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			CHK_FAIL( i4_ret );
			break;
			
		case AC_M_ONLY_AUDIO:
			memset( &t_strm_stop_info, 0, sizeof( MHP_STRM_STOP_BY_TYPE ) );
			t_strm_stop_info.e_strm_type = ST_AUDIO;
			t_strm_stop_info.ui4_time = DEFAULT_STOP_SVC_TIMEOUT;
			
			i4_ret = x_svctx_get_using_strm_data( aci_svctx, ST_AUDIO, &g_audioCompId, &scdb_rec );
			CHK_FAIL( i4_ret );

			if ( _pf_request_op_nfy )
			{
				i4_ret = _pf_request_op_nfy( MHP_REQUEST_CMD_STOP_BROADCAST, ( VOID* )&t_strm_stop_info, _pv_req_tag, 0 );
			}
			else
			{
				aci_dbg(( "[TNGLUE] _pf_request_op_nfy is null\n" ));
				i4_ret = -1;
			}
			
			CHK_FAIL( i4_ret );
			aci_setAudioStatus( AC_FALSE );
			break;
	}

#endif
    aci_dbg(( "glue: ac_m_stopBroadcast() 2\n" ));
    ACI_UNLOCK( pidLock );
}



#endif

#endif /* _U_MHP_H_ */
