/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_opera_hbbtv.h
 *  $RCSfile: $
 *  $Revision$
 *  $Date$
 *  $Author$
 *
 *  @par Description:This header file contains HBBTV exported constants, macros, and types.
 */
/*----------------------------------------------------------------------------*/
#ifndef _U_OPERA_HBBTV_H_
#define _U_OPERA_HBBTV_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include <features.h>

__BEGIN_DECLS

#include "u_common.h"
#include "u_sm_video_hdlr.h"

/*!
 * @addtogroup    groupMW_HBBTV
 * @brief  This header file contains Opera HBBTV exported constants, macros, and types.
 *
 * @see groupMW_HBBTV
 *
 * @{
 * */

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
#define OPERA_HBBTVR_OK                   ((INT32)    0)    /**<OK                    */
#define OPERA_HBBTVR_INV_ARG              ((INT32)   -1)    /**<Invalid arguments     */
#define OPERA_HBBTVR_INV_HANDLE           ((INT32)   -2)    /**<Invalid handle        */
#define OPERA_HBBTVR_OUT_OF_MEMORY        ((INT32)   -3)    /**<Out of memory         */
#define OPERA_HBBTVR_OUT_OF_HANDLE        ((INT32)   -4)    /**<Out of handle         */
#define OPERA_HBBTVR_NOT_INIT             ((INT32)   -5)    /**<Not initialized       */
#define OPERA_HBBTVR_NOT_RUNNING          ((INT32)   -6)    /**<Not running           */
#define OPERA_HBBTVR_ALREADY_INIT         ((INT32)   -7)    /**<Already initialized   */
#define OPERA_HBBTVR_ALREADY_START        ((INT32)   -8)    /**<Already started       */
#define OPERA_HBBTVR_INTERNAL_ERROR       ((INT32)   -9)    /**<Internal error        */
#define OPERA_HBBTVR_CLI_ERROR            ((INT32)  -10)    /**<Cli init error        */
#define OPERA_HBBTVR_FAIL                 ((INT32)  -11)    /**<Fail                  */
#define OPERA_HBBTVR_INIT_FAILED          ((INT32)  -12)    /**<initialization error  */
#define OPERA_HBBTVR_LOAD_PAT_FAIL        ((INT32)  -13)    /**<load PAT failed       */
#define OPERA_HBBTVR_CREATE_TIMER_FAIL    ((INT32)  -14)    /**<create timer failed   */
#define OPERA_HBBTVR_START_TIMER_FAIL     ((INT32)  -15)    /**<create timer failed   */
#define OPERA_HBBTVR_ALREADY_EXISTED      ((INT32)  -16)    /**<already existed       */
#define OPERA_HBBTVR_LOAD_PMT_FAIL        ((INT32)  -17)    /**<load PMT failed       */
#define OPERA_HBBTVR_OUT_OF_RESOURCES     ((INT32)  -18)    /**<out of resource       */
#define OPERA_HBBTVR_LOAD_AIT_FAIL        ((INT32)  -19)    /**<load AIT failed       */
#define OPERA_HBBTVR_SET_LISTENER_FAIL    ((INT32)  -20)    /**<set listener failed   */
#define OPERA_HBBTVR_INV_COND             ((INT32)  -21)    /**<invalid condition     */
#define OPERA_HBBTVR_SEMAPHORE_FAILED     ((INT32)  -22)    /**<lock semaphore failed */
#define OPERA_HBBTVR_CMPB_ERROR           ((INT32)  -23)    /**<cmpb failed           */
#define OPERA_HBBTVR_DRM_ERROR            ((INT32)  -24)    /**<DRM failed            */
#define OPERA_HBBTVR_AIT_NOT_EXISTED      ((INT32)  -25)    /**<load AIT failed       */
#define OPERA_HBBTVR_ENG_RESTARTED        ((INT32)  -26)    /**<HbbTV Engine restarted*/
#define OPERA_HBBTVR_NOT_FOUND            ((INT32)  -27)    /**<Not found             */
#define OPERA_HBBTVR_EDB_FAIL             ((INT32)  -28)    /**<EDB FAIL              */
#define OPERA_HBBTVR_GET_SVL_REC_FAIL     ((INT32)  -29)    /**<EDB FAIL              */
#define OPERA_HBBTVR_ASYC_NOTIFY          ((INT32)  -30)    /**<EDB FAIL              */
#define OPERA_HBBTVR_NOT_ALLOWED          ((INT32)  -31)    /**<Hbbtv not allowed in svl*/
#define OPERA_HBBTVR_NOT_SUPPORT          ((INT32)  -32)    /**<Hbbtv not supported   */
#define OPERA_HBBTVR_MSG_TYPE_INVALID     ((INT32)  -33)    /**<Hbbtv message invalid */
#define OPERA_HBBTVR_LOAD_PMT_PENDING     ((INT32)  -34)    /**<Load PMT pending */




typedef enum
{
    HBBTV_APP_MODE_UNKNOWN = -1,  /* Must be set to '-1' else I loose an entry in the bit mask. */
    HBBTV_APP_MODE_HBBTV,
    HBBTV_APP_MODE_BBC
}   HBBTV_APP_MODE_TYPE_T;

#define HBBTV_APP_MODE_MASK_ALL          ((UINT32) 255)
#define HBBTV_APP_MODE_MASK_HBBTV        MAKE_BIT_MASK_32 (HBBTV_APP_MODE_HBBTV)
#define HBBTV_APP_MODE_MASK_BBC          MAKE_BIT_MASK_32 (HBBTV_APP_MODE_BBC)

#define OPERA_HBBTV_SCREEN_WIDTH                  ((UINT32)   1920)
#define OPERA_HBBTV_SCREEN_HEIGHT                 ((UINT32)   1080)
#define OPERA_HBBTV_SCREEN_ASP_RATIO_16_9_WIDTH   ((UINT32)  16)
#define OPERA_HBBTV_SCREEN_ASP_RATIO_16_9_HEIGHT  ((UINT32)   9)
#define OPERA_HBBTV_SCREEN_ASP_RATIO_4_3_WIDTH    ((UINT32)   4)
#define OPERA_HBBTV_SCREEN_ASP_RATIO_4_3_HEIGHT   ((UINT32)   3)


#define HBBTV_MAX_TUNER_NAME      (SYS_NAME_LEN+1)
#define HBBTV_MAXBRDCST_MEDIUM    ((UINT8)8)

typedef enum _OPERA_HBBTV_ASP_RATIO_T
{
    OPERA_HBBTV_ASP_RATIO_4_3     = 0x00,
    OPERA_HBBTV_ASP_RATIO_16_9    = 0x01
} OPERA_HBBTV_ASP_RATIO_T;

typedef enum _HBBTV_BRWS_STATUS_TAG
{
    HBBTV_STATUS_UNKNOWN,
    HBBTV_STATUS_BACKGROUND,
    HBBTV_STATUS_HBBTV_RUNNING,
    HBBTV_STATUS_NONE_HBBTV_SVC,
    HBBTV_STATUS_BWS_RESTARTING,
    HBBTV_STATUS_MAX
} HBBTV_BRWS_STATUS_E;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_SUBTITLE_ATTR_T
 *  @brief HBBTV subtitle attribute setting
 *  @code
 *  typedef enum _OPERA_HBBTV_VIDEO_REGION_T
 *  {
 *      VSH_REGION_INFO_T t_region;
 *  } OPERA_HBBTV_VIDEO_REGION_T;
 *
 *  @endcode
 *  @li@c  t_region - position informaiton (x, y, width, high)
 */
/*------------------------------------------------------------------*/
typedef struct _OPERA_HBBTV_VIDEO_REGION_T
{
    VSH_REGION_INFO_T  t_region;
} OPERA_HBBTV_VIDEO_REGION_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_SUBTITLE_ENABLE_T
 *  @brief HBBTV subtitle enable status
 *  @code
 *  typedef enum _HBBTV_SUBTITLE_ENABLE_T
 *  {
 *      OPERA_HBBTV_SUBTITLE_OFF = 0,
 *      OPERA_HBBTV_SUBTITLE_ON,
 *      OPERA_HBBTV_SUBTITLE_MUTE_ON
 *  } OPERA_HBBTV_SUBTITLE_ENABLE_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_SUBTITLE_OFF               - subtitle off
 *  @li@c  OPERA_HBBTV_SUBTITLE_ON                -  subtitle off
 *  @li@c  OPERA_HBBTV_SUBTITLE_MUTE_ON           - subtitle  on when mute
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_SUBTITLE_ENABLE_T
{
    OPERA_HBBTV_SUBTITLE_OFF = 0,
    OPERA_HBBTV_SUBTITLE_ON,
    OPERA_HBBTV_SUBTITLE_MUTE_ON
} OPERA_HBBTV_SUBTITLE_ENABLE_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_SUBTITLE_ATTR_T
 *  @brief HBBTV subtitle attribute setting
 *  @code
 *  typedef enum _OPERA_HBBTV_SUBTITLE_ATTR_T
 *  {
 *      OPERA_HBBTV_SUBTITLE_ATTR_NORMAL = 0,
 *      OPERA_HBBTV_ATTR_HEARING_IMPAIRED
 *  } OPERA_HBBTV_SUBTITLE_ATTR_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_SUBTITLE_ATTR_NORMAL     - user setting normal
 *  @li@c  OPERA_HBBTV_ATTR_HEARING_IMPAIRED    - user setting hearing impaired
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_SUBTITLE_ATTR_T
{
    OPERA_HBBTV_SUBTITLE_ATTR_NORMAL = 0,
    OPERA_HBBTV_ATTR_HEARING_IMPAIRED,
    OPERA_HBBTV_SUBTITLE_ATTR_UNKNOWN
} OPERA_HBBTV_SUBTITLE_ATTR_T;


/*! @enum HBBTV_STREAM_SUBTITLE_TYPE_T
 *  @brief subtitle type
 */
typedef enum
{
    HBBTV_STREAM_SUBTITLE_UNKNOWN = 0,  ///< unknown
    HBBTV_STREAM_SUBTITLE_TTML_TYPE,    ///< TTML, related parameter is: IMTK_STREAM_COMMON_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_DIVX_TYPE,    ///< Divx, related parameter is: IMTK_STREAM_COMMON_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_TEXT_ASS,     ///< Text_ASS, related parameter is: IMTK_STREAM_COMMON_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_TEXT_SRT,     ///< Text_SRT, related parameter is: IMTK_STREAM_COMMON_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_RLE,          ///< RLE, related parameter is: IMTK_STREAM_COMMON_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_CC_TYPE,      ///< Closecaption, related parameter is: IMTK_STREAM_CC_INFO_T
    HBBTV_STREAM_SUBTITLE_DVB_TYPE,     ///< DVB, related parameter is: IMTK_STREAM_DVB_SUBTITLE_INFO_T
    HBBTV_STREAM_SUBTITLE_TELTEXT_TYPE  ///< Teltext, related parameter is: IMTK_STREAM_TELETEXT_SUBTITLE_INFO_T
} HBBTV_STREAM_SUBTITLE_TYPE_T;

/*! @enum HBBTV_STREAM_SUBTITLE_TYPE_T
 *  @brief subtitle type
 */
typedef struct _HBBTV_STREAM_SUBTITLE_T {
    UINT32 index;                     ///< the index
    BOOL active;                      ///< active or not
    ISO_639_LANG_T lang;              ///< iso 639 lang
    HBBTV_STREAM_SUBTITLE_TYPE_T type;       ///< type
    OPERA_HBBTV_SUBTITLE_ATTR_T attr; ///< attribute
} HBBTV_STREAM_SUBTITLE_T;

/*! @enum HBBTV_STREAM_SUBTITLE_LIST_T
 *  @brief subtitle list
 */
typedef struct _HBBTV_STREAM_SUBTITLE_LIST_TAG {
    UINT32 nmemb;                      ///< the number of member
    UINT32 ntotal;
    HBBTV_STREAM_SUBTITLE_T items[0];  ///< the variable array
} HBBTV_STREAM_SUBTITLE_LIST_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_PRIVACY_TYPE_T
 *  @brief HBBTV privacy type.
 *  @code
 *  typedef enum _OPERA_HBBTV_PRIVACY_TYPE_T
 *  {
 *      OPERA_HBBTV_PRIVACY_UNKNOW              = 0,
 *      OPERA_HBBTV_PRIVACY_DO_NOT_TRACK,
 *      OPERA_HBBTV_PRIVACY_3RD_PARTY_COOKIE,
 *      OPERA_HBBTV_PRIVACY_PERSISTENT_STORAGE,
 *      OPERA_HBBTV_PRIVACY_BLOCK_TRACK_SITES,
 *      OPERA_HBBTV_PRIVACY_END
 *  } OPERA_HBBTV_PRIVACY_TYPE_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_PRIVACY_DO_NOT_TRACK        - Do not track
 *  @li@c  OPERA_HBBTV_PRIVACY_3RD_PARTY_COOKIE    - third party cookie
 *  @li@c  OPERA_HBBTV_PRIVACY_PERSISTENT_STORAGE  - Pesistent storage
 *  @li@c  OPERA_HBBTV_PRIVACY_BLOCK_TRACK_SITES   - blocking tracking web sites.
 *  @li@c  OPERA_HBBTV_PRIVACY_END                 - end
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_PRIVACY_TYPE_T
{
    OPERA_HBBTV_PRIVACY_UNKNOW              = 0,
    OPERA_HBBTV_PRIVACY_DO_NOT_TRACK,
    OPERA_HBBTV_PRIVACY_3RD_PARTY_COOKIE,
    OPERA_HBBTV_PRIVACY_PERSISTENT_STORAGE,
    OPERA_HBBTV_PRIVACY_BLOCK_TRACK_SITES,
    OPERA_HBBTV_PRIVACY_END
} OPERA_HBBTV_PRIVACY_TYPE_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_PRIVACY_CFG_E
 *  @brief HBBTV privacy type.
 *  @code
 *   typedef enum {
 *       OPERA_HBBTV_PRIVACY_CFG_OFF = 0,
 *       OPERA_HBBTV_PRIVACY_CFG_ON  = 1,
 *       OPERA_HBBTV_PRIVACY_CFG_NOT_SET = 2
 *   } OPERA_HBBTV_PRIVACY_E;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_PRIVACY_CFG_ON      - Do not track
 *  @li@c  OPERA_HBBTV_PRIVACY_CFG_OFF     - third party cookie
 *  @li@c  OPERA_HBBTV_PRIVACY_CFG_NOT_SET - Pesistent storage
 */
/*------------------------------------------------------------------*/
typedef enum {
    OPERA_HBBTV_PRIVACY_CFG_OFF = 0,
    OPERA_HBBTV_PRIVACY_CFG_ON  = 1,
    OPERA_HBBTV_PRIVACY_CFG_NOT_SET = 2
} OPERA_HBBTV_PRIVACY_E;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_DEVICEID_CFG_E
 *  @brief HBBTV device ID config type.
 *  @code
 *   typedef enum {
 *       OPERA_HBBTV_DEVICEID_CFG_OFF = 0,
 *       OPERA_HBBTV_DEVICEID_CFG_ON  = 1,
 *   } OPERA_HBBTV_PRIVACY_E;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_DEVICEID_CFG_ON      - with device id
 *  @li@c  OPERA_HBBTV_DEVICEID_CFG_OFF     - no device id
 */
/*------------------------------------------------------------------*/
typedef enum {
    OPERA_HBBTV_DEVICEID_CFG_OFF = 0,
    OPERA_HBBTV_DEVICEID_CFG_ON  = 1
} OPERA_HBBTV_DEVICEID_CFG_E;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_DRM_TYPE_T
 *  @brief HBBTV drm type.
 *  @code
 *  typedef enum _OPERA_HBBTV_DRM_TYPE_T
 *  {
 *      OPERA_HBBTV_DRM_TYPE_MARLIN = 0,
 *      OPERA_HBBTV_DRM_TYPE_PLAYREADY,
 *      OPERA_HBBTV_DRM_TYPE_NO_DRM,
 *      OPERA_HBBTV_DRM_TYPE_END
 *  } OPERA_HBBTV_DRM_TYPE_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_DRM_TYPE_MARLIN        - marlin
 *  @li@c  OPERA_HBBTV_DRM_TYPE_PLAYREADY     - playready
 *  @li@c  OPERA_HBBTV_DRM_TYPE_NO_DRM        - no drm
 *  @li@c  OPERA_HBBTV_DRM_TYPE_END           - end
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_DRM_TYPE_T
{
    OPERA_HBBTV_DRM_TYPE_MARLIN = 0,
    OPERA_HBBTV_DRM_TYPE_PLAYREADY,
    OPERA_HBBTV_DRM_TYPE_NO_DRM,
    OPERA_HBBTV_DRM_TYPE_END
} OPERA_HBBTV_DRM_TYPE_T;

/*------------------------------------------------------------------*/
/*! @struct  OPERA_HBBTV_DRM_PRIO_ARR_T
 *  @brief  For client to set drm priority.
 *  @code
 *  typedef struct
 *  {
 *      OPERA_HBBTV_DRM_TYPE_T  e_drm_type;
 *  } OPERA_HBBTV_DRM_PRIO_ARR_T;
 *  @endcode
 *  @li@c  e_drm_type       - drm type
 */
/*------------------------------------------------------------------*/
typedef struct
{
    OPERA_HBBTV_DRM_TYPE_T  e_drm_type;
} OPERA_HBBTV_DRM_PRIO_ARR_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_DRM_PRIO_TYPE_T
 *  @brief HBBTV DRM priority type.
 *  @code
 *  typedef enum _OPERA_HBBTV_DRM_PRIO_TYPE_T
 *  {
 *      OPERA_HBBTV_DRM_PRIO_TYPE_UNKNOWN = 0,
 *      OPERA_HBBTV_DRM_PRIO_TYPE_MARLIN_PLAYREADY,
 *      OPERA_HBBTV_DRM_PRIO_TYPE_PLAYREADY_MARLIN,
 *      OPERA_HBBTV_DRM_PRIO_TYPE_NO_DRM
 *  } OPERA_HBBTV_DRM_PRIO_TYPE_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_DRM_PRIO_TYPE_UNKNOWN           - marlin
 *  @li@c  OPERA_HBBTV_DRM_PRIO_TYPE_MARLIN_PLAYREADY  - playready
 *  @li@c  OPERA_HBBTV_DRM_PRIO_TYPE_PLAYREADY_MARLIN  - no drm
 *  @li@c  OPERA_HBBTV_DRM_PRIO_TYPE_NO_DRM            - end
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_DRM_PRIO_TYPE_T
{
    OPERA_HBBTV_DRM_PRIO_TYPE_UNKNOWN = 0,
    OPERA_HBBTV_DRM_PRIO_TYPE_MARLIN_PLAYREADY,
    OPERA_HBBTV_DRM_PRIO_TYPE_PLAYREADY_MARLIN,
    OPERA_HBBTV_DRM_PRIO_TYPE_NO_DRM
} OPERA_HBBTV_DRM_PRIO_TYPE_T;

/*------------------------------------------------------------------*/
/*! @struct  OPERA_HBBTV_RUNNING_APP_T
 *  @brief HBBTV running app info structure
 *  @code
 *  typedef enum _OPERA_HBBTV_VIDEO_REGION_T
 *  {
 *      VSH_REGION_INFO_T t_region;
 *  } OPERA_HBBTV_VIDEO_REGION_T;
 *
 *  @endcode
 *  @li@c  t_region - position informaiton (x, y, width, high)
 */
/*------------------------------------------------------------------*/

typedef struct _OPERA_HBBTV_RUNNING_APP_T
{
    UINT32      ui4_org_id;
    UINT16      ui2_app_id;
    BOOL        b_broadcast_related;
    BOOL        b_autostart_launching;
    BOOL        b_digital_ttx;
    BOOL        b_secure_page;
    BOOL        b_accept;
}OPERA_HBBTV_RUNNING_APP_T;

/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_KEY_CODE_T
 *  @brief HBBTV key code that mapping to opera OMI key.
 *  @code
 *  typedef enum _OPERA_HBBTV_KEY_CODE_T
 *  {
 *      OPERA_HBBTV_KEY_UNDEFINED = 0,
 *      ......
 *      OPERA_HBBTV_KEY_PLAY_PAUSE
 *  } OPERA_HBBTV_KEY_CODE_T;
 *
 *  @endcode
 *  @li@c  OPERA_HBBTV_KEY_UNDEFINED     - Undefined key.
 *  @li@c  ......
 *  @li@c  OPERA_HBBTV_KEY_PLAY_PAUSE    - Play/Pause key.
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_KEY_CODE_T {
    OPERA_HBBTV_KEY_UNDEFINED = 0,
    OPERA_HBBTV_KEY_CANCEL = 3,
    OPERA_HBBTV_KEY_BACKSPACE = 8,
    OPERA_HBBTV_KEY_TAB = 9,
    OPERA_HBBTV_KEY_CLEAR = 12,
    OPERA_HBBTV_KEY_ENTER = 13,
    OPERA_HBBTV_KEY_SHIFT = 16,
    OPERA_HBBTV_KEY_CTRL = 17,
    OPERA_HBBTV_KEY_ALT = 18,
    OPERA_HBBTV_KEY_PAUSE = 19,
    OPERA_HBBTV_KEY_CAPS_LOCK = 20,
    OPERA_HBBTV_KEY_KANA = 21,
    OPERA_HBBTV_KEY_FINAL = 24,
    OPERA_HBBTV_KEY_KANJI = 25,
    OPERA_HBBTV_KEY_ESCAPE = 27,
    OPERA_HBBTV_KEY_CONVERT = 28,
    OPERA_HBBTV_KEY_NONCONVERT = 29,
    OPERA_HBBTV_KEY_ACCEPT = 30,
    OPERA_HBBTV_KEY_MODE_CHANGE = 31,
    OPERA_HBBTV_KEY_SPACE = 32,
    OPERA_HBBTV_KEY_PAGE_UP = 33,
    OPERA_HBBTV_KEY_PAGE_DOWN = 34,
    OPERA_HBBTV_KEY_END = 35,
    OPERA_HBBTV_KEY_HOME = 36,
    OPERA_HBBTV_KEY_LEFT = 37,
    OPERA_HBBTV_KEY_UP = 38,
    OPERA_HBBTV_KEY_RIGHT = 39,
    OPERA_HBBTV_KEY_DOWN = 40,
    OPERA_HBBTV_KEY_PRINT_SCREEN = 44,
    OPERA_HBBTV_KEY_INSERT = 45,
    OPERA_HBBTV_KEY_DELETE = 46,
    OPERA_HBBTV_KEY_HELP = 47,
    OPERA_HBBTV_KEY_0 = 48,
    OPERA_HBBTV_KEY_1 = 49,
    OPERA_HBBTV_KEY_2 = 50,
    OPERA_HBBTV_KEY_3 = 51,
    OPERA_HBBTV_KEY_4 = 52,
    OPERA_HBBTV_KEY_5 = 53,
    OPERA_HBBTV_KEY_6 = 54,
    OPERA_HBBTV_KEY_7 = 55,
    OPERA_HBBTV_KEY_8 = 56,
    OPERA_HBBTV_KEY_9 = 57,
    OPERA_HBBTV_KEY_A = 65,
    OPERA_HBBTV_KEY_B = 66,
    OPERA_HBBTV_KEY_C = 67,
    OPERA_HBBTV_KEY_D = 68,
    OPERA_HBBTV_KEY_E = 69,
    OPERA_HBBTV_KEY_F = 70,
    OPERA_HBBTV_KEY_G = 71,
    OPERA_HBBTV_KEY_H = 72,
    OPERA_HBBTV_KEY_I = 73,
    OPERA_HBBTV_KEY_J = 74,
    OPERA_HBBTV_KEY_K = 75,
    OPERA_HBBTV_KEY_L = 76,
    OPERA_HBBTV_KEY_M = 77,
    OPERA_HBBTV_KEY_N = 78,
    OPERA_HBBTV_KEY_O = 79,
    OPERA_HBBTV_KEY_P = 80,
    OPERA_HBBTV_KEY_Q = 81,
    OPERA_HBBTV_KEY_R = 82,
    OPERA_HBBTV_KEY_S = 83,
    OPERA_HBBTV_KEY_T = 84,
    OPERA_HBBTV_KEY_U = 85,
    OPERA_HBBTV_KEY_V = 86,
    OPERA_HBBTV_KEY_W = 87,
    OPERA_HBBTV_KEY_X = 88,
    OPERA_HBBTV_KEY_Y = 89,
    OPERA_HBBTV_KEY_Z = 90,
    OPERA_HBBTV_KEY_NUMPAD0 = 96,
    OPERA_HBBTV_KEY_NUMPAD1 = 97,
    OPERA_HBBTV_KEY_NUMPAD2 = 98,
    OPERA_HBBTV_KEY_NUMPAD3 = 99,
    OPERA_HBBTV_KEY_NUMPAD4 = 100,
    OPERA_HBBTV_KEY_NUMPAD5 = 101,
    OPERA_HBBTV_KEY_NUMPAD6 = 102,
    OPERA_HBBTV_KEY_NUMPAD7 = 103,
    OPERA_HBBTV_KEY_NUMPAD8 = 104,
    OPERA_HBBTV_KEY_NUMPAD9 = 105,
  /**
     The "*" on the numeric pad.
  */
    OPERA_HBBTV_KEY_MULTIPLY = 106,
  /**
     The "+" on the numeric pad.
  */
    OPERA_HBBTV_KEY_ADD = 107,
  /**
     The "." on the numeric pad.
  */
    OPERA_HBBTV_KEY_SEPARATOR = 108,
  /**
     The "-" on the numeric pad.
  */
    OPERA_HBBTV_KEY_SUBTRACT = 109,
  /**
     The "," on the numeric pad.
  */
    OPERA_HBBTV_KEY_DECIMAL = 110,
  /**
     The "/" on the numeric pad.
  */
    OPERA_HBBTV_KEY_DIVIDE = 111,
    OPERA_HBBTV_KEY_F1 = 112,
    OPERA_HBBTV_KEY_F2 = 113,
    OPERA_HBBTV_KEY_F3 = 114,
    OPERA_HBBTV_KEY_F4 = 115,
    OPERA_HBBTV_KEY_F5 = 116,
    OPERA_HBBTV_KEY_F6 = 117,
    OPERA_HBBTV_KEY_F7 = 118,
    OPERA_HBBTV_KEY_F8 = 119,
    OPERA_HBBTV_KEY_F9 = 120,
    OPERA_HBBTV_KEY_F10 = 121,
    OPERA_HBBTV_KEY_F11 = 122,
    OPERA_HBBTV_KEY_F12 = 123,
    OPERA_HBBTV_KEY_NUM_LOCK = 144,
    OPERA_HBBTV_KEY_SCROLL_LOCK = 145,
    OPERA_HBBTV_KEY_META = 151,
  /**
     The key with the ; and : characters.
  */
    OPERA_HBBTV_KEY_OEM_SEMICOLON = 186,
  /**
     The key with the = and + characters.
  */
    OPERA_HBBTV_KEY_OEM_EQUALS = 187,
  /**
     The key with the , and < characters.
  */
    OPERA_HBBTV_KEY_OEM_COMMA = 188,
  /**
     The key with the - and _ characters.
  */
    OPERA_HBBTV_KEY_OEM_MINUS = 189,
  /**
     The key with the . and > characters.
  */
    OPERA_HBBTV_KEY_OEM_PERIOD = 190,
  /**
     The key with the / and ? characters.
  */
    OPERA_HBBTV_KEY_OEM_SLASH = 191,
  /**
     The key with the ` and ~ characters.
  */
    OPERA_HBBTV_KEY_OEM_LEFT_QUOTE = 192,
  /**
     The key with the [ and { characters.
  */
    OPERA_HBBTV_KEY_OEM_OPEN_BRACKET = 219,
  /**
     The key with the \ and | characters.
  */
    OPERA_HBBTV_KEY_OEM_BACKSLASH = 220,
  /**
     The key with the ] and } characters.
  */
    OPERA_HBBTV_KEY_OEM_CLOSE_BRACKET = 221,
  /**
     The key with the ' and " characters.
  */
    OPERA_HBBTV_KEY_OEM_RIGHT_QUOTE = 222,
    OPERA_HBBTV_KEY_RED = 403,
    OPERA_HBBTV_KEY_GREEN = 404,
    OPERA_HBBTV_KEY_YELLOW = 405,
    OPERA_HBBTV_KEY_BLUE = 406,
    OPERA_HBBTV_KEY_GREY = 407,
    OPERA_HBBTV_KEY_BROWN = 408,
    OPERA_HBBTV_KEY_POWER = 409,
    OPERA_HBBTV_KEY_DIMMER = 410,
    OPERA_HBBTV_KEY_WINK = 411,
    OPERA_HBBTV_KEY_REWIND = 412,
    OPERA_HBBTV_KEY_STOP = 413,
    OPERA_HBBTV_KEY_EJECT_TOGGLE = 414,
    OPERA_HBBTV_KEY_PLAY = 415,
    OPERA_HBBTV_KEY_RECORD = 416,
    OPERA_HBBTV_KEY_FAST_FWD = 417,
    OPERA_HBBTV_KEY_PLAY_SPEED_UP = 418,
    OPERA_HBBTV_KEY_PLAY_SPEED_DOWN = 419,
    OPERA_HBBTV_KEY_PLAY_SPEED_RESET = 420,
    OPERA_HBBTV_KEY_RECORD_SPEED_NEXT = 421,
    OPERA_HBBTV_KEY_GO_TO_START = 422,
    OPERA_HBBTV_KEY_GO_TO_END = 423,
    OPERA_HBBTV_KEY_TRACK_PREV = 424,
    OPERA_HBBTV_KEY_TRACK_NEXT = 425,
    OPERA_HBBTV_KEY_RANDOM_TOGGLE = 426,
    OPERA_HBBTV_KEY_CHANNEL_UP = 427,
    OPERA_HBBTV_KEY_CHANNEL_DOWN = 428,
    OPERA_HBBTV_KEY_STORE_FAVORITE_0 = 429,
    OPERA_HBBTV_KEY_STORE_FAVORITE_1 = 430,
    OPERA_HBBTV_KEY_STORE_FAVORITE_2 = 431,
    OPERA_HBBTV_KEY_STORE_FAVORITE_3 = 432,
    OPERA_HBBTV_KEY_RECALL_FAVORITE_0 = 433,
    OPERA_HBBTV_KEY_RECALL_FAVORITE_1 = 434,
    OPERA_HBBTV_KEY_RECALL_FAVORITE_2 = 435,
    OPERA_HBBTV_KEY_RECALL_FAVORITE_3 = 436,
    OPERA_HBBTV_KEY_CLEAR_FAVORITE_0 = 437,
    OPERA_HBBTV_KEY_CLEAR_FAVORITE_1 = 438,
    OPERA_HBBTV_KEY_CLEAR_FAVORITE_2 = 439,
    OPERA_HBBTV_KEY_CLEAR_FAVORITE_3 = 440,
    OPERA_HBBTV_KEY_SCAN_CHANNELS_TOGGLE = 441,
    OPERA_HBBTV_KEY_PINP_TOGGLE = 442,
    OPERA_HBBTV_KEY_SPLIT_SCREEN_TOGGLE = 443,
    OPERA_HBBTV_KEY_DISPLAY_SWAP = 444,
    OPERA_HBBTV_KEY_SCREEN_MODE_NEXT = 445,
    OPERA_HBBTV_KEY_VIDEO_MODE_NEXT = 446,
    OPERA_HBBTV_KEY_VOLUME_UP = 447,
    OPERA_HBBTV_KEY_VOLUME_DOWN = 448,
    OPERA_HBBTV_KEY_MUTE = 449,
    OPERA_HBBTV_KEY_SURROUND_MODE_NEXT = 450,
    OPERA_HBBTV_KEY_BALANCE_RIGHT = 451,
    OPERA_HBBTV_KEY_BALANCE_LEFT = 452,
    OPERA_HBBTV_KEY_FADER_FRONT = 453,
    OPERA_HBBTV_KEY_FADER_REAR = 454,
    OPERA_HBBTV_KEY_BASS_BOOST_UP = 455,
    OPERA_HBBTV_KEY_BASS_BOOST_DOWN = 456,
    OPERA_HBBTV_KEY_INFO = 457,
    OPERA_HBBTV_KEY_GUIDE = 458,
    OPERA_HBBTV_KEY_TELETEXT = 459,
    OPERA_HBBTV_KEY_SUBTITLE = 460,
    OPERA_HBBTV_KEY_BACK = 461,
    OPERA_HBBTV_KEY_MENU = 462,
    OPERA_HBBTV_KEY_PLAY_PAUSE = 463,

    /* Customer Key Code */
    OPERA_HBBTV_KEY_EXIT = 800,
    OPERA_HBBTV_KEY_MTS = 801
} OPERA_HBBTV_KEY_CODE_T;

/*------------------------------------------------------------------*/
/*! @brief Callback to notify the GUI states
 *  @param [in]  ui4_msg  - HBBTV_MSG_GRD_KEY_DOWN
 *  @param [in]  pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_opera_hbbtv_osd_nfy_fct)(
    UINT32  ui4_msg,
    VOID*   pv_param,
    VOID*   pv_tag);

/*------------------------------------------------------------------*/
/*! @brief Callback to notify the osd show/hide
 *  @param [in]  ui4_msg  - HBBTV_MSG_GRD_KEY_DOWN
 *  @param [in]  pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_opera_hbbtv_osd_show_nfy_fct) (
    BOOL    b_show,
    VOID*   pv_tag);

/*------------------------------------------------------------------*/
/*! @brief Callback to notify Galio to start the specified URL
 *  @param [in]  ui4_msg  - HBBTV_MSG_GRD_KEY_DOWN
 *  @param [in]  pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_opera_hbbtv_layers_goto_url)(
    const CHAR*     s_uri,
    VOID*           pv_tag );

/*------------------------------------------------------------------*/
/*! @brief Callback to notify Galio to create the specified application
 *  @param [in]  ui4_msg  - HBBTV_MSG_GRD_KEY_DOWN
 *  @param [in]  pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_opera_hbbtv_layers_create_app)(
    BOOL            b_brdcst_related,
    const CHAR*     s_uri,
    VOID*           pv_tag );

/*------------------------------------------------------------------*/
/*! @brief Callback to notify Galio of the key event
 *  @param [in]  ui4_msg  - HBBTV_MSG_GRD_KEY_DOWN
 *  @param [in]  pv_param - Parameters
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_opera_hbbtv_layers_input_keypress)(
    UINT32          ui4_key_code,
    UINT32          ui4_type,
    VOID*           pv_tag );

/*------------------------------------------------------------------*/
/*! @brief Callback to notify internet browser to set/get information.
 *  @param [in]  i4_msg_type  - message type
 *  @param [in]  pv_data      - data
 *  @param [in]  z_data_len   - data len
 *  @param [in]  pv_nfy_tag   - tag value
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_opera_hbbtv_browser_com_nfy_fct)(
    INT32               i4_msg_type,
    VOID*               pv_data,
    UINT32              z_data_len,
    VOID*               pv_nfy_tag);


typedef enum _HBBTV_APP_SRC_TYPE_T
{
    HBBTV_APP_SRC_TYPE_NONE = 0,
    HBBTV_APP_SRC_TYPE_HBBTV_NORMAL,
    HBBTV_APP_SRC_TYPE_BBC_MHEG,
    HBBTV_APP_SRC_TYPE_BBC_NETTV,
    HBBTV_APP_SRC_TYPE_BBC_DIAL,
    HBBTV_APP_SRC_TYPE_END
} HBBTV_APP_SRC_TYPE_T;

typedef struct _HBBTV_APP_INFO_T
{
    HBBTV_APP_SRC_TYPE_T    e_app_type;
    CHAR*                   s_uri;
} HBBTV_APP_INFO_T;

typedef struct _HBBTV_APP_KILL_INFO_T
{
    BOOL b_sync_kill;
    BOOL b_auto_start;
    BOOL b_select_svc;
    BOOL b_resume;
    HBBTV_APP_SRC_TYPE_T    e_app_src_type;
} HBBTV_APP_KILL_INFO_T;

typedef enum _HBBTV_APP_SVC_TYPE_T
{
    HBBTV_APP_SVC_TYPE_UNKNOWN = 0,
    HBBTV_APP_SVC_TYPE_BBC_SPORTS,
    HBBTV_APP_SVC_TYPE_BBC_NEWS,
    HBBTV_APP_SVC_TYPE_BBC_IPLAYER,
    HBBTV_APP_SVC_TYPE_NL_NRJ12,
    HBBTV_APP_SVC_TYPE_NL_CHERIE25,
    HBBTV_APP_SVC_TYPE_ES_EITB,
    HBBTV_APP_SVC_TYPE_FR_ARTE,
    HBBTV_APP_SVC_TYPE_DE_ARTE,
    HBBTV_APP_SVC_TYPE_DK_DR,
    HBBTV_APP_SVC_TYPE_END
} HBBTV_APP_SVC_TYPE_T;

typedef enum _HBBTV_APP_EVENT_T
{
    HBBTV_APP_EVENT_UNKNOWN = 0,
    HBBTV_APP_EVENT_CREATINNG,
    HBBTV_APP_EVENT_CREATED,
    HBBTV_APP_EVENT_KILLED,
    HBBTV_APP_EVENT_LAUNCH_FAIL
} HBBTV_APP_EVENT_T;

typedef INT32 (*x_hbbtv_app_event_nfy_fct)( HBBTV_APP_EVENT_T   e_app_event,
                                            HBBTV_APP_SRC_TYPE_T    e_app_type,
                                            UINT32                  ui4_data,
                                            VOID*                   pv_tag );

#define SHARE_FD_ID_ALL_SVC      's'
#define SHARE_FD_ID_COMP_LIST    'c'
#define SHARE_FD_ID_GENERIC      'g'
#define SHARE_FD_ID_PROG_LIST    'p'
#define SHARE_FD_ID_PROG_PAGE    'e'
#define SHARE_FD_ID_AUTH_URL     'a'

#define FVP_CFG_FILE "/3rd_rw/fvp.cfg"
/*! @}  */
__END_DECLS

#endif /* _U_OPERA_HBBTV_H_ */


