/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
 
/*----------------------------------------------------------------------------*/
/*! @file u_sb.h 
 *  $RCSfile: $
 *  $Revision: #4 $
 *  $Date: 2015/07/04 $
 *  $Author: xiaojun.hu $
 *  
 *  @par Description:
 *         This header file contains SVL Builder specific
 *         definitions, which are exported.
 */
/*----------------------------------------------------------------------------*/
#ifndef _U_SB_H_
#define _U_SB_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_SVL_BLDR
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_handle_grp.h"
#include "u_svl.h"
#include "u_lol.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* SVL Builder Return values */
#define SBR_OK                          ((INT32)     0)    /**< ok       */
#define SBR_ALREADY_INIT                ((INT32)    -1)    /**< engine already init       */
#define SBR_BLDR_NOT_FOUND              ((INT32)    -2)    /**< no such build name       */
#define SBR_DUP_NAME                    ((INT32)    -3)    /**< duplicate name       */
#define SBR_FILE_NOT_FOUND              ((INT32)    -4)    /**< storage file not found       */
#define SBR_INTERNAL_ERROR              ((INT32)    -5)    /**< internal error       */
#define SBR_INV_ARG                     ((INT32)    -6)    /**< invalid arguments       */
#define SBR_INV_COND                    ((INT32)    -7)    /**< invalid condition       */
#define SBR_INV_HANDLE                  ((INT32)    -8)    /**< invalid handle       */
#define SBR_LIST_NOT_FOUND              ((INT32)    -9)    /**< no such channel list       */
#define SBR_NOT_INIT                    ((INT32)   -10)    /**< sb eng is not init       */
#define SBR_NOT_MATCH                   ((INT32)   -11)    /**< can't match sb eng       */
#define SBR_NOT_STARTER                 ((INT32)   -12)    /**< scan is not started       */
#define SBR_OUT_OF_HANDLES              ((INT32)   -13)    /**< out of handle       */
#define SBR_OUT_OF_MEM                  ((INT32)   -14)    /**< out of memory       */
#define SBR_ERR_LOCK                    ((INT32)   -15)    /**< can't lock critical code       */
#define SBR_ERR_UNLOCK                  ((INT32)   -16)    /**< can't unlock critical code       */
#define SBR_ERR_READ_REC                ((INT32)   -17)    /**< read svl record error       */
#define SBR_ERR_UPDT_REC                ((INT32)   -18)    /**< update svl record error       */
#define SBR_ERR_DEL_REC                 ((INT32)   -19)    /**< delete svl record error       */
#define SBR_ERR_SAVE_LIST               ((INT32)   -20)    /**< save channel list error       */
#define SBR_ERR_LOAD_LIST               ((INT32)   -21)    /**< load channel list error       */
#define SBR_ERR_CLEAN_LIST              ((INT32)   -22)    /**< clean channel list error       */
#define SBR_ERR_OUT_OF_RESOURCES        ((INT32)   -23)    /**< out of resouce       */
#define SBR_NOT_ENOUGH_SPACE            ((INT32)   -24)    /**< space not enough       */
#define SBR_NOT_FOUND                   ((INT32)   -25)    /**< not found       */
#define SBR_OUT_OF_RANGE                ((INT32)   -26)    /**< over the the scan range        */
#define SBR_INV_KEY_TYPE                ((INT32)   -27)    /**< invalid sb_set or sb_get key type       */
#define SBR_INV_KEY_INFO                ((INT32)   -28)    /**< invalid sb_set or sb_get key info       */
#define SBR_NOT_SUPPORT                 ((INT32)   -29)    /**< this operation is not support       */
#define SBR_INV_DB_VERSION              ((INT32)   -30)    /**< invalid database version       */
#define SBR_LIST_EMPTY                  ((INT32)   -31)    /**< list is empty  */ 

/* SVL Builder Notification Reason */
#define SB_REASON_UNKNOWN               ((UINT32)  (0x00000000))    /**<scan stop reason unknown        */
#define SB_REASON_COMPLETE              ((UINT32)  (0x00000001))    /**<scan stop reason complete        */
#define SB_REASON_PROGRESS              MAKE_BIT_MASK_32 (1)    /**<scan condition in progress        */
#define SB_REASON_CANCEL                MAKE_BIT_MASK_32 (2)    /**<scan stop reason cancel        */
#define SB_REASON_ABORT                 MAKE_BIT_MASK_32 (3)    /**<scan stop reason abort        */
#define SB_REASON_FREQ_REPORT           MAKE_BIT_MASK_32 (4)    /**<scan condition to report current frequency*/
#define SB_REASON_CHANNELS              MAKE_BIT_MASK_32 (5)    /**<scan condition to report found channel number without progress update */
#define SB_REASON_CHANNEL_UPDATE_WITHOUT_PROGRESS    MAKE_BIT_MASK_32 (6)    /**<for dvbs scan , same progress may contain different TPs which leads to channel update while progress stay still*/
#define SB_REASON_LCN_CONFLICTS         MAKE_BIT_MASK_32 (7)    /**<scan condition to report found conflict LCNs */

#define SB_VNET_ALL                     ((UINT32)   (0x00000001))    /**<svl record bit mask        */
#define SB_VNET_ACTIVE                  MAKE_BIT_MASK_32 (1)    /**<svl record bit mask        */
#define SB_VNET_EPG                     MAKE_BIT_MASK_32 (2)    /**<svl record bit mask        */
#define SB_VNET_VISIBLE                 MAKE_BIT_MASK_32 (3)    /**<svl record bit mask        */
#define SB_VNET_FAVORITE1               MAKE_BIT_MASK_32 (4)    /**<svl record bit mask        */
#define SB_VNET_FAVORITE2               MAKE_BIT_MASK_32 (5)    /**<svl record bit mask        */
#define SB_VNET_FAVORITE3               MAKE_BIT_MASK_32 (6)    /**<svl record bit mask        */
#define SB_VNET_FAVORITE4               MAKE_BIT_MASK_32 (7)    /**<svl record bit mask        */
#define SB_VNET_BLOCKED                 MAKE_BIT_MASK_32 (8)    /**<svl record bit mask        */
#define SB_VNET_OOB_LIST                MAKE_BIT_MASK_32 (9)    /**<svl record bit mask        */
#define SB_VNET_INB_LIST                MAKE_BIT_MASK_32 (10)    /**<svl record bit mask        */
#define SB_VNET_SCRAMBLED               MAKE_BIT_MASK_32 (11)    /**<svl record bit mask        */
#define SB_VNET_BACKUP1                 MAKE_BIT_MASK_32 (12)    /**<svl record bit mask        */
#define SB_VNET_BACKUP2                 MAKE_BIT_MASK_32 (13)    /**<svl record bit mask        */
#define SB_VNET_BACKUP3                 MAKE_BIT_MASK_32 (14)    /**<svl record bit mask        */
#define SB_VNET_FAKE                    MAKE_BIT_MASK_32 (15)    /**<svl record bit mask        */
#define SB_VNET_USER_TMP_UNLOCK         MAKE_BIT_MASK_32 (16)    /**<svl record bit mask        */
#define SB_VNET_CH_NAME_EDITED          MAKE_BIT_MASK_32 (17)    /**<svl record bit mask        */
#define SB_VNET_LCN_APPLIED             MAKE_BIT_MASK_32 (18)    /**<svl record bit mask        */
#define SB_VNET_USE_DECODER             MAKE_BIT_MASK_32 (19)    /**<svl record bit mask        */
#define SB_VNET_ACTIVE_EPG_EDITED       MAKE_BIT_MASK_32 (20)    /**<deprecated, please use  SB_VOPT_CH_NUM/NAME_EDITED        */
#define SB_VNET_FREQ_EDITED             MAKE_BIT_MASK_32 (21)    /**<svl record bit mask        */
#define SB_VNET_REMOVAL                 MAKE_BIT_MASK_32 (22)    /**<svl record bit mask        */
#define SB_VNET_REMOVAL_TO_CONFIRM      MAKE_BIT_MASK_32 (23)    /**< svl record bit mask       */

#define SB_VNET_USER_INDEX_BGN          ((INT32) 24)    /**< svl record bit mask       */
#define SB_VNET_USER_INDEX_END          ((INT32) 31)    /**<svl record bit mask        */

#define SB_MAKE_VNET_USER(x)            MAKE_BIT_MASK_32 ((x))    /**< svl record bit mask       */

/* Redefine for DVB */
#define SB_VNET_NUMERIC_SELECTABLE      MAKE_BIT_MASK_32 (9)    /**< svl record bit mask       */
#define SB_VNET_RADIO_SERVICE           MAKE_BIT_MASK_32 (10)    /**<svl record bit mask        */
#define SB_VNET_ANALOG_SERVICE          MAKE_BIT_MASK_32 (12)    /**< svl record bit mask       */
#define SB_VNET_TV_SERVICE              MAKE_BIT_MASK_32 (13)    /**<svl record bit mask        */
#define SB_VNET_USE_DECODER_2           MAKE_BIT_MASK_32 (14)    /**<svl record bit mask        */
#define SB_VNET_NEW_SVC_BY_BGM_SCAN     MAKE_BIT_MASK_32 (18)    /**<svl record bit mask        */
#define SB_VNET_BRDCST_SVC              MAKE_BIT_MASK_32 (20)    /**<indicate service is installed from broadcast, not CICAM        */


/* Redefine for PAL/SECAM */
#define SB_VNET_ACI_APPIED              MAKE_BIT_MASK_32 (13)    /**<svl record bit mask        */
#define SB_VNET_DEFAULT_CH              MAKE_BIT_MASK_32 (14)    /**<svl record bit mask        */


/* SVL record options mask */
#define SB_VOPT_ALL                     ((UINT32)   (0x00000001))    /**<svl record bit mask        */
#define SB_RECORD_NOT_SAVE_CH_NUM       MAKE_BIT_MASK_32 (1)    /**<svl record bit mask        */
#define SB_VOPT_NOT_SAVE_CH_NUM         MAKE_BIT_MASK_32 (1)    /**<svl record bit mask        */
   
#define SB_VOPT_USER_TMP_UNLOCK         MAKE_BIT_MASK_32 (2)    /**<svl record bit mask        */
#define SB_VOPT_CH_NAME_EDITED          MAKE_BIT_MASK_32 (3)    /**<svl record bit mask        */
#define SB_VOPT_FREQ_EDITED             MAKE_BIT_MASK_32 (4)    /**<svl record bit mask        */
#define SB_VOPT_CH_NAME_EMPTY_EDITED    MAKE_BIT_MASK_32 (5)


#define SB_VOPT_USER_DEFINED            (MAKE_BIT_MASK_32 (31)| \
                                         MAKE_BIT_MASK_32 (30)| \
                                         MAKE_BIT_MASK_32 (29)| \
                                         MAKE_BIT_MASK_32 (28)| \
                                         MAKE_BIT_MASK_32 (27)| \
                                         MAKE_BIT_MASK_32 (26)| \
                                         MAKE_BIT_MASK_32 (25)| \
                                         MAKE_BIT_MASK_32 (24))    /**<svl record bit mask        */

/* DVB */
#define SB_VOPT_LCN_APPLIED             MAKE_BIT_MASK_32 (5)    /**<svl record bit mask        */
#define SB_VOPT_MANUAL_OBTAINED         MAKE_BIT_MASK_32 (6)    /**< svl record bit mask       */
#define SB_VOPT_HD_SIMULCAST            MAKE_BIT_MASK_32 (7)    /**<svl record bit mask        */
#define SB_VOPT_SDT_AVAILABLE           MAKE_BIT_MASK_32 (9)    /**<svl record bit mask        */
#define SB_VOPT_CH_NUM_EDITED           MAKE_BIT_MASK_32 (10)    /**<svl record bit mask        */
#define SB_VOPT_PORTUGAL_HD_SIMULCAST   MAKE_BIT_MASK_32 (11)    /**<svl record bit mask        */
#define SB_VOPT_CURRENT_COUNTRY         MAKE_BIT_MASK_32 (12)    /**<svl record bit mask        */
#define SB_VOPT_DELETED_BY_USER         MAKE_BIT_MASK_32 (13)    /**<svl record bit mask        */
#define SB_VOPT_NVOD_REF                MAKE_BIT_MASK_32 (14)    /**<svl record bit mask        */
#define SB_VOPT_NVOD_TS                 MAKE_BIT_MASK_32 (15)    /**<svl record bit mask        */
#define SB_VOPT_SVC_REMOVE_NORDIG_SIMULICAST   MAKE_BIT_MASK_32 (16)    /**<svl record bit mask        */
#define SB_VOPT_HD_SIMULICAST_LCN_APPLIED   MAKE_BIT_MASK_32 (17)    /**<svl record bit mask        */
#define SB_VOPT_REMOVED_BY_HD_SIMULICAST MAKE_BIT_MASK_32 (18)    /**<svl record bit mask        */
#define SB_VOPT_LCN_REORDERING          MAKE_BIT_MASK_32 (19)
#define SB_VOPT_3D_SERVICE              MAKE_BIT_MASK_32 (20)
#define SB_VOPT_NZL_REMOVE_DUPLICATE_LCN_CH          MAKE_BIT_MASK_32 (21)
#define SB_VOPT_ITA_LCN_CONFLICT_CH             MAKE_BIT_MASK_32 (21)
#define SB_VOPT_STRM_REQ_BLOCK          MAKE_BIT_MASK_32 (22)   /**<svl record bit mask        */
#define SB_VOPT_CAM_SVC                 SB_VOPT_FREQ_EDITED
#define SB_VOPT_INTERACT_RESTRICT       MAKE_BIT_MASK_32 (24)   /**<svl record bit mask        */

/* PAL/SECAM */
#define SB_VOPT_ACI_APPIED              MAKE_BIT_MASK_32 (7)    /**<svl record bit mask        */

#define SB_VOPT_SIGNAL_BOOSTER          MAKE_BIT_MASK_32 (8)    /**<svl record bit mask        */

/*Redefine for ISDB-T*/
#define SB_VOPT_HAS_SMD                 MAKE_BIT_MASK_32 (13)    /**<svl record bit mask        */
#define SB_VOPT_IS_BRDCSTING            MAKE_BIT_MASK_32 (14)    /**<svl record bit mask        */
#define SB_VOPT_MODIFIED_VISIBLE        MAKE_BIT_MASK_32 (17)
#define SB_VOPT_IS_ONE_SEGMENT          MAKE_BIT_MASK_32 (18)


/* SVL record options mask */
#define SB_VOPT2_TUNER_LNA              ((UINT32)   (0x00000001))    /** <svl record bit mask */
#define SB_VOPT2_CI_PMT_FROM_CAM         MAKE_BIT_MASK_32 (1)
#define SB_VOPT2_CI_EVENT_DES_FROM_CAM   MAKE_BIT_MASK_32 (2)
#define SB_VOPT2_CI_COMP_DES_FROM_CAM    MAKE_BIT_MASK_32 (3)
#define SB_VOPT2_CI_RATING_DES_FROM_CAM  MAKE_BIT_MASK_32 (4)
#define SB_VOPT2_CI_CONTENT_DES_FROM_CAM MAKE_BIT_MASK_32 (5)
#define SB_VOPT2_SDT_ACTUAL_AVAILABLE    MAKE_BIT_MASK_32 (6)
#define SB_VOPT2_SVC_HIDDEN_BY_USER      MAKE_BIT_MASK_32 (7) /** The svl record is hidden by the user, like UK <YouView>. */
#define SB_VOPT2_NEW_SVC_BY_MONITOR      MAKE_BIT_MASK_32 (8)
#define SB_VOPT2_SVC_EIT_SCHEDULE        MAKE_BIT_MASK_32 (9)


/* SVL component mask */
#define SB_COMP_MASK_NONE                           ((UINT32)   (0x00000001))
#define SB_COMP_MASK_SD                             MAKE_BIT_MASK_32 (1)
#define SB_COMP_MASK_HD                             MAKE_BIT_MASK_32 (2)
#define SB_COMP_MASK_MULTICHANNEL                   MAKE_BIT_MASK_32 (3)
#define SB_COMP_MASK_VISUALLY_IMPAIRED              MAKE_BIT_MASK_32 (4)
#define SB_COMP_MASK_AUDIO_HARD_OF_HEARING          MAKE_BIT_MASK_32 (5)
#define SB_COMP_MASK_SUBTITLE                       MAKE_BIT_MASK_32 (6)
#define SB_COMP_MASK_SUBTITLE_HARD_OF_HEARING       MAKE_BIT_MASK_32 (7)
#define SB_COMP_MASK_4K                             MAKE_BIT_MASK_32 (8)
#define SB_COMP_MASK_NORMAL_AUDIO                   MAKE_BIT_MASK_32 (9)


/*ISDB-T*/
/*TSL record operation masks*/
#define SB_TSL_MASK_CONFIG_DATA_SVC                 MAKE_BIT_MASK_32 (1)/*the user configure in scan result*/

/*DVBS*/
/*TSL record operation masks*/
#define SB_TSL_MASK_ADDED_BY_USER                   MAKE_BIT_MASK_32 (1)/*the ts record is added by user*/
#define SB_TSL_MASK_ADDED_BY_SCAN                   MAKE_BIT_MASK_32 (2)/*the ts record is added by auto scan */
#define SB_TSL_MASK_EDITED                          MAKE_BIT_MASK_32 (3)/*the ts record is edited by user */
#define SB_TSL_MASK_CONFIG_FREE_SVC_ONLY            MAKE_BIT_MASK_32 (4)/*the user configure in scan result*/
#define SB_TSL_MASK_CONFIG_TV_SVC_ONLY              MAKE_BIT_MASK_32 (5)/*the user configure in scan result */
#define SB_TSL_MASK_CONFIG_RADIO_SVC_ONLY           MAKE_BIT_MASK_32 (6)/*the user configure in scan result */
#define SB_TSL_MASK_CONFIG_TV_RADIO_SVC_ONLY        MAKE_BIT_MASK_32 (7)/*the user configure in scan result */
#define SB_TSL_MASK_FOUND_IN_SAT_SEARCH             MAKE_BIT_MASK_32 (8)/*the ts record is found in sat search*/
#define SB_TSL_MASK_FOUND_IN_AUTO_SEARCH            MAKE_BIT_MASK_32 (9)/*the ts record is found in auto search*/
#define SB_TSL_MASK_FOUND_IN_NETWORK_SEARCH         MAKE_BIT_MASK_32 (10)/*the ts record is found in network search*/
#define SB_TSL_MASK_HOME_TP                         MAKE_BIT_MASK_32 (11)/*the ts record is home transponder*/
#define SB_TSL_MASK_TUNED_MUX                       MAKE_BIT_MASK_32 (12)/*the ts record is searched by the mux really be tuned */
#define SB_TSL_MASK_LCN_SEARCH_DISABLE              MAKE_BIT_MASK_32 (13)/*the user configure lcn search disable */
#define SB_TSL_MASK_CONFIG_SCRAMBLE_ONLY            MAKE_BIT_MASK_32 (14)/*the user configure in scan result */

#define SB_TSL_MASK_CUSTOM_DEFINED_4                MAKE_BIT_MASK_32 (28)/*Customer defined */
#define SB_TSL_MASK_CUSTOM_DEFINED_3                MAKE_BIT_MASK_32 (29)/*Customer defined */
#define SB_TSL_MASK_CUSTOM_DEFINED_2                MAKE_BIT_MASK_32 (30)/*Customer defined */
#define SB_TSL_MASK_CUSTOM_DEFINED_1                MAKE_BIT_MASK_32 (31)/*Customer defined */


/*Network mask defination*/
#define SB_NWL_MASK_FAV_CANDIDATE       MAKE_BIT_MASK_32 (1)
#define SB_NWL_MASK_FAV                 MAKE_BIT_MASK_32 (2)
#define SB_NWL_MASK_REMOVAL             MAKE_BIT_MASK_32 (3)
#define SB_NWL_MASK_CI                  MAKE_BIT_MASK_32 (4)

/*Satellite record operation masks*/
#define SB_SATL_MASK_ADDED                      SATL_MASK_ADDED/*the satellite record is added by user*/
#define SB_SATL_MASK_EDITED                     SATL_MASK_EDITED/*the satellite record is edited by user */
#define SB_SATL_MASK_HIDDEN                     SATL_MASK_HIDDEN/*the satellite record is hidden by user */
#define SB_SATL_MASK_LNB                        SATL_MASK_LNB/*the satellite record is used for LNB style UI*/
#define SB_SATL_MASK_LNB_SIGNAL_DETECTED        SATL_MASK_LNB_SIGNAL_DETECTED/*signal detected in this LNB*/
#define SB_SATL_MASK_CONFIG_FREE_SVC_ONLY       SATL_MASK_CONFIG_FREE_SVC_ONLY/*the user configure in scan result*/
#define SB_SATL_MASK_CONFIG_TV_SVC_ONLY         SATL_MASK_CONFIG_TV_SVC_ONLY/*the user configure in scan result */
#define SB_SATL_MASK_CONFIG_RADIO_SVC_ONLY      SATL_MASK_CONFIG_RADIO_SVC_ONLY/*the user configure in scan result */
#define SB_SATL_MASK_CONFIG_TV_RADIO_SVC_ONLY   SATL_MASK_CONFIG_TV_RADIO_SVL_ONLY/*the user configure in scan result */
#define SB_SATL_MASK_CONFIG_SCAN_TYPE_FULL      SATL_MASK_CONFIG_SCAN_TYPE_FULL/*the scan type saved for BGM */
#define SB_SATL_MASK_CONFIG_SCAN_TYPE_NETWORK   SATL_MASK_CONFIG_SCAN_TYPE_NETWORK/*the scan type saved for BGM */


#define SB_CH_IDX_FOR_ANALOG                    0x00
#define SB_CH_IDX_FOR_TV                        0x01
#define SB_CH_IDX_FOR_RADIO                     0x02
#define SB_CH_IDX_FOR_DATA                      0x03
#define SB_CH_IDX_FOR_ALL                       0x04



#define SB_MAX_ENG_NAME_LEN             16

/* SVL Builder States */
/*------------------------------------------------------------------*/
/*! @enum SB_COND_T
 *  @brief Svl builder engine condition
 *  @code
 *  typedef enum
 *  {
 *      SB_COND_AVAIL = 0,
 *      SB_COND_UNCONFIG,
 *      SB_COND_INITING,
 *      SB_COND_BUILDING,
 *      SB_COND_UPDATE,
 *      SB_COND_CLOSED 
 *  } SB_COND_T;
 *  @endcode
 *  @li@c  SB_COND_AVAIL                                     - Current state is available for scan
 *  @li@c  SB_COND_UNCONFIG                                  - Current state is never scaned
 *  @li@c  SB_COND_INITING                                   - Current state is initializing, can't scan
 *  @li@c  SB_COND_BUILDING                                  - Current state is scaning
 *  @li@c  SB_COND_UPDATE                                    - The update is notified by table manager
 *  @li@c  SB_COND_CLOSED                                    - Current state is closed, can't scan
 */
/*------------------------------------------------------------------*/
typedef enum
{
    SB_COND_AVAIL = 0,
    SB_COND_UNCONFIG,
    SB_COND_INITING,
    SB_COND_BUILDING,
    SB_COND_UPDATE,
    SB_COND_CLOSED /* used to notify the client */
} SB_COND_T;


/* SVL Builder notification function type of state changes */
/*------------------------------------------------------------------*/
/*! @brief x_sb_nfy_fct callback function is call back when scan engine condition changed and registered on SB handle creating.
 *  @param [in] the notified engine handle
 *  @param [in] the condtion of notified engine
 *  @param [in] the tag value passed by AP
 *  @return      NA   
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_nfy_fct) (
      HANDLE_T          h_builder,
      SB_COND_T         e_builder_cond,
      VOID*             pv_nfy_tag);


/* SVL Builder notification function type of building progress/status */
/*------------------------------------------------------------------*/
/*! @brief x_sb_scan_nfy_fct is call back when scan engine notify scan progress. 
 *  @param [in] the notified sb engine    
 *  @param [in] the notified reason
 *  @param [in] the notified data which include the channels scaned
 *  @param [in] the tag value passed by AP
 *  @return      NA
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_scan_nfy_fct) (
      HANDLE_T          h_builder,
      UINT32            ui4_nfy_reason,
      INT32             i4_nfy_data,
      VOID*             pv_nfy_tag);

#define SB_TSL_REC_ID_OFFSET_MASK           ((UINT16) 0xf000)    /**< internal use       */
#define SB_TSL_GET_PTC_FROM_RECID(x)        (((UINT16) (x)) & (~SB_TSL_REC_ID_OFFSET_MASK))    /**<internal use        */

#define SB_ANA_TSL_REC_ID_OFFSET            (((UINT16) 0x8000) & SB_TSL_REC_ID_OFFSET_MASK)    /**<internal use        */
#define SB_MAKE_ANA_TSL_REC_ID(x)           ((UINT16) (x) | SB_ANA_TSL_REC_ID_OFFSET)    /**< internal use       */

#define SB_DIG_TSL_REC_ID_OFFSET            (((UINT16) 0) & SB_TSL_REC_ID_OFFSET_MASK)    /**<internal use        */
#define SB_MAKE_DIG_TSL_REC_ID(x)           ((UINT16) (x) | SB_DIG_TSL_REC_ID_OFFSET)    /**< internal use       */

#define SB_AUX_DIG_TSL_REC_ID_OFFSET        (((UINT16) 0x4000) & SB_TSL_REC_ID_OFFSET_MASK)    /**<internal use        */
#define SB_MAKE_AUX_DIG_TSL_REC_ID(x)       ((UINT16) (x) | SB_AUX_DIG_TSL_REC_ID_OFFSET)    /**< internal use       */

/* Macros for DVB channel number */
/* These macros are designed to conform to the ATSC ones so that using the ATSC macros can
   also retrieve the "major" channel numbers */
#define SB_DVB_CH_NUM_FLAG              ((UINT32) 0x80)    /**<  to indicate it's one-part channel number     */

#define SB_DVB_CH_TYPE_RANGE                     3         /*The last 3bit is used for service type*/ 
#define SB_DVB_MAX_CHANNEL_INDEX_WITH_CH_TYPE  ((UINT8) 0x7)


#define SB_DVB_MAX_CHANNEL_INDEX        ((UINT8) 0xF)    /**<  bit masks for channel index */

#define SB_DVB_GEN_CHANNEL_ID(major, index) \
    ((((UINT32) (major) & 0x3FFF) << 18) | \
     ((index & SB_DVB_MAX_CHANNEL_INDEX)) | \
     SB_DVB_CH_NUM_FLAG)    /**< to generate channel id with major number and index */

#define SB_DVB_GEN_CHANNEL_ID_WITH_LCN(major, lcn, index) \
    ((((UINT32) (major) & 0x3FFF) << 18) | \
     (((UINT32) (lcn) & 0x3FF)  <<  8) | \
     ((index & SB_DVB_MAX_CHANNEL_INDEX)) | \
     SB_DVB_CH_NUM_FLAG                    | \
     (((UINT32)(lcn) & 0xC00)>>5))    /**< to generate channel id with major number, lcn and index */
                     
#define SB_DVB_GET_CHANNEL_NUM(channel_id) \
    ((UINT16)(((UINT32)(channel_id) >> 18) & 0x3FFF))    /**< to get the major channel number with channel_id       */
                    
#define SB_DVB_GET_LCN(channel_id) \
    ((UINT16)(((UINT32)(channel_id) >> 8) & 0x3ff)| \
    (((UINT32)channel_id & 0x60)<< 5))    /**< to get LCN with channel_id       */


#define SB_DVB_GET_CHANNEL_INDEX(channel_id) \
    ((UINT8) (((UINT32)(channel_id)) & SB_DVB_MAX_CHANNEL_INDEX))    /**< to get channel index with channel_id       */



/*DVB MF*/
/*mf type, for dvb-c,s multi-feed function, the value in ui1_mf_type*/
#define SB_DVB_MF_SERVICE_TYPE_NONE     ((UINT8) 0)
#define SB_DVB_MF_SERVICE_TYPE_PORTAL   ((UINT8) 1)
#define SB_DVB_MF_SERVICE_TYPE_SUBFEED  ((UINT8) 2)

/*	for multi language support,	move these from sb_dtmb_eng	*/
#define MAX_MULTI_LANG_NUM          3       /* ENG CHN CHI */
#define MAX_MULTI_LANG_CH_NUM       800     

typedef struct {
    CHAR                s_channel_name[MAX_PROG_NAME_LEN + 1];
    ISO_3166_COUNT_T    t_country_code;
} ENG_MULTI_LANG_T;


/* Get function keys */
/*------------------------------------------------------------------*/
/*! @enum
 *  @brief the key type for sb_set or sb_get
 *  @code
 *  typedef enum
 *  {
 *      SB_KEY_TYPE_UNKNOWN = 0,
 *      SB_KEY_TYPE_FREQ_RANGE,
 *      SB_KEY_TYPE_COUNTRY_CODE,
 *      SB_KEY_TYPE_FLTR_ACI_INFO,
 *      SB_KEY_TYPE_ACI_SLCT_CITY,
 *      SB_KEY_TYPE_ACI_CANCEL,
 *      SB_KEY_TYPE_TUNER_FREQ_RANGE,   
 *      SB_KEY_TYPE_NW_CHANGE_NFY,      
 *      SB_KEY_TYPE_AUTO_UPDATE_OFF,        
 *      SB_KEY_TYPE_SET_TVD_SNK_NAME,          
 *      SB_KEY_TYPE_RESET_TVD_SNK_NAME,     
 *      SB_KEY_TYPE_INSERT_FAKE_SVL_TSL,	
 *      SB_KEY_TYPE_INSERT_DEFAULT_SVL_TSL, 
 *      SB_KEY_TYPE_FAKE_SETTING,           
 *      SB_KEY_TYPE_TOTAL_NUMBER,        
 *      SB_KEY_TYPE_CHANNEL_DATA_INTEGRITY,
 *      SB_KEY_TYPE_DISABLE_NW_CHANGE_NFY,  
 *  } SB_KEY_TYPE_T;
 *  @endcode
 *  @li@c  SB_KEY_TYPE_UNKNOWN                               - internal use
 *  @li@c  SB_KEY_TYPE_FREQ_RANGE                            - internal use
 *  @li@c  SB_KEY_TYPE_COUNTRY_CODE                          - internal use
 *  @li@c  SB_KEY_TYPE_FLTR_ACI_INFO                         - internal use
 *  @li@c  SB_KEY_TYPE_ACI_SLCT_CITY                         - internal use
 *  @li@c  SB_KEY_TYPE_ACI_CANCEL                            - internal use
 *  @li@c  SB_KEY_TYPE_TUNER_FREQ_RANGE                      - Output: SB_FREQ_RANGE_T
 *  @li@c  SB_KEY_TYPE_NW_CHANGE_NFY                         - Input: SB_DVB_NW_CHANGE_NFY_T
 *  @li@c  SB_KEY_TYPE_AUTO_UPDATE_OFF                       - BOOL: enable/disable auto update
 *  @li@c  SB_KEY_TYPE_SET_TVD_SNK_NAME                      - Input: Set TV Decoder Sink component Name only used on NTSC
 *  @li@c  SB_KEY_TYPE_RESET_TVD_SNK_NAME                    - Input: Reset TV Decoder Sink component Name only used on NTSC
 *  @li@c  SB_KEY_TYPE_INSERT_FAKE_SVL_TSL                   - Input: Insert SVL TSL records only used on NTSC
 *  @li@c  SB_KEY_TYPE_INSERT_DEFAULT_SVL_TSL                - Input: Insert SVL TSL records for default setting
 *  @li@c  SB_KEY_TYPE_FAKE_SETTING                          - BOOL:enable/disable, Set channel list to fake channel or default channel,default is on
 *  @li@c  SB_KEY_TYPE_CHANNEL_DATA_INTEGRITY                - check data os channels whether is integrate
 *  @li@c  SB_KEY_TYPE_DISABLE_NW_CHANGE_NFY                 - BOOL: true:Disable, false:Enable, disable nw change notification
 *  @li@c  SB_KEY_TYPE_TOTAL_NUMBER                          - total number of key type
 */
/*------------------------------------------------------------------*/
typedef enum
{
    SB_KEY_TYPE_UNKNOWN = 0,
    SB_KEY_TYPE_FREQ_RANGE,
    SB_KEY_TYPE_COUNTRY_CODE,
    SB_KEY_TYPE_SVC_ID_RULE,
    SB_KEY_TYPE_FLTR_ACI_INFO,
    SB_KEY_TYPE_ACI_SLCT_CITY,
    SB_KEY_TYPE_ACI_CANCEL,
    SB_KEY_TYPE_USER_OPERATION_FINISH,
    SB_KEY_TYPE_TUNER_FREQ_RANGE,   /* Output: SB_FREQ_RANGE_T */
    SB_KEY_TYPE_NW_CHANGE_NFY,      /* Input: SB_DVB_NW_CHANGE_NFY_T */
    SB_KEY_TYPE_AUTO_UPDATE_OFF,        /*BOOL: enable/disable*/
    SB_KEY_TYPE_SVC_NAME_AFTER_AUTO_UPDATE_OFF,
    SB_KEY_TYPE_SET_TVD_SNK_NAME,          /*Input: Set TV Decoder Sink component Name only used on NTSC*/
    SB_KEY_TYPE_RESET_TVD_SNK_NAME,     /*Input: Reset TV Decoder Sink component Name only used on NTSC*/
    SB_KEY_TYPE_INSERT_FAKE_SVL_TSL,	/*Input: Insert SVL TSL records only used on NTSC */
    SB_KEY_TYPE_INSERT_DEFAULT_SVL_TSL, /*Input: Insert SVL TSL records for default setting */
    SB_KEY_TYPE_FAKE_SETTING,           /*BOOL:enable/disable, Set channel list to fake channel or default channel,default is on*/
    SB_KEY_TYPE_CHANNEL_DATA_INTEGRITY,  /* check data os channels whether is integrate */
    SB_KEY_TYPE_DISABLE_NW_CHANGE_NFY,  /* BOOL: true:Disable, false:Enable, disable nw change notification*/
    SB_KEY_TYPE_FAV_NET,
    SB_KEY_TYPE_TUNER_NOTIFIED_FREQ,
    SB_KEY_TYPE_LCN_CONFLICT_NUM_GROUPS, /*DGTVi: output: pv_key_data: pointer contains number of conflict group (UINT8)*/
    /*DGTVi: Input: pv_key_info: group number(UINT8), 
          output: pv_key_data: pointers contains structure SB_DVB_LCN_CONFLICT_GROUP_T*/
    SB_KEY_TYPE_LCN_CONFLICT_NUM_CHANNELS_IN_GROUP, 
    /*DGTVi: Input: pv_key_info: group number(UINT8), 
          output: pv_key_data: pointers contains structure SB_DVB_LCN_CONFLICT_CHANNEL_T, should fill ui1_index inside the structure*/
    SB_KEY_TYPE_LCN_CONFLICT_CHANNEL_IN_GROUP,
    /*DGTVi: Input: pv_key_info: group number(UINT8), 
          output: pv_key_data: pointers contains structure SB_DVB_LCN_CONFLICT_CHANNEL_T, should fill ui1_index inside the structure*/    
    SB_KEY_TYPE_LCN_CONFLICT_PREFERRED_CHANNEL_IN_GROUP,
    SB_KEY_TYPE_LCN_CONFLICT_USER_OP_END,
    SB_KEY_TYPE_CH_LST_LCNV2_CONFLICT_NUM,
    SB_KEY_TYPE_CH_LST_LCNV2_CONFLICT_ENTRY_BY_IDX,
    SB_KEY_TYPE_CH_LST_LCNV2_CONFLICT_PREFERRED,
    /*to get number of regions:
        Input: pv_key_info: SB_DVB_REGION_CODE_T
        if you want to query primary region, give ui1_level = 0;
        if you want to query second region, give ui1_level = 1, and aui1_code = {pri, 0, 0}
        if you want to query tertiary region, give ui1_level =2, and aui1_code = {pri, sec, 0}
        output: pv_key_data: pointer contains number of regions (UINT16)
        (*pz_key_data_len) must be bigger than sizeof (UINT16)*/
    SB_KEY_TYPE_NUM_REGIONS,
      /* use this key to get regions/set region 
        get:
        Input: pv_key_info: SB_DVB_REGION_CODE_T
        if you want to query primary region, give ui1_level = 0;
        if you want to query second region, give ui1_level = 1, and aui1_code = {pri, 0, 0}
        if you want to query tertiary region, give ui1_level =2, and aui1_code = {pri, sec, 0}
        output: pv_key_data: start pointer contains array of regions (SB_DVB_REGION_T)
        in/out: (*pz_key_data_len) must be bigger than sizeof(SB_DVB_REGION_T)*num_of_regions
        set:
        Input: 
        pv_key_data: pointer contains the region code (SB_DVB_REGION_CODE_T)
        z_key_data_len must be bigger than sizeof(SB_DVB_REGION_CODE_T)
        if the target region is primary region, give ui1_level = 1, and aui1_code={pri, 0, 0};
        if the target region is second region, give ui1_level =2, and aui1_code = {pri, sec, 0}
        if the target region is tertiary region, give ui1_level =3, and aui1_code = {pri, sec, ter}*/
    
    SB_KEY_TYPE_REGION,
    SB_KEY_TYPE_NUM_ALL_REGIONS,
    SB_KEY_TYPE_ALL_REGION,
    SB_KEY_TYPE_QUERY_PREFERED_REGION,
     /* to set a preferred GUI language
        Input:
        pv_key_data: pointer contains the language code (ISO_639_LANG_T) 
        z_key_data_len must be equal sizeof(ISO_639_LANG_T)*/
    SB_KEY_TYPE_GUI_LANG,
    /* to set MF temp svl id (only for DVB-C)
        example:
        c_sb_set(h_builder, SB_KEY_TYPE_MF_TEMP_SVL_ID, NULL, sizeof(UINT16), ui2_tmp_svl_id)*/
    SB_KEY_TYPE_MF_TEMP_SVL_ID,
    /*to get last successful tuning parameters (only for DVB-C)
        example:
        z_len = sizeof(SB_DVBC_SCAN_DATA_T);
        c_sb_get(h_builder, SB_KEY_TYPE_LAST_TUNING_PARA, NULL, &z_len, &t_dvbc_scan_data)*/
    SB_KEY_TYPE_LAST_TUNING_PARA,
    SB_KEY_TYPE_CALC_FREQ_OFFSET_CB,
    /* to set operator name (only for DVB-C)
        example:
        c_sb_set(h_builder, SB_KEY_TYPE_OPERATOR_NAME, NULL, OPERATOR_NAME_COUNT_LEN, &t_operator_name)*/    
    SB_KEY_TYPE_OPERATOR_NAME,
    /*to get number of scanned channels (only for DVB-C)
        example:
        z_len = sizeof(SB_DVBC_NUM_CH);
        c_sb_get(h_builder, SB_KEY_TYPE_NUM_SCANNED_CH, NULL, &z_len, &t_dvbc_num_ch)*/    
    SB_KEY_TYPE_NUM_SCANNED_CH,
    SB_KEY_TYPE_CP_OPERATOR_TUNE,   /* delivery system descriptors */
    SB_KEY_TYPE_CP_OPERATOR_TUNE_COMPLETE,
    SB_KEY_TYPE_CP_OPERATOR_TUNE_STATUS,
    SB_KEY_TYPE_CP_OPERATOR_NIT,    /* Network Information Table (NIT) sections */
    SB_KEY_TYPE_CP_USER_OP_END,
    SB_KEY_TYPE_CP_CAM_REPLACED,
    SB_KEY_TYPE_CP_SVC_MAX_CH_NUM,
    SB_KEY_TYPE_CP_OPERATOR_OSDT,
    /*to get the flag for updated msg during BGM scan in DVBT Italy.*/
    SB_KEY_TYPE_FLAG_UPDATED_MSG,
    SB_KEY_TYPE_FLAG_LCN_OBTAINED,

    SB_KEY_TYPE_MAIN_FREQ,        /* Tee mainfr full */
    SB_KEY_TYPE_MAIN_FREQ_NFY,     /* Tee mainfr full */
   
	SB_KEY_TYPE_MONITOR,            /* Tee monitor NIT version  */

    SB_KEY_TYPE_SVC_LIST_FULL_NFY,
    SB_KEY_TYPE_BAT_INFO_SELECTED,
    SB_KEY_TYPE_IS_M7_OPERATOR,
    SB_KEY_TYPE_SAVE_ORBIT_INFO,
    SB_KEY_TYPE_CONVERT_FREQ,
    
    SB_KEY_TYPE_LOL_REC_INFO,       /* for LOL rec info get/set */
    SB_KEY_TYPE_TUNER_FINE_TUNE_DELTA_FREQ,

    SB_KEY_TYPE_INSTALL_SPECIFIED_NVOD,
    SB_KEY_TYPE_EXTRA_SVC_TYPE,
    SB_KEY_TYPE_BU_INFO,

    /* to set a guide preferred language for multi-language select.
       Input:
       pv_key_data: pointer contains the language code (ISO_639_LANG_T) 
       z_key_data_len must be equal sizeof(ISO_639_LANG_T)*/
    SB_KEY_TYPE_GUIDE_LANG,

    SB_KEY_TYPE_MONITOR_NFY,
    SB_KEY_TYPE_MON_TRD_NFY,      /* target region notify in monitor state */
    SB_KEY_TYPE_NEW_SVL_REC_NFY,  /*new svl record notify*/
    SB_KEY_TYPE_TKGS_VERSION, /* Set or get TKGS version */

    SB_KEY_TYPE_TKGS_MODE, /* Set or get TKGS mode */

    SB_KEY_TPYE_REGISTER_GET_INFO_NFY, /* Register callback function to get scan information */

    SB_KEY_TYPE_TKGS_USR_MESSAGE, /* Set or get TKGS user message */

    SB_KEY_TYPE_SCANNING_NFY, /*Register callback function to notify AP scanning status info */
    SB_KEY_TYPE_STORE_LCN_CONFLICT,
    SB_KEY_TYPE_LOAD_LCN_CONFLICT,
    SB_KEY_TYPE_ORDERING_TYPE,       /*For Italy mediaset requirement.*/
    
    SB_KEY_TYPE_TIME_ZONE, /* Set or get time zone */
	
    SB_KEY_TYPE_POST_CODE, /* Set or get post code for Freesat of UK */
    SB_KEY_TYPE_NUM_ALL_REGIONS_EX,
    SB_KEY_TYPE_ALL_REGION_EX,

    SB_KEY_TYPE_TOTAL_NUMBER        /* total number of key types */
} SB_KEY_TYPE_T;


/*------------------------------------------------------------------*/
/*! @enum
 *  @brief the sub key type for SB_KEY_TYPE_LOL_REC_INFO
 *  @code
 *  typedef enum
 *  {
 *      SB_SUB_KEY_TYPE_LOL_FOR_LCN_V2 = 0,
 *      SB_SUB_KEY_TYPE_LOL_TOTAL_NUMBER
 *  } SB_KEY_TYPE_T;
 *  @endcode
 *  @li@c  SB_SUB_KEY_TYPE_LOL_FOR_LCN_V2      - sub key for LCN V2 info
 *  @li@c  SB_SUB_KEY_TYPE_LOL_TOTAL_NUMBER  - total number of key type
 */
/*------------------------------------------------------------------*/
typedef enum
{
    SB_SUB_KEY_TYPE_LOL_FOR_LCN_V2=0,
    SB_SUB_KEY_TYPE_LOL_TOTAL_NUMBER
}SB_SUB_KEY_TYPE_LOL_REC_INFO_T;
/*------------------------------------------------------------------*/
/*! @struct SB_LOL_REC_INFO_T
 *  @brief define the struct LOL rec info
 *  @code
 *  typedef struct _SB_LOL_REC_INFO_T
 *  {
 *      SB_SUB_KEY_TYPE_LOL_REC_INFO_T  t_sub_key;
 *      LOL_REC_T                       t_lol_rec;
 *  }   SB_LOL_REC_INFO_T;
 *  @endcode
 *  @li@c  t_sub_key                                  - sub key type, reference to SB_SUB_KEY_TYPE_LOL_REC_INFO_T.
 *  @li@c  t_lol_rec                                    - the detail of LOL_REC_T
 */
/*------------------------------------------------------------------*/
typedef struct _SB_LOL_REC_INFO_T
{
    SB_SUB_KEY_TYPE_LOL_REC_INFO_T  t_sub_key;
    LOL_REC_T                       t_lol_rec;
}SB_LOL_REC_INFO_T;

/*------------------------------------------------------------------*/
/*! @struct SB_FREQ_RANGE_T
 *  @brief define the frequency of scan range
 *  @code
 *  typedef struct _SB_FREQ_RANGE_T
 *  {
 *      UINT32  ui4_upper_freq;
 *      UINT32  ui4_lower_freq;
 *  }   SB_FREQ_RANGE_T;
 *  @endcode
 *  @li@c  ui4_upper_freq                                    - scan range upper bound
 *  @li@c  ui4_lower_freq                                    - scan range lower bound
 */
/*------------------------------------------------------------------*/
typedef struct _SB_FREQ_RANGE_T
{
    UINT32  ui4_upper_freq;
    UINT32  ui4_lower_freq;
}   SB_FREQ_RANGE_T;

/* The structure SB_NEW_SVC_NFY_DATA is used as the argument 
   of the new service notification function. */
/*------------------------------------------------------------------*/
/*! @struct SB_NEW_SVC_NFY_DATA
 *  @brief The structure SB_NEW_SVC_NFY_DATA is used as the argument of the new service notification function.
 *  @code
 *  typedef struct _SB_NEW_SVC_NFY_DATA
 *  {
 *      BRDCST_TYPE_T       e_brdcst_type;
 *      SVL_SERVICE_TYPE_T  e_serv_type;
 *      CHAR                ac_name[MAX_PROG_NAME_LEN+1];
 *  } SB_NEW_SVC_NFY_DATA;
 *  @endcode
 *  @li@c  e_brdcst_type                                     - the bradcast type
 *  @li@c  e_serv_type                                       - service type
 *  @li@c  ac_name[MAX_PROG_NAME_LEN+1]                      - channel name
 */
/*------------------------------------------------------------------*/
typedef struct _SB_NEW_SVC_NFY_DATA
{
    BRDCST_TYPE_T       e_brdcst_type;
    SVL_SERVICE_TYPE_T  e_serv_type;
    CHAR                ac_name[MAX_PROG_NAME_LEN+1];
} SB_NEW_SVC_NFY_DATA;

/* The structure SB_NEW_SVL_REC_DATA is used as the argument 
   of the new svl record notification function. */
/*------------------------------------------------------------------*/
/*! @struct SB_NEW_SVL_REC_DATA
 *  @brief The structure SB_NEW_SVL_REC_DATA is used as the argument of the new svl record notification function.
 *  @code
 *  typedef struct _SB_NEW_SVL_REC_DATA
 *  {
 *      BRDCST_TYPE_T       e_brdcst_type;
 *      SVL_REC_T*          pt_svl_rec;
 *  } SB_NEW_SVL_REC_DATA;
 *  @endcode
 *  @li@c  e_brdcst_type                                     - the bradcast type
 *  @li@c  pt_svl_rec                                        - svl record to set bit mask
 */
/*------------------------------------------------------------------*/
typedef struct _SB_NEW_SVL_REC_DATA
{
    BRDCST_TYPE_T       e_brdcst_type;
    SVL_REC_T*          pt_svl_rec; 
} SB_NEW_SVL_REC_DATA;


#define SB_MAX_LCN_V2_LIST_NAME_LEN       24
#define SB_MAX_NUM_CH_LIST_IDS            32

typedef struct _SB_LCNV2_CH_LST_INFO
{    
    CHAR            ac_channel_list_name[SB_MAX_LCN_V2_LIST_NAME_LEN];
    UINT8           ui1_channel_list_id;    
} SB_LCNV2_CH_LST_INFO;


typedef UINT8   DVBT_OPERATOR_T;    /**<  operator name  */

#define DVBT_OPERATOR_UNKNOWN           ((DVBT_OPERATOR_T) 0)
#define DVBT_OPERATOR_BASIC             ((DVBT_OPERATOR_T) 1)
#define DVBT_OPERATOR_MEDIASET          ((DVBT_OPERATOR_T) 2)

typedef enum
{    
    SB_ORDERING_TYPE_UNKNOWN    = DVBT_OPERATOR_UNKNOWN,    
    SB_ORDERING_TYPE_BASIC      = DVBT_OPERATOR_BASIC,
    SB_ORDERING_TYPE_Mediaset   = DVBT_OPERATOR_MEDIASET,
    SB_ORDERING_TYPE_NUM
}   SB_ORDERING_TYPE;

typedef enum
{
    SB_SVC_ID_RULE_DEFAULT = 0,
    SB_SVC_ID_RULE_ONID_TSID_SVCID_SVCNAME,
    SB_SVC_ID_RULE_ONID_TSID_SVCID,
    SB_SVC_ID_RULE_ONID_SVCID_SVCNAME,
    SB_SVC_ID_RULE_ONID_SVCID,
    SB_SVC_ID_RULE_FREQ_TSID_SVCID,
    SB_SVC_ID_RULE_FREQ_POL_ONID_TSID_SVCID_SVCNAME,
    SB_SVC_ID_RULE_TSID_SVCID,
    SB_SVC_ID_RULE_ONID_NWID_TSID_SVCID,
    SB_SVC_ID_RULE_FREQ_POL_SVCID_SVCNAME,
    SB_SVC_ID_RULE_FREQ_POL_ONID_TSID_SVCID,
    SB_SVC_ID_RULE_TSID_SVCID_SVCNAME,
    SB_SVC_ID_RULE_NUM
}   SB_SVC_ID_RULE;


/* The prototype of notification function with svl record for setting bit mask  */
/*------------------------------------------------------------------*/
/*! @brief The prototype of notification function with svl record for setting bit mask
 *  @param [in] The notified data SB_NEW_SVC_NFY_DATA
 *  @param [in] the tag value passed in to scan engine
 *  @return      VOID
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_new_svl_rec_nfy_fct) (
        SB_NEW_SVL_REC_DATA*  pt_nfy_data,
        VOID*                 pv_nfy_tag);


/*------------------------------------------------------------------*/
/*! @struct _SB_NEW_SVL_REC_NFY_T
 *  @brief _SB_NEW_SVL_REC_NFY_T is used to setup a notification function with c_sb_set and keytype is SB_KEY_TYPE_NEW_SVL_REC_NFY
 *  @code
 *  typedef struct _SB_NEW_SVL_REC_NFY_T {
 *      x_sb_new_svl_rec_nfy_fct  pf_new_svl_rec_nfy;
 *      VOID*                     pv_new_svl_rec_nfy_tag;
 *  } SB_DVBC_NW_CHANGE_NFY_T;
 *  @endcode
 *  @li@c  pf_new_svl_rec_nfy            - the function pointer of new svl record notifcation
 *  @li@c  pv_new_svl_rec_nfy_tag        - the tag would be used when invoke the notification function.
 */
/*------------------------------------------------------------------*/
typedef struct _SB_NEW_SVL_REC_NFY_T {
    x_sb_new_svl_rec_nfy_fct  pf_new_svl_rec_nfy;
    VOID*                     pv_new_svl_rec_nfy_tag;
} SB_NEW_SVL_REC_NFY_T;

/* The prototype of the new service notification function */
/*------------------------------------------------------------------*/
/*! @brief The prototype of the new service notification function
 *  @param [in] The notified data SB_NEW_SVC_NFY_DATA
 *  @param [in] the tag value passed in to scan engine
 *  @return      VOID
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_new_svc_nfy_fct) (
        SB_NEW_SVC_NFY_DATA* 	pt_nfy_data,
        VOID*                 pv_nfy_tag);



/*------------------------------------------------------------------*/
/*! @brief It is used for notifying user the svc list full.
 *  @param [in] pv_nfy_tag The tag for notifying user.
 *  @param [out] None.
 *  @return
 *  @retval      - None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_svc_list_full_nfy_fct) (
        VOID*                       pv_nfy_tag);


/*------------------------------------------------------------------*/
/*! @struct SB_DVBS_SVC_LIST_FULL_NFY_T
 *  @brief 
 *  @code
 */
/*------------------------------------------------------------------*/
typedef struct _SB_SVC_LIST_FULL_NFY_T {
    x_sb_svc_list_full_nfy_fct       pf_svc_list_full_nfy;
    VOID*                            pv_svc_list_full_nfy_tag;
} SB_SVC_LIST_FULL_NFY_T;


typedef struct _SB_DVB_SCANNED_CH_T
{
    UINT16                  ui2_num_tv_ch;
    UINT16                  ui2_num_radio_ch;
    UINT16                  ui2_num_app_ch;
} SB_DVB_SCANNED_CH_T;


typedef enum{
    SB_SVC_HEVC_INSTALL_LEVEL_0,        /* Not to install HEVC services */
    SB_SVC_HEVC_INSTALL_LEVEL_1,        /* Install 2k HEVC(HD) services */ 
    SB_SVC_HEVC_INSTALL_LEVEL_2,        /* Install 4k HEVC(UHD) services */    
    SB_SVC_HEVC_INSTALL_LEVEL_3,        /* Install 2k HEVC(HD) 8bit services*/
    SB_SVC_HEVC_INSTALL_LEVEL_4,        /* Install 4k HEVC(HD) HFR services*/
}SB_SVC_HEVC_INSTALL_LEVEL;

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_SVL_BLDR */
/*----------------------------------------------------------------------------*/
/*! @page pageMW_SVL_BLDR SVL Builder Detailed Description
 *  @par
 *  @section secMW_SVL_BLDR_1  1. Introduction
 *  @par
 *      In order to provide the basic digital TV operations, such as channel change or events display, the channel information must be cached or recorded in advance. Middleware and applications (Navigator, for example) can share the recorded information. NetWork List (NWL), SerVice List (SVL), and Transport Stream List (TSL) are designed for this purpose.
 *  @par
 *      The SVL Builder (SB) component, which is responsible for retrieving channel information from transport stream, and building NWL, SVL, and TSL. SB is able to create a connection of specified source (Tuner, for example) with Connection Manager (CM, a component of Middleware, see document SW Design Middleware-Connection Manager for more details), and retrieving channel information from the specified source (Tuner, for example). SB is able to change to next RF channel when it has retrieved all channel information from current transport stream. By interfacing with Table Manager (a component of Middleware, see document SW Design Middleware-Table Manager for more details), SB is able to retrieve the required service information and construct the NWL/SVL/TSL database records.
 *  @par
 *      SVL Bulder also provides the auto update scan (Monitor) feature, which can auto detect and update information of channels of the current transport stream.
 *  @par
 *  @section secMW_SVL_BLDR_2  2. Concept
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_1  2.1. TSL
 *  @par
 *      The content of a TSLDB object should be saved to a TSL file immediately after the scan process is finished.  This type of files would be loaded into TSLDB when the SB component is initialized.  If no such file exists, SB component will create a default TSL according to the builder type.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_2  2.2. SVL
 *  @par
 *      The content of an SVLDB object should be saved to an SVL file immediately after the scan process is finished.  This type of files would be loaded into SVLDB when the SB component is initialized.  If no such file exists, SB component will create an empty SVLDB object.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_3  2.3. TSL and SVL record
 *  @par
 *      The relation of SVL record and TSL record is many-to-one, as illustrated in following figure. If TSL record broadcast type is analog, there is one-to-one SVL record. If the broadcast type of TSL record is digital, there may be some SVL records relative to it. The fields, rui2_tsl_id_ and rui2_tsl_rec_id_, in the structure SVL_REC_HEADER_T will point to a TSL record.
 *  @par
 *      @image html sb_tsl_svl_record.gif "TSL and SVL record"
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_4  2.4. Auto Scan
 *  @par
 *      Auto Scan will scan all channels of ATV and DTV. we also call it full mode scanning. The data of SVL and TSL will be cleaned before starting to scan. If the TV is carried from one city to other city, you had better do auto scan at first.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_5  2.5. Update Scan
 *  @par
 *      Update Scan will scan all the channels, but the channels on which captured the programs previous scanning will be skipped. If there are some new channels, you could do update scan to save time.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_6  2.6. Range Scan
 *  @par
 *      Range Scan will scan the channels which are between the range indicated by user.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_2_7  2.7. Single RF Scan
 *  @par
 *      Single RF scan is just for DTV. Obviously it only scan one channel indicatd by user.
 *  @par
 *  @section secMW_SVL_BLDR_3  3. Brief Design
 *  @par
 *  @subsection ssecMW_SVL_BLDR_3_1  3.1. Selector
 *  @par
 *      The selector passes the protocol-agnostic function calls to a proper builder, monitors the builder_s internal states, and gets the requested information back to applications.
 *      The selector receives some messages and parses which builder engine task function to call. It can change thread context for other module.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_3_2  3.2. Builder engine
 *  @par
 *      A builder engine should have the capability of interfacing with Table Manager, Connection Manager, and SVLDB.  By registration, a function table (see section 6.2.1) is duplicated in an engine control block.  All engine control blocks are maintained in a double-linked list. Such as DVB, ATSC, CQAM engine, they are divided by the DTV standard.
 *  @par
 *  @subsection ssecMW_SVL_BLDR_3_3  3.3. Notifications
 *  @par
 *      The callback function (*x_sb_scan_nfy_fct) is specified when calling the API x_sb_start_scan.  The suitable time to notify the application depends on the real implementation of builders.  For example, a builder may go through every record in the associated TSLDB object, where each record represents a PTC.  Every time a PTC record is processed (virtual channels inside it are recorded in SVLDB object), the scan progress notify function is called to notify the application of more detected channel(s).
 *  @par
 *  @section secMW_SVL_BLDR_4  4. Typical Case
 *  @par
 *      @dontinclude sb_sample.c
 *  @par
 *  @subsection ssecMW_SVL_BLDR_4_1  4.1. Open builder
 *  @par
 *      The following code is to open antenna digital builder an it takes the ATSC for example. If you want to open other builder, just replace the open data with corresponding data structure.
 *  @par
 *      @skip Open builder
 *      @until }
 *  @par
 *  @subsection ssecMW_SVL_BLDR_4_2  4.2. Close builder
 *  @par
 *      @skip Close builder
 *      @until }
 *  @par
 *  @subsection ssecMW_SVL_BLDR_4_3  4.3. Start scan
 *  @par
 *      The following sample code implements antenna digital channel scan function in menu.
 *  @par
 *  @subsubsection sssecMW_SVL_BLDR_4_3_1  4.3.1. Auto Scan
 *  @par
 *      @skip Scan notify
 *      @until }
 *  @par
 *      @skip Auto scan
 *      @until }
 *  @par
 *  @subsubsection sssecMW_SVL_BLDR_4_3_2  4.3.2. Update Scan
 *  @par
 *      @skip Update scan
 *      @until }
 *  @par
 *  @subsubsection sssecMW_SVL_BLDR_4_3_3  4.3.3. Range Scan
 *  @par
 *      @skip Range scan
 *      @until }
 *  @par
 *  @subsubsection sssecMW_SVL_BLDR_4_3_4  4.3.4. Single RF Scan
 *  @par
 *      @skip Single RF scan
 *      @until }
 *  @par
 *  @subsection ssecMW_SVL_BLDR_4_4  4.4. Cancel scan
 *  @par
 *      @skip Cancel scan
 *      @until }
 */
#endif /* _U_SB_H_ */

