/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_sm_bypass_event.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Stream Manager Bypass handler specific 
 *         events.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_BYPASS_HDLR
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_SM_BYPASS_EVENT_H_
#define _U_SM_BYPASS_EVENT_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_common.h"
#include "u_sm_event.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/

#define SM_EVN_BYPASS_APP_OPEN_REQ            ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 0)    /**<        */
#define SM_EVN_BYPASS_APP_PLAY_REQ            ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 1)    /**<        */
#define SM_EVN_BYPASS_APP_STOP_REQ            ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 2)    /**<        */
#define SM_EVN_BYPASS_APP_CLOSE_REQ           ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 3)    /**<        */
#define SM_EVN_BYPASS_DRV_OPENED              ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 4)    /**<        */
#define SM_EVN_BYPASS_DRV_PLAYED              ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 5)    /**<        */
#define SM_EVN_BYPASS_DRV_STOPPED             ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 6)    /**<        */
#define SM_EVN_BYPASS_DRV_CLOSED              ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 7)    /**<        */
#define SM_EVN_BYPASS_DRV_ERR                 ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 8)    /**<        */
#define SM_EVN_BYPASS_SCDB_RECORD_DEL         ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 9)    /**<        */
#define SM_EVN_BYPASS_INTERNAL_ERR            ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 10)    /**<        */
#define SM_EVN_BYPASS_VIDEO_RESOLUTION_CHG    ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 11)    /**<        */
#define SM_EVN_BYPASS_ANA_TV_SIGNAL           ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 12)    /**<        */
#define SM_EVN_BYPASS_ANA_TV_SIGNAL_DETECTING ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 13)    /**<        */
#define SM_EVN_BYPASS_ANA_COLOR_SYSTEM        ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 14)    /**<        */
#define SM_EVN_BYPASS_RESTART                 ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 15)    /**<        */
#define SM_EVN_BYPASS_HDCP_CONTENT            ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 16)    /**<        */
#define SM_EVN_BYPASS_MACROVISION             ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 17)    /**<        */
#define SM_EVN_BYPASS_CODEC_NOT_SUPPORT       ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 18)    /**<        */
#define SM_EVN_BYPASS_VIDEO_PLAY_DONE         ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + 19)    /**<        */

#define SM_EVN_BYPASS_GRP_SIZE ((SM_EVN_TYPE_T) 0x00010000)

#define SM_EVN_BYPASS_GRP_APP  ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + SM_EVN_BYPASS_GRP_SIZE*0 )
#define SM_EVN_BYPASS_GRP_DEC  ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + SM_EVN_BYPASS_GRP_SIZE*1 )
#define SM_EVN_BYPASS_GRP_TVE  ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + SM_EVN_BYPASS_GRP_SIZE*2 )
#define SM_EVN_BYPASS_GRP_VOUT ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + SM_EVN_BYPASS_GRP_SIZE*3 )
#define SM_EVN_BYPASS_GRP_PM   ((SM_EVN_TYPE_T) SM_EVN_GRP_BYPASS + SM_EVN_BYPASS_GRP_SIZE*4 )

#define SM_EVN_BYPASS_CODE_DRV_OPENED       ((SM_EVN_TYPE_T) 0)
#define SM_EVN_BYPASS_CODE_DRV_CLOSED       ((SM_EVN_TYPE_T) 1)
#define SM_EVN_BYPASS_CODE_DRV_CONNECTED    ((SM_EVN_TYPE_T) 2)
#define SM_EVN_BYPASS_CODE_DRV_DISCONNECTED ((SM_EVN_TYPE_T) 3)
#define SM_EVN_BYPASS_CODE_DRV_PLAYED       ((SM_EVN_TYPE_T) 4)
#define SM_EVN_BYPASS_CODE_DRV_STOPPED      ((SM_EVN_TYPE_T) 5)

#define SM_MAKE_DRV_EVN( grp, code ) ((SM_EVN_TYPE_T) grp + code )

#define SM_EVN_BYPASS_DEC_OPENED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_OPENED )
#define SM_EVN_BYPASS_DEC_CLOSED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_CLOSED )
#define SM_EVN_BYPASS_DEC_CONNECTED    SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_CONNECTED )
#define SM_EVN_BYPASS_DEC_DISCONNECTED SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_DISCONNECTED )
#define SM_EVN_BYPASS_DEC_PLAYED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_PLAYED )
#define SM_EVN_BYPASS_DEC_STOPPED      SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_DEC, SM_EVN_BYPASS_CODE_DRV_STOPPED )

#define SM_EVN_BYPASS_TVE_OPENED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_OPENED )
#define SM_EVN_BYPASS_TVE_CLOSED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_CLOSED )
#define SM_EVN_BYPASS_TVE_CONNECTED    SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_CONNECTED )
#define SM_EVN_BYPASS_TVE_DISCONNECTED SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_DISCONNECTED )
#define SM_EVN_BYPASS_TVE_PLAYED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_PLAYED )
#define SM_EVN_BYPASS_TVE_STOPPED      SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_TVE, SM_EVN_BYPASS_CODE_DRV_STOPPED )

#define SM_EVN_BYPASS_VOUT_OPENED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_OPENED )
#define SM_EVN_BYPASS_VOUT_CLOSED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_CLOSED )
#define SM_EVN_BYPASS_VOUT_CONNECTED    SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_CONNECTED )
#define SM_EVN_BYPASS_VOUT_DISCONNECTED SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_DISCONNECTED )
#define SM_EVN_BYPASS_VOUT_PLAYED       SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_PLAYED )
#define SM_EVN_BYPASS_VOUT_STOPPED      SM_MAKE_DRV_EVN( SM_EVN_BYPASS_GRP_VOUT, SM_EVN_BYPASS_CODE_DRV_STOPPED )

#define SM_EVN_BYPASS_PM_COND_CLOSED  ((SM_EVN_TYPE_T) SM_EVN_BYPASS_GRP_PM + 0)
#define SM_EVN_BYPASS_PM_COND_PLAYED  ((SM_EVN_TYPE_T) SM_EVN_BYPASS_GRP_PM + 1)
#define SM_EVN_BYPASS_PM_COND_STOPPED ((SM_EVN_TYPE_T) SM_EVN_BYPASS_GRP_PM + 2)

/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/

#endif /* _U_SM_BYPASS_EVENT_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of groupMW_STRM_MNGR_BYPASS_HDLR */
/*----------------------------------------------------------------------------*/

