/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*! @file u_sm_cc_ntsc_cst.h
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/2 $
 * $SWAuthor: Iolo Tsai $
 * $MD5HEX: 1593c95d2597b81c733d8df2f0da0c12 $
 *
 * @par Description: 
 *         This header file contains NTSC Closed Caption Code-Set Translator
 *         exported definitions.
 *---------------------------------------------------------------------------*/

     
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_CC_HDLR_C1
  *  @{  
  */
/*----------------------------------------------------------------------------*/

#ifndef _U_SM_CC_NTSC_CST_H_
#define _U_SM_CC_NTSC_CST_H_
/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_common.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief     Indicates the ntsc closed caption charset.
*  @code 
*  typedef enum 
*  { 
*      SM_CC_NTSC_CODE_SET_STANDARD = 0, 
*      SM_CC_NTSC_CODE_SET_FPCS,
*      SM_CC_NTSC_CODE_SET_SPCS, 
*      SM_CC_NTSC_CODE_SET_GB_2312
*      SM_CC_NTSC_CODE_SET_KSC_5601_1987
*      SM_CC_NTSC_CODE_SET_FRCS
*  } SM_CC_NTSC_CODE_SET_T; 
*  @endcode 
*  @li@c  SM_CC_NTSC_CODE_SET_STANDARD           -Indicates the standard charset.
*  @li@c  SM_CC_NTSC_CODE_SET_FPCS                    -Indicates the fpcs charset.
*  @li@c  SM_CC_NTSC_CODE_SET_SPCS                   -Indicates the spcs charset.
*  @li@c  SM_CC_NTSC_CODE_SET_GB_2312              -Indicates the gb_2312 charset.
*  @li@c  SM_CC_NTSC_CODE_SET_KSC_5601_1987    -Indicates the ksc_5601_1987 charset.
*  @li@c  SM_CC_NTSC_CODE_SET_FRCS                    -Indicates the frcs charset.
*/  
/*------------------------------------------------------------------*/
typedef enum
{
    SM_CC_NTSC_CODE_SET_STANDARD = 0,
    SM_CC_NTSC_CODE_SET_FPCS, /* FPCS: first private character set */
    SM_CC_NTSC_CODE_SET_SPCS, /* SPCS: second private character set */
    SM_CC_NTSC_CODE_SET_GB_2312,
    SM_CC_NTSC_CODE_SET_KSC_5601_1987,
    SM_CC_NTSC_CODE_SET_FRCS  /* FRCS: first registered character set */
} SM_CC_NTSC_CODE_SET_T;




/*------------------------------------------------------------------*/
/*! @brief  This type defines the ntsc closed caption charset translating function prototype.
  *  @param [in] puc_src            Reference to the source string.
  *  @param [in] z_src_len         Indicates the length of source string.
  *  @param [out] pwc_utf          Reference to the output string.
  *  @param [out] pz_dst_len     Indicates the length of destination string.
  */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sm_cc_ntsc_cst_translate_fct)( UCHAR*   puc_src, 
                                                 SIZE_T   z_src_len,
                                                 UTF16_T* pwc_utf, 
                                                 SIZE_T*  pz_dst_len );
                                       
/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/


#endif /* _U_SM_CC_NTSC_CST_H_ */


/*----------------------------------------------------------------------------*/
/*! @} */                                 /* end of groupMW_STRM_MNGR_CC_HDLR_C1 */
/*----------------------------------------------------------------------------*/



