/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_fp_display.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains FP display driver specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_FP_DISPLAY_H_
#define _X_FP_DISPLAY_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define FPD_GET_TYPE_CAPABILITY    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define FPD_GET_TYPE_LEVEL         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))
#define FPD_GET_TYPE_MODE          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))
#define FPD_GET_TYPE_COLOR         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 3))
#define FPD_GET_TYPE_NUM_OF_LINES  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 4))
#define FPD_GET_TYPE_LINE_LEN      (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 5))
#define FPD_GET_TYPE_CONTENT       (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 6))

/* Set operations */
#define FPD_SET_TYPE_LEVEL    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1))
#define FPD_SET_TYPE_MODE     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 2))
#define FPD_SET_TYPE_COLOR    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))
#define FPD_SET_TYPE_CONTENT  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 6))
#define FPD_SET_TYPE_CLEAR    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 7))

/* Flags used during registration */
#define FPD_FLAG_ALPHA_NUMERIC  ((UINT32) 0x00010000)


/* Color and sink types */
typedef enum
{
    FPD_COLOR_SINGLE = 0,
    FPD_COLOR_RED,
    FPD_COLOR_GREEN,
    FPD_COLOR_YELLOW,
    FPD_COLOR_BLUE,
    FPD_COLOR_AMBER,
    FPD_COLOR_WHITE
}   FPD_COLOR_T;

/* Brightness level types */
typedef enum
{
    FPD_LEVEL_0 = 0,
    FPD_LEVEL_10,
    FPD_LEVEL_20,
    FPD_LEVEL_30,
    FPD_LEVEL_40,
    FPD_LEVEL_50,
    FPD_LEVEL_60,
    FPD_LEVEL_70,
    FPD_LEVEL_80,
    FPD_LEVEL_90,
    FPD_LEVEL_100
}   FPD_LEVEL_T;

/* Mode types */
typedef enum
{
    FPD_MODE_STEADY = 0,
    FPD_MODE_BLINK,
    FPD_MODE_CIRCLE_RIGHT,
    FPD_MODE_CIRCLE_LEFT
}   FPD_MODE_T;

/* Speed types */
typedef enum
{
    FPD_SPEED_SLOW = 0,
    FPD_SPEED_MEDIUM,
    FPD_SPEED_FAST
}   FPD_SPEED_T;

/* Capability color */
#define FPD_CAP_COLOR_SINGLE  (((UINT32) 1) << FPD_COLOR_SINGLE)
#define FPD_CAP_COLOR_RED     (((UINT32) 1) << FPD_COLOR_RED)
#define FPD_CAP_COLOR_GREEN   (((UINT32) 1) << FPD_COLOR_GREEN)
#define FPD_CAP_COLOR_YELLOW  (((UINT32) 1) << FPD_COLOR_YELLOW)
#define FPD_CAP_COLOR_BLUE    (((UINT32) 1) << FPD_COLOR_BLUW)
#define FPD_CAP_COLOR_AMBER   (((UINT32) 1) << FPD_COLOR_AMBER)
#define FPD_CAP_COLOR_WHITE   (((UINT32) 1) << FPD_COLOR_WHITE)

/* Capability level */
#define FPD_CAP_LEVEL_0    (((UINT32) 1) << FPD_LEVEL_0)
#define FPD_CAP_LEVEL_10   (((UINT32) 1) << FPD_LEVEL_10)
#define FPD_CAP_LEVEL_20   (((UINT32) 1) << FPD_LEVEL_20)
#define FPD_CAP_LEVEL_30   (((UINT32) 1) << FPD_LEVEL_30)
#define FPD_CAP_LEVEL_40   (((UINT32) 1) << FPD_LEVEL_40)
#define FPD_CAP_LEVEL_50   (((UINT32) 1) << FPD_LEVEL_50)
#define FPD_CAP_LEVEL_60   (((UINT32) 1) << FPD_LEVEL_60)
#define FPD_CAP_LEVEL_70   (((UINT32) 1) << FPD_LEVEL_70)
#define FPD_CAP_LEVEL_80   (((UINT32) 1) << FPD_LEVEL_80)
#define FPD_CAP_LEVEL_90   (((UINT32) 1) << FPD_LEVEL_90)
#define FPD_CAP_LEVEL_100  (((UINT32) 1) << FPD_LEVEL_100)

/* Capability speed */
#define FPD_CAP_SPEED_SLOW    (((UINT32) 1) << FPD_STATE_SLOW)
#define FPD_CAP_SPEED_MEDIUM  (((UINT32) 1) << FPD_STATE_MEDIUM)
#define FPD_CAP_SPEED_FAST    (((UINT32) 1) << FPD_STATE_FAST)

/* Capability info */
typedef struct _FPD_CAPABILITY_INFO_T
{
    UINT32  ui4_level;
    UINT32  ui4_color;
    UINT32  ui4_blink_speed;
    UINT32  ui4_circle_right_speed;
    UINT32  ui4_circle_left_speed;
}   FPD_CAPABILITY_INFO_T;

/* Content clear info */
typedef struct _FPD_CLEAR_INFO_T
{
    UINT16  ui2_line_num;

    SIZE_T  z_offset;

    SIZE_T  z_len;
}   FPD_CLEAR_INFO_T;

/* Content display info. Used to show ASCII or UNICODE */
/* represented as sequences of UTF-8 code values.      */
typedef struct _FPD_CONTENT_CHAR_INFO_T
{
    UINT16  ui2_line_num;

    SIZE_T  z_offset;

    CHAR*  ps_content;
}   FPD_CONTENT_CHAR_INFO_T;

/* Mode info */
typedef struct _FPD_MODE_INFO_T
{
    FPD_MODE_T   e_mode;
    FPD_SPEED_T  e_speed;
}   FPD_MODE_INFO_T;


#endif /* _X_FP_DISPLAY_H_ */
