/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_usb_bt.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains sicd device related definitions.
 *---------------------------------------------------------------------------*/

#ifndef _X_USB_BT_H_
#define _X_USB_BT_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "x_common.h"
#include "x_rm.h"

/************************************************************************/
/* Error code                                                           */
/************************************************************************/
#define USB_BT_OK                       ((INT32)    0)
#define USB_BT_DEVICE_BUSY              ((INT32)   -1)
#define USB_BT_MEDIA_REMOVED            ((INT32)   -2)
#define USB_BT_ALIGNMENT_ERROR          ((INT32)   -3)
#define USB_BT_USB_ERROR                ((INT32)   -4)
#define USB_BT_UNKNOWN_ERROR            ((INT32)   -5)

/*-----------------------------------------------------------------------------
 *  Constants and type definitions
 *---------------------------------------------------------------------------*/
/* Traffic types in ui1_traffic_type */
#define USB_BT_XFER_CONTROL	    0
#define USB_BT_XFER_ISOC	    1
#define USB_BT_XFER_BULK	    2
#define USB_BT_XFER_INT        	3

#define USB_BT_STATE_FREE                      ((uint8_t)0x00)
#define USB_BT_STATE_CONNECT                   ((uint8_t)0x01)
#define USB_BT_STATE_CONFIGURED                ((uint8_t)0x02)
#define USB_BT_STATE_SET_IDLE                  ((uint8_t)0x03)


typedef struct _USB_BT_IO_REQ_T USB_BT_IO_REQ_T;

typedef void (*usb_bt_io_complete_fnc)(VOID*                pv_tag, 
                                       USB_BT_IO_REQ_T*     pt_bt_iO);

typedef struct _USB_BT_ISO_T 
{
    UINT32      ui4_data_length;
    UINT32      ui4_actual_data_length;
    INT32       i4_status;
}USB_BT_ISO_T;

struct _USB_BT_IO_REQ_T
{
    UINT8                   ui1_traffic_type;           /*I: 0 ~ 3: select transfer type.*/
    BOOL                    b_is_tx;                    /*I: 1: Tx, 0: Rx */

    UINT8*                  aui1_setup_packet;          /*I: Only for ui1_traffic_type == USB_BT_XFER_CONTROL 8 bytes*/

    UINT8*                  aui1_data_buffer;           /*I/O: data phase data buffer; Tx:data to dev, Rx:dev to client */

    UINT32                  ui4_data_length;            /*I: If ui1_traffic_type == MUSB_BT_XFER_CONTROL, 
                                                             ui4_data_length = the size of data at data phase. It does not include setup packet. */

    UINT32                  ui4_actual_data_length;     /*O: actual data length gotton*/
    UINT32                  ui4_status;                  /*O: 0: ok, others: fail.*/

    UINT32                  ui4_packets_number;
    USB_BT_ISO_T*           at_iso_packet; 
    
    usb_bt_io_complete_fnc  pf_bt_io_complete;          /*callback function*/
    VOID*                   pv_tag;
};

typedef struct _USB_BT_IO_REQ_NUM_T
{
    BOOL                    b_is_tx;
    UINT8                   ui1_traffic_type;           /*I: 0 ~ 3: select transfer type.*/
    UINT32                  ui4_io_request_in_queue;    /*O: num of uncompleted request in driver queue*/
}USB_BT_IO_REQ_NUM_T;

/*-----------------------------------------------------------------------------
 * Get types
 *---------------------------------------------------------------------------*/
#define USB_BT_GET_TYPE_IO_REQ_NUM (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))  // USB_BT_IO_REQ_NUM_T
#define USB_BT_GET_TYPE_DEV_NUM (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))     // UINT32* 
#define USB_BT_GET_TYPE_DEV_STATUS (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))  // UINT32* (USB_BT_STATE)


/*-----------------------------------------------------------------------------
 * Set types
 *---------------------------------------------------------------------------*/
#define USB_BT_SET_TYPE_SEND_IO_REQ (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0)) // CSR_BT_IO_REQ_T: send CSR BT request to dev
#define USB_BT_SET_TYPE_SEND_IO_REQ_ISR  (RM_SET_TYPE_ISR_CONTEXT | USB_BT_SET_TYPE_SEND_IO_REQ)

#endif /* _X_CSR_BT_H_ */
