/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_usb_bt.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains sicd device related definitions.
 *---------------------------------------------------------------------------*/

#ifndef _X_USB_SERIAL_H_
#define _X_USB_SERIAL_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "x_common.h"
#include "x_rm.h"
#include "x_rs_232.h"

/************************************************************************/
/* Error code                                                           */
/************************************************************************/
#define USB_SERIAL_OK                       ((INT32)    0)
#define USB_SERIAL_DEVICE_BUSY              ((INT32)   -1)
#define USB_SERIAL_MEDIA_REMOVED            ((INT32)   -2)
#define USB_SERIAL_ALIGNMENT_ERROR          ((INT32)   -3)
#define USB_SERIAL_USB_ERROR                ((INT32)   -4)
#define USB_SERIAL_UNKNOWN_ERROR            ((INT32)   -5)

/*-----------------------------------------------------------------------------
 *  Constants and type definitions
 *---------------------------------------------------------------------------*/
/* Traffic types in ui1_traffic_type */
#define USB_SERIAL_XFER_CONTROL	    0
#define USB_SERIAL_XFER_ISOC	    1
#define USB_SERIAL_XFER_BULK	    2
#define USB_SERIAL_XFER_INT        	3

#define USB_SERIAL_STATE_FREE                      ((uint8_t)0x00)
#define USB_SERIAL_STATE_CONNECT                   ((uint8_t)0x01)
#define USB_SERIAL_STATE_CONFIGURED                ((uint8_t)0x02)
#define USB_SERIAL_STATE_SET_IDLE                  ((uint8_t)0x03)



/* Notify condition */
typedef enum
{
    USB_SERIAL_COND_REC_DATA = 0,
    USB_SERIAL_COND_REC_BUFFER,
    USB_SERIAL_COND_REC_FRAME_ERROR,
    USB_SERIAL_COND_REC_OVERFLOW,
    USB_SERIAL_COND_XMT,
    USB_SERIAL_COND_XMT_EMPTY,
    USB_SERIAL_COND_CTRL_CHG
}   USB_SERIAL_COND_T;



/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* Get operations */
#define USB_SERIAL_GET_TYPE_CAPABILITY      RS_232_GET_TYPE_CAPABILITY         
#define USB_SERIAL_GET_TYPE_SETUP			RS_232_GET_TYPE_SETUP               
#define USB_SERIAL_GET_TYPE_CTRL_LINE		RS_232_GET_TYPE_CTRL_LINE           
#define USB_SERIAL_GET_TYPE_REC_DATA		RS_232_GET_TYPE_REC_DATA            
#define USB_SERIAL_GET_TYPE_REC_MULTI_DATA	RS_232_GET_TYPE_REC_MULTI_DATA      
#define USB_SERIAL_GET_TYPE_REC_DATA_LEN	RS_232_GET_TYPE_REC_DATA_LEN        
#define USB_SERIAL_GET_TYPE_XMT_DATA_LEN	RS_232_GET_TYPE_XMT_DATA_LEN        
#define USB_SERIAL_GET_TYPE_OPERATION_MODE	RS_232_GET_TYPE_OPERATION_MODE      

#define USB_SERIAL_GET_TYPE_REC_DATA_NFY		RS_232_GET_TYPE_REC_DATA_NFY        
#define USB_SERIAL_GET_TYPE_REC_MULTI_DATA_NFY 	RS_232_GET_TYPE_REC_MULTI_DATA_NFY  
#define USB_SERIAL_GET_TYPE_XMT_DATA_LEN_NFY	RS_232_GET_TYPE_XMT_DATA_LEN_NFY    

/* Set operations */
#define USB_SERIAL_SET_TYPE_SETUP			RS_232_SET_TYPE_SETUP               
#define USB_SERIAL_SET_TYPE_CTRL_LINE		RS_232_SET_TYPE_CTRL_LINE           
//#define USB_SERIAL_SET_TYPE_NFY_FCT			RS_232_SET_TYPE_NFY_FCT             
#define USB_SERIAL_SET_TYPE_CTRL_NFY_COND	RS_232_SET_TYPE_CTRL_NFY_COND       
#define USB_SERIAL_SET_TYPE_XMT_DATA		RS_232_SET_TYPE_XMT_DATA            
#define USB_SERIAL_SET_TYPE_XMT_MULTI_DATA	RS_232_SET_TYPE_XMT_MULTI_DATA      
#define USB_SERIAL_SET_TYPE_OPERATION_MODE	RS_232_SET_TYPE_OPERATION_MODE      

#define USB_SERIAL_SET_TYPE_XMT_DATA_NFY	RS_232_SET_TYPE_XMT_DATA_NFY        
#define USB_SERIAL_SET_TYPE_XMT_MULTI_DATA_NFY	RS_232_SET_TYPE_XMT_MULTI_DATA_NFY


#define USB_SERIAL_SET_TYPE_NFY_FCT	(RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))


#if 0
/*-----------------------------------------------------------------------------
 * Get types
 *---------------------------------------------------------------------------*/
#define USB_SERIAL_GET_TYPE_IO_REQ_NUM (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))  // USB_SERIAL_IO_REQ_NUM_T
#define USB_SERIAL_GET_TYPE_DEV_NUM (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))     // UINT32* 
#define USB_SERIAL_GET_TYPE_DEV_STATUS (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))  // UINT32* (USB_SERIAL_STATE)


/*-----------------------------------------------------------------------------
 * Set types
 *---------------------------------------------------------------------------*/
#define USB_SERIAL_SET_TYPE_SEND_IO_REQ (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0)) // CSR_SERIAL_IO_REQ_T: send CSR SERIAL request to dev
#define USB_SERIAL_SET_TYPE_SEND_IO_REQ_ISR  (RM_SET_TYPE_ISR_CONTEXT | USB_SERIAL_SET_TYPE_SEND_IO_REQ)
#define USB_SERIAL_SET_TYPE_GET_IO_REQ (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1)) 
#define USB_SERIAL_SET_TYPE_GET_IO_REQ_ISR  (RM_SET_TYPE_ISR_CONTEXT | USB_SERIAL_SET_TYPE_GET_IO_REQ)
#endif
#endif /* _X_CSR_SERIAL_H_ */
