/*----------------------------------------------------------------------------*
 * No Warranty                                                                *
 * Except as may be otherwise agreed to in writing, no warranties of any      *
 * kind, whether express or implied, are given by MTK with respect to any MTK *
 * Deliverables or any use thereof, and MTK Deliverables are provided on an   *
 * "AS IS" basis.  MTK hereby expressly disclaims all such warranties,        *
 * including any implied warranties of merchantability, non-infringement and  *
 * fitness for a particular purpose and any warranties arising out of course  *
 * of performance, course of dealing or usage of trade.  Parties further      *
 * acknowledge that Company may, either presently and/or in the future,       *
 * instruct MTK to assist it in the development and the implementation, in    *
 * accordance with Company's designs, of certain softwares relating to        *
 * Company's product(s) (the "Services").  Except as may be otherwise agreed  *
 * to in writing, no warranties of any kind, whether express or implied, are  *
 * given by MTK with respect to the Services provided, and the Services are   *
 * provided on an "AS IS" basis.  Company further acknowledges that the       *
 * Services may contain errors, that testing is important and Company is      *
 * solely responsible for fully testing the Services and/or derivatives       *
 * thereof before they are used, sublicensed or distributed.  Should there be *
 * any third party action brought against MTK, arising out of or relating to  *
 * the Services, Company agree to fully indemnify and hold MTK harmless.      *
 * If the parties mutually agree to enter into or continue a business         *
 * relationship or other arrangement, the terms and conditions set forth      *
 * hereunder shall remain effective and, unless explicitly stated otherwise,  *
 * shall prevail in the event of a conflict in the terms in any agreements    *
 * entered into between the parties.                                          *
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2005, MediaTek Inc.
 * All rights reserved.
 *
 * Unauthorized use, practice, perform, copy, distribution, reproduction,
 * or disclosure of this information in whole or in part is prohibited.
 *---------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: imgrz_if.h,v $
 * $Revision: #6 $
 *
 *---------------------------------------------------------------------------*/

/** @file imgrz_if.h
 *  image resizer driver main interfaces
 *
 */

#ifndef IMGRZ_IF_H
#define IMGRZ_IF_H


//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------


/*lint -save -e961 */
#include "x_common.h"
#include "x_typedef.h"
#include "x_timer.h"
#include "x_assert.h"
#include "x_os.h"
//#include "x_bim.h"
#include "x_clksrc.h"
#include "x_hal_5381.h"
#include "x_hal_926.h"
/*lint -restore */


//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------
#define IMGRZ_TRUE              1
#define IMGRZ_FALSE             0
#define IMGRZ_RISC_MODE

#if defined(CC_MT5890)|| defined(CC_MT5891) || defined(CC_MT5882)||defined(CC_MT5886) ||defined(CC_MT5863)
#define IMGRZ_HW_IO_MMU
#endif
#if (defined(CC_MT5893) || defined(CC_MT5895)) && defined(CONFIG_MTK_M4U)
#define IMGRZ_HW_M4U
#endif

#if defined(CC_MT5863) && defined(IMGRZ_HW_IO_MMU)
#define IMGRZ_HW_64BIT_IOMMU
#endif

/// String size
#define D_RZ_STR_SIZE           50

/// Software mode
#define D_RZ_HAVE_SW_MOD        (1 << (INT32)E_RZ_SW_MOD)

/// Hardware mode
#define D_RZ_HAVE_HW_MOD        (1 << (INT32)E_RZ_HW_MOD)

/// Align 16 bytes
#define D_RZ_ALIGN16(X) (((X) + 0xF) & (~0xF))	///< for scaler pitch alignment
#define D_RZ_ALIGN32(X) (((X) + 0x1F) & (~0x1F))
//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

//#define REMAP_HWRZID        ((u4Which >= IMGRZ_NUM) ? 0 : _rIdx2ImgrzHwIdTbl[u4Which])

#define REMAP_HWRZID   imgrz_map_hwid(u4Which)
#define E_FIRST_RESIZER     0
#if defined(CC_MT5893) || defined(CC_MT5895)
    #define IMGRZ_NUM   5
    #define IMGRZ4VDO_NUM   (IMGRZ_NUM - 1)
    #define E_SECND_RESIZER     1
    #define E_THIRD_RESIZER     2
    #define E_FORTH_RESIZER     3
    #define E_FIFTH_RESIZER     4
    #define E_RZ_DEFAULT_ID     E_FIFTH_RESIZER
#elif defined(CC_MT5890) || defined(CC_MT5886)
    #define IMGRZ_NUM   3
	#define IMGRZ4VDO_NUM	(IMGRZ_NUM - 1)
    #define E_SECND_RESIZER     1
    #define E_THIRD_RESIZER     2
    #define E_RZ_DEFAULT_ID 	E_SECND_RESIZER
#elif defined(CC_MT5891)
	#define IMGRZ_NUM	4
	#define IMGRZ4VDO_NUM	(IMGRZ_NUM - 1)
    #define E_SECND_RESIZER     1
    #define E_THIRD_RESIZER     2
	#define E_FORTH_RESIZER		3
    #define E_RZ_DEFAULT_ID 	E_SECND_RESIZER
#elif defined(CC_MT5882) || defined(CC_MT5863)
    #define IMGRZ_NUM   2
    #define IMGRZ4VDO_NUM	(IMGRZ_NUM - 1)
    #define E_SECND_RESIZER     1
    #define E_RZ_DEFAULT_ID 	E_SECND_RESIZER
#else
    #define E_SECND_RESIZER     0
    #define IMGRZ_NUM   1
    #define IMGRZ4VDO_NUM	IMGRZ_NUM
    #define E_RZ_DEFAULT_ID 	E_FIRST_RESIZER
#endif

/// Error types
typedef enum
{
    E_RZ_OK = 0,
    E_RZ_INV_ARG,
    E_RZ_OUT_OF_MEM,
    E_RZ_UNINIT,
    E_RZ_UNDEF_ERR
} E_RZ_ERR_CODE_T;

/// OSD input color mode (direct/index)
typedef enum
{
    E_RZ_OSD_IN_CM_INDEX  = 0,
    E_RZ_OSD_IN_CM_DIRECT = 1
} E_RZ_OSD_IN_CM_T;

/// OSD color mode (index)
typedef enum
{
    E_RZ_OSD_IDX_CM_2BPP   = 1,
    E_RZ_OSD_IDX_CM_4BPP   = 2,
    E_RZ_OSD_IDX_CM_8BPP   = 3
} E_RZ_OSD_IDX_CM_T;

/// OSD color mode (direct)
typedef enum
{
    E_RZ_OSD_DIR_CM_RGB565      = 0,
    E_RZ_OSD_DIR_CM_ARGB1555    = 1,
    E_RZ_OSD_DIR_CM_ARGB4444    = 2,
    E_RZ_OSD_DIR_CM_ARGB8888    = 3,
    E_RZ_OSD_DIR_CM_AYCbCr8888  = 3,
    E_RZ_OSD_DIR_CM_YCbYCr      = 4,
    E_RZ_OSD_DIR_CM_CbYCrY      = 5,
    E_RZ_OSD_DIR_CM_RGBA8888    = 6
} E_RZ_OSD_DIR_CM_T;

/// Scaling type for dealing with alpha change
typedef enum
{
    E_RZ_A_SCL_TYPE_GENERAL          = 0,
    E_RZ_A_SCL_TYPE_REFER_TO_LEFT    = 1,
    E_RZ_A_SCL_TYPE_REFER_TO_NEAREST = 2
} E_RZ_A_SCL_TYPE_T;

/// VDO output mode
typedef enum
{
    E_RZ_VDO_OUTMD_420 = 0,
    E_RZ_VDO_OUTMD_422 = 1,
    E_RZ_VDO_OUTMD_444 = 2
} E_RZ_VDO_OUTMD_T;

/// Inout type (video/jpeg/osd)
typedef enum
{
    E_RZ_INOUT_TYPE_VDOMD  = 0,
    E_RZ_INOUT_TYPE_JPGMD  = 1,
    E_RZ_INOUT_TYPE_OSDMD  = 2,
    E_RZ_INOUT_TYPE_OSDUI  = 3,
    E_RZ_INOUT_TYPE_VDOENCODE=4,
    E_RZ_INOUT_TYPE_VDORMRPR=5,
    E_RZ_INOUT_TYPE_VDOENCODE_V2=6,
    E_RZ_INOUT_TYPE_OSD_CSC =7,
} E_RZ_INOUT_TYPE_T;

/// Modules
typedef enum
{
    E_RZ_SW_MOD = 0,
    E_RZ_HW_MOD,
    E_RZ_MODULE_LAST
} E_RZ_MODULE_T;

typedef enum
{
	E_RZ_OSD_CSC_FORMAT_ARGB8888=0,
	E_RZ_OSD_CSC_FORMAT_AYUV8888=1
} E_RZ_CSC_FORMAT_T;

typedef enum
{
	E_RZ_VDO_BOTTOM_FILED=0,
	E_RZ_VDO_TOP_FILED=1
} E_RZ_VDO_FIELD_TYPE_T;

typedef enum
{
    E_RZ_OSD_ALCOM_MODE_CLEAR         = 0x0,
    E_RZ_OSD_ALCOM_MODE_DST_IN        = 0x1,
    E_RZ_OSD_ALCOM_MODE_DST_OUT       = 0x2,
    E_RZ_OSD_ALCOM_MODE_DST_OVER      = 0x3,
    E_RZ_OSD_ALCOM_MODE_SRC           = 0x4,
    E_RZ_OSD_ALCOM_MODE_SRC_IN        = 0x5,
    E_RZ_OSD_ALCOM_MODE_SRC_OUT       = 0x6,
    E_RZ_OSD_ALCOM_MODE_SRC_OVER      = 0x7,
    E_RZ_OSD_ALCOM_MODE_DST           = 0x8,
    E_RZ_OSD_ALCOM_MODE_SRC_ATOP      = 0x9,
    E_RZ_OSD_ALCOM_MODE_DST_ATOP      = 0xA,
    E_RZ_OSD_ALCOM_MODE_XOR           = 0xB,
    E_RZ_OSD_ALCOM_MODE_NONE          = 0xC,
    E_RZ_OSD_ALCOM_MODE_ADD           = 0xD
} E_RZ_OSD_ALCOM_MODE;

typedef enum {
	IMGRZ_DONE_OK=100,
	IMGRZ_DONE_TIMEOUT,
	IMGRZ_DONE_FAULT
}IMGRZ_DONE_TYPE;

typedef enum _RZ_UFO_TYPE_
{
	RZ_UFO_TYPE_8BIT,
	RZ_UFO_TYPE_10BIT_COMPACT,
	RZ_UFO_TYPE_10BIT_REORDER,
	RZ_UFO_TYPE_10BIT_COMPACT_UNCOMPRESS
	//RZ_UFO_TYPE_10BIT_BYPASS,
}RZ_UFO_TYPE;


// VDO scale parameter setting
typedef struct _RZ_VDO_SCL_PARAM_SET_T
{
    UINT32 u4Size;
    UINT32 u4IsRsIn;        // block|raster based input
    UINT32 u4InMode;        // 420|422 input
    UPTR   u4YSrcBase;      // y
    UINT32 u4YSrcBufLen;
    UINT32 u4CSrcBufLen;
    UINT32 u4YSrcHOffset;
    UINT32 u4YSrcVOffset;
    UINT32 u4YSrcW;
    UINT32 u4YSrcH;
    UPTR   u4CSrcBase;      // cbcr
    UINT32 u4CSrcHOffset;
    UINT32 u4CSrcVOffset;
    UINT32 u4CSrcW;
    UINT32 u4CSrcH;

    UINT32 u4IsRsOut;       // block|raster based output
    UINT32 u4OutMode;       // 420|422|444 output
    UINT32 u4IsVdo2Osd;     // just for vdo2ayuv
    UPTR   u4YTgBase;       // y
    UINT32 u4YTgCM;
    UINT32 u4YTgBufLen;
    UINT32 u4CTgBufLen;
    UINT32 u4YTgHOffset;
    UINT32 u4YTgVOffset;
    UINT32 u4YTgW;
    UINT32 u4YTgH;
    UPTR   u4CTgBase;       // cbcr
    UINT32 u4CTgW;
    UINT32 u4CTgH;
    UINT32 u4SrcSwap;
    UINT32 u4OutSwap;
    //For webp
    UINT32 u4IsFstBl;       // first block line of Webp
    UINT32 u4IsLstBl;       // last  block line of webp
    UINT32 u4LoopFilterEnable;
    UINT32 u4IsVdoPartialModeEnable;     // WEBP and rz hand-off on ?
    UINT32 u4IsWebPRzOn;     // WEBP and rz hand-off on ?
    UINT32 u4IsSameaddr;
    UINT32 u4RzTmpBuf;
    UINT32 u4Venc_skip;		//5368, E_RZ_INOUT_TYPE_VDOENCODE_V2
    UINT32 u4VdoCbCrSwap;
    UINT32 u4ABlend;
    UINT32 u4PreLoadYAddr;
    UINT32 u4PreLoadCAddr;
    UINT32 u4FieldTypeEnable;
    UINT32 u4FieldType;

    //for ufo
    BOOL fgUfoEn;     //ufo enable
    RZ_UFO_TYPE eUfoType;  //ufo type
    UPTR   u4UfoYBufSZSA;  //Y src SZbuf Addr
    UPTR   u4UfoCBufSZSA;  //C src SZbuf Addr

    //end for ufo
    UPTR u4RzIOMMU_MMUTableAddr;

    UINT32 u4IsVdo2ARGB;//for vdo2argb, u4IsVdo2ARGB & u4IsVdo2Osd both be true
    UINT32 u4VdoMultiPartition;
    BOOL fgUfoJumpMode; //ufo source jumpmode
    BOOL fg10bitout;
    BOOL fgP010out;
	BOOL fg10bitRaster; //10bit source as Raster mode.
	BOOL fgdisableOnePhase;//for heif hw bug
} RZ_VDO_SCL_PARAM_SET_T;

// JPG scale parameter setting
typedef struct _RZ_JPG_SCL_PARAM_SET_T
{
    UINT32 u4Size;
    UINT32 u4IsRsIn;        // block|raster based input
    UPTR   u4YSrcBase1;     // y1
    UPTR   u4YSrcBase2;     // y2
    UINT32 u4YSrcBufLen;
    UINT32 u4CSrcBufLen;
    UINT32 u4YSrcHOffset;
    UINT32 u4YSrcVOffset;
    UINT32 u4YSrcW;
    UINT32 u4YSrcH;
    UPTR   u4CbSrcBase1;    // cb1
    UPTR   u4CbSrcBase2;    // cb2
    UINT32 u4CbSrcHOffset;
    UINT32 u4CbSrcVOffset;
    UINT32 u4CbSrcW;
    UINT32 u4CbSrcH;
    UPTR   u4CrSrcBase1;    // cr1
    UPTR   u4CrSrcBase2;    // cr2
    UINT32 u4CrSrcHOffset;
    UINT32 u4CrSrcVOffset;
    UINT32 u4CrSrcW;
    UINT32 u4CrSrcH;

    UINT32 u4IsRsOut;       // block|raster based output
    UINT32 u4OutMode;       // 420|422|444 output
    UINT32 u4IsVdo2Osd;     // output in osd format ?
    UPTR   u4YTgBase;       // y
    UINT32 u4YTgCM;
    UINT32 u4YTgBufLen;
    UINT32 u4CTgBufLen;
    UINT32 u4YTgHOffset;
    UINT32 u4YTgVOffset;
    UINT32 u4YTgW;
    UINT32 u4YTgH;
    UPTR   u4CTgBase;       // cbcr
    UINT32 u4CTgW;
    UINT32 u4CTgH;

    UINT32 u4IsFstBl;       // first block line of jpg
    UINT32 u4IsLstBl;       // last  block line of jpg
    UINT32 u4JpgVfacY;
    UINT32 u4JpgVfacCb;
    UINT32 u4JpgVfacCr;
    UINT32 u4IsJpgRzOn;     // jpg and rz hand-off on ?
    UINT32 u4IsJpgSameaddr;
    UINT32 u4ColorComp;
    UINT32 u4JpgTempAddr;
    UINT32 u4OutSwap;

    BOOL   fgGray;
    BOOL   fgPreload;
    BOOL   fgBlassign;

    //paritial output
    BOOL fgJpgModePartialOutput;
    UPTR u4RzIOMMU_MMUTableAddr;
} RZ_JPG_SCL_PARAM_SET_T;

// OSD scale parameter setting
typedef struct _RZ_OSD_SCL_PARAM_SET_T
{
    UINT32 u4Size;
    UPTR   u4SrcBase;
    UINT32 u4IsSrcDirCM;    // direct|index color mode
    UINT32 u4SrcCM;
    UINT32 u4SrcBufLen;
    UINT32 u4SrcHOffset;
    UINT32 u4SrcVOffset;
    UINT32 u4SrcW;
    UINT32 u4SrcH;

    UPTR   u4TgBase;
    UINT32 u4TgCM;
    UINT32 u4TgBufLen;
    UINT32 u4TgHOffset;
    UINT32 u4TgVOffset;
    UINT32 u4TgW;
    UINT32 u4TgH;

	//seamless update
    UINT32 u4CPTDataCount;
    UINT32 *pu4CPTData;
    UINT32 u4PartialUpdate;
    UINT32 u4PartialUpdate_ClipSrcx;
    UINT32 u4PartialUpdate_ClipSrcy;
    UINT32 u4PartialUpdate_ClipSrcw;
    UINT32 u4PartialUpdate_ClipSrch;
    UINT32 u4PartialUpdate_ClipTgx;
    UINT32 u4PartialUpdate_ClipTgy;
    UINT32 u4PartialUpdate_ClipTgw;
    UINT32 u4PartialUpdate_ClipTgh;

	//csc setting
	UINT32 u4Csc_enable;
	E_RZ_CSC_FORMAT_T	csc_src_format;
	E_RZ_CSC_FORMAT_T	csc_des_format;

    UINT32 u4IsFstBl;       // first block line of png
    UINT32 u4IsLstBl;       // last  block line of png
    BOOL	 fgBlassign;
    BOOL   fgPartialMode;
	UPTR   u4PngTempAddr;
    UINT32 u4RowBufHeight;
    UINT32 u4IsPngSameaddr;

    BOOL   fgOSDTurboMode;                  /*Must be TRUE if composition mode Enabled*/
    BOOL   fgAlphaComposition;              /*TRUE to Enable composition mode, FALSE to disable*/
    E_RZ_OSD_ALCOM_MODE  u4AlphaComposition_Mode;/*composition mode*/
    UINT32 u4AlphaValue;                    /*Ar, alpha value 0-255*/
    UINT32 u4AlphaComposition_PreloadAddr;  /*Another buffer base*/
    UINT32 u4PremultSrcR;                   /*TRUE if   source       read    buffer is premultiplied format*/
    UINT32 u4PremultDstR;                   /*TRUE if   destination read    buffer is premultiplied format*/
    UINT32 u4PremultDstW;                   /*TRUE if   destination write   buffer is premultiplied format*/
    UINT32 u4PremultOvrflw;                 /*Clip RGB to alpha value when reading premultiplied fomat*/

	BOOL fgRBExChange;
	BOOL fgDoDither;
	UPTR   u4RzIOMMU_MMUTableAddr;
	UINT32 u4IOMMURead;
	UINT32 u4IOMMUWrite;
} RZ_OSD_SCL_PARAM_SET_T;


/*cb run in irq context, pls return quickly*/
typedef int (*imgrz_done_cb) (unsigned int u4Which,
				IMGRZ_DONE_TYPE u4Type, void *data);
//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------
extern UINT32 imgrz_map_hwid(UINT32 u4Which);
//---------------------------------------------------------------------------
//  IMGRZ_Init
//
/** Initialize gfx driver.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Init(void);
extern void IMGRZ_Init_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_Reset
//
/** Reset engine.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Reset(void);
extern void IMGRZ_Reset_Ex(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_Wait
//
/** Wait for flushing engine completely.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Wait(void);
extern void IMGRZ_Wait_Ex(UINT32 u4Which);


#if !defined(IMGRZ_RISC_MODE)
//---------------------------------------------------------------------------
//  IMGRZ_MwFlush
//
/** Flush cmdque and force engine to do operations (for MW use).
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_MwFlush_Ex(UINT32 u4Which);
#endif

//---------------------------------------------------------------------------
//  IMGRZ_Flush
//
/** Flush cmdque and force engine to do operations (for Drv use).
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Flush(void);
extern void IMGRZ_Flush_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_Lock
//
/** Take the imgrz resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Lock(void);
extern void IMGRZ_Lock_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_TryLock
//
/** Try to take the imgrz resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
//extern void IMGRZ_TryLock(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_Unlock
//
/** Release the imgrz resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Unlock(void);
extern void IMGRZ_Unlock_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_LockCmdque
//
/** Take the imgrz-cmdq resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
//extern void IMGRZ_LockCmdque(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_UnlockCmdque
//
/** Release the imgrz-cmdq resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_UnlockCmdque(void);
//extern void IMGRZ_UnlockCmdque_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_LockWaitIsr
//
/** Take the imgrz- waitIsr cmd resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
//extern void IMGRZ_LockWaitIsr(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_UnlockWaitIsr
//
/** Release the imgrz-waitIsr cmd resource.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_UnlockWaitIsr(void);
//extern void IMGRZ_UnlockWaitIsr_Ex(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_QueryHwIdle
//
/** Query engine status.
 *
 *  @retval 1   Idle
 *  @retval 0   Busy
 */
//---------------------------------------------------------------------------
extern INT32 IMGRZ_QueryHwIdle(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_ReInit
//
/** Re initial imgrz.
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_ReInit(void);
extern void IMGRZ_ReInit_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_SetScaleOpt
//
/** Set scale option.
 *
 *  @param  eInOutType   Scale option
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_SetScaleOpt(E_RZ_INOUT_TYPE_T eInOutType);
extern void IMGRZ_SetScaleOpt_Ex(UINT32 u4Which, E_RZ_INOUT_TYPE_T eInOutType);


//---------------------------------------------------------------------------
//  IMGRZ_Scale
//
/** Execute scale operation.
 *
 *  @param  pvSclParam  scale parameters
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Scale(void *pvSclParam);
extern void IMGRZ_Scale_Ex(UINT32 u4Which, void *pvSclParam);
extern void IMGRZ_Async_Ex(UINT32 u4Which, imgrz_done_cb fDoneCb, void *pvData);


//---------------------------------------------------------------------------
//  IMGRZ_GetScaleOpt
//
/** get scale option.
 *  @retval void
 */
//---------------------------------------------------------------------------
extern E_RZ_INOUT_TYPE_T IMGRZ_GetScaleOpt(void);
extern E_RZ_INOUT_TYPE_T IMGRZ_GetScaleOpt_Ex(UINT32 u4Which);


//---------------------------------------------------------------------------
//  IMGRZ_SetDigest
//
/** Set a video output frame to a thumbnail result
 *
 *  @param  pvSrc  source parameters
 *  @param  pvDst  distination parameters
 *  @param  fgOnePass FALSE: 2pass (only video scaling), TRUE: 1 pass (scaling and color conv)
 *  @param  u4WorkingBuf  working buffer address for 2 pass
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_SetDigest(void *pvSrc, void *pvDst, BOOL fgOnePass, UINT32 u4WorkingBuf, UINT32 u4Which);

void IMGRZ_SetDigest_Ex(void *pvSrc, void *pvDst, BOOL fgOnePass, UINT32 u4WorkingBuf, UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_Break
//
/** Save imgrz parameters for breaking
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Break(void);
extern void IMGRZ_Break_Ex(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_Resume
//
/** Resume the process of imgrz of jpg mcu row mode
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_Resume(void);
extern void IMGRZ_Resume_Ex(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_OFF
//
/** disable clock of imgrz
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern void IMGRZ_OFF(void);
extern void IMGRZ_OFF_Ex(UINT32 u4Which);

//---------------------------------------------------------------------------
//  IMGRZ_OFF
//
/** disable clock of imgrz
 *
 *  @retval void
 */
//---------------------------------------------------------------------------
extern UINT32 IMGRZ_GetScaleLine(void);
extern UINT32 IMGRZ_GetScaleLine_Ex(UINT32 u4Which);
extern BOOL IMGRZ_CkgenOnOff(UINT32 u4Id, BOOL fgEnable);

extern void IMGRZ_AYUV_ARGB_Conv(UPTR  u4TgBase,UINT32 u4BufLen,UINT32 u4Width, UINT32 u4Height,
    UINT32 u4OffsetX,  UINT32 u4OffsetY, UINT32 u4MMUTableAddr, BOOL fgAYUV2ARGB);
extern void IMGRZ_AYUV_ARGB_Conv_Ex(UPTR u4Which, UINT32  u4TgBase,UINT32 u4BufLen,UINT32 u4Width, UINT32 u4Height,
    UINT32 u4OffsetX,  UINT32 u4OffsetY, UINT32 u4MMUTableAddr, BOOL fgAYUV2ARGB);

extern void IMGRZ_DMA(UINT32 u4SrcAddr,UINT32 u4DstAddr, UINT32 u4Pitch,UINT32 u4Height, UINT32 u4MMUTableAddr);
extern void IMGRZ_DMA_Ex(UINT32 u4Which, UINT32 u4SrcAddr,UINT32 u4DstAddr, UINT32 u4Pitch,UINT32 u4Height, UINT32 u4MMUTableAddr);
extern UPTR IMGRZ_DMA_Ex2(UINT32 u4Which, UPTR u4SrcAddr,UPTR u4DstAddr, UINT32 u4Size, UPTR u4MMUTableAddr);

#define OSD_CLR_MODE_CONV_OSD_4444    4
#define OSD_CLR_MODE_CONV_OSD_8888    3
#define OSD_CLR_MODE_CONV_OSD_565     0
//extern void IMGRZ_OSD_CLR_MODE_CONV(UINT32 u4Which, UINT32 u4SrcAddr,UINT32 u4DstAddr, UINT32 u4Pitch, UINT32 u4Height, UINT32 fromClrMode, UINT32 toClrMode,BOOL fgDoDither);

extern void IMGRZ_Cfg_MMU_IO(BOOL MMU_Read, BOOL MMU_Write);
extern void IMGRZ_Cfg_MMU_IO_Ex(UINT32 u4Which, BOOL MMU_Read, BOOL MMU_Write);
#if defined(CC_MT5890)
extern void IMGRZ_ReadSpecReg_Ex(UINT32 u4Which);
#endif
extern void IMGRZ_DecompressUFO(void *pvSrc, UINT32 u4Which);

#endif // IMGRZ_IF_H



