/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: pd_atd_if.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

#ifndef PD_ATD_IF_H
#define PD_ATD_IF_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
//#include "pd_tuner_type.h"
//#include "atd_register.h"
#include "tuner_interface_if.h"
#include "u_handle.h"
#include "fe_tuner_common_if.h"
//-----------------------------------------------------------------------------
// Configurations
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

enum
{
    ATV_CONN_MODE_CHAN_CHANGE = 0,
    ATV_CONN_MODE_CHAN_SCAN,
    ATV_CONN_MODE_FINE_TUNE,
    ATV_CONN_MODE_FM_RADIO,
    ATV_CONN_MODE_NUM
};        


//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
typedef struct _ATD_PD_CTX_T
{
// ATD PI context
    void*        sAtdPiCtx;
// Hardware Configure
    UINT32      Frequency;                  /* Carrier frequency (in Hz) */
    UINT8       u1VIFLock;
    UINT8       u1TunerAcqType;
    UINT8       u1SubSysIdx;
    UINT32      u4TvSubSysMask;
    UINT8       ucTvSysGroupNum;
    UINT32      *pu4TvSysGroup;
    VOID        *psTunerAnaAttr;
// Glue-layer variables
    HANDLE_T    hHalLock;  //sema
    UINT32      u4DrvCustTunerCtrl;
    UINT32      u4AcqFrequency;
    BOOL        fgBreakMonitor;
    BOOL        fgPIMoniStatus; //1: doing monitor 0:not doing monitor
    BOOL        fgDisStatus;
} ATD_PD_CTX_T, *PATD_PD_CTX_T;


//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------- 
/** ATD_GetTVSystem
 *  Get ATV TV system setting.
 *  @retval   ATV TV System.   
 */
//-----------------------------------------------------------------------------
extern UINT8 ATD_GetTVSystem(void); //wenming 2012-09-13:Add for other modules get ATV TV system setting

//----------------------------------------------------------------------------- 
/** ATD_GetIFAGC
 *  Get IFAGC control word value from ATD.
 *  @retval   (UINT16) IFAGC control word value.   
 */
//-----------------------------------------------------------------------------
extern UINT16 ATD_GetIFAGC(void);

//----------------------------------------------------------------------------- 
/** ATD_SetLNA
 *  Set tuner LNA ON/OFF.
 *  @param  fgLNA: flag to control LNA ON/OFF, 0 for OFF and 1 for ON
 *  @retval   0: Set tuner LNA fail.
 *  @retval   1: Set tuner LNA success. 
 */
//-----------------------------------------------------------------------------
extern BOOL ATD_SetLNA(BOOL fgLNA);
extern BOOL ATD_SetAntIn(UINT8 fgAntIn);
//----------------------------------------------------------------------------- 
/** ATD_SetDeScrambleBox
 *  Set DeScrambleBox on/off
 *  @param  fgScramble: flag to control DeScrambleBox ON/OFF, 0 for OFF and 1 for ON
 *  @retval   0: Set tuner DeScrambleBox fail.
 *  @retval   1: Set tuner DeScrambleBox success. 
 */
//-----------------------------------------------------------------------------

extern BOOL ATD_SetDeScrambleBox(BOOL fgScramble );

//----------------------------------------------------------------------------- 
/** AtdEU_AudSetSystem
 *  Set Demod System.
 *  @param  u1SubSysIdx: system type
 *  @param  u1ChMode: 0:scan mode,   1:normal mode
 *  @retval none 
 */
//-----------------------------------------------------------------------------
#ifdef CC_SUPPORT_AUTO_DETECT_AUD_SYSTEM
extern void AtdEU_AudSetSystem(UINT8 u1SubSysIdx, UINT8 u1ChMode);
#endif

#endif /* PD_ATD_IF_H */

