#ifndef SWDMX_EMU_ALP_IF_H
#define SWDMX_EMU_ALP_IF_H

#include "atsc30_drvif.h"
#include "atsc30_file_dump.h"
#include "alp_hal_hw.h"

typedef enum
{
    eSWDMX_ALP_STATE_STOP,          // stop
    eSWDMX_ALP_STATE_PAUSE,         // pause
    eSWDMX_ALP_STATE_PLAY,          // play
    eSWDMX_ALP_STATE_MAX
}swdmx_emu_alp_state_t;

typedef enum
{
    eSWDMX_SET_APPLY_DEFAULT_SETTINGS  = 0,

    eSWDMX_SET_DATA_READ_SIZE          = 1,
    eSWDMX_SET_DMX_SINGLE_MOVE_SIZE    = 2,

    eSWDMX_SET_ALP_DMX_TS_IDX          = 3,
    eSWDMX_SET_ALP_HW_MODE             = 4,

    eSWDMX_SET_ALP_HAL_FW_SEND_PTP     = 5,
    eSWDMX_SET_ALP_HAL_FW_SEND_PLP_ID  = 6,
    eSWDMX_SET_ALP_HAL_FW_SEND_ALL_PKT = 7,
    eSWDMX_SET_ALP_INT_TRIG_COUNT      = 8,

    eSWDMX_SET_ALP_DUMP_CAOUT         = 9,
    eSWDMX_SET_ALP_DUMP_ALP_CB_DATA   = 10,
    eSWDMX_SET_ALP_DUMP_UDP_IP_DATA   = 11,
    eSWDMX_SET_ALP_DUMP_LLS_DATA      = 12,

    eSWDMX_SET_SRC_INPUT_FILE_NAME    = 13,
    eSWDMX_SET_SRC_INPUT_ALP_PKT_LEN_FILE_NAME = 14,
    eSWDMX_SET_SRC_STREAM_FILTER_FILE_NAME = 15,

    eSWDMX_SET_OUTPUT_ALP_FILE_NAME   = 16,
    eSWDMX_SET_OUTPUT_IP_FILE_NAME    = 17,
    eSWDMX_SET_OUTPUT_LLS_FILE_NAME   = 18,
    eSWDMX_SET_CAOUT_DUMP_FILE_NAME   = 19,

    eSWDMX_SET_DDI_MOVE_VALID_INTVL    = 20,
    eSWDMX_SET_DEMOD_INPUT             = 21,
    eSWDMX_SET_FE_INPUT_PACKET_FMT     = 22,
    eSWDMX_SET_HW_EXTRA_FLAGS          = 23,
    eSWDMX_SET_HW_FE_TYPE              = 24,
    eSWDMX_SET_DUMP_INFO               = 25,
    eSWDMX_SET_PLP_ID_ARRAY            = 26,
    eSWDMX_SET_PACKET_UPLOAD_TYPE_MASK = 27,
    eSWDMX_SET_DEMOD_FRONT_END_TYPE    = 28,
    eSWDMX_SET_NEED_CFG_STREAM_FILTER  = 29,
    eSWDMX_SET_IDLE_MOVE_DATA  = 30,
    eSWDMX_SET_INFO_TYPE_UNKNOWN,
}swdmx_alp_emu_set_type_t;

#define SWDMX_EMU_ALP_FILE_NAME_MAX_LENGTH         (256)

typedef struct
{
    BOOL   input_from_demod;
    BOOL   need_cfg_alp_strm_filter;
    char   fn_input_pkt_stream[SWDMX_EMU_ALP_FILE_NAME_MAX_LENGTH];
    char   fn_input_pkt_len_rec[SWDMX_EMU_ALP_FILE_NAME_MAX_LENGTH];
    char   fn_input_filter[SWDMX_EMU_ALP_FILE_NAME_MAX_LENGTH];

    UINT8  dmx_ddi_interval;
    alp_hal_framer_mode_t hw_framer_mode;

    UINT32 single_read_size;
    UINT32 dmx_single_move_size;
    UINT32 driver_upload_packet_type_mask ;
}swdmx_alp_emu_config_info_t;

typedef struct
{
    BOOL   eos;
    BOOL   read_file_end;
    BOOL   first_data_move;
    UINT32 feeder_data_size;

    UINT32* alp_pkt_rec_arr;
    UINT32 alp_pkt_tot_cnt;
    UINT32 alp_pkt_read_idx;
    UINT32 alp_pkt_move_idx;

    UPTR   dmx_move_start_addr;

    UPTR fp_alp_input_strm;
    UPTR fp_input_alp_pkt_len_rec;
    UPTR fp_alp_strm_filter;

    UINT64 cur_read_pos;
    UINT64 cur_dmx_move_pos;
    UINT64 total_file_size;
}swdmx_alp_emu_data_ctrl_t;

typedef struct
{
    UINT8 plp_id_cnt;
    UINT8 alp_strm_cnt;

    UINT8 plp_id_filter[eATSC30_HW_SUPPORT_PLP_NUM];
    alp_stream_t alp_strm[eATSC30_HW_MAX_SUPPORT_STREAM_NUM];
}swdmx_alp_emu_stream_filter_t;

typedef enum
{
    SWDMX_FEEDER_BUF_SIZE             = (8  * 1024 * 1024UL),
    SWDMX_INPUT_PKT_LEN_REC_BUF_SIZE  = (8  * 1024 * 1024UL),
    SWDMX_STRM_FILTER_BUF_SIZE        = (1  * 1024 * 1024UL),
    HW_CAOUT_BUF_SIZE                 = (8  * 1024 * 1024UL),
    FW_BUF_POOL_SIZE                  = (0  * 1024 * 1024),
    DRVIF_BUF_POOL_SIZE               = (8  * 1024 * 1024),
    SWDMX_SINGLE_FILE_DUMP_BUF_SIZE   = (8  * 1024 * 1024UL),
}SWDM_EMU_ALP_BUFFER_SIZE_CONFIG_E;

typedef enum
{
    EMU_BUF_TYPE_FEEDER      = 0,
    EMU_BUF_TYPE_PKT_LEN     = 1,
    EMU_BUF_TYPE_STRM_FILTER = 2,
    EMU_BUF_TYPE_HW_CAOUT    = 3,
    EMU_BUF_TYPE_FW_POOL     = 4,
    EMU_BUF_TYPE_DRVIF_POOL  = 5,
    EMU_BUF_TYPE_DUMP_POOL   = 6,
    EMU_BUF_TYPE_MAX         = 7,
}swdmx_alp_emu_buf_type_t;

typedef struct
{
    UPTR buf_start;
    UINT32 buf_size;
}swdmx_alp_emu_buf_info_t;

typedef enum
{
    DUMP_TYPE_CAOUT = 0,
    DUMP_TYPE_DRV_ALP_RAW = 1,
    DUMP_TYPE_DRV_IP = 2,
    DUMP_TYPE_DRV_LLS = 3,
    DUMP_TYPE_MAX,
}swdmx_alp_emu_dump_type_t;

typedef struct
{
    BOOL  first_dump;
    BOOL  enable_dump;
    CHAR  dump_file_name[SWDMX_EMU_ALP_FILE_NAME_MAX_LENGTH];
    UPTR  aux_ring_buf;
    UPTR  aux_ring_buf_size;
    atsc30_file_dump_handler_t dump_hdlr;
}swdmx_alp_emu_dump_info_t;

typedef struct
{
    UINT8                         src_id;
    UINT8                         framer_idx;
    UINT8                         dmx_inst_id;
    BOOL                          driver_init_done;
    BOOL                          enable_recv_udp_ip_batch;
    BOOL                          enable_recv_alp_batch;
    BOOL                          enable_idle_move_data;
    alp_hal_frontend_input_packet_type fe_pkt_type;
    swdmx_emu_alp_state_t         play_state;
    HANDLE_T                      play_cmd_Q_handle;
    HANDLE_T                      emu_thread_handle;
    HANDLE_T                      emu_recv_udp_ip_en_nfy_queue_handle;
    HANDLE_T                      emu_recv_udp_ip_thread_handle;
    HANDLE_T                      emu_recv_alp_thread_handle;
    HANDLE_T                      emu_recv_alp_en_nfy_queue_handle;
    swdmx_alp_emu_config_info_t   cfg;

    swdmx_alp_emu_data_ctrl_t     data_ctrl;
    swdmx_alp_emu_stream_filter_t strm_filter;

    swdmx_alp_emu_buf_info_t      buf_info[EMU_BUF_TYPE_MAX];
    swdmx_alp_emu_dump_info_t     dump_info[DUMP_TYPE_MAX];
}swdmx_alp_emu_inst_t;

EXTERN BOOL swdmx_alp_emu_send_play_command(const UINT8 src_id, const swdmx_emu_alp_state_t rNewState);
EXTERN BOOL swdmx_alp_emu_init(VOID);
EXTERN BOOL swdmx_alp_emu_set_info(UINT8 src_id, swdmx_alp_emu_set_type_t eType, UPTR ptrPara);
EXTERN BOOL swdmx_alp_emu_config(const UINT8 src_id);

#endif /* SWDMX_EMU_ALP_IF_H*/
