/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: uart_hw_drvif.h,v $
 * $Revision: #2 $
 *
 *---------------------------------------------------------------------------*/

/** @file uart_hw_drvif.h
 *  Brief of file uart_hw_drvif.h.
 *  Details of file uart_hw_drvif.h (optional).
 */

#ifndef UART_HW_DRVIF_H
#define UART_HW_DRVIF_H

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "x_serial.h"
#include "x_os.h"
#include "x_bim.h"
#include "uart_if.h"

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

// *********************************************************************
// UART fifo size
// *********************************************************************
#define UART0_FIFO_SIZE             ((UINT32)32 - 1)
#define UART1_FIFO_SIZE             ((UINT32)32 - 1)
#define UARTPD_FIFO_SIZE            ((UINT32)32 - 1)

// *********************************************************************
// UART register related
// *********************************************************************
#define UART_PD_BASE                (UINT32)(0x800)

#define REG_SER_PORT                (UINT32)0x00    // Data port for port 0
#define REG_SER01_PORT              (UINT32)0xc0    // Data port for port 1
#define REG_SERPD_PORT              (UINT32)0x800   // Data port for port 2 (PD)

#define REG_SER_CTL                 (UINT32)0x04
#define REG_SERPD_CTL               (UINT32)(UART_PD_BASE + 0x04)
#define TRANS_MODE_ON               (UINT32)(0x1 << 6)
#define SET_TRANS_MODE_ON           (UINT32)0xE2
#define REG_SER01_PORT_4BYTES       (UINT32)0xc4

// Communication control register for port 0, port 1 and port 2 (port PD)
#define REG_SER_CTR                 (UINT32)0x48
#define REG_SER01_CTR               (UINT32)0xc8
#define REG_SERPD_CTR               (UINT32)(UART_PD_BASE + 0x30)

#define BUAD_RATE_115200            (UINT32)(0x0<<8)
#define BUAD_RATE_115200x2          (UINT32)(0x1<<8)
#define BUAD_RATE_115200x4          (UINT32)(0x2<<8)
#define BUAD_RATE_115200x8          (UINT32)(0x3<<8)
#define BUAD_RATE_57600             (UINT32)(0x4<<8)
#define BUAD_RATE_38400             (UINT32)(0x5<<8)
#define BUAD_RATE_19200             (UINT32)(0x6<<8)
#define BUAD_RATE_9600              (UINT32)(0x7<<8)
#define BUAD_RATE_4800              (UINT32)(0x8<<8)
#define BUAD_RATE_2400              (UINT32)(0x9<<8)
#define BUAD_RATE_1200              (UINT32)(0xA<<8)
#define BUAD_RATE_300               (UINT32)(0xB<<8)
#define BUAD_RATE_110               (UINT32)(0xC<<8)
#define BUAD_RATE_PD_57600          (UINT32)(0x1<<8)
#define BUAD_RATE_PD_38400          (UINT32)(0x2<<8)
#define BUAD_RATE_PD_19200          (UINT32)(0x3<<8)
#define BUAD_RATE_PD_9600           (UINT32)(0x4<<8)
#define BUAD_RATE_PD_2400           (UINT32)(0x5<<8)
#define PARITY_EVEN                 (UINT32)(0x1<<5)
#define PARITY_ODD                  (UINT32)(0x0<<5)
#define CHECK_PARITY_ON             (UINT32)(0x1<<4)
#define CHECK_PARITY_OFF            (UINT32)(0x0<<4)
#define DIS_RX_ZERO_ON              (UINT32)(0x1<<3)
#define DIS_RX_ZERO_OFF             (UINT32)(0x0<<3)
#define STOP_BIT_1                  (UINT32)(0x0<<2)
#define STOP_BIT_1_2                (UINT32)(0x1<<2)
#define STOP_BIT_2                  (UINT32)(0x1<<2)
#define STOP_BIT_1_EXT              (UINT32)(0x0<<7)
#define STOP_BIT_1_2_EXT            (UINT32)(0x1<<7)
#define STOP_BIT_2_EXT              (UINT32)(0x1<<7)
#define BIT_LEN_5                   (UINT32)(0x3<<0)
#define BIT_LEN_6                   (UINT32)(0x2<<0)
#define BIT_LEN_7                   (UINT32)(0x1<<0)
#define BIT_LEN_8                   (UINT32)(0x0<<0)
#define UART_PARITY_ENABLE          (UINT32)CHECK_PARITY_ON
#define UART_PARITY_EVEN            (UINT32)PARITY_EVEN
#define CTR_NORMAL                  (UINT32)(BUAD_RATE_115200 | CHECK_PARITY_OFF | BIT_LEN_8)
#define CTR_HIGH_SPEED              (UINT32)(BUAD_RATE_115200x8 | CHECK_PARITY_OFF | BIT_LEN_8)

// Buffer status register for port 0, port 1 and port 2 (port PD)
#define REG_SER_STATUS              (UINT32)0x4c
#define REG_SER01_STATUS            (UINT32)0xcc
#define REG_SERPD_STATUS            (UINT32)(UART_PD_BASE + 0x34)
#define SER_WRITE_ALLOW             (UINT32)(UART0_FIFO_SIZE << 8)
#define SER_READ_ALLOW              (UINT32)(UART0_FIFO_SIZE)

#define REG_SER_BUF_CTL             (UINT32)0x50 // Buffer control register for port 0
#define REG_SER01_BUF_CTL           (UINT32)0xd0 // Buffer control register for port 1
#define REG_SERPD_BUF_CTL           (UINT32)((UART_PD_BASE) + 0x38)
#define CLEAR_RX_BUFFER             (UINT32)(1<<6)
#define CLEAR_TX_BUFFER             (UINT32)(1<<7)

#define REG_SER_INT_EN              (UINT32)0x0c // Interrupt enable register
#define REG_SER_INT_STATUS          (UINT32)0x10 // Interrupt status register
#define INT_SER0_RX_RISC_DATA       (UINT32)(0x1<<22) // Normal mode
#define INT_SER0_RX_OVERFLOW        (UINT32)(0x1<<21)
#define INT_SER0_TX_EMPTY           (UINT32)(0x1<<19)
#define INT_SER0_RX_TOUT            (UINT32)(0x1<<18)
#define INT_SER0_RX_FULL            (UINT32)(0x1<<17)
#define INT_SER0_RX_ERR             (UINT32)(0x1<<16)
#define INT_SER1_DMAW_TOUT          (UINT32)(0x1<<11)
#define INT_SER1_DMAW_OVERFLOW      (UINT32)(0x1<<10)
#define INT_SER1_DMAW_FULL          (UINT32)(0x1<<9)
#define INT_SER1_DMAR_EMPTY         (UINT32)(0x1<<8)
#define INT_SER1_RX_OVERFLOW        (UINT32)(0x1<<5)
#define INT_SER1_MODEM              (UINT32)(0x1<<4)
#define INT_SER1_TX_EMPTY           (UINT32)(0x1<<3)
#define INT_SER1_RX_TOUT            (UINT32)(0x1<<2)
#define INT_SER1_RX_FULL            (UINT32)(0x1<<1)
#define INT_SER1_RX_ERR             (UINT32)(0x1<<0)

#define INT_SER0_MASK               (UINT32)0x007F0000
#define INT_SER1_MASK               (UINT32)0x00000F3F

#define REG_SERPD_INT_EN            (UINT32)(UART_PD_BASE + 0x0c)
#define REG_SERPD_INT_STATUS        (UINT32)(UART_PD_BASE + 0x10)
#define INT_SERPD_T8032_DATA        (UINT32)(0x1<<7)
#define INT_SERPD_RISC_DATA         (UINT32)(0x1<<6) // Normal mode
#define INT_SERPD_RX_OVERFLOW       (UINT32)(0x1<<5)
#define INT_SERPD_MODEM             (UINT32)(0x1<<4)
#define INT_SERPD_TX_EMPTY          (UINT32)(0x1<<3)
#define INT_SERPD_RX_TOUT           (UINT32)(0x1<<2)
#define INT_SERPD_RX_FULL           (UINT32)(0x1<<1)
#define INT_SERPD_RX_ERR            (UINT32)(0x1<<0)
#define INT_SERPD_MASK              (UINT32)0x0000007F

#define REG_SER1_DMAW_EN            (UINT32)0x140
#define DMAW_CIR                    (UINT32)(0x1<<1)
#define DMAW_EN                     (UINT32)(0x1<<0)
#define REG_SER1_DMAW_RST           (UINT32)0x144
#define REG_SER1_DMAW_LEVEL         (UINT32)0x148
#define REG_SER1_DMAW_SPTR          (UINT32)0x150
#define REG_SER1_DMAW_EPTR          (UINT32)0x154
#define REG_SER1_DMAW_WPTR          (UINT32)0x158
#define REG_SER1_DMAW_RPTR          (UINT32)0x15c

#define REG_SER1_DMAR_EN            (UINT32)0x160
#define DMAR_CIR                    (UINT32)(0x1<<1)
#define DMAR_EN                     (UINT32)(0x1<<0)
#define REG_SER1_DMAR_RST           (UINT32)0x164
#define REG_SER1_DMAR_LEVEL         (UINT32)0x168
#define REG_SER1_DMAR_SPTR          (UINT32)0x170
#define REG_SER1_DMAR_EPTR          (UINT32)0x174
#define REG_SER1_DMAR_WPTR          (UINT32)0x178
#define REG_SER1_DMAR_RPTR          (UINT32)0x17c

#define REG_SER_MERGE_MODE          (UINT32)0x14 // Merge mode control register
#define REG_NROMAL_MODE_BUF_SZ      (UINT32)0x1c
#define REG_PD_NROMAL_MODE_BUF_SZ   (UINT32)(UART_PD_BASE + 0x1c)


#define REG_SER_MSR                 (UINT32)0xd4
#define MODEM_DTR                   (UINT32)(1 << 0)
#define MODEM_RTS                   (UINT32)(1 << 1)
#define MODEM_HW_RTS                (UINT32)(1 << 2)
#define MODEM_HW_CTS                (UINT32)(1 << 3)
#define MODEM_RTS_TRIGGER_MASK      (UINT32)(0xf << 4)
#define MODEM_DSR                   (UINT32)(1 << 8)
#define MODEM_CTS                   (UINT32)(1 << 9)
#define MODEM_RI                    (UINT32)(1 << 10)
#define MODEM_DCD                   (UINT32)(1 << 11)
#define MODEM_DSR_CHG               (UINT32)(1 << 12)
#define MODEM_CTS_CHG               (UINT32)(1 << 13)
#define MODEM_RI_CHG                (UINT32)(1 << 14)
#define MODEM_DCD_CHG               (UINT32)(1 << 15)

#define MODEM_RTS_TRIGGER_VALUE     (UINT32)(0xf << 4)

#if 1
/* uart 5 UART_BT*/
#define UART5_BASE              (0x180) // 0xc180 //(0x880)// 0x28880

#define UART5_DATA              (UART5_BASE + 0x000)
#define UART5_DATA_BYTE         (UART5_DATA + 0x000)

#define UART5_STATUS_OLD        (0x0 + 0x004)// 0xc004
    #define U5_TRANSPARENT (1 << 6)

#define UART5_INT_EN            (0x0 + 0x00c)// 0xc00C
    //#define U5_T8032D      (1 << 7)  // normal mode t8032 data
    //#define U5_RISCD       (1 << 6)  // normal mode risc data
    #define U5_OVRUN       (1 << 26)  // rxd buffer overflow r:23
    #define U5_OVRUN_R  (1 << 23)
	
    #define U5_MRIE       (1 << 25)  //  r:28
    #define U5_MRIE_R       (1 << 28)  //

    #define U5_TBUF        (1 << 24)  // txd buffer empty r:27
    #define U5_TBUF_R        (1 << 27)
	
    #define U5_TOUT        (1 << 23)  // rxd timeout r:26
    #define U5_TOUT_R        (1 << 26)
	
    #define U5_RBUF        (1 << 7)  // rxd data available r:25
    #define U5_RBUF_R        (1 << 25)
	 
    #define U5_RERR        (1 << 6)  // rxd error  r: 24
    #define U5_RERR_R        (1 << 24)

    #define U5_INTMSK      (U5_OVRUN | U5_MRIE | U5_TBUF | U5_TOUT | U5_RBUF | U5_RERR)
	#define U5_INTMSK_R      (U5_OVRUN_R | U5_MRIE_R | U5_TBUF_R | U5_TOUT_R | U5_RBUF_R | U5_RERR_R)
	#define U5_INTALL	   (U5_TOUT | U5_RBUF | U5_RERR)

#define UART5_INT_ID            (0x010)//(UART5_BASE + 0x010)
//#define UART5_RS232_MODE        (UART5_BASE + 0x018)
//#define UART5_BUFFER_SIZE       (UART5_BASE + 0x01c)
#define UART5_COMMCTRL          (UART5_BASE + 0x008)
#define UART5_STATUS            (UART5_BASE + 0x00c)
#define UART5_BUFCTRL           (UART5_BASE + 0x010)
#endif


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------

#define UART_READ8(offset)              ((offset < UART_PD_BASE) ? \
                                         IO_READ8(RS232_BASE, (offset)) : \
                                         IO_READ8(PDWNC_BASE, (offset)))
#define UART_READ16(offset)             ((offset < UART_PD_BASE) ? \
                                         IO_READ16(RS232_BASE, (offset)) : \
                                         IO_READ16(PDWNC_BASE, (offset)))
#define UART_READ32(offset)             ((offset < UART_PD_BASE) ? \
                                         IO_READ32(RS232_BASE, (offset)) : \
                                         IO_READ32(PDWNC_BASE, (offset)))

#define UART_WRITE8(offset, value)                                      \
            do {                                                        \
                if (offset < UART_PD_BASE)                              \
                    IO_WRITE8(RS232_BASE, (offset), (value));           \
                else                                                    \
                    IO_WRITE8(PDWNC_BASE, (offset), (value));           \
            } while (0)

#define UART_WRITE16(offset, value)                                     \
            do {                                                        \
                if (offset < UART_PD_BASE)                              \
                    IO_WRITE16(RS232_BASE, (offset), (value));          \
                else                                                    \
                    IO_WRITE16(PDWNC_BASE, (offset), (value));          \
            } while (0)

#define UART_WRITE32(offset, value)                                     \
            do {                                                        \
                if (offset < UART_PD_BASE)                              \
                    IO_WRITE32(RS232_BASE, (offset), (value));          \
                else                                                    \
                    IO_WRITE32(PDWNC_BASE, (offset), (value));          \
            } while (0)

#define UART_SET_BIT(Reg, Bit)          UART_WRITE32(Reg, UART_READ32(Reg) | (Bit))
#define UART_CLR_BIT(Reg, Bit)          UART_WRITE32(Reg, UART_READ32(Reg) & (~(Bit)))


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

extern void UART_WRITE_DATA(UINT32 u4Port, UINT8 u1Data);

extern UINT8 UART_READ_DATA(UINT32 u4Port);

extern void UART_HwWaitTxBufClear(UINT32 u4Port);

extern void UART_HwEnableTxIntr(UINT32 u4Port);

extern void UART_HwDiableTxIntr(UINT32 u4Port);

extern void UART_HwInit(void);

extern void UART_HwSetParity(UINT32 u4Port, UINT32 u4Parity);

extern void UART_HwSetCustomBaudRate(UINT32 u4Port, UINT32 u4BaudRate);

extern void UART_HwSetBaudRate(UINT32 u4Port, UINT32 u4BaudRate);

extern void UART_HwSetStopBit(UINT32 u4Port, UINT32 u4StopBit);

extern void UART_HwSetDataLen(UINT32 u4Port, UINT32 u4DataLen);

extern void UART_HwGetBaudRate(UINT32 u4Port, RS_232_SETUP_INFO_T * prSetupInfo);

extern void UART_HwGetParity(UINT32 u4Port, RS_232_SETUP_INFO_T * prSetupInfo);

extern void UART_HwGetStopBit(UINT32 u4Port, RS_232_SETUP_INFO_T * prSetupInfo);

extern void UART_HwGetDataLen(UINT32 u4Port, RS_232_SETUP_INFO_T * prSetupInfo);

extern UINT32 UART_HwGetRxDataCnt(UINT32 u4Port);

extern UINT32 UART_HwGetTxEmptyCnt(UINT32 u4Port);

extern BOOL UART_HwWriteAllow(UINT32 u4Port);

extern BOOL UART_HwReadAllow(UINT32 u4Port);

extern void UART_HwClearFifo(UINT32 u4Port, UINT32 u4Option);

extern UINT32 UART_HwGetIntStatus(UINT32 u4Port);

extern void UART_HwClearIntStatus(UINT32 u4Port,UINT32 u4Data);

extern void UART_HwWrite(UINT32 u4Port, const UINT8 * pu1Data, UINT32 u4NumberToWrite);

extern void UART_HwModemCtr(BOOL fgEnable);

extern UINT32 UART_GetRxDMABuf(UINT32 u4Port, UINT8 *pBuffer, UINT32 u4NumToRead);

extern void UART_HwDmaWriteDram(UPTR u4SPTR, UINT32 u4Size, UINT32 u4Threshold, BOOL fgResume);

extern void UART_HwSavePara(void);

extern void UART_HwDmaWriteDisable(void);

extern void UART_HwDmaReadDisable(void);

extern void UART_HwDmaReadDram(UPTR u4SPTR, UINT32 u4Size, BOOL fgCircular);

extern BOOL UART_HwTransMode(void);

extern BOOL UART_HwCheckPortTransMode(UINT32 u4Port);

extern UINT32 _mt53xx_u5_INTEN_register_mask(UINT32 u4Reg);


#endif // UART_HW_DRVIF_H
