/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_dm.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file contains DM exported APIs  
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_DM_H_
#define _C_DM_H_

#include "u_common.h"
#include "u_handle.h"
#include "u_fm.h"
#include "u_dm.h"

/*!
 * @defgroup   groupMW_DM Device Manager
 * @ingroup     groupMW
 * @brief    This file contains DM exported APIs.
 * 
 * @see groupMW
 *
 * @{
 * */

/*------------------------------------------------------------------*/
/*! @brief  Register the event callback function.
 *  @param  [in] t_evt_cat  -   Event type to be listened to.
 *  @param  [in] pf_nfy     -   Callback function registered by event listener.
 *  @param  [in] pt_filtr   -   Event filter set by event listener.
 *  @param  [in] pv_tag     -   Proprietary information set by event listener. The same
 *                              information will be called back to event listener when 
 *                              event issued.
 *  @param  [out] ph_nfy    -   Handle for the event registration.  This handle is used
 *                              when event listener wants to remove the registration.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 *  @retval DMR_HANDLE          -   Handle acquisition error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_reg_nfy_fct(
    DM_EVT_CAT_T    t_evt_cat,
    x_dm_nfy_fct    pf_nfy,      
    DM_EVT_FILTR_T* pt_filtr,
    VOID*           pv_tag, 
    HANDLE_T*       ph_nfy);

/*------------------------------------------------------------------*/
/*! @brief  Unregister the event callback function.
 *  @param  [in]  h_nfy  -  Handle for the event registration that event listener removes.
 *  @param  [out] None.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_unreg_nfy_fct(
    HANDLE_T        h_nfy);

/*------------------------------------------------------------------*/
/*! @brief  Retrieve the information for a specific device.
 *  @param  [in]  h_dev  -  Handle for the target device.
 *  @param  [out] pt_inf -  Information data structure retrieved.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_get_device_info(
    HANDLE_T        h_dev, 
    DM_DEV_INF_T*   pt_inf);

/*------------------------------------------------------------------*/
/*! @brief  Enumerate devices that meet a specific condition.
 *  @param  [in]  pt_filtr      -   The filter data to specify the condition.
 *  @param  [in]  ui4_dev_ns    -   The array length of the pt_stat.
 *  @param  [out] pt_stat       -   Array of DM_DEV_STAT_T data structure allocated by caller.
 *  @param  [out] pui4_dev_ns   -   Number of returned elements in pt_stat.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_get_devices(
    DM_DEV_FILTR_T* pt_filtr, 
    DM_DEV_STAT_T*  pt_stat,
    UINT32*         pui4_dev_ns);

/*------------------------------------------------------------------*/
/*! @brief  Get the device that meet the specific condition.
 *  @param  [in]  pt_spec   -   The filter data to specify the condition.
 *  @param  [out] ph_dev    -   Pointer of the device handle.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_get_device(
    DM_DEV_SPEC_T*  pt_spec,
    HANDLE_T*       ph_dev);

/*------------------------------------------------------------------*/
/*! @brief  umount and remove device.
 *  @param  [in]  ps_dev_name  -   dev name.
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_umount_remove(CHAR * ps_dev_name);


/*------------------------------------------------------------------*/
/*! @brief  Retrieve auto mount setting for a specific device type.
 *  @param  [in] t_hw_type  -   The target device's hardware type.
 *  @param  [in] t_dev_type -   The target device's device type.
 *  @param  [out] None.
 *  @return Return a boolean value
 *  @retval TRUE    -   The device type is auto mount enabled.
 *  @retval FALSE   -   The device type is auto mount disabled.
 */
/*------------------------------------------------------------------*/
extern BOOL c_dm_query_automnt(
    DM_HW_TYPE_T    t_hw_type,
    DM_DEV_TYPE_T   t_dev_type);

/*------------------------------------------------------------------*/
/*! @brief  Get the usb device port by device name
 *  @param  [in] ps_dev_name -   The target device's device name.
 *  @param  [out] pu1i_changed_usb_port - The target device's usb port number.
 *  @return Return Error number.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_get_dev_port_by_name(
		const CHAR* ps_dev_name,
		UINT8 *pu1i_changed_usb_port);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_spindle_control(
    HANDLE_T                h_dev, 
    DM_DEV_IO_SPIN_CTRL_T   t_spin_ctrl,
    VOID*                   pv_tag,
    x_dm_ioc_fct            pf_ioc_fct,
    HANDLE_T*               ph_ioc);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_hdd_spindle_control(
    HANDLE_T        h_dev, 
    VOID*           pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_tray_control(
    HANDLE_T                h_dev, 
    DM_DEV_IO_TRAY_CTRL_T   t_tray_ctrl,
    void*                   pv_tag,
    x_dm_ioc_fct            pf_ioc_fct,
    HANDLE_T*               ph_ioc);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_dvd_struct(
    HANDLE_T                            h_dev,
    DM_DEV_IO_MED_READ_DVD_STRUCT_T*    pt_struct);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_kcd(
    HANDLE_T                    h_dev,
    DM_DEV_IO_MED_READ_KCD_T*   pt_kcd);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_com_report_key(
    HANDLE_T                    h_dev, 
    DM_DEV_IO_COM_REPORT_KEY_T* pt_key);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_com_send_key(
    HANDLE_T                    h_dev, 
    DM_DEV_IO_COM_SEND_KEY_T*   pt_key);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_com_get_feature(
    HANDLE_T                        h_dev, 
    DM_DEV_IO_COM_GET_FEATURE_T*    pt_feature);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_cd(
    HANDLE_T                    h_dev, 
    DM_DEV_MED_DISC_READ_CD_T*  pt_read_cd);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_cd_async(
    HANDLE_T                    h_dev, 
    DM_DEV_MED_DISC_READ_CD_T*  pt_read_cd,
    VOID*                       pv_tag,
    x_dm_ioc_fct                pf_ioc_fct,
    HANDLE_T*                   ph_ioc);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_cd_text(
    HANDLE_T                    h_dev, 
    DM_DEV_MED_READ_CD_TEXT_T*  pt_read_txt);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_disc_info(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_standby_timer(
    HANDLE_T    h_dev, 
    UINT32*     pui4_time);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_hdd_get_standby_timer(
    HANDLE_T    h_dev, 
    UINT32*     pui4_time);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_set_standby_timer(
    HANDLE_T    h_dev, 
    UINT32      ui4_time);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_hdd_set_standby_timer(
    HANDLE_T    h_dev, 
    UINT32      ui4_time);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_flush_cache(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_hdd_flush_cache(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_low_format(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_nand_low_format(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_disc_finalize(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_disc_unfinalize(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_disc_upgrade(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_set_io_speed(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_hw_reset(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_release_bus(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_test_unit_ready(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_event_status(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_mechanism_status(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_set_book_type(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_flashless_download(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_set_transfer_mode(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_buffer_capacity(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_do_opc(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_read_dvd_structure(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_vendor_inquiry(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_auto_tune(
    HANDLE_T    h_dev, 
    VOID*       pv_parm);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_med_nwa(
    HANDLE_T    h_dev, 
    UINT32*     pui4_nwa);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_med_toc_hdr(
    HANDLE_T                        h_dev, 
    DM_DEV_MED_DISC_TOC_HEADER_T*   pt_toc_hdr);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_med_toc_ent(
    HANDLE_T                        h_dev,
    DM_DEV_MED_DISC_TOC_ENTRY_T*    pt_toc_ent);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_sess_ns(
    HANDLE_T    h_dev,
    UINT32*     pui4_sess_ns);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_sess_inf(
    HANDLE_T                        h_dev,
    UINT32                          ui4_sess_no,
    DM_DEV_MED_DISC_SESS_INFO_T*    pt_inf);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_trk_ns(
    HANDLE_T    h_dev,
    UINT32*     pui4_trk_ns);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_trk_inf(
    HANDLE_T                        h_dev,
    UINT32                          ui4_trk_no,
    DM_DEV_MED_DISC_TRK_INFO_T*     pt_inf);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_sector_sz(
    HANDLE_T                              h_dev,
    UINT32                                ui4_lba,
    DM_DEV_MED_DISC_SECTOR_SIZE_TYPE_T*   pt_sz);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_last_sense_key(
    HANDLE_T    h_dev, 
    UINT32*     pui4_key);

/*------------------------------------------------------------------*/
/*! @brief  This API is not used in DTV platform.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_odd_get_current_speed(
    HANDLE_T    h_dev, 
    UINT32*     pui4_speed);


/*------------------------------------------------------------------*/
/*! @brief  Send IO request tp specific BT device.
 *  @param [in] h_dev       -   Handle of the BT device.
 *  @param [in] b_in_isr    -   Indicates if this API is called in ISR(Interrupt Service Routine)
 *  @param [in] pt_io_req   -   Pointer of the IO request.
 *  @param [out] None.
 *  @note  Zink CSR BT
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_bt_send_io_req(
    HANDLE_T                h_dev, 
    BOOL                    b_in_isr,
    DM_USB_BT_IO_REQ_T*     pt_io_req);
                          
/*------------------------------------------------------------------*/
/*! @brief  Retrieve how many requests can be sent in each transfer type.
 *  @param [in]  h_dev      -   Handle of the BT device.
 *  @param [out] pt_req_num -   Pointer of the DM_USB_BT_IO_REQ_NUM_T.
 *  @note  Zink CSR BT
 *  @return Return the error code.
 *  @retval DMR_OK              -   Success.
 *  @retval DMR_ERR             -   General error return.
 *  @retval DMR_INVALID_PARM    -   Invalid parameter data.
 *  @retval DMR_INTERNAL        -   Internal data structure error.
 *  @retval DMR_OS              -   OSAI error.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dm_bt_get_io_req_num(
    HANDLE_T                    h_dev, 
    DM_USB_BT_IO_REQ_NUM_T*     pt_req_num); 


extern INT32 c_dm_get_usb_port_number(
    UINT8*                      pui1_usb_port_num);


extern INT32 c_dm_iso_attach(
    const CHAR* ps_iso);


extern INT32 c_dm_iso_detach(
    const CHAR* ps_iso);

extern INT32 c_dm_get_device_serialNum(
    const CHAR* ps_devName,
    CHAR*       ps_serialNum);

extern INT32 c_dm_vold_nfy_handler(
    const CHAR* ps_mnt,
    const CHAR* ps_dev,
    UINT32      ui4_event);

extern INT32 c_dm_vold_set_do_cb(
    x_dm_vold_cb_func pf_nfy);

extern INT32 c_dm_get_dongle_index(
    const CHAR* ps_puid);

extern INT32 c_dm_dev_resend_notify(
    );

/*! @}  */

#endif
