/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_opera_hbbtv.h
 *  $RCSfile: $
 *  $Revision$
 *  $Date$
 *  $Author$
 *
 *  @par Description: This header file declares Opera HBBTV exported APIs.
 */
/*----------------------------------------------------------------------------*/
#ifndef _C_OPERA_HBBTV_H_
#define _C_OPERA_HBBTV_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "u_common.h"
#include "x_os.h"
#include "u_opera_hbbtv.h"
#include "u_opera_hbbtv_util.h"

/*!
 * @defgroup    groupMW_OPERA_HBBTV Opera HBBTV
 * @ingroup     groupMW
 * @brief    This header file declares Opera HBBTV exported APIs.
 *
 * @see groupMW
 *
 * @{
 * */

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
/*! @brief  Judge whether hbbtv is supported or not.
 *  @param [in]  None
 *  @param [out] None
 *  @return Return whether hbbtv is supported or not
 *  @retval True :  support.
 *  @retval False : not support
 */
/*------------------------------------------------------------------*/
extern BOOL c_opera_hbbtv_support( VOID );

/*------------------------------------------------------------------*/
/*! @brief  start MW HBBTV engine
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_INIT         -   Not init HBBTV engine.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_start( VOID );

/*------------------------------------------------------------------*/
/*! @brief  stop MW HBBTV engine
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_INIT         -   Not init HBBTV engine.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stop( VOID );

/*------------------------------------------------------------------*/
/*! @brief  exit broser process
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_INIT         -   Not init HBBTV engine.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_exit_process( VOID );

/*------------------------------------------------------------------*/
/*! @brief  create a new application
 *  @param [in]  s_uri - the uri of the application
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK  -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_create_app(
    BOOL                        b_brdcst_related,
    const CHAR*                 s_uri );

/*------------------------------------------------------------------*/
/*! @brief  init HBBTV engine's osd.
 *  @param [in]  h_canvas_hbbtv    -   handle of HbbTV canvas.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal HBBTV component error.
 *  @retval OPERA_HBBTVR_ALREADY_INIT     -   Already init.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_init_osd(
    HANDLE_T                    h_canvas_hbbtv );

/*------------------------------------------------------------------*/
/*! @brief  show HBBTV UI.
 *  @param [in]  b_get_focus    -   Indicates if try to get focus.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_RUNNING      -   HBBTV engine is not running
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_show( VOID );

/*------------------------------------------------------------------*/
/*! @brief  hide HBBTV UI.
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_RUNNING      -   HBBTV engine is not running
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_hide( VOID );

/*------------------------------------------------------------------*/
/*! @brief  set hbbtv wgl to focus.
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal error.
 *  @retval OPERA_HBBTVR_NOT_RUNNING      -   HBBTV engine is not running
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_wgl_set_focus( VOID );

/*------------------------------------------------------------------*/
/*! @brief  Set osd related notification
 *  @param [in]  pf_nfy      -   nofiny function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_osd_reg_nfy(
    x_opera_hbbtv_osd_nfy_fct       pf_nfy,
    VOID*                           pv_tag );

/*------------------------------------------------------------------*/
/*! @brief  Set osd show/hide notification
 *  @param [in]  pf_nfy      -   notify function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_osd_show_reg_nfy(
    x_opera_hbbtv_osd_show_nfy_fct      pf_nfy,
    VOID*                               pv_tag );

/*------------------------------------------------------------------*/
/*! @brief  Handle the user key press event.
 *  @param [in]  ui4_key_code    -   Pressed key code.
 *  @param [in]  ui4_msg         -   .
 *  @param [out] pb_processed    -   Indicates if the key is included in current function group.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK               -   Success.
 *  @retval OPERA_HBBTVR_INV_ARG          -   Invalid parameter.
 *  @retval OPERA_HBBTVR_INTERNAL_ERROR   -   Internal HBBTV component error.
 *  @retval OPERA_HBBTVR_NOT_RUNNING      -   HBBTV engine is not running
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_key_press(
    UINT32                      ui4_key_code,
    UINT32                      ui4_msg );

/*------------------------------------------------------------------*/
/*! @brief  Set Country ID
 *  @param [in]  s3166_country      -   Specific system country code.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_country (
    ISO_3166_COUNT_T            s3166_country );

/*------------------------------------------------------------------*/
/*! @brief  Set GUI language.
 *  @param [in]  s639_lang      -   Specific language.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_gui_lang (
    ISO_639_LANG_T              s639_lang );

/*------------------------------------------------------------------*/
/*! @brief  Set audio description status.
 *  @param [in]  b_ad_enabled      -   Specific ad status.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_audio_desc (
    BOOL                        b_ad_enabled );

/*------------------------------------------------------------------*/
/*! @brief  Set default audio language.
 *  @param [in]  s639_lang      -   Specific language.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_default_audio_lang (
    ISO_639_LANG_T              s639_lang );

/*------------------------------------------------------------------*/
/*! @brief  Set default subtitle language.
 *  @param [in]  s639_lang      -   Specific language.
 *  @param [in]  e_enable       -   Subtitel on/off  or mute_on status.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_default_subtitle_lang (
    ISO_639_LANG_T                      s639_lang,
    OPERA_HBBTV_SUBTITLE_ENABLE_T       e_enable);

/*------------------------------------------------------------------*/
/*! @brief  Set default subtitle language.
 *  @param [in]  attr       -   Specific subtitel attribute configuration, normal or hearing  impared.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_default_subtitle_attr (
    OPERA_HBBTV_SUBTITLE_ATTR_T         e_attr );

/*------------------------------------------------------------------*/
/*! @brief  Get prefer subtitle information
 *  @param [io]  pSubtitle - only .lang && .type && .attr are available
 *  @param [in]  nmemb     - count of items in pSubtitle
 *  @param [out] ntotal    - total of items
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK - Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_subtitle_get_prefer (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb,
    size_t *ntotal);

/*------------------------------------------------------------------*/
/*! @brief  Set prefer subtitle information
 *  @param [in]  pSubtitle - only .lang && .type && .attr are available
 *  @param [in]  nmemb     - count of items in pSubtitle
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK - Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_subtitle_set_prefer (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmenb);

/*------------------------------------------------------------------*/
/*! @brief  Get subtitle information list
 *  @param [io]  pSubtitle - point of items
 *  @param [in]  nmemb     - count of items for pSubtitle
 *  @param [out] ntotal    - total of items
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK - Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_subtitle_get_list (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb,
    size_t *ntotal);

/*------------------------------------------------------------------*/
/*! @brief  Set the active subtitle
 *  @param [in]  pSubtitle - only .index && .active are available
 *  @param [in]  nmemb     - count of items in pSubtitle
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK - Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_subtitle_set_active (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb);

/*------------------------------------------------------------------*/
/*! @brief  Get the number of Audio track when streaming is playing.
 *  @param [in]  None
 *  @param [out] pui2_num : number of Audio track
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK         -   Success.
           OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stm_get_audio_num( UINT16 *pui2_num );

/*------------------------------------------------------------------*/
/*! @brief  Get the audio language when streaming is playing.
 *  @param [in]  ui2_idx : Index of the audio track
 *  @param [out] s639_lang : Audo language in the format of ISO 639
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK          -   Success.
            OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stm_get_audio_lang( UINT16 ui2_idx, ISO_639_LANG_T s639_lang );

/*------------------------------------------------------------------*/
/*! @brief  Get the audio track when streaming is playing.
 *  @param [in]  None
 *  @param [out] pui2_idx : Index of the audio track
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK          -   Success.
            OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stm_get_audio( UINT16 *pui2_idx );

/*------------------------------------------------------------------*/
/*! @brief  Set the audio track when streaming is playing.
 *  @param [in]  ui2_idx : Index of the audio track
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK          -   Success.
            OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stm_set_audio( UINT16 ui2_idx );

/*------------------------------------------------------------------*/
/*! @brief  Change next audio.
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK   - Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_next_audio(VOID);

/*------------------------------------------------------------------*/
/*! @brief  Set visibility
 *  @param [in]  b_visible : visible or not.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK          -   Success.
            OPERA_HBBTVR_FAIL        -   Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_stm_set_visibility( BOOL b_visible );

/*------------------------------------------------------------------*/
/*! @brief  Set rating age.
 *  @param [in]  ui8_rating_age -   Specific ad status.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_rating_age (
    UINT8                             ui8_rating_age );
/*------------------------------------------------------------------*/
/*! @brief  Set privacy policy.
 *  @param [in]  ui2_privacy_type - privacy type.
 *  @param [in]  b_enable         - enable/disable.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_privacy(
    OPERA_HBBTV_PRIVACY_TYPE_T ui2_privacy_type,
    OPERA_HBBTV_PRIVACY_E privacy_cfg);

/*------------------------------------------------------------------*/
/*! @brief  set the device id configuration.
 *  @param [in]  e_dev_id_cfg - device id config
 *  @param [out] HBBTVR_* value
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_device_id_cfg(OPERA_HBBTV_DEVICEID_CFG_E e_dev_id_cfg);

/*------------------------------------------------------------------*/
/*! @brief  Set drm priority array.
 *  @param [in]  pt_drm_prio_arr -   Specific drm priority array.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK            -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_set_drm_priority_array (
    OPERA_HBBTV_DRM_PRIO_ARR_T*     pt_drm_prio_arr );

/*------------------------------------------------------------------*/
/*! @brief  Clear hbbtv cookie.
 *  @param [in]  None
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK            -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_clear_cookie ();

/*------------------------------------------------------------------*/
/*! @brief  go to the speficied URL notification
 *  @param [in]  pf_nfy      -   notify function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_browser_goto_url_reg_nfy(
    x_opera_hbbtv_layers_goto_url     pf_nfy,
    VOID*                             pv_tag);

/*------------------------------------------------------------------*/
/*! @brief  create the specified application notification.
 *  @param [in]  pf_nfy      -   notify function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_create_app_reg_nfy(
    x_opera_hbbtv_layers_create_app     pf_nfy,
    VOID*                               pv_tag);

/*------------------------------------------------------------------*/
/*! @brief  Key press notification
 *  @param [in]  pf_nfy      -   notify function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_input_keypress_reg_nfy(
    x_opera_hbbtv_layers_input_keypress pf_nfy,
    VOID*                               pv_tag);

/*------------------------------------------------------------------*/
/*! @brief  browser communicate notification
 *  @param [in]  pf_nfy      -   notify function.
 *  @param [in]  pv_tag      -   tag.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_reg_browser_com_nfy(
    x_opera_hbbtv_browser_com_nfy_fct   pf_nfy,
    VOID*                               pv_tag);

/*------------------------------------------------------------------*/
/*! @brief  release hbbtv resource.
 *  @param [in]  None
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK    -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_release_res( VOID );

/*------------------------------------------------------------------*/
/*! @brief  Get Hbbtv mode.
 *  @param [in]   None.
 *  @param [out]  pui4_hbbtv_mode_mask  -  hbbtv mode mask.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_get_mode(
    UINT32* pui4_hbbtv_mode_mask );

/*------------------------------------------------------------------*/
/*! @brief  Set Hbbtv mode.
 *  @param [in]  ui4_hbbtv_mode_mask  -  hbbtv mode mask.
 *  @param [out] None
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_set_mode(
    UINT32 ui4_hbbtv_mode_mask );

/*------------------------------------------------------------------*/
/*! @brief  query mime type whether support.
 *  @param [in]  s_mime_type  -  mime type.
 *  @param [out] pb_supported - whether support the mime type.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_get_mime_type_support(
    const CHAR*                 s_mime_type,
    BOOL*                       pb_supported );

/*------------------------------------------------------------------*/
/*! @brief  set the service type of hbbtv application.
 *  @param [in]  e_app_svc  -  the service type of hbbtv application.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_OPERA_HBBTVR_OK -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_set_app_svc_type(
    HBBTV_APP_SVC_TYPE_T        e_app_svc );

/*------------------------------------------------------------------*/
/*! @brief  get the service type of hbbtv application.
 *  @param [in]  None.
 *  @param [out] pe_app_svc  -  the service type of hbbtv application.
 *  @return Return the error code.
 *  @retval OPERA_OPERA_HBBTVR_OK  -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_get_app_svc_type(
    HBBTV_APP_SVC_TYPE_T*       pe_app_svc );

/*------------------------------------------------------------------*/
/*! @brief  stop 3rd app (eg. BC).
 *  @param [in]  None.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_OPERA_HBBTVR_OK  -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_stop_3rd_app(VOID);


/*------------------------------------------------------------------*/
/*! @brief  launch hbbtv application.
 *  @param [in]  pt_hbbtv_app_info  -  the info of hbbtv app which will be launched.
 *  @param [in]  pf_launch_status_nfy  -  status notify function.
 *  @param [in]  pv_tag             -  tag value.
 *  @param [out] ph_handle          - whether support the mime type.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_launch_app(
    HBBTV_APP_INFO_T*               pt_hbbtv_app_info,
    x_hbbtv_app_event_nfy_fct       pf_launch_status_nfy,
    VOID*                           pv_tag );

/*------------------------------------------------------------------*/
/*! @brief  kill hbbtv application.
 *  @param [in]  pt_app_kill_info  -  the info of hbbtv app which will be killed.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_kill_app(
    HBBTV_APP_KILL_INFO_T*          pt_app_kill_info);

/*------------------------------------------------------------------*/
/*! @brief  get applicatou source type.
 *  @param [in]  None.
 *  @param [out] pe_app_src_type  - application source type.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_get_app_type(
    HBBTV_APP_SRC_TYPE_T*           pe_app_src_type );

/*------------------------------------------------------------------*/
/*! @brief  register application status listener function.
 *  @param [in]  pf_nfy_fct  - notify function.
 *  @param [in]  pf_nfy_fct  - notify function tag value.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_reg_app_event_listener(
        x_hbbtv_app_event_nfy_fct   pf_nfy_fct,
        VOID*                       pv_nfy_tag);

/*------------------------------------------------------------------*/
/*! @brief  unregister application status listener function.
 *  @param [in]  pf_nfy_fct  - notify function.
 *  @param [in]  pf_nfy_fct  - notify function tag value.
 *  @param [out] None.
 *  @return Return the error code.
 *  @retval OPERA_HBBTVR_OK           -   Success.
 */
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_unreg_app_event_listener(
        x_hbbtv_app_event_nfy_fct   pf_nfy_fct,
        VOID*                       pv_nfy_tag);

/*------------------------------------------------------------------*/
/*! @brief  Get service abitrator
 *  @param [in]  None
 *  @param [out]
 *  @return
 *  @retval OPERA_HBBTVR_OK           -   Success.
                 OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/

extern INT32 c_opera_hbbtv_res_get_svc_abitrator(
            OPERA_HBBTV_LOCK_OWNER           t_owner,
            OPERA_HBBTV_RES_TYPE              t_res_type,
            SEMA_OPTION_T                     t_option,
            opera_hbbtv_res_client_cb_fct     pf_client_cb_fct,
            VOID*                             pv_tag);

/*------------------------------------------------------------------*/
/*! @brief  Put service abitrator
 *  @param [in]  None
 *  @param [out]
 *  @return
 *  @retval OPERA_HBBTVR_OK           -   Success.
                 OPERA_HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/
extern INT32 c_opera_hbbtv_res_put_svc_abitrator(
            OPERA_HBBTV_LOCK_OWNER            t_owner,
            OPERA_HBBTV_RES_TYPE              t_res_type);
/*------------------------------------------------------------------*/
/*! @brief  c_hbbtv_app_status_nfy, transter hbbtv msg to 3rd.
 *  @param [in]  i4_msg_type
 *  @param [in] pv_data
 *  @return Return the error code.
 *  @retval HBBTVR_OK           -   Success.
                 HBBTVR_FAIL        - Fail. It may be due to streaming is not playing.
 */
/*------------------------------------------------------------------*/

extern INT32 c_hbbtv_app_status_nfy(
                                INT32    i4_msg_type,
                                VOID*    pv_data,
                                SIZE_T   z_data_len );
extern INT32 c_hbbtv_set_browser_status(HBBTV_BRWS_STATUS_E e_status);

/*------------------------------------------------------------------*/
/*! @brief  c_hbbtv_fvp_reset, reset the parameters for FVP.
 *  @param [in]  timesec
 *  @return Return the error code.
 *  @retval HBBTVR_OK           - Success.
 *           HBBTVR_FAIL        - Fail
*/
/*------------------------------------------------------------------*/
extern INT32 c_hbbtv_fvp_reset(unsigned long long timesec);
/*! @}  */

#endif /* _C_OPERA_HBBTV_H_ */
