/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_txt.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file exports all txt library API and constant for 
 *         other module using.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_TXT
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_TXT_H_
#define _C_TXT_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_handle.h"
#include "u_txt.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* This structure enumerates all event types for the callback function (*x_img_nfy_fct). */
#ifdef __cplusplus
extern "C" {
#endif


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------*/

/*! @brief  c_txt_create 
 *              Create txt instance handle.
 *  @param [in] &ph_txt 
 *  @param [out] Txt instance handle
 *  
 *  @Return Return the error code.
 *  @retval TXT_OK      Success. The API execution is successful. 
 *  @retval TXT_ERR     FAIL
 */
/*------------------------------------------------------------------*/

extern INT32 c_txt_open (const CHAR*         buffer,
                            const UINT32        ui4_buffer_size,
                            UINT8               ui1_open_mode,  
                            c_txt_nfy_fct       pf_nfy,         
                            const VOID*         pv_tag,         
                            HANDLE_T*           ph_txt,
                            TXT_SRC_TYPE_T      e_type);


/*! @brief  c_txt_free 
 *              Create txt instance handle.
 *  @param [in] ph_txt 
 *  @param [out] Free txt instance.
 *  
 *  @Return Return the error code.
 *  @retval TXT_OK      Success. The API execution is successful. 
 *  @retval TXT_ERR     FAIL
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_close(HANDLE_T h_txt);



extern INT32 c_txt_set_attr(HANDLE_T h_txt,
                              UINT32 ui4_cmd_num,
                              TXT_SET_CMD_T* pt_cmd);

/*! @brief  c_txt_get_code
 *              get txt code   
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [out] null
 *  @return Return txt code.
 *  @retval TXT_CODE_UTF8                    
 *  @retval TXT_CODE_UTF8BE    
 *  @retval TXT_CODE_UNICODE 
 *  @retval TXT_CODE_UNICODEBE 
 *  @retval TXT_CODE_UTF32 
 *  @retval TXT_CODE_UTF32BE 
 *  @retval UNKNOWN        Not known the txt code.
 */
/*------------------------------------------------------------------*/

extern INT32 c_txt_get_code_type(void*                 pv_buffer, 
                               UINT32                ui4_buffer_length, 
                               TXT_SRC_TYPE_T        e_src_type, 
                               TXT_ECODE_TYPE_T*      pe_code_type);


extern INT32 c_txt_get_code(HANDLE_T h_txt, 
                            TXT_ECODE_TYPE_T* pe_code_type);


/*! @brief  c_txt_scan
 *              Scan txt file,get txt total page and save each page info.         
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [in] ui4_scan_cnt       scan page count.if the value is 0xffffffff,scan all file.
 *  @param [out] scaned page number(call back).
 *
 *  @Return void.
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_scan(HANDLE_T h_txt, UINT32 ui4_scan_cnt,
                  c_txt_nfy_fct pf_nfy,const VOID* pv_tag);


/*! @brief  c_txt_cancel_scan 
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [out] null
 *
 *  @Return void
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_cancel_scan(HANDLE_T h_txt);

/*! @brief  c_txt_get_page_data
 *            (1)Get special page data form page list,and load data to row list.
 *            (2)Async method
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [in] ui4_goto_num    Go to page number.
 *  @param [in] pf_nfy  callback func
 *  @param [out] (1)Page data (2)Call back to AP
 *
 *  @Return void.
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_get_page_data( HANDLE_T h_txt,UINT32 ui4_goto_num,
                          c_txt_nfy_fct pf_nfy,const VOID* pv_tag );


/*! @brief  c_txt_scroll_up_row
 *            Get next row data(compare current displaying page). 
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [out] Next row data
 *
 *  @Return Return the error code.
 *  @retval TXT_OK      Success. The API execution is successful. 
 *  @retval TXT_ERR     FAIL
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_scroll_up_row(HANDLE_T h_txt, UINT16 ui2_scroll_num);

/*! @brief  c_txt_scroll_down_row
 *            Get previous row data(compare current displaying page). 
 *  @param [in] h_txt   Current text instance handle. 
 *  @param [out] Previous row data
 *
 *  @Return Return the error code.
 *  @retval TXT_OK      Success. The API execution is successful. 
 *  @retval TXT_ERR     FAIL
 */
/*------------------------------------------------------------------*/
extern INT32 c_txt_scroll_down_row(HANDLE_T h_txt, UINT16 ui2_scroll_num);


extern INT32 c_txt_is_support_ecode(TXT_ECODE_TYPE_T e_code_type);

extern INT32 c_txt_get_file_size(HANDLE_T h_txt, UINT32* pui4_file_size);

extern INT32 c_txt_get_page_rows(HANDLE_T h_txt, UINT16* pui2_rows);

extern INT32 c_txt_get_last_page_rows(HANDLE_T h_txt, UINT16* pui2_rows);

extern INT32 c_txt_get_start_line(HANDLE_T h_txt, UINT32* pui4_page_start_line_idx);
extern INT32 c_txt_pause(HANDLE_T h_txt);

extern INT32 c_txt_resume(HANDLE_T h_txt);

extern INT32 c_txt_style_change(HANDLE_T h_txt, 
                              UINT32* pui4_current_display_page, 
                              UINT32*  pui4_total_page);

  
#ifdef __cplusplus
}
#endif

#endif /* _C_TXT_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_TXT*/
/*----------------------------------------------------------------------------*/
