/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_df_mngr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Download Function Manager specific 
 *      definitions, which are exported.
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_DFM_H_
#define _U_DFM_H_   


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_handle.h"
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_DF_MNGR Download Function Manager
 *
 *  @ingroup groupMW
 *  @brief The ISDB logo data could be loaded from this module.
 *
 *  %MW Application API would be used by Application, lot of these APIs would
 *   be wrapped from Exported API. Using these APIs, application could load ISDB 
 *   logo data from the download function manager.
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
 #define SIMP_LOGO_LEN     ((UINT8) 0x5)    /**< Define the length of simple logo. */


/* Notify conditions. Note that this enumeration may only contain */
/* entries with a value '0' to '63' (inclusive). All other values */
/* are reserved for internal notify conditions.                   */
/*------------------------------------------------------------------*/
/*! @enum DFM_COND_T
 *  @brief The condition of loading logo data in Download Function Manager.
 *  @code
 *   typedef enum{
 *      DFM_COND_UNAVAILABLE = 0,
 *      DFM_COND_TRANSITION,
 *      DFM_COND_AVAILABLE,
 *      DFM_COND_UPDATE
 *   }  DFM_COND_T;
 *  @endcode
 *  @li@c  DFM_COND_UNAVAILABLE                              - The logo data is unavailable .
 *  @li@c  DFM_COND_TRANSITION                               - The logo data is transition.
 *  @li@c  DFM_COND_AVAILABLE                                - The logo data is available.
 *  @li@c  DFM_COND_UPDATE                                   - The logo data is updated by transport stream.
 */
/*------------------------------------------------------------------*/
 typedef enum{
 
    DFM_COND_UNAVAILABLE = 0,
    DFM_COND_TRANSITION,
    DFM_COND_AVAILABLE,
    DFM_COND_UPDATE
 }  DFM_COND_T;

typedef UINT8      DFM_LOGO_TYPE_T;    /**<  Define the logo type */

#define DFM_LOGO_TYPE_SD4_3_SMAlL       ((DFM_LOGO_TYPE_T) 0x01)    /**<  The PNG logo mark size pattern is SD4:3 small. */
#define DFM_LOGO_TYPE_SD16_9_SMALL      ((DFM_LOGO_TYPE_T) 0x02)    /**<  The PNG logo mark size pattern is SD16:9 small. */
#define DFM_LOGO_TYPE_HD_SMALL          ((DFM_LOGO_TYPE_T) 0x04)    /**<  The PNG logo mark size pattern is HD small. */
#define DFM_LOGO_TYPE_SD4_3_LARGE       ((DFM_LOGO_TYPE_T) 0x08)    /**<  The PNG logo mark size pattern is SD4:3 large. */
#define DFM_LOGO_TYPE_SD16_9_LARGE      ((DFM_LOGO_TYPE_T) 0x10)    /**<  The PNG logo mark size pattern is SD16:9 large. */
#define DFM_LOGO_TYPE_HD_LARGE          ((DFM_LOGO_TYPE_T) 0x20)    /**<  The PNG logo mark size pattern is HD large. */
#define DFM_LOGO_TYPE_SIMP              ((DFM_LOGO_TYPE_T) 0x40)    /**<  The simple logo type. */
#define DFM_LOGO_TYPE_UNKNOWN           ((DFM_LOGO_TYPE_T) 0x80)    /**<  Unknown type. */

 
/* Notify function. */
/*------------------------------------------------------------------*/
/*! @brief The callback function of Download Function Manager.
 *  @param [in] h_logo The handle for loading logo data.
 *  @param [in] e_nfy_cond The condition of loading logo data.
 *  @param [in] pv_nfy_tag The private tag for loading logo data.
 *  @param [in] ui4_data The reserved data for loading logo data, it is used for notified logo types.
 *  @param [out] None.
 *  @return
 *  @retval      - None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_dfm_nfy_fct)(HANDLE_T       h_logo,
                           DFM_COND_T     e_nfy_cond,
                           VOID*          pv_nfy_tag,
                           UINT32         ui4_data);


/* Return values. */
#define DFMR_NO_DATA               ((INT32)   1)    /**<  No logo data.*/
#define DFMR_OK                    ((INT32)   0)    /**<  Routine successful. */
#define DFMR_NOT_INIT              ((INT32)  -1)    /**<  The Download Function Manager is not initialized. */
#define DFMR_ALREADY_INIT          ((INT32)  -2)    /**<  The Download Function Manager already initialized. */
#define DFMR_INV_ARG               ((INT32)  -3)    /**<  One or more of the arguments contain invalid data.*/
#define DFMR_INV_HANDLE            ((INT32)  -4)    /**<  One invalid handle in argument. */
#define DFMR_OUT_OF_HANDLES        ((INT32)  -5)    /**<  No handle available. */
#define DFMR_INV_NAME              ((INT32)  -6)    /**<  One invalid name in argument. */
#define DFMR_UNKNOWN_NAME          ((INT32)  -7)    /**<  Unknown name is used. */
#define DFMR_OUT_OF_RESOURCES      ((INT32)  -8)    /**<  No resource available. */
#define DFMR_NO_SUPPORT            ((INT32)  -9)    /**<  The Download Function Manager is not supportted*/
#define DFMR_INV_HANDLE_STATE      ((INT32) -10)    /**<  The state of handle is invalid. */
#define DFMR_NOT_ENOUGH_SPACE      ((INT32) -11)    /**<  No enough space is available. */
#define DFMR_INV_LOAD_INFO         ((INT32) -12)    /**<  The loading information is invalid. */
#define DFMR_INV_ENG_HANDLE        ((INT32) -13)    /**<  The handle of Download Function Manager is invalid.*/
#define DFMR_FAIL                  ((INT32) -14)    /**<  It fail in current operation. */
#define DFMR_DUPLICATE_REC         ((INT32) -15)    /**<  This is a duplicate logo record.*/
#define DFMR_INV_COND              ((INT32) -16)    /**<  The current condition is valid in this opteration. */
#define DFMR_INTERNAL_ERROR        ((INT32) -17)    /**<  Internal error.*/
#define DFMR_AEE_OUT_OF_RESOURCES  ((INT32) -18)    /**<  No resource available in AEE.*/
#define DFMR_AEE_NO_RIGHTS         ((INT32) -19)    /**<  No right do this operation.*/


#endif /* _U_DFM_H */

/*----------------------------------------------------------------------------*/
/*! @} */                                           /* end of groupMW_DF_MNGR */
/*----------------------------------------------------------------------------*/


