/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_wgl_obj_type.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains public interfaces, constant and type 
 *         definitions of the Widget Library.
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_WGL
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
 
#ifndef _U_WGL_OBJ_TYPE_H_
#define _U_WGL_OBJ_TYPE_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_wgl.h"
#include "u_wgl_tools.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/*Content Type Definition*/
#define WGL_CONTENT_FRAME_DEF               WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_FRAME, 1)    /**<Frame widget's default content type used in c_wgl_create_widget()         */
#define WGL_CONTENT_ICON_DEF                WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_ICON, 1)    /**<Icon widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_TEXT_DEF                WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_TEXT, 1)    /**< Text widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_HYPER_TEXT_DEF          WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_HYPER_TEXT, 1)    /**<  Hyper Text widget's default content type used in c_wgl_create_widget()          */
#define WGL_CONTENT_BUTTON_DEF              WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_BUTTON, 1)    /**< Button widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_EDIT_BOX_DEF            WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_EDIT_BOX, 1)    /**<Edit widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_PROGRESS_BAR_DEF        WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_PROGRESS_BAR, 1)    /**<   Progress widget's default content type used in c_wgl_create_widget()         */
#define WGL_CONTENT_CC_DEF                  WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_CC, 1)    /**<CC  widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_LIST_BOX_DEF            WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_LIST_BOX, 1)    /**< List Box widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_LIST_BOX_HORIZ          WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_EDIT_BOX, 2)    /**< Edit widget's content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_TKTP_DEF                WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_TKTP, 1)    /**<Ticker Tape widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_HTS_DEF                 WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_HTS, 1)    /**<Hyper Text widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_SBTL_DEF                WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_SBTL, 1)    /**<Subtitle widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_TTX_DEF                 WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_TTX, 1)    /**<Teletext widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_ANIM_DEF                WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_ANIM, 1)    /**< Animation widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_GROUP_DEF               WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_GROUP, 1)    /**< Group widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_CC_ISDB_DEF             WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_CC_ISDB, 1)    /**<CC ISDB widget's default content type used in c_wgl_create_widget()            */
#define WGL_CONTENT_IMGCF_DEF               WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_IMGCF, 1)    /**< Text widget's default content type used in c_wgl_create_widget()           */
#define WGL_CONTENT_A3_CC_DEF                  WGL_MAKE_CONTENT_ID(HT_WGL_WIDGET_A3_CC, 1)    /**<A3 CC  widget's default content type used in c_wgl_create_widget()  */   

/*Border Type Definition*/
#define WGL_BORDER_BASE                     WGL_MAKE_BORDER_ID(0)    /**<        */
#define WGL_BORDER_ID_FIRST                 WGL_MAKE_BORDER_ID(1)    /**<        */

/*-----------------------*/
#define WGL_BORDER_NULL                     WGL_MAKE_BORDER_ID(1)    /**< Widget's border type is NULL Border  used in c_wgl_create_widget()     */
#define WGL_BORDER_TRANSP                   WGL_MAKE_BORDER_ID(2)    /**< Widget's border type is Transparent Border used in c_wgl_create_widget()      */
#define WGL_BORDER_UNIFORM                  WGL_MAKE_BORDER_ID(3)    /**<Widget's border type is Uniform Border used in c_wgl_create_widget()       */
#define WGL_BORDER_RAISED                   WGL_MAKE_BORDER_ID(4)    /**< Widget's border type is Raised Border  used in c_wgl_create_widget()     */
#define WGL_BORDER_DEPRESSED                WGL_MAKE_BORDER_ID(5)    /**< Widget's border type is Depressed Border  used in c_wgl_create_widget()    */
#define WGL_BORDER_DROP_SHADOW_LEFT         WGL_MAKE_BORDER_ID(6)    /**<Widget's border type is Drop Shadow Left Border used in c_wgl_create_widget()       */
#define WGL_BORDER_DROP_SHADOW_RIGHT        WGL_MAKE_BORDER_ID(7)    /**< Widget's border type is Drop Shadow Right Border used in c_wgl_create_widget()      */
#define WGL_BORDER_TIMG                     WGL_MAKE_BORDER_ID(8)    /**<Widget's border type is Timg Border used in c_wgl_create_widget()       */
/*-----------------------*/

#define WGL_BORDER_ID_LAST                  WGL_MAKE_BORDER_ID(8)    /**<        */
/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/



#ifdef __cplusplus
}
#endif

#endif

/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_WGL*/
/*----------------------------------------------------------------------------*/


