/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_atv_mngr.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/7 $
 * $SWAuthor: Albert Liu $
 * $MD5HEX: fca6199f7c434f05d3b618b3e53cdaa9 $
 *
 * Description: 
 *         This header file contains ATV Manager exported constants, macros,
 *         types, and functions.
 * History:
 *           
 *---------------------------------------------------------------------------*/

#ifndef _X_ATV_MNGR_H_
#define _X_ATV_MNGR_H_


/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "x_handle.h" 
#include "x_common.h"
#include "u_sys_name.h"
#include "u_atv_mngr.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Data length */
typedef enum
{
    ATV_RS_232_DATA_LEN_4 = 0,
    ATV_RS_232_DATA_LEN_5,
    ATV_RS_232_DATA_LEN_6,
    ATV_RS_232_DATA_LEN_7,
    ATV_RS_232_DATA_LEN_8,
    ATV_RS_232_DATA_LEN_9
}   ATV_RS_232_DATA_LEN_T;

/* Parity */
typedef enum
{
    ATV_RS_232_PARITY_NONE = 0,
    ATV_RS_232_PARITY_EVEN,
    ATV_RS_232_PARITY_ODD,
    ATV_RS_232_PARITY_MARK,
    ATV_RS_232_PARITY_SPACE
}   ATV_RS_232_PARITY_T;

/* Data speed */
typedef enum
{
    ATV_RS_232_SPEED_75 = 0,
    ATV_RS_232_SPEED_110,
    ATV_RS_232_SPEED_134,
    ATV_RS_232_SPEED_150,
    ATV_RS_232_SPEED_300,
    ATV_RS_232_SPEED_600,
    ATV_RS_232_SPEED_1200,
    ATV_RS_232_SPEED_1800,
    ATV_RS_232_SPEED_2400,
    ATV_RS_232_SPEED_4800,
    ATV_RS_232_SPEED_9600,
    ATV_RS_232_SPEED_14400,
    ATV_RS_232_SPEED_19200,
    ATV_RS_232_SPEED_38400,
    ATV_RS_232_SPEED_57600,
    ATV_RS_232_SPEED_115200,
    ATV_RS_232_SPEED_128000,
    ATV_RS_232_SPEED_921600,
    ATV_RS_232_SPEED_230400,
    ATV_RS_232_SPEED_460800
}   ATV_RS_232_SPEED_T;

/* Stop bit */
typedef enum
{
    ATV_RS_232_STOP_BIT_1 = 0,
    ATV_RS_232_STOP_BIT_1_5,
    ATV_RS_232_STOP_BIT_2
}   ATV_RS_232_STOP_BIT_T;


/* Setup structure */
typedef struct _ATV_RS_232_SETUP_INFO_T
{
    ATV_RS_232_SPEED_T     e_speed;
    ATV_RS_232_DATA_LEN_T  e_data_len;
    ATV_RS_232_PARITY_T    e_parity;
    ATV_RS_232_STOP_BIT_T  e_stop_bit;
}   ATV_RS_232_SETUP_INFO_T;


typedef struct _X_ATV_MANAGER_CONFIG_T
{
    UINT8               ui1_version;
    
    /* Setting of dispatcher */
    UINT8               ui1_noresp_req_handler_max_cnt;
    UINT8               ui1_withresp_req_handler_max_cnt;
    UINT8               ui1_resp_handler_max_cnt;

    /* Setting of RTX manager */
    THREAD_DESCR_T      t_rtx_thread_info;
    UINT8               ui1_tx_delay_time;
    UINT8               ui1_tx_delay_byte_cnt;
    
    /* Setting of Flow Manager */
    UINT8               ui1_max_reliable_msg_cnt;
    UINT8               ui1_noresp_req_buf_cnt;
    UINT8               ui1_withresp_req_buf_cnt;

    CHAR                s_hardware_name[SYS_NAME_LEN + 1];
    ATV_RS_232_SETUP_INFO_T t_rs_232_setup_info;
} X_ATV_MANAGER_CONFIG_T;


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
extern BOOL x_atv_manager_is_init(VOID);

extern BOOL x_atv_manager_is_connect(VOID);

extern INT32 x_atv_manager_open(VOID);

extern INT32 x_atv_manager_send_noresp_req(
    UINT8   ui1_inmsg_type,
    UINT16  ui2_inmsg_body_len,
    UINT8*  pui1_inmsg_body);

extern INT32 x_atv_manager_send_withresp_req(
    HANDLE_T    h_handle,
    UINT8       ui1_send_retry_cnt,
    UINT16      ui2_send_timeout,
    UINT8       ui1_inmsg_type,
    UINT16      ui2_inmsg_body_len,
    UINT8*      pui1_inmsg_body,
    UINT8*      pui1_inmsg_id);

extern INT32 x_atv_manager_send_resp(
    UINT8   ui1_inmsg_id,
    UINT8   ui1_inmsg_type,
    UINT16  ui2_inmsg_body_len,
    UINT8*  pui1_inmsg_body);

extern INT32 x_atv_manager_reg_noresp_req_handler(
    UINT32*                     pui4_inmsg_type_table,
    x_atv_msg_handler_nfy_fct   pf_atv_msg_handler,
    VOID *  pv_tag,
    HANDLE_T *  ph_handle);

extern INT32 x_atv_manager_reg_withresp_req_handler(
    UINT32*                     pui4_inmsg_type_table,
    x_atv_msg_handler_nfy_fct   pf_atv_msg_handler,
    VOID*                       pv_tag,
    HANDLE_T*                   ph_handle);

extern INT32 x_atv_manager_reg_resp_handler(
    UINT32*                     pui4_inmsg_type_table,
    x_atv_msg_handler_nfy_fct   pf_atv_msg_handler,
    x_atv_fail_handler_nfy_fct  pf_atv_fail_handler,
    VOID*                       pv_tag,
    HANDLE_T*                   ph_handle);


extern INT32 x_atv_manager_unreg_handler(
    HANDLE_T                    h_handle);


#endif /* _X_ATV_MNGR_H_ */
