/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_chr_dev.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains character device related definitions.
 *---------------------------------------------------------------------------*/

#ifndef _X_CHR_DEV_H_
#define _X_CHR_DEV_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "x_common.h"

/*-----------------------------------------------------------------------------
 *  Constants and type definitions
 *---------------------------------------------------------------------------*/

typedef enum
{
    CHRDEV_COND_FAIL     = -1,
    CHRDEV_COND_READ_OK  = 1,
    CHRDEV_COND_WRITE_OK = 2,
    CHRDEV_COND_ERASE_OK = 3
} CHRDEV_COND_T;


/*-----------------------------------------------------------------------------
 * Notify function
 *---------------------------------------------------------------------------*/

typedef VOID (*x_chrdev_nfy_fct)(VOID          *pv_nfy_tag,
                                 CHRDEV_COND_T e_nfy_cond,
                                 VOID          *pv_data);

typedef struct _CHRDEV_NFY_INFO_T
{
    VOID             *pv_tag;
    x_chrdev_nfy_fct  pf_chrdev_nfy;
} CHRDEV_NFY_INFO_T;


/*-----------------------------------------------------------------------------
 * Parameter definitions of get/set types
 *---------------------------------------------------------------------------*/

typedef struct _CHRDEV_LONG_DATA_INFO_T
{
    UINT32            ui4_offset;
    UINT32            ui4_len;
    VOID              *pv_data;
    UINT32            ui4_handle;         /* for abort function, default is 0x00 */
    CHRDEV_NFY_INFO_T t_nfy_info;         /* for individual notification */
} CHRDEV_LONG_DATA_INFO_T;

typedef struct _CHRDEV_SHORT_DATA_INFO_T
{
    UINT32            ui4_offset;
    UINT8             ui1_data;
    UINT32            ui4_handle;         /* for abort function, default is 0x00 */
    CHRDEV_NFY_INFO_T t_nfy_info;         /* for individual notification */
} CHRDEV_SHORT_DATA_INFO_T;

typedef struct _CHRDEV_SECTOR_RANGE_INFO_T
{
    UINT32            ui4_start_sector;
    UINT32            ui4_number_of_sectors;
    UINT32            ui4_handle;         /* for abort function, default is 0x00 */
    CHRDEV_NFY_INFO_T t_nfy_info;         /* for individual notification */
} CHRDEV_SECTOR_RANGE_INFO_T;

typedef struct _CHRDEV_SECTOR_SEG_INFO_T
{
    UINT32 ui4_num_sectors;
    UINT32 ui4_sector_size;
} CHRDEV_SECTOR_SEG_INFO_T;

typedef struct _CHRDEV_SECTOR_TBL_INFO_T
{
    UINT32 ui4_num_entries;
    CHRDEV_SECTOR_SEG_INFO_T *pt_sector_info;
} CHRDEV_SECTOR_TBL_INFO_T;

/* Parameter data structure for I/O control */
typedef struct _CHRDEV_IO_CTRL_T {
    UINT32            u4IoctrlType;       /* I/O control types */
    void              *pvIoData;          /* I/O parameter data pointer */
    UINT32            u4IoDataSz;         /* I/O parameter data size */
    UINT32            ui4_handle;         /* for abort function, default is 0x00 */
    CHRDEV_NFY_INFO_T t_nfy_info;         /* for individual notification */
} CHRDEV_IO_CTRL_T;

/* Parameter data structure for abort */
typedef struct _CHRDEV_ABORT_T {
    UINT32            u4Handle;           /* handle of I/O data */
    BOOL              fgEnable;           /* enable or disable driver abort state */
} CHRDEV_ABORT_T;

/*-----------------------------------------------------------------------------
 * IO control types
 *---------------------------------------------------------------------------*/
#define CHR_DEV_IO_CTRL_TYPE_ENABLE_WP ((UINT32) 0x200)

/*-----------------------------------------------------------------------------
 * Get types
 *---------------------------------------------------------------------------*/
#define CHRDEV_GET_TYPE_READ_BYTE \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define CHRDEV_GET_TYPE_SYNC_READ \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))
#define CHRDEV_GET_TYPE_SIZE \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))
#define CHRDEV_GET_TYPE_NUMBER_OF_SECTORS \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 3))
#define CHRDEV_GET_TYPE_SECTOR_TBL_INFO \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 4))
#define CHRDEV_GET_TYPE_MAX_SYNC_READ_SIZE \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 5))
#define CHRDEV_GET_TYPE_MAX_SYNC_WRITE_SIZE \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 6))
#define CHRDEV_GET_TYPE_MW_PARTITION_OFFSET \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 7))    
#define CHRDEV_GET_TYPE_PROPRIETARY_PROPERTY \
    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0x80))  /* get type value over 0x80 should be defined in device proprietary header file */

/*-----------------------------------------------------------------------------
 * Set types
 *---------------------------------------------------------------------------*/
#define CHRDEV_SET_TYPE_WRITE_BYTE \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0))
#define CHRDEV_SET_TYPE_SYNC_WRITE \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1))
#define CHRDEV_SET_TYPE_ASYNC_READ \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 2))
#define CHRDEV_SET_TYPE_ASYNC_WRITE \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))
#define CHRDEV_SET_TYPE_ERASE \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 4))
#define CHRDEV_SET_TYPE_NFY_FCT \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 5))
#define CHRDEV_SET_TYPE_IO_CTRL \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 6))
#define CHRDEV_SET_TYPE_ABORT \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 7))
#define CHRDEV_SET_TYPE_PROPRIETARY_PROPERTY \
    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0x80)) /* set type value over 0x80 should be defined in device proprietary header file */


/*-----------------------------------------------------------------------------
 * Character device flags
 *---------------------------------------------------------------------------*/
#define CHRDEV_FLAG_ASYNC_READ  ((UINT32) 0x00010000)
#define CHRDEV_FLAG_ASYNC_WRITE ((UINT32) 0x00020000)

#endif /* _X_CHR_DEV_H_ */
