/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_ci_storage.h,v $
 * $Revision: #1 $
 * $Date: 2015/06/01 $
 * $Author: tinghong.liang $
 * $CCRevision:$
 * $SWAuthor: $
 * $MD5HEX: $
 *
 * Description: 
 *
 *---------------------------------------------------------------------------*/
 
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_CI
*  @{ 
*/
/*----------------------------------------------------------------------------*/
#ifndef _X_CI_STORAGE_H_
#define _X_CI_STORAGE_H_

/*-------------------------------------------------------------------------
                    include files
-------------------------------------------------------------------------*/
#include "u_sys_name.h"
#include "u_ci.h"

/*-------------------------------------------------------------------------
                    define, micro, structure
-------------------------------------------------------------------------*/
extern UINT8 ciplus_storage_root_cert[];
extern UINT8 ciplus_storage_dev_cert[];
extern UINT8 ciplus_storage_brand_cert[];
extern UINT8 ciplus_storage_host_prng_key[];
extern UINT8 ciplus_storage_host_dh_g[];
extern UINT8 ciplus_storage_host_dh_p[];
extern UINT8 ciplus_storage_host_dh_q[];
extern UINT8 ciplus_storage_hdq[];
extern UINT8 ciplus_storage_host_siv[];
extern UINT8 ciplus_storage_host_slk[];
extern UINT8 ciplus_storage_host_clk[];

extern UINT16 ciplus_storage_root_cert_size;
extern UINT16 ciplus_storage_dev_cert_size;
extern UINT16 ciplus_storage_brand_cert_size;
extern UINT16 ciplus_storage_host_prng_key_size;
extern UINT16 ciplus_storage_host_dh_g_size;
extern UINT16 ciplus_storage_host_dh_p_size;
extern UINT16 ciplus_storage_host_dh_q_size;
extern UINT16 ciplus_storage_hdq_size;
extern UINT16 ciplus_storage_host_siv_size;
extern UINT16 ciplus_storage_host_slk_size;
extern UINT16 ciplus_storage_host_clk_size; 

#define CI_AUTH_CTX_OFFSET          (0x10000)
#define CI_ECP_CERT_OFFSET          (0x8000)

#define CI_RAW_CRED_HDR_OFFSET      ((UINT16)4)
#define CI_RAW_CRED_TAG_SIZE        ((UINT16)1)
#define CI_RAW_CRED_ITEM_LEN_SIZE   ((UINT16)2)
#define SAH1_DIGEST_LEN             ((UINT16)20)
#define CI_MAX_KEY_PACK             (0x8000)
#define CI_DEVICE_ID_SIZE           ((UINT16)16)

//#define CIPLUS_KEY_SUPPORT_SERIAL_NUMBER
#define CI_PLUS_AKH_PATH   "/perm/ciplus_akh"
#define CI_PLUS_KEY_RAW_MAX_SIZE   (6*1024)


/*-------------------------------------------------------------------------
                    Functions
 ------------------------------------------------------------------------*/

extern VOID x_ci_sha1 (UINT8 *pi1_buffer, UINT32 ui4_length, UINT8 *pui1_digest);
extern INT32 x_ci_update_ciplus_cred(CHAR *pc_src_file_path_in);
extern INT32 x_ci_update_ciplus_cred_ex(CHAR *pc_src_file_path_in, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_update_ciplus_cred_without_backup(CHAR *pc_src_file_path_in);
extern INT32 x_ci_update_ciplus_cred_without_backup_ex(CHAR *pc_src_file_path_in, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_update_ciplus_cred_with_buffer(CHAR *aui1_src_buffer, UINT32 ui4_len);
extern INT32 x_ci_update_ciplus_cred_with_buffer_ex(CHAR *aui1_src_buffer, UINT32 ui4_len, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_erase_ciplus_cred(VOID);
extern INT32 x_ci_erase_ciplus_precred(VOID);
extern INT32 x_ci_write_ciplus_pre_cred(CHAR *aui1_src_buffer, UINT32 ui4_len);
extern INT32 x_ci_write_ciplus_pre_cred_ex(CHAR *aui1_src_buffer, UINT32 ui4_len, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_read_ciplus_pre_cred(CHAR *aui1_dst_buffer, UINT32 *ui4_len);
extern INT32 x_ci_read_ciplus_pre_cred_ex(CHAR *aui1_dst_buffer, UINT32 *ui4_len, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_activate_ciplus_cred(VOID);
extern INT32 x_ci_activate_ciplus_cred_ex(CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_get_ciplus_cred_info(CI_PLUS_CRED_INFO_T *t_info_data);
extern INT32 x_ci_get_ciplus_cred_info_ex(CI_PLUS_CRED_INFO_T *t_info_data, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_get_ciplus_cred_pre_info(CI_PLUS_CRED_INFO_T *t_info_data);
extern INT32 x_ci_get_ciplus_cred_pre_info_ex(CI_PLUS_CRED_INFO_T *t_info_data, CI_CERT_TYPE_T t_cert_type);
extern INT32 x_ci_secure_storage_mem_encryption(CHAR *puc_deviceId, CHAR *puc_deviceData,
                                                  CHAR *pc_src_buf,INT32 l_src_len,
                                                  CHAR *pc_dst_buf, INT32 *pl_dst_len);
extern INT32 x_ci_secure_storage_mem_decryption(CHAR *puc_deviceId, CHAR *puc_deviceData,
                                                  CHAR *pc_src_buf, INT32 l_src_len,
                                                  CHAR *pc_dst_buf, INT32 *pl_dst_len);
extern INT32 x_ci_read_device_random_constant(UINT8 *aui1_data, UINT8 ui1_len);
extern INT32 x_ci_aes128cbc_decrypt(UINT8 *pui1_src_buffer,
                                 UINT32 ui4_src_length,
                                 UINT8 *pui1_dst_buffer,
                                 UINT32 *ui4_dst_length,                                
                                 UINT8 *pui1_key,
                                 UINT8 *pui1_iv);
extern INT32 _ci_storage_read_file_and_decr(CHAR* pc_dst_path, UINT8* pui1_data_buffer,UINT16*pui2_dec_len,UINT8* pui1_serial_buffer,UINT16*pui2_serail_len);


#endif /* _X_CI_STORAGE_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_CI*/
/*----------------------------------------------------------------------------*/
