/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_cl.h,v $
 * $Revision: #1 $ 1.0
 * $Date: 2015/05/29 $ 2004/11/26
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/9 $
 * $SWAuthor: Asa Wang $
 * $MD5HEX: 647c2c155958dab5df4b6be5038f60de $ Asa
 *
 * Description: 
 *         This file exports all compression library API and constant for 
 *         other module using.
 *---------------------------------------------------------------------------*/

#ifndef _X_CL_H_
#define _X_CL_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_handle.h"
#include "u_cl.h"
/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

/* the callback function used in x_cl_open_eng, x_cl_open_archive, 
   x_open_entry_by_name and x_cl_open_entry_by_index. */
typedef VOID (*x_cl_nfy_fct) (
              HANDLE_T              h_handle,   /* (I) The handle will be destoried */
              VOID*                 pv_nfy_tag, /* (I)The client data specified by the client */
              CL_NFY_EVENT_TYPE_T   e_event     /* (I)usually is CL_NFY_EVENT_TYPE_T */
              );

/* Open an engine for compression/decompression operation. */
extern INT32 x_cl_open_eng (
             const CHAR*    ps_name,  /* (I)identifies sixteen characters engine name, and the character is case sensitive.*/
             x_cl_nfy_fct   pf_nfy,   /* (I)specify the notify callback function. */
             VOID*          pv_tag,   /* (I)the argument prepared by the user, and it will send back to user in (*x_cl_nfy_fct). */
             HANDLE_T*      ph_engine /* (O)The handle of the engine which want to be opened. */ 
             );

/* Close the entry, archive or engine after no more using. */
extern INT32 x_cl_close (
             HANDLE_T           h_handle /* (I)the handle of a object which you already opened. */
             );

/* This routine gets the length of compressed buffer after actually performs data compression from source buffer. */
extern INT32 x_cl_get_compress_len (
             HANDLE_T               h_engine,      /* (I)identifies the handle of the engine which performs compression operation */
             const VOID*            pv_src_buf,    /* (I)point to the source buffer, and data in this buffer will be compressed. */ 
             SIZE_T                 z_src_len,     /* (I)specify the length of the source buffer. */
             UINT32                 ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                  pv_param,      /* (I)specify compression paramters, set NULL is using default */
             SIZE_T*                pz_dest_len    /* (O)contains the compressed length after source  buffer is compressed. */
             );

/* This routine compresses data from source buffer to destination buffer. */
extern INT32 x_cl_compress (
             HANDLE_T               h_engine,      /* (I)identifies the handle of the engine, which performs compression operation */
             const VOID*            pv_src_buf,    /* (I)point to the source buffer, which contains source data waiting for compression. */
             SIZE_T                 z_src_len,     /* (I)specify the length of the source buffer. */
             UINT32                 ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                  pv_param,      /* (I)specify compression paramters, set NULL is using default */
             VOID*                  pv_dest_buf,   /* (O)point to the destination buffer. */ 
             SIZE_T*                pz_dest_len    /* (I/O)specify the length of the destination buffer./specify the really compression length of destination buffer. */
             );

/* Get the length of the decompressed buffer after really decompress data from source buffer. */
extern INT32 x_cl_get_decompress_len (
             HANDLE_T                 h_engine,      /* (I)identifies the handle of the engine which performs compression operation */
             const VOID*              pv_src_buf,    /* (I)point to the source buffer, which contains source data waiting for decompression. */
             SIZE_T                   z_src_len,     /* (I)specify the length of the source buffer. */
             UINT32                   ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                    pv_param,      /* (I)specify compression paramters, set NULL is using default */
             SIZE_T*                  pz_dest_len    /* (O)contains the total compressed length after source buffer is compressed. */
             );

/* Get the length of the decompressed buffer after really decompress data from source buffer. */
extern INT32 x_cl_get_decompress_ex_len (
             HANDLE_T                 h_engine,      /* (I)identifies the handle of the engine which performs compression operation */
             const VOID*              pv_src_buf,    /* (I)point to the source buffer, which contains source data waiting for decompression. */
             SIZE_T                   z_src_len,     /* (I)specify the length of the source buffer. */
             UINT32                   ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                    pv_param,      /* (I)specify compression paramters, set NULL is using default */
             SIZE_T*                  pz_dest_len    /* (O)contains the total compressed length after source buffer is compressed. */
             );

/* Decompress data from source buffer to destination buffer. */
extern INT32 x_cl_decompress (
             HANDLE_T                 h_engine,      /* (I)identifies the handle of the engine which performs compression operation */
             const VOID*              pv_src_buf,    /* (I)point to the source buffer. */
             SIZE_T                   z_src_len,     /* (I)specify the length of the source buffer. */ 
             SIZE_T                   z_skip_len,    /* (I)indicate the number of bytes to skip from the beginning of decompression output. */
             UINT32                   ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                    pv_param,      /* (I)specify compression paramters, set NULL is using default */
             VOID*                    pv_dest_buf,   /* (O)point to the destination buffer. */
             SIZE_T*                  pz_dest_len,   /* (I/O)specify the length of the destination buffer./specify the really decompression length of destination buffer. */
             HANDLE_T*                ph_prev_state  /* (I/O)set NULL if one shot;specify previous decompress state/ next time decompress state */ 
             );

/* Decompress data from source buffer to destination buffer. */
extern INT32 x_cl_decompress_ex(
             HANDLE_T                 h_engine,      /* (I)identifies the handle of the engine which performs compression operation */
             const VOID*              pv_src_buf,    /* (I)point to the source buffer. */
             SIZE_T                   z_src_len,     /* (I)specify the length of the source buffer. */ 
             SIZE_T                   z_skip_len,    /* (I)indicate the number of bytes to skip from the beginning of decompression output. */
             UINT32                   ui4_param_type,/* (I)specify type of compression paramters, set CL_NULL_PARAM_TYPE is using default */  
             VOID*                    pv_param,      /* (I)specify compression paramters, set NULL is using default */
             VOID*                    pv_dest_buf,   /* (O)point to the destination buffer. */
             SIZE_T*                  pz_dest_len,   /* (I/O)specify the length of the destination buffer./specify the really decompression length of destination buffer. */
             HANDLE_T*                ph_prev_state  /* (I/O)set NULL if one shot;specify previous decompress state/ next time decompress state */ 
             );


/* This routine creates and opens an archive for later entry operation. */
extern INT32 x_cl_open_archive (
             HANDLE_T       h_engine,  /* (I)specified the handle of an engine. */
             const CHAR*    ps_name,   /* (I)specified the path name string of the archive. */
             UINT8          ui1_mode,  /* (I)specified the access permit mode, and only support CL_READ_MODE. */
             x_cl_nfy_fct   pf_nfy,    /* (I)specify the notify callback function. When ph_archive is not valid any more, the callback function pf_nfy will be called. */
             VOID*          pv_tag,    /* (I)the argument prepared by the user, and it will send back to user in (*x_cl_nfy_fct). */
             HANDLE_T*      ph_archive /* (O)retrieve the archive handle. */ 
             );

/* This routine creates or opens an entry, which has the path name string ps_name in the archive. */
extern INT32 x_cl_open_entry_by_name (
             HANDLE_T       h_archive, /* (I)the handle of the archive. */
             const CHAR*    ps_name,   /* (I)specified the path name string of the entry which will be opened. */
             x_cl_nfy_fct   pf_nfy,    /* (I)specify the notify callback function. When ph_entry is not valid any more, the callback function pf_nfy will be called. */
             VOID*          pv_tag,    /* (I)the argument prepared by the user, and it will send back to user in (*x_cl_nfy_fct). */
             HANDLE_T*      ph_entry   /* (O)retrieve the handle of the entry. */
             );

/* This routine opens an entry in the archive by index of entry numbers. */
extern INT32 x_cl_open_entry_by_index (
             HANDLE_T       h_archive,       /* (I)the handle of the archive. */
             UINT32         ui4_entry_index, /* (I)specified the index of the entry which will be opened, and it is 0 base. */
             x_cl_nfy_fct   pf_nfy,          /* (I)specify the notify callback function. When ph_entry is not valid any more, the callback function pf_nfy will be called. */
             VOID*          pv_tag,          /* (I)the argument prepared by the user, and it will send back to user in (*x_cl_nfy_fct). */
             HANDLE_T*      ph_entry         /* (O)retrieve the handle of the entry. */
             );

/* This Routine will read and decompress data from the current entry file to the destination buffer, after open an entry in the archive. */
extern INT32 x_cl_decompress_entry (
             HANDLE_T       h_entry,    /* (I)the handle of the entry. */
             SIZE_T         z_skip_len, /* (I)indicate the number of Bytes which shall be skipped before data shall be actually written to the destination buffer. */
             VOID*          pv_dest,    /* (O)the destination buffer that contains reading data. */
             SIZE_T*        pz_dest_len /* (I/O)specified the length of  the destination buffer./specify the really decompression length of destination buffer. */
             );

/* This Routine will get the length of the current entry file except z_skip_len. */
extern INT32 x_cl_get_decompress_entry_len (
             HANDLE_T       h_entry,    /* (I)the handle of the entry. */
             SIZE_T*        pz_dest_len /* (O)specify the actual length, which is needed to decompress the entry data. */
             );

/* This API will query information of the archive. */
extern INT32 x_cl_query_info (
             HANDLE_T               h_archive,       /* (I)the handle of the archive. */
             CL_QUERY_INFO_TYPE_T   e_type,          /* (I)specified the type of the query operation. */
             VOID*                  pv_filter,       /* (I)specified the querying filter to query the archive. */
             VOID*                  pv_query_buf,    /* (O)retrieve the query information in the buffer. */
             SIZE_T*                pz_query_buf_len /* (I/O)specified the length of the query buffer./specified the actual size of query information needed.  */
             );

/* This routine will get the length of query information of the archive. */
extern INT32 x_cl_get_query_info_len (
             HANDLE_T               h_archive,       /* (I)the handle of the archive. */
             CL_QUERY_INFO_TYPE_T   e_type,          /* (I)specified the type of the query operation. */
             VOID*                  pv_filter,       /* (I)specified the querying filter to query the archive. */
             SIZE_T*                pz_query_buf_len /* (O)specified the actual size of query information needed. */
             );

/*This API will query information of the archive.*/
extern INT32 x_cl_get_num_entries (
             HANDLE_T   h_archive,       /* (I)the handle of the archive. */
             UINT32*    pui4_num_entris  /* (O)retrieve the number of entries.*/ 
             );

#endif /* _X_CL_H_ */
