/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_demux.h,v $
 * $Revision$
 * $Date$
 * $Author$
 *
 * Description:
 *         This header file contains Demux driver specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_DEMUX_H_
#define _X_DEMUX_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_mm_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define DEMUX_GET_TYPE_PID          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  0))
#define DEMUX_GET_TYPE_STREAM_ID    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  1))
#define DEMUX_GET_TYPE_CTRL         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  3))
#define DEMUX_GET_TYPE_READ_BUFFER  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  6))
#define DEMUX_GET_TYPE_FILTER_LEN   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  9))
#define DEMUX_GET_TYPE_SCRS         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 10))
#define DEMUX_GET_TYPE_STC          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 11))
#define DEMUX_GET_TYPE_SPEED        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 12))
#define DEMUX_GET_TYPE_MEDIA_FORMAT (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 13))
#define DEMUX_GET_TYPE_BLOCK_PSI    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 14))
#define DEMUX_GET_TYPE_CURR_LBA     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 15))
#define DEMUX_GET_TYPE_CURR_TICK_NUM (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 16)) /* for Timeshift */
#define DEMUX_GET_TYPE_UOP_CAPABILITY    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 17))
#define DEMUX_GET_TYPE_POST_INFO         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 18))
#define DEMUX_GET_TYPE_STC_OFFSET   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 19))
#define DEMUX_GET_TYPE_TEMI          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 20))
#define DEMUX_GET_TYPE_TLV_DMX_INFO  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 21))
#define DEMUX_GET_TYPE_TTML_PTS      (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 22))


#define DEMUX_GET_TYPE_READ_BUFFER_NFY  (RM_GET_TYPE_ISR_CONTEXT | DEMUX_GET_TYPE_READ_BUFFER)

/* Set operations */
#define DEMUX_SET_TYPE_PID            ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  0)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_STREAM_ID      ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  1)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_NFY_FCT         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  2))
#define DEMUX_SET_TYPE_CTRL           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  3)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_ALLOC_BUFFER   ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  4)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_FREE_BUFFER    ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  5)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_UNLOCK_BUFFER   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  7))
#define DEMUX_SET_TYPE_SECTION_FILTER  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  8))
#define DEMUX_SET_TYPE_FILTER_LEN      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  9))
#define DEMUX_SET_TYPE_DEST_ID         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 10))
#define DEMUX_SET_TYPE_FLUSH_BUFFER    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 11))
#define DEMUX_SET_TYPE_SCRS_NFY_FCT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 12))
#define DEMUX_SET_TYPE_MEDIA_FORMAT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 13))
#define DEMUX_SET_TYPE_ADD_RANGE       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 14))
#define DEMUX_SET_TYPE_DEL_RANGE       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 15))
#define DEMUX_SET_TYPE_SPEED          ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 16)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_SEEK_POSTION    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 17))
#define DEMUX_SET_TYPE_BLOCK_PSI      ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 18)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_DMX_MODE       ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 19)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_ES_WITH_PTS_IN ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 20)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_REC_BUF_RANGE   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 21))
#define DEMUX_SET_TYPE_MONITOR_LBA    ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 22)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_DECRYPT_INFO    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 23))
#define DEMUX_SET_TYPE_CRYPT_CMD       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 24))
#define DEMUX_SET_TYPE_PRIORITY        (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 25))
#define DEMUX_SET_TYPE_AB_POSITION     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 26))
#define DEMUX_SET_TYPE_MM_AUTOTEST     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 27))
#define DEMUX_SET_TYPE_STC_ID         ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 28)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_FREE_RUN       ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 29)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_3D_PROPERTY     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 30))
#define DEMUX_SET_TYPE_HDMVC_INFO     ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 31)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_PARSE_HDR_CMD  ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 32)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_FLUSH_FEEDER   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 33))
#define DEMUX_SET_TYPE_VID_CHG_PID_NFY_FCT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 34))
#define DEMUX_SET_TYPE_AUD_CHG_PID_NFY_FCT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 35))
#define DEMUX_SET_TYPE_VID_PID_CHG_DONE    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 36))
#define DEMUX_SET_TYPE_AUD_PID_CHG_DONE    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 37))
#define DEMUX_SET_TYPE_VID_CODEC_CHG_DONE    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 38))//NO PARA
#define DEMUX_SET_TYPE_AUD_CODEC_CHG_DONE    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 39))
#define DEMUX_SET_TYPE_ALL_GNRC_STREAM_ID    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 40))
#define DEMUX_SET_TYPE_TS_DURATION            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 41))
#define DEMUX_SET_TYPE_PVR_PLAYBACK_INFO         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 42))
#define DEMUX_SET_TYPE_PUSH_CHANGE_AUDIO      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 43))
#define DEMUX_SET_TYPE_EIT_SECTION            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 44))
#define DEMUX_SET_TYPE_PLAY_ONE_FRM           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 45)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_PLAYBACK_TIME_LIMIT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 46))
#define DMUX_SET_TYPE_DATA_SPEED_VALID_DURATION  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 47))
#define DEMUX_SET_TYPE_BUF_UNFERFLOW_TIME_LIMIT (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 48))
#define DEMUX_SET_TYPE_BUF_READY_TIME_LIMIT     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 49))
#define DEMUX_SET_TYPE_MMP_SUPPORT_CI_PLUS  ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 50)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_ALGORITHM     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 51))
#define DEMUX_SET_TYPE_ODD_KEY       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 52))
#define DEMUX_SET_TYPE_EVEN_KEY      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 53))
#define DEMUX_SET_TYPE_DESC_CTRL     ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  54)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_DESC_KEY_INDEX (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 55))
#define DEMUX_SET_TYPE_FAST_START_PLAYBACK_TIME_LIMIT  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 56))
#define DEMUX_SET_TYPE_AUD_ES_BYTE_RATE  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 57))
#define DEMUX_SET_TYPE_VIDEO_CODEC_NOT_SUPPORT  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 58))
#define DEMUX_SET_TYPE_TRICK_END_POS  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 59))
#define DEMUX_SET_TYPE_REMUX_CTRL     ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  60)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_REC_START      ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  61)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_REC_STOP       ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  62)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_LTS_ID         ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 63)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_RENDER_POSITION  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 64))
#define DEMUX_SET_TYPE_PUSH_FLUSH_AUDIO  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 65))
#define DEMUX_SET_TYPE_TEMI_PID       ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  66)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_STC_VAL        ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 67)) | RM_SET_TYPE_ARG_NO_REF)
#define DEMUX_SET_TYPE_MULTI_STRM_FLAG  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 68))
#define DEMUX_SET_TYPE_MULTI_STRM_MODE  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 69))
/*key and alg structure copy from x_desc.h   */

/* Descrambler algorithms */
typedef enum
{
    SWDMX_TO_DESC_ALG_NONE = 0,          /* None scramble */
    SWDMX_TO_DESC_ALG_DES_ECB,           /* DES-56-ECB*/
    SWDMX_TO_DESC_ALG_DES_CBC,           /* DES-56-CCB*/
    SWDMX_TO_DESC_ALG_3DES_ECB,          /* 3DES-ECB*/
    SWDMX_TO_DESC_ALG_3DES_CBC,          /* 3DES-CCB*/
    SWDMX_TO_DESC_ALG_DVB,               /* DVB */
    SWDMX_TO_DESC_ALG_AES_128_ECB,       /* AES-128-ECB*/
    SWDMX_TO_DESC_ALG_AES_128_CBC,        /* AES-128-CBC*/
    SWDMX_TO_DESC_ALG_CA_SECURE
} SWDMX_TO_DESC_ALG_T;


/* DES EBC Key information */
typedef struct _SWDMX_TO_DES_ECB_KEY_INFO_T
{
    SIZE_T          z_key_len;    /* size of key in byte */
    const UINT8*    pui1_key;     /* pointer of key buffer */
} SWDMX_TO_DES_ECB_KEY_INFO_T;

/* AES CBC Key information */
typedef struct _SWDMX_TO_AES_128_CBC_KEY_INFO_T
{
    SIZE_T          z_key_len;    /* size of key in byte */
    const UINT8*    pui1_key;     /* pointer of key buffer */
    SIZE_T          z_iv_len;     /* size of iv in byte */
    const UINT8*    pui1_iv;      /* pointer of iv buffer */
} SWDMX_TO_AES_128_CBC_KEY_INFO_T;

/* Definition of descrambler key information */
typedef struct _SWDMX_TO_DESC_KEY_INFO_T
{
    SWDMX_TO_DESC_ALG_T e_descrambler_alg;                   /* descrambler algorithm */
    union
    {
        SWDMX_TO_DES_ECB_KEY_INFO_T t_des_ecb_key;          /*DES-56-ECB*/
        SWDMX_TO_AES_128_CBC_KEY_INFO_T t_aes_128_cbc_key;  /*AES-128-CBC*/
    }u_key_info;
}   SWDMX_TO_DESC_KEY_INFO_T;

/* Flags used during registration */
#define DEMUX_FLAG_INCL_SECTION_LENGTH  ((UINT32) 0x00010000)
#define DEMUX_FLAG_NEG_PATTERN_MATCH    ((UINT32) 0x00020000)
#define DEMUX_FLAG_ALLOW_FILTER_OFFSET  ((UINT32) 0x00040000)


/*STC sould be looked up in bitstream*/
#define STC_STREAM_EMBEDDED ((STC_T)0xFFFFFFFF00000000)


/* Notify conditions */
typedef enum
{
    DEMUX_COND_DATA_REC = 0,
    DEMUX_COND_OVERFLOW,
    DEMUX_COND_SCRS
}   DEMUX_COND_T;

/* Scramble status */
typedef enum
{
    DEMUX_SCRS_UNKNOWN = 0,
    DEMUX_SCRS_CLEAR,
    DEMUX_SCRS_SCRAMBLED
}   DEMUX_SCRS_T;

typedef enum
{
    DEMUX_MODE_UNKNOWN = 0,
    DEMUX_MODE_DTV,
    DEMUX_MODE_MM,
    DEMUX_MODE_TIME_SHIFT
}   DEMUX_MODE_T;

/* DEMUX_GET_TYPE_UOP_CAPABILITY */
typedef enum
{
    DEMUX_UOP_UNKNOWN = 0,
    DEMUX_UOP_SEEK,
    DEMUX_UOP_TRICK
}   DEMUX_UOP_T;

typedef struct _DEMUX_UOP_CAP_INFO_T
{
    DEMUX_UOP_T  e_uop;
    BOOL         b_is_allowed;
    union
    {
        MM_SPEED_TYPE_T t_speed;
        /*to be extend*/
    } u;
}   DEMUX_UOP_CAP_INFO_T;

/*for mw to update the section mask*/
typedef enum
{
    DEMUX_EIT_SET_SECTION = 0,
    DEMUX_EIT_SET_TABLE,
    DEMUX_EIT_SET_CHANNEL,
    DEMUX_EIT_SET_ALL_CHANNEL
}DEMUX_EIT_SET_TYPE_T;

typedef struct _DEMUX_CHANNEL_EIT_KEY_T
{
  UINT16 ui2_onid;
  UINT16 ui2_tsid;
  UINT16 ui2_svcid;
}DEMUX_CHANNEL_EIT_KEY_T;

typedef struct _DEMUX_SECTION_KEY_T
{
  DEMUX_CHANNEL_EIT_KEY_T e_key;
  UINT8  ui1_tableid; //talbe id
  UINT8  ui1_version; //talbe version
  UINT8  ui1_secnum; //section number.
}DEMUX_SECTION_KEY_T;

typedef struct _DEMUX_SECTION_KEY_SET_T
{
  DEMUX_SECTION_KEY_T tSecKey;
  DEMUX_EIT_SET_TYPE_T eSetType;
  BOOL fgSet;
}DEMUX_SECTION_KEY_SET_T;


typedef enum
{
    DEMUX_POST_INFO_UNKNOWN = 0,
    DEMUX_POST_INFO_MEDIA_BTE_OFST,
    DEMUX_POST_INFO_MEDIA_TM_OFST,
    DEMUX_POST_INFO_MEDIA_RENDER_OFST,
    DEMUX_POST_INFO_RANGE_BTE_OFST,
    DEMUX_POST_INFO_RANGE_TM_OFST
} DEMUX_POST_INFO_TYPE_T;

typedef struct _RENDER_INFO_T
{
    UINT64  ui8_stc;
    UINT64  ui8_aud_pts_info;
    UINT64  ui8_aud_frame_position;
    UINT64  ui8_i_pts_info;
    UINT64  ui8_pts_info;
    UINT64  ui8_i_frame_position;
    UINT64  ui8_frame_position;
    UINT64  ui8_frame_position_disp;
    INT32   i4_temporal_reference;
    UINT16  ui2_decoding_order;
    UINT32  ui4_timestap;
}RENDER_INFO_T;

typedef struct _DEMUX_POST_INFO_T
{
    DEMUX_POST_INFO_TYPE_T  e_type;
    UINT64 ui8_pos_info;
    union
    {
        RENDER_INFO_T render_info;
    } u;
} DEMUX_POST_INFO_T;

/* Notify function */
typedef BOOL (*x_demux_nfy_fct) (VOID*         pv_nfy_tag,
                                 DEMUX_COND_T  e_nfy_cond,
                                 UINT32        ui4_data_1,
                                 UINT32        ui4_data_2,
                                 UINT32        ui4_data_3,
                                 UINT64        ui8_ext_data_1,
                                 UINT64        ui8_ext_data_2,
                                 UINT32        ui4_ext_data_3
                                 );

/* Notify function */
typedef BOOL (*x_swdemux_pid_chg_nfy_fct) (VOID*  pv_nfy_tag,
                                           UINT32  ui4_data_1,
                                           UINT32  ui4_data_2,
                                           UINT32  ui4_data_3
                                           );

typedef struct _SWDEMUX_PID_CHG_NFY_INFO_T
{
    VOID*  pv_tag;
    x_swdemux_pid_chg_nfy_fct  pf_swdemux_pid_chg_nfy;
} SWDEMUX_PID_CHG_NFY_INFO_T;


/* Destination type and id info */
typedef struct _DEMUX_DEST_INFO_T
{
    DRV_TYPE_T  e_type;

    UINT16  ui2_id;
}   DEMUX_DEST_INFO_T;

/* Notify info setting */
typedef struct _DEMUX_NFY_INFO_T
{
    VOID*  pv_tag;

    x_demux_nfy_fct  pf_demux_nfy;
}   DEMUX_NFY_INFO_T;

/* Read demux buffer */
typedef struct _DEMUX_READ_BUFFER_INFO_T
{
    UINT8*  pui1_dest;

    VOID*  pv_src_tag;
    UINT8  ui1_serial_num;
    SIZE_T  z_skip_len;
    SIZE_T  z_copy_len;
}   DEMUX_READ_BUFFER_INFO_T;

/* Set section filter */
typedef struct _DEMUX_SECTION_FILTER_INFO_T
{
    UINT8  ui1_tbl_id_pattern;
    UINT8  ui1_tbl_id_mask;

    UINT8  ui1_pattern_offset;
    UINT8  ui1_pattern_len;
    UINT8*  pui1_pos_pattern;
    UINT8*  pui1_pos_mask;

    UINT8*  pui1_neg_pattern;
    UINT8*  pui1_neg_mask;

    BOOL  b_check_crc;
}   DEMUX_SECTION_FILTER_INFO_T;

/* Unlock buffer */
typedef struct _DEMUX_UNLOCK_BUFFER_INFO_T
{
    VOID*  pv_src_tag;
    UINT8  ui1_serial_num;
    SIZE_T  z_src_len;

}   DEMUX_UNLOCK_BUFFER_INFO_T;

/* STC value */
#define DEMUX_STC_FLAG_STABLE      ((UINT8) 0x01)
#define DEMUX_STC_FLAG_33_BIT_STC  ((UINT8) 0x02)

typedef struct _DEMUX_STC_INFO_T
{
    STC_T  t_stc;

    UINT8  ui1_flags;
}   DEMUX_STC_INFO_T;

/* DMX PES for multi-media */
typedef enum
{
    DMX_MM_PES_DIVX,
    DMX_MM_PES_MKV,
    DMX_MM_PES_PS,
    DMX_MM_PES_MAX
} DEMUX_MM_PES_FMT_T;

typedef struct
{
    UINT32 ui4_data_1;
    UINT32 ui4_data_2;
    UINT32 ui4_data_3;
    UINT32 ui4_pts;
    UINT32 ui4_duration;
} DEMUX_MKV_PACK_T;

#endif /* _X_DEMUX_H_ */
