/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_dlna_media_type.h
*  $RCSfile: $
*  $Revision: #1 $
*  $Date: 2015/05/29 $
*  $Author: brianpc.huang $
*  
*  @par Description:
*         This header file contains DLNA media type exported API.
*/
/*----------------------------------------------------------------------------*/

/*!
* @addtogroup  groupMW_INET_DLNA
* @brief       Provide an external API about DLNA.
* %DLNA API would be used by user who wants to play media stream in local network  
* 
* @see 
*
* @{
* */


#ifndef X_DLNA_MEDIA_TYPE_H
#define X_DLNA_MEDIA_TYPE_H

#include "u_dlna_media_type.h"
#include "u_dlna_dmp.h"


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_get_media_type_by_profile_id(CHAR * ps_profile_id, CHAR ** pps_mime_type)
*  @brief  Gets the media type by profile ID
*          - Called while who wants to Get the media type by profile ID
*  @param  [in]ps_profile_id          - The profile id string
*  @param  [out]pps_mime_type         - The mime type string
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Get type ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_get_media_type_by_profile_id
*
* Description: get object media type by profile id
*
* Inputs:  ps_profile_id - the dlna object media type profile id
*
* Outputs: pps_mime_type - object mime type, 
*                this type may include many mime types the space token is ";", 
*                such as the profile "HEAAC_L4", the media type is "audio/mp4;audio/3gpp;audio/vnd.dlna.adts"
*
*
* Returns: DLNA_INVALID_ARGUMENT - the argument is invalid
*          DLNAR_OK               - get profile inof ok 
----------------------------------------------------------------------------*/
extern INT32 x_dlna_get_media_type_by_profile_id(CHAR * ps_profile_id, CHAR ** pps_mime_type);



/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_get_mime_type(  DLNA_DMP_PROFILE_INFO_T * pt_profile_info, 
                                        DLNA_DMP_OBJECT_DRM_TYPE_T e_type, 
                                        CHAR ** pps_mime_type)
*  @brief  Gets the MIME type by profile info and DRM type
*          - Called while who wants to Get the MIME type
*  @param  [in]pt_profile_info        - The profile info
*  @param  [in]e_type                 - The DRM type
*  @param  [in]pps_mime_type          - The mime type string
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Get type ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_get_mime_type
*
* Description: get object media type by profile id
*
* Inputs:  pt_profile_info - the dlna object media type profile
*          e_type          - the object drm type, this type returned by x_dlna_dmp_get_object_drm_type 
*
* Outputs: pps_mime_type - object mime type
*
* Returns: DLNAR_INVALID_ARGUMENT - the argument is invalid
*          DLNAR_OK               - get profile info ok 
----------------------------------------------------------------------------*/
extern INT32 x_dlna_get_mime_type(DLNA_DMP_PROFILE_INFO_T * pt_profile_info, 
                           DLNA_DMP_OBJECT_DRM_TYPE_T e_type, 
                           CHAR ** pps_mime_type);
    


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_get_media_type_extension(CHAR* ps_protocol, CHAR ** pps_ext_name)
*  @brief  Gets the media type extension
*          - Called while who wants to Get the media type extension
*  @param  [in]ps_protocol           - The protocol string
*  @param  [in]pps_ext_name          - The media type extension string
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Get type extension ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_get_media_type_extension
*
* Description: get object media type by profile string
*
* Inputs:  ps_protocol  - the dlna media protocol info string
*         
* Outputs: pps_ext_name - media extensions name
*
* Returns: DLNAR_OK               - get profile info ok 
----------------------------------------------------------------------------*/                       
extern INT32 x_dlna_get_media_type_extension(CHAR* ps_protocol, CHAR ** pps_ext_name);

#endif

/*! @} */
