/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_dsmcc_oc.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains DSM-CC engine specific definitions, which
 *         are exported.
 *-------------------------------------------------------------------------*/

#ifndef _X_DSMCC_OC_H_
#define _X_DSMCC_OC_H_


/*---------------------------------------------------------------------------
                    include files
---------------------------------------------------------------------------*/

#include "x_handle_grp.h"
#include "x_common.h"
#include "x_tm.h"
#include "u_dsmcc_oc.h"


/*---------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

extern INT32 x_dsmcc_oc_init (const CHAR*  ps_name);

extern INT32 x_dsmcc_oc_open_srg_by_pid_ts_descr (HANDLE_T       h_root,
                                                  MPEG_2_PID_T   t_pid,
                                                  TS_DESCR_T*    pt_ts_descr,
                                                  TM_SRC_TYPE_T  e_src_type,
                                                  VOID*          pv_src_info,
                                                  VOID*          pv_nfy_tag,
                                                  x_tm_nfy_fct   pf_nfy,
                                                  HANDLE_T*      ph_srg,
                                                  TM_COND_T*     pe_cond);

extern INT32 x_dsmcc_oc_open_srg_by_pid_ts_id (HANDLE_T       h_root,
                                               MPEG_2_PID_T   t_pid,
                                               UINT16         ui2_ts_id,
                                               TM_SRC_TYPE_T  e_src_type,
                                               VOID*          pv_src_info,
                                               VOID*          pv_nfy_tag,
                                               x_tm_nfy_fct   pf_nfy,
                                               HANDLE_T*      ph_srg,
                                               TM_COND_T*     pe_cond);

extern INT32 x_dsmcc_oc_parse_srg_private_data (HANDLE_T        h_obj,
                                                VOID*           pv_data,
                                                SIZE_T          z_data_len,
                                                x_tm_parse_fct  pf_parse,
                                                TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_oc_get_entry_info (HANDLE_T                   h_obj,
                                        DSMCC_OC_ENTRY_INFO_T*     pt_entry_info,
                                        TM_COND_T*                 pe_cond);

extern INT32 x_dsmcc_oc_get_subentry_info (HANDLE_T                   h_obj,
                                           UINT16                     ui2_idx,
                                           DSMCC_OC_SUBENTRY_INFO_T*  pt_subentry_info,
                                           TM_COND_T*                 pe_cond);

extern INT32 x_dsmcc_oc_get_sgw_dii_info (HANDLE_T                  h_obj,
                                          DSMCC_OC_SGW_DII_INFO_T*  pt_sgw_dii_info,
                                          TM_COND_T*                pe_cond);

extern INT32 x_dsmcc_oc_get_subentry_cnt (HANDLE_T    h_obj,
                                          UINT16*     pui2_cnt,
                                          TM_COND_T*  pe_cond);

extern INT32 x_dsmcc_oc_open_dir_by_name (HANDLE_T      h_obj,
                                          const CHAR*   ps_name,
                                          VOID*         pv_nfy_tag,
                                          x_tm_nfy_fct  pf_nfy,
                                          HANDLE_T*     ph_dir,
                                          TM_COND_T*    pe_cond);

extern INT32 x_dsmcc_oc_open_file_by_name (HANDLE_T      h_obj,
                                           const CHAR*   ps_name,
                                           BOOL          b_file_cache,
                                           VOID*         pv_nfy_tag,
                                           x_tm_nfy_fct  pf_nfy,
                                           HANDLE_T*     ph_file,
                                           TM_COND_T*    pe_cond);

extern INT32 x_dsmcc_oc_open_file_by_path(HANDLE_T      h_obj,
                                          const CHAR*   ps_path,
                                          BOOL          b_file_cache,
                                          VOID*         pv_nfy_tag,
                                          x_tm_nfy_fct  pf_nfy,
                                          HANDLE_T*     ph_file,
                                          TM_COND_T*    pe_cond);

extern INT32 x_dsmcc_oc_read_file (HANDLE_T    h_file,
                                   UINT64      ui8_offs,
                                   SIZE_T      z_buff_len,
                                   UINT8*      pui1_buff,
                                   SIZE_T*     pz_buff_read,
                                   TM_COND_T*  pe_cond);
extern INT32 x_dsmcc_oc_read_file_timeout (HANDLE_T    h_file,
                                           UINT64      ui8_offs,
                                           SIZE_T      z_buff_len,
                                           UINT8*      pui1_buff,
                                           SIZE_T*     pz_buff_read,
                                           UINT32      ui4_timeout,
                                           TM_COND_T*  pe_cond);
extern INT32 x_dsmcc_oc_read_file_async (HANDLE_T      h_file,
                                         UINT64        ui8_offs,
                                         SIZE_T        z_buff_len,
                                         UINT8*        pui1_buff,
                                         VOID*         pv_nfy_tag,
                                         x_tm_nfy_fct  pf_nfy,
                                         HANDLE_T*     ph_async_read,
                                         TM_COND_T*    pe_cond);

extern INT32 x_dsmcc_oc_get_file_info (HANDLE_T               h_file,
                                       DSMCC_OC_FILE_INFO_T*  pt_file_info,
                                       TM_COND_T*             pe_cond);
                                        
extern INT32 x_dsmcc_oc_open_strm_event_by_path(HANDLE_T      h_obj,
                                                const CHAR*   ps_path,
                                                VOID*         pv_nfy_tag,
                                                x_tm_nfy_fct  pf_nfy,
                                                HANDLE_T*     ph_strm_event,
                                                TM_COND_T*    pe_cond);
extern INT32 x_dsmcc_oc_open_strm_event_by_name (HANDLE_T      h_obj,
                                                 const CHAR*   ps_name,
                                                 VOID*         pv_nfy_tag,
                                                 x_tm_nfy_fct  pf_nfy,
                                                 HANDLE_T*     ph_strm_event,
                                                 TM_COND_T*    pe_cond);

extern INT32 x_dsmcc_oc_get_strm_event_taps_cnt (HANDLE_T    h_obj,
                                                 UINT32      ui4_taps_filter_mask,
                                                 UINT8*      pui1_cnt,
                                                 TM_COND_T*  pe_cond);

extern INT32 x_dsmcc_oc_get_strm_event_events_cnt (HANDLE_T    h_obj,
                                                   UINT8*      pui1_cnt,
                                                   TM_COND_T*  pe_cond);

extern INT32 x_dsmcc_oc_get_strm_event_event_info_by_name (HANDLE_T                   h_strm_event,
                                                           DSMCC_OC_GET_EVENT_INFO_T* pt_get_strm_event_info,
                                                           DSMCC_OC_EVENT_INFO_T*     pt_strm_event_info,
                                                           TM_COND_T*                 pe_cond);

extern INT32 x_dsmcc_oc_get_strm_event_event_info_by_idx (HANDLE_T                          h_strm_event,
                                                          DSMCC_OC_GET_EVENT_INFO_BY_IDX_T* pt_get_event_info_by_idx,
                                                          DSMCC_OC_EVENT_INFO_T*            pt_strm_event_info,
                                                          TM_COND_T*                        pe_cond);

#endif /* _X_DSMCC_OC_H_ */

