/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


#ifndef X_FONT_DOWNLOAD_H
#define X_FONT_DOWNLOAD_H



typedef enum {
   FT_DOWNLOAD_EVENT_NONE,
   FT_DOWNLOAD_EVENT_DOWNLOAD_SUCCESS,
   FT_DOWNLOAD_EVENT_DOWNLOAD_ERR,
   FT_DOWNLOAD_EVENT_ADDFONT_SUCCESS,
   FT_DOWNLOAD_EVENT_ADDFONT_ERROR,
}FT_DOWNLOAD_EVENT_T;

#define FT_DOWNLOAD_ERRCODE_OK         ((INT32)0)
#define FT_DOWNLOAD_ERRCODE_INV_ARG    ((INT32)-1)
#define FT_DOWNLOAD_ERRCODE_NO_MEM     ((INT32)-2)
#define FT_DOWNLOAD_ERRCODE_NOT_FOUND  ((INT32)-3)
#define FT_DOWNLOAD_ERRCODE_NO_RES     ((INT32)-4)


typedef INT32 (*FONT_DOWNLOAD_EVENT_CALLBACK_FCT)(
      UINT32              ui4_download_id,
      FT_DOWNLOAD_EVENT_T e_event,
      INT32               i4_errcode,
      VOID*               pv_arg);

#define FONT_DOWNLOAD_MAX_FONT_NAME_LEN (32)
#define FONT_DOWNLOAD_MAX_FONT_URL_LEN  (512)
#define FONT_DOWNLOAD_MAX_IP_LEN        (16)

typedef struct
{
    CHAR  s_font_name[FONT_DOWNLOAD_MAX_FONT_NAME_LEN];
    CHAR  s_font_url[FONT_DOWNLOAD_MAX_FONT_URL_LEN];
    BOOL  b_multi_font;
    BOOL  b_last_url;
}FONT_DOWNLOAD_INF_T;

extern INT32 x_font_download_start(
    FONT_DOWNLOAD_INF_T*                pt_font_inf,
    FONT_DOWNLOAD_EVENT_CALLBACK_FCT    pf_callback,
    UINT32*                             pui4_download_id,
    VOID*                               pv_tag
    );

extern INT32 x_font_download_stop(UINT32 ui4_download_id);
extern BOOL x_font_download_exists(CHAR* s_font_name);
extern INT32 x_font_get_download_ip(UINT32 ui4_download_id, CHAR* ps_ip, UINT32 ui4_len);

#endif
