/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file 3d_photo_playback.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file exports all image library API and constant for 
 *         other module using.
 */
/*----------------------------------------------------------------------------*/

#ifndef _MPO_PHOTO_PLAYBACK_H_
#define _MPO_PHOTO_PLAYBACK_H_


#include "x_gl.h"
#include "x_img.h"
#include "x_wgl.h"
#include "x_dbg.h"
#include "x_img_tfx.h"


#define IMG_RC_DISPLAY_NAME                 SN_PRES_MAIN_DISPLAY


#define GUI_PLANE_ID                        0

#define _IMG_DRC_FLAG_READY               ((UINT8) 0x01)
#define _IMG_DRC_FLAG_USED                   ((UINT8) 0x02)


#define _IMG_PE_ACCURACY_CONST              ((UINT32)10000)
#define ZOOM_CONST  100
#define ZOOM_MIN    12
#define ZOOM_MAX    800

#define PAN_CONST   100
#define PAN_MIN     -100
#define PAN_MAX     100


typedef struct _IMG_CROP_RECT_T
{
    UINT32  ui4_left_offset;
    UINT32  ui4_right_offset;
    UINT32  ui4_top_offset;
    UINT32  ui4_bottom_offset;
}IMG_CROP_RECT_T;


typedef struct _IMG_DRC_BUF_T
{
    GL_HVIEW_T                  h_view;                         /*view handle*/
    GL_HSURFACE_T               h_surf;                         /*surface handle*/
    IMG_ROTATE_OP_TYPE_T        e_rotate;                       /*rotation of this display*/
    UINT16                      ui2_zoom;
    GL_POINT_T                  t_center_offset;
    UINT32                      ui4_scale;                       /*scale of this display*/
    UINT32                      ui4_x;
    UINT32                      ui4_y;
    UINT32                      ui4_w;
    UINT32                      ui4_h;
    UINT32                      ui4_buf_width;
    UINT32                      ui4_buf_height;

    IMG_CROP_RECT_T             t_surface_crop_rect;
}   IMG_DRC_BUF_T;

typedef struct _IMG_DRC_DISPLAY_RES_T
{
    IMG_DRC_BUF_T               t_drc_buf;
    UINT32                      ui4_viewport;     /*viewport of this display*/
    UINT8                       ui1_flag;
}   IMG_DRC_DISPLAY_RES_T;
typedef enum{
    E_3D_JPS,
    E_3D_PNS,
    E_3D_BMS,
    E_3D_MPO,
    E_3D_UNKNOWN
}E_3D_IMG_TYPE;

typedef struct _IMG_METADATA_T
{
    UINT32                      ui4_src_width;
    UINT32                      ui4_src_height;
}IMG_METADATA_T;

#pragma pack(push, 1)

typedef struct BITMAPFILEHEADER_T
{
    UINT16 ui2_bmpType;
    UINT32 ui4_bmpSize;
    UINT16 ui2_Reserved1;
    UINT16 ui2_Reserved2;
    UINT32 ui4_bmpOffBits;
} BITMAPFILEHEADER_T;

typedef struct BITMAPINFOHEADER_T
{
    UINT32 ui4_Size;
    UINT32 ui4_Width;
    UINT32 ui4_Height;
    UINT16 ui2_Planes;
    UINT16 ui2_BitCount;
    UINT32 ui4_Compression;
    UINT32 ui4_SizeImage;
    UINT32 ui4_XPelsPerMeter;
    UINT32 ui4_YPelsPerMeter;
    UINT32 ui4_ClrUsed;
    UINT32 ui4_ClrImportant;
} BITMAPINFOHEADER_T;

typedef struct RGBQUAD_T
{
    UCHAR uc1_rgbBlue;
    UCHAR uc1_rgbGreen;
    UCHAR uc1_rgbRed;
    UCHAR uc1_rgbReserved;
} RGBQUAD_T;

typedef struct BITMAPINFO_T
{
    BITMAPINFOHEADER_T t_bmpiHeader;
    RGBQUAD_T t_pixelColors[1];
} BITMAPINFO_T;

typedef struct BITMAPFILE_T
{
    BITMAPFILEHEADER_T t_bmpHeader;
    BITMAPINFO_T t_bmpInfo;
}BITMAPFILE_T;

#pragma pack(pop)
typedef struct
{
    HANDLE_T                    h_img;
    IMG_DRC_BUF_T*              pt_buf_crnt_decode;
    IMG_DRC_BUF_T*              pt_buf_crnt_rotate;
    IMG_DRC_BUF_T*              pt_buf_crnt_zoom;
    IMG_DRC_BUF_T*              pt_buf_animation_gif;
    IMG_DRC_BUF_T*              pt_buf_tfx;
    BOOL                        b_is_4K2K;
    E_3D_IMG_TYPE               e_file_type;
}IMG_FILE_BUF_T;


#define FILE_CURRENT_IDX    0
#define FILE_NEXT_IDX       1
#define FILE_TOTAL_IDX      2

#define SVCTX_NAME_MAIN     "main_svctx"


#define _IMG_DRC_DISPLAY_RES_MAX_COUNT       ((UINT8) 4)


typedef struct _IMG_DRC_DATA_T
{
    GL_HSCREEN_T                h_screen;
    GL_HPLANE_T                 h_plane;
    UINT32                      ui4_screen_w;
    UINT32                      ui4_screen_h;
    UINT32                      ui4_canvas_w;
    UINT32                      ui4_canvas_h;
    INT8                        i1_img_z_order;
    UINT32                      ui4_flag;
    GL_POINT_T                  t_screen_offset;
    IMG_DRC_DISPLAY_RES_T      at_disp_pool[_IMG_DRC_DISPLAY_RES_MAX_COUNT];
    IMG_DRC_BUF_T*              pt_buf_crnt_disp;

    IMG_ROTATE_OP_TYPE_T        e_rotate;                       /*rotation of this display*/
    UINT16                      ui2_zoom;

    IMG_FILE_BUF_T              at_file_info[FILE_TOTAL_IDX];

    BOOL                        b_init;

    IMG_METADATA_T              t_image_metadata;

    HANDLE_T                    h_scc_main_vid;
    BOOL                        b_enable_3d;
    HANDLE_T                    h_svctx;

    HANDLE_T                    h_tfx;
    INT32                       i4_status;
    HANDLE_T                    h_tfx_lock;
    BOOL                        b_effect;
    HANDLE_T                    h_mutex_lock;
    BOOL                        b_is_4k2k_panel;
    UINT8                       ui1_crnt_img_idx;
    x_img_event_nfy_fct         pf_img_nfy;
    VOID*                       pv_img_tag;
}   IMG_DRC_DATA_T;

typedef struct
{
    INT32 i4_status;
    VOID* pv_data;
}IMG_TFX_MSG_T;

#define TYPE_OFFSET 100
typedef enum
{
    IMG_NFY_DEC_HANDLE_CLOSED_TYPE = TYPE_OFFSET,
    IMG_NFY_DEC_LOAD_COMPLETE_TYPE,
    IMG_NFY_DEC_LOAD_COMPLETE_WITH_TAG,
    IMG_NFY_DEC_GIF_ONE_FRM_LOAD_COMPLETE_TYPE,
    IMG_NTY_DEC_GIF_LOAD_STOP
} IMG_DEC_NFY_EVENT_TYPE_T;


extern INT32 x_img_playback_rc_init(VOID);
extern INT32 x_img_playback_rc_deinit(VOID);
extern INT32 x_img_playback_open(const CHAR* ps_file_name, x_img_nfy_fct   pf_nfy, const VOID* pv_tag, HANDLE_T* ph_img);
extern INT32 x_img_playback_decode(HANDLE_T h_img);
extern INT32 x_img_playback_display(HANDLE_T h_img);
extern INT32 x_img_playback_close(HANDLE_T h_img);
extern INT32 x_img_playback_get(HANDLE_T h_img, IMG_INFO_TYPE_T e_info_type, VOID* pv_info, UINT32* pui4_len);
extern INT32 x_img_playback_set_3d(HANDLE_T h_img, BOOL b_enable_3d);
extern INT32 x_img_playback_rotate(HANDLE_T h_img, IMG_ROTATE_OP_TYPE_T e_rotate_type);
extern INT32  x_img_playback_zoom(HANDLE_T h_img, UINT16 ui2_zoom);
extern INT32  x_img_playback_pan(HANDLE_T h_img, INT8 i1_horzontal, INT8  i1_vertical);
extern INT32 x_img_playback_tfx_play(HANDLE_T h_img, IMG_TFX_EFFECT_T    e_effect);
extern INT32 x_img_playback_tfx_stop(VOID);
extern INT32 x_img_playback_get_thumbnail(
    const CHAR*         ps_file_name,
    UINT32              ui4_width, 
    UINT32              ui4_height, 
    IMG_COLORMODE_T     e_cm, 
    VOID*               pv_addr,
    UINT32              ui4_buf_len);
extern INT32 x_img_playback_get_thumbnail_to_file(
    const CHAR*         ps_file_name,
    UINT32              ui4_width, 
    UINT32              ui4_height, 
    IMG_COLORMODE_T     e_cm, 
    const CHAR*         ps_thumb_path);
extern INT32 x_img_playback_fm_get_info(const CHAR* ps_file_path, FM_FILE_INFO_T* pt_info);
extern INT32 x_img_playback_reg_callback(x_img_event_nfy_fct  pf_nfy, VOID * pv_tag);
extern INT32 x_img_playback_show_pattern(const CHAR* ps_file_name);
extern INT32 x_img_playback_exit_pattern(VOID);
#endif

