/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_indicator.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains Indicator driver specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_INDICATOR_H_
#define _X_INDICATOR_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define IND_GET_TYPE_CAPABILITY  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define IND_GET_TYPE_STATE       (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))
#define IND_GET_TYPE_MODE        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))
#define IND_GET_TYPE_COLOR       (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 3))

/* Set operations */
#define IND_SET_TYPE_STATE  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1))
#define IND_SET_TYPE_MODE   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 2))
#define IND_SET_TYPE_COLOR  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))


/* Color types */
typedef enum
{
    IND_COLOR_SINGLE = 0,
    IND_COLOR_RED,
    IND_COLOR_GREEN,
    IND_COLOR_YELLOW,
    IND_COLOR_BLUE,
    IND_COLOR_AMBER,
    IND_COLOR_WHITE
}   IND_COLOR_T;

/* Mode types */
typedef enum
{
    IND_MODE_STEADY = 0,
    IND_MODE_BLINK,
    IND_MODE_CIRCLE_FWD,
    IND_MODE_CIRCLE_REV
}   IND_MODE_T;

/* Speed types */
typedef enum
{
    IND_SPEED_SLOW = 0,
    IND_SPEED_MEDIUM,
    IND_SPEED_FAST
}   IND_SPEED_T;

/* State types */
typedef enum
{
    IND_STATE_DISABLE = 0,
    IND_STATE_DIFFUSE,
    IND_STATE_ENABLE
}   IND_STATE_T;


/* Capability color */
#define IND_CAP_COLOR_SINGLE  (((UINT32) 1) << IND_COLOR_SINGLE)
#define IND_CAP_COLOR_RED     (((UINT32) 1) << IND_COLOR_RED)
#define IND_CAP_COLOR_GREEN   (((UINT32) 1) << IND_COLOR_GREEN)
#define IND_CAP_COLOR_YELLOW  (((UINT32) 1) << IND_COLOR_YELLOW)
#define IND_CAP_COLOR_BLUE    (((UINT32) 1) << IND_COLOR_BLUW)
#define IND_CAP_COLOR_AMBER   (((UINT32) 1) << IND_COLOR_AMBER)
#define IND_CAP_COLOR_WHITE   (((UINT32) 1) << IND_COLOR_WHITE)

/* Capability speed */
#define IND_CAP_SPEED_SLOW    (((UINT32) 1) << IND_STATE_SLOW)
#define IND_CAP_SPEED_MEDIUM  (((UINT32) 1) << IND_STATE_MEDIUM)
#define IND_CAP_SPEED_FAST    (((UINT32) 1) << IND_STATE_FAST)

/* Capability state */
#define IND_CAP_STATE_DISABLE  (((UINT32) 1) << IND_STATE_DISABLE)
#define IND_CAP_STATE_DIFFUSE  (((UINT32) 1) << IND_STATE_DIFFUSE)
#define IND_CAP_STATE_ENABLE   (((UINT32) 1) << IND_STATE_ENABLE)

/* Capability information */
typedef struct _IND_CAPABILITY_INFO_T
{
    UINT32  ui4_state;
    UINT32  ui4_color;
    UINT32  ui4_blink_speed;
    UINT32  ui4_circle_fwd_speed;
    UINT32  ui4_circle_rev_speed;
}   IND_CAPABILITY_INFO_T;


/* Mode setting */
typedef struct _IND_MODE_INFO_T
{
    IND_MODE_T   e_mode;
    IND_SPEED_T  e_speed;
}   IND_MODE_INFO_T;


#endif /* _X_INDICATOR_H_ */
