/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*
 * $RCSfile: x_mdb_fs.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/DTV_X_ATSC/2 $
 * $SWAuthor: Bin.liu $
 * $MD5HEX: a72575edb609b7b51f412d6003e7428c $
 *
 * Description: 
 *          This file implements the simulated MDB file system.
 *       
 *---------------------------------------------------------------------------*/

#ifndef X_MDB_MDFS
#define X_MDB_MDFS
#include "x_os.h"
#include "u_mdb.h"
#include "u_mfs.h"

#ifdef ASSERT
#undef ASSERT
#define ASSERT(cond)                                                    \
{                                                                       \
    if (!(cond))                                                        \
    {                                                                   \
        DBG_LOG_PRINT(("[mtpfs] ERR: %s(%u): Assert %s", __FUNCTION__, __LINE__, #cond)); \
        DBG_ASSERT((cond), ~0);                                         \
    }                                                                   \
}
#endif

extern INT32  x_mfs_init(const CHAR *ps_root_path, HANDLE_T h_mdb);

extern INT32  x_mfs_unit(const CHAR *ps_root_path);

extern INT32 x_mfs_open_dir(const CHAR *ps_path,HANDLE_T *ph_dir);
    
extern INT32  x_mfs_seek_dir(HANDLE_T h_dir,UINT8 ui1_whence,INT32  i4_ent_off);

extern INT32 x_mfs_read_dir_entries(
                                                        HANDLE_T        h_dir,
                                                        MFS_DIR_ENTRY_T  *pt_dir_entry,
                                                        UINT32         ui4_count,
                                                        UINT32        *pui4_entries);

extern INT32 x_mfs_get_dir_info(
                                                        const CHAR  *ps_path,
                                                        MFS_DIR_INFO_T *pt_info);

extern INT32 x_mfs_dir_close(HANDLE_T h_file);


typedef struct
{
    INT32 (*x_mfs_init)(const CHAR *ps_root_path, HANDLE_T h_mdb);
    INT32 (*x_mfs_unit)(const CHAR *ps_root_path);
    INT32 (*x_mfs_open_dir)(const CHAR *ps_path,HANDLE_T  *ph_dir);
    INT32 (*x_mfs_seek_dir)(HANDLE_T h_dir,UINT8 ui1_whence,INT32  i4_ent_off);
    INT32 (*x_mfs_read_dir_entries)(
                                                        HANDLE_T        h_dir,
                                                        MFS_DIR_ENTRY_T  *pt_dir_entry,
                                                        UINT32         ui4_count,
                                                        UINT32        *pui4_entries);
    INT32 (*x_mfs_get_dir_info)(
                                                        const CHAR  *ps_path,
                                                        MFS_DIR_INFO_T *pt_info);
    INT32 (*x_mfs_dir_close)(HANDLE_T h_file);


} MFS_IF_T;

#endif //X_MDB_MDFS
