/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_mem_buf.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *   This header file contains exported API for to store and retrive
 *   object from the mini-buffer [an object is an array of bytes.]
 *
 *   The mini-buffer provides API to store object in the buffer
 *   (x_mem_buf_write_obj()).  On success,
 *   this API returns object ID (integer value) which application can
 *   use to retrive the object (get a copy) or to delete this object from
 *   the mini-buffer.
 *
 *   
 *---------------------------------------------------------------------------*/

#ifndef _X_MEM_BUF_H_
#define _X_MEM_BUF_H_

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/

#include "x_common.h"

#include "u_mem_buf.h"

/*-------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

extern INT32 x_mem_buf_create
(
    UINT16           ui2_initial_nb_of_obj,
    UINT32           ui4_initial_buf_sz,
    MEM_BUF_T**      ppt_mem_buf
);

extern INT32 x_mem_buf_write_object
(
    MEM_BUF_T*       pt_mem_buf,
    const VOID*      pt_object,
    UINT32           ui4_len,
    OBJECT_ID_T*     pt_object_id
);


extern INT32 x_mem_buf_get_object_len
(
    MEM_BUF_T*       pt_mem_buf,
    OBJECT_ID_T      t_object_id,
    UINT32*          pui4_len
);

extern INT32 x_mem_buf_get_object
(
    MEM_BUF_T*       pt_mem_buf,
    OBJECT_ID_T      t_object_id,
    VOID*            pv_buf,
    SIZE_T*          pz_str_sz
);

extern INT32 x_mem_buf_cmp_object
(
    MEM_BUF_T*       pt_mem_buf,
    OBJECT_ID_T      t_object_id,
    const VOID*      pt_object,
    UINT32           ui4_len
);

extern INT32 x_mem_buf_del_object
(
    MEM_BUF_T*       pt_mem_buf,
    OBJECT_ID_T      t_object_id
);

extern INT32 x_mem_buf_del_all_objects
(
    MEM_BUF_T*       pt_mem_buf
);
    
extern INT32 x_mem_buf_free
(
    MEM_BUF_T*       pt_mem_buf
);

extern INT32 x_mem_buf_persistent_sz
(
    MEM_BUF_T*       pt_mem_buf
);

extern INT32 x_mem_buf_serialize
(
    MEM_BUF_T*       pt_mem_buf,
    UINT32           aui4_byte_to_copy[],
    UINT8*           aui1_addr_vec[]
);

extern MEM_BUF_T* x_mem_buf_create_from_byte_stream
(
    UINT32           aui4_byte_to_copy[],
    UINT8*           aui1_addr_vec[]
);

extern MEM_BUF_T*  x_mem_buf_dup
(
    MEM_BUF_T*       pt_mem_buf
);

extern VOID x_mem_buf_diagnostic
(
    MEM_BUF_T*  pt_mem_buf
);

#endif   /* _X_MEM_BUF_H_ */
