/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*-----------------------------------------------------------------------------
 * Description: network configuration functions
 *---------------------------------------------------------------------------*/
#ifndef X_NET_CONFIG_H
#define X_NET_CONFIG_H

#include "u_common.h"
#include "u_net_common.h"
#include "u_net_oid.h"


#include <netinet/in.h>


#include "u_dhcpc_api.h"

extern INT32 x_net_network_init(VOID);
extern BOOL x_net_network_check_init(VOID);
extern INT32 x_net_network_deinit(VOID);
extern INT32 x_net_get_if_type (VOID);
extern VOID  x_net_ip_reg_cli(VOID);
extern INT32 x_net_get_dev_num(VOID);
extern INT32 x_net_ip_config(CHAR *psz_name, UINT32 ui4_address, UINT32 ui4_netmask, UINT32 ui4_gw);
extern INT32 x_net_dns_config(UINT32 ui4_dns1, UINT32 ui4_dns2);
extern VOID  x_net_set_hostname(CHAR *ps_hostname);

extern INT32 x_dhcpc_start (CHAR *ps_ni_name, x_dhcpc_nfy_fct fn);
extern INT32 x_dhcpc_stop(CHAR *ps_ni_name);
extern INT32 x_dhcpc_restart(CHAR *ps_ni_name);
extern INT32 x_dhcpc_get_info(CHAR *ps_ni_name, MT_DHCP4_INFO_T *pt_info);

extern INT32 x_net_disable (VOID);
extern INT32 x_net_enable (VOID);

extern INT32 x_net_network_ready (VOID *pv_arg);



extern INT32 x_net_ip_v6_config(CHAR *psz_name, UINT32 ui4_prefix_len, CHAR* pt_ipv6_addr, CHAR* pt_ipv6_gw);
extern INT32 x_net_dns_v6_config(CHAR* pc_dns1, CHAR* pc_dns2);
extern INT32 x_net_ip_v6_auto(CHAR *ps_ni_name, BOOL b_enable, BOOL b_restart_if);
extern INT32 x_net_ip_v6_disable(const CHAR *ps_ni_name);
extern INT32 x_net_ip_v6_dhcp(CHAR *ps_ni_name, BOOL b_enable, INT32 dhcp_type);
extern INT32 x_net_dns_v6_auto(CHAR *ps_ni_name, BOOL b_enable);
extern VOID x_net_ping_v6(CHAR * ps_ip,
                          INT32  i4_len,
                          INT16  i2_wait,
                          VOID (*notify_fn) (INT16 i2_rsp_time));
extern INT32 x_net_is_network_connect_v6(CHAR *ps_url);

extern INT32 x_net_get_v6_info(const CHAR* ps_ni_name, MT_IPV6_INFO_T* pt_info);

extern INT32 x_net_network_reg_nfy_ipv6(x_network_status_nfy pf_nfy, UINT32 ui4_tag, NET_IPV6_MODE_T e_mode);

extern INT32 x_net_set_prefer_v6(BOOL enable);
extern INT32 x_net_ni_rmv6config_all(char* dev);


extern char** x_net_dns_lookup2(const char* hostname, int family, int* size);
extern INT32 x_net_tcp_connect_test(const CHAR *url, int family, int port);

#endif
