/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef X_NET_SMB_H
#define X_NET_SMB_H

#include "u_common.h"
#include "u_net_common.h"

extern INT64 _smb_get_ms (VOID);
extern INT64 _smb_get_tick (VOID);
extern UINT32 _smb_os_get_sys_tick (VOID);

extern INT32 net_smb_dbg;
#define NET_SMB_MODULE_NAME "SMB"
#define SMB_LOG_ERROR(...) do { INT64 i8_cctt = 0; if (net_smb_dbg <= 0) break; i8_cctt = _smb_get_ms(); DBG_LOG_PRINT(("[%s] %lld.%03lld, %s.%d >>> ", NET_SMB_MODULE_NAME, i8_cctt/1000, i8_cctt%1000, __FUNCTION__, __LINE__)); DBG_LOG_PRINT((__VA_ARGS__)); } while (0)
#define SMB_LOG_INFO(...) do { INT64 i8_cctt = 0; if (net_smb_dbg <= 1) break; i8_cctt =_smb_get_ms(); DBG_LOG_PRINT(("[%s] %lld.%03lld, %s.%d >>> ", NET_SMB_MODULE_NAME, i8_cctt/1000, i8_cctt%1000, __FUNCTION__, __LINE__)); DBG_LOG_PRINT((__VA_ARGS__)); } while (0)
#define SMB_LOG_DETAIL(...) do { INT64 i8_cctt = 0; if (net_smb_dbg <= 2) break; i8_cctt =_smb_get_ms(); DBG_LOG_PRINT(("[%s] %lld.%03lld, %s.%d >>> ", NET_SMB_MODULE_NAME, i8_cctt/1000, i8_cctt%1000, __FUNCTION__, __LINE__)); DBG_LOG_PRINT((__VA_ARGS__)); } while (0)

typedef enum {
    NET_SMB_DEVICE_EVENT_ADD,
    NET_SMB_DEVICE_EVENT_REMOVE
} NET_SMB_DEVICE_EVENT_T;

typedef VOID (*x_net_smb_dev_notify) (NET_SMB_DEVICE_EVENT_T e, const CHAR* ps_srv, VOID* pv_tag);

extern INT32 x_net_smb_grps (HANDLE_T *ph_handle);
extern INT32 x_net_smb_grps_show (HANDLE_T h_handle);
extern INT32 x_net_smb_grps_delete (HANDLE_T h_handle);

extern INT32 x_net_smb_servers_group_get (HANDLE_T h_handle, const CHAR *ps_server, CHAR **pps_group);
extern INT32 x_net_smb_servers_group_get2 (const CHAR *ps_server, CHAR **pps_group);
extern INT32 x_net_smb_servers_check_server (HANDLE_T h_handle, const CHAR* ps_server);
extern INT32 x_net_smb_servers_check_server2 (const CHAR* ps_server);
extern INT32 x_net_smb_servers_num (HANDLE_T h_handle);
extern INT32 x_net_smb_servers_get (HANDLE_T h_handle, INT32 i4_idx, CHAR **pps_server);
extern INT32 x_net_smb_servers (HANDLE_T *ph_handle);
extern INT32 x_net_smb_servers_show (HANDLE_T h_handle);
extern INT32 x_net_smb_servers_delete (HANDLE_T h_handle);
extern INT32 x_net_smb_servers_comp (HANDLE_T h_left, HANDLE_T h_right);
extern INT32 x_net_smb_servers_comp_details (HANDLE_T h_left, HANDLE_T h_right, CHAR ***ppps_great, INT32 *pi4_great_num, CHAR ***ppps_less, INT32 *pi4_less_num);
extern INT32 x_net_smb_comp_details_free (CHAR **pps_strs, INT32 i4_num);

extern INT32 x_net_smb_shares_who (HANDLE_T h_handle, CHAR** pps_server);
extern INT32 x_net_smb_shares_num (HANDLE_T h_handle);
extern INT32 x_net_smb_shares_get (HANDLE_T h_handle, INT32 i4_idx, CHAR **pps_share);
extern INT32 x_net_smb_shares (HANDLE_T *ph_handle, const CHAR *ps_server);
extern INT32 x_net_smb_shares_show (HANDLE_T h_handle);
extern INT32 x_net_smb_shares_delete (HANDLE_T h_handle);
extern INT32 x_net_smb_shares_comp (HANDLE_T h_left, HANDLE_T h_right);
extern INT32 x_net_smb_shares_comp_details (HANDLE_T h_left, HANDLE_T h_right, CHAR ***ppps_great, INT32 *pi4_great_num, CHAR ***ppps_less, INT32 *pi4_less_num);

extern INT32 x_net_smb_mount (const CHAR* ps_server, const CHAR* ps_share, const CHAR* ps_dev);
extern INT32 x_net_smb_umount (const CHAR* ps_server, const CHAR* ps_share, const CHAR* ps_dev);
extern INT32 x_net_smb_umount_all (VOID);
extern INT32 x_net_smb_umount_server (const CHAR* ps_server);
extern INT32 x_net_smb_mount_points_show (VOID);

extern INT32 x_net_smb_set_root (const CHAR* ps_root);
extern CHAR* x_net_smb_get_root (VOID);

extern INT32 x_net_smb_init (VOID);
extern INT32 x_net_smb_deinit (VOID);
extern INT32 x_net_smb_start (x_net_smb_dev_notify fn);
extern INT32 x_net_smb_stop (VOID);
extern INT32 x_net_smb_stop_3rd(VOID);

extern INT32 x_net_smb_perms_show (VOID);
extern INT32 x_net_smb_set_perm (const CHAR* ps_server, const CHAR* ps_share, const CHAR* ps_user, const CHAR* ps_passwd, const CHAR* ps_group, INT32 i4_auto);
extern INT32 x_net_smb_free_all_perm (VOID);

extern INT32 x_net_smb_add_server_ex (const CHAR* ps_ip, const CHAR* ps_grp);
extern INT32 x_net_smb_remove_server_ex (const CHAR* ps_ip);
extern INT32 x_net_smb_show_server_ex (VOID);

extern INT32 x_net_smb_ip_cache_show (VOID);
extern INT32 x_net_smb_show_cmd_list (VOID);

extern INT32 x_net_smb_refresh (VOID);
extern INT32 x_net_smb_refresh_ex (VOID);
extern INT32 x_net_smb_mount_points_refresh (HANDLE_T h_handle);


#endif /* X_NET_SMB_H */


