/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_netdrv.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains Demux driver specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_NETDRV_H_
#define _X_NETDRV_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"
#include "x_rm_dev_types.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define NET_GET_TYPE_CTRL           (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  0))

#define NET_GET_TYPE_PROMISC        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  1))

#define NET_GET_TYPE_OWN_HWADDR     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  2))

#define NET_GET_TYPE_HWADDR_COUNT   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  3))

#define NET_GET_TYPE_HWADDR         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  4))


/* Set operations */
#define NET_SET_TYPE_CTRL          ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  0)) | RM_SET_TYPE_ARG_NO_REF)

#define NET_SET_TYPE_PROMISC       ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  1)) | RM_SET_TYPE_ARG_NO_REF)

#define NET_SET_TYPE_MTU           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  2)) | RM_SET_TYPE_ARG_NO_REF)

#define NET_SET_TYPE_IFADDR         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  3))

#define NET_SET_TYPE_ADD_HWADDR     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  4))

#define NET_SET_TYPE_DEL_HWADDR     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  5))

#define NET_SET_TYPE_ADD_MULTI      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  6))

#define NET_SET_TYPE_DEL_MULTI      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  7))

#define NET_SET_TYPE_XMIT_FRAME     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  8))

#define NET_SET_TYPE_UNLOCK_BUFFER  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  9))

#define NET_SET_TYPE_NFY_FCT        (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  10))


/* Notify conditions */
typedef enum
{
    NET_DRV_COND_RECV = 0,
    NET_DRV_COND_RECV_LOWMARK,
    NET_DRV_COND_OVERRUN,
    NET_DRV_COND_XMIT_OK,
    NET_DRV_COND_XMIT_FAIL,
    NET_DRV_COND_HW_RESET
}   NET_DRV_COND_T;


/* Notify function */
typedef BOOL (*x_network_nfy_fct) (VOID*            pv_nfy_tag,
                                   NET_DRV_COND_T   e_nfy_cond,
                                   UINT32           ui4_data_1,
                                   UINT32           ui4_data_2);


/* Notify info setting */
typedef struct _NET_DRV_NFY_INFO_T
{
    VOID*  pv_tag;
    x_network_nfy_fct  pf_network_nfy;
}   NET_DRV_NFY_INFO_T;


typedef struct _NET_DRV_BUFFER_T
{
    UINT8       *pui1_data;
    SIZE_T      z_size;
} NET_DRV_BUFFER_T;


typedef struct _NET_DRV_XMIT_FRAME_T
{
    VOID                *pv_priv_tag;
    UINT32              ui4_count;
    NET_DRV_BUFFER_T    t_buf[3];
} NET_DRV_XMIT_FRAME_T;


typedef struct _NET_DRV_RECV_FRAME_T
{
    VOID        *pv_priv_tag;
    UINT8       *pui1_data;
    SIZE_T      z_size;
} NET_DRV_RECV_FRAME_T;


/* one IF IP address template */
typedef struct _NET_DRV_IFADDR_
{
    UINT32  ui4_ifaddr;
} NET_DRV_IFADDR_T;


/* one HW address template */
typedef struct _NET_HWADDR_
{
    UINT8   ui1_hwaddr[6];
} NET_HWADDR_T;


typedef struct _NET_DRV_HWADDR_
{
    UINT8         ui1_index;   /* index (order) in HW store */
    NET_HWADDR_T  t_hwaddr;    /* data space of "ui1_count" of  NET_HWADDR_T */
} NET_DRV_HWADDR_T;


typedef struct _NET_DRV_MULTIADDR_
{
    NET_HWADDR_T    t_lo;
    NET_HWADDR_T    t_hi;
} NET_DRV_MULTIADDR_T;


/* Unlock buffer */
typedef struct _NET_DRV_UNLOCK_BUFFER_INFO_T
{
    VOID*  pv_drv_tag;

} NET_DRV_UNLOCK_BUFFER_INFO_T;


#endif /* _X_NETDRV_H_ */
