/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

 /*----------------------------------------------------------------------------*/
/*! @file x_netflix_api.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains netflix specific
 *         definitions, which are exported.
 */
/*----------------------------------------------------------------------------*/

#include "u_common.h"
#include "u_netflix_api.h"
#include "u_dbg.h"

#ifdef  DBG_MODULE_NAME_PREFIX
#undef  DBG_MODULE_NAME_PREFIX
#endif
#define DBG_MODULE_NAME_PREFIX netflix_get_log_prefix()      /* log prefix */
extern char* netflix_get_log_prefix(VOID);


/*------------------------------------------------------------------*/
/*! @brief judge if netflix is activated
 *  @return      result
 */
/*------------------------------------------------------------------*/
extern BOOL x_netflix_is_activated(VOID);

/*------------------------------------------------------------------*/
/*! @brief get netflix ESN
 *  @param [in,out] pc_buf - buffer to retrieve esn
 *  @param [in]ui4_len - buffer length
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_netflix_get_esn(CHAR* pc_buf, UINT32 ui4_len);

/*------------------------------------------------------------------*/
/*! @brief get netflix nrdVersion
 *  @param [in,out] pc_buf - buffer to retrieve nrdVersion
 *  @param [in]ui4_len - buffer length
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_netflix_get_nrdVersion(CHAR* pc_buf, UINT32 ui4_len);

/*------------------------------------------------------------------*/
/*! @brief get netflix customer
 *  @param [in,out] pc_buf - buffer to retrieve customer
 *  @param [in]ui4_len - buffer length
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_netflix_get_customer(CHAR* pc_buf, UINT32 ui4_len);

/*------------------------------------------------------------------*/
/*! @brief deactive netflix
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_netflix_deactivate(VOID);

/*------------------------------------------------------------------*/
/*! @brief set netflix idfile
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_nflx_set_idfile(CHAR* p_buf, UINT32 ui4_len);

/*------------------------------------------------------------------*/
/*! @brief get netflix idfile size
 *  @return result
 */
/*------------------------------------------------------------------*/
extern UINT32 x_nflx_getsize_idfile(VOID);

/*------------------------------------------------------------------*/
/*! @brief get netflix idfile
 *  @return result
 */
/*------------------------------------------------------------------*/
extern NETFLIXR_CODE_T x_nflx_get_idfile(CHAR* p_buf, UINT32* ui4_len);

