/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_nvram.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains NVRAM driver specific definitions, which
 *         are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_NVRAM_H_
#define _X_NVRAM_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define NVRAM_GET_TYPE_READBYTE          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define NVRAM_GET_TYPE_SYNCREAD          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))
#define NVRAM_GET_TYPE_SIZE              (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))
#define NVRAM_GET_TYPE_NUMBEROFSECTORS   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 3))
#define NVRAM_GET_TYPE_SECTORTBLINFO     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 4))
#define NVRAM_GET_TYPE_MAXSYNCREADSIZE   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 5))
#define NVRAM_GET_TYPE_MAXSYNCWRITESIZE  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 6))

/* Set operations */
#define NVRAM_SET_TYPE_WRITEBYTE   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0))
#define NVRAM_SET_TYPE_SYNCWRITE   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1))
#define NVRAM_SET_TYPE_ASYNCREAD   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 2))
#define NVRAM_SET_TYPE_ASYNCWRITE  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))
#define NVRAM_SET_TYPE_ERASE       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 4))
#define NVRAM_SET_TYPE_NFY_FCT     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 5))

/* Flags used during registration */
#define NVRAM_FLAG_ASYNCREAD   ((UINT32) 0x00010000)
#define NVRAM_FLAG_ASYNCWRITE  ((UINT32) 0x00020000)


/* Notify conditions */
typedef enum
{
    NVRAM_COND_FAILURE = -1,
    NVRAM_COND_READ    = 1,
    NVRAM_COND_WRITE   = 2,
    NVRAM_COND_ERASE   = 3
} NVRAM_COND_T;


/* Long data info. */
typedef struct _NVRAM_LONG_DATA_INFO_T
{
    SIZE_T  z_offset;
    SIZE_T  z_len;
    UINT8*  pui1_data;
}   NVRAM_LONG_DATA_INFO_T;

/* Sector range info. */
typedef struct _NVRAM_SECTORRANGE_INFO_T
{
    SIZE_T  z_startsector;
    SIZE_T  z_numberofsectors;
}   NVRAM_SECTORRANGE_INFO_T;

/* Sector table info. */
typedef struct _NVRAM_SECTORTBL_INFO_T
{
    SIZE_T  z_numberofsectors;
    SIZE_T* pz_sectorsize;
}   NVRAM_SECTORTBL_INFO_T;

/* Short data info. */
typedef struct _NVRAM_SHORT_DATA_INFO_T
{
    SIZE_T  z_offset;
    UINT8   ui1_data;
}   NVRAM_SHORT_DATA_INFO_T;


/* Notify function */
typedef VOID (*x_nvram_nfy_fct) (VOID*         pv_nfy_tag,
                                 NVRAM_COND_T  e_nfy_cond,
                                 VOID*         pv_data);


/* Notify info. */
typedef struct _NVRAM_NFY_INFO_T
{
    VOID*           pv_tag;
    x_nvram_nfy_fct pf_nvram_nfy;
}   NVRAM_NFY_INFO_T;


#endif /* _X_NVRAM_H_ */
