/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_pwr_ctrl.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: $
 * $SWAuthor: Alec Lu $
 * $MD5HEX: $
 *
 * Description:
 *         This header file contains Power Control driver specific
 *         definitions, which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_PC_H_
#define _X_PC_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "x_common.h"
#include "x_rm.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* Get operations */
#define PC_GET_TYPE_WAKE_UP_CAP             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  0))
#define PC_GET_TYPE_WAKE_UP_CTRL_CAP        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  1))
#define PC_GET_TYPE_WAKE_UP_SETUP           (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  2))
#define PC_GET_TYPE_WAKE_UP_REASON          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  3))
#define PC_GET_TYPE_WAKE_UP_TIME            (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  4))
#define PC_GET_TYPE_MISC_CTRL               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  5))
#define PC_GET_TYPE_MISC_STATUS             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  6))
#define PC_GET_TYPE_GPIO_MODE               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  7))
#define PC_GET_TYPE_GPIO_STATUS             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  8))
#define PC_GET_TYPE_GPIO_MONITOR_CTRL       (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  9))
#define PC_GET_TYPE_SIF                     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 10))
#define PC_GET_TYPE_SIF_ADVANCED            (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 11))
#define PC_GET_TYPE_SIF_BY_GPIO             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 12))
#define PC_GET_TYPE_UPGRADE_CTRL            (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 13))
#define PC_GET_TYPE_SERVO_AD                (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 14))
#define PC_GET_TYPE_POWER_DOWN_REASON       (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 15))
#define PC_GET_TYPE_DEV_ID                  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 16))
#define PC_GET_TYPE_POWER_STATE             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 17))
#define PC_GET_TYPE_GPIO_QUERY               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 18))
#define PC_GET_TYPE_WOL_CTRL                (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 19))
#define PC_GET_TYPE_EEPROM                  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 20))
#define PC_GET_TYPE_UPGRADE_FROM_PARTITION_CTRL                  (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 21))
#define PC_GET_TYPE_WAKEUP_IRKEY           (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 22))
#define PC_GET_TYPE_HOTKEY_NUM               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 23))
#define PC_GET_TYPE_ALL_HOTKEY               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 24))
#define PC_GET_TYPE_QUIET_HOT_BOOT_STATUS   (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 25))
#define PC_GET_TYPE_WIFI_WOL_CTRL          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 26))
#define PC_GET_TYPE_BT_WAKE_UP_CTRL         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 27))


/* Set operations */
#define PC_SET_TYPE_WAKE_UP_SETUP           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  0)) | RM_SET_TYPE_ARG_NO_REF)
#define PC_SET_TYPE_WAKE_UP_TIME            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  1))
#define PC_SET_TYPE_NFY_FCT                 (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  2))
#define PC_SET_TYPE_MISC_CTRL               ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  3)) | RM_SET_TYPE_ARG_NO_REF)
#define PC_SET_TYPE_POWER_DOWN              (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  4))
#define PC_SET_TYPE_REBOOT                  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  5))
#define PC_SET_TYPE_GPIO_MODE               (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  6))
#define PC_SET_TYPE_GPIO_STATUS             (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  7))
#define PC_SET_TYPE_GPIO_MONITOR_CTRL       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  8))
#define PC_SET_TYPE_SIF                     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  9))
#define PC_SET_TYPE_SIF_ADVANCED            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 10))
#define PC_SET_TYPE_SIF_BY_GPIO             (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 11))
#define PC_SET_TYPE_UPGRADE_CTRL            ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 12)) | RM_SET_TYPE_ARG_NO_REF)
#define PC_SET_TYPE_POWER_STATE             (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 13))
#define PC_SET_TYPE_WAKE_UP_REASON          (RM_GET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 14))
#define PC_SET_TYPE_POWER_DOWN_REASON       (RM_GET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 15))
#define PC_SET_TYPE_POWER_SUSPEND           (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  16))
#define PC_SET_TYPE_GPIO_REG                (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  17))
#define PC_SET_TYPE_WOL_CTRL                (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  18))
#define PC_SET_TYPE_NUMBERS_OF_KEY          ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 19)) | RM_SET_TYPE_ARG_NO_REF)
#define PC_SET_TYPE_THE_LAST_OF_FIVE_KEY    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  20))
#define PC_SET_TYPE_EEPROM                  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  21))
#define PC_SET_TYPE_POWER_RESUME            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  22))
#define PC_SET_TYPE_UPGRADE_FROM_PARTITION_CTRL            ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 23)) | RM_SET_TYPE_ARG_NO_REF)
#define PC_SET_TYPE_WIFI_WOL_CTRL           (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  24))
#define PC_SET_TYPE_WAKEUP_IRKEY            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  25))
#define PC_SET_TYPE_POWERDOWN_EVENT         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  26))
#define PC_SET_TYPE_QUIET_HOT_BOOT_STATUS   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  27))
#define PC_SET_TYPE_IR_QUIET_HOT_BOOT_CLEAR (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  28))
#define PC_SET_TYPE_BT_WAKE_UP_CTRL         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  29))


/* Definition of GPIO type */
#define PC_GPIO_TYPE_GPIO           ((UINT32) 0x00000000)
#define PC_GPIO_TYPE_OP_CTRL        ((UINT32) 0x10000000)

/* Wake-up reason */
typedef enum
{
    PC_WAKE_UP_REASON_UNKNOWN = 0,
    PC_WAKE_UP_REASON_VGA,
    PC_WAKE_UP_REASON_RTC,
    PC_WAKE_UP_REASON_FP,
    PC_WAKE_UP_REASON_IRRC,
    PC_WAKE_UP_REASON_UART,
    PC_WAKE_UP_REASON_AC_POWER,
    PC_WAKE_UP_REASON_HDMI,
    PC_WAKE_UP_REASON_UART_NORMAL,
    PC_WAKE_UP_REASON_RC_DIGIT_0,  /* Remote controller */
    PC_WAKE_UP_REASON_RC_DIGIT_1,
    PC_WAKE_UP_REASON_RC_DIGIT_2,
    PC_WAKE_UP_REASON_RC_DIGIT_3,
    PC_WAKE_UP_REASON_RC_DIGIT_4,
    PC_WAKE_UP_REASON_RC_DIGIT_5,
    PC_WAKE_UP_REASON_RC_DIGIT_6,
    PC_WAKE_UP_REASON_RC_DIGIT_7,
    PC_WAKE_UP_REASON_RC_DIGIT_8,
    PC_WAKE_UP_REASON_RC_DIGIT_9,
    PC_WAKE_UP_REASON_RC_DIGIT_11,
    PC_WAKE_UP_REASON_RC_DIGIT_12,
    PC_WAKE_UP_REASON_RC_PRG_UP,
    PC_WAKE_UP_REASON_RC_PRG_DOWN,
    PC_WAKE_UP_REASON_RC_INP_SRC,
    PC_WAKE_UP_REASON_RC_ANALOG,
    PC_WAKE_UP_REASON_RC_DIGITAL,
    PC_WAKE_UP_REASON_RC_DIGITAL_ANALOG,
    PC_WAKE_UP_REASON_FP_PRG_UP,   /* Front panel */
    PC_WAKE_UP_REASON_FP_PRG_DOWN,
    PC_WAKE_UP_REASON_FP_INP_SRC,
    PC_WAKE_UP_REASON_DVD,
    PC_WAKE_UP_REASON_RTC_SPECIAL,
    PC_WAKE_UP_REASON_WATCHDOG,
    PC_WAKE_UP_REASON_SCART,
    PC_WAKE_UP_REASON_ETHERNET,
    PC_WAKE_UP_REASON_RESUME_FROM_SUSPEND,
    PC_WAKE_UP_REASON_WIFI,
    PC_WAKE_UP_REASON_BT,
    PC_WAKE_UP_REASON_RC_NETFLIX,
	PC_WAKE_UP_REASON_CAST,
  	PC_WAKE_UP_REASON_RC_YOUTUBE,	//align to turnkey
	PC_WAKE_UP_REASON_EWBS,		//align to turnkey
	PC_WAKE_UP_REASON_CM4,		//align to turnkey
	
    PC_WAKE_UP_REASON_WOPETHERNET = 60,
    PC_WAKE_UP_REASON_CUSTOM_1 = 61,
    PC_WAKE_UP_REASON_CUSTOM_2 = 62,
    PC_WAKE_UP_REASON_CUSTOM_3 = 63,
    PC_WAKE_UP_REASON_CUSTOM_4 = 64,
    //add new 16 wakeup reason
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW1 = 65,
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW2 = 66,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW3 = 67,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW4 = 68,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW5 = 69,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW6 = 70,
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW7 = 71,
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW8 = 72,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW9 = 73,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW10 = 74,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW11 = 75,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW12 = 76,
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW13 = 77,
	PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW14 = 78,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW15 = 79,
    PC_PDWNC_WAKE_UP_REASON_CUSTOM_POWER_NEW16 = 80,
    PC_PDWNC_WAKE_UP_REASON_PH_TMCC = 81
}   PC_WAKE_UP_REASON_T;

/* Wake-up sub-reason mask */
#define PC_WAKE_UP_SUB_REASON_QUIET_BOOT                MAKE_BIT_MASK_32(0)
#define PC_WAKE_UP_SUB_REASON_CUSTOM_1             MAKE_BIT_MASK_32(28)    /**<   Customized Wake-up sub-reason mask indicates the quiet boot.   */
#define PC_WAKE_UP_SUB_REASON_CUSTOM_2             MAKE_BIT_MASK_32(29)    /**<   Customized Wake-up sub-reason mask indicates the quiet boot.   */
#define PC_WAKE_UP_SUB_REASON_CUSTOM_3             MAKE_BIT_MASK_32(30)    /**<   Customized Wake-up sub-reason mask indicates the quiet boot.   */
#define PC_WAKE_UP_SUB_REASON_CUSTOM_4             MAKE_BIT_MASK_32(31)    /**<   Customized Wake-up sub-reason mask indicates the quiet boot.   */


/* Power-down reason */
typedef enum
{
    PC_POWER_DOWN_REASON_UNKNOWN = 0,
    PC_POWER_DOWN_REASON_FP,
    PC_POWER_DOWN_REASON_IRRC,
    PC_POWER_DOWN_REASON_CUSTOM_1 = 61,
    PC_POWER_DOWN_REASON_CUSTOM_2 = 62,
    PC_POWER_DOWN_REASON_CUSTOM_3 = 63,
    PC_POWER_DOWN_REASON_CUSTOM_4 = 64
}   PC_POWER_DOWN_REASON_T;

/* Reset mode */
typedef enum
{
    PC_RESET_MODE_USER = 0,
    PC_RESET_MODE_FACTORY,
    PC_RESET_MODE_CUSTOM_1,
    PC_RESET_MODE_CUSTOM_2,
    PC_RESET_MODE_CUSTOM_3,
    PC_RESET_MODE_CUSTOM_4
}   PC_RESET_MODE_T;

/* Power state control */
typedef enum
{
    PC_PWR_STATE_CTRL_STANBY = 0, /* Only backlight is off but CPU is still running */
    PC_PWR_STATE_CTRL_STANBY_2,
    PC_PWR_STATE_CTRL_SEMI_STANBY,
    PC_PWR_STATE_CTRL_RUNNING,
    PC_PWR_STATE_CTRL_NORMAL,
    PC_PWR_STATE_CTRL_LAST_VALID_ENTRY
}   PC_PWR_STATE_CTRL_T;

/* GPIO mode */
typedef enum
{
    PC_GPIO_MODE_INPUT = 0,
    PC_GPIO_MODE_OUTPUT
}   PC_GPIO_MODE_T;

/* GPIO status */
typedef enum
{
    PC_GPIO_STATUS_UNKNOWN = 0,
    PC_GPIO_STATUS_LOW,
    PC_GPIO_STATUS_HIGH
}   PC_GPIO_STATUS_T;

/* GPIO monitoring control */
/*
    For PC_GET_TYPE_GPIO_MONITOR_CTRL and PC_SET_TYPE_GPIO_MONITOR_CTRL operations,
    it refers to an UINT64 variable where:
    [55:0] => GPIO 0~55 monitoring control
    [63:56] => OPCTRL 0~7 monitoring control

    As to the notification, ui4_data is used as follows when e_nfy_cond = PC_COND_GPIO_UPDATE
    [15:0] => GPIO/OPCTRL ID
    [31:16] => GPIO/OPCTRL status (0=LOW, 1=HIGH)
*/

/* Wake-up info */
typedef struct _PC_WAKE_UP_INFO_T
{
    PC_WAKE_UP_REASON_T     e_reason;
    VOID*                   pv_data;
}   PC_WAKE_UP_INFO_T;

/* Power-down info */
typedef struct _PC_POWER_DOWN_INFO_T
{
    PC_POWER_DOWN_REASON_T  e_reason;
    VOID*                   pv_data;
}   PC_POWER_DOWN_INFO_T;

/* GPIO mode info */
typedef struct _PC_GPIO_MODE_INFO_T
{
    UINT32              ui4_id;
    PC_GPIO_MODE_T      e_mode;
}   PC_GPIO_MODE_INFO_T;
/* GPIO mode info */
typedef struct _PC_GPIO_MONITOR_CTRL_EX_T
{
    UINT32    ui4_gpio;
    BOOL      b_monitor;
}   PC_GPIO_MONITOR_CTRL_EX_T;


/* GPIO status info */
typedef struct _PC_GPIO_STATUS_INFO_T
{
    UINT32              ui4_id;
    PC_GPIO_STATUS_T    e_status;
}   PC_GPIO_STATUS_INFO_T;

/* EEPROM info */
typedef struct _PC_EEPROM_INFO_T
{
    UINT32      ui4_offset;
    UINT8       ui1_data;
}   PC_EEPROM_INFO_T;


/* SIF info */
typedef struct _PC_SIF_INFO_T
{
    UINT32      ui4_addr;
    UINT32      ui4_offset;
    UINT8       ui1_data;
}   PC_SIF_INFO_T;

/* SIF info (advanced version) */
typedef enum
{
    PC_SIF_CH_ID_SYS_BUS    = 0x00,
    PC_SIF_CH_ID_TUNER_BUS  = 0x01,
    PC_SIF_CH_ID_EDID_BUS   = 0x10
}   PC_SIF_CH_ID_T;

typedef struct _PC_SIF_INFO_ADVANCED_T
{
    PC_SIF_CH_ID_T  e_channel_id;
    UINT32          ui4_addr;       /* Device address */
    UINT32          ui4_offset;     /* Sub-address (or called offset) */
    UINT8           ui1_offset_len; /* Offset length: 0~3 */
    UINT8*          pui1_data;
    UINT8           ui1_data_len;
}   PC_SIF_INFO_ADVANCED_T;

/* SIF info by GPIO */
typedef struct _PC_SIF_INFO_BY_GPIO_T
{
    UINT32      ui4_addr_gpio_id;
    UINT32      ui4_data_gpio_id;
    UINT32      ui4_addr;       /* Device address */
    UINT32      ui4_offset;     /* Sub-address (or called offset) */
    UINT8       ui1_offset_len; /* Offset length: 0~3 */
    UINT8*      pui1_data;
    UINT8       ui1_data_len;
}   PC_SIF_INFO_BY_GPIO_T;

/* Servo AD info */
typedef struct _PC_SERVO_AD_INFO_T
{
    UINT32      ui4_id;
    UINT32      ui4_data;
}   PC_SERVO_AD_INFO_T;

/* Notify condition */
typedef enum
{
    PC_COND_UNKNOWN = 0,
    PC_COND_WAKE_UP_EXPIRED,
    PC_COND_GPIO_UPDATED,
    PC_COND_WAKE_UP_TIMER_SETUP_NORMAL,  /* TWOWORLDS only. ui4_data will be RTC */
    PC_COND_WAKE_UP_TIMER_SETUP_BGM,     /* TWOWORLDS only. ui4_data will be RTC */
    PC_COND_WAKE_UP_TIMER_SETUP_CLEAR    /* TWOWORLDS only. */
}   PC_COND_T;

/* Notify function */
typedef VOID (*x_pc_nfy_fct)(VOID*      pv_nfy_tag,
                             PC_COND_T  e_nfy_cond,
                             UINT32     ui4_data);

/* Notify setting structure */
typedef struct _PC_NFY_INFO_T
{
    VOID*           pv_tag;
    x_pc_nfy_fct    pf_nfy;
}   PC_NFY_INFO_T;

/* Wake-up capabilities (for wake-up setup & capability operations) */
#define PC_WAKE_UP_CAP_VGA                  MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_VGA - 1)
#define PC_WAKE_UP_CAP_RTC                  MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RTC - 1)
#define PC_WAKE_UP_CAP_FP                   MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_FP - 1)
#define PC_WAKE_UP_CAP_IRRC                 MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_IRRC - 1)
#define PC_WAKE_UP_CAP_UART                 MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_UART - 1)
#if 0
#define PC_WAKE_UP_CAP_AC_POWER             MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_UART - 1)
#define PC_WAKE_UP_CAP_HDMI                 MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_HDMI - 1)
#define PC_WAKE_UP_CAP_UART_NORMAL          MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_UART_NORMAL - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_0           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_0 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_1           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_1 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_2           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_2 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_3           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_3 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_4           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_4 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_5           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_5 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_6           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_6 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_7           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_7 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_8           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_8 - 1)
#define PC_WAKE_UP_CAP_RC_DIGIT_9           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_DIGIT_9 - 1)
#define PC_WAKE_UP_CAP_RC_PRG_UP            MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_PRG_UP - 1)
#define PC_WAKE_UP_CAP_RC_PRG_DOWN          MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_PRG_DOWN - 1)
#define PC_WAKE_UP_CAP_RC_INP_SRC           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_RC_INP_SRC - 1)
#define PC_WAKE_UP_CAP_FP_PRG_UP            MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_FP_PRG_UP - 1)
#define PC_WAKE_UP_CAP_FP_PRG_DOWN          MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_FP_PRG_DOWN - 1)
#define PC_WAKE_UP_CAP_FP_INP_SRC           MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_FP_INP_SRC - 1)
#define PC_WAKE_UP_CAP_CUSTOM_1             MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_CUSTOM_1 - 1)
#define PC_WAKE_UP_CAP_CUSTOM_2             MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_CUSTOM_2 - 1)
#define PC_WAKE_UP_CAP_CUSTOM_3             MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_CUSTOM_3 - 1)
#define PC_WAKE_UP_CAP_CUSTOM_4             MAKE_BIT_MASK_32(PC_WAKE_UP_REASON_CUSTOM_4 - 1)
#endif

/* Miscellaneous controls */
/*
    Operation | PC_MISC_CTRL_NO_STANDBY_2 PC_MISC_CTRL_NO_STANDBY
    ---------------------------------------------------------------
    Standby   |            0                       0
    On        |            0                       1
    Last sts  |            1                       x
*/
#define PC_MISC_CTRL_LAST_STS                   PCL_MISC_CTRL_NO_STANDBY_2

#define PC_MISC_CTRL_NO_STANDBY                 MAKE_BIT_MASK_32(0) /* This bit is valid when PC_MISC_CTRL_NO_STANDBY_2 = 0 */
#define PC_MISC_CTRL_FACTORY_MODE               MAKE_BIT_MASK_32(1)
#define PC_MISC_CTRL_POWER_LED_OFF              MAKE_BIT_MASK_32(2)
#define PC_MISC_CTRL_NO_STANDBY_2               MAKE_BIT_MASK_32(3)
#define PC_MISC_CTRL_FIRMWARE_UPGRADE           MAKE_BIT_MASK_32(4)
#define PC_MISC_CTRL_COMPANION_CHIP_UPGRADE     MAKE_BIT_MASK_32(5)
#define PC_MISC_CTRL_QUIET_BOOT                 MAKE_BIT_MASK_32(6) /* Backlight is off and power LED is unchanged */
#define PC_MISC_CTRL_DUAL_BANK                  MAKE_BIT_MASK_32(7)
#define PC_MISC_CTRL_NO_IR_STANDBY_BY_DRIVER    MAKE_BIT_MASK_32(8)
#define PC_MISC_CTRL_DUAL_BANK_SYSTEM           MAKE_BIT_MASK_32(9) /* index 0/1 for system image and 8032 up */
#define PC_MISC_CTRL_DUAL_BANK_ROOTFS           MAKE_BIT_MASK_32(10) /* index 0/1 for root file system */
#define PC_MISC_CTRL_DUAL_BANK_KERNEL           MAKE_BIT_MASK_32(11) /* index 0/1 for kernel */
#define PC_MISC_CTRL_FAST_BOOT                  MAKE_BIT_MASK_32(12) /* index 0/1 for kernel */
#define PC_MISC_CTRL_NORMAL_BOOT_WITH_SUSPEND   MAKE_BIT_MASK_32(13) /* index 0/1 for kernel */
#define PC_MISC_CTRL_NORMAL_BOOT_WITHOUT_SUSPEND    MAKE_BIT_MASK_32(14) /* index 0/1 for kernel */
#define PC_MISC_CTRL_DUAL_BANK_CAST_SYSTEM      MAKE_BIT_MASK_32(15)
#define PC_MISC_CTRL_DUAL_BANK_3RD              MAKE_BIT_MASK_32(16)
#define PC_MISC_CTRL_DUAL_BANK_CHROME           MAKE_BIT_MASK_32(17)
#define PC_MISC_CTRL_AMBER_LED_ON               MAKE_BIT_MASK_32(18) /* Control of Amber LED on/off */ /* 18 ~ 23, porting from Sxxx branch */
#define PC_MISC_CTRL_FAST_CLOCK                 MAKE_BIT_MASK_32(19) /* FAST CLOCK */
#define PC_MISC_CTRL_APP_CTRL_POWER             MAKE_BIT_MASK_32(20)  /* Control of Power status */
#define PC_MISC_CTRL_APP_OAD_STANDBY            MAKE_BIT_MASK_32(21)  /* Control of Power status for OAD upgrade*/
#define PC_MISC_CTRL_APP_HOTEL_MODE             MAKE_BIT_MASK_32(22)  /* Control of Power status for OAD upgrade*/
#define PC_MISC_CTRL_NO_LOG_FLAG                MAKE_BIT_MASK_32(23)  /* Control No log flag: 0 no log; 1 ignore */
#define PC_MISC_CTRL_DUAL_BANK_LINUX_ROOTFS     MAKE_BIT_MASK_32(24)  /* for CastTV two-worlds */


/* Miscellaneous status */
#define PC_MISC_STS_POWER_DOWN_BY_AC_CUT        MAKE_BIT_MASK_32(0)

/* Upgrade controls */
#define PC_UPGRADE_CTRL_NOR_0                   MAKE_BIT_MASK_32(0)
#define PC_UPGRADE_CTRL_NOR_1                   MAKE_BIT_MASK_32(1)
#define PC_UPGRADE_CTRL_NOR_2                   MAKE_BIT_MASK_32(2)
#define PC_UPGRADE_CTRL_NOR_3                   MAKE_BIT_MASK_32(3)
#define PC_UPGRADE_CTRL_NOR_4                   MAKE_BIT_MASK_32(4)
#define PC_UPGRADE_CTRL_NOR_5                   MAKE_BIT_MASK_32(5)
#define PC_UPGRADE_CTRL_NOR_6                   MAKE_BIT_MASK_32(6)
#define PC_UPGRADE_CTRL_NOR_7                   MAKE_BIT_MASK_32(7)
#define PC_UPGRADE_CTRL_NVM                     MAKE_BIT_MASK_32(8)
#define PC_UPGRADE_CTRL_STANDBY_MICRO_P         MAKE_BIT_MASK_32(9)
#define PC_UPGRADE_CTRL_COMPANION_CHIP          MAKE_BIT_MASK_32(10)

/* Device ID */
typedef struct _PC_DEV_ID_T
{
    UINT8   aui1_dev_id[8];
} PC_DEV_ID_T;

/*Ethernet Wol Ctrl */
typedef enum
{
    PC_ETHERNET_WOL_DISABLE = 0,
    PC_ETHERNET_WOL_ENABLE
}   PC_Ethernet_WOL_CTRL_T;

/*Wifi Wol Ctrl */
typedef enum
{
    PC_WIFI_WOL_DISABLE = 0,
    PC_WIFI_WOL_ENABLE
}PC_WIFI_WOL_CTRL_T;

/*BT wake up Ctrl*/
typedef enum
{
    PC_WAKE_UP_ON_BT_DISABLE = 0,
    PC_WAKE_UP_ON_BT_ENABLE
}PC_WAKE_UP_ON_BT_CTRL_T;



typedef enum
{
    PC_IRRX_WAKEUPKEY_NONE         = 0,
    PC_IRRX_WAKEUPKEY_NORMAL     = 1,
    PC_IRRX_WAKEUPKEY_HOTKEY ,
}PC_IRRC_WAKKEY_TYPE;

typedef struct _PC_IRRC_WAKKEY_T
{
    UINT32 ui4_WakeupKey;
    PC_IRRC_WAKKEY_TYPE e_fgIrkey;
    UINT32 ui4_Raw;
}PC_IRRC_WAKKEY_T;

typedef struct _PC_IRRC_ALL_HOTKEY_T
{
    UINT32 *pui4_hotkey;
    UINT8     u1_keynum;
}PC_IRRC_ALL_HOTKEY_T;

typedef enum
{
    PC_QUIET_HOT_BOOT_DISABLE = 0,
    PC_QUIET_HOT_BOOT_ENABLE
}   PC_QUIET_HOT_BOOT_STATUS_T;

#endif /* _X_PC_H_ */

