/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile:
 * $Revision:
 * $Date:
 * $Author:
 *
 * Description: 
 *         This header file contains RSA driver specific definitions, which
 *         are exported.
 *---------------------------------------------------------------------------*/

#ifndef X_RSA_H
#define X_RSA_H

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "x_common.h"
#include "x_rm.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* Get operations */
#define CRYPTO_GET_TYPE_RSA_PRV_ENC        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define CRYPTO_GET_TYPE_RSA_PUB_DEC        (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))

/* Data structure for RSA private key */
typedef struct _RSA_PRIVATE_KEY_T
{
    UINT16      ui2_bits; /* length in bits of modulus */
    UINT8*      pui1_modulus;
    UINT32      ui4_modulus_len;
    UINT8*      pui1_public_exponent;
	UINT32      ui4_public_exponent_len;    
	UINT8*      pui1_private_exponent;
	UINT32      ui4_private_exponent_len;
	UINT8*      pui1_prime1;
	UINT32      ui4_prime1_len;
	UINT8*      pui1_prime2;
	UINT32      ui4_prime2_len;
	UINT8*      pui1_prime1_exponent;
	UINT32      ui4_prime1_exponent_len;	
	UINT8*      pui1_prime2_exponent;
	UINT32      ui4_prime2_exponent_len;
	UINT8*      pui1_coefficient;
	UINT32      ui4_coefficient_len;				
}   RSA_PRIVATE_KEY_T;

/* Data structure for RSA public key */
typedef struct _RSA_PUBLIC_KEY_T
{
    UINT16      ui2_bits; /* length in bits of modulus */
    UINT8*      pui1_modulus;
    UINT32      ui4_modulus_len;
    UINT8*      pui1_public_exponent;
	UINT32      ui4_public_exponent_len;    
}   RSA_PUBLIC_KEY_T;

/* RSA private encryption data info */
typedef struct _RSA_PRV_ENC_DATA_T
{
	UINT8*              pui1_input;
	UINT32              ui4_input_len;		
	RSA_PRIVATE_KEY_T   t_rsa_private_key;
	UINT8*              pui1_output;
	UINT32              ui4_output_len;
} RSA_PRV_ENC_DATA_T;

/* RSA public descryption data info */
typedef struct _RSA_PUB_DEC_DATA_T
{
	UINT8*              pui1_input;
	UINT32              ui4_input_len;		
	RSA_PUBLIC_KEY_T    t_rsa_public_key;
	UINT8*              pui1_output;
	UINT32              ui4_output_len;
} RSA_PUB_DEC_DATA_T;

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/           

#endif	/* X_RSA_H */

