/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_sm_audio_hdlr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Audio Stream handler specific definitions,
 *         which are controled by Stream Manager.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_STRM_MNGR_AUDIO_HDLR Audio Handler
 *
 *  @ingroup groupMW_STRM_MNGR
 *  @brief Provides Macros, defines, typedefs enums and struct in audio handler for TV Application
 *
 *
 *  @see groupMW_STRM_MNGR
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _X_SM_AUDIO_HDLR_H_
#define _X_SM_AUDIO_HDLR_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_sm_audio_hdlr.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/

/*data strcuct for auto mute parameter*/
/*------------------------------------------------------------------*/
/*! @struct
 *  @brief
 *  @code
 *  typedef struct _SM_AS_AUTO_MUTE_PARA_T
 *  {  
 *      INT8 i1_lost_rate_up_bound;
 *      INT8 i1_lost_rate_low_bound;
 *      INT8 i1_error_rate_bound;
 *      INT8 i1_react_freq;
 *  }SM_AS_AUTO_MUTE_PARA_T;
 *  @endcode
 *  @li@c  i1_lost_rate_up_bound                             -
 *  @li@c  i1_lost_rate_low_bound                            -
 *  @li@c  i1_error_rate_bound                               -
 *  @li@c  i1_react_freq                                     -
 */
/*------------------------------------------------------------------*/
typedef struct _SM_AS_AUTO_MUTE_PARA_T
{  
    INT8 i1_lost_rate_up_bound;
    INT8 i1_lost_rate_low_bound;
    INT8 i1_error_rate_bound;
    INT8 i1_react_freq;
}SM_AS_AUTO_MUTE_PARA_T;


/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief   Init auto mute parameter .
 *  @param [in] SM_AS_AUTO_MUTE_PARA_T  struct contain lost rate, error rate and react frequece.
 *  @param [out]
 *  @note
 *  @see  x_sm_ash_auto_mute_para_init
 *  @return
 *  @retval      -SMR_INTERNAL_ERROR     inpute parameter error.
 *  @retval      -SMR_ok                         init successful.
 */
/*------------------------------------------------------------------*/
extern INT32  x_sm_ash_auto_mute_para_init(SM_AS_AUTO_MUTE_PARA_T* pt_as_auto_mute_para);            
#endif /* _X_SM_AUDIO_HDLR_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */                                             /* end of groupMW_STRM_MNGR_AUDIO_HDLR */
/*----------------------------------------------------------------------------*/

