/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_sm_cc_hdlr.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/11 $
 * $SWAuthor: Iolo Tsai $
 * $MD5HEX: f6ce0384875c26c9670d7f9f39606dc7 $
 *
 * Description: 
 *         This header file contains Closed Caption Stream handler specific 
 *         definitions, which are exported to applications.
 *---------------------------------------------------------------------------*/

#ifndef _X_SM_CC_HDLR_H_
#define _X_SM_CC_HDLR_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_sm_cc_hdlr.h"
#include "u_sm_event.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
#define SM_EVN_CCS_GRP_SIZE  ((SM_EVN_TYPE_T) 0x00010000)
    
#define SM_EVN_CCS_GRP_APP   ((SM_EVN_TYPE_T) SM_EVN_GRP_CCS + SM_EVN_CCS_GRP_SIZE*0 )
#define SM_EVN_CCS_GRP_DEC   ((SM_EVN_TYPE_T) SM_EVN_GRP_CCS + SM_EVN_CCS_GRP_SIZE*1 )
#define SM_EVN_CCS_GRP_TVE   ((SM_EVN_TYPE_T) SM_EVN_GRP_CCS + SM_EVN_CCS_GRP_SIZE*2 )
    
#define SM_EVN_CCS_CODE_DRV_OPENED       ((SM_EVN_TYPE_T) 0)
#define SM_EVN_CCS_CODE_DRV_CLOSED       ((SM_EVN_TYPE_T) 1)
#define SM_EVN_CCS_CODE_DRV_PLAYED       ((SM_EVN_TYPE_T) 2)
#define SM_EVN_CCS_CODE_DRV_STOPPED      ((SM_EVN_TYPE_T) 3)
    
#define SM_MAKE_DRV_EVN( grp, code ) ((SM_EVN_TYPE_T) grp + code )
    
#define SM_EVN_CCS_DEC_OPENED         SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_DEC, SM_EVN_CCS_CODE_DRV_OPENED )
#define SM_EVN_CCS_DEC_CLOSED         SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_DEC, SM_EVN_CCS_CODE_DRV_CLOSED )
#define SM_EVN_CCS_DEC_CC_PLAYED      SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_DEC, SM_EVN_CCS_CODE_DRV_PLAYED )
#define SM_EVN_CCS_DEC_CC_STOPPED     SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_DEC, SM_EVN_CCS_CODE_DRV_STOPPED )
    
#define SM_EVN_CCS_TVE_OPENED         SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_TVE, SM_EVN_CCS_CODE_DRV_OPENED )
#define SM_EVN_CCS_TVE_CLOSED         SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_TVE, SM_EVN_CCS_CODE_DRV_CLOSED )
#define SM_EVN_CCS_TVE_PLAYED         SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_TVE, SM_EVN_CCS_CODE_DRV_PLAYED )
#define SM_EVN_CCS_TVE_STOPPED        SM_MAKE_DRV_EVN( SM_EVN_CCS_GRP_TVE, SM_EVN_CCS_CODE_DRV_STOPPED )


#define SM_CC_HDLR_MSGQ_DFLT_MAX_SIZE           1024
#define SM_CC_HDLR_THRD_DFLT_PRIORITY           127
#define SM_CC_HDLR_THRD_DFLT_STACK_SIZE         4096*2


/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
extern INT32 x_sm_cc_hdlr_init( const SM_CCH_CONFIG_T* pt_config );

extern INT32 x_sm_cc_hdlr_set_dtv_cc_att( 
    const SM_CCH_DTVCC_ATTRIBS_INFO_T* pt_dtvcc_att );

#endif /* _X_SM_CC_HDLR_H_ */
