/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_sm_sbtl_hdlr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Subtitle Stream handler specific 
 *         definitions, which are exported to applications.
 */
/*----------------------------------------------------------------------------*/

#ifndef _X_SM_SBTL_HDLR_H_
#define _X_SM_SBTL_HDLR_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_sm_sbtl_hdlr.h"

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_SBTL_HDLR
*  @{  
*/
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief Function prototype of subtitle ROM object render function
 *  @param [in]     ui2_obj_id                  - Object ID
 *  @param [in]     ui1_obj_type                - Object type
 *  @param [in]     pt_sbtl_render_info         - Object position and color
 *  @param [out]
 *  @note
 *  @see
 *  @return
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef INT32 (*sm_sbtl_rom_render_fct)(
    UINT16                      ui2_obj_id, 
    UINT8                       ui1_obj_type,
    SM_SBTLH_RENDER_INFO_T*     pt_sbtl_render_info);


/*------------------------------------------------------------------*/
/*! @brief Function prototype of subtitle text object
 *  @param [in]     pui2_text                   - Text array
 *  @param [in]     ui2_text_len                - Text array length
 *  @param [in]     pt_sbtl_render_info         - Object position and color
 *  @param [out]    
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      SMR_OK                 - The routine was successful.
 *  @retval      SMR_INTERNAL_ERROR     - The routine was fail.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*sm_sbtl_text_render_fct)(
    UINT16*                     pui2_text,
    UINT16                      ui2_text_len,
    SM_SBTLH_RENDER_INFO_T*     pt_sbtl_render_info);

/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief Initialize the DVB subtitle configuration
 *  @param [in]     pt_config                   - Configuration context
 *  @param [out]
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      SMR_OK                 - The routine was successful.
 *  @retval      SMR_INTERNAL_ERROR     - The routine was fail.
 */
/*------------------------------------------------------------------*/
extern INT32 x_sm_sbtl_hdlr_init(const SM_SBTLH_CONFIG_T* pt_config);

/*------------------------------------------------------------------*/
/*! @brief Register rendering function for object provided by a ROM in the IRD
 *  @param [in]     pf_sbtl_rom_render_fct      - Function pointer
 *  @param [out]
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      SBTLR_OK               - The routine was successful.
 *  @retval      SBTLR_INV_ARG          - pf_sbtl_rom_render_fct is NULL.
 */
/*------------------------------------------------------------------*/
extern INT32 x_sm_sbtl_hdlr_reg_rom_render(
    sm_sbtl_rom_render_fct pf_sbtl_rom_render_fct);

/*------------------------------------------------------------------*/
/*! @brief Register rendering function for text type object
 *  @param [in]     pf_sbtl_text_render_fct     - Function pointer
 *  @param [out]    
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      SBTLR_OK               - The routine was successful.
 *  @retval      SBTLR_INV_ARG          - pf_sbtl_text_render_fct is NULL.
 */
/*------------------------------------------------------------------*/
extern INT32 x_sm_sbtl_hdlr_reg_text_render(
    sm_sbtl_text_render_fct pf_sbtl_text_render_fct);

#endif /* _X_SM_SBTL_HDLR_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                            /* end of groupMW_STRM_MNGR_SBTL_HDLR */
/*----------------------------------------------------------------------------*/


