/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_str_buf.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *   This header file contains exported API for to
 *   store and retrive character string from the mini-buffer.
 *
 *   The mini-buffer provides API to store char string (NULL terminated)
 *   string in the buffer (x_str_buf_write_string()).  On success,
 *   this API returns string ID (integer value) which application can
 *   use to retrive the string (get a copy) or to delete this string from
 *   the mini-buffer.
 *
 *   
 *---------------------------------------------------------------------------*/

#ifndef _X_STR_BUF_H_
#define _X_STR_BUF_H_

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/

#include "x_common.h"

#include "u_str_buf.h"

/*-------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

extern INT32 x_str_buf_create
(
    UINT16           ui2_initial_nb_of_str,
    UINT32           ui4_initial_buf_sz,
    STR_BUF_T**      ppt_strbuf
);

extern INT32 x_str_buf_write_string
(
    STR_BUF_T*       pt_strbuf,
    const CHAR*      pt_string,
    STRING_ID_T*     pt_string_id
);


extern INT32 x_str_buf_get_strlen
(
    STR_BUF_T*       pt_strbuf,
    STRING_ID_T      t_string_id,
    UINT32*          pui4_len
);

extern INT32 x_str_buf_get_string
(
    STR_BUF_T*       pt_strbuf,
    STRING_ID_T      t_string_id,
    CHAR*            ps_string,
    SIZE_T*          pz_str_sz
);

extern INT32 x_str_buf_cmp_string
(
    STR_BUF_T*       pt_strbuf,
    STRING_ID_T      t_string_id,
    const CHAR*      ps_string
);

extern INT32 x_str_buf_del_string
(
    STR_BUF_T*       pt_strbuf,
    STRING_ID_T      t_string_id
);

extern INT32 x_str_buf_del_all_string
(
    STR_BUF_T*       pt_strbuf
);
    
extern INT32 x_str_buf_free
(
    STR_BUF_T*       pt_strbuf
);

extern INT32 x_str_buf_persistent_sz
(
    STR_BUF_T*       pt_strbuf
);

extern INT32 x_str_buf_serialize
(
    STR_BUF_T*       pt_strbuf,
    UINT32           aui4_byte_to_copy[],
    UINT8*           aui1_addr_vec[]
);

extern STR_BUF_T* x_str_buf_create_from_byte_stream
(
    UINT32           aui4_byte_to_copy[],
    UINT8*           aui1_addr_vec[]
);

extern STR_BUF_T*  x_str_buf_dup
(
    STR_BUF_T*       pt_strbuf
);

extern VOID x_str_buf_diagnostic(STR_BUF_T*       pt_strbuf);

#endif   /* _X_STR_BUF_H_ */
