/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_wgl_common.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/3 $
 * $SWAuthor: Al Hsieh $
 * $MD5HEX: ee6e2de6d3dc3066e4041b8ccac1cde2 $
 *
 * Description: 
 *         This header file contains public interfaces, constant and type 
 *         definitions of the Widget Library.
 *---------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_WGL   Widget Library
 *
 *  @ingroup groupMW
 *  @brief Provides constant,type definitions and function of the widget library.
 *
 *  %MW Application API would be used by TV Application, lot of these APIs would
 *   be wrapped from Exported API
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/ 
#ifndef _X_WGL_COMMON_H_
#define _X_WGL_COMMON_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "x_common.h"
#include "u_wgl_common.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#define WGL_STATE_ENABLED               ((UINT16) 0x0001)/**< Indicates the specified widget is enabled. */
#define WGL_STATE_VISIBLE               ((UINT16) 0x0002)/**< Indicates the specified widget is visible. */
#define WGL_STATE_ACTIVATED             ((UINT16) 0x0004)/**< Indicates the specified widget is activated. */
#define WGL_STATE_FOCUSED               ((UINT16) 0x0008)/**< Indicates the specified widget has the focus. */
#define WGL_STATE_PUSHED                ((UINT16) 0x0010)/**< Indicates the specified widget is in pushed state. */


#define WGL_ATTR_FOCUSABLE              ((UINT32) 0x0001) /**< Indicates the specified widget has the capability to get focus. */

#define IS_STATE_ENABLED(ui2_state)                                     \
        ((ui2_state) & WGL_STATE_ENABLED)

#define IS_STATE_VISIBLE(ui2_state)                                     \
        ((ui2_state) & WGL_STATE_VISIBLE)

#define IS_STATE_ACTIVATED(ui2_state)                                   \
        ((ui2_state) & WGL_STATE_ACTIVATED)

#define IS_STATE_FOCUSED(ui2_state)                                     \
        ((ui2_state) & WGL_STATE_FOCUSED)

#define IS_STATE_PUSHED(ui2_state)                                      \
        ((ui2_state) & WGL_STATE_PUSHED)

typedef VOID *WGL_WIDGET_CONTEXT_HANDLE_T;/**< Indicates the widget context handle type pointer. */

typedef VOID *WGL_WIDGET_ANIM_CONTEXT_T;/**< Indicates the widget context structure type pointer. */

typedef struct _WIDGET_CREATE_INPUT_T WGL_WIDGET_CREATE_INPUT_T;/**< Indicates the widget widget create input structure type pointer. */

/*------------------------------------------------------------------*/
/*! @brief  The widget create instance function pointer which is called in x_wgl_create_widget() and implemented by every type widget
 *  @param  [in]    pt_ctx_wgt         - Contains  the widget context handle pointer
 *  @param  [in]    pt_wgt_create       -Contains the pointer to create input information
 *  @note
 *  @see
 *  @return return the error code
 *  @retval     WGLR_OK			The function call is successful.
 *  @retval     WGLR_INV_ARG			pt_ctx_wgt or pt_wgt_create is NULL .
 *  @retval     WGLR_OUT_OF_MEMORY			Memory resource is not enough
 *  @retval     WGLR_INV_TYPE			pt_wgt_create contains invalid content or border type.
 *  @retval     WGLR_NO_BORDER			The widget has null border and with create style WGL_STL_GL_BDR_FILL_CNT_BK.
 */
/*------------------------------------------------------------------*/

typedef INT32 (*wgl_create_widget_instance_fct)( 
    WGL_WIDGET_CONTEXT_HANDLE_T         *pt_ctx_wgt,
    WGL_WIDGET_CREATE_INPUT_T           *pt_wgt_create);


/*------------------------------------------------------------------*/
/*! @brief  The widget destroy instance function pointer which is called in x_wgl_destroy_widget() and implemented by every type widget
 *  @param  [in]    pt_ctx_wgt         - Contains  the widget context handle pointer
 *  @note
 *  @see
 *  @return return the error code
 *  @retval     WGLR_OK			The function call is successful.
 *  @retval     WGLR_INV_ARG			pt_ctx_wgt  is NULL .
 */
/*------------------------------------------------------------------*/
typedef INT32 (*wgl_destroy_widget_instance_fct)(
    WGL_WIDGET_CONTEXT_HANDLE_T         t_ctx_wgt);

/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @struct _WIDGET_CREATE_INPUT_T
 *  @brief  This structure is used while the widget instance is created
 *  @code
 *struct _WIDGET_CREATE_INPUT_T
 *{
 *  UINT32                  ui4_type_content;
 *   UINT32                  ui4_type_border;
 *  const GL_RECT_T*        pt_rect;
 *   WGL_CANVAS_INFO_T       t_cvs_info;
 *   GL_HPALETTE_T           h_palette;
 *   VOID*                   pv_param;
 * };
 *  @endcode
 *  @li@c  ui4_type_content                               - Indicates the widget's content type.
 *  @li@c  ui4_type_border                                - Indicates the widget's border type
 *  @li@c  pt_rect                                             - Indicates the widget's region whose coordinate is based on the widget's parent.
 *  @li@c  t_cvs_info                                         - Indicates the widget's canvas info.
 *  @li@c  t_cvs_info                                         - Indicates the widget's canvas's palette handle if has.
 *  @li@c  pv_param                                         - Indicates the private data and defined by concrete widget type.
 */
/*------------------------------------------------------------------*/
struct _WIDGET_CREATE_INPUT_T
{
    UINT32                  ui4_type_content;
    UINT32                  ui4_type_border;
    const GL_RECT_T*        pt_rect;
    WGL_CANVAS_INFO_T       t_cvs_info;
    GL_HPALETTE_T           h_palette;
    VOID*                   pv_param;
};

/*------------------------------------------------------------------*/
/*! @struct WGL_WIDGET_CTRL_INTF_T
 *  @brief  This structure defines widget's function table which must be implemented by every type widget.
 *  @code
 *typedef struct _WGL_WIDGET_CTRL_INTF_T
 *{
 *  wgl_create_widget_instance_fct      pf_create_instance;
 *   wgl_destroy_widget_instance_fct     pf_destroy_instance;
 *   wgl_widget_proc_fct                 pf_msg_proc;
 *   UINT32                              ui4_attribute;
 * } WGL_WIDGET_CTRL_INTF_T;
 *  @endcode
 *  @li@c  pf_create_instance                               - Indicates the widget's  create function pointer.
 *  @li@c  pf_destroy_instance                                - Indicates the widget's destroy function pointer
 *  @li@c  pf_msg_proc                                      - Indicates the widget's message handling function pointer
 *  @li@c  ui4_attribute                                         - Indicates the widget's attribute, now only WGL_ATTR_FOCUSABLE is valid value.
 */
/*------------------------------------------------------------------*/
typedef struct _WGL_WIDGET_CTRL_INTF_T
{
    wgl_create_widget_instance_fct      pf_create_instance;
    wgl_destroy_widget_instance_fct     pf_destroy_instance;
    wgl_widget_proc_fct                 pf_msg_proc;
    UINT32                              ui4_attribute;
} WGL_WIDGET_CTRL_INTF_T;



/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_WGL*/
/*----------------------------------------------------------------------------*/

