/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_com.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ethernet common header file
 *
 * Author:
 * -------
 *   HsHuang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#ifndef _ETHER_COM_H_
#define _ETHER_COM_H_

#include "x_bim.h"
#include "x_timer.h"
#include "x_common.h"
//#include "x_hal_1176.h"
#include "x_hal_926.h"
#include "x_net_drv_if.h"

#include "net_if.h"
#include "../net_debug.h"
#include "ether_priv.h"
#include "ether_reg.h"
#include "ether_phy.h"
#include "ether_hal.h"

#define STORE_MAC_ADDRESS_TO_FLASH 	0
//#define SCATTER_TEST 1
//#define  DRV_LOOP_BACK_TEST 1
#define INTERNAL_PHY 1 //1
#define INTER_PHY_ADDR  0
#define ALIGNMENT_16BYTES_TX_BUFF_TEST 0
#if (INTERNAL_PHY)
#define DEBUG_MII_PIN 0
#define USE_EXT_CLOCK_SOURCE 1
#endif
#ifdef __KERNEL__
#define CONFIG_DRV_LINUX_DATA_CONSISTENCY 1
#define x_alloc_aligned_dma_mem BSP_AllocAlignedDmaMemory
#define x_free_aligned_dma_mem  BSP_FreeAlignedDmaMemory
#define x_alloc_dma_mem BSP_AllocDmaMemory
#define x_free_dma_mem  BSP_FreeDmaMemory
#define x_sscanf sscanf
#else
#define CONFIG_DRV_LINUX_DATA_CONSISTENCY 0
#endif

/****************************************************************************
** External ethernet API
****************************************************************************/
#if 0
extern BOOL  GET_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
extern void  SET_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
extern void  CLR_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
#endif

/* The macro reply device status information */
#define GET_ETHER_INFO(prMac, Status)  \
	    (BOOL)(prMac->u4DevStatus & Status)

/* The macro set device status information */
#define SET_ETHER_INFO(prMac, Status)  \
    ETHER_SEMA_LOCK(prMac->hDevStatusSema);\
    (prMac->u4DevStatus |= Status);\
    ETHER_SEMA_UNLOCK(prMac->hDevStatusSema)

/* The macro clear device status information */
#define CLR_ETHER_INFO(prMac, Status)  \
    ETHER_SEMA_LOCK(prMac->hDevStatusSema);\
    (prMac->u4DevStatus &= ~Status);\
    ETHER_SEMA_UNLOCK(prMac->hDevStatusSema)    

/****************************************************************************
** Constant definitions
****************************************************************************/
/* Buffer and Descriptor size */
#ifdef SCATTER_TEST //only for verification
#define	PKT_BUFFER_SIZE             256    /* The size is better be 128-bit aligned and must 

                                               be larger than 1518 byte */

#else
#define	PKT_BUFFER_SIZE             1536    /* The size is better be 128-bit aligned and must 

                                               be larger than 1518 byte */
#endif                                               

#define MAC_CRC_LEN         		4     	/* MAC CRC length */
#define MAC_IP_CHKSUM_LEN   		2     	/* MAC IP checksum length */
#define ETHER_HEADER_LEN    		14    	/* ethernet header length */
#define ETHER_VALID_PKT_LEN    		1518   	/* valid ethernet packet length */

#define ETHER_MAX_ADDR_LEN			6		/* Largest hardware address length */
#define ETHER_MAX_MULTICAST_NUM		15		/* Largest number of multicast list */


/* ethernet device status information */
typedef enum 
{
    ETHER_DEV_INIT      =   ((UINT32)0x01 << 0),    /* device driver is init or not */
    ETHER_DEV_START     =   ((UINT32)0x01 << 1),    /* device driver is started */
	ETHER_LINK_UP		=	((UINT32)0x01 << 2),	/* net device link up or not */
    ETHER_DEV_RDY       =   ((UINT32)0x01 << 3),    /* device driver is ready for tx/rx */
    RX_TO_RECV_PKT      =   ((UINT32)0x01 << 4),    /* rx buffer has packets and need to receive it */
    RX_BUF_OVERRUN      =   ((UINT32)0x01 << 5)     /* rx buffer is overrun */
} ETHER_DEV_INFO_T;

/****************************************************************************
** Macro definitions
****************************************************************************/
#define ETHER_NONCACHE				NONCACHE
#define ETHER_CACHE				    CACHE

/* The macro is used to indicate 6 byte mac address */
#define ETH_MAC2STR(a)  (a)[0], (a)[1], (a)[2], (a)[3], (a)[4], (a)[5]

#endif // _ETHER_COM_H_
