/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_hal.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ethernet HAL header file
 *
 * Author:
 * -------
 *   HsHuang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#ifndef _ETHER_HAL_H_
#define _ETHER_HAL_H_

/****************************************************************************
** Constant definitions
****************************************************************************/
//#define ETH_RMII_PHY
/* Duplex status */
typedef enum 
{
    HALF_DUPLEX,	
    FULL_DUPLEX, 		
    UNKNOWN		
} ETHER_DUPLEX_MODE_T;

/****************************************************************************
** Macro
****************************************************************************/
/* The macro check whether MAC is RX IP checksum mode  */
#define IS_MAC_IP_CHKSUM()       0

/* The following macros are DMA engine interrupt handling */
#define u4DmaGetInt(u4IntMask)   (dReadMAC(RW_DLY_INT_CFG) & u4IntMask)

#define vDmaIntClear(u4IntClr)   vWriteMAC(RW_DLY_INT_CFG, u4IntClr)

#define vDmaIntEnable(u4IntEn)   vWriteMAC(RW_INT_MASK, (dReadMAC(RW_INT_MASK) & (~u4IntEn)))

#define vDmaIntDisable(u4IntDis) vWriteMAC(RW_INT_MASK, (dReadMAC(RW_INT_MASK) | u4IntDis))

#define vDmaIntClearAll()        vWriteMAC(RW_DLY_INT_CFG, DMA_INT_ENABLE_ALL)

/* The macro get current host transmit descriptor address */
#define u4DmaGetCurrTxDesc()     dReadMAC(RW_TX_DPTR)

/* The macro resume tx descriptor, Poll Demand */
//#ifdef CC_MT5399
#if defined(CC_MT5890) || defined(CC_MT5891)
#define vDmaTxStartAndResetTXDesc()   SetBitMAC(RW_TX_DMA_CTRL, TX_START) 
#define vDmaTxResume()                SetBitMAC(RW_TX_DMA_CTRL, TX_RESUME)   
#else
#define vDmaTxResume()           vWriteMAC(RW_TX_DMA_CTRL, TX_EN)   
#endif

/* The macro resume rx descriptor, Poll Demand */
//#ifdef CC_MT5399
#if defined(CC_MT5890) || defined(CC_MT5891)
#define vDmaRxStartAndResetRXDesc()   SetBitMAC(RW_RX_DMA_CTRL, RX_START) 
#define vDmaRxResume()                SetBitMAC(RW_RX_DMA_CTRL, RX_RESUME)  
#else
#define vDmaRxResume()            vWriteMAC(RW_RX_DMA_CTRL, RX_EN)  
#endif


/* The macro check if current transmission is completed  */
#define fgDescIsTxSent(prTxDesc)  ((prTxDesc->rTxCtrl.Bits.fgOwn == OWNED_BY_CPU) && prTxDesc->rTxCtrl.Bits.u2BufSize)

/* The macro check transmission result is OK or not  */
#define fgDescIsTxValid(prCtrl)  1 

/* The macro check received Rx result is OK or not  */
#define fgDescIsRxValid(prCtrl) (!(prCtrl->Bits.fgOversizeErr) && (!(prCtrl->Bits.fgCrcErr)) &&\
                                    (prCtrl->Bits.fgFirstSeg) && \
                                    (prCtrl->Bits.fgLastSeg))


#define fgIsDMATxEnable()	((dReadMAC(RW_DMA_CFG) & TX_SUSPEND) == 0)
/****************************************************************************
** Structure and Type definitions
****************************************************************************/
typedef UINT32 ETHER_WAKEUP_FRAME_T[8];

/****************************************************************************
** Export API
****************************************************************************/
void vHalSetMacAddr(MAC_ADDRESS_T *pMacAddr);
void vHalGetMacAddr(MAC_ADDRESS_T *pMacAddr);
void vHalAddMulticastAddr(MAC_ADDRESS_T *pMacAddr, UINT32 u4AddrIdx);
void vHalDelMulticastAddr(UINT32 u4AddrIdx);
void vHalGetMulticastAddr(MAC_ADDRESS_T *pMacAddr, UINT32 u4AddrIdx);
void vHalGetLinkSpeed(UINT32 *pu4Speed);
void vHalSetLinkSpeed(UINT32 *pu4Speed);
void vHalGetPromiseMode(UINT32 *pu4Promise);
void vHalSetPromiseMode(UINT32 *pu4Promise);
void vHalGetDuplexMode(UINT32 *pu4Duplex);
void vHalSetDuplexMode(UINT32 *pu4Duplex);
void vHalWakeupFrameFilterSet(ETHER_WAKEUP_FRAME_T *prWkFrame);
void vHalSetMacPwrDown(BOOL fgPwrDown);
INT32 i4HalPhyStatusChk(PETHER_ADAPTER_T prMac);
BOOL fgHalIsPwrDownWakeup(void);
INT32 i4HalSetEtherStatInfo(INT32 i4Cmd, VOID *pvParam);
INT32 i4HalGetEtherStatInfo(INT32 i4Cmd, VOID *pvParam);
BOOL fgHalIsMulticastSet(UINT32 u4AddrIdx);
INT32 i4HalSyncDescIdx(PETHER_ADAPTER_T prMac);
void vHalCalThroughput(PETHER_ADAPTER_T prMac);

void vRMiiInit(void);
void vMacInit(PETHER_ADAPTER_T prMac);
void vMmcInit(void);
void vDmaInit(PETHER_ADAPTER_T prMac);
void vDmaRxStart(void);
void vDmaRxStop(PETHER_ADAPTER_T prMac);
void vDmaTxStart(void);
void vDmaTxStop(PETHER_ADAPTER_T prMac);
void vDmaIntEnableAll(void);
void vDmaIntDisableAll(void);
UINT32 u4DmaGetAndClrInt(UINT32 u4IntMask);


/****************************************************************************
** External ethernet API
****************************************************************************/


#endif // _ETHER_HAL_H_
