/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_phy.c $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ethernet PHY code
 *
 * Author: Hs Huang
 * -------
 *
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#define _PHY_

#include "ether_com.h"
#include "ether_drv.h"
#include "x_pinmux.h"
/****************************************************************************
** Constant definitions
****************************************************************************/
#define PHY_ID_NUM_LOW_WORD     0x5AA5


// *********************************************************************
// Function    : i4MacPhyRead
// Description : MII PHY register read
// Parameter   : u4PhyAddr - phy device address
//               u4RegAddr - phy register offset
//               pu4Data - output read data
// Return      : 0 if success
// *********************************************************************
INT32 i4MacPhyRead(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 *pu4Data)
{
    UINT32 u4Data, u4TimeOut;

    ASSERT(pu4Data != NULL);
    
   u4Data = 0;
   u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
   u4Data |= ((u4RegAddr << PHY_REG_SHIFT) & PHY_REG);
   u4Data |= RD_CMD;
   vWriteMAC(RW_PHY_CTRL0, u4Data);

    u4TimeOut = 0xffff;
    while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
    { 
        u4TimeOut --;
        if(u4TimeOut == 0x00)
        {
            WARN_ETHER("MII read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
    }
  
     *pu4Data = (UINT32)(dReadMAC(RW_PHY_CTRL0) >> RW_DATA_SHIFT);        // Get PHY data
     SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK

    return (NET_DRV_OK);          
}

// *********************************************************************
// Function    : void i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
// Description : MII PHY register write
// Parameter   : u4PhyAddr, u4RegAddr, u4RegData
// Return      : none
// *********************************************************************
INT32 i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
{
    UINT32 u4Data, u4TimeOut;
  
   u4Data = (u4RegData & 0xFFFF) << RW_DATA_SHIFT;
   u4Data |= ((u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR);
   u4Data |= ((u4RegAddr << PHY_REG_SHIFT) & PHY_REG);
   u4Data |= WT_CMD;
  
   vWriteMAC(RW_PHY_CTRL0, u4Data);
  
    u4TimeOut = 0xffff;
    while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
    { 
        u4TimeOut --;
        if(u4TimeOut == 0x00)
        {
            WARN_ETHER("MII write PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
    }
    SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK
  
    return(NET_DRV_OK);
}

INT32 i4MacCL45Read(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 dwLen, UINT32 *pu4Data)
{
    UINT32 u4Data, u4TimeOut,dwCmd;

    ASSERT(pu4Data != NULL);

	
   SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK
   
   u4Data = 0;
   u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
   u4Data |= ((u4RegAddr << 16)&0xFFFF0000);
   u4Data |= (((u4RegAddr >> 16)&0x1F)<<8);
   u4Data |= CL45_ENABLE;
   u4Data |= CL45_ADDR;
   vWriteMAC(RW_PHY_CTRL0, u4Data);
   
    u4TimeOut = 0xffff;
    while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
    { 
        u4TimeOut --;
        if(u4TimeOut == 0x00)
        {
            WARN_ETHER("MII read PHY timeout\n");
            return(NET_DRV_HW_ERROR);
        }
    }
    SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK

	if(dwLen<=1)
		dwCmd = CL45_READ ;
	else
		dwCmd = CL45_MULTI_READ ;

	while(dwLen)
	{
		u4Data = 0;
		u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
		u4Data |= ((u4RegAddr << 16)&0xFFFF0000);
		u4Data |= (((u4RegAddr >> 16)&0x1F)<<8);
		u4Data |= CL45_ENABLE;
		u4Data |= dwCmd;
		vWriteMAC(RW_PHY_CTRL0, u4Data);
		
		 u4TimeOut = 0xffff;
		 while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
		 { 
			 u4TimeOut --;
			 if(u4TimeOut == 0x00)
			 {
				 WARN_ETHER("MII read PHY timeout\n");
				 return(NET_DRV_HW_ERROR);
			 }
		 }
		 *pu4Data = (UINT32)(dReadMAC(RW_PHY_CTRL0) >> RW_DATA_SHIFT);		  // Get PHY data
		 SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK
		 
         pu4Data++;
		 dwLen--;
	}



    return (NET_DRV_OK);    
}

// *********************************************************************
// Function    : void i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
// Description : MII PHY register write
// Parameter   : u4PhyAddr, u4RegAddr, u4RegData
// Return      : none
// *********************************************************************
INT32 i4MacCL45Write(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData)
{
	UINT32 u4Data, u4TimeOut;
	
		
   SetBitMAC(RW_PHY_CTRL0, RW_OK);	//clear RW_OK
   
   u4Data = 0;
   u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
   u4Data |= ((u4RegAddr << 16)&0xFFFF0000);
   u4Data |= (((u4RegAddr >> 16)&0x1F)<<8);
   u4Data |= CL45_ENABLE;
   u4Data |= CL45_ADDR;
   vWriteMAC(RW_PHY_CTRL0, u4Data);
   
	u4TimeOut = 0xffff;
	while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
	{ 
		u4TimeOut --;
		if(u4TimeOut == 0x00)
		{
			WARN_ETHER("MII read PHY timeout\n");
			return(NET_DRV_HW_ERROR);
		}
	}
	SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK


	u4Data = 0;
	u4Data = (u4PhyAddr << PHY_ADDR_SHIFT) & PHY_ADDR;
	u4Data |= ((u4RegData << 16));
	u4Data |= (((u4RegAddr >> 16)&0x1F)<<8);
	u4Data |= CL45_ENABLE;
	u4Data |= CL45_WRITE;
	vWriteMAC(RW_PHY_CTRL0, u4Data);
	
	 u4TimeOut = 0xffff;
	 while((dReadMAC(RW_PHY_CTRL0) & RW_OK) == 0) // Wait for R/W command complete 
	 { 
		 u4TimeOut --;
		 if(u4TimeOut == 0x00)
		 {
			 WARN_ETHER("MII read PHY timeout\n");
			 return(NET_DRV_HW_ERROR);
		 }
	 }
	 SetBitMAC(RW_PHY_CTRL0, RW_OK);  //clear RW_OK
	
	 return (NET_DRV_OK);	
}

// *********************************************************************
// Function    : void i4MacPhyReset(UINT32 u4PhyAddr)
// Description : PHY device SW reset
// Parameter   : u4PhyAddr
// Return      : none
// *********************************************************************
INT32 i4MacPhyReset(UINT32 u4PhyAddr)
{
    INT32 i4Ret;
    
    i4Ret = i4MacPhyWrite(u4PhyAddr, MII_PHY_CTRL_REG, SW_RESET);
    if(i4Ret != NET_DRV_OK)
    {
        ERR_ETHER("ethernet PHY reset fail\n");
    }

    return(i4Ret);
}

#if (INTERNAL_PHY)
// *********************************************************************
// Function    : void i4MacIntPhyInit(UINT32 u4PhyAddr)
// Description : PHY device SW init
// Parameter   : u4PhyAddr
// Return      : none
// *********************************************************************
INT32 i4MacIntPhyInit(UINT32 u4PhyAddr)
{
    INT32 i4Ret = 0;
    UINT32 u4Data;

#if 0
    //test page
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x2a30); 
    i4MacPhyWrite(u4PhyAddr, 0x03, 0x000c); //10M power saving
    i4MacPhyWrite(u4PhyAddr, 0x08, 0x0211); //disable gating of RCLK125 according to YT's suggest
    i4MacPhyWrite(u4PhyAddr, 0x0E, 0x35); //analog
    i4MacPhyWrite(u4PhyAddr, 0x18, 0xE402);  //analog
    #if 0 //(CONFIG_CHIP_VER_CURR == CONFIG_CHIP_VER_MT8555)
    i4MacPhyRead(u4PhyAddr, 0x0A, &u4Data);  //ADC clock INvert
    i4MacPhyWrite(u4PhyAddr, 0x0A, u4Data|(1<<13));
     
    #endif 
    //extend page
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x01);   
    i4MacPhyWrite(u4PhyAddr, 0x1b, 0x1a0); //enable auto-crossover as autoneg is disabled

    //main page
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x00);   
    i4MacPhyWrite(u4PhyAddr, 0x17, 0x0020); //ACTI_EN for power save when link down
    i4MacPhyWrite(u4PhyAddr, 0x18, 0x1f00); //phy control
    
    i4Ret = i4MacPhyWrite(u4PhyAddr, 0x1F, 0x0); //return to main page.
    if(i4Ret != NET_DRV_OK)
    {
        ERR_ETHER("ethernet internal PHY init fail\n");
    }
#endif
   
    /* config ethernet LED */
    BSP_PinSet(PIN_PHYLED0, 1);
    BSP_PinSet(PIN_PHYLED1, 1);

#if defined(CC_MT5890) || defined(CC_MT5891)
    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x3500); // Extended test page

    i4MacPhyWrite(u4PhyAddr, 0x08, 0xfc55);
    
    i4MacPhyRead(u4PhyAddr, 0x02, &u4Data);
  	u4Data = u4Data | 1;
    i4MacPhyWrite(u4PhyAddr, 0x02, u4Data); 

    vNetDelay1us(1);
    
    i4MacPhyRead(u4PhyAddr, 0x02, &u4Data);
  	u4Data = u4Data & ~1;
    i4MacPhyWrite(u4PhyAddr, 0x02, u4Data);

    i4MacPhyWrite(u4PhyAddr, 0x07, 0x0c10);

    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x2a30);
    i4MacPhyWrite(u4PhyAddr, 0x1d, 0x1051);
    i4MacPhyWrite(u4PhyAddr, 0x15, 0x0f7f);
    i4MacPhyWrite(u4PhyAddr, 0x19, 0x778f);
    
    i4MacPhyRead(u4PhyAddr, 0x1a, &u4Data);
  	u4Data = u4Data & ~(0xf);
    u4Data = u4Data | 0x8;
    i4MacPhyWrite(u4PhyAddr, 0x1a, u4Data); 

    i4MacPhyRead(u4PhyAddr, 0x1e, &u4Data);
  	u4Data = u4Data & ~(0xf000);
    u4Data = u4Data | 0xa000;
    i4MacPhyWrite(u4PhyAddr, 0x1e, u4Data); 
    
#endif

#if defined(CC_MT5891) //enable wukong adc clk
i4MacPhyWrite(u4PhyAddr, 0x1f, 0x2a30);
i4MacPhyWrite(u4PhyAddr, 0x1, 0x8000);
i4MacPhyWrite(u4PhyAddr, 0x1f, 0x0);
#endif

    // set LED
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x0002);  // misc page
    i4MacPhyWrite(u4PhyAddr, 0x00, 0x80f0);  // EXT_CTL([15]) = 1
    i4MacPhyWrite(u4PhyAddr, 0x03, 0xc000);  // LED0 on clean
    i4MacPhyWrite(u4PhyAddr, 0x04, 0x003f);  // LED0 blink for 10/100/1000 tx/rx
    i4MacPhyWrite(u4PhyAddr, 0x05, 0xc007);  // LED1 on for 10/100/1000M linkup
    i4MacPhyWrite(u4PhyAddr, 0x06, 0x0000);  // LED1 blink clean
    i4MacPhyWrite(u4PhyAddr, 0x1F, 0x0000);  // main page
  
    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x2a30); //set to Test Page
    i4MacPhyRead(u4PhyAddr, 0x1A, &u4Data);
  	u4Data = (u4Data & (0xfff0))|(0x03 & 0x0f);//100M amp
    i4MacPhyWrite(u4PhyAddr, 0x1A, u4Data);
	i4MacPhyRead(u4PhyAddr, 0x15, &u4Data);
	u4Data = (u4Data &(0xff00) )|((0x7f & 0x7f)<< 0);//100M slew rate
	i4MacPhyWrite(u4PhyAddr, 0x15, u4Data); 
	i4MacPhyRead(u4PhyAddr, 0x18, &u4Data);
	u4Data = (u4Data & (0xfffC))|((0 & 0x03)<< 0);//10M amp
	i4MacPhyWrite(u4PhyAddr, 0x18, u4Data);
	i4MacPhyRead(u4PhyAddr, 0x1E, &u4Data);
	u4Data = (u4Data & (0x0fff))|((0x3)<< 12);//10M amp
	i4MacPhyWrite(u4PhyAddr, 0x1E, u4Data);

    i4MacPhyWrite(u4PhyAddr, 0x1f, 0);

    UNUSED(i4Ret);
    return(i4Ret);
}
#endif

#if defined(CC_MT5890) || defined(CC_MT5891)

INT32 i4MacPhyZCALInit(UINT32 u4PhyAddr)
{
    INT32 i4Ret = 0;
    
    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x0002); // misc page
    i4MacPhyWrite(u4PhyAddr, 0x0d, 0x1016); 

    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x00);
    
    UNUSED(i4Ret);
    return(i4Ret);
}

INT32 i4MacPhyZCALRemapTable(UINT32 u4PhyAddr, UINT8 value)
{
    INT32 i4Ret = 0;
    UINT32 u4Data;
    
    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x2a30);

    i4MacPhyRead(u4PhyAddr, 0x14, &u4Data);
    u4Data |= (0x1 << 12);
    u4Data &= ~(0x0f00);    
    u4Data |= value << 8;
    u4Data &= ~(0xff); 
    i4MacPhyWrite(u4PhyAddr, 0x14, u4Data);

    i4MacPhyWrite(u4PhyAddr, 0x1f, 0x00);
    UNUSED(i4Ret);
    return(i4Ret);
}
#endif

// *********************************************************************
// Function    : void i4MacPhyFullDup(UINT32 u4PhyAddr, BOOL fgFullDup)
// Description : Set PHY duplex mode
// Parameter   : fgFullDup, TRUE-> Full duplex, FALSE-> half duplex
// Return      : none
// *********************************************************************
INT32 i4MacPhyFullDup(UINT32 u4PhyAddr, BOOL fgFullDup)
{
    UINT32 u4Data;
    INT32 i4Ret;
    
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_CTRL_REG, &u4Data);
    if(i4Ret == NET_DRV_OK)
    {
        u4Data &= (~AN_ENABLE);
    }
    else
    {
        return(i4Ret);
    }
    
    if(fgFullDup)
    {
        u4Data |= DUPLEX_FULL;
    }
    else
    {
        u4Data &= ~DUPLEX_FULL;
    }
    	
    i4Ret = i4MacPhyWrite(u4PhyAddr, MII_PHY_CTRL_REG, u4Data);
    
    return(i4Ret);
}

// *********************************************************************
// Function    : void i4MacPhyLoopback(UINT32 u4PhyAddr, BOOL fgLoopback)
// Description : Set PHY loopback mode
// Parameter   : fgLoopback, TRUE-> loopback mode, FALSE-> normal mode
// Return      : none
// *********************************************************************
INT32 i4MacPhyLoopback(UINT32 u4PhyAddr, BOOL fgLoopback)
{
    UINT32 u4Data;
    INT32 i4Ret;
    	
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_CTRL_REG, &u4Data);
    if(i4Ret != NET_DRV_OK)
    {
        return(i4Ret);
    }
    
    if(fgLoopback)
    {
        u4Data |= LOOKBACK_EN;
    }
    else
    {
        u4Data &= ~LOOKBACK_EN;
    }
    	
    i4Ret = i4MacPhyWrite(u4PhyAddr, MII_PHY_CTRL_REG, u4Data);

    return(i4Ret);
}

// *********************************************************************
// Function    : i4MacPhyAddrGet
// Description : Get PHY physical address by r/w register result
// Parameter   : prMac
// Return      : 0 on success, others on failure
// *********************************************************************
INT32 i4MacPhyAddrGet(PETHER_ADAPTER_T prMac)
{
   #if (!INTERNAL_PHY) //kenny mark it
    UINT32 u4Data, u4PhyAddr;
    UINT32 u4DataTmp;
   #endif

    ASSERT(prMac != NULL);
    #if (!INTERNAL_PHY) //kenny mark it
    for(u4PhyAddr = 0; u4PhyAddr < 32; u4PhyAddr++)
    {
        // MII_PHY_ID0_REG can be r/w on SMSC8700, but read only for RTL8201
        i4MacPhyRead(u4PhyAddr, MII_PHY_ID0_REG, &u4DataTmp);
        i4MacPhyWrite(u4PhyAddr, MII_PHY_ID0_REG, PHY_ID_NUM_LOW_WORD);
        i4MacPhyRead(u4PhyAddr, MII_PHY_ID0_REG, &u4Data);
        if(u4Data == PHY_ID_NUM_LOW_WORD)
        {
            prMac->u4PhyAddr = u4PhyAddr;
            INFO_ETHER("PHY MII address: 0x%02x\n", u4PhyAddr);
            i4MacPhyWrite(u4PhyAddr, MII_PHY_ID0_REG, u4DataTmp);
            return (NET_DRV_OK);
        }
    }
    #else
    prMac->u4PhyAddr = 0;//kenny temply 
    #endif
    //INFO_ETHER("Get PHY MII address fail\n");
    return (NET_DRV_ERROR);          
}

// *********************************************************************
// Function    : i4MacPhyNegotitate
// Description : MAC PHY link status check and negotiation
// Parameter   : prMac
// Return      : NET_DRV_OK - MII access PHY register OK
//             : negative - MII access PHY register fail
// *********************************************************************
INT32 i4MacPhyNegotitate(PETHER_ADAPTER_T prMac)
{
    UINT32 u4PhyAddr;
    UINT32 u4CtrlReg0, u4StatusReg1; 
    UINT32 u4AnlparReg5, u4AnerReg6; 
    PMAC_MII_T prMii;
    INT32 i4Ret;

    ASSERT(prMac != NULL);
    
    prMii = &prMac->rMii;
    prMii->fgSpeed1000 = FALSE;

    u4PhyAddr = prMac->u4PhyAddr;
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_CTRL_REG, &u4CtrlReg0);
    if(i4Ret != NET_DRV_OK)
    {
        WARN_ETHER("MII access PHY register fail\n");
        return(i4Ret);
    }
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_STATUS_REG, &u4StatusReg1);

#if 0
    if ((u4StatusReg1 & LINK) == 0)
    {
		NOTE_ETHER("Link Down \n");
		Printf("0x%04x  0x%04x \n", u4CtrlReg0,u4StatusReg1);
    }
#endif
    
    if(i4Ret != NET_DRV_OK)
    {
        WARN_ETHER("MII access PHY register fail\n");
        return(i4Ret);
    }
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_ANER_REG, &u4AnerReg6);
    if(i4Ret != NET_DRV_OK)
    {
        WARN_ETHER("MII access PHY register fail\n");
        return(i4Ret);
    }

    prMii->fgLinkUp = (u4StatusReg1 & LINK) ? TRUE : FALSE;
    prMii->fgLpNwAble = (u4AnerReg6 & 0x01) ? TRUE : FALSE;

    if(u4CtrlReg0 & AN_ENABLE)
    {
        /* auto-negotiation is enabled and completed */
	    if(prMii->fgLinkUp)
	    {
	        /* cable is conntected and linked up */
	            if(u4StatusReg1 & AN_COMPLT)
	            {
	        		SET_ETHER_INFO(prMac, ETHER_LINK_UP);
	            }	
	            else
	            {
	                /* From experiment, this case is never occurred on RTL8201.
	                   It means that once mode is changed(may be speed, deplex or plug in/out),
	                   PHY status will be linked down, i.e. prMii->fgLinkUp = 0 */
	                DBG_ETHER("auto-negotiation is not completed\n");
	            }
	    }
	    else
	    {
	        /* cable is not conntected, no need to check other status */
	        CLR_ETHER_INFO(prMac, ETHER_LINK_UP);
	    	return(NET_DRV_OK);
		}
    }
    else
   	{
        /* auto negotiation is disabled */
        if(prMii->fgLinkUp)
      	{
            /* cable is conntected and linked up */
            SET_ETHER_INFO(prMac, ETHER_LINK_UP);
    	}
        else
    	{
            /* cable is not conntected, no need to check other status */
            CLR_ETHER_INFO(prMac, ETHER_LINK_UP);
            return(NET_DRV_OK);
      	}
  	}

	/* check speed and deplex mode */
    i4Ret = i4MacPhyRead(u4PhyAddr, MII_PHY_ANLPAR_REG, &u4AnlparReg5);
    if(i4Ret != NET_DRV_OK)
  	{
        WARN_ETHER("MII access PHY register fail\n");
        return(i4Ret);
    }
      
    if(u4AnlparReg5 & (PARTNER_100BASE | PARTNER_100BASE_FULL))
    {
        prMii->fgSpeed100 = TRUE;
        prMii->fgFullDup = (u4AnlparReg5 & PARTNER_100BASE_FULL) ? TRUE : FALSE;
    }
    else
    {
        prMii->fgSpeed100 = FALSE;
        prMii->fgFullDup = (u4AnlparReg5 & PARTNER_10BASE_FULL) ? TRUE : FALSE;
    }

    return (NET_DRV_OK);
}

