/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


#include "x_typedef.h"
#include "x_bim.h"
#include "x_dram.h"
#include "x_pdwnc.h"
#include "x_ckgen.h"
#include "drvcust_if.h"

#ifdef CC_MT5886
#include "ddr.h"
#endif

#include "chip_txt.h"

extern void CHIP_BimInit(void);
//#ifndef CC_MTK_LOADER
extern void CHIP_CheckBoard(void);
//#endif /* CC_MTK_LOADER */

#ifdef CC_MTK_PRELOADER
extern void CHIP_ClockSetting(void);
extern void CHIP_1stBootRegInit(void);
extern void CLK_SetDdr(void);

UINT32 CHIP_T8032Cmd(PDWNC_T8032_CMD_T *rCmd)
{
    UINT32 u4Val = 0;
    UINT32 i = 0, j = 0;
    UINT32 u1Ack = 0;

    PDWNC_T8032CMD_SET_CMD((rCmd->u1Cmd) | ((rCmd->u1SubCmd) << 8));
    while (1)
    {
        vIO32WriteFldAlign(PDWNC_ARM_INTEN, 0x1, FLD_AUX_INTEN);
        vIO32WriteFldAlign(PDWNC_ARM_INT, 0x1, FLD_ARM_INT);

        while (i < 100000)
        {
            //CHIP_Delay_us(100);
            if (IO32ReadFldAlign(PDWNC_INTSTA, FLD_AUX_INT_STA))
            {
                u4Val = PDWNC_T8032CMD_GET_RESPONSE();
                if ((u4Val & 0xff) != (rCmd->u1Cmd))
                {
                    continue;
                }
                u1Ack = 1;
                break;
            }
            i++;
        }
        if (i < 10000)
        {
            CHIP_DisplayString("Preloader T8032 ack !\n");
            vIO32WriteFldAlign(PDWNC_INTCLR, 0x1, FLD_AUX_INTCLR);
            u1Ack = 1;
            break;
        }
        else
        {
            vIO32WriteFldAlign(PDWNC_INTCLR, 0x1, FLD_AUX_INTCLR);
            i = 0;
        }
        if (++j > 2 /*T8032_ACK_TIMEOUT*/)
        {
            CHIP_DisplayString("Preloader T8032 NO ack!\n");
            // Ack timeout
            u1Ack = 0;
            break;
        }
    }
    return u1Ack;
}

#define ARM_FORCE_TAKE_BACK_UART0 0
static void CHIP_NotifyT8032(void)
{
    UINT32 u4Val = 0;
    UINT32 u1Ack = 0;
    PDWNC_T8032_CMD_T rCmd;
#if defined(CC_S_PLATFORM) && defined(CC_CONSOLE_LOCK)
#if defined(CC_OX_CUST) || defined(CC_WK_CUST) || defined(CC_HK_CUST) || defined(CC_LMP_CUST) || defined(CC_PHX_CUST) || defined(CC_CP_CUST)
    UINT32 u4ConsoleVal = 0;
    UINT32 u4IceVal = 0;
#endif
#endif

    CHIP_DisplayString("\nDC wakeup Power ready check time : ");
    CHIP_DisplayInteger((u4IO32Read4B(PDWNC_WKRSC) / 24000));
    CHIP_DisplayString("ms.\n");

    rCmd.u1Cmd = PDWNC_CMD_ARMT8032_ENTER_NORMAL;
    rCmd.u1SubCmd = 1;
    u1Ack = CHIP_T8032Cmd(&rCmd);
    u4Val = PDWNC_T8032CMD_GET_RESPONSE();
#if defined(CC_S_PLATFORM) && defined(CC_CONSOLE_LOCK)
#if defined(CC_OX_CUST) || defined(CC_WU_CUST) || defined(CC_HK_CUST) || defined(CC_LMP_CUST) || defined(CC_PHX_CUST) || defined(CC_CP_CUST)
    rCmd.u1Cmd = PDWNC_CMD_ARMT8032_CONSOLELOCK_QUERY;
    u1Ack = CHIP_T8032Cmd(&rCmd);
    u4ConsoleVal = PDWNC_T8032CMD_GET_RESPONSE();
    //ICE
    rCmd.u1Cmd = PDWNC_CMD_ARMT8032_ICE_ModeQuery;
    u1Ack = CHIP_T8032Cmd(&rCmd);
    u4IceVal = PDWNC_T8032CMD_GET_RESPONSE();

#endif
#endif

    if (((u4Val & 0xff000000) == ARM_FORCE_TAKE_BACK_UART0) || (u1Ack == 1))
    {
        //CHIP_DisplayString("U0==>ARM!\n");
#if defined(CC_MT5881)
        vIO32Write4B(PDWNC_PINMUX1, u4IO32Read4B(PDWNC_PINMUX1) & 0xf03fffff);
#elif defined(CC_MT5882)
        vIO32Write4B(PDWNC_PINMUX2, u4IO32Read4B(PDWNC_PINMUX2) & 0xffff81ff);

        #if defined(CC_S_PLATFORM) && (defined(CC_LMP_CUST) || defined(CC_PHX_CUST))
        #if defined(CC_CONSOLE_LOCK)
        if ((u4ConsoleVal & 0xff000000) == ARM_FORCE_TAKE_BACK_UART0)
        {
            CHIP_DisplayString("U0==>ARM!\n");
        }
        else
        {
            CHIP_DisplayString("U1==>ARM!\n");
            vIO32Write4B(PDWNC_PINMUX2, u4IO32Read4B(PDWNC_PINMUX2) | 0x00002400);
        }
        #endif
        #endif
#elif defined(CC_MT5883)
        vIO32Write4B(PDWNC_PINMUX2, u4IO32Read4B(PDWNC_PINMUX2) & 0xffff81ff);
#elif defined(CC_MT5890)
        #if defined(CC_S_PLATFORM) && defined(CC_CONSOLE_LOCK)
        if ((u4ConsoleVal & 0xff000000) == ARM_FORCE_TAKE_BACK_UART0)
        {
        #endif
            vIO32Write4B(PDWNC_PINMUX0, u4IO32Read4B(PDWNC_PINMUX0) & 0xfffcffff);
            vIO32Write4B(PDWNC_PINMUX1, u4IO32Read4B(PDWNC_PINMUX1) & 0xfffcffff);
        #if defined(CC_S_PLATFORM) && defined(CC_CONSOLE_LOCK)
        }
        #endif

//#elif defined(CC_MT5891)
//        vIO32Write4B(PDWNC_PINMUX3, u4IO32Read4B(PDWNC_PINMUX3) & 0xff03ffff);
//#else
//        vIO32Write4B(PDWNC_PINMUX0, u4IO32Read4B(PDWNC_PINMUX0) & 0xfffcffff);
#elif (defined(CC_MT5891)||defined(CC_MT5893))
        #if defined(CC_S_PLATFORM) && (defined(CC_WK_CUST) || defined(CC_HK_CUST))
        #if defined(CC_CONSOLE_LOCK)
        if ((u4ConsoleVal & 0xff000000) == ARM_FORCE_TAKE_BACK_UART0)
        {
            CHIP_DisplayString("CONSOLE ON mode_3.\n");
            vIO32Write4B(PDWNC_PINMUX3, u4IO32Read4B(PDWNC_PINMUX3) & 0xff03ffff);
        }
        else
        {
            CHIP_DisplayString("CONSOLE OFF mode_3.\n");
            vIO32Write4B(PDWNC_PINMUX3, (u4IO32Read4B(PDWNC_PINMUX3) | 0x00480000));
        }
        if ((u4IceVal & 0xff000000) == 0x0)
        {
            vIO32Write4B(0x1000802c,u4IO32Read4B(0x1000802c)&0xbdffffff);
            CHIP_DisplayString("ICE OFF mode_3.\n");
        }
        else
        {
            vIO32Write4B(0x1000802c,u4IO32Read4B(0x1000802c)|0x42000000); //ICE mode
            CHIP_DisplayString("ICE ON mode_3.\n");
        }
        #else
        vIO32Write4B(PDWNC_PINMUX3, u4IO32Read4B(PDWNC_PINMUX3) & 0xff03ffff);
        #endif
        #endif
#elif defined(CC_MT5895)
        #if defined(CC_S_PLATFORM) && defined(CC_CP_CUST)
        #if defined(CC_CONSOLE_LOCK)
#if 0
        if ((u4ConsoleVal & 0xff000000) == ARM_FORCE_TAKE_BACK_UART0)
        {
            CHIP_DisplayString("CONSOLE ON mode_3.\n");
            vIO32Write4B(PDWNC_PINMUX3, u4IO32Read4B(PDWNC_PINMUX3) & 0xf00fffff);
        }
        else
        {
            CHIP_DisplayString("CONSOLE OFF mode_3.\n");
            vIO32Write4B(PDWNC_PINMUX3, (u4IO32Read4B(PDWNC_PINMUX3) | 0x02200000));
        }
#endif
        if ((u4IceVal & 0xff000000) == 0x0)
        {
            vIO32Write4B(0x1000802c,u4IO32Read4B(0x1000802c)&0xbdffffff);
            CHIP_DisplayString("ICE OFF mode_3.\n");
        }
        else
        {
            vIO32Write4B(0x1000802c,u4IO32Read4B(0x1000802c)|0x42000000); //ICE mode
            CHIP_DisplayString("ICE ON mode_3.\n");
        }
        #else
        vIO32Write4B(PDWNC_PINMUX3, u4IO32Read4B(PDWNC_PINMUX3) & 0xf00fffff);
        #endif
        #endif
#endif
    }
    else
    {
        //CHIP_DisplayString("U0==>T8032!\n");
    }
}
#endif // CC_MTK_PRELOADER

void CHIP_BootEntry0(void)
{
#ifdef CC_MTK_PRELOADER
    // clear SRAM for preloader BSS section.
    #if defined(CC_MT5895) && (!defined(CC_MTK_HOTBOOT_PRELOADER)) && (!defined(CC_MTK_HOTBOOT_START))
    extern void *memset(void *src, int c, size_t count);
    memset(SRAM_PDWNC_BASE, 0, SRAM_PDWNC_SIZE);
    #endif

#if (!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))
    #if !defined(CC_NAND_BOOT) && !defined(CC_EMMC_BOOT)
    if ((BIM_READ32(REG_RW_TIMER_CTRL) & (TMR2_CNTDWN_EN | TMR2_AUTOLD_EN)) == 0)
    {
        // Start timer 2 for bootup time counter enable.
        BIM_WRITE32(REG_RW_TIMER2_LLMT, 0xffffffff);
        BIM_WRITE32(REG_RW_TIMER2_LOW , 0xffffffff);
        BIM_WRITE32(REG_RW_TIMER2_HLMT, 0xffffffff);
        BIM_WRITE32(REG_RW_TIMER2_HIGH, 0xffffffff);
        BIM_ClrTimeLog(0);
        BIM_ClrTimeLog(1);
        BIM_ClrTimeLog(2);
        BIM_ClrTimeLog(3);
        BIM_WRITE32(REG_RW_TIMER_CTRL, (TMR2_CNTDWN_EN | TMR2_AUTOLD_EN));
    }
    #endif



    if (!CHIP_IsRemap())
    {
        BIM_SetTimeLog(0);
    #ifdef CC_DUALLOADER_SUPPORT
        vIO32Write4B(PDWNC_WDT2, 0x40000000); //~44s @24MHz    need more time
        if (BIM_READ32(REG_RW_GPRDW6) == DUALLOADER_A)
        {
            BIM_WRITE32(REG_RW_GPRDW6, DUALLOADER_B);
        }
        else if (BIM_READ32(REG_RW_GPRDW6) != DUALLOADER_B)
        {
            BIM_WRITE32(REG_RW_GPRDW6, DUALLOADER_A);
            vIO32Write4B(PDWNC_WDTCTL, 0xff0000); //enable WDT2   //if do not use the dual loader time out just matk this
        }
    #endif
    }
#endif //(!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))
#endif // CC_MTK_PRELOADER

#if (!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))
    // ; Show first 'o' to note that i-cache enable is ok.
    // MOV     r0, #0x6f
    // BL      CHIP_DisplayChar
    CHIP_DisplayChar('o');

    // ; Setup BIM initial register
    // BL      CHIP_BimInit
    CHIP_BimInit();

//#ifndef CC_MTK_LOADER
    // ; Setup board clock
    // BL      CHIP_CheckBoard
    CHIP_CheckBoard();
//#endif /* CC_MTK_LOADER */

    // ; Show second 'o' to note that BIM enable is ok.
    // MOV     r0, #0x6f
    // BL      CHIP_DisplayChar
    CHIP_DisplayChar('o');

#ifdef CC_MTK_PRELOADER
    if (!CHIP_IsRemap() && !BSP_IsFPGA())
    {
        // ; Check if remapped, initialize BIM and DRAM for the first time only
        // BL      CHIP_ClockSetting
        CHIP_ClockSetting();
    }
#endif // CC_MTK_PRELOADER

    // ; Show 't' to note that clock initialization is ok.
    // MOV     r0, #0x74
    // BL      CHIP_DisplayChar
    CHIP_DisplayChar('t');
#endif //(!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))

    // ; Check Dram initialized
    // BL      CHIP_IsRemap
    // CMP     r0, #0
    // BNE     SoftInit
#ifdef CC_MTK_PRELOADER
    if (!CHIP_IsRemap())
    {
#if (!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))
        // MOV     r0, #0x2d
        // BL      CHIP_DisplayChar
        CHIP_DisplayChar('-');

        // ; Setup 1st boot register initialization
        // BL      CHIP_1stBootRegInit
        CHIP_1stBootRegInit();
#if defined(CC_MT5863)
       //set hpd to low
       vIO32Write4B(PDWNC_PINMUX3,  u4IO32Read4B(PDWNC_PINMUX3)  & 0xFFFFFFF);//set gpio function:gpio
       vIO32Write4B(PDWNC_GPIOEN0,  u4IO32Read4B(PDWNC_GPIOEN0)  | 0x020000);//set output
       vIO32Write4B(PDWNC_GPIOOUT0, u4IO32Read4B(PDWNC_GPIOOUT0) & 0xfffdffff);//set output 0

       vIO32Write4B(PDWNC_PINMUX4, u4IO32Read4B(PDWNC_PINMUX1)  & 0xFFFFFFF1) ;
       vIO32Write4B(PDWNC_GPIOEN0,  u4IO32Read4B(PDWNC_GPIOEN0)  | 0x400000);
       vIO32Write4B(PDWNC_GPIOOUT0, u4IO32Read4B(PDWNC_GPIOOUT0) & 0xffbfffff);

       vIO32Write4B(PDWNC_PINMUX4,  u4IO32Read4B(PDWNC_PINMUX1)  & 0xFFFFFF0F);
       vIO32Write4B(PDWNC_GPIOEN1,  u4IO32Read4B(PDWNC_GPIOEN0)  | 0x00000001);
       vIO32Write4B(PDWNC_GPIOOUT1, u4IO32Read4B(PDWNC_GPIOOUT0) & 0xfffffffe);
#endif
//BIM_SetTimeLog(0);
    #ifndef CC_FPGA
        #if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5865) || defined(CC_MT5895)
        #ifdef CC_DUALLOADER_SUPPORT
        if (!(u4IO32Read4B(PDWNC_PDSTAT0) || u4IO32Read4B(PDWNC_PDSTAT1) || (u4IO32Read4B(PDWNC_WDTCTL) & 0xff00ffff)))
        #else
        if (!(u4IO32Read4B(PDWNC_PDSTAT0) || u4IO32Read4B(PDWNC_PDSTAT1) || u4IO32Read4B(PDWNC_WDTCTL)))
        #endif
        {
            #if defined(CC_MT5895)
			vIO32Write4B(PDWNC_RESRV4, 0x0);// bit1 -> FLD_RG_CM4_GA_STATUS, bit0 -> FLD_RG_HOTBOOT_FLOW_CTRL
            #endif
			vIO32Write4B(PDWNC_RESRV5, 0x0);
        }
        if ((IO32ReadFldAlign(PDWNC_CONCFG1, FLD_RG_DMSUS_RESERVED) == 0x0) && (IO32ReadFldAlign(PDWNC_RESRV5, FLD_RG_WDT_DRAMSR) != 0x1))
        {
        	vIO32WriteFldAlign(PDWNC_CONCFG2, 0x1, FLD_RG_DMSUS);
            CHIP_Delay_us(10);
            vIO32WriteFldAlign(PDWNC_CONCFG2, 0x0, FLD_RG_DMSUS);
        }
        #endif
        #if defined(CC_MT5890) || defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5865) || defined(CC_MT5895)
        if (u4IO32Read4B(PDWNC_PDSTAT0) || u4IO32Read4B(PDWNC_PDSTAT1))
        #else
        if (u4IO32Read4B(PDWNC_PDSTAT))
        #endif
        {
            CHIP_NotifyT8032();
        }
        #if defined(CC_MT5886)
        else if (IO32ReadFldAlign(PDWNC_RESRV5, FLD_RG_WDT_DRAMSR) != 0x1)
        {
            if (BSP_GetIcVersion() == IC_VER_5886_AA)
            {
                vIO32WriteFldAlign(PDWNC_CLKPDN, 0x1, FLD_T8032PD);
                vIO32Write4B(PDWNC_MEMPD, (u4IO32Read4B(PDWNC_WDTCTL) | 0x7));
            }
        }
        #endif
    #endif // CC_FPGA
#endif // (!defined(CC_MT5895))||(defined(CC_MT5895)&&(!defined(CC_MTK_HOTBOOT_PRELOADER)))
#ifndef CC_MTK_HOTBOOT_START
        //BIM_SetTimeLog(1);
        // ; Setup Dram
        // BL      CLK_SetDdr
        CLK_SetDdr();
#endif // CC_MTK_HOTBOOT_START
    }
#endif // CC_MTK_PRELOADER
}
