/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-
 * Copyright (c) 1986, 1988, 1991, 1993
 *	The Regents of the University of California.  All rights reserved.
 * (c) UNIX System Laboratories, Inc.
 * All or some portions of this file are derived from material licensed
 * to the University of California by American Telephone and Telegraph
 * Co. or Unix System Laboratories, Inc. and are reproduced herein with
 * the permission of UNIX System Laboratories, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)subr_prf.c	8.3 (Berkeley) 1/21/94
 * $FreeBSD: src/lib/libstand/printf.c,v 1.2.2.1 1999/08/29 14:57:34 peter Exp $
 */

/*
 * Standaloneified version of the FreeBSD kernel printf family.
 */

#include "x_lint.h"

LINT_SAVE_AND_DISABLE   // Do not lint this file

#include "x_serial.h"
#include "x_assert.h"

#if defined(CC_MT5893) || defined(CC_MT5895)
#include "dramc_pi_api.h"
#else
#include "ddr.h"
#endif

#include "u_common.h"

#ifndef CC_MTK_LOADER
#ifdef DRV_DBG_LOG_TO_FILE
// #include "u_common.h"
#endif
#endif

#define NBBY        (8)

#if defined(DRAM_DEBUG) || !defined(CC_ROM_BOOT) || !defined(CC_MTK_PRELOADER)

typedef unsigned long u_long;
typedef unsigned int u_int;
typedef unsigned char u_char;

static char	*ksprintn (u_long num, int base, int *len, char *buf);
static int	kvprintf(char const *fmt, VOID (*func)(CHAR), void *arg, int radix, VA_LIST ap);

void vprintf(const char *fmt, VA_LIST ap);
void vsprintf(char *buf, const char *cfmt, VA_LIST ap);

EXTERN VOID outbyte(CHAR c);
/* assuming sizeof(void *) == sizeof(int) */
#ifndef CC_MTK_LOADER
#ifdef DRV_DBG_LOG_TO_FILE
EXTERN INT32 x_log_to_drv_file_query(CHAR** ppBuffer,UINT32 *p_ui4Len);
EXTERN VOID x_log_to_drv_file_output_fct(CONST CHAR* ps_stmt);
EXTERN VOID *x_memset(VOID *s, INT32 c, UINT32 n);
EXTERN INT32 x_vsnprintf (CHAR*        ps_str,
                          SIZE_T       z_size,
                          CONST CHAR*  ps_format,
                          VA_LIST      VA_LIST);
#endif
#else
#ifdef CC_MTKLOADER_NO_LOG
EXTERN UINT32 LDR_IsEnableLog(void);
#endif
#endif

const CHAR hex2ascii_array[] = {    '0', '1', '2', '3', '4', '5', '6', '7',
                                    '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

#define hex2ascii(h)    (hex2ascii_array[h])

INT32
Printf(const CHAR *fmt, ...)
{
    VA_LIST t_ap;
    int retval = 0;

#ifndef CC_MTK_LOADER
    #ifdef DRV_DBG_LOG_TO_FILE
    register INT32 *varg = (INT32 *)(&fmt);
    CHAR *p = NULL;
    UINT32 ui4_len;

    varg = varg;
    if (x_log_to_drv_file_query(&p, &ui4_len))
    {
        VA_START(t_ap, fmt);
        x_memset(p, 0, ui4_len);
        retval = x_vsnprintf(p, ui4_len - 1, fmt, t_ap);
        //nRet = print(&p, varg);
        x_log_to_drv_file_output_fct(p);
        VA_END(t_ap);
    }
    else
    #endif
#else
    #ifdef CC_MTKLOADER_NO_LOG
    if (LDR_IsEnableLog())
    #endif
#endif
    {
        VA_START(t_ap, fmt);
        retval = kvprintf(fmt, outbyte, NULL, 10, t_ap);
        VA_END(t_ap);
    }
    return retval;
}

void
vprintf(const char *fmt, VA_LIST ap)
{
	kvprintf(fmt, outbyte, NULL, 10, ap);
}

INT32
SPrintf(CHAR *buf, const CHAR *cfmt, ...)
{
	int retval;
	VA_LIST ap;

	VA_START(ap, cfmt);
	retval = kvprintf(cfmt, NULL, (void *)buf, 10, ap);
	buf[retval] = '\0';
	VA_END(ap);
	return retval;
}

void
vsprintf(char *buf, const char *cfmt, VA_LIST ap)
{
	int	retval;

	retval = kvprintf(cfmt, NULL, (void *)buf, 10, ap);
	buf[retval] = '\0';
}

/*
 * Put a number (base <= 16) in a buffer in reverse order; return an
 * optional length and a pointer to the NULL terminated (preceded?)
 * buffer.
 */
static char *
ksprintn(u_long ul, int base, int *lenp, char *buf)
{					/* A long in base 8, plus NULL. */
/*	char buf[sizeof(long) * NBBY / 3 + 2]; */
	register char *p;

	p = buf;
	do {
		*++p = hex2ascii(ul % base);
	} while (ul /= base);
	if (lenp)
		*lenp = p - buf;
	return (p);
}

/*
 * Scaled down version of printf(3).
 *
 * Two additional formats:
 *
 * The format %b is supported to decode error registers.
 * Its usage is:
 *
 *	printf("reg=%b\n", regval, "<base><arg>*");
 *
 * where <base> is the output base expressed as a control character, e.g.
 * \10 gives octal; \20 gives hex.  Each arg is a sequence of characters,
 * the first of which gives the bit number to be inspected (origin 1), and
 * the next characters (up to a control character, i.e. a character <= 32),
 * give the name of the register.  Thus:
 *
 *	kvprintf("reg=%b\n", 3, "\10\2BITTWO\1BITONE\n");
 *
 * would produce output:
 *
 *	reg=3<BITTWO,BITONE>
 *
 * XXX:  %D  -- Hexdump, takes pointer and separator string:
 *		("%6D", ptr, ":")   -> XX:XX:XX:XX:XX:XX
 *		("%*D", len, ptr, " " -> XX XX XX XX ...
 */
static int
kvprintf(char const *fmt, VOID (*func)(CHAR), void *arg, int radix, VA_LIST ap)
{
#define PCHAR(c) { if (func) (*func)(c); else *d++ = c; retval++; }
	char buf[sizeof(long) * NBBY / 3 + 2] = {0};
	char *p, *q, *d;
	u_char *up;
	int n;
	CHAR ch;
	u_long ul;
	int base, lflag, tmp, width, ladjust, sharpflag, neg, sign, dot;
	int dwidth;
	char padc;
	int retval = 0;

	if (!func)
		d = (char *) arg;
	else
		d = NULL;

	if (fmt == NULL)
		fmt = "(fmt null)\n";

	if (radix < 2 || radix > 36)
		radix = 10;

	for (;;) {
		padc = ' ';
		width = 0;
		ch = (u_char)*fmt++;
		while ((ch) != '%') {
			if (ch == '\0')
				return retval;
			PCHAR(ch);
		    ch = (u_char)*fmt++;
		}
		lflag = 0; ladjust = 0; sharpflag = 0; neg = 0;
		sign = 0; dot = 0; dwidth = 0;
reswitch:
        ch = (u_char)*fmt++;
        switch (ch) {
		case '.':
			dot = 1;
			goto reswitch;
		case '#':
			sharpflag = 1;
			goto reswitch;
		case '+':
			sign = 1;
			goto reswitch;
		case '-':
			ladjust = 1;
			goto reswitch;
		case '%':
			PCHAR(ch);
			break;
		case '*':
			if (!dot) {
				width = VA_ARG(ap, int);
				if (width < 0) {
					ladjust = !ladjust;
					width = -width;
				}
			} else {
				dwidth = VA_ARG(ap, int);
			}
			goto reswitch;
		case '0':
			if (!dot) {
				padc = '0';
				goto reswitch;
			}
		case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
				for (n = 0;; ++fmt) {
					n = n * 10 + ch - '0';
					ch = *fmt;
					if (ch < '0' || ch > '9')
						break;
				}
			if (dot)
				dwidth = n;
			else
				width = n;
			goto reswitch;
		case 'b':
			ul = VA_ARG(ap, int);
			p = VA_ARG(ap, char *);
			for (q = ksprintn(ul, *p++, NULL, &buf[0]); *q;)
				PCHAR(*q--);

			if (!ul)
				break;

			for (tmp = 0; *p;) {
				n = *p++;
				if (ul & (1 << (n - 1))) {
					PCHAR(tmp ? ',' : '<');
					for (; (n = *p) > ' '; ++p)
						PCHAR((CHAR)n);
					tmp = 1;
				} else
					for (; *p > ' '; ++p)
						continue;
			}
			if (tmp)
				PCHAR('>');
			break;
		case 'c':
			PCHAR(VA_ARG(ap, int));
			break;
		case 'D':
			up = VA_ARG(ap, u_char *);
			p = VA_ARG(ap, char *);
			if (!width)
				width = 16;
			while(width--) {
				PCHAR(hex2ascii(*up >> 4));
				PCHAR(hex2ascii(*up & 0x0f));
				up++;
				if (width)
					for (q=p;*q;q++)
						PCHAR(*q);
			}
			break;
		case 'd':
			ul = lflag ? VA_ARG(ap, long) : VA_ARG(ap, int);
			sign = 1;
			base = 10;
			goto number;
		case 'l':
			lflag = 1;
			goto reswitch;
		case 'n':
			ul = lflag ? VA_ARG(ap, u_long) : VA_ARG(ap, u_int);
			base = radix;
			goto number;
		case 'o':
			ul = lflag ? VA_ARG(ap, u_long) : VA_ARG(ap, u_int);
			base = 8;
			goto number;
		case 'p':
			ul = (u_long)VA_ARG(ap, void *);
			base = 16;
			sharpflag = 1;
			goto number;
		case 's':
			p = VA_ARG(ap, char *);
			if (p == NULL)
				p = "(null)";
			if (!dot) {
                register INT32 len = 0;
                register const CHAR *ptr;
                for (ptr = p; *ptr; ++ptr) ++len;
				// n = x_strlen (p);
                n = len;
			} else
				for (n = 0; n < dwidth && p[n]; n++)
					continue;

			width -= n;

			if (!ladjust && width > 0)
				while (width--)
					PCHAR(padc);
			while (n--)
				PCHAR(*p++);
			if (ladjust && width > 0)
				while (width--)
					PCHAR(padc);
			break;
		case 'u':
			ul = lflag ? VA_ARG(ap, u_long) : VA_ARG(ap, u_int);
			base = 10;
			goto number;
		case 'x':
		case 'X':
			ul = lflag ? VA_ARG(ap, u_long) : VA_ARG(ap, u_int);
			base = 16;
number:			if (sign && (long)ul < 0L) {
				neg = 1;
				ul = -(long)ul;
			}
			p = ksprintn(ul, base, &tmp, &buf[0]);
			if (sharpflag && ul != 0) {
				if (base == 8)
					tmp++;
				else if (base == 16)
					tmp += 2;
			}
			if (neg)
				tmp++;

			if (!ladjust && width && (width -= tmp) > 0)
				while (width--)
					PCHAR(padc);
			if (neg)
				PCHAR('-');
			if (sharpflag && ul != 0) {
				if (base == 8) {
					PCHAR('0');
				} else if (base == 16) {
					PCHAR('0');
					PCHAR('x');
				}
			}

			while (*p)
				PCHAR(*p--);

			if (ladjust && width && (width -= tmp) > 0)
				while (width--)
					PCHAR(padc);

			break;
		default:
			PCHAR('%');
			if (lflag)
				PCHAR('l');
			PCHAR(ch);
			break;
		}
	}
#undef PCHAR
}

LINT_RESTORE

#endif //#if !defined(CC_ROM_BOOT) && !defined(CC_MTK_PRELOADER)

