/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "drv_scaler_drvif.h"
#include "vdp_display.h"
#include "scpos_debug.h"
#include "drv_scaler_psc.h"
#include "drv_mjc_sys.h"
#include "hw_psc.h"
#include "hw_ospe.h"
#include "panel.h"

#include "hw_w2dr2r.h"
#include "hw_mjc.h"
#include "drv_vrr.h"

#if defined(SCPIP_SUPPORT_POST_SCALER)


#define PSC_FRAME_TRACK_TARGET  250
#define OSD_HSYNC_DLY     0
#define OSD_VSYNC_DLY     0xFFFD

typedef struct
{
    UINT8 bPscModeChanged;
    VRM_INFO_T pscIn;
    VRM_INFO_T pscOut;
	UINT32 u4HDeltaPhaseIntg;	            //H Scaling factor (integer part)
	UINT32 u4HDeltaPhaseFrac;	            //H Scaling factor (fraction part)
	UINT32 u4VDeltaPhaseIntg;               //V Scaling factor (integer part)
	UINT32 u4VDeltaPhaseFrac;	            //V Scaling factor (fraction part)	
    UINT32 u4HInitPhaseSign;                //H Scaling phase  (sign)
    UINT32 u4HInitPhaseIntg;                //H Scaling phase  (integer)	    
    UINT32 u4HInitPhaseFrac;                //H Scaling phase  (fraction)
    UINT32 u4VInitPhaseSign;                //V Scaling phase  (sign)
    UINT32 u4VInitPhaseIntg;                //V Scaling phase  (integer)
    UINT32 u4VInitPhaseFrac;                //V Scaling phase  (fraction)
	UINT32 u4InputWidth;			        //input width(pixel based)
	UINT32 u4InputHeight;                   //input height(pixel based)
	UINT32 u4OutputX;			            //output X(pixel based)
	UINT32 u4OutputY;                       //output Y(pixel based)
	UINT32 u4OutputWidth;                   //output Width(pixel based)
	UINT32 u4OutputHeight;	                //output Height(pixel based)   
	UINT32 u4CropOfstX;			            //crop output starting point offset X
	UINT32 u4CropOfstY;                     //crop output starting point offset Y
	UINT32 u4CropROIWidth;                  //crop output ROI Width
	UINT32 u4CropROIHeight;	                //crop output ROI Height
	UINT8 bCropEnable;
	UINT8 bVscalerEnable;
	BOOL bNeedSetData;                    //update XY later (Before Set data trigger)
}PSC_INFO_T;

PSC_INFO_T pscInfo;

UINT8 pscInitFlag = SV_FALSE;
UINT32	g_cropDelay=1;

#ifndef CC_MTK_LOADER
//static functions
static void _vPscUpdateDispPrm(void);
static void _vPscUpdateInOutRegion(void);
static void _vPscUpdateOutputResolution(UINT32 outX, UINT32 outY, UINT32 outW, UINT32 outH);
static void _vPscUpdateReg(void);
static void _vPscSetImportOnOff(UINT8 bOnOff);                
#endif
static void vPscSetOsdHVSyncDly( UINT16 u2Osd_HsyncDly, UINT16 u2Osd_VsyncDly );

static void _vPscInitSyncAdj(void);
EXTERN BOOL fgFrameTrackIsFlgSet(UINT32 u4Flag);
    
#ifndef CC_MTK_LOADER

void vPscFireModeChanged(void)
{
    pscInfo.bPscModeChanged = SV_TRUE;
}

void vPscClearModeChanged(void)
{
    pscInfo.bPscModeChanged = SV_FALSE;
}

UINT8 fgPscIsModeChanged(void)
{
    RVChannel* mainChannel;
    RVChannel* subChannel;
    UINT8 ret = SV_FALSE;

    mainChannel = getChannel(SV_VP_MAIN);
    subChannel = getChannel(SV_VP_PIP);
    
    if ( (mainChannel->bIsChannelOn ==SV_TRUE && mainChannel->bIsFreeze == SV_FALSE) || 
         (subChannel->bIsChannelOn ==SV_TRUE && subChannel->bIsFreeze == SV_FALSE) ||
         (pscInitFlag == SV_TRUE))   //channel is on & not freezed
    {
        if(pscInfo.bPscModeChanged == SV_TRUE)
        {
            ret = SV_TRUE;
        }
    }
    return ret;
}
/******************************************
*  @Cobra porting done
******************************************/
void vPscSetImportOff(UINT8 bOnOff)
{
    if(bOnOff == SV_ON)
    {
		vPscSetResToggle(SV_ON);
    }
    else
    {	
		vPscSetResToggle(SV_OFF);
    }
}

extern UINT32 GCD(UINT32 a, UINT32 b);

UINT32 vPscCalPscRefOfst(void)
{ 
    UINT32 u4PscOfst;

    if(bDrvIsHdmiVRRMode())
    {
        // 60~48  trig_line (vdec + 16)
        // ofst = (vsync + vbp + vdec/2) + (vtot_min - vtot_max) + tor(50)
        //if(bDrvVideoGetRefreshRate(VDP_1) <= 60)

        // 120~48 trig_line (vdec + 16)
        // min delay = 1 + ofst(0.x)
        // ofst = trig line should get curren wr frm index. wr frm index update in vsync.
        if(u1VRRLowLatencyModeType() == E_VRR_LL_MODE_4k120_60)
        {
            u4PscOfst = u4ScpipGetPanelVtotal(E_REF_TG)/2;
        }
        else
        {
            //u4PscOfst = u4ScpipGetPanelVtotal(E_REF_TG)/20 + 10;

            u4PscOfst = u4ScpipGetPanelVtotal(E_REF_TG)/2 + 10;
        }
    }
    else
    {
        UINT32 u4TrackTarget = vPscCalPscTrkLine();

        UINT32 u4Ratio;
        UINT8 u1InFR;
        UINT8 u1OutFR;

        #define SCALE 100

        u1InFR = u2DrvGetInternalFrameRate();
        u1OutFR = vDrvGetLCDFreq();

        u4Ratio = u1OutFR * SCALE / GCD(u1InFR, u1OutFR);

        u4PscOfst = u4ScpipGetPanelVtotal(E_REF_TG) - (u4TrackTarget * SCALE / u4Ratio);
    }

    return u4PscOfst;
}


UINT32 vPscCalPscTrkLine(void)
{ 
    UINT32 u4TrackTarget;
    UINT32 u4Vtotal = wDrvGetOutputVTotal();
    UINT32 u4VFp = wDrvGetVsyncFp();

    if(fgFrameTrackIsFlgSet(FRAME_TRACK_MODE_DDDS_BY_REF))
    {
        u4Vtotal = u4ScpipGetPanelVtotal(E_REF_TG);
        u4VFp = u4ScpipGetPanelVtotal(E_REF_TG)-u4ScpipGetPanelHeight(E_REF_TG)-u4ScpipGetVsyncFp(E_REF_TG)-u4ScpipGetVsyncBp(E_REF_TG);
    }
    else
    {
        if(u4ScpipGetVTModeCtrl(E_PSC_TG) == E_TRK_FRONT_PORCH)
            u4VFp = wDrvGetVsyncWidth();
    }

    if(u1IsPanelSupport4K2K120() == SV_TRUE)
    {
        if((vGetVDPColorMode(SV_VP_MAIN) == SV_COLOR_MODE_444) 
            || (u2DrvGetInternalFrameRate() >= 96))
        {
            // game/graphic mode or highframerate case, memc do repeat mode.
            u4TrackTarget = u4Vtotal - u4Vtotal/20 - u4VFp;
        }
        else
        {
            //1->2 frame convert
            // keep 0.5 frame output delay + 0.5 input delay control by memc
            u4TrackTarget = u4Vtotal/2;
        }
    }
    else if(u1DrvIsPnl4k2k60WithMemc())
    {
        if((vGetVDPColorMode(SV_VP_MAIN) == SV_COLOR_MODE_444) || (bDrvVideoGetRefreshRate(VDP_1) >= 96 && !fgApiVideoIsVgaTiming(VDP_1)))
        {
            // game/graphic mode or highframerate case, memc do repeat mode.
            u4TrackTarget = u4Vtotal - u4Vtotal/20 - u4VFp;
        }
        else
        {
            //1->2 frame convert
            // keep 0.5 frame output delay + 0.5 input delay control by memc
            u4TrackTarget = u4Vtotal/2;
        }
    }
    else
    {
        // in paneless model, keep repeat mode.
        u4TrackTarget = u4Vtotal - u4Vtotal/20 - u4VFp;
    }

    return u4TrackTarget;
}

/******************************************
*  @ cobra need review
******************************************/
void vPscConfigFrameTrack(UINT32 u4Step)
{
    UINT8 u1InFR;
    UINT8 u1OutFR;
    UINT8 u1TrackVNum;
    UINT8 bLockWin;
    //UINT32 u4PreVofst;
    //UINT32 u4PanelVTotal;
    //UINT32 u4UpperBound;
    //UINT32 u4LowerBound;
    //UINT32 u4LockRange;
    UINT8 u1MaskInput=0;
    UINT32 u4TrackTargetVal;

    bLockWin = 2;

    u1InFR = u2DrvGetInternalFrameRate();
    u1OutFR = vDrvGetLCDFreq();
    if(VDP_LogChkMsk(VDP_1,VDP_LOG_DBG_MSK_TRK)) LOG(0,"@MTK_TRK [%s][%d] Sig:%d MJC IN:%d Out:%d \n",__FUNCTION__,__LINE__,
        bDrvVideoSignalStatus(SV_VP_MAIN), u1InFR, u1OutFR);
    if ((bDrvVideoSignalStatus(SV_VP_MAIN) != SV_VDO_STABLE) || (u1InFR==0) || (u1OutFR==0))
        return;

    u1TrackVNum = u1InFR / GCD(u1InFR, u1OutFR);
    bLockWin *= u1TrackVNum;

    if (u1TrackVNum==1)  //input == output
    {
        u1MaskInput=0;
    }
    else
    {
        u1MaskInput=u1TrackVNum;
    }

    //u4PanelVTotal = u4PscGetPanelVTotal();
    
    //1. set frame track interval
    vPscMaskInputFrame(u1MaskInput);

    u4TrackTargetVal = vPscCalPscTrkLine();

    if(VDP_LogChkMsk(VDP_1,VDP_LOG_DBG_MSK_TRK)) LOG(0,"@MTK_TRK [%s][%d] TRK_LINE:%d VT:%d \n",__FUNCTION__,__LINE__,
        u4TrackTargetVal,wDrvGetOutputVTotal());

    //2. set frame track target. current's target is PRE_VOFST_BYPASS
    //using 96 default value?? need to review!!
    vPscSetFrameTrackTargetDistance(u4TrackTargetVal);
    
    //6. set track range, need to do

    //Following is  patch for oclk jitter issue
    if(!IS_DISP_4K2K_TYPE3)
    {//Non-BE model
        if(PANEL_Is4K2K60())
        {//4K60 APH model
            vPscSetFrameTrackLockWin(4);
            vPscSetFrameTrackLockTolerateLineNum(6);
        }
        else if(PANEL_Is4K2K120())
        {//4K120 AX model
            vPscSetFrameTrackLockWin(4);
            vPscSetFrameTrackLockTolerateLineNum(15);
        }
        else
        {
            vPscSetFrameTrackLockWin(bLockWin);
            vPscSetFrameTrackLockTolerateLineNum(bLockWin);
        }
    }
    else
    {//BE model
        vPscSetFrameTrackLockWin(bLockWin);
        vPscSetFrameTrackLockTolerateLineNum(bLockWin);
    }
    
    //8.not cross vsync   // 0: not cross Vsync, 1: frm_trk in  shortest path
    vPscSetFrameTrackCrossVsync(0);

	//7. other setting
	//vPscSetFrameTrackOnceOnly(UINT8 bOnOff)
}

/******************************************
*  @ Cobra porting done
******************************************/
void vPscSetFrameTrackOnOff(UINT32 u4OnOff)
{
    #if defined(__MODEL_slt__) || defined(DRV_SUPPORT_EXTMJC) // Frame tracking always off for SLT and external MJC
    vPscSetFrameTrackStep(0);
    vPscSetDddsFrameTrackEnable(SV_OFF);
    #else
    if(VDP_LogChkMsk(VDP_1,VDP_LOG_DBG_MSK_TRK)) LOG(0,"@MTK_TRK [%s][%d] u4OnOff:%d \n",__FUNCTION__,__LINE__,u4OnOff);
    if (u4OnOff)  // notice the order
    {  
        if(fgFrameTrackIsFlgSet(FRAME_TRACK_MODE_DDDS_BY_PSTSCALER)) // DDDS frame track mode
        {
            //set to DDDS Adjust mode        
            vPscSetFrameTrackStep(0);
            if(u1DrvIsVdpBypCase())
            {
                vPscSetDddsFrameTrackEnable(SV_OFF);
            }
            else
            {
                vPscSetDddsFrameTrackEnable(SV_ON);
            }
        }
        else 
        {
            //set to V Total Adjust mode         
            vPscSetDddsFrameTrackEnable(SV_OFF);
			if(u1DrvIsVdpBypCase())
        	{
        	    vPscSetFrameTrackStep(0);
        	}
			else
			{
			    vPscSetFrameTrackStep(5);
			}
        }    
    }
    else
    {
         vPscSetFrameTrackStep(0);
         vPscSetDddsFrameTrackEnable(SV_OFF);
    }
    #endif    
}

void vPscSwInit(void)
{    
    pscInfo.bPscModeChanged = SV_FALSE;
    pscInfo.bNeedSetData = SV_FALSE;
	pscInfo.u4OutputX = 0XFFFF;
	pscInfo.u4OutputY = 0XFFFF;
	pscInfo.bVscalerEnable = SV_OFF;
}

static void _vPscUpdateInOutRegion(void)
{
    VRMGetModuleVRMInfo(SV_VP_MAIN, VRM_MODULE_PSC, &(pscInfo.pscIn), &(pscInfo.pscOut));
}

static void _vPscUpdateScaliingFactorPrm(UINT32 inW, UINT32 inH, UINT32 outW, UINT32 outH)
{
	UINT32 u4HPhaseInc;
	UINT32 u4VPhaseInc;
	
    //for H up case
    if ( inW == 0 || outW == 0 || outW < inW)
    { 
    	LOG(3,"!!PSC InputW or OutputW size is 0 or OutputW < InputW. inW(%d) inH(%d) outW(%d) outH(%d)\n", inW, inH, outW, outH);
		return;
    }
    else  
    {    
        //H scaling delta phase
		u4HPhaseInc = (inW * DFT_UNIT_PHASE(SV_VP_MAIN)) / outW;
		LOG(3, "PSC scaling factor H in(%d %d) out(%d %d)\n", inW, inH, outW, outH);
		
		pscInfo.u4HDeltaPhaseIntg = u4HPhaseInc >> DFT_FRAC_PHASE_BITS(SV_VP_MAIN);	
		pscInfo.u4HDeltaPhaseFrac = u4HPhaseInc & DFT_FRAC_PHASE_MASK(SV_VP_MAIN);
		LOG(3, "PSC scaling factor H(0x%x 0x%x)\n", pscInfo.u4HDeltaPhaseIntg, 
													pscInfo.u4HDeltaPhaseFrac);

        //H scaling init phase
		pscInfo.u4HInitPhaseIntg = 0;
	    if(pscInfo.u4HDeltaPhaseFrac == 0)
    	{
			pscInfo.u4HInitPhaseFrac = 0;
			pscInfo.u4HInitPhaseSign = 0;    	
    	}
	    else
    	{
			pscInfo.u4HInitPhaseFrac = DFT_UNIT_PHASE(SV_VP_MAIN) + 
				( ((inW-1)*DFT_UNIT_PHASE(SV_VP_MAIN)) - ((outW-1)*pscInfo.u4HDeltaPhaseFrac))/2;
			pscInfo.u4HInitPhaseSign = 1;            
    	}
		LOG(3, "PSC Scaling phase H(0x%x 0x%x 0x%x)\n", pscInfo.u4HInitPhaseSign,
									  pscInfo.u4HInitPhaseIntg,
									  pscInfo.u4HInitPhaseFrac);
    }

	if ( inH == 0 || outH == 0 || outH < inH)
	{ 
		pscInfo.bVscalerEnable = SV_OFF;
		LOG(3,"!!Vertical scaler off, PSC InputH or OutputH size is 0 or OutputH < InputH. inW(%d) inH(%d) outW(%d) outH(%d)\n", inW, inH, outW, outH);
		return;
	}
	else
	{
		//V scaling delta phase
		u4VPhaseInc = (inH * DFT_UNIT_PHASE(SV_VP_MAIN)) / outH;
		LOG(3, "PSC scaling factor V in(%d %d) out(%d %d)\n", inW, inH, outW, outH);
		
		pscInfo.u4VDeltaPhaseIntg = u4VPhaseInc >> DFT_FRAC_PHASE_BITS(SV_VP_MAIN);
	    pscInfo.u4VDeltaPhaseFrac = u4VPhaseInc & DFT_FRAC_PHASE_MASK(SV_VP_MAIN);
		LOG(3, "PSC scaling factor V(0x%x 0x%x)\n", pscInfo.u4VDeltaPhaseIntg, 
													pscInfo.u4VDeltaPhaseFrac);
		//V scaling init phase
		pscInfo.u4VInitPhaseIntg = 0;
		if(pscInfo.u4VDeltaPhaseFrac == 0)
		{
			pscInfo.u4VInitPhaseFrac = 0;
			pscInfo.u4VInitPhaseSign = 0;
		}
		else
		{
            pscInfo.u4VInitPhaseFrac =( DFT_UNIT_PHASE(SV_VP_MAIN) + (((inH-1)*DFT_UNIT_PHASE(SV_VP_MAIN))>>1) - (((outH-1)*pscInfo.u4VDeltaPhaseFrac)>>1)  );
			pscInfo.u4VInitPhaseSign = 1;
		}
		pscInfo.bVscalerEnable = SV_ON;
		LOG(3, "PSC Scaling phase V(0x%x 0x%x 0x%x)\n", pscInfo.u4VInitPhaseSign,
										  pscInfo.u4VInitPhaseIntg,
										  pscInfo.u4VInitPhaseFrac);
    }
	
}

static void _vPscUpdateInputResolution(UINT32 inW,UINT32 inH)
{
//	if(inW==1952)
  //  	pscInfo.u4InputWidth = 3840;
//	else
		pscInfo.u4InputWidth = inW;		
    pscInfo.u4InputHeight = inH;
}

static void _vPscUpdateOutputResolution(UINT32 outX, UINT32 outY, UINT32 outW, UINT32 outH)
{
    CRIT_STATE_T csState;    

    if(pscInfo.u4OutputX != outX || pscInfo.u4OutputY != outY)
    {
        vPscSetNeedSetDataFlag(SV_TRUE);
        LOG(3,"PSC SET DATA FLG = SV_TRUE\n");            
    }

    csState = x_crit_start();
    pscInfo.u4OutputX = outX;
    pscInfo.u4OutputY = outY;
    pscInfo.u4OutputWidth = outW;
    pscInfo.u4OutputHeight = outH;
    x_crit_end(csState);
}

static void _vPscUpdateDispCropPrm(UINT32 outX, UINT32 outY, UINT32 inW, UINT32 inH)
{
	if(inW==1952)
    	pscInfo.u4CropROIWidth = 3840;
	else
		pscInfo.u4CropROIWidth = inW;	
    pscInfo.u4CropROIHeight = inH;
    
    pscInfo.u4CropOfstX= pscInfo.u4CropROIWidth-inW;
    pscInfo.u4CropOfstY= outY;
}

static void _vPscUpdateDispPrm()
{    
    //1. update input & output size
    _vPscUpdateInOutRegion();
    //2. update H up scaling factor
    _vPscUpdateScaliingFactorPrm(pscInfo.pscIn.u2Width,pscInfo.pscIn.u2Height, pscInfo.pscOut.u2Width,pscInfo.pscOut.u2Height);
    //3. update input resolution
    _vPscUpdateInputResolution(pscInfo.pscIn.u2Width,pscInfo.pscIn.u2Height);
    //4. update output resolution
    _vPscUpdateOutputResolution(pscInfo.pscOut.u2X,pscInfo.pscOut.u2Y,
                                        pscInfo.pscOut.u2Width,pscInfo.pscOut.u2Height);
    //5. update disp crop prm
    _vPscUpdateDispCropPrm(pscInfo.pscOut.u2X,pscInfo.pscOut.u2Y, pscInfo.pscOut.u2Width,pscInfo.pscIn.u2Height);
}
/******************************************
*  @Cobra porting done
******************************************/
static void _vPscSetImportOnOff(UINT8 bOnOff)
{
    LOG(3,"Set PSC import(%d)\n",bOnOff);
	vPscSetResImportEnable(bOnOff);
}

static void _vPscUpdateReg(void)
{
    /**** start import protect ****/
    _vPscSetImportOnOff(SV_OFF);                
    //1. set input resolution
    vPscSetInputWidthHeight(pscInfo.u4InputWidth,pscInfo.u4InputHeight);
    //2. set output resolution
    //if(bPscIsNeedSetData()== SV_TRUE)
    {
        vPscSetHmskStartOfst(pscInfo.u4OutputX);
        vPscSetVmskStartOfst(pscInfo.u4OutputY);    
    }

    vPscSetOutputWidthHeight(pscInfo.u4OutputWidth, pscInfo.u4OutputHeight);    	
    
    //MEMC Output height
    vPscSetMEMCOutputWidth(pscInfo.u4InputWidth);
    
    //3. set H Delta phase
    vPscSetHDeltaPhase(pscInfo.u4HDeltaPhaseIntg, pscInfo.u4HDeltaPhaseFrac);
    //4. set H Initial phase
    vPscSetHInitPhase(pscInfo.u4HInitPhaseSign, pscInfo.u4HInitPhaseIntg, pscInfo.u4HInitPhaseFrac);
    //5. Set V Delta Phase
    vPscSetVDeltaPhase(pscInfo.u4VDeltaPhaseIntg, pscInfo.u4VDeltaPhaseFrac);
    //6. Set V Initial Phase
    vPscSetVInitPhase(pscInfo.u4VInitPhaseSign, pscInfo.u4VInitPhaseIntg, pscInfo.u4VInitPhaseFrac);

    //7. set mjc dispmode register (need review)
    if(u1DrvIsVdpBypCase())
    {
        PSC_WRITE_FLD(PSCSYS_11, SV_ON, PSC_INV_INPUT_MVDE);
    }
    else
    {   
        PSC_WRITE_FLD(PSCSYS_11, SV_OFF, PSC_INV_INPUT_MVDE);
    }

	//8. set disp crop prm
	if(u1IsPanelSupport4K2K120()==TRUE)
	{
		vPscSetCoreSel(1, 0);//core0
		v1PscSetRiscChannelSel(1,0);
		vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight);//(0x780, 0x870);//input w/2, h
		vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight);//(0x3d0, 0x870);//output w, h
		vPscSetDispCropOfst(0, 0);
		PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT);
		vPscSetDispCropEnable(SV_ON);
		vPscSetCoreSel(0, 1);//core1
		v1PscSetRiscChannelSel(1, 1);
		vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight);//(0x780, 0x870);//input w/2, h
		vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight);//(0x3d0, 0x870);//output w, h
		vPscSetDispCropOfst(pscInfo.u4CropOfstX/2,0);//(0x3b0, 0);
		PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT);
		vPscSetDispCropEnable(SV_ON);
	
		vPscSetCoreSel(1,1);
		v1PscSetRiscChannelSel(1,0);
		
	}

	//9. set vertical scaler on/off
	vPscSetVScalerEnable(pscInfo.bVscalerEnable);

    //_vPscSetImportOnOff(SV_ON);
    /**** end of import protect ****/
}

void vPscUpdateReg_SETDATA(void)
{
	LOG(3,"vPscUpdateReg_SETDATA \n");
    //2. set output resolution
    vPscSetHmskStartOfst(pscInfo.u4OutputX);
	vPscSetVmskStartOfst(pscInfo.u4OutputY);
}

void vPsc4k1k(UINT8 bOnOff)
{
    if (bOnOff) {
        vPscSetCoreSel(1, 0);//core0
        v1PscSetRiscChannelSel(1,0);
        vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight/2);//(0x780, 0x438);//input w/2, h
        vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight/2);//(0x3d0, 0x438);//output w, h
        vPscSetInputWidthHeight(pscInfo.u4InputWidth,pscInfo.u4InputHeight/2);
        vPscSetVDeltaPhase(0, 0x80000);
        vPscSetCoreSel(0, 1);//core1
        v1PscSetRiscChannelSel(1, 1);
        vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight/2);//(0x780, 0x438);//input w/2, h
        vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight/2);//(0x3d0, 0x438);//output w, h
        vPscSetInputWidthHeight(pscInfo.u4InputWidth,pscInfo.u4InputHeight/2);
        vPscSetVDeltaPhase(0, 0x80000);
        vPscSetCoreSel(1,1);
        v1PscSetRiscChannelSel(1,0);
    }
}

BOOL bPscIsNeedSetData(void)
{
    return pscInfo.bNeedSetData;
}

void vPscSetNeedSetDataFlag(UINT8 bFlag)
{
    pscInfo.bNeedSetData = bFlag;
}

void vPSCDumpResInfo(void)
{
}

void vPscPrintCurrentScale(void)
{
    LOG(0,"PSC H(0x%x-0x%x)\n",pscInfo.u4HDeltaPhaseIntg, pscInfo.u4HDeltaPhaseFrac);
}

void vDrvPscCropDelay(UINT32	cropDelay)
{
	g_cropDelay=cropDelay;
	LOG(0,"Set crop delay=%d us\n", g_cropDelay);
}

void vPscBypassPathEnable(UINT8 bOnOff)
{
    UINT16 Psc_Vblk, Psc_TargetLine, B2R_TargetLine,HDMI_TargetLine;
    UINT8 bType = bGetVideoDecType(SV_VP_MAIN);

    //psc bypass mode switch
    PSC_WRITE_FLD(PSCSYS_00, bOnOff, PSC_BYPASS_PATH_SEL);
    //PSC_WRITE_FLD(PSCSYS_06, bOnOff, PSC_TG_SLAVE_SYNC_MODE);
    //PSC_WRITE_FLD(PSCSYS_06, bOnOff, PSC_TG_SLAVE_MODE);

    

    
    if(bType == SV_VD_DVI)
//HDMi mux switch
    {
        vIO32WriteFldAlign(HDMI_HD_EN, SV_OFF, HDMI_HD_EN_VB1_DATA_MUX);
        vIO32WriteFldAlign(HDMI_HD_EN+0x10000, SV_OFF, HDMI_HD_EN_VB1_DATA_MUX);

        vIO32WriteFldAlign(REG_MEMC_420_444, SV_ON, MEMC_TOP_C_420_TO_444_BYPASS);
    }
    else//b2r mux switch
    {
        vIO32WriteFldAlign(HDMI_HD_EN, SV_ON, HDMI_HD_EN_VB1_DATA_MUX);
        vIO32WriteFldAlign(HDMI_HD_EN+0x10000, SV_ON, HDMI_HD_EN_VB1_DATA_MUX);

        //memc top config
        vIO32WriteFldAlign(REG_MEMC_420_444, ~SV_ON, MEMC_TOP_C_420_TO_444_BYPASS);
    }

    if(u1IsPanelSupport4K2K120()==TRUE)
    {
        if(bOnOff==SV_ON)
        {
            //turn off disp crop and vscaler
            vPscSetCoreSel(1, 0);//core0
            v1PscSetRiscChannelSel(1,0);
            PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT); 
/*
            if(IS_DISP_4K2K_FORMAT_YUV444)
            {
                vIO32WriteFldAlign(REG_MEMC_TOP_YUV2RGB_0, 0, MEMC_TOP_C_YUV2RGB_EN);//Need condition t
            }
            else
            {
                vIO32WriteFldAlign(REG_MEMC_TOP_YUV2RGB_0, 1, MEMC_TOP_C_YUV2RGB_EN);//Need condition t
            }
*/
            vPscSetDispCropEnable(SV_OFF);
            vPscSetVScalerEnable(SV_OFF);

            vPscSetCoreSel(0, 1);//core1
            v1PscSetRiscChannelSel(1, 1);
            PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT);
            vPscSetDispCropEnable(SV_OFF);
            vPscSetVScalerEnable(SV_OFF);

            vPscSetCoreSel(1,1);
            v1PscSetRiscChannelSel(1,0);

            //vScpipSetVTModeCtrl(E_PSC_TG,E_TRK_VSYNC_WIDTH);
        }
        else
        {
            vPscSetCoreSel(1, 0);//core0
            v1PscSetRiscChannelSel(1,0);
            vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight);//(0x780, 0x870);//input w/2, h
            vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight);//(0x3d0, 0x870);//output w, h
            vPscSetDispCropOfst(0, 0);
            PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT);
            //LOG(0,"%s%d\n",__FUNCTION__,__LINE__);
            //vIO32WriteFldAlign(REG_MEMC_TOP_YUV2RGB_0, 0, MEMC_TOP_C_YUV2RGB_EN);//Need condition to limit this operation,when color space is yuv
            vPscSetDispCropEnable(SV_ON);
            vPscSetCoreSel(0, 1);//core1
            v1PscSetRiscChannelSel(1, 1);
            vPscSetDispCropROIWidthHeight(pscInfo.u4CropROIWidth/2,pscInfo.u4CropROIHeight);//(0x780, 0x870);//input w/2, h
            vPscSetDispCropInputWidthHeight(pscInfo.u4OutputWidth/2,pscInfo.u4OutputHeight);//(0x3d0, 0x870);//output w, h
            vPscSetDispCropOfst(pscInfo.u4CropOfstX/2,0);//(0x3b0, 0);
            PSC_WRITE_FLD(DISP_CROP_DEBUG, SV_ON, DISP_CROP_FORCE_COMMIT);
            vPscSetDispCropEnable(SV_ON);

            vPscSetCoreSel(1,1);
            v1PscSetRiscChannelSel(1,0);

            vPscSetVScalerEnable(pscInfo.bVscalerEnable);
        }
    }

    //set target line
    if(bOnOff==SV_ON)
    {
	    UINT32 u4NewVFp = u4PscGetPanelVSyncFp();

        if(u4ScpipGetVTModeCtrl(E_PSC_TG) == E_TRK_FRONT_PORCH)
        {
            u4NewVFp = u4PscGetPanelVTotal() - u4PscGetPanelHeight() - u4PscGetPanelVSyncBp() - u4NewVFp;
            if(VDP_LogChkMsk(VDP_1,VDP_LOG_DBG_MSK_TRK)) LOG(0,"PSC swapVFP %d -> %d \n",u4PscGetPanelVSyncFp(),u4NewVFp);
        }

        Psc_Vblk = u4PscGetPanelVTotal()-(u4PscGetPanelHeight() + u4NewVFp + u4PscGetPanelVSyncBp() );
        //Psc_Vblk=u4PscGetPanelVTotal()-( u4PscGetPanelHeight()+u4PscGetPanelVSyncFp()+u4PscGetPanelVSyncBp() );
        
        if( (Psc_Vblk+u4PscGetPanelVSyncBp() )<10)
            Psc_TargetLine=10-(Psc_Vblk+u4PscGetPanelVSyncBp() );
        else
            Psc_TargetLine=0;
       
        if(bType == SV_VD_DVI)
        {
            Psc_TargetLine = 1;//delay 1 line for some pq module process between r2r and pst
            HDMI_TargetLine = 0;
             vScpipSetR2RTingenTargetLine(HDMI_TargetLine);
            LOG(0," HDMI_TargetLine=%d\n",HDMI_TargetLine );
        }
        else
        {
            B2R_TargetLine= ( Psc_TargetLine+Psc_Vblk+u4PscGetPanelVSyncBp() ) - 10;
            vScpipSetB2RTingenTargetLine(B2R_TargetLine);
            LOG(0," B2R_TargetLine=%d\n",B2R_TargetLine );
        }
        LOG(0,"PscVtotal=%d, PscHeight=%d, PscVSyncFp=%d, PscVSyncBp=%d\n", u4PscGetPanelVTotal(), u4PscGetPanelHeight(), u4NewVFp, u4PscGetPanelVSyncBp() );
        LOG(0,"Psc_Vblk=%d, Psc_TargetLine=%d\n", Psc_Vblk, Psc_TargetLine);
         vScpipSetPSCTingenTargetLine(Psc_TargetLine);
    }
    #if 0
    else
    {
        //normal case with ref timegen trk;
        Psc_TargetLine = (u4PscGetPanelVTotal() * 2 / 5); // 0.6 input delay
        LOG(0,"PscVtotal=%d, Target_line=%d\n",u4PscGetPanelVTotal(),Psc_TargetLine);
        vScpipSetPSCTingenTargetLine(Psc_TargetLine);
    }
    #endif
}


void vPscNormProc(void)
{
    //UINT8 u1CropCnt, u1CropCnt_tmp, u1CropCnt_tmp2;
    UINT8 u1PathSel;// 0 Bypass path, 1 PQ path
    if(fgPscIsModeChanged()==SV_TRUE)
    {
        u1PathSel=(u1DrvIsVdpBypCase()? SV_ON : SV_OFF);

        //1. update display parameter
        _vPscUpdateDispPrm();
        //2. update related register
        _vPscUpdateReg();
        //3. bypass/VDP mode check
        vPscBypassPathEnable(u1PathSel);
        //4. clear mode change
        vPscClearModeChanged();
    }
#if 0
    if(u1IsPanelSupport4K2K120())
		{
				//set final crop
				vIO32WriteFldAlign(CROPPING_DLY,1,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				vIO32WriteFldAlign(CROPPING_DLY,0,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				u1CropCnt=IO32ReadFldAlign(CROPPING_DLY, CROPPING_LATENCY_CNT);
				vIO32WriteFldAlign(CROPPING_DLY,1,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				vIO32WriteFldAlign(CROPPING_DLY,0,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				u1CropCnt_tmp=IO32ReadFldAlign(CROPPING_DLY, CROPPING_LATENCY_CNT);
				vIO32WriteFldAlign(CROPPING_DLY,1,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				vIO32WriteFldAlign(CROPPING_DLY,0,CROPPING_CNT_RST); 
				//x_thread_delay(g_cropDelay);
				u1CropCnt_tmp2=IO32ReadFldAlign(CROPPING_DLY, CROPPING_LATENCY_CNT);
				if(u1CropCnt_tmp==u1CropCnt && u1CropCnt_tmp2==u1CropCnt)
					vDrvSetWobblingFinalCrop(1, u1CropCnt-4, 1, 0);
				//LOG(0,"Set final crop=%d\n", u1CropCnt-4);
  	}
#endif
}
#endif

void vPscHwInit(void)
{
    //1. init MJC dispmode
    //_vPscInitMjcDispmode();
    //2. init SyncAdj/Active Adj/Read Delay
    _vPscInitSyncAdj();
    //3. set initial watel level
    //vPscSetDispFifoWaterLevel(PSC_DEFAULT_WTLV);
    //4. set psc h init phase
    //vPscSetHInitPhase(0, 0, 0);
    //5. select OSD v signal to VDE #VSYNC_SEL 0: Vsync , 1:VDE
    if (IS_IC_5895())
    {
        vPscSelectOsdVsync(0);
    }
    else
    {
        vPscSelectOsdVsync(1);
    }

    //inverse osd h/v sync 
    vPscSetInvOsdHVSync(SV_TRUE, SV_TRUE);

    vPscSetOsdHVSyncDly( OSD_HSYNC_DLY, OSD_VSYNC_DLY );
    
    vPscSetCoreSel(1,1);
    
    v1PscSetRiscChannelSel(1,0);
    
    if(!fgFrameTrackIsRefTrk())
    {
        //vPscTGSlaveSyncMode(0); //set to recomend vale '0'.
        vScpipSetSlaveSyncMode(E_PSC_TG,SV_OFF);
    }

    pscInitFlag = SV_TRUE;
}

static void vPscSetOsdHVSyncDly( UINT16 u2Osd_HsyncDly, UINT16 u2Osd_VsyncDly )
{
   vPscSetOsdHsyncDly( u2Osd_HsyncDly );
   vPscSetOsdVsyncDly( u2Osd_VsyncDly );
}

static void _vPscInitSyncAdj(void)
{
  	//not use
}

void vPscPreInit(void)
{
    static UINT32 _pscPreset = 0;

    if (_pscPreset == 0)
    {
        _pscPreset = 1;

        //todo: do pre init here
        vPscHwInit();
    }   
}

#endif
