/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: drv_vbyone.c,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

#include "general.h"
#include "x_typedef.h"
#include "nptv_debug.h"
#include "hw_vbyone.h"
#include "hw_mlvds.h"
#include "hw_lvds.h"
#include "hw_tg.h"
#include "hw_scpos.h"
#include "hw_ospe.h"
#include "drv_lvds.h"
#include "drv_vbyone.h"
#include "drv_vdoclk.h"
#include "drv_display.h"
#include "x_pinmux.h"
#include "x_chip_id.h"

// #define CC_VB1_DISABLE

#define VB1_4CH_EN  1
#if VB1_4CH_EN
#define VB1_2NDCH_OFFSET 0x600
#define VB1_3RDCH_OFFSET 0x400
#define VB1_4THCH_OFFSET 0x200
#endif

#define VB1_LANE_PAD_OFF 0xFF

BOOL _fgVB1UseFixTERM = TRUE;
UINT8 _u1VB1FixTERMVal = 0x8;  // 100ohm as default

#ifdef SUPPORT_PANEL_GRAPH_MODE
UINT8 _u1VB1OSDTest;
UINT8 _u1VB1OSDDeEn;
UINT8 _u1VB1OSDSel;
UINT8 _u1LineBufByp;
#endif

UINT32 _u4VB1PadEn = 0;

UINT8 _u1CRCChkCnt = 4;

///***************************For Capy(5895)*******************************
// no swap: A + C(OSD) + B + D
static const UINT64 _au4PanelVB1LaneNOSwap[32] =
{
    // 1st 8 lanes: 0x8064~0x8068
    VB1_DATA_LANE0, VB1_DATA_LANE1, VB1_DATA_LANE2, VB1_DATA_LANE3,
    VB1_DATA_LANE4, VB1_DATA_LANE5, VB1_DATA_LANE6, VB1_DATA_LANE7,

    // 3rd 8 lanes(osd 8 lanes): 0x8068~0x806C
    VB1_GRAPH_LANE0, VB1_GRAPH_LANE1, VB1_GRAPH_LANE2, VB1_GRAPH_LANE3,
    VB1_GRAPH_LANE4, VB1_GRAPH_LANE5, VB1_GRAPH_LANE6, VB1_GRAPH_LANE7,
    
    // 2nd 8 lanes: 0x806C~0x8070
    VB1_DATA_LANE8, VB1_DATA_LANE9, VB1_DATA_LANE10, VB1_DATA_LANE11,
    VB1_DATA_LANE12, VB1_DATA_LANE13, VB1_DATA_LANE14, VB1_DATA_LANE15,

    // 4th 8 lanes: 0x8074~0x8078
    VB1_DATA_LANE24, VB1_DATA_LANE25, VB1_DATA_LANE26, VB1_DATA_LANE27,
    VB1_DATA_LANE28, VB1_DATA_LANE29, VB1_DATA_LANE30, VB1_DATA_LANE31
};

// no swap: A + C(OSD) + B + D
static UINT32 au4LaneDefaultTbl[32]={VB1_LANE0,  VB1_LANE1,  VB1_LANE2,  VB1_LANE3,
                                    VB1_LANE4,  VB1_LANE5,  VB1_LANE6,  VB1_LANE7,
                                    VB1_OSD_LANE0, VB1_OSD_LANE1, VB1_OSD_LANE2, VB1_OSD_LANE3,
                                    VB1_OSD_LANE4, VB1_OSD_LANE5, VB1_OSD_LANE6, VB1_OSD_LANE7,
                                    VB1_LANE8, VB1_LANE9, VB1_LANE10, VB1_LANE11,
                                    VB1_LANE12, VB1_LANE13, VB1_LANE14, VB1_LANE15,
                                    VB1_LANE24, VB1_LANE25, VB1_LANE26, VB1_LANE27,
                                    VB1_LANE28, VB1_LANE29, VB1_LANE30, VB1_LANE31};

// for RG_LVDSA_EXT_EN bit allignment
static UINT32 au4PadRemapTbl[32]={LVDS_PAD_LANE0_AO0, LVDS_PAD_LANE1_AO1,   LVDS_PAD_LANE2_AO2, LVDS_PAD_LANE3_AOCLK,
                                 LVDS_PAD_LANE4_AO3, LVDS_PAD_LANE5_AO4,   LVDS_PAD_LANE6_AE0, LVDS_PAD_LANE7_AE1,
                                 LVDS_PAD_LANE8_AE2, LVDS_PAD_LANE9_AECLK, LVDS_PAD_LANE10_AE3, LVDS_PAD_LANE11_AE4,
                                 LVDS_PAD_LANE12_BO0, LVDS_PAD_LANE13_BO1, LVDS_PAD_LANE14_BO2, LVDS_PAD_LANE15_BOCLK,
                                 LVDS_PAD_LANE16_BO3, LVDS_PAD_LANE17_BO4, LVDS_PAD_LANE18_BE0, LVDS_PAD_LANE19_BE1,
                                 LVDS_PAD_LANE20_BE2, LVDS_PAD_LANE21_BECLK, LVDS_PAD_LANE22_BE3, LVDS_PAD_LANE23_BE4,
                                 LVDS_PAD_LANE24, LVDS_PAD_LANE25, LVDS_PAD_LANE26, LVDS_PAD_LANE27,
                                 LVDS_PAD_LANE28, LVDS_PAD_LANE29, LVDS_PAD_LANE30, LVDS_PAD_LANE31};
///**********************************************************************************

#if 0
static UINT32 u4PortA_1;  // 0x3586c
static UINT32 u4PortA_2;  // 0x35870
static UINT32 u4PortB_1;  // 0x3526c
static UINT32 u4PortB_2;  // 0x35270
static UINT8  u1OstgCHFlg;
#endif

#ifdef CC_COMBINE_4K_2K
extern UINT32 DRVCUST_Get_BE_Type(void);
#endif

void vVByOneWriteFldAlign(UINT32 u4Reg, UINT32 u4Val, UINT32 u4Fld)
{
	vIO32WriteFldAlign(u4Reg, u4Val, u4Fld);
	if (VB1_4CH_EN)
	{	
		vIO32WriteFldAlign((u4Reg-VB1_2NDCH_OFFSET), u4Val, u4Fld);
        vIO32WriteFldAlign((u4Reg-VB1_3RDCH_OFFSET), u4Val, u4Fld);
        vIO32WriteFldAlign((u4Reg-VB1_4THCH_OFFSET), u4Val, u4Fld);
	}
}

void vVByOneSetPadOn(UINT32 u4Pair, UINT8 u1GraphPair)
{
	UINT8 u1Lane = 0, u1LaneCnt = 0, u1GraphLaneCnt = 0;
	UINT32 *pu4LaneMap;
	
	if ( u4Pair == SINGLE_PORT)
	{
		u1LaneCnt = 1;
		LOG(3, "[VB1][SetPadOn] SINGLE_PORT\n");
	}
	else if ( u4Pair == DUAL_PORT) 
	{
		u1LaneCnt = 2;
		LOG(3, "[VB1][SetPadOn] DUAL_PORT\n");
	}
	else if ( u4Pair == FOUR_PORT)   
	{
		u1LaneCnt = 4;
		LOG(3, "[VB1][SetPadOn] FOUR_PORT\n");
	}
	else if ( u4Pair == OCTA_PORT)    
	{
		u1LaneCnt = 8;
		LOG(3, "[VB1][SetPadOn] OCTA_PORT\n");
	}
	else if (u4Pair == HEXA_PORT)
	{
		u1LaneCnt = 16;
		LOG(3, "[VB1][SetPadOn] HEXA_PORT\n");
	}

    if (IS_SUPPORT_VB1_GRAPH_MODE)
    {
    	if ( u1GraphPair == GM_LANE_1)  
    	{
    		u1GraphLaneCnt = 1;
    		LOG(3, "[VB1][SetPadOn] GM_LANE_1\n");
    	}
    	else if ( u1GraphPair == GM_LANE_2) 
    	{
    		u1GraphLaneCnt = 2;
    		LOG(3, "[VB1][SetPadOn] GM_LANE_1\n");
    	}
    	else if ( u1GraphPair == GM_LANE_4) 
    	{
    		u1GraphLaneCnt = 4;
    		LOG(3, "[VB1][SetPadOn] GM_LANE_4\n");
    	}
        else if ( u1GraphPair == GM_LANE_8) 
    	{
    		u1GraphLaneCnt = 8;
    		LOG(3, "[VB1][SetPadOn] GM_LANE_8\n");
    	}
    }

	if(IS_VB1_ALL_LANE_ON)
	{
		u1LaneCnt = 32;
		u1GraphLaneCnt = 0;
		LOG(3, "[VB1][SetPadOn] IS_VB1_ALL_LANE_ON\n");
	}

	LOG(3, "[VB1][SetPadOn] u1LaneCnt=%d, u1GraphLaneCnt=%d\n", u1LaneCnt, u1GraphLaneCnt);

    pu4LaneMap = au4LaneDefaultTbl;
    _u4VB1PadEn = 0;

	LOG(3, "LVDSA_EXT_EN[7:0]   V7   V6   V5   V4   V3   V2   V1   V0\n");
	LOG(3, "LADSA_pad:          AE1  AE0  AO4  AO3  AOCK AO2  AO1  AO0\n");
	LOG(3, "LVDSA_EXT_EN[15:8]  O7   O6   O5   O4   O3   O2   O1   O0\n");
	LOG(3, "LADSA_pad:          BOCK BO2  BO1  BO0  AE4  AE3  AECK AE2\n");
	LOG(3, "LVDSA_EXT_EN[23:16] V15  V14  V13  V12  V11  V10  V9   V8\n");
	LOG(3, "LADSA_pad:          BE4  BE3  BECK BE2  BE1  BE0  BO4  BO3\n");
    LOG(3, "LVDSA_EXT_EN[31:24] V23  V22  V21  V20  V19  V18  V17  V16\n");
	LOG(3, "LADSA_pad:          32   31   30   29   28   27   26   25\n");

    for (u1Lane = 0; u1Lane < 32; u1Lane++)
    {
        if (u1Lane < u1LaneCnt)
            _u4VB1PadEn += (1 << au4PadRemapTbl[pu4LaneMap[u1Lane]]);

        if (u1GraphLaneCnt > 0)
        {
            if (((u1LaneCnt <= 8) && (8 <= u1Lane) && (u1Lane < (8+u1GraphLaneCnt)))
                || ((u1LaneCnt == 16) && (16 <= u1Lane) && (u1Lane < (16+u1GraphLaneCnt))))
                _u4VB1PadEn += (1 << au4PadRemapTbl[pu4LaneMap[u1Lane]]);
        }

        LOG(3, "%d) lane%d bit_%d 0x%x\n", u1Lane, pu4LaneMap[u1Lane], au4PadRemapTbl[pu4LaneMap[u1Lane]], _u4VB1PadEn);
    }

	if(IS_VB1_ALL_LANE_ON)  // just for IC verification
	{
		_u4VB1PadEn = 0xffffffff;
	}
  
#ifdef CC_UBOOT
    if ((IO_READ32(0x10008000,0x06C4)&(1U << 13))==0)
#else
    if (!IS_Support8KDD())
#endif
    {
        _u4VB1PadEn &= 0xffffff;
        LOG(0,"8k_output(32 lanes) disable by EFUSE!!\n");
        LOG(0, "[<=24 lanes]0x%x\n", _u4VB1PadEn);
    }

	vIO32WriteFldAlign(REG_LVDS_TX_CFG13, _u4VB1PadEn, RG_LVDSA_EXT_EN);
	LOG(0, "[VB1][SetPadOn] LVDSA_EXT_EN=0x%lx\n", IO32ReadFldAlign(REG_LVDS_TX_CFG13, RG_LVDSA_EXT_EN));
}

/**
 * @brief vVByOneAnalogSetting
 * VB1 analog setting  
 * @param  void
 * @retval void
 */
void vVByOneAnalogSetting(void)
{
#ifndef CC_VB1_DISABLE
    UINT32 u4Value;
	UINT32 u4PreEmpEn;
	UINT8 u1PreEmpVal;
	UINT32 u4DrvPwd=0;

	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22, 0x0, RG_LVDSA_VSW_CALIB_SEL_D11);        // Capy remove  
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG21, 0x1c, RG_LVDSA_ABIST_VL_ODD_SEL_D11);  // Capy remove 
	
	u4Value = PANEL_GetDrivingCurrent();
    if (u4Value <= 15)
    {
        vDrvLVDSSetDriving((UINT8)u4Value);
    }
    else
    {
        vDrvLVDSSetDriving(7);  // [Capy] 7: 9mA
    }

	u4DrvPwd = 0;
	u4PreEmpEn = (UINT32)PANEL_GetPreEmphasisEn();
	u1PreEmpVal = PANEL_GetPreEmphasisVal();

	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x1, RG_LVDSA_TVCM);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0x4, RG_LVDSA_BIAS_SEL);  // Capy remove 
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x7, RG_LVDSA_NSRC);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x7, RG_LVDSA_PSRC);
	vVByOneSetPadOn(LVDS_OUTPUT_PORT, GM_LANE_NUM);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG18, 0x0, RG_LVDSA_LVDS_SEL);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG17, 0x0, RG_LVDSA_CLK_IQ_SEL);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22, 0x0, RG_LVDSA_ENEDGE_SEL_D11);  // Capy remove
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_RESYNC_CLK_IQ_SEL_D11);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_RESYNC_LVDS_SEL_D11);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG12, P_Fld(0x1, RG_LVDSA_DIV5_EN)|P_Fld(0x2, RG_LVDSA_CKPATH_EN));
    vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_SLDO_LVROD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG25, 0x0, RG_LVDSA_SLDO_PWD);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_CKLDO_LVROD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0x1, RG_LVDSA_DRV_OPMODE_EN);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x0, RG_LVDSA_CHBIAS_PWD);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(0, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(0, RG_LVDSA_CKCHLDO_PWD));

    if (!u4PreEmpEn)  // disable pre-emphasis
    {
    #if 1
        u4PreEmpEn = _u4VB1PadEn;
    #else  // just for IC verification
        u4PreEmpEn = 0xFFFFFFFF;
    #endif
    }
    else  // enable pre-emphasis
    {
    #if 1
        u4PreEmpEn = 0xFFFFFFFF - _u4VB1PadEn;
    #else  // just for IC verification
        u4PreEmpEn = 0;
    #endif
    }
    LOG(0, "[PreEmp]u4PreEmpEn=0x%8lx\n", u4PreEmpEn);

#if 1
    u4DrvPwd = 0xFFFFFFFF - _u4VB1PadEn;
#else  // just for IC verification
    u4DrvPwd = 0;
#endif
    LOG(0, "[DrvPwd]u4DrvPwd=0x%8lx\n", u4DrvPwd);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG15, u4DrvPwd, RG_LVDSA_DRV_PWD);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG16, u4PreEmpEn, RG_LVDSA_PRE_PWD);
	vVByOnePreEmphasisSet(u1PreEmpVal);
    
	vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0x7, RG_LVDSA_NSRC_PRE);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0x7, RG_LVDSA_PSRC_PRE);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG19, P_Fld(1, RG_LVDSA_HSN_MODE_EN)|P_Fld(1, RG_LVDSA_HSP_MODE_EN)|P_Fld(0, RG_LVDSA_PREDRV_LVDS_EN));
    vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0, RG_LVDSA_PREDRV_VOD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG39, 0, RG_LVDSA_PREDRV_PWD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG26, 0, RG_LVDSA_PREDRV_LDO_PWD);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0xE7FFFF, RG_LVDSA_ABIST_SKEW_EN_D11);  // Capy remove
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x0, RG_LVDSA_DIV5_EN);
	HAL_Delay_us(LVDSA_EN_REST_DELAYTIME);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x1, RG_LVDSA_DIV5_EN);	
#endif
}

void vVByOneOSDAnalogSetting(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x0, RG_LVDSA_DIV5_EN);
	HAL_Delay_us(LVDSA_EN_REST_DELAYTIME);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x1, RG_LVDSA_DIV5_EN);	
#endif
}

/**
 * @brief vVByOneAnalogOffSetting
 * VB1 analog setting  
 * @param  void
 * @retval void
 */
void vVByOneAnalogOffSetting(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0, RG_LVDSA_DIV5_EN);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0, RG_LVDSA_EXT_EN);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG19, P_Fld(0, RG_LVDSA_HSN_MODE_EN)|P_Fld(0, RG_LVDSA_HSP_MODE_EN));
	vVByOnePreEmphasisOnOff(SV_OFF);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0, RG_LVDSA_DRV_OPMODE_EN);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG15, 0xFFFFFFFF, RG_LVDSA_DRV_PWD);
    //vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(1, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(1, RG_LVDSA_CKCHLDO_PWD));
    //vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 1, RG_LVDSA_CHBIAS_PWD);
#endif
}

void vVByOne_Impedance_Cal(void)
{
#ifndef CC_VB1_DISABLE
	UINT8 i = 0;
	UINT32 u4tmp = 0, u4Driving_level_Backup = 0;

	u4Driving_level_Backup = vDrvLVDSGetDriving();

	vVByOneWriteFldAlign(VB1SYS6, 0, VB1SYS6_START);
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_START);	
	#endif
	vIO32WriteFldAlign(TMGR16,0xfffff,TMGR16_RG_TMDS_CTRL_REG_EN);
	vIO32WriteFldAlign(TMGR17,0x3ff,TMGR17_RG_TMDS_REG_VALUE);
	vIO32WriteFldAlign(TMGR5,0x7,TMGR5_RG_FIFO_CTRL);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22,0x1,RG_LVDSA_VSW_CALIB_SEL_D11);         // Capy remove 
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22,0x14,RG_LVDSA_ABIST_VL_EVEN_SEL_D11);  // Capy remove
	vDrvLVDSSetDriving(5); // default 6mA
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12,0x1,RG_LVDSA_TVCM);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG14,0x4,RG_LVDSA_BIAS_SEL);  // Capy remove 
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12,0x7,RG_LVDSA_NSRC);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12,0x7,RG_LVDSA_PSRC);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG13,0xFFFFFFFF,RG_LVDSA_EXT_EN);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG18,0,RG_LVDSA_LVDS_SEL);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG17,0,RG_LVDSA_CLK_IQ_SEL);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22,0,RG_LVDSA_ENEDGE_SEL_D11);  // Capy remove
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23,0,RG_LVDSA_RESYNC_CLK_IQ_SEL_D11);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23,0,RG_LVDSA_RESYNC_LVDS_SEL_D11);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG12,P_Fld(0x1, RG_LVDSA_DIV5_EN)|P_Fld(0x2, RG_LVDSA_CKPATH_EN));
    vIO32WriteFldAlign(REG_LVDS_TX_CFG23,0,RG_LVDSA_SLDO_LVROD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG25,0,RG_LVDSA_SLDO_PWD);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG23,0,RG_LVDSA_CKLDO_LVROD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG14,1,RG_LVDSA_DRV_OPMODE_EN);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG12,0,RG_LVDSA_CHBIAS_PWD);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(0, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(0, RG_LVDSA_CKCHLDO_PWD));
	vIO32WriteFldAlign(REG_LVDS_TX_CFG15,0x0,RG_LVDSA_DRV_PWD);
	vVByOnePreEmphasisOnOff(SV_OFF);
	//vVByOnePreEmphasisSet(1);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG14,0x7,RG_LVDSA_NSRC_PRE);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG14,0x7,RG_LVDSA_PSRC_PRE);
	vIO32WriteFldMulti(REG_LVDS_TX_CFG19, P_Fld(1, RG_LVDSA_HSN_MODE_EN)|P_Fld(1, RG_LVDSA_HSP_MODE_EN)|P_Fld(0, RG_LVDSA_PREDRV_LVDS_EN));
    vIO32WriteFldAlign(REG_LVDS_TX_CFG23,0,RG_LVDSA_PREDRV_VOD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG39,0,RG_LVDSA_PREDRV_PWD);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG26,0,RG_LVDSA_PREDRV_LDO_PWD);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12,0x1,RG_LVDSA_DIV5_EN);
	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22,0x1,RG_LVDSA_IMP_COMP_EN_D11);  // Capy remove
	vIO32WriteFldAlign(REG_LVDS_TX_CFG24,0x1,RG_CALI_COMPARATOR_RANGE);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG24,0x1,RG_CHOP_CLK_EN);

	vUtDelay2us(100);
	vIO32WriteFldMulti(TMGR7, P_Fld(0, TMGR7_RG_LOOPPAT_2TO1)|P_Fld(0, TMGR7_RG_LOOPPAT_EN)|P_Fld(1, TMGR7_IMP_CAL_SET)
							|P_Fld(0, TMGR7_RG_RXIMP)|P_Fld(0, TMGR7_RG_AUTOK_RX_IMP)|P_Fld(0, TMGR7_RG_TXIMP)
                            |P_Fld(0, TMGR7_REG_IMP_DP_DIS)|P_Fld(0, TMGR7_RG_IMPFREQSEL)|P_Fld(1, TMGR7_RG_AUTOK_TX_IMP)
                            |P_Fld(1, TMGR7_REG_SEL_1_ALL)|P_Fld(0, TMGR7_SAPIS_MODE_SYNC)|P_Fld(0, TMGR7_CTL_EN_CALIBRATION));
	
	vUtDelay2us(100);
	vIO32WriteFldMulti(TMGR7, P_Fld(0, TMGR7_RG_LOOPPAT_2TO1)|P_Fld(0, TMGR7_RG_LOOPPAT_EN)|P_Fld(1, TMGR7_IMP_CAL_SET)
							|P_Fld(0, TMGR7_RG_RXIMP)|P_Fld(0, TMGR7_RG_AUTOK_RX_IMP)|P_Fld(0, TMGR7_RG_TXIMP)
                            |P_Fld(0, TMGR7_REG_IMP_DP_DIS)|P_Fld(0, TMGR7_RG_IMPFREQSEL)|P_Fld(1, TMGR7_RG_AUTOK_TX_IMP)
                            |P_Fld(1, TMGR7_REG_SEL_1_ALL)|P_Fld(1, TMGR7_SAPIS_MODE_SYNC)|P_Fld(1, TMGR7_CTL_EN_CALIBRATION));

	do
	{
		u4tmp = IO32ReadFldAlign(TMGR24,TMGR24_HOLD);
		vUtDelay2us(10);
		i++;
		if(i>200)
		{
			LOG(0, "VB1 impedance cal error then break......\n ");
			break;
		}
		
	}while(!u4tmp);
	
	u4tmp = IO32ReadFldAlign(TMGR24,TMGR24_DA_IMPCAL_TX_LAT);

	if(u4tmp>0 && u4tmp<0xf)
	{
		LOG(0, "VB1 VB1 impedance cal  pass and value = 0x%lx .................\n",u4tmp);
	}
	
	if (_fgVB1UseFixTERM)
	{
		//LOG(0,"VB1 VB1 impedance cal  fail .........................\n");
		LOG(0, "VB1 VB1 impedance cal  set to default terminaion %d.........................\n", _u1VB1FixTERMVal);
		vIO32WriteFldAlign(TMGR7, 0, TMGR7_IMP_CAL_SET);
        #if 0
		vIO32WriteFldMulti(TMGR8, P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_ECK)|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O0)|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O1)
									|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O2)|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O3)|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O4)
									|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_O5)|P_Fld(_u1VB1FixTERMVal, TMGR8_RG_LVDSA_TERM_OCK));
		vIO32WriteFldMulti(TMGR9, P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E0)|P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E1)|P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E2)
									|P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E3)|P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E4)|P_Fld(_u1VB1FixTERMVal, TMGR9_RG_LVDSA_TERM_E5));
        if (IS_IC_5893())
        {
		vIO32WriteFldMulti(TMGR19, P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_ECK)|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O0)|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O1)
									|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O2)|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O3)|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O4)
									|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_O5)|P_Fld(_u1VB1FixTERMVal, TMGR19_RG_LVDSB_TERM_OCK));
		vIO32WriteFldMulti(TMGR20, P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E0)|P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E1)|P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E2)
									|P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E3)|P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E4)|P_Fld(_u1VB1FixTERMVal, TMGR20_RG_LVDSB_TERM_E5));
	    }
        #endif
        vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 1, RG_LVDSA_TERM_EN);
        vDrvLVDSSetTermination(_u1VB1FixTERMVal);  
	}

	//vIO32WriteFldAlign(REG_LVDS_TX_CFG22, 0x0, RG_LVDSA_IMP_COMP_EN_D11);  // Capy remove

    vDrvLVDSSetDriving(u4Driving_level_Backup);
#endif
}

void vVByOneSetOstgCHFlag(void)
{
    if (PANEL_Is4K2K60())
    {
        vDrvSetOstgCHSelFlag(OSTG_CH0_ONLY);
    }
    else if (PANEL_Is4K2K120()||PANEL_Is8K4K30())
    {
        vDrvSetOstgCHSelFlag(OSTG_CH0_AND_CH1);
    }
    else
    {
        vDrvSetOstgCHSelFlag(OSTG_CH0_ONLY);
    }
}

#if defined(CONFIG_OPM) || defined(CC_SUPPORT_STR)
    BOOL fgVB1Init = FALSE;
#else
    static BOOL fgVB1Init = FALSE;
#endif
void vVByOneInit(void)
{
#ifndef CC_VB1_DISABLE
	//UINT8 i = 0;            // Capy remove
	//UINT32 u4tmp = 0;   // Capy remove
    UINT64 *pu4PinMap,*pu4PNSwap;

    if (IO32ReadFldAlign(DDDS_08, FLAG_VB1_INITIAL) == 1)
    {
        fgVB1Init = TRUE;
    }

	if (!fgVB1Init)
	{
	#if 0
	#ifdef __MODEL_slt__
		vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0x1, FLD_OCLK_INV);
		vIO32WriteFldAlign(CKGEN_PSWCLK_CFG, 0x1, FLD_LVDS_CLK_CTS_TOG);
		LOG(0,"[VB1]oclk_inv = 1 !!!\n");
	#endif
	#endif

    #if 0  // Capy remove
	do
	{
		u4tmp = u4IO32Read4B(RGS_LVDSTX_PLL_CFG0);
		vUtDelay2us(10);
		i++;
		if(i>200)
		{
			LOG(0, "VB1 Vpll clock check error......\n ");
			break;
		}
		
	}while((u4tmp &0xC0000000)!=0x80000000); //bit 31 30
    #endif

    // switch to LVDS TX mode               
	vIO32WriteFldAlign(TMGR2, 2, TMGR2_RG_FIFO_CK_SEL); 
	
	vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_BIT_INV);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_7_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_6_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_5_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_4_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_3_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_2_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_1_EN);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_LANE_0_EN);
	vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_FIELDBET);
	vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_PATGEN_START);
	vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_DE_RST_CTRL);
	vVByOneWriteFldAlign(VB1SYS0, VB1SYS0_LANE_NUM_4, VB1SYS0_LANE_NUM);
	vVByOneWriteFldAlign(VB1SYS0, VB1SYS0_COL_4BYTES, VB1SYS0_COL);
	
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1OSDSYS0, P_Fld(0, VB1OSDSYS0_BIT_INV)|
								P_Fld(0, VB1OSDSYS0_VSYN_INV)|
								P_Fld(0, VB1OSDSYS0_HSYN_INV)|
								P_Fld(1, VB1OSDSYS0_LANE_3_EN)|
								P_Fld(1, VB1OSDSYS0_LANE_2_EN)|
								P_Fld(1, VB1OSDSYS0_LANE_1_EN)|
								P_Fld(1, VB1OSDSYS0_LANE_0_EN)|
								P_Fld(0, VB1OSDSYS0_FIELDBET)|
								P_Fld(0, VB1OSDSYS0_PATGEN_START)|
								P_Fld(1, VB1OSDSYS0_DE_RST_CTRL)|
								P_Fld(VB1OSDSYS0_LANE_NUM_4, VB1OSDSYS0_LANE_NUM)|
								P_Fld(VB1OSDSYS0_COL_4BYTES, VB1OSDSYS0_COL));
	#endif

	#ifdef CC_SCPOS_3DTV_SUPPORT
	if (IS_LVDS_DISP_3D_SHUTTER)
	{
		vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_VB1_3DLR_EN); 
	}
	else
	{
		vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_VB1_3DLR_EN);
	} 
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_VB1_3DLR_EN);	
	#endif
	#endif

	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_CRC_EN);
	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_LANE_PRBS_EN);
	vVByOneWriteFldAlign(VB1SYS3, 1, VB1SYS3_LANE3_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 1, VB1SYS3_LANE2_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 1, VB1SYS3_LANE1_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 1, VB1SYS3_LANE0_ANA);

	vVByOneWriteFldAlign(VB1SYS1, 7, VB1SYS1_LANE7_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 6, VB1SYS1_LANE6_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 5, VB1SYS1_LANE5_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 4, VB1SYS1_LANE4_MUX);
	
	vVByOneWriteFldAlign(VB1SYS1, 3, VB1SYS1_LANE3_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 2, VB1SYS1_LANE2_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 1, VB1SYS1_LANE1_MUX);
	vVByOneWriteFldAlign(VB1SYS1, 0, VB1SYS1_LANE0_MUX);
	
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1OSDSYS1, P_Fld(3, VB1OSDSYS1_LANE3_MUX)|
								P_Fld(2, VB1OSDSYS1_LANE2_MUX)|
								P_Fld(1, VB1OSDSYS1_LANE1_MUX)|
								P_Fld(0, VB1OSDSYS1_LANE0_MUX));			
	#endif

	// ==Lane Swap ==
    if(NULL == (pu4PinMap = (UINT64 *)DRVCUST_PanelGetUptr(ePanelVB1LaneSwap)))  // A + B + C + D defined in panel table
	{
        LOG(0, "NO VB1LaneSwap in panel table\n");
        pu4PinMap = (UINT64 *)_au4PanelVB1LaneNOSwap;   // A + C(OSD) + B + D
	}
    else  
    {
    #ifndef CC_S_PLATFORM
        if (LVDS_OUTPUT_PORT <= OCTA_PORT)
        {
            pu4PinMap = (UINT64 *)_au4PanelVB1LaneNOSwap;   // A + C(OSD) + B + D
        }
    #endif    
    }
    
    // 3rd 8lane(OSD)
    #ifdef SUPPORT_PANEL_GRAPH_MODE
    vIO32WriteFldMulti(TMGR27,  P_Fld(pu4PinMap[15], TMGR27_B_PORT_OCK_SEL)|
                                P_Fld(pu4PinMap[14], TMGR27_B_PORT_O2_SEL)|
                                P_Fld(pu4PinMap[13], TMGR27_B_PORT_O1_SEL)|
                                P_Fld(pu4PinMap[12], TMGR27_B_PORT_O0_SEL));
    vIO32WriteFldMulti(TMGR26,  P_Fld(pu4PinMap[11], TMGR26_A_PORT_E4_SEL)|
                                P_Fld(pu4PinMap[10], TMGR26_A_PORT_E3_SEL)|
                                P_Fld(pu4PinMap[9], TMGR26_A_PORT_ECK_SEL)|
                                P_Fld(pu4PinMap[8], TMGR26_A_PORT_E2_SEL));
    #endif
    // 1st 8lane
    vIO32WriteFldMulti(TMGR26,  P_Fld(pu4PinMap[7], TMGR26_A_PORT_E1_SEL)|
                                P_Fld(pu4PinMap[6], TMGR26_A_PORT_E0_SEL));
    vIO32WriteFldMulti(TMGR25,  P_Fld(pu4PinMap[5], TMGR25_A_PORT_O4_SEL)|
                                P_Fld(pu4PinMap[4], TMGR25_A_PORT_O3_SEL)|
                                P_Fld(pu4PinMap[3], TMGR25_A_PORT_OCK_SEL)|
                                P_Fld(pu4PinMap[2], TMGR25_A_PORT_O2_SEL)|
                                P_Fld(pu4PinMap[1], TMGR25_A_PORT_O1_SEL)|
                                P_Fld(pu4PinMap[0], TMGR25_A_PORT_O0_SEL));
    // ***2nd 8lane***
    vIO32WriteFldMulti(TMGR28,  P_Fld(pu4PinMap[23], TMGR28_B_PORT_E4_SEL)|
                                P_Fld(pu4PinMap[22], TMGR28_B_PORT_E3_SEL)|
                                P_Fld(pu4PinMap[21], TMGR28_B_PORT_ECK_SEL)|
                                P_Fld(pu4PinMap[20], TMGR28_B_PORT_E2_SEL)|
                                P_Fld(pu4PinMap[19], TMGR28_B_PORT_E1_SEL)|
                                P_Fld(pu4PinMap[18], TMGR28_B_PORT_E0_SEL));
    vIO32WriteFldMulti(TMGR27,  P_Fld(pu4PinMap[17], TMGR27_B_PORT_O4_SEL)|
                                P_Fld(pu4PinMap[16], TMGR27_B_PORT_O3_SEL));
    // ***4th 8lane***
    vIO32WriteFldMulti(TMGR30,  P_Fld(pu4PinMap[31], TMGR30_VB1_PORT7_SEL)|
                                P_Fld(pu4PinMap[30], TMGR30_VB1_PORT6_SEL));
    vIO32WriteFldMulti(TMGR29,  P_Fld(pu4PinMap[29], TMGR29_VB1_PORT5_SEL)|
                                P_Fld(pu4PinMap[28], TMGR29_VB1_PORT4_SEL)|
                                P_Fld(pu4PinMap[27], TMGR29_VB1_PORT3_SEL)|
                                P_Fld(pu4PinMap[26], TMGR29_VB1_PORT2_SEL)|
                                P_Fld(pu4PinMap[25], TMGR29_VB1_PORT1_SEL)|
                                P_Fld(pu4PinMap[24], TMGR29_VB1_PORT0_SEL));

    vVByOneSetOstgCHFlag();
    
    if (IS_SUPPORT_VB1_GRAPH_MODE)
    {
        #ifndef CC_S_PLATFORM
        if (IS_OSD_SEPERATE_FOR_EXT_BE)
        #endif
        {
            vIO32WriteFldAlign(TMGR6, USE_AS_OSD, TMGR6_VB1_CHC_OSD_SEL);  // VB1 CH-C is used for CH-OSD
        }
    }

	// == PN Swap ==
	if(NULL == (pu4PNSwap = (UINT64*)DRVCUST_PanelGetUptr(ePanelVB1LanePNSwap)))
	{
	
	}
	else
	{
		#ifdef SUPPORT_PANEL_GRAPH_MODE
		vIO32WriteFldMulti(VB1OSDSYS3,	P_Fld(pu4PNSwap[39], VB1OSDSYS3_LANE7_INV)|
		                                P_Fld(pu4PNSwap[38], VB1OSDSYS3_LANE6_INV)|
		                                P_Fld(pu4PNSwap[37], VB1OSDSYS3_LANE5_INV)|
		                                P_Fld(pu4PNSwap[36], VB1OSDSYS3_LANE4_INV)|
		                                P_Fld(pu4PNSwap[35], VB1OSDSYS3_LANE3_INV)|
										P_Fld(pu4PNSwap[34], VB1OSDSYS3_LANE2_INV)|
										P_Fld(pu4PNSwap[33], VB1OSDSYS3_LANE1_INV)|
										P_Fld(pu4PNSwap[32], VB1OSDSYS3_LANE0_INV));
		#endif		

		// 1st 8lane
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[7], VB1SYS3_LANE7_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[6], VB1SYS3_LANE6_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[5], VB1SYS3_LANE5_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[4], VB1SYS3_LANE4_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[3], VB1SYS3_LANE3_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[2], VB1SYS3_LANE2_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[1], VB1SYS3_LANE1_INV);
		vIO32WriteFldAlign(VB1SYS3, pu4PNSwap[0], VB1SYS3_LANE0_INV);	

		// ***2nd 8lane***
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[15], VB1SYS3_LANE7_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[14], VB1SYS3_LANE6_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[13], VB1SYS3_LANE5_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[12], VB1SYS3_LANE4_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[11], VB1SYS3_LANE3_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[10], VB1SYS3_LANE2_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[9], VB1SYS3_LANE1_INV);
		vIO32WriteFldAlign(VB1SYS3_2ND, pu4PNSwap[8], VB1SYS3_LANE0_INV);

        // ***3rd 8lane***
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[23], VB1SYS3_LANE7_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[22], VB1SYS3_LANE6_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[21], VB1SYS3_LANE5_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[20], VB1SYS3_LANE4_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[19], VB1SYS3_LANE3_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[18], VB1SYS3_LANE2_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[17], VB1SYS3_LANE1_INV);
		vIO32WriteFldAlign(VB1SYS3_3RD, pu4PNSwap[16], VB1SYS3_LANE0_INV);

        // ***4th 8lane***
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[31], VB1SYS3_LANE7_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[30], VB1SYS3_LANE6_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[29], VB1SYS3_LANE5_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[28], VB1SYS3_LANE4_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[27], VB1SYS3_LANE3_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[26], VB1SYS3_LANE2_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[25], VB1SYS3_LANE1_INV);
		vIO32WriteFldAlign(VB1SYS3_4TH, pu4PNSwap[24], VB1SYS3_LANE0_INV);
	}

	vVByOneWriteFldAlign(VB1SYS1, 0, VB1SYS1_MON_SEL);	  	

	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldAlign(VB1OSDSYS1,0, VB1OSDSYS1_MON_SEL);	  	  	
	#endif

	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_ANA_RX_EN);
	vVByOneWriteFldAlign(VB1SYS3, 0x3FF, VB1SYS3_LANE3_ANA_VALUE);	// only use lane 3 caibration result      

	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1OSDSYS3, P_Fld(0, VB1OSDSYS3_ANA_RX_EN)|
								P_Fld(0x3FF, VB1OSDSYS3_LANE3_ANA_VALUE));         
	#endif

    vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_CTRL);
    vIO32WriteFldAlign(TMGR5, 1, TMGR5_RG_FIFO_EN);

	vVByOne_Impedance_Cal();

	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_LANE3_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_LANE2_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_LANE1_ANA);
	vVByOneWriteFldAlign(VB1SYS3, 0, VB1SYS3_LANE0_ANA);
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1OSDSYS3,	P_Fld(0, VB1OSDSYS3_LANE3_ANA)|
									P_Fld(0, VB1OSDSYS3_LANE2_ANA)|
									P_Fld(0, VB1OSDSYS3_LANE1_ANA)|
									P_Fld(0, VB1OSDSYS3_LANE0_ANA));
	#endif

    //vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(1, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(1, RG_LVDSA_CKCHLDO_PWD));
    //vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x1, RG_LVDSA_CHBIAS_PWD);

	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1OSDSYS6, P_Fld(1, VB1OSDSYS6_DE_EN)|P_Fld(1, VB1OSDSYS6_EN));
    if(IS_DISP_4K2K_TYPE3)
    {
        vIO32WriteFldAlign(SCPIP_SCSYS_67, 0x0, SCSYS_67_GFX_OSD_HSYNC_DLY); //sony_BE total issue
    }

	//vIO32WriteFldAlign(SCPIP_SCSYS_14, 0x2, SCSYS_14_SC_OSD_SRC_SEL);
	vIO32WriteFldMulti(SCPIP_SCSYS_66, P_Fld(0, SCSYS_66_GFX_OSD_HSYNC_SEL)|P_Fld(0, SCSYS_66_GFX_OSD_VSYNC_SEL));	
	#endif

	vIO32WriteFldAlign(DDDS_08, 1, FLAG_VB1_INITIAL);
	}
#endif
}

void vVByOneFormatConfig(void)
{
#ifndef CC_VB1_DISABLE
    UINT32 u4MSBSW;
    
    u4MSBSW = LVDS_DISP_FORMAT;

	if (u4MSBSW == LVDS_MSB_SW_ON) // NS, VESA
	{
		vVByOneWriteFldAlign(VB1SYS1, 1, VB1SYS1_TTL_REORDER);	
	}
	else if (u4MSBSW == LVDS_MSB_SW_OFF) // JEIDA
	{
		vVByOneWriteFldAlign(VB1SYS1, 0, VB1SYS1_TTL_REORDER);	  		
	}
#endif
}

void vDrvVB14K2KPanelInterfaceConfig(void)
{
	if (IS_DISABLE_LVDS_LINEBUFFER)
	{
		vIO32WriteFldAlign(VINTF_B0, 1,  VINTF_LINE_BUFFER_BYPASS); 
	}	
	else
	{
		vIO32WriteFldAlign(VINTF_B0, 0,  VINTF_LINE_BUFFER_BYPASS); 
	}

    vIO32WriteFldAlign(TMGR31, SRC_FROM_OSTG, TMGR31_MSTAR_SRC_SEL);

#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord_2SecEn())
#else
	if (IS_VB1_2SEC_ENABLE)
#endif
	{
		vDrvVByOne2SecOnOff(SV_ON);
	}
	else
	{
		vDrvVByOne2SecOnOff(SV_OFF);
	}

#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord_4SecEn())
#else
	if (IS_VB1_4SEC_ENABLE)
#endif
	{
		vDrvVByOne4SecOnOff(SV_ON);
	}
	else
	{
		vDrvVByOne4SecOnOff(SV_OFF);
	}

#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord_8SecEn())
#else
	if (IS_VB1_8SEC_ENABLE)
#endif
	{
		vDrvVByOne8SecOnOff(SV_ON);
	}
	else
	{
		vDrvVByOne8SecOnOff(SV_OFF);
	}
}

void vDrvVB1Config(void)
{   
#ifndef CC_VB1_DISABLE
    // set VB1 format
    vVByOneFormatConfig();

	// line buffer bypass & 2-section
	vDrvVB14K2KPanelInterfaceConfig();

	#ifdef __MODEL_slt__
	vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_VSYN_INV);
	vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_HSYN_INV);
	#else
    if (HSYNC_POLARITY == HSYNC_LOW)
    {  
        vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_HSYN_INV);                
    }
    else
    {        
        vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_HSYN_INV);                
    }

    if (VSYNC_POLARITY == VSYNC_LOW)
    {       
        vVByOneWriteFldAlign(VB1SYS0, 0, VB1SYS0_VSYN_INV);                
    }
    else
    {
        vVByOneWriteFldAlign(VB1SYS0, 1, VB1SYS0_VSYN_INV);                     
    }
	#endif
#endif   
}

void vVByOneOSDFormatConfig(void)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
    UINT32 u4MSBSW;
	#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord2_GraphModeEn())
	#else
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	#endif
		u4MSBSW = GraphMode_FORMAT;
	else
		u4MSBSW = GM_FORMAT_CASE0_SMODE1;

	if (u4MSBSW == GM_FORMAT_CASE0_SMODE1) // Case 0 -> Smode1
	{
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_SEL);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_A_SW);			
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_MS);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_BTI_ORDER_SW);
	}
	else if (u4MSBSW == GM_FORMAT_CASE1_SMODE0) // Case 1 -> Smode0
	{
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_SEL);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_A_SW);			
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_MS);
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_BTI_ORDER_SW);
	}
	else if (u4MSBSW == GM_FORMAT_CASE2_MMODE0) // Case 2 -> Mmode0
	{
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_SEL);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_A_SW);		
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_MS);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_BTI_ORDER_SW);
	}	
	else if (u4MSBSW == GM_FORMAT_CASE3_INHOUSE) // Case 3 -> In-House Mode
	{
	#if 0
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_SEL);  // Wukong-pro will fix
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_A_SW);	
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_MS);
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_BTI_ORDER_SW);
	#else
		// R/G/B mapping is the same for Smode0 and INHOUSE; Alpha need to swap.
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_SEL);
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_A_SW);			
		vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_MS);
		vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_BTI_ORDER_SW);
		vVByOneOsdARGBMap_SM0ToInhouse();
	#endif
	}	

	#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord2_GraphModeEn())
	#else
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	#endif
	{
		#ifdef __MODEL_slt__
		vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_VSYN_INV);
		vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_HSYN_INV);
		#else	
		if (GRAPH_HSYNC_POLARITY == HSYNC_LOW)
	    {  
	        vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_HSYN_INV);                
	    }
	    else
	    {        
	        vIO32WriteFldAlign(VB1OSDSYS0, 1, VB1OSDSYS0_HSYN_INV);                
	    }

	    if (GRAPH_VSYNC_POLARITY == VSYNC_LOW)
	    {       
	        vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_VSYN_INV);                
	    }
	    else
	    {
	        vIO32WriteFldAlign(VB1OSDSYS0, 1, VB1OSDSYS0_VSYN_INV);                     
	    }
		#endif
	}
    else
    {
        // SYNC: low
        vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_HSYN_INV);
        vIO32WriteFldAlign(VB1OSDSYS0, 0, VB1OSDSYS0_VSYN_INV);
    }
#endif	
#endif
}


/**
 * @brief vVByOneDigitalReset
 * disable VB1 digital module
 * @param  void
 * @retval void
 */
void vVByOneDigitalReset(void)
{
#ifndef CC_VB1_DISABLE
    vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_CTRL);
    vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_EN);
    vVByOneWriteFldAlign(VB1SYS6, 0, VB1SYS6_START);    
	#ifdef SUPPORT_PANEL_GRAPH_MODE
    vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_START);    
	#endif
#endif
}

void vVByOneOSDDigitalReset(void)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
    vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_START);   
#endif
#endif
}

void vDrvVByOneOSDDigitalOn(void)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldAlign(VB1OSDSYS6, 0x1, VB1OSDSYS6_START);
#endif
#endif
}

/**
 * @brief vDrvVB1On
 * Power On VBYONE
 * @param  void
 * @retval void
 */
void vDrvVByOneOn(void)
{
#ifndef CC_VB1_DISABLE
    UINT32 u4DispBit;
    UINT32 u4VB1Lane;
	UPTR u4LocknPin, u4LocknPinFunc;
	UPTR u4HtpdnPin, u4HtpdnPinFunc;
    
    u4DispBit = DISP_BIT;
    u4VB1Lane = LVDS_OUTPUT_PORT;

    if ((DRVCUST_PanelQueryUptr(ePanelVB1LocknPin,&u4LocknPin)==0)&&(DRVCUST_PanelQueryUptr(ePanelVB1LocknPinFunc,&u4LocknPinFunc)==0))
	{
		if ((0 == u4LocknPin) || (0 == u4LocknPinFunc))
		{
			LOG(0, "[VB1][LOCKN] PANEL_VB1_LOCKN_PIN & PANEL_VB1_LOCKN_PIN_FUNC = NULL\n");
		}
		else
		{
			LOG(0, "[VB1][LOCKN] 0x%lx set to FUNCTION%ld\n", u4LocknPin, u4LocknPinFunc);
	        UNUSED(BSP_PinSet(u4LocknPin, u4LocknPinFunc));
		}
	}
	else
	{
		LOG(0, "[VB1][LOCKN] Did not define PANEL_VB1_LOCKN_PIN & PANEL_VB1_LOCKN_PIN_FUNC!!!\n");
	}
	if ((DRVCUST_PanelQueryUptr(ePanelVB1HtpdnPin,&u4HtpdnPin)==0)&&(DRVCUST_PanelQueryUptr(ePanelVB1HtpdnPinFunc,&u4HtpdnPinFunc)==0))
	{
		if ((0 == u4HtpdnPin) || (0 == u4HtpdnPinFunc))
		{
			LOG(0, "[VB1][HTPDN] PANEL_VB1_HTPDN_PIN & PANEL_VB1_HTPDN_PIN_FUNC = NULL\n");
		}
		else
		{
			LOG(0, "[VB1][HTPDN] 0x%lx set to FUNCTION%ld\n", u4HtpdnPin, u4HtpdnPinFunc);
	        UNUSED(BSP_PinSet(u4HtpdnPin, u4HtpdnPinFunc));
		}
	}
	else
	{
		LOG(0, "[VB1][HTPDN] Did not define PANEL_VB1_HTPDN_PIN & PANEL_VB1_HTPDN_PIN_FUNC!!!\n");
	}
    
	switch (u4DispBit)
	{
	case DISP_36BIT:	   
		vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_5BYTES,VB1SYS0_COL);
	   break;
   	 case DISP_30BIT:   	 	
		vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_4BYTES,VB1SYS0_COL);
    	break;		
	 case DISP_24BIT:
     case DISP_18BIT:
   	 	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_3BYTES,VB1SYS0_COL);
    	break;
	default:
    	break;
	}

	vDrvVB1Config();
	if((u4VB1Lane == OCTA_PORT) || (u4VB1Lane == HEXA_PORT))
	{
		if (PANEL_IsSupport4K2KDD() || (IS_DISP_4K2K_TYPE3 && (GetPANEL4K2K_TYPE() == PANEL_TYPE_2K1K_120)))
			vIO32WriteFldAlign(CKGEN_OCLK_TEST, 3,FLD_VB1_CK_SEL); // Gazelle ES2 : /8 Gazelle ES1 :PD
		else
			vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2,FLD_VB1_CK_SEL);
	   
	   vVByOneAnalogSetting();
	   vVByOneDigitalReset();			   
	   vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_4,VB1SYS0_LANE_NUM);	   
	   vVByOneWriteFldAlign(VB1SYS0,0xFF,VB1SYS0_LANE_EN_MUX);
	}
	else if(u4VB1Lane == FOUR_PORT)
	{
		vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2,FLD_VB1_CK_SEL);
		vVByOneAnalogSetting();
		vVByOneDigitalReset();			   
		vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_4,VB1SYS0_LANE_NUM);	   
		if(IS_VB1_ALL_LANE_ON)
			vVByOneWriteFldAlign(VB1SYS0,0xFF,VB1SYS0_LANE_EN_MUX);
		else
			vVByOneWriteFldAlign(VB1SYS0,0xF,VB1SYS0_LANE_EN_MUX);
	}
	else if (u4VB1Lane == DUAL_PORT)
	{
		vIO32WriteFldAlign(CKGEN_OCLK_TEST, 1,FLD_VB1_CK_SEL);
		vVByOneAnalogSetting();
		vVByOneDigitalReset();
		vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_2,VB1SYS0_LANE_NUM);
		if(IS_VB1_ALL_LANE_ON)
			vVByOneWriteFldAlign(VB1SYS0,0xFF,VB1SYS0_LANE_EN_MUX);
		else
		{
		#ifdef PANEL_FHD_BACKEND
			if (GetCurrentPanelIndex()==PANEL_FHD_BACKEND)
				vVByOneWriteFldAlign(VB1SYS0,0xF,VB1SYS0_LANE_EN_MUX);
			else
		#endif
				vVByOneWriteFldAlign(VB1SYS0,0x3,VB1SYS0_LANE_EN_MUX);			
		}
			   
	}
	else if (u4VB1Lane == SINGLE_PORT)
	{
		vIO32WriteFldAlign(CKGEN_OCLK_TEST, 0,FLD_VB1_CK_SEL);
		vVByOneAnalogSetting();
		vVByOneDigitalReset();
		vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_1,VB1SYS0_LANE_NUM);
		if(IS_VB1_ALL_LANE_ON)
			vVByOneWriteFldAlign(VB1SYS0,0xFF,VB1SYS0_LANE_EN_MUX);
		else
			vVByOneWriteFldAlign(VB1SYS0,0x3,VB1SYS0_LANE_EN_MUX);
	}
		   
	vVByOneWriteFldAlign(VB1SYS0,0x1,VB1SYS0_DE_RST_CTRL);
	vVByOneWriteFldAlign(VB1SYS1,3,VB1SYS1_LANE3_MUX);
	vVByOneWriteFldAlign(VB1SYS1,2,VB1SYS1_LANE2_MUX);
	vVByOneWriteFldAlign(VB1SYS1,1,VB1SYS1_LANE1_MUX);
	vVByOneWriteFldAlign(VB1SYS1,0,VB1SYS1_LANE0_MUX);
	vVByOneWriteFldAlign(VB1SYS1,0,VB1SYS1_MON_SEL);

	vDrvVByOneOSDOn();
	
    vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_CTRL);
    vIO32WriteFldAlign(TMGR5, 1, TMGR5_RG_FIFO_EN);
	vDrvVByOneStartEn();  // start for 8/16 lanes
	vDrvVByOneOSDDigitalOn();
#endif
}

void vDrvVByOneGFXClkSet(void)
{
	UINT32 u4VB1Lane;
	#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord2_GraphModeEn())
	#else
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	#endif
		u4VB1Lane = GM_LANE_NUM;
	else
		u4VB1Lane = GM_LANE_8;

    LOG(1,"%s %d vb1 Lane:%d \n",__FUNCTION__,__LINE__,u4VB1Lane);

    if(u4VB1Lane == GM_LANE_8)
    {
        if (((LVDS_OUTPUT_PORT == OCTA_PORT)||(LVDS_OUTPUT_PORT == HEXA_PORT)) && PANEL_IsSupport4K2KDD())
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
            if(IS_IC_5895_ES1())
            {
                vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV5_SEL);   // 600/1
                vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 600/2 1ch to 2ch
            }
            else
            {
                //eco to swap two ckgen.
                vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV5_SEL);   // 600/2 1ch to 2ch
                vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 600/1
            }
            vIO32WriteFldAlign(TMGR4, 1, TMGR4_OSD_1CH2CH_EN);
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 600/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 600/1
			}
		}
        else  //sony always keep OCTA_PORT
        {
        	if (IS_DISP_4K2K_TYPE3 && (GetPANEL4K2K_TYPE() == PANEL_TYPE_2K1K_120))  // For 4K2K60 OSD
    		{
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
			}
			else
			{
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_CLK75M2_CK_SEL);  // 300/4
			}
            
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 300/1
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 300/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 300/1
			}
        }

    }
	else if(u4VB1Lane == GM_LANE_4)
	{
		if ((LVDS_OUTPUT_PORT == OCTA_PORT) && PANEL_IsSupport4K2KDD())
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV5_SEL);   // 600/2
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV4_SEL);   // 600/2
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 1,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 600/2
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OSD_OCLK_SEL);   // for OSD: 600/2
			}
		}
		else if (LVDS_OUTPUT_PORT == HEXA_PORT)
		{
			// 4k4k@30Hz
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
            vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV5_SEL);   // 600/2
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 600/1
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 600/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 600/1
			}
		}
		else if (LVDS_OUTPUT_PORT == FOUR_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_CLK75M2_CK_SEL);  // 300/4
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 300/1
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 300/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 300/1
			}
		}
        else  //sony always keep OCTA_PORT
        {
            vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_CLK75M2_CK_SEL);  // 300/4
            vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV5_SEL);   // 300/1
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);   // 300/1
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 300/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 300/1
			}
        }
	}
	else if (u4VB1Lane == GM_LANE_2)
	{
		if ((LVDS_OUTPUT_PORT == OCTA_PORT)||(LVDS_OUTPUT_PORT == HEXA_PORT))
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
	   		vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_OCLK_DIV4_SEL);   // 600/4
	   		if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
	   			vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 2,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 600/4
	   			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_OSD_OCLK_SEL);   // for OSD: 600/4
	   		}
		}
		else if (LVDS_OUTPUT_PORT == FOUR_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_CLK75M2_CK_SEL);  // 300/4
	   		vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV4_SEL);   // 300/2
	   		if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
	   			vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 1,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 300/2
	   			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OSD_OCLK_SEL);   // for OSD: 300/2
	   		}
		}
		else if(LVDS_OUTPUT_PORT == DUAL_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_CLK75M2_CK_SEL);  // 148.5MHz/2=74.25
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);	// oclk = oclk_osd =148.5MHz	
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 150/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 150/1
			}
		}
	}
	else if (u4VB1Lane == GM_LANE_1)
	{
		if ((LVDS_OUTPUT_PORT == OCTA_PORT)||(LVDS_OUTPUT_PORT == HEXA_PORT))
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_CLK75M2_CK_SEL);  // 600/8
	   		vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_OCLK_DIV4_SEL);   // 600/8
	   		if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
	   			vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 3,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 600/8
	   			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 3,FLD_OSD_OCLK_SEL);   // for OSD: 600/8
	   		}
		}
		else if (LVDS_OUTPUT_PORT == FOUR_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_CLK75M2_CK_SEL);  // 300/4
	   		vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_OCLK_DIV4_SEL);   // 300/4
	   		if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
	   			vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 2,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 300/4
	   			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2,FLD_OSD_OCLK_SEL);   // for OSD: 300/4
	   		}
		}
		else if(LVDS_OUTPUT_PORT == DUAL_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_CLK75M2_CK_SEL);  // 150/2
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OCLK_DIV4_SEL);   // 150/2
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 1,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 150/2
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1,FLD_OSD_OCLK_SEL);   // for OSD: 150/2
			}
		}			
		else if(LVDS_OUTPUT_PORT == SINGLE_PORT)
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_CLK75M2_CK_SEL);
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OCLK_DIV4_SEL);
			if (IS_SUPPORT_VB1_GRAPH_MODE)
			{
				vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen: 75/1
				vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD: 75/1
			}
		}
	}
}

/**
 * @brief vDrvVB1On
 * Power On VBYONE
 * @param  void
 * @retval void
 */
void vDrvVByOneOSDOn(void)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
    UINT32 u4DispBit;   
	UINT32 u4VB1Lane;
	UPTR u4OsdLocknPin, u4OsdLocknPinFunc;
	UPTR u4OsdHtpdnPin, u4OsdHtpdnPinFunc;

	#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord2_GraphModeEn())
	#else
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	#endif
		u4VB1Lane = GM_LANE_NUM;
	else
		u4VB1Lane = GM_LANE_8;

    u4DispBit = DISP_BIT;

    if ((DRVCUST_PanelQueryUptr(ePanelVB1OsdLocknPin,&u4OsdLocknPin)==0)&&(DRVCUST_PanelQueryUptr(ePanelVB1OsdLocknPinFunc,&u4OsdLocknPinFunc)==0))
	{
		if ((0 == u4OsdLocknPin) || (0 == u4OsdLocknPinFunc))
		{
			LOG(0, "[VB1][OSD_LOCKN] Did not define PANEL_VB1_OSD_LOCKN_PIN & PANEL_VB1_OSD_LOCKN_PIN_FUNC!!!\n");
		}
		else
		{
			LOG(0, "[VB1][OSD_LOCKN] 0x%x set to FUNCTION%d\n", u4OsdLocknPin, u4OsdLocknPinFunc);
	        UNUSED(BSP_PinSet(u4OsdLocknPin, u4OsdLocknPinFunc));
		}
	}
	if ((DRVCUST_PanelQueryUptr(ePanelVB1OsdHtpdnPin,&u4OsdHtpdnPin)==0)&&(DRVCUST_PanelQueryUptr(ePanelVB1OsdHtpdnPinFunc,&u4OsdHtpdnPinFunc)==0))
	{
		if ((0 == u4OsdHtpdnPin) || (0 == u4OsdHtpdnPinFunc))
		{
			LOG(0, "[VB1][OSD_HTPDN] Did not define PANEL_VB1_OSD_HTPDN_PIN & PANEL_VB1_OSD_HTPDN_PIN_FUNC!!!\n");
		}
		else
		{
			LOG(0, "[VB1][OSD_HTPDN] 0x%x set to FUNCTION%d\n", u4OsdHtpdnPin, u4OsdHtpdnPinFunc);
	        UNUSED(BSP_PinSet(u4OsdHtpdnPin, u4OsdHtpdnPinFunc));
		}
	}
	
	switch (u4DispBit)
	{
	case DISP_36BIT:	   
		vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_COL_5BYTES,VB1OSDSYS0_COL);
	   break;
   	 case DISP_30BIT:   	 	
		vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_COL_4BYTES,VB1OSDSYS0_COL);
    	break;		
	 case DISP_24BIT:
     case DISP_18BIT:
   	 	vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_COL_3BYTES,VB1OSDSYS0_COL);
    	break;
	default:
    	break;
	}

	vVByOneOSDFormatConfig();

	vIO32WriteFldAlign(CKGEN_MON_OUT_CKCFG, 0,FLD_GFX_TIME_GEN_CK);   // for GFX timng gen default clk
	vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0,FLD_OSD_OCLK_SEL);   // for OSD default clk
	vDrvVByOneGFXClkSet();

	vVByOneOSDAnalogSetting();
	vVByOneOSDDigitalReset();
	
    if(u4VB1Lane == GM_LANE_8)
    {
        vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_LANE_NUM_4,VB1OSDSYS0_LANE_NUM);
        vIO32WriteFldAlign(VB1OSDSYS0,0xFF,VB1OSDSYS0_LANE_EN_MUX);
    }
	else if(u4VB1Lane == GM_LANE_4)
	{   	
	   vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_LANE_NUM_4,VB1OSDSYS0_LANE_NUM);	   
	   vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);
	}
	else if (u4VB1Lane == GM_LANE_2)
	{
		vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_LANE_NUM_2,VB1OSDSYS0_LANE_NUM);
		if(IS_VB1_ALL_LANE_ON)
			vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);	   	
		else
			vIO32WriteFldAlign(VB1OSDSYS0,0x3,VB1OSDSYS0_LANE_EN_MUX);
	}
	else if (u4VB1Lane == GM_LANE_1)
	{
		vIO32WriteFldAlign(VB1OSDSYS0,VB1OSDSYS0_LANE_NUM_1,VB1OSDSYS0_LANE_NUM);
		if(IS_VB1_ALL_LANE_ON)
			vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);	   	
		else
			vIO32WriteFldAlign(VB1OSDSYS0,0x3,VB1OSDSYS0_LANE_EN_MUX);
	}
		   
	vIO32WriteFldAlign(VB1OSDSYS0,0x1,VB1OSDSYS0_DE_RST_CTRL);
	vIO32WriteFldMulti(VB1OSDSYS1, P_Fld(3, VB1OSDSYS1_LANE3_MUX)|
								P_Fld(2, VB1OSDSYS1_LANE2_MUX)|
								P_Fld(1, VB1OSDSYS1_LANE1_MUX)|
								P_Fld(0, VB1OSDSYS1_LANE0_MUX)|
								P_Fld(0, VB1OSDSYS1_MON_SEL));
	
    //vIO32WriteFldAlign(VB1OSDSYS6, 1, VB1OSDSYS6_START);	// move behind VB1_video_start-->1
#endif
#endif
}

void vDrvVByOneStartEn(void)
{
#ifndef CC_VB1_DISABLE
	vVByOneWriteFldAlign(VB1SYS6, 1, VB1SYS6_START);
    //if (LVDS_OUTPUT_PORT==HEXA_PORT)  // [JW] just for verification
	{
	    vIO32WriteFldAlign(VB1SYS6_2ND, 1, VB1SYS6_234_VB1_START_SEL);  // use 1st_8lane start
	    vIO32WriteFldAlign(VB1SYS6_3RD, 1, VB1SYS6_234_VB1_START_SEL);  // use 1st_8lane start
	    vIO32WriteFldAlign(VB1SYS6_4TH, 1, VB1SYS6_234_VB1_START_SEL);  // use 1st_8lane start
	}
#endif
}

/**
 * @brief vDrvVB1Off
 * Power off VBYONE digital
 * @param  void
 * @retval void
 */
void vDrvVByOneOff(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_CTRL);
	vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_EN);
	vVByOneWriteFldAlign(VB1SYS6, 0, VB1SYS6_START);
#ifdef SUPPORT_PANEL_GRAPH_MODE
	vDrvVByOneOSDOff();
#endif
#endif
}

/**
 * @brief vDrvVB1OSDOff
 * Power off digital VBYONE OSD path
 * @param  void
 * @retval void
 */
void vDrvVByOneOSDOff(void)
{
#ifndef CC_VB1_DISABLE
	#ifdef SUPPORT_PANEL_GRAPH_MODE
     vIO32WriteFldAlign(VB1OSDSYS6, 0, VB1OSDSYS6_START);
	#endif
#endif
}

BOOL fgIfVByOneCRCCheckPass(void)
{
	UINT32 u4VB1Lane;
	UINT16 u2CRCStaA, u2CRCStaB;

	u4VB1Lane = LVDS_OUTPUT_PORT;
	u2CRCStaA = IO32ReadFldAlign(VB1SYS8, VB1SYS8_CRC_FAIL_STA);
	u2CRCStaB = IO32ReadFldAlign(VB1SYS8_2ND, VB1SYS8_CRC_FAIL_STA);
	
	if(u4VB1Lane == HEXA_PORT)
	{
	    LOG(0, "L0~L7\n");
		LOG(0, "0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx\n", 
            IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE), IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE), IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE), IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE),
			IO32ReadFldAlign(VB1CRC4, VB1CRC4_CRC4_VALUE), IO32ReadFldAlign(VB1CRC5, VB1CRC5_CRC5_VALUE), IO32ReadFldAlign(VB1CRC6, VB1CRC6_CRC6_VALUE), IO32ReadFldAlign(VB1CRC7, VB1CRC7_CRC7_VALUE));
        LOG(0, "L8~L15\n");
        LOG(0, "0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx 0x%lx\n", 
            IO32ReadFldAlign(VB1CRC0_2ND, VB1CRC0_CRC0_VALUE), IO32ReadFldAlign(VB1CRC1_2ND, VB1CRC1_CRC1_VALUE), IO32ReadFldAlign(VB1CRC2_2ND, VB1CRC2_CRC2_VALUE), IO32ReadFldAlign(VB1CRC3_2ND, VB1CRC3_CRC3_VALUE),
			IO32ReadFldAlign(VB1CRC4_2ND, VB1CRC4_CRC4_VALUE), IO32ReadFldAlign(VB1CRC5_2ND, VB1CRC5_CRC5_VALUE), IO32ReadFldAlign(VB1CRC6_2ND, VB1CRC6_CRC6_VALUE), IO32ReadFldAlign(VB1CRC7_2ND, VB1CRC7_CRC7_VALUE));

		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		LOG(0, "BCrcSta=0x%lx\n",u2CRCStaB);
		if (u2CRCStaA || u2CRCStaB)
		{
			LOG(0,"16 lane CRC fail\n");
			return (FALSE);
		}
		else
		{
			LOG(0,"16 lane CRC pass\n");
			return (TRUE);
		}	
	}
	else if(u4VB1Lane == OCTA_PORT)
	{
		LOG(0, "L0~L7\n");
		LOG(0, "0x%8x 0x%8x 0x%8x 0x%8x 0x%8x 0x%8x 0x%8x 0x%8x\n", 
            IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE), IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE), IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE), IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE),
			IO32ReadFldAlign(VB1CRC4, VB1CRC4_CRC4_VALUE), IO32ReadFldAlign(VB1CRC5, VB1CRC5_CRC5_VALUE), IO32ReadFldAlign(VB1CRC6, VB1CRC6_CRC6_VALUE), IO32ReadFldAlign(VB1CRC7, VB1CRC7_CRC7_VALUE));

		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if (u2CRCStaA)
		{
			LOG(0,"8 lane CRC fail\n");
			return (FALSE);
		}
		else
		{
			LOG(0,"8 lane CRC pass\n");
			return (TRUE);
		}
	}	
	else if(u4VB1Lane == FOUR_PORT)
	{
		LOG(0, "L0~L3\n");
		LOG(0, "0x%lx 0x%lx 0x%lx 0x%lx\n", 
            IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE), IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE), IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE), IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE));

		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if (u2CRCStaA)
		{
			LOG(0,"4 lane CRC fail\n");
			return (FALSE);
		}
		else
		{
			LOG(0,"4 lane CRC pass\n");
			return (TRUE);
		}
	}
	else if (u4VB1Lane == DUAL_PORT)
	{   
		LOG(0, "L0~L1\n");
		LOG(0, "0x%lx 0x%lx\n", 
            IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE), IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE));

		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if (u2CRCStaA)
		{
			LOG(0,"2 lane CRC fail\n");
			return (FALSE);
		}	
		else
		{
			LOG(0,"2 lane CRC pass\n");
			return (TRUE);
		}
	}
	else if (u4VB1Lane == SINGLE_PORT)
	{
		LOG(0, "[L0]0x%lx\n", IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE));

		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if (u2CRCStaA)
		{
	    	LOG(0,"1 lane CRC fail\n");
			return (FALSE);
		}	
		else
		{
	    	LOG(0,"1 lane CRC pass\n");
			return (TRUE);
		}
	}
	else
	{
		return (FALSE);
	}
}

/**
 * @brief vDrvVByOneCRCCheck
 * Data Path crc check
 * @param  void
 * @retval void
 */
void vDrvVByOneCRCCheck(void)
{
#ifndef CC_VB1_DISABLE
	UINT8 u1Back3DLREN;
#if 0
	#if defined(CC_MT5893) || defined(CC_MT5895)
	UINT64 *pu4PinMap;
	#endif
    PFN_SET_TURN_ON_BACKLT pfnSetTurnOnBacklt = NULL;
    PFN_SET_TURN_OFF_BACKLT pfnSetTurnOffBacklt = NULL;
#endif

	u1Back3DLREN = (UINT8)IO32ReadFldAlign(VB1SYS0,VB1SYS0_VB1_3DLR_EN);
	
#if 0
    if (IS_IC_5893())
    {
	UNUSED(DRVCUST_InitQueryUptr(eDrvTurnOffBacklt, (UPTR *)(void *)&(pfnSetTurnOffBacklt)));
    if(pfnSetTurnOffBacklt) pfnSetTurnOffBacklt();

	pu4PinMap = (UINT64 *)_au4PanelVB1LaneNOSwap;
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
	#endif									

	// 1st 8lane
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[7], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
									P_Fld(pu4PinMap[6], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7, P_Fld(pu4PinMap[5], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
									P_Fld(pu4PinMap[4], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
									P_Fld(pu4PinMap[3], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[2], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[1], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[0], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	// ***2nd 8lane***
	vIO32WriteFldMulti(VB1SYSCTRL8_2ND, P_Fld(pu4PinMap[19], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
										P_Fld(pu4PinMap[18], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7_2ND, P_Fld(pu4PinMap[17], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
										P_Fld(pu4PinMap[16], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
										P_Fld(pu4PinMap[15], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
										P_Fld(pu4PinMap[14], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
										P_Fld(pu4PinMap[13], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
										P_Fld(pu4PinMap[12], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	x_thread_delay(400); //400ms
    }
#endif

	vVByOneWriteFldAlign(VB1SYS0,0x0,VB1SYS0_VB1_3DLR_EN);
	vVByOneWriteFldAlign(VB1RES,0x1,VB1RES_DISP_R_CONTRL);
	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
	HAL_Delay_us(4* 20000); // delay 4vsyn
	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
	HAL_Delay_us(1* 20000);
	
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
	HAL_Delay_us(4* 20000); // delay 4vsync

	UNUSED(fgIfVByOneCRCCheckPass());

	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);

	vVByOneWriteFldAlign(VB1SYS0,u1Back3DLREN,VB1SYS0_VB1_3DLR_EN);

#if 0
    if (IS_IC_5893())
    {
	if (PANEL_Is4K2K60())
	{
	#if defined(CC_S_PLATFORM)&&defined(CC_HK_CUST)
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2ndSP):((UINT64 *)_au4PanelVB1LaneSwap1stSP);
	#else
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2nd):((UINT64 *)_au4PanelVB1LaneSwapOrg);
	#endif
	}
	else if (PANEL_Is4K2K120()||PANEL_Is8K4K30())
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwap4K120;
	}
	else
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwapOrg;
	}

	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
	#endif									

	// 1st 8lane
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[7], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
									P_Fld(pu4PinMap[6], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7, P_Fld(pu4PinMap[5], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
									P_Fld(pu4PinMap[4], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
									P_Fld(pu4PinMap[3], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[2], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[1], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[0], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	// ***2nd 8lane***
	vIO32WriteFldMulti(VB1SYSCTRL8_2ND, P_Fld(pu4PinMap[19], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
										P_Fld(pu4PinMap[18], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7_2ND, P_Fld(pu4PinMap[17], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
										P_Fld(pu4PinMap[16], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
										P_Fld(pu4PinMap[15], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
										P_Fld(pu4PinMap[14], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
										P_Fld(pu4PinMap[13], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
										P_Fld(pu4PinMap[12], VB1SYSCTRL7_LANE_0_SWAP_SEL));

	UNUSED(DRVCUST_InitQueryUptr(eDrvTurnOnBacklt, (UPTR *)(void *)&(pfnSetTurnOnBacklt)));
    if(pfnSetTurnOnBacklt) pfnSetTurnOnBacklt();
    }
    else
    {
        UNUSED(pu4PinMap);
    }
#endif
#endif
}

void vDrvVByOneCRCCheckInVDE(void)
{
#ifndef CC_VB1_DISABLE
    UINT8 u1Back3DLREN;

    u1Back3DLREN = (UINT8)IO32ReadFldAlign(VB1SYS0,VB1SYS0_VB1_3DLR_EN);

    vVByOneWriteFldAlign(VB1SYS0,0x0,VB1SYS0_VB1_3DLR_EN);
    vVByOneWriteFldAlign(VB1ANA0,CHK_IN_VDE,VB1ANA0_CRC_MODE_SEL);
    vVByOneWriteFldAlign(VB1ANA0,0,VB1ANA0_CRC_RST);
    vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
    HAL_Delay_us(10);
    vVByOneWriteFldAlign(VB1ANA0,1,VB1ANA0_CRC_RST);
    vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
    HAL_Delay_us(4* 20000); // delay 4vsyn
    vVByOneWriteFldAlign(VB1ANA0,0,VB1ANA0_CRC_RST);
    vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
    HAL_Delay_us(1* 20000);

    vVByOneWriteFldAlign(VB1ANA0,1,VB1ANA0_CRC_RST);
    vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
    HAL_Delay_us(_u1CRCChkCnt * 20000); // delay 4vsync

    UNUSED(fgIfVByOneCRCCheckPass());

    vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);

    vVByOneWriteFldAlign(VB1SYS0,u1Back3DLREN,VB1SYS0_VB1_3DLR_EN);
    vVByOneWriteFldAlign(VB1ANA0,CHK_ALL_VSYNC,VB1ANA0_CRC_MODE_SEL);
    vVByOneWriteFldAlign(VB1ANA0,1,VB1ANA0_CRC_RST);
#endif
}

/**
 * @brief vDrvVByOneOSDCRCCheck
 * OSD path crc check
 * @param  void
 * @retval void
 */
void vDrvVByOneOSDCRCCheck(void)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
	UINT32 u4VB1Lane;
	UINT8 u1CRCSta;
    #if 0
    PFN_SET_TURN_OFF_BACKLT pfnSetTurnOffBacklt = NULL;
    
	//#if defined(CC_MT5893) || defined(CC_MT5895)
	UINT64 *pu4PinMap;

	UNUSED(DRVCUST_InitQueryUptr(eDrvTurnOffBacklt, (UPTR *)(void *)&(pfnSetTurnOffBacklt)));
    if(pfnSetTurnOffBacklt) pfnSetTurnOffBacklt();

	pu4PinMap = (UINT64 *)_au4PanelVB1LaneNOSwap;
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
	x_thread_delay(400); //400ms
	#endif

	#ifdef LOAD_PANEL_FROM_FLASH_CUSTOM
	if (FlashPanel_GetControlWord2_GraphModeEn())
	#else
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	#endif
    	u4VB1Lane = GM_LANE_NUM;
	else
        u4VB1Lane = GM_LANE_8;

	vIO32WriteFldAlign(VB1OSDRES,0x1,VB1OSDRES_DISP_R_CONTRL);
	vIO32WriteFldAlign(VB1OSDSYS3,0x0,VB1OSDSYS3_CRC_EN);
	vIO32WriteFldAlign(VB1OSDSYS3,0x1,VB1OSDSYS3_CRC_EN);

	HAL_Delay_us(4* 20000); // delay 4vsyn
	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
	HAL_Delay_us(1* 20000);
	
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
	HAL_Delay_us(4* 20000); // delay 4vsync

	u1CRCSta = IO32ReadFldAlign(VB1OSDSYS8, VB1OSDSYS8_CRC_FAIL_STA);
    if(u4VB1Lane == GM_LANE_8)
	{
		LOG(0, "[lane0]0x%lx\n[lane1]0x%lx\n[lane2]0x%lx\n[lane3]0x%lx\n[lane4]0x%lx\n[lane5]0x%lx\n[lane6]0x%lx\n[lane7]0x%lx\n", IO32ReadFldAlign(VB1OSDCRC0, VB1OSDCRC0_CRC0_VALUE),
			IO32ReadFldAlign(VB1OSDCRC1, VB1OSDCRC1_CRC1_VALUE), IO32ReadFldAlign(VB1OSDCRC2, VB1OSDCRC2_CRC2_VALUE), IO32ReadFldAlign(VB1OSDCRC3, VB1OSDCRC3_CRC3_VALUE),
			IO32ReadFldAlign(VB1OSDCRC4, VB1OSDCRC4_CRC4_VALUE),
			IO32ReadFldAlign(VB1OSDCRC5, VB1OSDCRC5_CRC5_VALUE), IO32ReadFldAlign(VB1OSDCRC6, VB1OSDCRC6_CRC6_VALUE), IO32ReadFldAlign(VB1OSDCRC7, VB1OSDCRC7_CRC7_VALUE));

		if(u1CRCSta)
		{
			LOG(0,"OSD 8 lane CRC fail\n");
		}
		else
		{
			LOG(0,"OSD 8 lane CRC pass\n");
		}
	}
    else if(u4VB1Lane == GM_LANE_4)
	{
		LOG(0, "[lane0]0x%lx\n[lane1]0x%lx\n[lane2]0x%lx\n[lane3]0x%lx\n", IO32ReadFldAlign(VB1OSDCRC0, VB1OSDCRC0_CRC0_VALUE),
			IO32ReadFldAlign(VB1OSDCRC1, VB1OSDCRC1_CRC1_VALUE), IO32ReadFldAlign(VB1OSDCRC2, VB1OSDCRC2_CRC2_VALUE), IO32ReadFldAlign(VB1OSDCRC3, VB1OSDCRC3_CRC3_VALUE));

		if(u1CRCSta)
		{
			LOG(0,"OSD 4 lane CRC fail\n");
		}
		else
		{
			LOG(0,"OSD 4 lane CRC pass\n");
		}
	}
	else if (u4VB1Lane == GM_LANE_2)
	{   
		LOG(0, "[lane0]0x%lx\n[lane1]0x%lx\n", IO32ReadFldAlign(VB1OSDCRC0, VB1OSDCRC0_CRC0_VALUE), IO32ReadFldAlign(VB1OSDCRC1, VB1OSDCRC1_CRC1_VALUE));
		
		if(u1CRCSta)
		{
			LOG(0,"OSD 2 lane CRC fail\n");
		}	
		else
		{
			LOG(0,"OSD 2 lane CRC pass\n");
		}
	}
	else if (u4VB1Lane == GM_LANE_1)
	{
		LOG(0, "[lane0]0x%lx\n", IO32ReadFldAlign(VB1OSDCRC0, VB1OSDCRC0_CRC0_VALUE));
			
		if(u1CRCSta)
		{
	    	LOG(0,"OSD 1 lane CRC fail\n");
		}	
		else
		{
	    	LOG(0,"OSD 1 lane CRC pass\n");
		}
	}

	vIO32WriteFldAlign(VB1OSDSYS3,0x0,VB1OSDSYS3_CRC_EN);

#if 0
    if (IS_IC_5893())
    {
	if (PANEL_Is4K2K60())
	{
	#if defined(CC_S_PLATFORM)&&defined(CC_HK_CUST)
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2ndSP):((UINT64 *)_au4PanelVB1LaneSwap1stSP);
	#else
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2nd):((UINT64 *)_au4PanelVB1LaneSwapOrg);
	#endif
	}
	else if (PANEL_Is4K2K120()||PANEL_Is8K4K30())
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwap4K120;
	}
	else
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwapOrg;
	}

	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
    }
    else
    {
        UNUSED(pu4PinMap);
    }
#endif
#endif
#endif
}

void vDrvVByOneOSDPatternGen(UINT8 u1OnOff)
{
#ifndef CC_VB1_DISABLE
#ifdef SUPPORT_PANEL_GRAPH_MODE
	if (IS_SUPPORT_VB1_GRAPH_MODE)
	{
		vIO32WriteFldAlign(VINTF_B1, GRAPHMODE_GetHtotalByTiming(GFXTimingInfo.u1DCLKType), VINTF_PTGEN_H_TOTAL); 
		vIO32WriteFldAlign(VINTF_B1, GRAPHMODE_GetPanelWidth(), VINTF_PTGEN_H_ACTIVE);
		vIO32WriteFldAlign(VINTF_B4, GRAPHMODE_GetHSyncWidthByTiming(GFXTimingInfo.u1DCLKType), VINTF_PTGEN_H_WIDTH);
		vIO32WriteFldAlign(VINTF_B3, (GRAPHMODE_GetHtotalByTiming(GFXTimingInfo.u1DCLKType)-GRAPHMODE_GetPanelWidth()-GRAPHMODE_GetHSyncFpByTiming(GFXTimingInfo.u1DCLKType)), VINTF_PTGEN_H_START);
		vIO32WriteFldAlign(VINTF_B2, GRAPHMODE_GetVtotalByTiming(GFXTimingInfo.u1DCLKType), VINTF_PTGEN_V_TOTAL);
		vIO32WriteFldAlign(VINTF_B2, GRAPHMODE_GetPanelHeight(), VINTF_PTGEN_V_ACTIVE);
		vIO32WriteFldAlign(VINTF_B4, GRAPHMODE_GetVSyncWidthByTiming(GFXTimingInfo.u1DCLKType), VINTF_PTGEN_V_WIDTH);
		vIO32WriteFldAlign(VINTF_B3, (GRAPHMODE_GetVSyncBpByTiming(GFXTimingInfo.u1DCLKType)+GRAPHMODE_GetVSyncWidthByTiming(GFXTimingInfo.u1DCLKType)), VINTF_PTGEN_V_START);
		vIO32WriteFldAlign(VINTF_B11, GRAPHMODE_GetPanelHeight(), VINTF_PTGEN_V_OSD_ACTIVE);
		vIO32WriteFldAlign(VINTF_B11, (GRAPHMODE_GetVSyncBpByTiming(GFXTimingInfo.u1DCLKType)+GRAPHMODE_GetVSyncWidthByTiming(GFXTimingInfo.u1DCLKType)), VINTF_PTGEN_V_OSD_START);
    }
    else
    {   
        vIO32WriteFldAlign(VINTF_B1, PANEL_GetHTotalByTiming(bSI_DISPLAY_DCLK_TYPE), VINTF_PTGEN_H_TOTAL); 
		vIO32WriteFldAlign(VINTF_B1, PANEL_GetPanelWidth(), VINTF_PTGEN_H_ACTIVE);
		vIO32WriteFldAlign(VINTF_B4, PANEL_GetHSyncWidthByTiming(bSI_DISPLAY_DCLK_TYPE), VINTF_PTGEN_H_WIDTH);
		vIO32WriteFldAlign(VINTF_B3, (PANEL_GetHTotalByTiming(bSI_DISPLAY_DCLK_TYPE)-PANEL_GetPanelWidth()-PANEL_GetHSyncFpByTiming(bSI_DISPLAY_DCLK_TYPE)), VINTF_PTGEN_H_START);
		vIO32WriteFldAlign(VINTF_B2, PANEL_GetVTotalByTiming(bSI_DISPLAY_DCLK_TYPE), VINTF_PTGEN_V_TOTAL);
		vIO32WriteFldAlign(VINTF_B2, PANEL_GetPanelHeight(), VINTF_PTGEN_V_ACTIVE);
		vIO32WriteFldAlign(VINTF_B4, PANEL_GetVSyncWidthByTiming(bSI_DISPLAY_DCLK_TYPE), VINTF_PTGEN_V_WIDTH);
		vIO32WriteFldAlign(VINTF_B3, (PANEL_GetVSyncBpByTiming(bSI_DISPLAY_DCLK_TYPE)+PANEL_GetVSyncWidthByTiming(bSI_DISPLAY_DCLK_TYPE)), VINTF_PTGEN_V_START);
		vIO32WriteFldAlign(VINTF_B11, PANEL_GetPanelHeight(), VINTF_PTGEN_V_OSD_ACTIVE);
		vIO32WriteFldAlign(VINTF_B11, (PANEL_GetVSyncBpByTiming(bSI_DISPLAY_DCLK_TYPE)+PANEL_GetVSyncWidthByTiming(bSI_DISPLAY_DCLK_TYPE)), VINTF_PTGEN_V_OSD_START);
    }
    
	if (u1OnOff)
    {
    	_u1VB1OSDTest = IO32ReadFldAlign(VB1OSDSYS6, VB1OSDSYS6_TEST);
		_u1VB1OSDDeEn = IO32ReadFldAlign(VB1OSDSYS6, VB1OSDSYS6_DE_EN);
		_u1VB1OSDSel = IO32ReadFldAlign(VB1OSDSYS6, VB1OSDSYS6_SEL);
		_u1LineBufByp = IO32ReadFldAlign(VINTF_B0, VINTF_LINE_BUFFER_BYPASS);
    	vIO32WriteFldAlign(VB1OSDSYS6, 0x0, VB1OSDSYS6_TEST); // 0: from graphic path; 1: from video 0~3 4 lanes
		vIO32WriteFldAlign(VB1OSDSYS6, 0x1, VB1OSDSYS6_DE_EN);  // 0: DE from video path; 1: DE from graphic path
		vIO32WriteFldAlign(VB1OSDSYS6, 0x1, VB1OSDSYS6_SEL);  // 0: from EH; 1: from panel interface
		vIO32WriteFldAlign(VINTF_B0, 0x1, VINTF_LINE_BUFFER_BYPASS);
        vIO32WriteFldAlign(VINTF_B6, 1, VINTF_INTF_PTGEN_EN);
    }
    else
    {
        vIO32WriteFldAlign(VINTF_B6, 0, VINTF_INTF_PTGEN_EN);
		vIO32WriteFldAlign(VB1OSDSYS6, _u1VB1OSDTest, VB1OSDSYS6_TEST); // 0: from graphic path; 1: from video 0~3 4 lanes
		vIO32WriteFldAlign(VB1OSDSYS6, _u1VB1OSDDeEn, VB1OSDSYS6_DE_EN);  // 0: DE from video path; 1: DE from graphic path
		vIO32WriteFldAlign(VB1OSDSYS6, _u1VB1OSDSel, VB1OSDSYS6_SEL);  // 0: from EH; 1: from panel interface
		vIO32WriteFldAlign(VINTF_B0, _u1LineBufByp, VINTF_LINE_BUFFER_BYPASS);
    }
#endif
#endif
}

/**
 * @brief vGetVByOneStatus
 * Get VBYONE status
 * @param  void
 * @retval void
 */
void vGetVByOneStatus(void)
{
#ifndef CC_VB1_DISABLE	
	LOG(0,"[VB1] LOCKN = %ld \n", IO32ReadFldAlign(VB1SYS8,VB1SYS8_LOCKN));
	LOG(0,"[VB1] HTDPN = %ld \n", IO32ReadFldAlign(VB1SYS8,VB1SYS8_HTDPN));	
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	LOG(0,"[VB1_OSD] LOCKN = %ld \n", IO32ReadFldAlign(VB1OSDSYS8,VB1OSDSYS8_LOCKN));
	LOG(0,"[VB1_OSD] HTDPN = %ld \n", IO32ReadFldAlign(VB1OSDSYS8,VB1OSDSYS8_HTDPN));	
	#endif	
#endif	
}

#ifndef CC_VB1_DISABLE
UINT16 u2UnlockCnt = 0;
UINT16 u2UnHTDP = 0;
UINT8 u1VB1Chk = 0;
#endif	

void vSetVByOneUnLockChk(UINT8 u1En)
{
#ifndef CC_VB1_DISABLE	
	u1VB1Chk = u1En;
#endif
}

void vGetVByOneUnLock(void)
{
#ifndef CC_VB1_DISABLE	
	if (IO32ReadFldAlign(VB1SYS8,VB1SYS8_LOCKN)==1)
	{
	u2UnlockCnt++;
	LOG(0,"[VB1] UnLOCKN Count=%d \n", u2UnlockCnt);
	}
	if (IO32ReadFldAlign(VB1SYS8,VB1SYS8_HTDPN)==1)	
	{
	u2UnHTDP++;
	LOG(0,"[VB1] HTDPN_FAIL Count=%d \n", u2UnHTDP);	
	}
#endif	
}

void vGetVByOneUnLockCnt(void)
{
#ifndef CC_VB1_DISABLE	
	LOG(0,"[VB1] u2UnlockCnt=%d \n", u2UnlockCnt);
	LOG(0,"[VB1] u2UnHTDP=%d \n", u2UnHTDP);
#endif	

}

void vDrvVB1LockCtrl(UINT8 bOnOff, UINT8 bValue)
{
	vIO32WriteFldAlign(TMGR18,bOnOff,TMGR18_RG_LOCK_CTRL);
	vIO32WriteFldAlign(TMGR18,bValue,TMGR18_RG_LOCK_VALUE);
}

void vDrvVB1OSDLockCtrl(UINT8 bOnOff, UINT8 bValue)
{
	vIO32WriteFldAlign(TMGR18,bOnOff,TMGR18_RG_LOCK_CTRL_OSD);
	vIO32WriteFldAlign(TMGR18,bValue,TMGR18_RG_LOCK_VALUE_OSD);
}

static void VByOneToggleVOPLL(void)
{
#ifndef CC_VB1_DISABLE	
	// toggle VOPLL Power
	//vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 1, REG_LVDSTX_PLL_CFG0_VPLL_BG_LVDS_PWD);     // Capy remove
	//vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 1, REG_LVDSTX_PLL_CFG0_VPLL_BG_VB1_TX_PWD);  // Capy remove
	vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG1, 1, REG_LVDSTX_PLL_CFG1_VPLL_PWD);
	vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG5, 1, REG_LVDSTX_PLL_CFG5_PPLL_PWD); 	  
	HAL_Delay_us(400);
	vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG1, 0, REG_LVDSTX_PLL_CFG1_VPLL_PWD); 
	vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG5, 0, REG_LVDSTX_PLL_CFG5_PPLL_PWD);
	//vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_LVDS_PWD);     // Capy remove
	//vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_VB1_TX_PWD);  // Capy remove
	HAL_Delay_us(20);
#endif	
}

void vVByOneLane4_8bit_script(void)
{
#ifndef CC_VB1_DISABLE							   
	vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG1, P_Fld(0x1D, REG_LVDSTX_PLL_CFG1_VPLL_FBKDIV)|P_Fld(1, REG_LVDSTX_PLL_CFG1_VPLL_FBKSEL));
	VByOneToggleVOPLL();
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_3BYTES,VB1SYS0_COL);
	vVByOneWriteFldAlign(VB1SYS1,0,VB1SYS1_TTL_REORDER);
	vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2,FLD_VB1_CK_SEL);
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_4,VB1SYS0_LANE_NUM);
	vVByOneWriteFldAlign(VB1SYS0,0xff,VB1SYS0_LANE_EN_MUX);
	vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);
#endif
}

void vVByOneLane4_10bit_script(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG1, P_Fld(0x13, REG_LVDSTX_PLL_CFG1_VPLL_FBKDIV)|P_Fld(3, REG_LVDSTX_PLL_CFG1_VPLL_FBKSEL));
	VByOneToggleVOPLL();
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_4BYTES,VB1SYS0_COL);
	vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2,FLD_VB1_CK_SEL);
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_4,VB1SYS0_LANE_NUM);
	vVByOneWriteFldAlign(VB1SYS0,0xff,VB1SYS0_LANE_EN_MUX);
	vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);
#endif  
}

void vVByOneLane4_12bit_script(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG1, P_Fld(0x18, REG_LVDSTX_PLL_CFG1_VPLL_FBKDIV)|P_Fld(3, REG_LVDSTX_PLL_CFG1_VPLL_FBKSEL));       
	VByOneToggleVOPLL();
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_COL_5BYTES,VB1SYS0_COL);
	vVByOneWriteFldAlign(VB1SYS1,0,VB1SYS1_TTL_REORDER);
	vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2,FLD_VB1_CK_SEL);
	vVByOneWriteFldAlign(VB1SYS0,VB1SYS0_LANE_NUM_4,VB1SYS0_LANE_NUM);
	vVByOneWriteFldAlign(VB1SYS0,0xff,VB1SYS0_LANE_EN_MUX);
	vIO32WriteFldAlign(VB1OSDSYS0,0xF,VB1OSDSYS0_LANE_EN_MUX);
#endif  
}

void vVByOneTestMode(void)
{
#ifndef CC_VB1_DISABLE
	vIO32WriteFldAlign(REG_LVDS_TX_CFG16,0xFFFFFFFF,RG_LVDSA_PRE_PWD);
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_LANE3_ANA);
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_LANE2_ANA);
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_LANE1_ANA);
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_LANE0_ANA);	
	vVByOneWriteFldAlign(VB1SYS3,0x3FF,VB1SYS3_LANE3_ANA_VALUE);		
#endif  
}

#if 0  // Capy remove
void vVByOneLaneSwapForTest(UINT8 u1Mode)
{
#ifdef CC_MT5893
	switch(u1Mode)
	{
		case 1: // for VB1 analog measure
			u4PortA_1 = u4IO32Read4B(VB1SYSCTRL7);  // 0x3586c
			u4PortA_2 = u4IO32Read4B(VB1SYSCTRL8);  // 0x35870
			u4PortB_1 = u4IO32Read4B(VB1SYSCTRL7_2ND);  // 0x3526c
			u4PortB_2 = u4IO32Read4B(VB1SYSCTRL8_2ND);  // 0x35270
			
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_VALUE);
			vIO32WriteFldAlign(TMGR18,0x1,TMGR18_RG_LOCK_CTRL);
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_VALUE_OSD);
			vIO32WriteFldAlign(TMGR18,0x1,TMGR18_RG_LOCK_CTRL_OSD);

			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE0, VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL);
			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE1, VB1SYSCTRL8_LANE_7_SWAP_SEL);
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE2, VB1SYSCTRL7_LANE_3_SWAP_SEL);
			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE3, VB1SYSCTRL8_LANE_6_SWAP_SEL);
			break;
		case 0:
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_CTRL);
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_CTRL_OSD);
			vIO32Write4B(VB1SYSCTRL8, u4PortA_2);
			vIO32Write4B(VB1SYSCTRL7_2ND, u4PortB_1);
			break;
		default:
			break;
	}
#else
	UINT32 u4Lane0123=0;
	UINT32 u4Lane4567=0;
	UINT32 u4OSDLane0123=0;
	UINT32 u42ndLane0123=0;
	UINT32 u42ndLane4567=0;
	u4Lane0123 = IO32ReadFldAlign(VB1SYSCTRL7, VB1SYSCTRL7_LANE0123_SWAP);
	u4Lane4567 = (IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_LANE67_SWAP)<<8)+IO32ReadFldAlign(VB1SYSCTRL7,VB1SYSCTRL7_LANE45_SWAP);
	u4OSDLane0123 = IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_VB1_OSD_LANE0123_SWAP);
	LOG(0,"[VB1][Test]lane0123=0x%lx, lane4567=0x%lx, OSDlane0123=0x%lx\n", u4Lane0123, u4Lane4567, u4OSDLane0123);
	switch(u1Mode)
	{
		case 1:  // for VB1 analog measure
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_VALUE);
			vIO32WriteFldAlign(TMGR18,0x1,TMGR18_RG_LOCK_CTRL);
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_VALUE_OSD);
			vIO32WriteFldAlign(TMGR18,0x1,TMGR18_RG_LOCK_CTRL_OSD);
			vIO32WriteFldAlign(VB1SYSCTRL7,0x10,VB1SYSCTRL7_LANE45_SWAP);  // lane4/5/6/7 swap to lane0/1/2/3
			vIO32WriteFldAlign(VB1SYSCTRL8,0x32,VB1SYSCTRL8_LANE67_SWAP);  
			vIO32WriteFldAlign(VB1SYSCTRL8,0x3210,VB1SYSCTRL8_VB1_OSD_LANE0123_SWAP);  // OSDlane0/1/2/3 swap to lane0/1/2/3
			LOG(0,"[VB1][Test][mode_1]lane4567:0x%x, OSDlane0123:0x%x\n",
				(IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_LANE67_SWAP)<<8)+IO32ReadFldAlign(VB1SYSCTRL7,VB1SYSCTRL7_LANE45_SWAP),
				IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_VB1_OSD_LANE0123_SWAP));
			break;
		case 0: // resume
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_CTRL);
			vIO32WriteFldAlign(TMGR18,0x0,TMGR18_RG_LOCK_CTRL_OSD);
			vIO32WriteFldAlign(VB1SYSCTRL7,0x3210,VB1SYSCTRL7_LANE0123_SWAP);
			vIO32WriteFldAlign(VB1SYSCTRL7,0x54,VB1SYSCTRL7_LANE45_SWAP);
			vIO32WriteFldAlign(VB1SYSCTRL8,0x76,VB1SYSCTRL8_LANE67_SWAP);
			vIO32WriteFldAlign(VB1SYSCTRL8,0xba98,VB1SYSCTRL8_VB1_OSD_LANE0123_SWAP);
			LOG(0,"[VB1][Test][mode_0]lane0123:0x%x, lane4567:0x%x, OSDlane0123:0x%x\n",
				IO32ReadFldAlign(VB1SYSCTRL7,VB1SYSCTRL7_LANE0123_SWAP),
				(IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_LANE67_SWAP)<<8)+IO32ReadFldAlign(VB1SYSCTRL7,VB1SYSCTRL7_LANE45_SWAP),
				IO32ReadFldAlign(VB1SYSCTRL8,VB1SYSCTRL8_VB1_OSD_LANE0123_SWAP));
		default:
			break;
	}
#endif
}
#endif

void vVByOneSingleLanePadEn(UINT8 u1LaneEn)
{
    // LVDSA_EXT_EN_D11 {0:O0; 1:O1; 2:O2; 3:O3; 4:O4; 5:OCK; 6:E0; 7:E1; 8:E2; 9:E3; 10:E4; 11:ECK}
    UINT32 u2APortPadEn=0;

    if (IS_IC_5893())
    {
        switch (u1LaneEn)
        {
            case 1:  // lane0 --> AO0
                u2APortPadEn |= (1<<0);//ok
                break;
            case 2:  // lane1 --> AO1
                u2APortPadEn |= (1<<1);//ok
                break;
            case 3:  // lane2 --> AO2
                u2APortPadEn |= (1<<2);//ok
                break;
            case 4:  // lane3 --> AOCK
                u2APortPadEn |= (1<<5);//ok need check
                break;
            case 5:  // lane4 --> AO3
                u2APortPadEn |= (1<<3);//ok need check
                break;
            case 6:  // lane5 --> AO4
                u2APortPadEn |= (1<<4);//ok need check
                break;
            case 7:  // lane6 --> BE1(AE0)
                u2APortPadEn |= (1<<19);//ok
                break;
            case 8:  // lane7 --> BO2(AE1)
                u2APortPadEn |= (1<<14);//ok
                break;

            case 9:  // lane8 --> BO1
                u2APortPadEn |= (1<<13);//ok
                break;
            case 10:  // lane9 --> BE0
                u2APortPadEn |= (1<<18);//ok
                break;
            case 11:  // lane10 --> BO0
                u2APortPadEn |= (1<<12);//ok
                break;
            case 12:  // lane11 --> AE4
                u2APortPadEn |= (1<<10);// 11 need check
                break;
            case 13:  // lane12--> BO4
                u2APortPadEn |= (1<<16);// ok
                break;
            case 14: //lane13--> AE3
                u2APortPadEn |= (1<<9);//ok
                break;
            case 15:  // lane14 -->AECK
                u2APortPadEn |= (1<<11);
                break;
            case 16:  // lane15 --> BO3
                u2APortPadEn |= (1<<15);
                break;

            // OSD 4 lanes	
            case 17:  // OSD lane1--> AE2
                u2APortPadEn |= (1<<8);//ok
                break;
            case 18:  // OSD lane2--> AE1
                u2APortPadEn |= (1<<7);//ok
                break;
            case 19:  // OSD lane3 -->BOCK
                u2APortPadEn |= (1<<17);//need check
                break;
            case 20:  // OSD lane4 --> AE0
                u2APortPadEn |= (1<<6);//ok
                break;

            case 0:
            default:
                u2APortPadEn = 0xfffff;
                break;
        }
        if (u1LaneEn>0)
            LOG(0,"[VB1]lane%d enable!! u2APortPadEn=%x\n", u1LaneEn, u2APortPadEn);
        else
            LOG(0,"[VB1] 20 lanes enable!!! u2APortPadEn=%x\n", u2APortPadEn);
    }
    else
    {
        switch (u1LaneEn)
        {
            case 1:  // lane0 --> AO0
                u2APortPadEn |= (1<<0);
                break;
            case 2:  // lane1 --> AO1
                u2APortPadEn |= (1<<1);
                break;
            case 3:  // lane2 --> AO2
                u2APortPadEn |= (1<<2);
                break;
            case 4:  // lane3 --> AOCK
                u2APortPadEn |= (1<<5);
                break;
            case 5:  // lane4 --> AO3
                u2APortPadEn |= (1<<3);
                break;
            case 6:  // lane5 --> AO4
                u2APortPadEn |= (1<<4);
                break;
            case 7:  // lane6 --> AE0
                u2APortPadEn |= (1<<6);
                break;
            case 8:  // lane7 --> AE1
                u2APortPadEn |= (1<<7);
                break;
            case 9:  // lane8 --> AE2
                u2APortPadEn |= (1<<8);
                break;
            case 10:  // lane9 --> AECK
                u2APortPadEn |= (1<<11);
                break;
            case 11:  // lane10 --> AE3
                u2APortPadEn |= (1<<9);
                break;
            case 12:  // lane11 --> AE4
                u2APortPadEn |= (1<<10);
                break;
            case 0:
            default:
                u2APortPadEn = 0xfff;
                break;
        }
        if (u1LaneEn>0)
            LOG(0,"[VB1]lane%d enable!! u2APortPadEn=%x\n", u1LaneEn, u2APortPadEn);
        else
            LOG(0,"[VB1] 12 lanes enable!!! u2APortPadEn=%x\n", u2APortPadEn);
    }
    
    vIO32WriteFldAlign(REG_LVDS_TX_CFG13, u2APortPadEn, RG_LVDSA_EXT_EN);
}

void vVByOneVPLLBandwidthSet(UINT8 u1Bandwidth)
{
	UINT8 u1BR=0;
	UINT8 u1BC=0;
	UINT8 u1DIVEN=0;
	UINT8 u1BIR=0;
	UINT8 u1BIC=0;
	UINT8 u1BP=0;
	
	switch (u1Bandwidth)
	{
		case 1:  // open-loop bandwidth=0.5MHz
			LOG(0,"[VB1] VPLL_bandwidth=0.5MHz\n");
			u1BR = 0x1;
			u1BC = 0x3;
			u1DIVEN = 0x1;
			u1BIR = 0x1;
			u1BIC = 0x1;
			u1BP = 0xE;
			break;
		case 2:  // open-loop bandwidth=1MHz
			LOG(0,"[VB1] VPLL_bandwidth=1MHz\n");
			u1BR = 0x1;
			u1BC = 0x3;
			u1DIVEN = 0x1;
			u1BIR = 0x3;
			u1BIC = 0x6;
			u1BP = 0x7;
			break;
		case 0:  // default setting
		default:
			LOG(0,"[VB1] VPLL_bandwidth-->Default\n");
			u1BR = 0x3;
			u1BC = 0x3;
			u1DIVEN = 0x1;
			u1BIR = 0xA;
			u1BIC = 0x4;
			u1BP = 0xC;
			break;
	}
	vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG3, P_Fld(u1BR, REG_LVDSTX_PLL_CFG3_VPLL_BR)|P_Fld(u1BP, REG_LVDSTX_PLL_CFG3_VPLL_BP)/*|P_Fld(u1BIR, REG_LVDSTX_PLL_CFG3_VPLL_BIR)*/);
	//vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG2, P_Fld(u1BIC, REG_LVDSTX_PLL_CFG2_VPLL_BIC)|P_Fld(u1BC, REG_LVDSTX_PLL_CFG2_VPLL_BC)|P_Fld(u1DIVEN, REG_LVDSTX_PLL_CFG2_VPLL_DIVEN));
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG2, u1BC, REG_LVDSTX_PLL_CFG2_VPLL_BC);
}

void vVByOnePreEmphasisOnOff(UINT8 u1OnOff)
{
	if (!u1OnOff)
	{
		vIO32WriteFldAlign(REG_LVDS_TX_CFG16, 0xFFFFFFFF, RG_LVDSA_PRE_PWD);
	}
	else
	{
		vIO32WriteFldAlign(REG_LVDS_TX_CFG16, 0x0, RG_LVDSA_PRE_PWD);
	}
}

#define LVDSAPreEmIndex 7
PRIVATE UINT8 bLVDSAPreEm[LVDSAPreEmIndex+1] =
{
	0x00, 0x02,0x04, 0x06, 0x08, 0x0A,0x0C, 0x0E,
//   0.5mA, 1mA, 1.5mA, 2mA, 2.5mA, 3mA,  3.5mA, 4mA [Capy] 0.5mA + val*0.25mA       
//   0x00,   0x01,    0x02,  0x03,    0x04,    0x05,    0x06,  0x07,
//   1mA,    1.5mA,  2mA,  2.5mA,  3mA,    3.5mA,  4mA,  4.5mA [Husky] 1mA + val*0.5mA
};
void vVByOnePreEmphasisSet(UINT8 value)
{
    UINT32 u4PreEmVal;
	//value &= 0x7;  //0~7
    if (value > LVDSAPreEmIndex) value = LVDSAPreEmIndex;
    value = bLVDSAPreEm[value];
    u4PreEmVal = (value<<28)|(value<<24)|(value<<20)|(value<<16)|(value<<12)|(value<<8)|(value<<4)|value;
    vIO32WriteFldAlign(REG_LVDS_TX_CFG35, u4PreEmVal, RG_LVDSA_TVO_PREM_0);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG36, u4PreEmVal, RG_LVDSA_TVO_PREM_1);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG37, u4PreEmVal, RG_LVDSA_TVO_PREM_2);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG38, u4PreEmVal, RG_LVDSA_TVO_PREM_3);
}

UINT8 vVByOneGetPreEmphasis(void)
{
    UINT8 u1Value;
	u1Value = (UINT8)(IO32ReadFldAlign(REG_LVDS_TX_CFG35, RG_LVDSA_TVO_PREM_0)&0xF);
	u1Value = u1Value/2;  // level
    return (u1Value);
}

void vVByOneOSDYSEnable(UINT8 u1OnOff)
{
	vIO32WriteFldAlign(VINTF_B0, u1OnOff, VINTF_LINE_BUFFER_BYPASS);
	vIO32WriteFldAlign(OSD_MIX_2, u1OnOff,REG_OSD_INDEX_EN);
	vVByOneWriteFldAlign(VB1SYS0, u1OnOff,VB1SYS0_VB1_YS_EN);
}

void vVByOneOSDYSThreshold(UINT8 u1Val)
{
	vIO32WriteFldAlign(OSD_MIX_2, u1Val, REG_OSD_INDEX_THD);
}

void VByOne2SecPattern(void)
{
	vDrvVByOne2SecOnOff(SV_ON);
	vDrvVByOne4SecOnOff(SV_OFF);
	vDrvVByOne8SecOnOff(SV_OFF);
	vIO32WriteFldAlign(VINTF_B6, 0xA, VINTF_PTGEN_TYPE);
	vVByOneWriteFldAlign(VB1SYS6, 0x0, VB1SYS6_TSET_VB1_4567);
}

void VByOne4SecPattern(void)
{
	vDrvVByOne2SecOnOff(SV_OFF);
	vDrvVByOne4SecOnOff(SV_ON);
	vDrvVByOne8SecOnOff(SV_OFF);
	vIO32WriteFldAlign(VINTF_B6, 0xB, VINTF_PTGEN_TYPE);
	vVByOneWriteFldAlign(VB1SYS6, 0x0, VB1SYS6_TSET_VB1_4567);
}

void VByOne8SecPattern(void)
{
	vDrvVByOne2SecOnOff(SV_OFF);
	vDrvVByOne4SecOnOff(SV_OFF);
	vDrvVByOne8SecOnOff(SV_ON);
	vIO32WriteFldAlign(VINTF_B6, 0xC, VINTF_PTGEN_TYPE);
	vVByOneWriteFldAlign(VB1SYS6, 0x0, VB1SYS6_TSET_VB1_4567);
}

#if 0  // Capy remove
void vDrvVByOneOSTGCHSwap(UINT8 u1OnOff)
{
	if (u1OnOff==SV_ON)  // OSTG CH1/CH2 swap
	{
		u4PortA_1 = u4IO32Read4B(VB1SYSCTRL7);  // 0x3586c
		u4PortA_2 = u4IO32Read4B(VB1SYSCTRL8);  // 0x35870
		u4PortB_1 = u4IO32Read4B(VB1SYSCTRL7_2ND);  // 0x3526c
		u4PortB_2 = u4IO32Read4B(VB1SYSCTRL8_2ND);  // 0x35270
		u1OstgCHFlg = u1DrvGetOstgCHSelFlag();
		
		//4k2k60, 4k2k120, 8k4k30
		if (PANEL_Is4K2K60())  //4k2k60
		{
			// portA: OSTG CH1
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE0, VB1SYSCTRL7_LANE_0_SWAP_SEL);  // AO0(portA lane0):1st lane0
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE1, VB1SYSCTRL7_LANE_1_SWAP_SEL);  // AO1(portA lane1):1st lane1
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE2, VB1SYSCTRL7_LANE_2_SWAP_SEL);  // AO2(portA lane2):1st lane2
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE3, VB1SYSCTRL7_LANE_3_SWAP_SEL);  // AOCK(portA lane3):1st lane3
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE4, VB1SYSCTRL7_LANE_4_SWAP_SEL);  // AO3(portA lane4):1stlane4
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE5, VB1SYSCTRL7_LANE_5_SWAP_SEL);  // AO4(portA lane5):1st lane5
			vIO32WriteFldAlign(VB1SYSCTRL8_2ND, VB1_DATA_2ND_LANE6, VB1SYSCTRL8_LANE_7_SWAP_SEL);  // BE1(portB lane7):1st lane6
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE7, VB1SYSCTRL7_LANE_2_SWAP_SEL);  // BO2(portB lane2):1st lane7
			vDrvSetOstgCHSelFlag(OSTG_CH0_ONLY);
		}
		else if (PANEL_Is4K2K120()||PANEL_Is8K4K30())  // 4k2k120, 8k4k30
		{	
			// portA: OSTG CH2
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE8, VB1SYSCTRL7_LANE_0_SWAP_SEL);  // AO0(portA lane0):2nd lane0
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE9, VB1SYSCTRL7_LANE_1_SWAP_SEL);  // AO1(portA lane1):2nd lane1
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE10, VB1SYSCTRL7_LANE_2_SWAP_SEL);  // AO2(portA lane2):2nd lane2
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE11, VB1SYSCTRL7_LANE_3_SWAP_SEL);  // AOCK(portA lane3):2nd lane3
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE12, VB1SYSCTRL7_LANE_4_SWAP_SEL);  // AO3(portA lane4):2nd lane4
			vIO32WriteFldAlign(VB1SYSCTRL7, VB1_DATA_LANE13, VB1SYSCTRL7_LANE_5_SWAP_SEL);  // AO4(portA lane5):2nd lane5
			vIO32WriteFldAlign(VB1SYSCTRL8_2ND, VB1_DATA_2ND_LANE14, VB1SYSCTRL8_LANE_7_SWAP_SEL);  // BE1(portB lane7):2nd lane6
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE15, VB1SYSCTRL7_LANE_2_SWAP_SEL);  // BO2(portB lane2):2nd lane7
			// OSD 4lanes + portB: OSTG CH1
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE0, VB1SYSCTRL7_LANE_1_SWAP_SEL);  // BO1(portB lane1):1st lane0
			vIO32WriteFldAlign(VB1SYSCTRL8_2ND, VB1_DATA_2ND_LANE1, VB1SYSCTRL8_LANE_6_SWAP_SEL);  // BE0(portB lane6):1st lane1
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE2, VB1SYSCTRL7_LANE_0_SWAP_SEL);  // BO0(portB lane0):1st lane2
			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE3, VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL);  // AE4(portA OSD3):1st lane3
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE4, VB1SYSCTRL7_LANE_5_SWAP_SEL);  // BO4(portB lane5):1st lane4
			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE5, VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL);  // AE3(portA OSD2):1st lane5
			vIO32WriteFldAlign(VB1SYSCTRL8, VB1_DATA_LANE6, VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL);  // AECK(portA OSD1):1st lane6
			vIO32WriteFldAlign(VB1SYSCTRL7_2ND, VB1_DATA_2ND_LANE7, VB1SYSCTRL7_LANE_4_SWAP_SEL);  // BO3(portB lane4):1st lane7
			vDrvSetOstgCHSelFlag(OSTG_CH0_AND_CH1);
		}
	}
	else
	{
		vIO32Write4B(VB1SYSCTRL7, u4PortA_1);
		vIO32Write4B(VB1SYSCTRL8, u4PortA_2);
		vIO32Write4B(VB1SYSCTRL7_2ND, u4PortB_1);
		vIO32Write4B(VB1SYSCTRL8_2ND, u4PortB_2);
		vDrvSetOstgCHSelFlag(u1OstgCHFlg);
	}
}
#endif

#ifdef __MODEL_slt__
#define VB1_CRC_DELAY_VSYNC 10

void vDrvVByOneCRCReset(void)
{
#ifndef CC_VB1_DISABLE
	//UINT8 u1VsyncCnt;
	LOG(0,"---- VB1CRCReset START ----\n");
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x0, RG_LVDSA_DIV5_EN);
	HAL_Delay_us(40);
	vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x1, RG_LVDSA_DIV5_EN);
	HAL_Delay_us(40);
	vIO32WriteFldAlign(TMGR5, 0, TMGR5_RG_FIFO_EN);
	HAL_Delay_us(40);
	vIO32WriteFldAlign(TMGR5, 1, TMGR5_RG_FIFO_EN);
	HAL_Delay_us(40);
	#if 0
	vVByOneWriteFldAlign(VB1SYS6, 0, VB1SYS6_START);	
	for (u1VsyncCnt=0; u1VsyncCnt < 4; u1VsyncCnt++)
	{
		HAL_Delay_us(20000);
	}
	vDrvVByOneStartEn();  // start for 8/16 lanes
	#endif
	LOG(0,"---- VB1CRCReset END ----\n");
#endif
}

/**
 * @brief vDrvGetVByOneCRC
 * Data Path crc check
 * @param  void
 * @retval void
 */
void vDrvGetVByOneCRC(UINT32 *pu4VB1Lane, UINT32 *pu4Crc)
{
#ifndef CC_VB1_DISABLE
	UINT32 u4VB1Lane=0;
	UINT8 u1Back3DLREN;
	UINT8 u1VsyncPolarity;
	UINT8 u1VsyncCnt;
	UINT16 u2CRCStaA, u2CRCStaB;
	PFN_SET_TURN_ON_BACKLT pfnSetTurnOnBacklt = NULL;
	PFN_SET_TURN_OFF_BACKLT pfnSetTurnOffBacklt = NULL;
	
    #if 0
	//#if defined(CC_MT5893) || defined(CC_MT5895)
	UINT64 *pu4PinMap;

	UNUSED(DRVCUST_InitQueryUptr(eDrvTurnOffBacklt, (UPTR *)(void *)&(pfnSetTurnOffBacklt)));
    if(pfnSetTurnOffBacklt) pfnSetTurnOffBacklt();

	pu4PinMap = (UINT64 *)_au4PanelVB1LaneNOSwap;
	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
	#endif									

	// 1st 8lane
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[7], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
									P_Fld(pu4PinMap[6], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7, P_Fld(pu4PinMap[5], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
									P_Fld(pu4PinMap[4], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
									P_Fld(pu4PinMap[3], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[2], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[1], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[0], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	// ***2nd 8lane***
	vIO32WriteFldMulti(VB1SYSCTRL8_2ND, P_Fld(pu4PinMap[19], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
										P_Fld(pu4PinMap[18], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7_2ND, P_Fld(pu4PinMap[17], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
										P_Fld(pu4PinMap[16], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
										P_Fld(pu4PinMap[15], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
										P_Fld(pu4PinMap[14], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
										P_Fld(pu4PinMap[13], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
										P_Fld(pu4PinMap[12], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	x_thread_delay(400); //400ms
	#endif

	vVByOneWriteFldAlign(VB1SYS6, 0, VB1SYS6_START);	
	for (u1VsyncCnt=0; u1VsyncCnt < 4; u1VsyncCnt++)
	{
		HAL_Delay_us(20000);
	}
	vDrvVByOneStartEn();  // start for 8/16 lanes			
	for (u1VsyncCnt=0; u1VsyncCnt < 4; u1VsyncCnt++)
	{
		HAL_Delay_us(20000);
	}
	
	if (PANEL_GetDisplayType() == PANEL_DISPLAY_TYPE_VB1)
	{
		vDrvVB1LockCtrl(SV_ON,0);
	}
	
	u4VB1Lane = LVDS_OUTPUT_PORT;
	u1Back3DLREN = IO32ReadFldAlign(VB1SYS0,VB1SYS0_VB1_3DLR_EN);
	u1VsyncPolarity = IO32ReadFldAlign(VB1SYS0,VB1SYS0_VSYN_INV);
		
	vVByOneWriteFldAlign(VB1SYS0,0x0,VB1SYS0_VB1_3DLR_EN);
	vVByOneWriteFldAlign(VB1RES,0x1,VB1RES_DISP_R_CONTRL);
	vVByOneWriteFldAlign(VB1SYS0,0x0,VB1SYS0_VSYN_INV);
	
	for (u1VsyncCnt=0; u1VsyncCnt < VB1_CRC_DELAY_VSYNC; u1VsyncCnt++)
	{
		HAL_Delay_us(20000);
	}
	
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
	for (u1VsyncCnt=0; u1VsyncCnt < VB1_CRC_DELAY_VSYNC; u1VsyncCnt++)
	{
		HAL_Delay_us(20000);
	}

	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
	HAL_Delay_us(1* 20000);
	
	vVByOneWriteFldAlign(VB1SYS3,0x1,VB1SYS3_CRC_EN);
	HAL_Delay_us(4* 20000); // delay 4vsync

	u2CRCStaA = IO32ReadFldAlign(VB1SYS8, VB1SYS8_CRC_FAIL_STA);
	u2CRCStaB = IO32ReadFldAlign(VB1SYS8_2ND, VB1SYS8_CRC_FAIL_STA);
	
	if(u4VB1Lane == HEXA_PORT)
	{
		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		LOG(0, "BCrcSta=0x%lx\n",u2CRCStaB);
		if(u2CRCStaA || u2CRCStaB)
		{
			LOG(0,"16 lane CRC fail\n");
		}
		else
		{
			LOG(0,"16 lane CRC pass\n");
		}
		
		*pu4Crc = IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE);
		*(pu4Crc+1) = IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE);
		*(pu4Crc+2) = IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE);
		*(pu4Crc+3) = IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE);
		*(pu4Crc+4) = IO32ReadFldAlign(VB1CRC4, VB1CRC4_CRC4_VALUE);
		*(pu4Crc+5) = IO32ReadFldAlign(VB1CRC5, VB1CRC5_CRC5_VALUE);
		*(pu4Crc+6) = IO32ReadFldAlign(VB1CRC6, VB1CRC6_CRC6_VALUE);
		*(pu4Crc+7) = IO32ReadFldAlign(VB1CRC7, VB1CRC7_CRC7_VALUE);

		*(pu4Crc+8) = IO32ReadFldAlign(VB1CRC0_2ND, VB1CRC0_CRC0_VALUE);
		*(pu4Crc+9) = IO32ReadFldAlign(VB1CRC1_2ND, VB1CRC1_CRC1_VALUE);
		*(pu4Crc+10) = IO32ReadFldAlign(VB1CRC2_2ND, VB1CRC2_CRC2_VALUE);
		*(pu4Crc+11) = IO32ReadFldAlign(VB1CRC3_2ND, VB1CRC3_CRC3_VALUE);
		*(pu4Crc+12) = IO32ReadFldAlign(VB1CRC4_2ND, VB1CRC4_CRC4_VALUE);
		*(pu4Crc+13) = IO32ReadFldAlign(VB1CRC5_2ND, VB1CRC5_CRC5_VALUE);
		*(pu4Crc+14) = IO32ReadFldAlign(VB1CRC6_2ND, VB1CRC6_CRC6_VALUE);
		*(pu4Crc+15) = IO32ReadFldAlign(VB1CRC7_2ND, VB1CRC7_CRC7_VALUE);
		
		LOG(0,"[lane0]0x%x\n[lane1]0x%x\n[lane2]0x%x\n[lane3]0x%x\n[lane4]0x%x\n[lane5]0x%x\n[lane6]0x%x\n[lane7]0x%x\n", 
			*pu4Crc, *(pu4Crc+1), *(pu4Crc+2), *(pu4Crc+3), *(pu4Crc+4), *(pu4Crc+5), *(pu4Crc+6), *(pu4Crc+7));
		LOG(0,"[lane8]0x%x\n[lane9]0x%x\n[lane10]0x%x\n[lane11]0x%x\n[lane12]0x%x\n[lane13]0x%x\n[lane14]0x%x\n[lane15]0x%x\n", 
			*(pu4Crc+8), *(pu4Crc+9), *(pu4Crc+10), *(pu4Crc+11), *(pu4Crc+12), *(pu4Crc+13), *(pu4Crc+14), *(pu4Crc+15));
	} 
	else if(u4VB1Lane == OCTA_PORT)
	{
		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if(u2CRCStaA)
		{
			LOG(0,"8 lane CRC fail");
		}
		else
		{
			LOG(0,"8 lane CRC pass\n");
		}

		if(BSP_GetIcVersion()==IC_VER_5893_AA)
		{
			*(pu4Crc+0) = IO32ReadFldAlign(VB1CRC0_2ND, VB1CRC0_CRC0_VALUE);
			*(pu4Crc+1) = IO32ReadFldAlign(VB1CRC1_2ND, VB1CRC1_CRC1_VALUE);
			*(pu4Crc+2) = IO32ReadFldAlign(VB1CRC2_2ND, VB1CRC2_CRC2_VALUE);
			*(pu4Crc+3) = IO32ReadFldAlign(VB1CRC3_2ND, VB1CRC3_CRC3_VALUE);
			*(pu4Crc+4) = IO32ReadFldAlign(VB1CRC4_2ND, VB1CRC4_CRC4_VALUE);
			*(pu4Crc+5) = IO32ReadFldAlign(VB1CRC5_2ND, VB1CRC5_CRC5_VALUE);
			*(pu4Crc+6) = IO32ReadFldAlign(VB1CRC6_2ND, VB1CRC6_CRC6_VALUE);
			*(pu4Crc+7) = IO32ReadFldAlign(VB1CRC7_2ND, VB1CRC7_CRC7_VALUE);
			LOG(0,"[lane0]0x%08x [lane1]0x08%x\n", *pu4Crc, *(pu4Crc+1));
			LOG(0,"[lane2]0x%08x [lane3]0x08%x\n", *(pu4Crc+2), *(pu4Crc+3));
			LOG(0,"[lane4]0x%08x [lane5]0x08%x\n", *(pu4Crc+4), *(pu4Crc+5));
			LOG(0,"[lane6]0x%08x [lane7]0x08%x\n", *(pu4Crc+6), *(pu4Crc+7));
		}
		else
		{
			
			*pu4Crc = IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE);
			*(pu4Crc+1) = IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE);
			*(pu4Crc+2) = IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE);
			*(pu4Crc+3) = IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE);
			*(pu4Crc+4) = IO32ReadFldAlign(VB1CRC4, VB1CRC4_CRC4_VALUE);
			*(pu4Crc+5) = IO32ReadFldAlign(VB1CRC5, VB1CRC5_CRC5_VALUE);
			*(pu4Crc+6) = IO32ReadFldAlign(VB1CRC6, VB1CRC6_CRC6_VALUE);
			*(pu4Crc+7) = IO32ReadFldAlign(VB1CRC7, VB1CRC7_CRC7_VALUE);		
			LOG(0,"[lane0]0x%x\n[lane1]0x%x\n[lane2]0x%x\n[lane3]0x%x\n[lane4]0x%x\n[lane5]0x%x\n[lane6]0x%x\n[lane7]0x%x\n", 
				*pu4Crc, *(pu4Crc+1), *(pu4Crc+2), *(pu4Crc+3), *(pu4Crc+4), *(pu4Crc+5), *(pu4Crc+6), *(pu4Crc+7));							
		}
	} 
	else if(u4VB1Lane == FOUR_PORT)
	{
		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if(u2CRCStaA)
		{
			LOG(0,"4 lane CRC fail\n");
		}
		else
		{
			LOG(0,"4 lane CRC pass\n");
		}

		*pu4Crc = IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE);
		*(pu4Crc+1) = IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE);
		*(pu4Crc+2) = IO32ReadFldAlign(VB1CRC2, VB1CRC2_CRC2_VALUE);
		*(pu4Crc+3) = IO32ReadFldAlign(VB1CRC3, VB1CRC3_CRC3_VALUE);
		
		LOG(0,"[lane0]0x%x\n[lane1]0x%x\n[lane2]0x%x\n[lane3]0x%x\n", *pu4Crc, *(pu4Crc+1), *(pu4Crc+2), *(pu4Crc+3));
	}
	else if (u4VB1Lane == DUAL_PORT)
	{ 	
		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if(u2CRCStaA)
		{
			LOG(0,"2 lane CRC fail\n");
		} 
		else
		{
			LOG(0,"2 lane CRC pass\n");
		}
		*pu4Crc = IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE);
		*(pu4Crc+1) = IO32ReadFldAlign(VB1CRC1, VB1CRC1_CRC1_VALUE);
		LOG(0,"[lane0]0x%x\n[lane1]0x%x\n", *pu4Crc, *(pu4Crc+1));
	}
	else if (u4VB1Lane == SINGLE_PORT)
	{
		LOG(0, "ACrcSta=0x%lx\n",u2CRCStaA);
		if(u2CRCStaA)
		{
			LOG(0,"1 lane CRC fail\n");
		} 
		else
		{
			LOG(0,"1 lane CRC pass\n");
		}
		*pu4Crc = IO32ReadFldAlign(VB1CRC0, VB1CRC0_CRC0_VALUE);
		LOG(0,"[lane0]0x%x\n", *pu4Crc);
	}
	*pu4VB1Lane = u4VB1Lane;
	vVByOneWriteFldAlign(VB1SYS0,u1Back3DLREN,VB1SYS0_VB1_3DLR_EN);
	vVByOneWriteFldAlign(VB1SYS0,u1VsyncPolarity,VB1SYS0_VSYN_INV);
	vVByOneWriteFldAlign(VB1SYS3,0x0,VB1SYS3_CRC_EN);
	
	if (PANEL_GetDisplayType() == PANEL_DISPLAY_TYPE_VB1)
	{
		vDrvVB1LockCtrl(SV_OFF,0);
	}	

#if 0
//#if defined(CC_MT5893) || defined(CC_MT5895)
	if (PANEL_Is4K2K60())
	{
	#if defined(CC_S_PLATFORM)&&defined(CC_HK_CUST)
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2ndSP):((UINT64 *)_au4PanelVB1LaneSwap1stSP);
	#else
		pu4PinMap = IS_IC_5893_ES1()?((UINT64 *)_au4PanelVB1LaneSwap2nd):((UINT64 *)_au4PanelVB1LaneSwapOrg);
	#endif
	}
	else if (PANEL_Is4K2K120()||PANEL_Is8K4K30())
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwap4K120;
	}
	else
	{
		pu4PinMap = (UINT64 *)_au4PanelVB1LaneSwapOrg;
	}

	#ifdef SUPPORT_PANEL_GRAPH_MODE
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[11], VB1SYSCTRL8_OSD_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[10], VB1SYSCTRL8_OSD_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[9],  VB1SYSCTRL8_OSD_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[8],  VB1SYSCTRL8_OSD_LANE_0_SWAP_SEL));
	#endif									

	// 1st 8lane
	vIO32WriteFldMulti(VB1SYSCTRL8, P_Fld(pu4PinMap[7], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
									P_Fld(pu4PinMap[6], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7, P_Fld(pu4PinMap[5], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
									P_Fld(pu4PinMap[4], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
									P_Fld(pu4PinMap[3], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
									P_Fld(pu4PinMap[2], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
									P_Fld(pu4PinMap[1], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
									P_Fld(pu4PinMap[0], VB1SYSCTRL7_LANE_0_SWAP_SEL));
	// ***2nd 8lane***
	vIO32WriteFldMulti(VB1SYSCTRL8_2ND, P_Fld(pu4PinMap[19], VB1SYSCTRL8_LANE_7_SWAP_SEL)|
										P_Fld(pu4PinMap[18], VB1SYSCTRL8_LANE_6_SWAP_SEL));									
	vIO32WriteFldMulti(VB1SYSCTRL7_2ND, P_Fld(pu4PinMap[17], VB1SYSCTRL7_LANE_5_SWAP_SEL)|
										P_Fld(pu4PinMap[16], VB1SYSCTRL7_LANE_4_SWAP_SEL)|
										P_Fld(pu4PinMap[15], VB1SYSCTRL7_LANE_3_SWAP_SEL)|
										P_Fld(pu4PinMap[14], VB1SYSCTRL7_LANE_2_SWAP_SEL)|
										P_Fld(pu4PinMap[13], VB1SYSCTRL7_LANE_1_SWAP_SEL)|
										P_Fld(pu4PinMap[12], VB1SYSCTRL7_LANE_0_SWAP_SEL));

	UNUSED(DRVCUST_InitQueryUptr(eDrvTurnOnBacklt, (UPTR *)(void *)&(pfnSetTurnOnBacklt)));
    if(pfnSetTurnOnBacklt) pfnSetTurnOnBacklt();
#endif
#endif
}
#endif

