/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: osd_base_if.c,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

/** @file osd_base_if.c
 *  osd base header file, includes general osd setting
 */

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_bim.h"
#include "osd_hw.h"
#include "osd_drvif.h"
#include "osd_exp.h"
#include "osd_debug.h"
#ifndef CC_MTK_LOADER
#include "gfx_if.h"
#include "drvcust_if.h"
//#include "ostg_if.h"
#include "x_os.h"
#include "x_timer.h"
#ifdef CC_SUPPORT_OSD_GLUE_INTERFACE
#include "osd_glue_internal.h"
#endif
#endif
#include "fbm_drvif.h"

LINT_EXT_HEADER_BEGIN
#include "x_hal_5381.h"
#include "panel.h"
LINT_EXT_HEADER_END

#include "eeprom_if.h"

#ifdef CC_TDTV_3D_OSD_ENABLE
#include "drv_tdtv_drvif.h"
#endif

#include "drv_display.h"
#include "sv_const.h"
#include "drv_scaler.h"
#include "drv_ttd.h"
#ifdef SMALL_OUTWINDOW_TTD_NEWSETTING
#include "hw_ttd.h"
#include "hw_ospe.h"
#endif

#ifdef CC_FBM_SUPPORT_SRS
#include "hw_tdsharp.h"
#endif

#include "hw_psc.h"
#include "drv_scaler_psc.h"
#include "hw_ospe.h"

#ifndef OSD_LOGO_FADING_ENABLE
#if defined (CC_S_PLATFORM) && defined(CC_LMP_CUST)
#define OSD_LOGO_FADING_ENABLE (0)
#else
#define OSD_LOGO_FADING_ENABLE (1)
#endif
#endif

#define CHECK_OSD_CPBLTY_PARA(u4_type,u4_idx)    \
{                                                   \
    if(u4_type>=E_OSD_CPBLTY_MAX)                  \
    {                                               \
        return OSD_RET_INV_ARG;                    \
    }                                               \
    if(u4_idx>=OSD_PLANE_MAX_NUM)                  \
    {                                               \
        return OSD_RET_INV_ARG;                    \
    }                                               \
}

#define OSD_CHECK_FLIP_MIRROR(u4_plane)             \
  (OSD_GetCpblty(E_OSD_CPBLTY_FLIP, 0x0)||OSD_GetCpblty(E_OSD_CPBLTY_MIRROR,0x0))



//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------
static INT32 i4Hstart = 0x0;
#ifdef CC_LOADER_LOGO_LONG_TIME
static UINT32 u4OSDTopLayer = OSD_PLANE_3;
static UINT32 u4OSDMiddLayer = OSD_PLANE_2;
static UINT32 u4OSDBotLayer = OSD_PLANE_1;
#else
static UINT32 u4OSDTopLayer = OSD_PLANE_2;
static UINT32 u4OSDMiddLayer = OSD_PLANE_3;
static UINT32 u4OSDBotLayer = OSD_PLANE_1;
#endif
#ifdef OSD_TRANS_CTRL
static UINT32 _OSD_PlANE2_TransferBuf_1 = 0;
static UINT32 _OSD_PlANE2_TransferBuf_2 = 0;
static UINT32 _OSD_PlANE2_TransferBuf_CURRENT = 0;
#endif

#ifndef CC_MTK_LOADER
#ifdef CC_SUPPORT_STR
static BOOL b_suspend = FALSE;
static BOOL bFinishResume = FALSE;
#endif
#endif

#ifndef CC_MTK_LOADER
static BOOL fgLogoInit = FALSE;
static HANDLE_T _hOsdFlipToSema;

#include <linux/spinlock.h>
static DEFINE_SPINLOCK(_lockOsdPos);
static unsigned long _lockOsdPosFlag;
/* static HANDLE_T _hOsdLogoSema; */
#endif

#ifdef CC_TDTV_3D_OSD_ENABLE
BOOL gfgOsd_Sbs = FALSE;
BOOL gfgOsd_Tbb = FALSE;
UINT8 _u13DOsdShift = 0;
#endif

#ifdef CC_TDTV_3D_OSD_ENABLE
static UINT8 _u1Plane2RightRegion = 0;
static UINT8 _u1Plane3RightRegion = 0;

#endif

static UINT32 _fgOsdInit = 0;
#ifndef CC_MTK_LOADER
UINT32 osd_log_level = OSD_LL_INFO;
#else
UINT32 osd_log_level = OSD_LL_WARNING;
#endif

static UINT32 _fgTimNfyCnt = 0;
static BOOL OSD2_3DPrLshift_ENABLE_FLAG = FALSE;
static OSD_CPBLTY t_osd_cpblty;

static UINT32 u4_osd_out_v_max      = 0x0;
static UINT32 u4_osd_active_v_start      = 0x0;

static UINT32 u4_osd_under_flow_up  = 0x0;

#define OSD_DBG_INFO_MAX      0x20

UINT32 u4_osd_dbg_info[OSD_DBG_INFO_MAX]  = {0};

#define OSD_FRONT_PORTH_S      u4_osd_out_v_max
#define OSD_ACTIVE_V_S         u4_osd_active_v_start

#define OSD_UNDER_FLOW_ISR_FG  u4_osd_under_flow_up

#ifdef CC_UBOOT
UINT32 _fgOsdEnabelUpdate = 1;
#else
UINT32 _fgOsdEnabelUpdate = 1;
#endif

#if defined(__KERNEL__) && !defined(CC_UBOOT)
#if defined(CC_MT5891) || defined(CC_MT5882) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
static x_os_isr_fct pfnOldIsr;
static BOOL bIsCreateISR = FALSE;
static void vOsd1HeaderUpdateIntCB(UINT16 u2Vector);
static void vOsd2HeaderUpdateIntCB(UINT16 u2Vector);
static void vOsd3HeaderUpdateIntCB(UINT16 u2Vector);
static void vOsdVsyncIntCB(UINT16 u2Vector);
static void OSD_Check_TimingNfy(void);
#endif
#endif

static UINT8 bVyncCouter = 3;
#ifndef __USERDRV__
#ifndef CC_MTK_LOADER
static BOOL bCheckDecStatus[2] = {FALSE};
#endif
#endif
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
static INT32 OSD_InitRegBase(void);
#endif
static void OSD_InitCommon(void);
static INT32 OSD_InitClockSel(void);
#ifndef CC_MTK_LOADER
static INT32 OSD_InitGlue(void);
#endif

#if defined(CC_MT5893) || defined(CC_MT5895)
INT32 i4OsdCropSize = 0x20;   //default is 32
INT32 i4Osd1CropSize = 0x20;
INT32 i4Osd2CropSize = 0x0;
INT32 i4Osd3CropSize = 0x0;
#endif

static void OSD_RestoreLogLevel(void);
static void OSD_RetainLogLevel(void);
static void OSD_BASE_GetStartPosition(INT32 *pi4PosH, INT32 *pi4PosV);

void OSD_Set_Dbg_Info(UINT32 u4_idx, UINT32 u4_val)
{
    if (u4_idx >= OSD_DBG_INFO_MAX)
    {
        return;
    }

    u4_osd_dbg_info[u4_idx] = u4_val;
}

UINT32 OSD_Get_Dbg_Info(UINT32 u4_idx)
{
    if (u4_idx >= OSD_DBG_INFO_MAX)
    {
        return OSD_INVALID_VALUE_U32;
    }

    return u4_osd_dbg_info[u4_idx];
}

void OSD_Reset_UnderFlow(void)
{
    OSD_WRITE32(OSD_FIFO_MONITOR_OFFSET, (OSD_READ32(OSD_FIFO_MONITOR_OFFSET) | (1 << 2)));
    return;
}

UINT32 OSD_Base_Check_Abnormal_Plane(UINT32 u4_plane_base)
{
    INT32 i4_60     = 0x0;
    UINT32 u4_v_s   = 0x0;
    UINT32 u4_4c    = 0x0;
    INT32 i4_60_exp = 0x0;

    if (!(OSD_READ32(u4_plane_base) & 0x1))
    {
        return 0x0;
    }

    if (OSD_READ32(0xFC) & 0x1)
    {
        u4_osd_dbg_info[4]++;
        return 0x1;
    }

    if (!(OSD_READ32(u4_plane_base + 0x38) & 0x800))
    {
        u4_osd_dbg_info[5]++;
        return 0x1;
    }

    if (OSD_PLA3_REG_OFF == u4_plane_base)
    {
        IO_WRITE32(OSD_DE_S_REG, 0, OSD_DE_S_OSD3_VAL);
    }
    else if (OSD_PLA2_REG_OFF == u4_plane_base)
    {
        IO_WRITE32(OSD_DE_S_REG, 0, OSD_DE_S_OSD2_VAL);
    }
    else
    {
        return 0x0;
    }

    if ((OSD_READ32(0x4c) & 0x7FF) >= OSD_FRONT_PORTH_S)
    {
        i4_60 = (OSD_READ32(0x60) & 0x7FF);

        if (OSD_PLA2_REG_OFF == u4_plane_base)
        {
            if ((i4_60 + OSD_DEC_DELAY_MAX) <
                    (OSD_READ32(OSD_SC2_REG_OFF + 0x4) & 0x7FF))
            {
                u4_osd_dbg_info[6]++;
                u4_osd_dbg_info[10] = (OSD_READ32(0x4c) & 0x7FF);
                u4_osd_dbg_info[11] = i4_60;
                return 0x1;
            }
        }
        else if (OSD_PLA3_REG_OFF == u4_plane_base)
        {
            if ((i4_60 + OSD_DEC_DELAY_MAX) <
                    (OSD_READ32(OSD_SC3_REG_OFF + 0x4) & 0x7FF))
            {
                u4_osd_dbg_info[6]++;
                u4_osd_dbg_info[10] = (OSD_READ32(0x4c) & 0x7FF);
                u4_osd_dbg_info[11] = i4_60;
                return 0x1;
            }
        }
        //else
        //{
        //    u4_osd_dbg_info[12] = (OSD_READ32(0x4c) & 0x7FF);
        //    u4_osd_dbg_info[13] = i4_60;
        //}
    }
    else if ((OSD_READ32(0x4c) & 0x7FF) > (OSD_ACTIVE_V_S + OSD_DEC_DELAY_MAX))
    {
        u4_4c   = (OSD_READ32(0x4c) & 0x7FF);
        i4_60   = (OSD_READ32(0x60) & 0x7FF);

        if (OSD_PLA2_REG_OFF == u4_plane_base)
        {
            u4_v_s  = (OSD_READ32(0x504) & 0x7FF);
        }
        else if (OSD_PLA3_REG_OFF == u4_plane_base)
        {
            u4_v_s  = (OSD_READ32(0x604) & 0x7FF);
        }

        i4_60_exp = (INT32)((u4_4c * u4_v_s) / OSD_FRONT_PORTH_S);

        if ((i4_60 == 0x0) || (i4_60 < (i4_60_exp - OSD_DEC_DELAY_MAX)) ||
                (i4_60 > (i4_60_exp + OSD_DEC_DELAY_MAX)))
        {
            u4_osd_dbg_info[14] = i4_60_exp;
            u4_osd_dbg_info[15] = i4_60;
            u4_osd_dbg_info[16] = u4_4c;
            u4_osd_dbg_info[17] = u4_v_s;
            u4_osd_dbg_info[18]++;
            return 0x1;
        }
    }

    if (OSD_READ32(0xFC) & 0x1)
    {
        u4_osd_dbg_info[4]++;
        return 0x1;
    }

    return 0x0;
}

UINT32 OSD_Base_Check_Abnormal_Status(void)
{
    if (OSD_PLA_Get_En(OSD_PLANE_2) && OSD_PLA_Get_Dec(OSD_PLANE_2))
    {
        if (OSD_Base_Check_Abnormal_Plane(OSD_PLA2_REG_OFF))
        {
            return 0x1;
        }
    }

    if (OSD_PLA_Get_En(OSD_PLANE_3) && OSD_PLA_Get_Dec(OSD_PLANE_3))
    {
        if (OSD_Base_Check_Abnormal_Plane(OSD_PLA3_REG_OFF))
        {
            return 0x1;
        }
    }

    return 0x0;
}

void OSD_Reset_FSM_Step1(void)
{
    OSD_WRITE32(0x0FC, (OSD_READ32(0xFC) | (1 << 2)));
    OSD_WRITE32(0x294, (OSD_READ32(0x294) | ((1 << 6))));
    OSD_WRITE32(0x294, (OSD_READ32(0x294) & (~(1 << 4))));
    OSD_WRITE32(0x394, (OSD_READ32(0x394) | ((1 << 6))));
    OSD_WRITE32(0x394, (OSD_READ32(0x394) & (~(1 << 4))));
    OSD_WRITE32(0x4, 0x3);
    OSD_WRITE32(0x4, 0x2);
    return;
}

void OSD_Reset_FSM_Step2(void)
{
    OSD_WRITE32(0x294, (OSD_READ32(0x294) & (~(1 << 6))));
    OSD_WRITE32(0x294, (OSD_READ32(0x294) | ((1 << 4))));
    OSD_WRITE32(0x394, (OSD_READ32(0x394) & (~(1 << 6))));
    OSD_WRITE32(0x394, (OSD_READ32(0x394) | ((1 << 4))));
    OSD_WRITE32(0x4, 0x12);
    OSD_WRITE32(0x4, 0x13);
    OSD_WRITE32(0x0, 0x1);
    return;
}

void OSD_Base_UnderFlow_Isr_Action(void)
{
    if ((OSD_READ32(0xFC) & 0x1))
    {
        if (OSD_UNDER_FLOW_ISR_FG)
        {
            return;
        }

        OSD_Reset_FSM_Step1();
        OSD_UNDER_FLOW_ISR_FG = 0x1;
    }
}

void OSD_Base_Check_Underflow_Isr_Action(void)
{
    if (OSD_UNDER_FLOW_ISR_FG)
    {
        if (OSD_UNDER_FLOW_ISR_FG++ > 0x3)
        {
            OSD_Reset_FSM_Step2();
            OSD_UNDER_FLOW_ISR_FG = 0x0;
        }
    }
}

void OSD_Base_Check_Underflow_Action(void)
{
    static UINT32 u4_delay_cnt      = 0;
    static UINT32 u4_step           = 0x1;

    if (!(u4_osd_dbg_info[0] & 0x10000000))
    {
        return ;
    }

    if (0x1 == u4_step)
    {
        if (OSD_Base_Check_Abnormal_Status())
        {
            OSD_Reset_FSM_Step1();
            u4_delay_cnt    = 0x1;
            u4_step         = 0x2;
            u4_osd_dbg_info[10]++;
        }
    }
    else
    {
        if (u4_delay_cnt)
        {
            if (u4_delay_cnt++ > 0x3)
            {
                OSD_Reset_FSM_Step2();
                u4_delay_cnt    = 0x0;
                u4_step         = 0x1;
            }
        }
    }
}

INT32 OSD_SetCpblty(UINT32 u4_type, UINT32 u4_idx, UINT32 u4_val)
{
    INT32 i4_ret = OSD_RET_OK;
    CHECK_OSD_CPBLTY_PARA(u4_type, u4_idx);

    switch (u4_type)
    {
        case E_OSD_CPBLTY_CMPRSS:
            t_osd_cpblty.u4_cmprss = u4_val;
            break;

        case E_OSD_CPBLTY_CMPRSS_PLANE:
            t_osd_cpblty.u4_cmprss_plane[u4_idx] = u4_val;
            break;

        case E_OSD_CPBLTY_SBS_PLANE:
            t_osd_cpblty.u4_sbs_plane[u4_idx] = u4_val;
            break;

        case E_OSD_CPBLTY_3D_STATUS:
            t_osd_cpblty.u4_3d_status[u4_idx] = u4_val;
            break;

        case E_OSD_CPBLTY_FLIP:
            t_osd_cpblty.u4_flip = u4_val;
            break;

        case E_OSD_CPBLTY_MIRROR:
            t_osd_cpblty.u4_mirror = u4_val;
            break;

        default:
            i4_ret = OSD_RET_INV_ARG;
            break;
    }

    return i4_ret;
}

UINT32 OSD_GetCpblty(UINT32 u4_type, UINT32 u4_idx)
{
    UINT32 u4_val = OSD_INVALID_VALUE_U32;
    CHECK_OSD_CPBLTY_PARA(u4_type, u4_idx);

    switch (u4_type)
    {
        case E_OSD_CPBLTY_CMPRSS:
            u4_val = t_osd_cpblty.u4_cmprss;
            break;

        case E_OSD_CPBLTY_CMPRSS_PLANE:
            u4_val = t_osd_cpblty.u4_cmprss_plane[u4_idx];
            break;

        case E_OSD_CPBLTY_SBS_PLANE:
            u4_val = t_osd_cpblty.u4_sbs_plane[u4_idx];
            break;

        case E_OSD_CPBLTY_3D_STATUS:
            u4_val = t_osd_cpblty.u4_3d_status[u4_idx];
            break;

        case E_OSD_CPBLTY_FLIP:
            u4_val = t_osd_cpblty.u4_flip;
            break;

        case E_OSD_CPBLTY_MIRROR:
            u4_val = t_osd_cpblty.u4_mirror;
            break;

        default:
            break;
    }

    return u4_val;
}

INT32 OSD_SetFb3DCfg(UINT32 u4_plane, UINT32 u4_3d_mode, UINT32 u4_sync)
{
    INT32  i4_ret       = OSD_RET_OK;
#ifndef __USERDRV__
#ifndef CC_MTK_LOADER
#if defined(LINUX_TURNKEY_SOLUTION) && !defined(OSD_INDEPENDENT_VERIFY)
#ifndef CONFIG_TV_DRV_VFY
    OSD_SetCpblty(E_OSD_CPBLTY_3D_STATUS, u4_plane, u4_3d_mode);

    if (u4_sync)
    {
        i4_ret = fb_osd_reupdate_bmp_addr(u4_plane);
    }

#endif
#endif
#endif
#endif
    return i4_ret;
}

void OSD_InitCpblty(void)
{
    x_memset(&t_osd_cpblty, 0, sizeof(t_osd_cpblty));
    {
        OSD_SetCpblty(E_OSD_CPBLTY_CMPRSS_PLANE, OSD_PLANE_1, 0x0);
#ifdef CONFIG_OSD_PLANE2_COMPRESSION
        OSD_SetCpblty(E_OSD_CPBLTY_CMPRSS_PLANE, OSD_PLANE_2, 0x1);
#endif
#ifdef CONFIG_OSD_PLANE3_COMPRESSION
        OSD_SetCpblty(E_OSD_CPBLTY_CMPRSS_PLANE, OSD_PLANE_3, 0x1);
#endif
#if defined(CC_MT5893) || defined(CC_MT5895)
        // not enable dec function when dual ostg output
        if (OSD_IsDualOstgDisplay())
        {
            OSD_SetCpblty(E_OSD_CPBLTY_CMPRSS_PLANE, OSD_PLANE_2, 0x0);
            OSD_SetCpblty(E_OSD_CPBLTY_CMPRSS_PLANE, OSD_PLANE_3, 0x0);
        }
#endif
    }
    {
#ifdef CC_CONFIG_OSD_SBS_PLANE1
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_1, OSD_3D_SBSHALF_PR);
#endif
#ifdef CC_CONFIG_OSD_SBS_PLANE2
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_2, OSD_3D_SBSHALF_PR);
#endif
#ifdef CC_CONFIG_OSD_SBS_PLANE3
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_3, OSD_3D_SBSHALF_PR);
#endif
#ifdef CC_CONFIG_OSD_TB_PLANE1
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_1, OSD_3D_TPANDBT_PR);
#endif
#ifdef CC_CONFIG_OSD_TB_PLANE2
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_2, OSD_3D_TPANDBT_PR);
#endif
#ifdef CC_CONFIG_OSD_TB_PLANE3
        OSD_SetCpblty(E_OSD_CPBLTY_SBS_PLANE, OSD_PLANE_3, OSD_3D_TPANDBT_PR);
#endif
        OSD_SetCpblty(E_OSD_CPBLTY_3D_STATUS, OSD_PLANE_1, OSD_3D_NONE);
        OSD_SetCpblty(E_OSD_CPBLTY_3D_STATUS, OSD_PLANE_2, OSD_3D_NONE);
        OSD_SetCpblty(E_OSD_CPBLTY_3D_STATUS, OSD_PLANE_3, OSD_3D_NONE);
    }
    return ;
}

void OSD_UpdateReg(void)
{
    _OSD_UpdateReg();
}


#ifndef __USERDRV__
void OSD_TimingNfy(void)
{
    _fgTimNfyCnt = 5;
    OSD_LOG_I("OSD timing changed\n");

#if !(defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895))
    //Changing timing while OSD decompression is enabled may cause OSD decompression hang up.
    //we need to reset FSM after OSD decompression hangup
    if (OSD_PLA_Get_En(OSD_PLANE_2) && OSD_PLA_Get_Dec(OSD_PLANE_2))
    {
        bCheckDecStatus[0] = TRUE;
    }

    if (OSD_PLA_Get_En(OSD_PLANE_3) && OSD_PLA_Get_Dec(OSD_PLANE_3))
    {
        bCheckDecStatus[1] = TRUE;
    }
#endif
}

#ifndef CC_MTK_LOADER
static void OSD_Check_TimingNfy(void)
{
    static UINT32 u4DecStatus[2] = {0};

    //if (OSD_READ32(0xFC) & 0x1)
    //{
    //    LOG(0, "===>check abnormal status:0x210FC=0x%x\n", OSD_READ32(0xFC));
    //}

    if (_fgTimNfyCnt > 0)
    {
        _fgTimNfyCnt--;
    }

    if (_fgTimNfyCnt == 3)
    {
        if (bCheckDecStatus[0])
        {
            //OSD2 decompression status
            IO_WRITE32(IO_VIRT + 0x8094, 0, 0x90);
            u4DecStatus[0] = IO_READ32MSK(OSD_BASE_REG, 0x60, 0xF);
        }
        if (bCheckDecStatus[1])
        {
            //OSD3 decompression status
            IO_WRITE32(IO_VIRT + 0x8094, 0, 0xd0);
            u4DecStatus[1] = IO_READ32MSK(OSD_BASE_REG, 0x60, 0xF);
        }
        OSD_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) | UPDATE_TIMING_BIT);
    }
    else if (_fgTimNfyCnt == 2)
    {
        BOOL bDoFsmReset = FALSE;
        if (bCheckDecStatus[0])
        {
            IO_WRITE32(IO_VIRT + 0x8094, 0, 0x90);
            //21060 would change in each vsync if decompression is working
            if (u4DecStatus[0] == 2 && IO_READ32MSK(OSD_BASE_REG, 0x60, 0xF) == 2)
            {
                bDoFsmReset = TRUE;
            }
            bCheckDecStatus[0] = FALSE;
        }
        if (bCheckDecStatus[1])
        {
            IO_WRITE32(IO_VIRT + 0x8094, 0, 0xd0);
            if (u4DecStatus[1] == 2 && IO_READ32MSK(OSD_BASE_REG, 0x60, 0xF) == 2)
            {
                bDoFsmReset = TRUE;
            }
            bCheckDecStatus[1] = FALSE;
        }
        if (bDoFsmReset)
        {
            IO_WRITE32MSK(OSD_BASE_REG, 0x4, 0x0, 0x1);
            IO_WRITE32MSK(OSD_BASE_REG, 0x4, 0x1, 0x1);
            IO_WRITE32MSK(OSD_BASE_REG, 0x0, 0x1, 0x1);
            OSD_LOG_W("OSD decompression hang, do FSM reset\n");
        }
        OSD_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) & (~UPDATE_TIMING_BIT));
        _OSD_UpdateReg();
    }
}
#endif
#endif

void OSD_LockFlipTo(void)
{
#ifndef CC_MTK_LOADER
    VERIFY(x_sema_lock(_hOsdFlipToSema, X_SEMA_OPTION_WAIT) == OSR_OK);
#endif
}

void OSD_UnLockFlipTo(void)
{
#ifndef CC_MTK_LOADER
    VERIFY(x_sema_unlock(_hOsdFlipToSema) == OSR_OK);
#endif
}

void OSD_LockLogoDisplaying(void)
{
#ifndef CC_MTK_LOADER
    //VERIFY(x_sema_lock(_hOsdLogoSema, X_SEMA_OPTION_WAIT) == OSR_OK);
#endif
}

void OSD_UnLockLogoDisplaying(void)
{
#ifndef CC_MTK_LOADER
    //VERIFY(x_sema_unlock(_hOsdLogoSema) == OSR_OK);
#endif
}

void vSetOsdFlipMirror(BOOL fgMirrorEn, BOOL fgFlipEn)
{
    OSD_SetCpblty(E_OSD_CPBLTY_MIRROR, 0, fgMirrorEn);
    // OSD_LOG_D("vSetOsdFlipMirror FLIP \n");
    OSD_SetCpblty(E_OSD_CPBLTY_FLIP, 0, fgFlipEn);
    _OSD_RGN_AllFlipMirrorMode();
    return;
}

#ifndef CC_FLIP_MIRROR_FROM_DDI
#include "drv_display.h"
static void _OsdGetMirrorFlipSetting(void)
{
    UINT8 u1Config = 0x0;
#if defined(DRV_USE_EXTERNAL_3D_FRC) // MT8283_FLIP_MIRROR
    u1Config = ((u4GetFlipMirrorModule(SV_VP_MAIN) == FLIP_BY_EXT3DFRC) ||
                (u4GetFlipMirrorModule(SV_VP_MAIN) == FLIP_NONE)) ? 0 : u1GetFlipMirrorConfig();
#elif defined(CC_FLIP_MIRROR_SUPPORT)
    u1Config = u1GetFlipMirrorConfig();
#endif

    if (u1Config & SYS_FLIP_CONFIG_ON)
    {
        // OSD_LOG_D("_OsdGetMirrorFlipSetting FLIP \n");
        OSD_SetCpblty(E_OSD_CPBLTY_FLIP, 0, TRUE);
    }
    else
    {
        //OSD_LOG_D("_OsdGetMirrorFlipSetting FLIP \n");
        OSD_SetCpblty(E_OSD_CPBLTY_FLIP, 0, FALSE);
    }

    if (u1Config & SYS_MIRROR_CONFIG_ON)
    {
        OSD_SetCpblty(E_OSD_CPBLTY_MIRROR, 0, TRUE);
    }
    else
    {
        OSD_SetCpblty(E_OSD_CPBLTY_MIRROR, 0, FALSE);
    }

    return;
}
#endif

//-----------------------------------------------------------------------------
// Inter-file functions
//-----------------------------------------------------------------------------

INT32 OSD_BASE_SetClock(UINT32 u4Clock)
{
    IO_WRITE32MSK(OSD_CK_REG, 0, u4Clock, 0xf);
    return (INT32) OSD_RET_OK;
}

VOID OSD_BASE_SetPoclk(BOOL fgEnable)
{
    // 0xd368[22]=1 and 0xd368[7]=0 to select poclk
    IO_WRITE32MSK(OSD_POCLK_REG, 0, fgEnable << 22, BIT22);
}

VOID OSD_BASE_SetEclk(UINT32 u4Clock)
{
    // 0xd368[22]=1 and 0xd368[7]=0 to select poclk
    IO_WRITE32MSK(OSD_CLOCK_BASE, 0x368, u4Clock << 30, 0xC0000000U);
}


#if defined(__KERNEL__) && !defined(CC_UBOOT)
#include <linux/time.h>
#endif
void OSD_HandleVSync(void)
{
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    struct timespec t;
    UINT64 timestamp;

    t.tv_sec = t.tv_nsec = 0;
    getrawmonotonic(&t);
    timestamp = (UINT64)(t.tv_sec*1000000000LL + t.tv_nsec);
    OSD_EXP_UpdateVsyncTime(&timestamp);
    OSD_EXP_HandleVSync();
#endif
}

INT32 OSD_Base_AdjustOSDPosition(INT32 i4Adjust)
{
    i4Hstart += i4Adjust;
    OSD_BASE_AdjustPlanePosition(u4OSDTopLayer, u4OSDMiddLayer, u4OSDBotLayer, i4Hstart, 2);
    _OSD_BASE_SetScrnHStartCSR(i4Hstart - 0x6);
    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

INT32 OSD_Base_AdjustOSDVPosition(INT32 i4Adjust)
{
    UINT32 i4Vstart = 0;
    _OSD_BASE_GetScrnVStart(&i4Vstart);
    i4Vstart += i4Adjust;
    _OSD_BASE_SetScrnVStart(i4Vstart);
    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

#if defined(CC_MT5890) || defined(CC_MT5883)
VOID OSD_BASE_FSC_AdjustOffset(INT32 *pi4PosH, INT32 *pi4PosV)
{
    INT16 byp_delay_tab[OSD_BYPASS_MAX][2] =
    {
        { 59,  3}, //BYPASS_SHARPNESS
        { 27,  1}, //BYPASS_CBE_CTL
        { 21,  0}, //BYPASS_BITPLUS_Y
        { 21,  0}, //BYPASS_BITPLUS_S
        { 4,   0}, //BYPASS_3DCOLORMAP
#ifdef CC_FBM_SUPPORT_SRS
        { -367,   0}, //BYPASS_SRS
#else
        { 0,   0}, //BYPASS_SRS
#endif
#ifdef CC_MT5883
        { 0,   0}, //BYPASS_TTD
#else
        { 288, 0}, //BYPASS_TTD
#endif
    };

#ifndef CC_MTK_LOADER
    ASSERT((pi4PosH != NULL));
    ASSERT((pi4PosV != NULL));

    if (u1DrvSharpnessIsBypass())
    {
        OSD_LOG_I("[OSD]DBG u1DrvSharpnessIsBypass\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_SHARPNESS][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_SHARPNESS][1];
    }

    if (u1DrvCBEIsBypass())
    {
        OSD_LOG_I("[OSD]DBG u1DrvCBEIsBypass\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_CBE_CTL][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_CBE_CTL][1];
    }

    if (u1DrvBitPlusYIsBypass())
    {
        OSD_LOG_I("[OSD]DBG u1DrvBitPlusYIsBypass\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_BITPLUS_Y][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_BITPLUS_Y][1];
    }

    if (u1DrvBitPlusSIsBypass())
    {
        OSD_LOG_I("[OSD]DBG u1DrvBitPlusSIsBypass\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_BITPLUS_S][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_BITPLUS_S][1];
    }

    if (u1Drv3DCMSIsBypass())
    {
        OSD_LOG_I("[OSD]DBG u1Drv3DCMSIsBypass\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_3DCOLORMAP][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_3DCOLORMAP][1];
    }

    if (OSD_EXP_IS_vertical_sharpness_bypass())
    {
        *pi4PosH -= 3;
        *pi4PosV -= 3;
    }
    if(OSD_EXP_IS_BYPASSTTD())
#endif
#if !defined(CC_MTK_LOADER) || !defined(SMALL_OUTWINDOW_TTD_NEWSETTING)
    {
        OSD_LOG_I("[OSD]DBG bypass TTD\n");
        *pi4PosH -= byp_delay_tab[OSD_BYPASS_TTD][0];
        *pi4PosV -= byp_delay_tab[OSD_BYPASS_TTD][1];
    }
#endif

#ifdef CC_FBM_SUPPORT_SRS
{    if(! IO32ReadFldAlign(SRS_CTRL, C_SRS_BYPASS))
    {
        *pi4PosH -= byp_delay_tab[BYPASS_SRS][0];
        *pi4PosV -= byp_delay_tab[BYPASS_SRS][1];
    }
}
#else
    UNUSED(byp_delay_tab);
#endif

}
#endif

static void OSD_BASE_GetStartPosition(INT32 *pi4PosH, INT32 *pi4PosV)
{
    OSD_START_POS_T position;

    IGNORE_RET(_OSD_BASE_SetHsEdge(0));
    IGNORE_RET(_OSD_BASE_SetVsEdge(0));

    x_memset(&position, 0, sizeof(OSD_START_POS_T));
    vDrvGetOsdStartPosition(&position);
    if (position.i4VsEdge == 1)
    {
        IGNORE_RET(_OSD_BASE_SetVsEdge(1));
    }
    *pi4PosH = position.i4Hstart;
    *pi4PosV = position.i4Vstart;
}

EXTERN UINT32 DRVCUST_Get_BE_Type(void);
static VOID OSD_BASE_PSCTimingAdjustPosition(INT32 *pi4PosH, INT32 *pi4PosV)
{
    UINT32 u4HBackPorch  = 0x0;
    INT32 i4AdjustH = 0;
    INT32 i4AdjustV = 0;

    if (pi4PosH == NULL || pi4PosV == NULL)
    {
        OSD_LOG_E("Error input parameters: null pointer\n");
        return ;
    }

    IGNORE_RET(_OSD_BASE_SetHsEdge(OSD_BASE_HS_EDGE));
    //IGNORE_RET(_OSD_BASE_SetVsEdge(OSD_BASE_VS_EDGE));

    //if (u1DrvIsPnl4k2k60WithMemc())
    {
        IGNORE_RET(_OSD_BASE_SetVsEdge(OSD_BASE_SYNC_EDGE_RISING)); //PHA panel_set + panel less
        i4AdjustV = 0x5c;
    }
    /*else
    {
        IGNORE_RET(_OSD_BASE_SetVsEdge(OSD_BASE_VS_EDGE)); //APH panel less
        i4AdjustV = 0x3;
    }*/

    u4HBackPorch = wDrvGetHsyncBp();
    i4AdjustH = u4HBackPorch +  OSD_BASE_HSART_ADJ;
    OSD_LOG_D("Get wDrvGetHsyncBp [%d] \n", wDrvGetHsyncBp());

    if (IS_IC_5893())
    {
        if (OSD_IsDualOstgDisplay() && IS_IC_5893_ES1()) // tmp debug value
        {
            i4AdjustH += 1;
            i4AdjustV = 1;
        }
    }
#if defined(CC_MT5895)
    else if (IS_IC_5895())
    {
        i4AdjustH = i4AdjustH - 0x7;
        //in 5895,VRR will auto change vsync sel between 0 and 1.
        #if defined(CC_MT5895)
        if(!u1PscGetOsdVsyncSel())
        {
            i4AdjustV = u4PscGetPanelVSyncBp();
            OSD_LOG_D("Get u4PscGetPanelVSyncBp [%d] \n", u4PscGetPanelVSyncBp());
        }
        #endif
        if (OSD_IsDualOstgDisplay())
        {
            if (PANEL_Is4K2K120())
            {
                INT32 OSD_Offset = 0x40;
                IGNORE_RET(_OSD_BASE_SetVsEdge(OSD_BASE_SYNC_EDGE_RISING));
                if (!(OSD_Get_Dbg_Info(26)) && OSD_PLA_GetCropEnable()) //debug for 4K120.
                {
                    i4AdjustH = 0xAB;
                }
                else
                {
                    i4AdjustH = 0xAC;

                }
                i4AdjustV = 0x20;

                if (OSD_PLA_GetCropEnable())
                {
                    OSD_SetCropingSize(0x23 - OSD_Offset);
                }
                else
                {
                    #if defined(CC_MTK_LOADER) || defined(CC_UBOOT)
                    OSD_SetCropingSize(0x22 - 0x20); //for 4K120 upgrade logo position
                    OSD_LOG_D("[osd debug] OSD_SetCropingSize  0x22 - 0x20 \n");
                    #else
                    OSD_LOG_D("[osd debug]OSD_SetCropingSize  0x22  \n");
                    OSD_SetCropingSize(0x22);  //crop ovl=2
                    #endif
                }
            }
        }
    }
#endif
    *pi4PosH += i4AdjustH;
    *pi4PosV += i4AdjustV;
}


VOID OSD_BASE_UpdateAutoPosition(VOID)
{
    INT32 i4H = 0, i4V = 0;
    OSD_TIMING_SRC_T u4TimingFrom;

    if (_fgOsdInit)
    {
        // use spin_lock_irqsave to protect update pos because this may
        // be called within vdo isr[cannot use x_sema_lock]
#ifndef CC_MTK_LOADER
        spin_lock_irqsave(&_lockOsdPos, _lockOsdPosFlag);
#endif
        if (fgIsMJCToOSTG() && !IS_DISP_4K2K_TYPE3)
        {
            u4TimingFrom = OSD_TIMING_POST_SCALER;
            if (OSD_EXP_GetOSDMixPreMJCEn())
                u4TimingFrom = OSD_TIMING_FRONT_SCALER;
        }
        else
        {
            if (OSD_EXP_IS_GFX())
            {
                u4TimingFrom = OSD_TIMING_GFX;
            }
            else
            {
                u4TimingFrom = OSD_TIMING_FRONT_SCALER;
            }
        }

        #if defined(CC_MT5895)
        OSD_LOG_D(" OSD reg always update enable/disable \n");
        IO_WRITE32MSK(OSD_BASE, 0, 2, 2);    //recover shadow invalid
        if (OSD_TIMING_POST_SCALER == u4TimingFrom)
        {
            _OSD_BASE_SetSrcTimingSel(OSD_TIMING_SRC_PSC);
        }
        else
        {
            _OSD_BASE_SetSrcTimingSel(OSD_TIMING_SRC_FSC);
        }
        IO_WRITE32MSK(OSD_BASE, 0, 0, 2);
        #endif

        if (u4TimingFrom == OSD_TIMING_POST_SCALER)
        {
            OSD_LOG_D("osd timing is from post scaler \n");
            OSD_BASE_PSCTimingAdjustPosition(&i4H, &i4V);
        }
        else
        {
            if (u4TimingFrom == OSD_TIMING_FRONT_SCALER)
            {
                OSD_LOG_D("osd timing is from front scaler\n");
            }
            else
            {
                OSD_LOG_D("osd timing is from gfx timimg gen\n");
            }
#if defined(CC_S_PLATFORM) && (defined(CC_HK_CUST) || defined(CC_CP_CUST))
            if(IS_DISP_4K2K_TYPE3) //GFX separate mode
            {
                IGNORE_RET(_OSD_BASE_SetHsEdge(0));
                IGNORE_RET(_OSD_BASE_SetVsEdge(0));
                #if defined(CC_MT5893)
                #ifndef CC_MTK_LOADER
                if(GetPANEL4K2K_TYPE() == PANEL_TYPE_2K1K_120)
                {
                    i4H = 378;
                }
                else
                #endif
                {
                    #if defined(CC_MT5895)
                    i4H = 376;
                    #else
                    i4H = 379;
                    #endif
                }
                i4V = 36;
                #endif

                #if defined(CC_MT5895)
                if(((DRVCUST_Get_BE_Type() == DRV_CUST_BE_TYPE_S_4K_Y20)
                    || (DRVCUST_Get_BE_Type() == DRV_CUST_BE_TYPE_S_8K_Y20))
                    && !IS_IC_5895_ES1())
                {
                    i4H = 376;
                    i4V = 82;
                }
                else
                {
                    i4H = 376;
                    i4V = 36;
                }
                #endif
            }
            else
#endif
            {
                OSD_BASE_GetStartPosition(&i4H, &i4V);
            }
        }

        //if vertical start position is 0, OSD would flicker
        if(i4V == 0x0) i4V = 0x1;
        OSD_LOG_D("HStart=%d VStart=%d\n", i4H, i4V);
        OSD_BASE_AdjustPlanePosition(u4OSDTopLayer, u4OSDMiddLayer, u4OSDBotLayer, i4H, 2);
        _OSD_BASE_SetScrnHStartCSR(i4H - 0x6);
#if defined(CC_MT5893) || defined(CC_MT5895)
        // tmp value
        if (OSD_IsDualOstgDisplay()/* && IS_IC_5893_ES1()*/)
        {
            _OSD_R_BASE_SetScrnHStartCSR(i4H - 0x6 + i4OsdCropSize);
        }
#endif
        _OSD_BASE_SetScrnVStart(i4V);
        i4Hstart = i4H;
#ifndef CC_MTK_LOADER
        spin_unlock_irqrestore(&_lockOsdPos, _lockOsdPosFlag);
#endif
    }
}

INT32 OSD_Base_GetOffset(UINT32 *pu4H, UINT32 *pu4V)
{
    switch (u4OSDMiddLayer)
    {
        case OSD_PLANE_1:
            _OSD_BASE_GetScrnHStartOsd1(pu4H);
            break;

        case OSD_PLANE_2:
            _OSD_BASE_GetScrnHStartOsd2(pu4H);
            break;

        case OSD_PLANE_3:
            _OSD_BASE_GetScrnHStartOsd3(pu4H);
            break;

        default:
            break;
    }
    _OSD_BASE_GetScrnVStart(pu4V);
    return (INT32)OSD_RET_OK;
}

VOID OSD_Base_UpdateSetData(VOID)
{
    OSD_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) | UPDATE_TIMING_BIT);
    OSD_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) & (~UPDATE_TIMING_BIT));
#ifdef CC_MT5895
    OSD_R_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) | UPDATE_TIMING_BIT);
    OSD_R_WRITE32(OSD_BASE_MONITOR, OSD_READ32(OSD_BASE_MONITOR) & (~UPDATE_TIMING_BIT));
#endif
}

INT32 OSD_Base_SetHStart(void)
{
    if (!_fgOsdInit)
    {
        return (INT32) OSD_RET_OK;
    }

    OSD_EXP_InitOsdPath();
    _OSD_BASE_SetSyncSrcSel(OSD_BASE_SYNC_SRC_SEL);
    OSD_BASE_UpdateAutoPosition();
    OSD_Base_UpdateSetData();
    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

VOID OSD_BASE_AutoAlignPosition(VOID)
{
    UINT32 i4H, i4V;
    INT32 i4NewH, i4NewV;
    INT32 i4OutputWidth, i4OutputHeight;
    INT32 i4HOffset, i4VOffset, i4HDiff, i4VDiff;
    INT32 u4PixelCnt_O, u4PixelCnt_H, u4PixelCnt_V;

    i4OutputWidth  = OSD_BASE_GetDisplayWidth();
    i4OutputHeight = OSD_BASE_GetDisplayHeight();
    OSD_Base_GetOffset(&i4H, &i4V);
    OSD_LOG_D("OSD output size: %d x %d\n", i4OutputWidth, i4OutputHeight);

    vIO32WriteFldAlign(OSD_MIX_2, 0x1, REG_OSD_INDEX_EN);
    // Main path use only, graphic path can't use it.
    u4PixelCnt_O = IO32ReadFldAlign(OSD_MIX_2, STA_OSD_INDEX_CNT); // Original

    if (u4PixelCnt_O == (i4OutputWidth * i4OutputHeight))
    {
        OSD_LOG_E("OSD position is already correct!\n");
        return;
    }

    OSD_Base_AdjustOSDPosition(1);
    x_thread_delay(50); // For waiting counter update
    u4PixelCnt_H = IO32ReadFldAlign(OSD_MIX_2, STA_OSD_INDEX_CNT); // H shift 1 pixel

    OSD_Base_AdjustOSDPosition(-1);
    OSD_Base_AdjustOSDVPosition(1);
    x_thread_delay(50); // For waiting counter update
    u4PixelCnt_V = IO32ReadFldAlign(OSD_MIX_2, STA_OSD_INDEX_CNT); // V shift 1 pixel

    i4VOffset = u4PixelCnt_H - u4PixelCnt_O;
    i4HOffset = u4PixelCnt_V - u4PixelCnt_O;

    i4HDiff = DIFF( i4OutputWidth, ABS(i4HOffset));
    i4VDiff = DIFF(i4OutputHeight, ABS(i4VOffset));

    i4NewH = (ABS(i4HDiff) % i4OutputWidth ) ? ((i4VOffset > 0) ? (i4HDiff) : (-i4HDiff)) : 0;
    i4NewV = (ABS(i4VDiff) % i4OutputHeight) ? ((i4HOffset > 0) ? (i4VDiff) : (-i4VDiff)) : 0;

    if ((i4NewH < 0 && (ABS(i4NewH) > i4H)) || (i4NewV < 0 && (ABS(i4NewV) > i4V)))
    {
        OSD_LOG_E("Error H or V offset will < 0 !!\n");
        return;
    }

    i4NewV--; // already lower shift 1 pixel.
    OSD_Base_AdjustOSDPosition (i4NewH);
    OSD_Base_AdjustOSDVPosition(i4NewV);

    OSD_LOG_D("Old Hstart:0x%X, VStart:0x%X\n", i4H, i4V);
    OSD_Base_GetOffset(&i4H, &i4V);
    OSD_LOG_D("New Hstart:0x%X, VStart:0x%X\n", i4H, i4V);
}

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

INT32 OSD_BASE_SetYuvOutput(BOOL fgYuvOutput)
{
    IGNORE_RET(_OSD_BASE_SetYuvOut(fgYuvOutput));
    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

#ifndef CC_MTK_LOADER
#ifdef CC_SUPPORT_STR
#if defined(CC_MT5893) || defined(CC_MT5895)
#define MAX_OSD_MODULE_NUM          2
#else
#define MAX_OSD_MODULE_NUM          1
#endif
static UINT32 *pm_save_osd_regs[MAX_OSD_MODULE_NUM] = {NULL};

#define OSD_MODULE_REG_SIZE (0x800 - 4)

#define OSD_FMT_REG_SIZE (4+(0xb8-8))
#define OSD1_REG_SIZE (4+(0x178-0x100))
#define PMX_RRG_BASE (IO_VIRT + 0x27984)
#define OSD2_REG_SIZE (4+(0x284-0x200))
#define OSD3_REG_SIZE (4+(0x384-0x300))
#define OSD1SC_REG_SIZE (4+(0x4d8-0x400))
#define OSD2SC_REG_SIZE (4+(0x5d8-0x500))
#define OSD3SC_REG_SIZE (4+(0x6d8-0x600))
#define OSD_CSR_REG_SIZE (4+(0x73c-0x700))

#define OSTAG_OSDRESET_REG   (IO_ADDR(0xd1c4))
#define OSTAG_OSDRESET_BIT   (1 << 2)
#define OSTAG_OSDCLOCK_REG   (IO_ADDR(0xd1cc))
#define OSTAG_OSDCLOCK_BIT   (1 << 2)

#define SCALER_SETDATA_REG   (IO_ADDR(0x25304))
#define SCALER_SETDATA_BIT   (1 << 30)

#define IS_NULLPTR(ptr)      (NULL == (ptr))
#define FREE_IS_VALID(ptr)    if(ptr) x_mem_free((ptr))

#if defined(CC_LG_SNAP_SHOT)
#include <linux/timer.h>
static DEFINE_TIMER(defered_resume_timer, NULL, 0, (unsigned long)NULL);
static void OSD_pm_resume_core(void);
static void OSD_defered_pm_resume_handler(unsigned long data)
{
    OSD_pm_resume_core();
}
#endif
static INT32 OSD_RegToMem(UINT32 u4RegFrom, UINT32* u4MemTo, UINT32 u4Size)
{
    UINT32 i, j;
    if((u4RegFrom == 0)||(u4MemTo == NULL)||(u4Size == 0))
    {
        OSD_LOG_E("OSD_RegToMem param error,return\n");
        return -1;
    }
    j=0;
    for(i=0; i<u4Size;)
    {
        u4MemTo[j] = IO_READ32(u4RegFrom, i);
        j++;
        i += 4;
    }
    return 0;
}

static INT32 OSD_MemToReg(UINT32* u4MemFrom, UINT32 u4RegTo, UINT32 u4Size)
{
    UINT32 i, j;
    if((u4MemFrom == NULL)||(u4RegTo == 0)||(u4Size == 0))
    {
        OSD_LOG_E("OSD_MemToReg param error,return\n");
        return -1;
    }
    i=0;
    for(j=0; j<u4Size; )
    {
        IO_WRITE32(u4RegTo, j, u4MemFrom[i]);
        j += 4;
        i++;
    }
    return 0;
}

void OSD_pm_suspend(void)
{
    if (b_suspend)
    {
        return;
    }
    OSD_EXP_HandleResumeSuspend(TRUE);
    pm_save_osd_regs[0] = x_mem_alloc(OSD_MODULE_REG_SIZE);
#if defined(CC_MT5893) || defined(CC_MT5895)
    pm_save_osd_regs[1] = x_mem_alloc(OSD_MODULE_REG_SIZE);
#endif

#if defined(CC_MT5893) || defined(CC_MT5895)
    if (IS_NULLPTR(pm_save_osd_regs[0]) || IS_NULLPTR(pm_save_osd_regs[1]))
#else
    if (IS_NULLPTR(pm_save_osd_regs[0]))
#endif
    {
        FREE_IS_VALID(pm_save_osd_regs[0]);
#if defined(CC_MT5893) || defined(CC_MT5895)
        FREE_IS_VALID(pm_save_osd_regs[1]);
#endif
        return;
    }
    else
    {
        b_suspend = TRUE;
        OSD_RegToMem(OSD_BASE + 0x4, pm_save_osd_regs[0], OSD_MODULE_REG_SIZE);
        #if defined(CC_MT5893) || defined(CC_MT5895)
        OSD_RegToMem(OSD_RT_BASE + 0x4, pm_save_osd_regs[1], OSD_MODULE_REG_SIZE);
        #endif
    }

    //disable OSD1,2,3
    IO_WRITE32_CLEAR(OSD_PLA1_REG, 0, 0);
#if !defined(CC_LG_SNAP_SHOT)
    IO_WRITE32_CLEAR(OSD_PLA2_REG, 0, 0);
    IO_WRITE32_CLEAR(OSD_PLA3_REG, 0, 0);
    OSD_BASE_SetClock(OSD_CK_XTAL);
    //reset OSD clock
    IO_WRITE32_CLEAR(IO_ADDR(0xd1c4), 0, 1);
    //disable OSD clock
    IO_WRITE32_CLEAR(IO_ADDR(0xd1cc), 0, 1);
#endif
    _OSD_UpdateReg();
    OSD_EXP_BakUpMIX();

#ifdef CC_S_PLATFORM
#ifndef CC_MTK_LOADER
    fgLogoInit = FALSE;//for osd show
#endif
#endif
}

static void OSD_pm_resume_core(void)
{
    bVyncCouter = 3;  // resume for OSDIsPlaying()

    if (!b_suspend)
    {
        return;
    }
    //enable OSD clock
    IO_WRITE32_SET(IO_ADDR(0xd1cc), 0, 1);
    //reset OSD clock
    IO_WRITE32_SET(IO_ADDR(0xd1c4), 0, 1);
    OSD_InitClockSel();

    OSD_MemToReg(pm_save_osd_regs[0], (UINT32)(OSD_BASE+0x4), OSD_MODULE_REG_SIZE);
#if defined(CC_MT5893) || defined(CC_MT5895)
    OSD_MemToReg(pm_save_osd_regs[1], (UINT32)(OSD_RT_BASE+0x4), OSD_MODULE_REG_SIZE);
#endif
    OSD_WRITE32MSK(OSD_BASE_MONITOR, 1, UPDATE_TIMING_BIT);
    OSD_WRITE32MSK(OSD_BASE_MONITOR, 0, UPDATE_TIMING_BIT);
#if defined(CC_MT5893) || defined(CC_MT5895)
    OSD_R_WRITE32MSK(OSD_BASE_MONITOR, 1, UPDATE_TIMING_BIT);
    OSD_R_WRITE32MSK(OSD_BASE_MONITOR, 0, UPDATE_TIMING_BIT);
#endif
    OSD_EXP_ReSumeMix();
#if defined(CC_MT5893) || defined(CC_MT5895)
    if (OSD_IsOstg2Display())
    {
        _OSD_BASE_UpdateHwReg();
        _OSD_PLA_UpdateHwReg(OSD_PLANE_1);
        _OSD_PLA_UpdateHwReg(OSD_PLANE_2);
        _OSD_PLA_UpdateHwReg(OSD_PLANE_3);
        _OSD_PLA_UpdateHwReg(OSD_PLANE_5);
        _OSD_SC_UpdateHwReg(OSD_SCALER_1);
        _OSD_SC_UpdateHwReg(OSD_SCALER_2);
        _OSD_SC_UpdateHwReg(OSD_SCALER_3);
    }
#endif
    _OSD_UpdateReg();

    b_suspend = FALSE;
    x_mem_free(pm_save_osd_regs[0]);
#if defined(CC_MT5893) || defined(CC_MT5895)
    x_mem_free(pm_save_osd_regs[1]);
#endif
#ifdef CC_FASTBOOT_DRIVER_LOGO
    //disable OSD2,3
    OSD_PLA_Enable(OSD_PLANE_2, FALSE);
    OSD_PLA_Enable(OSD_PLANE_3, FALSE);
#endif
#ifdef CC_FLIP_MIRROR_SUPPORT
#ifndef CC_LG_SNAP_SHOT
    _OsdGetMirrorFlipSetting();
#endif
    _OSD_RGN_AllFlipMirrorMode();
#endif
    OSD_EXP_HandleResumeSuspend(FALSE);
}

//for do something while OSD not in suspend status
BOOL OSD_Base_Suspend_Status(void)
{
    return b_suspend;
}

//for do something the first time when OSD finish resuming
BOOL OSD_Get_ResumeStatus(void)
{
    return bFinishResume;
}

void OSD_Clear_ResumeStatus(void)
{
    bFinishResume = FALSE;
}

#if defined(CC_S_PLATFORM) && (defined(CC_WK_CUST) || defined(CC_HK_CUST) || defined(CC_CP_CUST))
#ifndef CC_MTK_LOADER
extern void vDrvOSDScalerConfig(void);
#endif
#endif
void OSD_pm_resume(void)
{
#if defined(CC_LG_SNAP_SHOT)
    extern int is_snapshot_making;

    if (!is_snapshot_making)
    {
        defered_resume_timer.expires = get_jiffies_64() + 3 * HZ;
        defered_resume_timer.function = OSD_defered_pm_resume_handler;
        add_timer(&defered_resume_timer);
    }
    else
    {
        OSD_pm_resume_core();
    }

#else
    OSD_pm_resume_core();
#endif
#if defined(CC_S_PLATFORM) && (defined(CC_WK_HUST) || defined(CC_HK_CUST) || defined(CC_CP_CUST))
#ifndef CC_MTK_LOADER
    if (!IS_DISP_4K2K_TYPE3)
    {
        OSD_Base_SetHStart();
        vDrvOSDScalerConfig();
    }
#endif
#endif
    //b_suspend = FALSE;
    bFinishResume = TRUE;
}
#else
//for do something while OSD not in suspend status
BOOL OSD_Base_Suspend_Status(void)
{
    return FALSE;
}

//for do something the first time when OSD finish resuming
BOOL OSD_Get_ResumeStatus(void)
{
    return FALSE;
}

void OSD_Clear_ResumeStatus(void)
{

}
#endif /*CC_SUPPORT_STR*/
#endif

//-----------------------------------------------------------------------------
/** osd global initialization function
 *  @param  void
 *  @retval OSD_RET_OK success
 */
//-----------------------------------------------------------------------------
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
static INT32 OSD_InitRegBase(void)
{
    _OSD_Base_Init_Reg();
    _OSD_PLA_InitHwReg();
    _OSD_SC_InitHwReg();
#if defined(CC_MT5886) || defined(CC_MT5863)
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    _OSD_MMU_Init_Reg();
#endif
#endif
    return (INT32) OSD_RET_OK;
}
#endif

#ifndef CC_MTK_LOADER
static INT32 OSD_InitGlue(void)
{
#ifdef CC_SUPPORT_OSD_GLUE_INTERFACE
    OSD_Misc_Init();
    OSD_InstallGeneral(OSD_PLANE_1, OSD_SCALER_1);
    OSD_InstallGeneral(OSD_PLANE_2, OSD_SCALER_2);
    OSD_InstallGeneral(OSD_PLANE_3, OSD_SCALER_3);
    OSD_InstallShutter(OSD_PLANE_1, OSD_SCALER_1);
    OSD_InstallShutter(OSD_PLANE_2, OSD_SCALER_2);
    OSD_InstallShutter(OSD_PLANE_3, OSD_SCALER_3);
    OSD_InstallCompress(OSD_PLANE_1, OSD_SCALER_1, NULL, FALSE);
    OSD_InstallCompress(OSD_PLANE_2, OSD_SCALER_2, NULL, FALSE);
    OSD_InstallCompress(OSD_PLANE_3, OSD_SCALER_3, NULL, FALSE);
#ifdef CONFIG_OSD_PLANE1_COMPRESSION
    OSD_GLUE_SetFunction(OSD_PLANE_1, "compress");
#else
    OSD_GLUE_SetFunction(OSD_PLANE_1, "general");
#endif
#ifdef CONFIG_OSD_PLANE2_COMPRESSION
    OSD_GLUE_SetFunction(OSD_PLANE_2, "compress");
#else
    OSD_GLUE_SetFunction(OSD_PLANE_2, "general");
#endif
#ifdef CONFIG_OSD_PLANE3_COMPRESSION
    OSD_GLUE_SetFunction(OSD_PLANE_3, "compress");
#else
    OSD_GLUE_SetFunction(OSD_PLANE_3, "general");
#endif
#endif
    return (INT32) OSD_RET_OK;
}
#endif

static void OSD_InitCommon(void)
{
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
        OSD_InitRegBase();
#endif
#if defined(CC_MT5890) || defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5893)
        //0xf0025350[13:14] SC_OSD_SRC_SEL 0:FSC time gen 1:PSC time gen 2:GFX time gen
        vDrvSwitchOSDRoute();
#endif
        // OSD_Base_Reg_Isr();
#ifndef CC_MTK_LOADER
        // Create OSD Semphore
        /* VERIFY(x_sema_create(&_hOsdLogoSema, X_SEMA_TYPE_MUTEX, */
        /*                   X_SEMA_STATE_UNLOCK) == OSR_OK); */
        VERIFY(x_sema_create(&_hOsdFlipToSema, X_SEMA_TYPE_BINARY,
                          X_SEMA_STATE_UNLOCK) == OSR_OK);
#endif
#if defined(__KERNEL__) && !defined(CC_UBOOT)
#if defined(CC_MT5891) || defined(CC_MT5882) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
        if(!bIsCreateISR)
        {
            //keep the code, but not register isr.
            if (0)
            {
                VERIFY(x_reg_isr(VECTOR_OSD, vOsd1HeaderUpdateIntCB, &pfnOldIsr) == OSR_OK);
                VERIFY(x_reg_isr(VECTOR_OSD2, vOsd2HeaderUpdateIntCB, &pfnOldIsr) == OSR_OK);
                VERIFY(x_reg_isr(VECTOR_OSD3, vOsd3HeaderUpdateIntCB, &pfnOldIsr) == OSR_OK);
            }
            VERIFY(x_reg_isr(VECTOR_OSD_VSYNC, vOsdVsyncIntCB, &pfnOldIsr) == OSR_OK);
            bIsCreateISR = TRUE;
        }
#endif
#endif
        OSD_InitCpblty();
#ifndef CC_FLIP_MIRROR_FROM_DDI
        _OsdGetMirrorFlipSetting();
#endif
#ifndef CC_MTK_LOADER
        OSD_InitGlue();
#endif
#if defined(__KERNEL__) && !defined(CC_UBOOT)
        OSD_InitVsync();
#endif
}

static INT32 OSD_InitClockSel(void)
{
    //Select video path
    //0xf0025350[10:10] SC_OSD_VSYNC_SEL 0:vsync 1:vde
    vIO32WriteFldAlign(SCPIP_SCSYS_14, 0x1, SCSYS_14_SC_OSD_VSYNC_SEL);
    //0xf005e350[11:11] PSC_OSD_HSYNC_SEL 0:hsync 1:hde
    vIO32WriteFldAlign(PSCSYS_14, 0x0, PSC_OSD_HSYNC_SEL);
    //set scaler down clock,if don't set,scaler down won't be effective
    //0xf00d368[30:31] OSD_ECLK_SEL 1x:osd_core_ck
#if defined(CC_MT5895)
    if (u1DrvIsPnl4k2k60WithMemcFF())
    {
        // in 4k60 non BE model, eclk should also select oclk
        OSD_BASE_SetEclk(0);
    }
    else
    {
        OSD_BASE_SetEclk(2);
    }
#else
    IO_WRITE32MSK(OSD_CLOCK_BASE, 0x368, 0x1 << 31, 0x80000000U);
#endif

#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5893) || defined(CC_MT5895)
    OSD_BASE_SetClock(OSD_CK_SYS);
#else
    OSD_BASE_SetClock(OSD_CK_SYS_DIV2);
#endif

    if (IS_IC_5895())
    {
        // set ovl clk src else osd flicker
        // osd use ovl clk to dock with smi clk
        if(IS_IC_5895_ES1())
        {
            //only for 5895 ES1, 5895 ES2 is different.
            IO_WRITE32MSK(SMI_CK_REG, 0, 0x2, 0x3);//smi_ck ES1-432M
            //IO_WRITE32MSK(OVL_CK_SRC, 0, 0x1, 0x7);//ovl_ck 432M
        }
      IO_WRITE32MSK(OVL_CK_SRC, 0, 0x1, 0x7);//ovl_ck 432M
    }
    else if (IS_IC_5893())
    {
        // set smi & ovl clk src else scaler down not work well for husky
        // 0xd250[0]  0xd4d0[2:0]
        IO_WRITE32MSK(SMI_CK_REG, 0, 0x1, 0x3);
        IO_WRITE32MSK(OVL_CK_SRC, 0, 0x6, 0x7);
    }

#if defined(CC_MT5890)
    if (IS_IC_5861())
    {
        OSD_BASE_SetClock(OSD_CK_SYS);
    }
#endif
    return (INT32) OSD_RET_OK;
}

void OSD_SetLogLevel(UINT32 level)
{
    osd_log_level = level;
}

UINT32 OSD_GetLogLevel(void)
{
    return osd_log_level;
}

//to keep osd log level from loader to kernel driver, w 0xf0021038 0x4 after stopping in loader.
//depends on CC_LOADER_LOGO_LONG_TIME feature
static void OSD_RestoreLogLevel(void)
{
    //0xf0021038[0:15] is unused and writable, so use this register to store log level.
    if (IO_READ32MSK(OSD_BASE, 0x38, 0xFFFF) <= 0x4)
    {
        OSD_SetLogLevel(IO_READ32MSK(OSD_BASE, 0x38, 0xFFFF));
    }
}

//after FSM reset, 0xf0021038 will be resetted, should retain the level value after OSD_Reset
//so that could restore log level in OSD_SwReset.
static void OSD_RetainLogLevel(void)
{
    IO_WRITE32MSK(OSD_BASE, 0x38, osd_log_level, 0xFFFF);
}

INT32 OSD_Init(void)
{
    if (_fgOsdInit == 0)
    {
        OSD_InitCommon();
        IGNORE_RET(OSD_Reset());
    #if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            OSD_PLA_InitialInternalRgnList();
        }
    #endif
        _fgOsdInit = 1;
        OSD_Base_SetHStart();
    }

#ifdef __MODEL_slt__
    OSD_LOG_I(" OSD_Reset() for slt\n");
    OSD_Reset();
    OSD_Base_SetHStart(); // set hsync/vsync edge
#endif

#ifdef OSD_BASE_INIT_FRC_HEADER
#if LG_OSD_PATH_TEST
    OSD_BASE_InitFRCHeader();
#endif
#endif
    return (INT32) OSD_RET_OK;
}

INT32 OSD_ReInit(void)
{
    _fgOsdInit = 0;
    return OSD_Init();
}

INT32 OSD_Reset(void)
{
    OSD_RestoreLogLevel();
#ifndef CC_MTK_FPGA
    IGNORE_RET(OSD_InitClockSel());
#endif
    IGNORE_RET(_OSD_BASE_InitRegs());
    IGNORE_RET(_OSD_3D_InitRegs());
#if defined(CC_MT5863) || defined(CC_MT5886)
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    OSD_MMU_Init();
#endif
#endif
    _OSD_RGN_InitApi();
    OSD_PLA_InitAll();
    OSD_SC_InitAll();
    IGNORE_RET(OSD_RGN_LIST_Init());
    IGNORE_RET(OSD_RGN_Init());
#ifdef DRV_SUPPORT_EXTMJC
    OSD_DMA_Init();
#endif
    OSD_EXP_InitOsdPath();
#ifdef SMALL_OUTWINDOW_TTD_NEWSETTING
#if defined(CC_MT5881) || defined(CC_MT5883)
    vIO32WriteFldAlign(TTDPO_00, 0x80, SYNC_DELAY_CYCLES);
    vIO32WriteFldAlign(TTDIR_02, 0x5F, IR_FIFO_LENGTH);
#else
    vIO32WriteFldAlign(TTDPO_01, 0x80, SYNC_DELAY_CYCLES);
    vIO32WriteFldAlign(TTDIR_02, 0x55, IR_FIFO_LENGTH);
#if defined(CC_MT5891)
    vIO32WriteFldAlign(OSTG_00, SV_OFF, C_BYPASS_TTD);
    vIO32WriteFldAlign(OSTG_00, 0x0, C_TTD_BEFORE_SRS);
#endif
#endif
#endif
    OSD_BASE_SetPlaneSwitchOrder(u4OSDTopLayer, u4OSDMiddLayer, u4OSDBotLayer);
    OSD_RetainLogLevel();

#ifndef CC_MTK_LOADER
#if defined(ANDROID_TWO_WORLDS) || defined(ANDROID)
    OSD_PLA_SetPremulEn(OSD_PLANE_3, 1);
#endif
#endif

    return (INT32) OSD_RET_OK;
}

#ifndef CC_MTK_LOADER
VOID OSD_RestorePlaneOrder(void)
{
    UINT32 u4Osd1Switch, u4Osd2Switch, u4Osd3Switch;
    //OSD1_SW top, OSD3_SW middle, OSD2_SW bottom
    _OSD_BASE_GetOSD1Switch(&u4Osd1Switch);
    _OSD_BASE_GetOSD2Switch(&u4Osd2Switch);
    _OSD_BASE_GetOSD3Switch(&u4Osd3Switch);
#ifdef CC_MT5883
    u4OSDTopLayer  = OSD_PLANE_1;
    u4OSDMiddLayer = u4Osd3Switch - 1;
    u4OSDBotLayer  = u4Osd2Switch - 1;
#else
    u4OSDTopLayer  = u4Osd1Switch - 1;
    u4OSDMiddLayer = u4Osd3Switch - 1;
    u4OSDBotLayer  = u4Osd2Switch - 1;
#endif
    switch (u4OSDMiddLayer)
    {
        case OSD_PLANE_1:
            _OSD_BASE_GetScrnHStartOsd1(&i4Hstart);
            break;

        case OSD_PLANE_2:
            _OSD_BASE_GetScrnHStartOsd2(&i4Hstart);
            break;

        case OSD_PLANE_3:
            _OSD_BASE_GetScrnHStartOsd3(&i4Hstart);
            break;

        default:
            break;
    }
}
#endif

INT32 OSD_SwInit(void)
{
    if (_fgOsdInit == 0)
    {
        OSD_InitCommon();
        IGNORE_RET(OSD_SwReset());
    #if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            OSD_PLA_InitialInternalRgnList();
        }
    #endif
        _fgOsdInit = 1;
        //internal will check _fgOsdInit,so invalid after set _fgOsdInit
        OSD_Base_SetHStart();
        //_fgOsdInit = 1;
    }

    return (INT32) OSD_RET_OK;
}

INT32 OSD_SwReset(void)
{
    UINT32 u4Base8Reg = IO_READ32(OSD_BASE_REG, 0x8);

    OSD_RestoreLogLevel();
    //when ABIST_EN(0xf0021008[21:21]) is enabled, OSD should be reseted
    //it is added to resolve issue of displaying blue screen if DC power off TV and wait 70s then DC power on.
    if (u4Base8Reg & 0x200000)
    {
        return OSD_Reset();
    }
    _OSD_BASE_SwSetReg();
#if defined(CC_MT5863) || defined(CC_MT5886)
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    //IOMMU should only be used in linux stage.
    //so init IOMMU here too.
    OSD_MMU_Init();
#endif
#endif
#ifndef CC_MTK_LOADER
    OSD_RestorePlaneOrder();
#endif
    _OSD_RGN_InitApi();
    OSD_PLA_SwResetAll();
    OSD_SC_SwResetAll();
    IGNORE_RET(OSD_RGN_LIST_Init());
    IGNORE_RET(OSD_RGN_Init());
#ifdef DRV_SUPPORT_EXTMJC
    OSD_DMA_Init();
#endif
    OSD_EXP_InitOsdPath();

#ifndef CC_MTK_LOADER
#if defined(ANDROID_TWO_WORLDS) || defined(ANDROID)
    OSD_PLA_SetPremulEn(OSD_PLANE_3, 1);
#endif
#endif

    return (INT32) OSD_RET_OK;
}

//-----------------------------------------------------------------------------
INT32 OSD_BASE_AdjustPlanePosition(UINT32 u4TopPlane, UINT32 u4MidPlane,
                                   UINT32 u4BotPlane, INT32 i4Position, UINT32 delay)
{
    UINT32 u4OSD1Postion = 0;
    UINT32 u4OSD2Postion = 0;
    UINT32 u4OSD3Postion = 0;
    INT32 i4OSDPosition = 0;
    UINT32 SrcWidth = 0;
    UINT32 DstWidth = 0;
    i4OSDPosition = i4Position - (INT32)delay;

    switch (u4TopPlane)
    {
        case OSD_PLANE_2:
            u4OSD2Postion = i4Position + delay;
            break;

        case OSD_PLANE_3:
            u4OSD3Postion = i4Position + delay;
            break;

        case OSD_PLANE_1:
            u4OSD1Postion = i4Position + delay;
            break;

        default:
            u4OSD1Postion = i4Position + delay;
            break;
    }

    switch (u4MidPlane)
    {
        case OSD_PLANE_2:
            u4OSD2Postion = i4Position;
            break;

        case OSD_PLANE_3:
            u4OSD3Postion = i4Position;
            break;

        case OSD_PLANE_1:
            u4OSD1Postion = i4Position;
            break;

        default:
            u4OSD3Postion = i4Position;
            break;
    }

    switch (u4BotPlane)
    {
        case OSD_PLANE_2:
            u4OSD2Postion = (i4OSDPosition < 0) ? i4Position : i4OSDPosition;
            break;

        case OSD_PLANE_3:
            u4OSD3Postion = (i4OSDPosition < 0) ? i4Position : i4OSDPosition;
            break;

        case OSD_PLANE_1:
            u4OSD1Postion = (i4OSDPosition < 0) ? i4Position : i4OSDPosition;
            break;

        default:
            u4OSD2Postion = (i4OSDPosition < 0) ? i4Position : i4OSDPosition;
            break;
    }

    _OSD_BASE_SetScrnHStartOsd1(u4OSD1Postion);
    _OSD_BASE_SetScrnHStartOsd2(u4OSD2Postion);
    _OSD_BASE_SetScrnHStartOsd3(u4OSD3Postion);
#if defined(CC_MT5893) || defined(CC_MT5895)
    // tmp value
    if (OSD_IsDualOstgDisplay()/* && IS_IC_5893_ES1()*/)
    {
        if (!(OSD_Get_Dbg_Info(26))) //debug for 4K120.
        {
            #ifndef CC_MTK_LOADER
            _OSD_SC_GetSrcHSize(OSD_SCALER_1, &SrcWidth);
            _OSD_SC_GetDstHSize(OSD_SCALER_1, &DstWidth);

            if (SrcWidth == 0)
            {
                OSD_LOG_E("[ERROR]OSD_SCALER_1[src : %d,dst : %d]\n", SrcWidth, DstWidth);
                return (INT32) OSD_RET_INV_ARG;
            }

            if (DstWidth / SrcWidth == 2)
            {
                OSD_LOG_D("[debug#New1]OSD_SCALER_1[src : %d,dst : %d]\n",SrcWidth,DstWidth);
               _OSD_R_BASE_SetScrnHStartOsd1(u4OSD1Postion + i4OsdCropSize);
            }
            else
            #endif
            {

                OSD_LOG_D("[debug #New3]OSD_SCALER_1[src : %d,dst : %d]\n",SrcWidth,DstWidth);
                _OSD_R_BASE_SetScrnHStartOsd1(u4OSD1Postion + i4OsdCropSize + i4Osd1CropSize);
            }

            _OSD_R_BASE_SetScrnHStartOsd2(u4OSD2Postion + i4OsdCropSize + i4Osd2CropSize);
            _OSD_R_BASE_SetScrnHStartOsd3(u4OSD3Postion + i4OsdCropSize + i4Osd3CropSize);
        }
        else
        {
            _OSD_R_BASE_SetScrnHStartOsd1(u4OSD1Postion + i4OsdCropSize );
            _OSD_R_BASE_SetScrnHStartOsd2(u4OSD2Postion + i4OsdCropSize );
            _OSD_R_BASE_SetScrnHStartOsd3(u4OSD3Postion + i4OsdCropSize );
        }
    }
#endif
    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_SetPlaneSwitchOrder(UINT32 u4TopPlane, UINT32 u4MidPlane,
                                   UINT32 u4BotPlane)
{
    //OSD1_SW top, OSD3_SW middle, OSD2_SW bottom
    UINT32 u4Osd1Switch = 0x0;
    UINT32 u4Osd2Switch = 0x0;
    UINT32 u4Osd3Switch = 0x0;
    UINT32 u4Osd1PremulAlpha = 0x0;
    UINT32 u4Osd2PremulAlpha = 0x0;
    UINT32 u4Osd3PremulAlpha = 0x0;
#if defined(CC_MT5881)
    UINT32 u4PmxOption ;
#endif

    OSD_PLA_GetPremulEn(OSD_PLANE_1, &u4Osd1PremulAlpha);
    OSD_PLA_GetPremulEn(OSD_PLANE_2, &u4Osd2PremulAlpha);
    OSD_PLA_GetPremulEn(OSD_PLANE_3, &u4Osd3PremulAlpha);

    u4OSDTopLayer  = u4TopPlane;
    u4OSDMiddLayer = u4MidPlane;
    u4OSDBotLayer  = u4BotPlane;
#ifdef CC_MT5883
    u4Osd1Switch = 0;
    u4Osd3Switch = (u4TopPlane == OSD_PLANE_1) ? (u4MidPlane + 1) : (u4TopPlane + 1);
    u4Osd2Switch = (u4TopPlane == OSD_PLANE_1) ? (u4BotPlane + 1) : ((u4TopPlane == OSD_PLANE_2) ? 0x3 : 0x2);
#else
    u4Osd1Switch = u4TopPlane + 1;
    u4Osd3Switch = u4MidPlane + 1;
    u4Osd2Switch = u4BotPlane + 1;
#endif
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
    if (_fgOsdInit == 0)
    {
        OSD_InitRegBase();
    }
#endif
    _OSD_BASE_SetOSD1Switch(u4Osd1Switch);
    _OSD_BASE_SetOSD2Switch(u4Osd2Switch);
    _OSD_BASE_SetOSD3Switch(u4Osd3Switch);
    OSD_LOG_I("OSD plane order[%d,%d,%d]\n",
        u4TopPlane + 1, u4MidPlane + 1, u4BotPlane + 1);
#if defined(CC_MT5881)
    u4PmxOption = IO_READ32(IO_VIRT + 0x27800, 0x4) & 0xC;

    if (u4PmxOption != 4)
    {
        OSD_BASE_AdjustPlanePosition(u4OSDTopLayer, u4OSDMiddLayer, u4OSDBotLayer, i4Hstart, 1);
    }
    else
    {
        OSD_BASE_AdjustPlanePosition(u4OSDTopLayer, u4OSDMiddLayer, u4OSDBotLayer, i4Hstart, 11);
    }
#endif
    OSD_BASE_AdjustPlanePosition(u4TopPlane, u4MidPlane, u4BotPlane, i4Hstart, 2);
    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());

    // keep premulti-alpha config
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    OSD_WaitVsync();
#endif
    OSD_PLA_SetPremulEn(OSD_PLANE_1, u4Osd1PremulAlpha);
    OSD_PLA_SetPremulEn(OSD_PLANE_2, u4Osd2PremulAlpha);
    OSD_PLA_SetPremulEn(OSD_PLANE_3, u4Osd3PremulAlpha);

    OSD_EXP_BypassApply();

    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_GetPlaneSwitchOrder(UINT32 *u4TopPlane, UINT32 *u4MiddlePlane, UINT32 *u4BottomPlane)
{
    *u4TopPlane = u4OSDTopLayer;
    *u4MiddlePlane = u4OSDMiddLayer;
    *u4BottomPlane = u4OSDBotLayer;

    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_SetDisplayMode(UINT32 u4DisplayMode)
{
    if (BSP_IsFPGA())
    {
        OSD_VERIFY_DISPLAY_MODE(u4DisplayMode);

        if (u4DisplayMode == (UINT32) OSD_DM_480P) // 2
        {
            // 720x480
            IO_WRITE32MSK(SCPOS_BASE, 0x24, ((UINT32) 0x359 << 20) | 0x20c,
                          0xfff007ff);
            PANEL_SetPanelWidth(720);
            PANEL_SetPanelHeight(480);
        }
        else if (u4DisplayMode == (UINT32) OSD_DM_1080P) // 7
        {
            // 1920x1080
            IO_WRITE32MSK(SCPOS_BASE, 0x24, ((UINT32) 0x897 << 20) | 0x464,
                          0xfff007ff);
            PANEL_SetPanelWidth(1920);
            PANEL_SetPanelHeight(1080);
        }
    }

    UNUSED(u4DisplayMode);
    return (INT32) OSD_RET_OK;
}

//-----------------------------------------------------------------------------
/** set osd plane position
 *  @param  u4Plane plane id
 *  @param  u4X x position
 *  @param  u4Y y position
 *  @retval OSD_RET_OK success
 */
//-----------------------------------------------------------------------------
INT32 OSD_BASE_SetOsdPosition(UINT32 u4Plane, UINT32 u4X, UINT32 u4Y)
{
#if defined(CC_MT5893) || defined(CC_MT5895)
    // Not right, this is unsign number
    UINT32 u4RightOffset = 0;
    //= GET_POSITIVE(u4X - OSD_BASE_GetDisplayWidth() / 2);
    if (u4X > OSD_BASE_GetDisplayWidth() / 2)
    {
        u4RightOffset = u4X - OSD_BASE_GetDisplayWidth() / 2;
    }
#endif

    OSD_VERIFY_PLANE(u4Plane);

    if (u4Plane == (UINT32) OSD_PLANE_1)
    {
        IGNORE_RET(_OSD_BASE_SetOsd1HStart(u4X));
        IGNORE_RET(_OSD_BASE_SetOsd1VStart(u4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_SetOsd1HStart(u4RightOffset));
        }
#endif
    }
    else if (u4Plane == (UINT32) OSD_PLANE_2)
    {
        IGNORE_RET(_OSD_BASE_SetOsd2HStart(u4X));
        IGNORE_RET(_OSD_BASE_SetOsd2VStart(u4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_SetOsd2HStart(u4RightOffset));
        }
#endif
    }
    else if (u4Plane == (UINT32) OSD_PLANE_3)
    {
        IGNORE_RET(_OSD_BASE_SetOsd3HStart(u4X));
        IGNORE_RET(_OSD_BASE_SetOsd3VStart(u4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_SetOsd3HStart(u4RightOffset));
        }
#endif
    }
    else if (u4Plane == (UINT32) OSD_PLANE_5)
    {
        IGNORE_RET(_OSD_BASE_SetCsrHStart(u4X));
        IGNORE_RET(_OSD_BASE_SetCsrVStart(u4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_SetCsrHStart(u4RightOffset));
        }
#endif
    }

    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

//-----------------------------------------------------------------------------
/** get osd plane position
 *  @param  u4Plane plane id
 *  @param  pu4X x position pointer
 *  @param  pu4Y y position pointer
 *  @retval OSD_RET_OK success
 */
//-----------------------------------------------------------------------------
INT32 OSD_BASE_GetOsdPosition(UINT32 u4Plane, UINT32 *pu4X, UINT32 *pu4Y)
{
    UINT32 u4PosX = 0, u4RightPosX = 0;

    OSD_VERIFY_PLANE(u4Plane);

    if (u4Plane == (UINT32) OSD_PLANE_1)
    {
        IGNORE_RET(_OSD_BASE_GetOsd1HStart(&u4PosX));
        IGNORE_RET(_OSD_BASE_GetOsd1VStart(pu4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_GetOsd1HStart(&u4RightPosX));
        }
#endif
    }
    else if (u4Plane == (UINT32) OSD_PLANE_2)
    {
        IGNORE_RET(_OSD_BASE_GetOsd2HStart(&u4PosX));
        IGNORE_RET(_OSD_BASE_GetOsd2VStart(pu4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_GetOsd2HStart(&u4RightPosX));
        }
#endif
    }
    else if (u4Plane == (UINT32) OSD_PLANE_3)
    {
        IGNORE_RET(_OSD_BASE_GetOsd3HStart(&u4PosX));
        IGNORE_RET(_OSD_BASE_GetOsd3VStart(pu4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        if (OSD_IsDualOstgDisplay())
        {
            IGNORE_RET(_OSD_R_BASE_GetOsd3HStart(&u4RightPosX));
        }
#endif

    }
    else if (u4Plane == (UINT32) OSD_PLANE_5)
    {
        IGNORE_RET(_OSD_BASE_GetCsrHStart(&u4PosX));
        IGNORE_RET(_OSD_BASE_GetCsrVStart(pu4Y));
#if defined(CC_MT5893) || defined(CC_MT5895)
        IGNORE_RET(_OSD_R_BASE_GetCsrHStart(&u4RightPosX));
#endif
    }

    *pu4X = u4PosX + u4RightPosX;

    return (INT32) OSD_RET_OK;
}


//Set 5363 SD TVE output config

INT32 OSD_SetTVEOut(BOOL fgEnable)
{
    OSD_LOG_D("OSD_SetTVEOut: %x\n", fgEnable);

    if (fgEnable)
    {
        IO_WRITE32(OSD_BASE, 0x7c, 0x680000);
        IO_WRITE32(OSD_BASE, 0x900, 0x00400401);
        IO_WRITE32(OSD_BASE, 0x90c, 0x006137fb);
        // OSD5 color space conversion
        IO_WRITE32(OSD_BASE, 0x960, 0x00420081);
        IO_WRITE32(OSD_BASE, 0x964, 0x00190010);
        IO_WRITE32(OSD_BASE, 0x968, 0x1026104a);
        IO_WRITE32(OSD_BASE, 0x96c, 0x00700080);
        IO_WRITE32(OSD_BASE, 0x970, 0x0070105e);
        IO_WRITE32(OSD_BASE, 0x974, 0x10120080);
        IO_WRITE32(OSD_BASE, 0x18, 0x00050005);
        IO_WRITE32(OSD_BASE, 0xc0, 0x109); // OSD,TVE timing select , external timing
        IO_WRITE32(OSD_BASE, 0xc4, 0x00300001); // OSD5 V_START
        IO_WRITE32(OSD_BASE, 0xd8, 0x3);
        IO_WRITE32(OSD_BASE, 0x8, IO_READ32(OSD_BASE, 0x8) | 0x40000); // OSD5 TVE Hsync/vsync output
        IO_WRITE32(OSD_BASE, 0, 0x9);
        // temp solution
        IO_WRITE32(IO_VIRT, 0xd368, 0x8c000822);
    }
    else
    {
        //IO_WRITE32MSK(OSD_BASE, 0x8, 0, 0x40128);
        IO_WRITE32MSK(OSD_BASE, 0xc0, 0, 0xa);
        IO_WRITE32(OSD_BASE, 0x100, 0);
        IO_WRITE32MSK(OSD_BASE, 0xc8, 0, 0x03600271);
        IO_WRITE32MSK(OSD_BASE, 0xcc, 0, 0x03600271);
        //IO_WRITE32MSK(OSD_BASE, 0x20, 0,0x001e000c);
        IO_WRITE32MSK(OSD_BASE, 0x20, 0, 0x0019002b);
        IO_WRITE32(OSD_BASE, 0, 8);
    }

    return (INT32) OSD_RET_OK;
}
UINT32 _u4OsdLogoState = 0;
INT32 OSD_ShowLogo(void)
{
#ifndef CC_MTK_LOADER
    static UINT32 u4RegionList = 0;
    static UINT32 u4Region = 0;
    UINT32 u4BmpPitch, u4ColorMode, u4DataSize;
    UINT32 u4PanelWidth, u4PanelHeight;
    static UINT32 u4OutWidth, u4OutHeight, u4Width, u4Height, u4OutX, u4OutY ;
#if OSD_LOGO_FADING_ENABLE
    UINT32 i;
#endif
    UPTR u4BmpAddr = 0;//,u4PalAddr;
    UPTR u4LogoBuf = 0;
    const UINT32 ScaleUpMutiple = 1;
    UINT32 u4FHDScaleUp = 1;
    UINT32 u4OsdPlane = OSD_PLANE_3;
    UINT32 u4Scaler = OSD_SCALER_3;
    INT32 i4OriRegionList = (-1);
    UINT8 u1OriPLA_Enable;
    UINT32 u4OriSC_Enable = 0;
    UINT32 u4OriSrcW = 0;
    UINT32 u4OriSrcH = 0;
    UINT32 u4OriDstW = 0;
    UINT32 u4OriDstH = 0;
    UINT32 u4OriIs16Bpp = 0;
    INT32 ret;

    _u4OsdLogoState =0;
    if (DRVCUST_OptGet(eDrvLogoEnable) == 0)
    {
        OSD_LOG_E("Driver Logo Disabled!\n");
        return (INT32) OSD_RET_OK;
    }
    _u4OsdLogoState =1;
    //get original setting
    i4OriRegionList = OSD_PLA_GetFirstRegionList(u4OsdPlane);

    if (OSD_PLA_GetEnable(u4OsdPlane, &u1OriPLA_Enable) < 0)
    {
        OSD_LOG_E("OSD_PLA_GetEnable fail\n");
    }

    UNUSED(OSD_SC_GetScalerInfo(u4Scaler, &u4OriSC_Enable, &u4OriSrcW,
                &u4OriSrcH, &u4OriDstW, &u4OriDstH, &u4OriIs16Bpp));
    UNUSED(u4OriIs16Bpp);

    u4PanelWidth = OSD_BASE_GetDisplayWidth();
    u4PanelHeight = OSD_BASE_GetDisplayHeight();
    //when panel is 2K2K, scaler srcHeight need to be 1080.
    if(u4PanelWidth == 1920 && u4PanelHeight == 2160)
        u4FHDScaleUp = 2;
    if (fgLogoInit == TRUE)
    {
        ret = OSD_PLA_FlipTo(u4OsdPlane, u4RegionList);
        OSD_LOG_I("Panel width:%u height:%u, ScaleUpMutiple:%u,u4OutWidth=%u,u4OutHeight=%u,u4OutX=%u,u4OutY=%u.\n",
                u4PanelWidth, u4PanelHeight, ScaleUpMutiple, u4OutWidth, u4OutHeight, u4OutX, u4OutY);
        if ((u4OutWidth < u4PanelWidth) &&
                (u4OutHeight < u4PanelHeight))
        {
            ret = OSD_SC_Scale(u4Scaler, 1, u4PanelWidth/ScaleUpMutiple, u4PanelHeight/(ScaleUpMutiple*u4FHDScaleUp),
                    u4PanelWidth, u4PanelHeight);
        }
        else
        {
            ret = OSD_SC_Scale(u4Scaler, 1, u4Width, u4Height, u4OutWidth,
                    u4OutHeight);
        }
        if (ret != OSD_RET_OK)
        {
            OSD_LOG_E("osd_sc_scaler error\n");
        }
    }
    else
    {
        ret = OSD_RGN_LIST_Create(&u4RegionList);
        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 1;
        }

        u4BmpPitch = 0;
        u4ColorMode = DRVCUST_OptGet(eDrvLogoColorMode);
        u4Width = DRVCUST_OptGet(eDrvLogoWidth);
        u4Height = DRVCUST_OptGet(eDrvLogoHeight);
        u4BmpAddr = DRVCUST_OptGetUptr(eDrvLogoBmpAddr);

        if (u4Width == 1366)
        {
            OSD_GET_PITCH_SIZE(u4ColorMode, u4Width + 2,
                    u4BmpPitch); // to set u4BmpPitch by u4ColorMode and u4Width.
        }
        else
        {
            OSD_GET_PITCH_SIZE(u4ColorMode, u4Width, u4BmpPitch);
        }

        u4BmpPitch = (u4BmpPitch + 0xF) & (~0xF);
        u4DataSize = u4BmpPitch * u4Height;

        if (u4DataSize == 0)
        {
            OSD_LOG_E("u4DataSize is 0\n");
            _u4OsdLogoState =2;
            return FALSE;
        }

        u4LogoBuf = (UINT32) BSP_AllocAlignedDmaMemory(u4DataSize, 4096);

        if (u4LogoBuf == 0)
        {
            OSD_LOG_E("u4LogoBuf null\n");
            _u4OsdLogoState =2;
            return FALSE;
        }

        x_memcpy((void *)(UPTR)VIRTUAL(u4LogoBuf), (void *)(UPTR)u4BmpAddr, u4DataSize);
        HalFlushInvalidateDCacheMultipleLine((UPTR)(VIRTUAL(u4LogoBuf)),
                u4DataSize);
        u4BmpAddr = u4LogoBuf;
        ret = OSD_RGN_Create(&u4Region, u4Width, u4Height, (void *)(UPTR)u4BmpAddr,
                u4ColorMode, u4BmpPitch, 0, 0, u4Width, u4Height);

        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 2;
        }

        OSD_RGN_Set(u4Region, OSD_RGN_DEC_EN, 0);
        OSD_RGN_Set(u4Region, (INT32) OSD_RGN_V_FLIP, OSD_GetCpblty(E_OSD_CPBLTY_FLIP, 0x0));
        OSD_RGN_Set(u4Region, (INT32) OSD_RGN_H_MIRROR, OSD_GetCpblty(E_OSD_CPBLTY_MIRROR, 0x0));
        ret = OSD_RGN_Insert(u4Region, u4RegionList);

        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 3;
        }

        ret = OSD_PLA_FlipTo(u4OsdPlane, u4RegionList);

        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 4;
        }

        u4OutWidth = u4Width;
        u4OutHeight = u4Height;

        u4OutX = ((u4PanelWidth / ScaleUpMutiple > u4OutWidth)) ? ((u4PanelWidth / ScaleUpMutiple - u4OutWidth) >> 1) : 0;
        u4OutY = ((u4PanelHeight / ScaleUpMutiple > u4OutHeight))? ((u4PanelHeight / (ScaleUpMutiple * u4FHDScaleUp) - u4OutHeight)>> 1):0;
        OSD_LOG_I("Panel width:%u height:%u, ScaleUpMutiple:%u,u4OutWidth=%u,u4OutHeight=%u,u4OutX=%u,u4OutY=%u.\n",
                  u4PanelWidth, u4PanelHeight, ScaleUpMutiple, u4OutWidth, u4OutHeight, u4OutX, u4OutY);
        //for using region position setting, but not use plane position setting(else, will set plane  position again when fb init)
        if ((u4OutWidth < u4PanelWidth) &&
                (u4OutHeight < u4PanelHeight))
        {
            ret = OSD_SC_Scale(u4Scaler, 1, u4PanelWidth/ScaleUpMutiple, u4PanelHeight/(ScaleUpMutiple*u4FHDScaleUp),
                    u4PanelWidth, u4PanelHeight);
        }
        else
        {
            ret = OSD_SC_Scale(u4Scaler, 1, u4Width, u4Height, u4OutWidth,
                    u4OutHeight);
        }
        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 5;
        }

        ret = OSD_RGN_Set(u4Region, OSD_RGN_POS_X, u4OutX);
        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 6;
        }

        ret = OSD_RGN_Set(u4Region, OSD_RGN_POS_Y, u4OutY);
        if (ret != OSD_RET_OK)
        {
            _u4OsdLogoState =2;
            return 7;
        }

        fgLogoInit = TRUE;
    }

    ret = OSD_PLA_Enable(u4OsdPlane, TRUE);
    if (ret != OSD_RET_OK)
    {
        _u4OsdLogoState =2;
        return 8;
    }

    OSD_PLA_SetAvailable(u4OsdPlane, FALSE);
#if OSD_LOGO_FADING_ENABLE
    OSD_RGN_Set(u4Region, OSD_RGN_ALPHA, 0);
#else
    OSD_RGN_Set(u4Region, OSD_RGN_ALPHA, 0xff);
#endif
    OSD_RGN_Set(u4Region, OSD_RGN_MIX_SEL, (UINT32) OSD_BM_REGION);
#if OSD_LOGO_FADING_ENABLE

    for (i = 0; i < 16; i++)
    {
        x_thread_delay(62);
        OSD_RGN_Set(u4Region, OSD_RGN_ALPHA, ((i + 1) * 16 - 1));
    }

    x_thread_delay(1000);

    for (i = 16; i > 0; i--)
    {
        OSD_RGN_Set(u4Region, OSD_RGN_ALPHA, (i * 16 - 1));
        x_thread_delay(62);
    }

    OSD_RGN_Set(u4Region, OSD_RGN_ALPHA, 0);
#else
#if defined(CC_S_PLATFORM) && defined(CC_LMP_CUST)
    x_thread_delay(1000);
#else
    x_thread_delay(3000);
#endif
#endif
    if (!OSD_PLA_GetSCCacheState(u4OsdPlane))
    {
        OSD_SC_Scale(u4Scaler, u4OriSC_Enable, u4OriSrcW, u4OriSrcH,
                u4OriDstW, u4OriDstH);
    }

    if (!OSD_PLA_GetFlipCacheState(u4OsdPlane))
    {
        if (i4OriRegionList >= 0)
        {
            OSD_PLA_FlipTo(u4OsdPlane, (UINT32) i4OriRegionList);
        }
        else
        {
            OSD_PLA_FlipToNone(u4OsdPlane);
        }
    }

    if (!OSD_PLA_GetEnableCacheState(u4OsdPlane))
    {
        OSD_PLA_Enable(u4OsdPlane, u1OriPLA_Enable);
    }
    OSD_PLA_SetAvailable(u4OsdPlane, TRUE);

#endif  //#ifndef CC_MTK_LOADER
    _u4OsdLogoState =2;
    return (INT32) OSD_RET_OK;
}

// Get OSD FIFO underflow status
BOOL OSD_GetUnderflowStatus(void)
{
    return (OSD_READ32(0xFC) & 0x1);
}


// Clear OSD FIFO underflow status
void OSD_ClearUnderflowStatus(void)
{
    IO_WRITE32(OSD_BASE, 0x034, 0x0);
    IO_WRITE32(OSD_BASE, 0x0FC, 0x4);
}

INT32 OSD_BASE_Get3DOsdShiftEn(UINT32 OSD_Plane, UINT32 *pu4Value)
{
    switch (OSD_Plane)
    {
        case OSD_PLANE_2:
            _OSD_3D_GetOsd2LShiftEn(pu4Value);
            break;

        case OSD_PLANE_3:
            _OSD_3D_GetOsd3LShiftEn(pu4Value);
            break;

        case OSD_PLANE_1:
            _OSD_3D_GetOsd1LShiftEn(pu4Value);
            break;

        default:
            break;
    }

    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_Get3DOsd2LShiftEn(UINT32 *pu4Value)
{
    _OSD_3D_GetOsd2LShiftEn(pu4Value);
    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_SetPlaneLineShift(UINT32 u4Plane, BOOL fgEnable, INT32 i4Value)
{
    OSD_VERIFY_PLANE(u4Plane);

    switch (u4Plane)
    {
        case OSD_PLANE_1:
            if (fgEnable)
            {
                OSD_3D_SetOsd1LShiftEn(TRUE);
                _OSD_3D_SetOsd1ShiftValue((i4Value > 0) ? i4Value : (0 - i4Value));
            }
            else//OSD_3DPrLshift_ENABLE_FLAG
            {
                OSD_3D_SetOsd1LShiftEn(FALSE);
                _OSD_3D_SetOsd1ShiftValue(0);
            }

            break;

        case OSD_PLANE_2:
            if (fgEnable)
            {
                OSD_3D_SetOsd2LShiftEn(TRUE);
                _OSD_3D_SetOsd2ShiftValue((i4Value > 0) ? i4Value : (0 - i4Value));
            }
            else
            {
                OSD_3D_SetOsd2LShiftEn(FALSE);
                _OSD_3D_SetOsd2ShiftValue(0);
            }

            break;

        case OSD_PLANE_3:
            if (fgEnable)
            {
                OSD_3D_SetOsd3LShiftEn(TRUE);
                _OSD_3D_SetOsd3ShiftValue((i4Value > 0) ? i4Value : (0 - i4Value));
            }
            else
            {
                OSD_3D_SetOsd3LShiftEn(FALSE);
                _OSD_3D_SetOsd3ShiftValue(0);
            }

            break;

        default:
            break;
    }

    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    VERIFY((INT32) OSD_RET_OK == _OSD_3D_UpdateHwReg());
    HalFlushDCache();
    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_Set3DPrLineShift(UINT32 u4Enable, UINT32 u4Shift)
{
    INT32 i4Plane2Region = -1, i4Plane3Region = -1, i4Plane1Region = -1;
    INT32 u4PlaneOutputXShift = 0, u4PlaneOutputX = 0;
    static UINT32 u4Enable_tag = 0;
    static UINT32 u4Plane1OutputH_tag = 0, u4Plane2OutputH_tag = 0,
                  u4Plane3OutputH_tag = 0;

    if (_fgOsdInit == 0)
    {
        return (INT32) OSD_RET_OK;
    }

    //OSD_LOG_D("u4Enable_tag = 0x%x\n", u4Enable_tag);

    if (!(u4Enable_tag))
    {
        _OSD_BASE_GetScrnHStartOsd1(&u4Plane1OutputH_tag);
        _OSD_BASE_GetScrnHStartOsd2(&u4Plane2OutputH_tag);
        _OSD_BASE_GetScrnHStartOsd3(&u4Plane3OutputH_tag);
        u4Enable_tag = 1;
        //OSD_LOG_D(">>u4Enable_tag = 0x%x\n", u4Enable_tag);
    }

    if (u4Shift < 0x80)
    {
        u4PlaneOutputXShift = -u4Shift / 2;
        _OSD_3D_SetOsd2LShiftInv((UINT32) FALSE);
    }
    else if (u4Shift > 0x80)
    {
        u4PlaneOutputXShift = (u4Shift - 0x80) / 2;
        _OSD_3D_SetOsd2LShiftInv((UINT32) TRUE);
    }

    i4Plane2Region = OSD_PLA_GetFirstRegion(OSD_PLANE_2);

    //OSD_LOG_D(" OSD_PLANE get region = %d \n", i4Plane2Region);
    if ((i4Plane2Region >= (UINT32) OSD_MAX_NUM_RGN) || (i4Plane2Region == (-1)))
    {
        //OSD_LOG_E(" OSD_PLANE get region fail \n");
        //return (INT32)OSD_RET_OK;
    }
    else if (i4Plane2Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane2OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd2(u4PlaneOutputX);
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_2, TRUE, u4Shift);
        }
        else
        {
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_2, FALSE, 0);
            _OSD_BASE_SetScrnHStartOsd2(u4Plane2OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    i4Plane3Region = OSD_PLA_GetFirstRegion(OSD_PLANE_3);

    if (i4Plane3Region >= (UINT32) OSD_MAX_NUM_RGN || i4Plane3Region < 0)
    {
        //OSD_LOG_E(" OSD_PLANE get region fail \n");
        // return (INT32)OSD_RET_OK;
    }
    else if (i4Plane3Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane3OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd3(u4PlaneOutputX);
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_3, TRUE, u4Shift);
        }
        else
        {
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_3, FALSE, 0);
            _OSD_BASE_SetScrnHStartOsd3(u4Plane3OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    i4Plane1Region = OSD_PLA_GetFirstRegion(OSD_PLANE_1);

    if ((i4Plane1Region >= (UINT32) OSD_MAX_NUM_RGN) || (i4Plane3Region == (-1)))
    {
        //OSD_LOG_E(" OSD_PLANE get region fail \n");
        // return (INT32)OSD_RET_OK;
    }
    else if (i4Plane1Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane1OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd1(u4PlaneOutputX);
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_1, TRUE, u4Shift);
        }
        else
        {
            OSD_BASE_SetPlaneLineShift(OSD_PLANE_1, FALSE, 0);
            _OSD_BASE_SetScrnHStartOsd1(u4Plane1OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    HalFlushDCache();
    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_Set3DShutterLineShift(UINT32 u4Enable, UINT32 u4Shift)
{
    UINT32 u4Plane2Region = 0, u4Plane3Region = 0, u4Plane1Region = 0;
    INT32 u4PlaneOutputXShift = 0, u4PlaneOutputX = 0;
    static UINT32 u4Enable_tag = 0;
    static UINT32 u4Plane1OutputH_tag = 0, u4Plane2OutputH_tag = 0,
                  u4Plane3OutputH_tag = 0;
    OSD_LOG_D("u4Enable_tag = 0x%x\n", u4Enable_tag);

    if (!(u4Enable_tag))
    {
        _OSD_BASE_GetScrnHStartOsd1(&u4Plane1OutputH_tag);
        _OSD_BASE_GetScrnHStartOsd2(&u4Plane2OutputH_tag);
        _OSD_BASE_GetScrnHStartOsd3(&u4Plane3OutputH_tag);
        u4Enable_tag = 1;
    }

    if (u4Shift < 0x80)
    {
        u4PlaneOutputXShift = -u4Shift / 2;
    }
    else if (u4Shift > 0x80)
    {
        u4PlaneOutputXShift = (u4Shift - 0x80) / 2;
    }

    u4Plane2Region = OSD_PLA_GetFirstRegion(OSD_PLANE_2);

    if (u4Plane2Region >= (UINT32) OSD_MAX_NUM_RGN)
    {
        OSD_LOG_E(" OSD_PLANE get region fail \n");
        //return (INT32)OSD_RET_OK;
    }
    else if (u4Plane2Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane2OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd2(u4PlaneOutputX);
            _OSD_3D_SetShutterShiftEn(TRUE);
            _OSD_3D_SetTgenLREn(TRUE);
            _OSD_3D_SetOsd2ShiftValue((u4Shift > 0) ? u4Shift : (0 - u4Shift));
        }
        else
        {
            _OSD_3D_SetShutterShiftEn(FALSE);
            _OSD_3D_SetTgenLREn(FALSE);
            _OSD_3D_SetOsd2ShiftValue(0);
            _OSD_BASE_SetScrnHStartOsd2(u4Plane2OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    u4Plane3Region = OSD_PLA_GetFirstRegion(OSD_PLANE_3);

    if (u4Plane3Region >= (UINT32) OSD_MAX_NUM_RGN)
    {
        OSD_LOG_E(" OSD_PLANE get region fail \n");
        // return (INT32)OSD_RET_OK;
    }
    else if (u4Plane3Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane3OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd3(u4PlaneOutputX);
            _OSD_3D_SetShutterShiftEn(TRUE);
            _OSD_3D_SetTgenLREn(TRUE);
            _OSD_3D_SetOsd2ShiftValue((u4Shift > 0) ? u4Shift : (0 - u4Shift));
        }
        else
        {
            _OSD_3D_SetShutterShiftEn(FALSE);
            _OSD_3D_SetTgenLREn(FALSE);
            _OSD_3D_SetOsd2ShiftValue(0);
            _OSD_BASE_SetScrnHStartOsd3(u4Plane3OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    u4Plane1Region = OSD_PLA_GetFirstRegion(OSD_PLANE_1);

    if (u4Plane1Region >= (UINT32) OSD_MAX_NUM_RGN)
    {
        OSD_LOG_E(" OSD_PLANE get region fail \n");
        // return (INT32)OSD_RET_OK;
    }
    else if (u4Plane1Region < OSD_MAX_NUM_RGN)
    {
        if (u4Enable == 1)
        {
            u4PlaneOutputX = u4Plane1OutputH_tag + u4PlaneOutputXShift;
            _OSD_BASE_SetScrnHStartOsd1(u4PlaneOutputX);
            _OSD_3D_SetShutterShiftEn(TRUE);
            _OSD_3D_SetTgenLREn(TRUE);
            _OSD_3D_SetOsd2ShiftValue((u4Shift > 0) ? u4Shift : (0 - u4Shift));
        }
        else
        {
            _OSD_3D_SetShutterShiftEn(FALSE);
            _OSD_3D_SetTgenLREn(FALSE);
            _OSD_3D_SetOsd2ShiftValue(0);
            _OSD_BASE_SetScrnHStartOsd1(u4Plane1OutputH_tag);
            u4Enable_tag = 0;
        }
    }

    VERIFY((INT32) OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    VERIFY((INT32) OSD_RET_OK == _OSD_3D_UpdateHwReg());
    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_SetLineShift(UINT32 u4Enable, UINT32 u4Shift)
{
    switch (u4Enable)
    {
        case 0:
            OSD_BASE_Set3DPrLineShift(FALSE, 0);
            OSD_BASE_Set3DShutterLineShift(FALSE, 0);
            break;

        case 1:
            OSD_BASE_Set3DPrLineShift(TRUE, u4Shift);
            break;

        case 2:
            OSD_BASE_Set3DShutterLineShift(TRUE, u4Shift);
            break;

        case 3:
            OSD_BASE_Set3DPrLineShift(FALSE, 0);
            break;

        case 4:
            OSD_BASE_Set3DShutterLineShift(FALSE, 0);
            break;

        default:
            break;
    }

    return (INT32) OSD_RET_OK;
}

INT32 OSD_BASE_GetHTotalPixel(UINT32 *pu4Value)
{
    if (pu4Value == NULL)
    {
        return -(INT32) OSD_RET_INV_ARG;
    }

    *pu4Value = (OSD_READ32(0x48) & 0x3FFF0000) >> 16;
    return (INT32) OSD_RET_OK;
}
BOOL OSD_BASE_Get_3DOSD2PLsEn()
{
    return OSD2_3DPrLshift_ENABLE_FLAG;
}

void OSD_BASE_Set_3DOSD2PLsEn(BOOL u4Enable)
{
    OSD2_3DPrLshift_ENABLE_FLAG = u4Enable;
}

INT32 OSD_3D_SetOsd2LShiftEn(UINT32 u4Value)
{
    UINT32 u4_rgn   = 0x0;
    UINT32 u4_dec   = 0x0;
    u4_rgn = OSD_PLA_GetFirstRegion(OSD_PLANE_2);
    VERIFY(OSD_RGN_Get(u4_rgn, OSD_RGN_DEC_EN, &u4_dec) == OSD_RET_OK);

    if (u4_dec)
    {
        OSD_BASE_Set_3DOSD2PLsEn((BOOL)u4Value);
    }
    else
    {
        _OSD_3D_SetOsd2LShiftEn((UINT8)u4Value);

        if (u4Value)
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x2000);
        }
        else
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x1000);
        }
    }

    return (INT32)OSD_RET_OK;
}

INT32 OSD_3D_SetOsd1LShiftEn(UINT32 u4Value)
{
    UINT32 u4_rgn   = 0x0;
    UINT32 u4_dec   = 0x0;
    u4_rgn = OSD_PLA_GetFirstRegion(OSD_PLANE_1);
    VERIFY(OSD_RGN_Get(u4_rgn, OSD_RGN_DEC_EN, &u4_dec) == OSD_RET_OK);

    if (u4_dec)
    {
        OSD_BASE_Set_3DOSD2PLsEn((BOOL)u4Value);
    }
    else
    {
        _OSD_3D_SetOsd1LShiftEn((UINT8)u4Value);

        if (u4Value)
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x2000);
        }
        else
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x1000);
        }
    }

    return (INT32)OSD_RET_OK;
}


INT32 OSD_3D_SetOsd3LShiftEn(UINT32 u4Value)
{
    UINT32 u4_rgn   = 0x0;
    UINT32 u4_dec   = 0x0;
    u4_rgn = OSD_PLA_GetFirstRegion(OSD_PLANE_3);
    VERIFY(OSD_RGN_Get(u4_rgn, OSD_RGN_DEC_EN, &u4_dec) == OSD_RET_OK);

    if (u4_dec)
    {
        OSD_BASE_Set_3DOSD2PLsEn((BOOL)u4Value);
    }
    else
    {
        _OSD_3D_SetOsd3LShiftEn((UINT8)u4Value);

        if (u4Value)
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x2000);
        }
        else
        {
            OSD_RGN_Set(u4_rgn, OSD_RGN_STEP_V, 0x1000);
        }
    }

    return (INT32)OSD_RET_OK;
}

BOOL OSD_BASE_IsDisplaying(void)
{
    BOOL fgEnable = 0;
    UINT32 u4Enabe0 = 0, u4Enabe1 = 0, u4Enable2 = 0;
#ifdef CONFIG_OSD_COMPRESSION
    UINT32 u4ResetValue = 0;
#endif
    _OSD_PLA_GetEnable(OSD_PLANE_1, &u4Enabe0);
    _OSD_PLA_GetEnable(OSD_PLANE_2, &u4Enabe1);
    _OSD_PLA_GetEnable(OSD_PLANE_3, &u4Enable2);
#ifdef CONFIG_OSD_COMPRESSION
    u4ResetValue = IO_READ32(OSD_BASE_REG, 0x4);
    IO_WRITE32(OSD_BASE_REG, 0x4, u4ResetValue & 0xFFFFFFFBU);
    OSD_UpdateReg();
#if defined(__KERNEL__) && !defined(CC_UBOOT)
    OSD_WaitVsync();
#endif
#endif

    if ((u4Enabe1 && IO_READ32(OSD_BASE_REG, 0x5F8)) ||
       (u4Enable2 && IO_READ32(OSD_BASE_REG, 0x6F8)) ||
       (u4Enabe0 && IO_READ32(OSD_BASE_REG, 0x4F8)))
    {
        fgEnable = TRUE;
    }
    else
    {
        fgEnable = FALSE;
    }

#ifdef CONFIG_OSD_COMPRESSION
    IO_WRITE32(OSD_BASE_REG, 0x4, u4ResetValue);
    OSD_UpdateReg();
#endif
    return fgEnable;
}

BOOL OSD_BASE_SonyIsDisplaying(void)
{
    static BOOL fgEnable = 0;
    UINT32 u4Enabe1 = 0, u4Enable2 = 0;
#ifdef CONFIG_OSD_COMPRESSION
    static UINT32 u4ResetValue = 0;
#endif
    //static UINT8 bVyncCouter = 3;
    _OSD_PLA_GetEnable(OSD_PLANE_2, &u4Enabe1);
    _OSD_PLA_GetEnable(OSD_PLANE_3, &u4Enable2);
#ifdef CONFIG_OSD_COMPRESSION

    if (bVyncCouter == 3)
    {
        u4ResetValue = IO_READ32(OSD_BASE_REG, 0x4);
        IO_WRITE32(OSD_BASE_REG, 0x4, u4ResetValue & 0xFFFFFFFBU);
        OSD_UpdateReg();
        //OSD_WaitVsync();
        //OSD_WaitVsync();
        // OSD_WaitVsync();
    }

#endif

    if (bVyncCouter-- == 0)
    {
        if ((u4Enabe1 && IO_READ32(OSD_BASE_REG, 0x5F8)) ||
            (u4Enable2 && IO_READ32(OSD_BASE_REG, 0x6F8)))
        {
            fgEnable = TRUE;
        }
        else
        {
            fgEnable = FALSE;
        }

#ifdef CONFIG_OSD_COMPRESSION
        IO_WRITE32(OSD_BASE_REG, 0x4, u4ResetValue);
        OSD_UpdateReg();
#endif
        bVyncCouter = 3;
    }

    return fgEnable;
}


VOID OSD_BASE_GetPanelSize(UINT32 *u4Width, UINT32 *u4Height)
{
    if (u4Width)
    {
        *u4Width = OSD_BASE_GetDisplayWidth();
    }

    if (u4Height)
    {
        *u4Height = OSD_BASE_GetDisplayHeight();
    }
}


INT32 OSD_3D_SetPlaneSwitch(UINT32 u4Enable)
{
#ifdef CC_ANDROID_3D_UI_SUPPORT
    _OSD_3D_SetPlaneSwitchEn(u4Enable);
    VERIFY((INT32)OSD_RET_OK == _OSD_3D_UpdateHwReg());
#endif
    return (INT32) OSD_RET_OK;
}

#ifdef CC_TDTV_3D_OSD_ENABLE
INT32 OSD_BASE_Enabe3DShutter(UINT32 u4Enable, UINT32 u4Shift)
{
    UINT32 u4Plane2Region = 0, u4Plane3Region = 0;
    UINT32 u4Plane2RightRegion = 0, u4Plane3RightRegion = 0;
    UPTR upLeftRgnAddr;
    UPTR upRightRgnAddr;
    UINT32 u4RegionPosX = 0;

    if (u4Enable == 0)
    {
        if (_u1Plane2RightRegion != 0)
        {
            OSD_VERIFY_REGION(_u1Plane2RightRegion);
            OSD_RGN_Delete(_u1Plane2RightRegion);
            _u1Plane2RightRegion = 0;
            _OSD_PLA_SetRightHeaderAddr(OSD_PLANE_2, 0);
            OSD_PLA_Set3DMode(OSD_PLANE_2, u4Enable);
            VERIFY((INT32)OSD_RET_OK == _OSD_PLA_UpdateHwReg(OSD_PLANE_2));
        }

        if (_u1Plane3RightRegion != 0)
        {
            OSD_VERIFY_REGION(_u1Plane3RightRegion);
            OSD_RGN_Delete(_u1Plane3RightRegion);
            _u1Plane3RightRegion = 0;
            _OSD_PLA_SetRightHeaderAddr(OSD_PLANE_3, 0);
            OSD_PLA_Set3DMode(OSD_PLANE_3, u4Enable);
            VERIFY((INT32)OSD_RET_OK == _OSD_PLA_UpdateHwReg(OSD_PLANE_3));
        }

        return (INT32)OSD_RET_OK;
    }

    u4Plane2Region = OSD_PLA_GetFirstRegion(OSD_PLANE_2);
    u4Plane3Region = OSD_PLA_GetFirstRegion(OSD_PLANE_3);

    if ((u4Plane2Region >= (UINT32)OSD_MAX_NUM_RGN) && (u4Plane3Region >= (UINT32)OSD_MAX_NUM_RGN))
    {
        OSD_LOG_E(" OSD_PLANE get region fail \n");
        return (INT32)OSD_RET_OK;
    }

    if (u4Plane2Region < OSD_MAX_NUM_RGN)
    {
        UINT32 u4BmpW, u4BmpH, u4DispW, u4DispH, u4ColorMode;
        UINT32 u4BmpAddr, u4BmpPitch;
        UINT32 u4ret;

        if (_u1Plane2RightRegion != 0)
        {
            OSD_LOG_D(" OSD Plane 2 3D  Shutter have enable l \n");
        }
        else
        {
            OSD_VERIFY_REGION(u4Plane2Region);
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_BMP_W, &u4BmpW));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_BMP_H, &u4BmpH));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_DISP_W, &u4DispW));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_DISP_H, &u4DispH));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_COLORMODE, &u4ColorMode));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_BMP_ADDR, &u4BmpAddr));
            IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_BMP_PITCH, &u4BmpPitch));
            u4ret = OSD_RGN_Create(&u4Plane2RightRegion, u4BmpW, u4BmpH, (void *)u4BmpAddr,
                                   u4ColorMode, u4BmpPitch, 0, 0, u4DispW, u4DispH);

            if (u4ret != OSD_RET_OK)
            {
                OSD_LOG_E("OSD create region fail \n");
                return 2;
            }

            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_RGN_GetAddress((UINT32)u4Plane2Region, &upLeftRgnAddr));
            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_RGN_GetAddress((UINT32)u4Plane2RightRegion, &upRightRgnAddr));
            x_memcpy((void *)upRightRgnAddr, (void *)upLeftRgnAddr, sizeof(OSD_RGN_UNION_T));

            if (u4Shift < 0x40 && u4Shift > 0)
            {
                IGNORE_RET(OSD_RGN_Get(u4Plane2Region, (INT32)OSD_RGN_POS_X, &u4RegionPosX));
                u4RegionPosX = u4RegionPosX + u4Shift;
                OSD_LOG_D(" OSD Plane test 000000   %d l \n", u4RegionPosX);
                IGNORE_RET(OSD_RGN_Set(u4Plane2RightRegion, (INT32)OSD_RGN_POS_X, u4RegionPosX));
            }

            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_PLA_SetRightHeaderAddr(OSD_PLANE_2, PHYSICAL(upRightRgnAddr)));
            OSD_PLA_Set3DMode(OSD_PLANE_2, u4Enable);
            VERIFY((INT32)OSD_RET_OK == _OSD_PLA_UpdateHwReg(OSD_PLANE_2));
            _u1Plane2RightRegion = u4Plane2RightRegion;
        }
    }

    if (u4Plane3Region < OSD_MAX_NUM_RGN)
    {
        UINT32 u4BmpW, u4BmpH, u4DispW, u4DispH, u4ColorMode;
        UINT32 u4BmpAddr, u4BmpPitch;
        UINT32 u4ret;

        if (_u1Plane3RightRegion != 0)
        {
            OSD_LOG_D(" OSD Plane 3 3D  Shutter have enable l \n");
        }
        else
        {
            OSD_VERIFY_REGION(u4Plane3Region);
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_BMP_W, &u4BmpW));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_BMP_H, &u4BmpH));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_DISP_W, &u4DispW));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_DISP_H, &u4DispH));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_COLORMODE, &u4ColorMode));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_BMP_ADDR, &u4BmpAddr));
            IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_BMP_PITCH, &u4BmpPitch));
            u4ret = OSD_RGN_Create(&u4Plane3RightRegion, u4BmpW, u4BmpH, (void *)u4BmpAddr,
                                   u4ColorMode, u4BmpPitch, 0, 0, u4DispW, u4DispH);

            if (u4ret != OSD_RET_OK)
            {
                OSD_LOG_E("OSD create region fail \n");
                return 2;
            }

            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_RGN_GetAddress((UINT32)u4Plane3Region, &upLeftRgnAddr));
            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_RGN_GetAddress((UINT32)u4Plane3RightRegion, &upRightRgnAddr));
            x_memcpy((void *)upLeftRgnAddr, (void *)&upRightRgnAddr, sizeof(OSD_RGN_UNION_T));

            if (u4Shift < 0x40 && u4Shift > 0)
            {
                IGNORE_RET(OSD_RGN_Get(u4Plane3Region, (INT32)OSD_RGN_POS_X, &u4RegionPosX));
                u4RegionPosX = u4RegionPosX + u4Shift;
                IGNORE_RET(OSD_RGN_Set(u4Plane3RightRegion, (INT32)OSD_RGN_POS_X, u4RegionPosX));
            }

            VERIFY((INT32)OSD_RET_OK ==
                   _OSD_PLA_SetRightHeaderAddr(OSD_PLANE_3, PHYSICAL(upRightRgnAddr)));
            OSD_PLA_Set3DMode(OSD_PLANE_3, u4Enable);
            VERIFY((INT32)OSD_RET_OK == _OSD_PLA_UpdateHwReg(OSD_PLANE_3));
            _u1Plane3RightRegion = u4Plane3RightRegion;
        }
    }

    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_Set3DStatus(void)
{
    S_TDTV_OSD_CONFIG sOsdCfg = eDrvTDTVOSDModeQuery();

    if (_u13DOsdShift != 0) //for cli test use
    {
        return (INT32)OSD_RET_OK;
    }

    switch (sOsdCfg.eOSDType)
    {
        case E_TDTV_OSD_2D:
            OSD_BASE_Enabe3DShutter(FALSE, 0);
            gfgOsd_Sbs = 0;
            gfgOsd_Tbb = 0;
            _u13DOsdShift = 0;
            break;

        case E_TDTV_OSD_FS:
            gfgOsd_Sbs = 0;
            gfgOsd_Tbb = 0;
            _u13DOsdShift = 0;
            OSD_BASE_Enabe3DShutter(TRUE, 0);
            break;

        case E_TDTV_OSD_SBS:
            OSD_BASE_Enabe3DShutter(FALSE, 0);
            gfgOsd_Sbs = 1;
            gfgOsd_Tbb = 0;
            _u13DOsdShift = 0;
            break;

        case E_TDTV_OSD_TB:
            OSD_BASE_Enabe3DShutter(FALSE, 0);
            gfgOsd_Sbs = 0;
            gfgOsd_Tbb = 1;
            _u13DOsdShift = 0;
            break;

        default:
            OSD_BASE_Enabe3DShutter(FALSE, 0);
            gfgOsd_Sbs = 0;
            gfgOsd_Tbb = 0;
            _u13DOsdShift = 0;
            break;
    }

    return (INT32)OSD_RET_OK;
}

#endif

#ifdef OSD_TRANS_CTRL
UINT32 OSD_BASE_GetTranBuf_Current(void)
{
    return _OSD_PlANE2_TransferBuf_CURRENT;
}

UINT32 OSD_BASE_GetTranBuf_Back(void)
{
    UINT32 u4Buf = OSD_BASE_GetTranBuf_Current();

    if (u4Buf == _OSD_PlANE2_TransferBuf_1)
    {
        return _OSD_PlANE2_TransferBuf_2;
    }
    else
    {
        return _OSD_PlANE2_TransferBuf_1;
    }
}

INT32 OSD_BASE_TransferTo_SYNC_Header(UINT32 u4Region)
{
    UINT32 u4Value = 0;
    OSD_VERIFY_REGION(u4Region);
    OSD_RGN_Get(u4Region, OSD_RGN_COLORMODE, &u4Value);

    if (u4Value == OSD_CM_ARGB8888_DIRECT32)
    {
        OSD_BASE_SetFRCHeader(OSD_FRCHD_FORMAT, 4);
    }
    else if (u4Value == OSD_CM_AYCBCR8888_DIRECT32)
    {
        OSD_BASE_SetFRCHeader(OSD_FRCHD_FORMAT, 5);
    }

    //OSD_RGN_Get(u4Region,,&u4Value);
    //OSD_BASE_SetFRCHeader(OSD_FRCHD_PIXORDER,u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_BMP_H, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_BMPHEIGHT, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_BMP_W, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_BMPWIDTH, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_ALPHA, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_GLOBALALPHA, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_ALPHA_SEL, &u4Value);

    if (u4Value == 1)
    {
        OSD_BASE_SetFRCHeader(OSD_FRCHD_PIXELALPHAEN, TRUE);
        OSD_BASE_SetFRCHeader(OSD_FRCHD_GLOBALALPHAEN, FALSE);
    }
    else
    {
        OSD_BASE_SetFRCHeader(OSD_FRCHD_GLOBALALPHAEN, TRUE);
        OSD_BASE_SetFRCHeader(OSD_FRCHD_PIXELALPHAEN, FALSE);
    }

    OSD_RGN_Get(u4Region, OSD_RGN_POS_Y, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_DISPYOFF, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_POS_X, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_DISPXOFF, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_OUT_H, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_DISPHEIGHT, u4Value);
    OSD_RGN_Get(u4Region, OSD_RGN_OUT_W, &u4Value);
    OSD_BASE_SetFRCHeader(OSD_FRCHD_DISPWIDTH, u4Value);
    return OSD_RET_OK;
}

INT32 OSD_BASE_TransferTo(UPTR u4SrcAddr, UPTR u4DstAddr)
{
#ifndef CC_MTK_LOADER
    UINT32 u4Region, u4ColorMode, u4Pitch, u4Width, u4Height;
    u4Region = OSD_PLA_GetFirstRegion(OSD_PLANE_2);
    OSD_VERIFY_REGION(u4Region);
    OSD_RGN_Get(u4Region, OSD_RGN_COLORMODE, &u4ColorMode);
    OSD_RGN_Get(u4Region, OSD_RGN_BMP_PITCH, &u4Pitch);
    OSD_RGN_Get(u4Region, OSD_RGN_BMP_W, &u4Width);
    OSD_RGN_Get(u4Region, OSD_RGN_BMP_H, &u4Height);
    OSD_BASE_TransferTo_SYNC_Header(u4Region);
    _OSD_COPY_FRCHeader(VIRTUAL(u4DstAddr));
    GFX_SetSrc((UINT8 *)u4SrcAddr, u4ColorMode, u4Pitch);
    GFX_SetDst((UINT8 *)u4DstAddr, u4ColorMode, u4Pitch);
    GFX_SetBltOpt(0, 0, 0);
    GFX_BitBlt(0, 0, 0, 1, u4Width, u4Height);
    GFX_Flush();
    OSD_RGN_Set(u4Region, OSD_RGN_BMP_ADDR, u4DstAddr);
#endif
    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_COPY_FRCHeader(UPTR upDstAddr)
{
    _OSD_COPY_FRCHeader(upDstAddr);
    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_InitFRCHeader(void)
{
    _OSD_InitFRCHeader();
    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_SetTransferCtl(UINT32 u4Enable)
{
    _OSD_BASE_SetTransferCtl(u4Enable);
    VERIFY((INT32)OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_GetTransferCtl(UINT32 *u4Enable)
{
    _OSD_BASE_GetTransferCtl(u4Enable);
    //VERIFY((INT32)OSD_RET_OK == _OSD_BASE_UpdateHwReg());
    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_GetFRCHeader(UINT32 u4SetCMD, UINT32 *u4Value)
{
    switch (u4SetCMD)
    {
        case OSD_FRCHD_FORMAT:
            return _OSD_FRCHeader_GetFormat(u4Value);

        case OSD_FRCHD_PIXORDER:
            return _OSD_FRCHeader_GetPixOrder(u4Value);

        case OSD_FRCHD_LRINDEX:
            return _OSD_FRCHeader_GetLRindex(u4Value);

        case OSD_FRCHD_BMPINDEX:
            return _OSD_FRCHeader_GetBmpindex(u4Value);

        case OSD_FRCHD_FRAMENUM:
            return _OSD_FRCHeader_GetFramenum(u4Value);

        case OSD_FRCHD_PREFIX:
            return _OSD_FRCHeader_GetPrefix(u4Value);

        case OSD_FRCHD_BMPHEIGHT:
            return _OSD_FRCHeader_GetBmpHeight(u4Value);

        case OSD_FRCHD_BMPWIDTH:
            return _OSD_FRCHeader_GetBmpWidth(u4Value);

        case OSD_FRCHD_GLOBALALPHA:
            return _OSD_FRCHeader_GetGlobalAlpha(u4Value);

        case OSD_FRCHD_PIXELALPHAEN:
            return _OSD_FRCHeader_GetPixelAlphaEn(u4Value);

        case OSD_FRCHD_GLOBALALPHAEN:
            return _OSD_FRCHeader_GetGlobalAlphaEn(u4Value);

        case OSD_FRCHD_ODPUPPER:
            return _OSD_FRCHeader_GetOsdUpper(u4Value);

        case OSD_FRCHD_DISPYOFF:
            return _OSD_FRCHeader_GetDispYOFF(u4Value);

        case OSD_FRCHD_DISPXOFF:
            return _OSD_FRCHeader_GetDispXOFF(u4Value);

        case OSD_FRCHD_DISPHEIGHT:
            return _OSD_FRCHeader_GetDispHeight(u4Value);

        case OSD_FRCHD_DISPWIDTH:
            return _OSD_FRCHeader_GetDispWidth(u4Value);

        default:
            break;
    }

    return (INT32)OSD_RET_OK;
}

INT32 OSD_BASE_SetFRCHeader(UINT32 u4SetCMD, UINT32 u4Value)
{
    switch (u4SetCMD)
    {
        case OSD_FRCHD_FORMAT:
            _OSD_FRCHeader_SetFormat(u4Value);
            break;

        case OSD_FRCHD_PIXORDER:
            _OSD_FRCHeader_SetPixOrder(u4Value);
            break;

        case OSD_FRCHD_LRINDEX:
            _OSD_FRCHeader_SetLRindex(u4Value);
            break;

        case OSD_FRCHD_BMPINDEX:
            _OSD_FRCHeader_SetBmpindex(u4Value);
            break;

        case OSD_FRCHD_FRAMENUM:
            _OSD_FRCHeader_SetFramenum(u4Value);
            break;

        case OSD_FRCHD_PREFIX:
            _OSD_FRCHeader_SetPrefix(u4Value);
            break;

        case OSD_FRCHD_BMPHEIGHT:
            _OSD_FRCHeader_SetBmpHeight(u4Value);
            break;

        case OSD_FRCHD_BMPWIDTH:
            _OSD_FRCHeader_SetBmpWidth(u4Value);
            break;

        case OSD_FRCHD_GLOBALALPHA:
            _OSD_FRCHeader_SetGlobalAlpha(u4Value);
            break;

        case OSD_FRCHD_PIXELALPHAEN:
            _OSD_FRCHeader_SetPixelAlphaEn(u4Value);
            break;

        case OSD_FRCHD_GLOBALALPHAEN:
            _OSD_FRCHeader_SetGlobalAlphaEn(u4Value);
            break;

        case OSD_FRCHD_ODPUPPER:
            _OSD_FRCHeader_SetOsdUpper(u4Value);
            break;

        case OSD_FRCHD_DISPYOFF:
            _OSD_FRCHeader_SetDispYOFF(u4Value);
            break;

        case OSD_FRCHD_DISPXOFF:
            _OSD_FRCHeader_SetDispXOFF(u4Value);
            break;

        case OSD_FRCHD_DISPHEIGHT:
            _OSD_FRCHeader_SetDispHeight(u4Value);
            break;

        case OSD_FRCHD_DISPWIDTH:
            _OSD_FRCHeader_SetDispWidth(u4Value);
            break;

        default:
            break;
    }

    return (INT32)OSD_RET_OK;
}

#endif

INT32 OSD_BASE_SetScrnHStart(UINT32 u4_plane, UINT32 u4_value)
{
    INT32 i4_ret = OSD_RET_OK;
    OSD_VERIFY_PLANE(u4_plane);

    switch (u4_plane)
    {
        case OSD_PLANE_1:
            _OSD_BASE_SetScrnHStartOsd1(u4_value);
            break;

        case OSD_PLANE_2:
            _OSD_BASE_SetScrnHStartOsd2(u4_value);
            break;

        case OSD_PLANE_3:
            _OSD_BASE_SetScrnHStartOsd3(u4_value);
            break;
#ifdef OSD_PLA5_SUPPORT

        case OSD_PLANE_5:
            _OSD_BASE_SetScrnHStartOsd3(u4_value);
            break;
#endif

        default:
            i4_ret = OSD_RET_INV_ARG;
            break;
    }

    return i4_ret;
}

INT32 OSD_BASE_SetScrnVStart(UINT32 u4_value)
{
    INT32 i4_ret = OSD_RET_OK;
    i4_ret = _OSD_BASE_SetScrnVStart(u4_value);
    return i4_ret;
}

INT32 OSD_BASE_UpdateHwReg(void)
{
    return _OSD_BASE_UpdateHwReg();
}
#ifndef __USERDRV__
void  OSD_BASE_SwitchResolution(void)
{
#if  defined(GAL_DDI_SUPPORT)
    static BOOL fgPrev3DModeOn[OSD_SCALER_MAX_NUM]; // the flag if the previous mode was in s3d or not.
    UINT32 u4Scaler;
    UINT32 u4Plane;
    UINT32 u4OriSC_Enable = 0;
    UINT32 u4OriSrcW = 0;
    UINT32 u4OriSrcH = 0;
    UINT32 u4OriDstW = 0;
    UINT32 u4OriDstH = 0;
    UINT32 u4OriIs16Bpp = 0;
    INT32  nMaxScalers;
    BOOL fgEnable;
    UINT32 i = 0;

    if (_fgOsdInit == 0)
    {
        return;
    }

    nMaxScalers = OSD_SCALER_MAX_NUM - 1;

    if (IS_PANEL_2D_N_3D_L12R12)
    {
        nMaxScalers += 1;
    }

    for (i = 0; i < nMaxScalers; i++)
    {
        u4Scaler = i;
        u4Plane =  u4Scaler + 1;
        OSD_PLA_GetEnable(u4Plane, &fgEnable);
        OSD_PLA_Enable(u4Plane, FALSE);
        OSD_SC_GetScalerInfo(u4Scaler, &u4OriSC_Enable,
                             &u4OriSrcW, &u4OriSrcH,
                             &u4OriDstW, &u4OriDstH,
                             &u4OriIs16Bpp);

        if (u4OriDstW == 0)
        {
            u4OriDstW = OSD_BASE_GetDisplayWidth();
        }

        if (u4OriDstH == 0)
        {
            u4OriDstH  = OSD_BASE_GetDisplayHeight();
        }

        OSD_LOG_D("scaler[%d]: src_w=%d, src_h=%d, dst_w=%d, dst_h=%d\n", u4Scaler, u4OriSrcW, u4OriSrcH,
            u4OriDstW, u4OriDstH);

        if (IS_PANEL_L12R12)
        {
            if (fgPrev3DModeOn[i])
            {
                u4OriDstH <<= 1;
            }

            OSD_SC_Scale(u4Scaler, 1, u4OriSrcW, u4OriSrcH, u4OriDstW, u4OriDstH);
            fgPrev3DModeOn[i] = TRUE;
        }
        else
        {
            if (u4OriDstH > OSD_BASE_GetDisplayHeight())
            {
                u4OriDstH >>= 1;
            }

            OSD_SC_Scale(u4Scaler, 1, u4OriSrcW, u4OriSrcH, u4OriDstW, u4OriDstH);
            fgPrev3DModeOn[i] = FALSE;
        }

        OSD_LOG_D("   => switch to src_w=%d, src_h=%d, dst_w=%d, dst_h=%d\n", u4OriSrcW, u4OriSrcH, u4OriDstW,
            u4OriDstH);
        OSD_PLA_Enable(u4Plane, fgEnable);
    }

#endif
}
#endif

#if defined(__KERNEL__) && !defined(CC_UBOOT)
#if defined(CC_MT5891) || defined(CC_MT5882) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
static void vOsd1HeaderUpdateIntCB(UINT16 u2Vector)
{
    UNUSED(u2Vector);
#if !(defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893)  || defined(CC_MT5895))
    // clear global isr
    BIM_ClearIrq(VECTOR_OSD);
#endif

    if(OSD_Get_Dbg_Info(12) & 0x1)
        OSD_LOG_D("Receive OSD1 header update irq \n");
    // clea irq
    IO_WRITE32_SET(OSD_PLA1_REG, 0, 22);
    IO_WRITE32_CLEAR(OSD_PLA1_REG, 0, 22);
}

static void vOsd2HeaderUpdateIntCB(UINT16 u2Vector)
{
    UNUSED(u2Vector);
#if !(defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895))
    // clear global isr
    BIM_ClearIrq(VECTOR_OSD2);
#endif

    if(OSD_Get_Dbg_Info(12) & 0x1)
        OSD_LOG_D("Receive OSD2 header update irq \n");
    // clea irq
    IO_WRITE32_SET(OSD_PLA2_REG, 0, 22);
    IO_WRITE32_CLEAR(OSD_PLA2_REG, 0, 22);
}

static void vOsd3HeaderUpdateIntCB(UINT16 u2Vector)
{
    UNUSED(u2Vector);
#if !(defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895))
    // clear global isr
    BIM_ClearIrq(VECTOR_OSD3);
#endif

    IO_WRITE32_SET(OSD_PLA3_REG, 0, 22); //clear OSD3 header IRQ
    IO_WRITE32_CLEAR(OSD_PLA3_REG, 0, 22); //disable clear OSD3 header IRQ
}


static void vOsdVsyncIntCB(UINT16 u2Vector)
{
    UNUSED(u2Vector);
#if !(defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895))
    // clear global isr
    BIM_ClearIrq(VECTOR_OSD_VSYNC);
#endif
    // clear irq
    IO_WRITE32_SET(OSD_BASE, 0, 7);
    IO_WRITE32_CLEAR(OSD_BASE, 0, 7);
#if defined(CC_MT5886) || defined(CC_MT5863)
    OSD_MMU_ApplyEnable();
#endif

    OSD_HandleVSync();
    OSD_Check_TimingNfy();
    if(OSD_Get_Dbg_Info(10) & 0x1)
    {
        static HAL_TIME_T lastTime;
        HAL_TIME_T currentTime;
        HAL_TIME_T deltaTime;
        HAL_GetTime(&currentTime);
        HAL_GetDeltaTime(&deltaTime, &currentTime, &lastTime);
        OSD_LOG_I("vsync period = %d ms\n", ((deltaTime.u4Seconds * 1000) + deltaTime.u4Micros / 1000));
        HAL_GetTime(&lastTime);
    }
}
#endif
#endif

BOOL OSD_Base_Init_Status(void)
{
    return (_fgOsdInit == 0) ? FALSE : TRUE;
}

UINT32 OSD_BASE_GetDisplayWidth(VOID)
{
#if defined(CC_MT5890) || defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5893) || defined(CC_MT5895)
#if defined(SUPPORT_PANEL_GRAPH_MODE)

    if (fgIsOSDFromGFX())
    {
        UINT32 u4Width, u4Height;
        u4Width = GRAPHMODE_GetPanelWidth();
        u4Height = GRAPHMODE_GetPanelHeight();

        if ((u4Width == 3840) && (u4Height == 1080))
        {
            u4Width = u4Width / 2;
        }
#if defined(CC_MT5893) || defined(CC_MT5895)
        // for 8kpanel, we handle this width as panel for other fun
        if (u4Width == 3840 && u4Height == 2160 && PANEL_Is8K4K30())
        {
            u4Width = 7680;
        }

        if (u4Width == 1920 && u4Height == 2160 && PANEL_Is4K2K120())
        {
            u4Width = 3840;
        }
#endif
        return u4Width;
    }

#endif
#endif
    return PANEL_GetPanelWidth();
}

UINT32 OSD_BASE_GetDisplayHeight(VOID)
{
#if defined(CC_MT5890) || defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5893) || defined(CC_MT5895)
#if defined(SUPPORT_PANEL_GRAPH_MODE)

    if (fgIsOSDFromGFX())
    {
        return GRAPHMODE_GetPanelHeight();
    }

#endif
#endif
    return PANEL_GetPanelHeight();
}

UINT32 OSD_BASE_GetPlaneDisplayRatio(UINT32 u4Plane, UPTR u4ScalerBaseAddr)
{
    UINT32 u4Enable, u4Width, u4Height, u4Value;
    OSD_VERIFY_PLANE(u4Plane);
    _OSD_PLA_GetEnable(u4Plane, &u4Enable);

    if (u4Enable == FALSE)
    {
        return 0;
    }

    u4Value = IO_READ32(u4ScalerBaseAddr, 0x8);
    u4Width = ((u4Value & 0xFFFF0000) >> 16);
    u4Height = u4Value & 0xFFFF;

    if (u4Width == 0 || u4Height == 0)
    {
        return 0;
    }

    u4Value = IO_READ32(u4ScalerBaseAddr, 0xF8);
    u4Value = (u4Value * 100) / (u4Width * u4Height);
    return (u4Value > 100) ? 100 : u4Value;
}


VOID OSD_BASE_GetDisplayRatio(UINT32 *pu4Plane1Ratio, UINT32 *pu4Plane2Ratio,
                              UINT32 *pu4Plane3Ratio)
{
    if (pu4Plane1Ratio != NULL)
    {
        *pu4Plane1Ratio = OSD_BASE_GetPlaneDisplayRatio(OSD_PLANE_1, OSD_SC1_REG);
    }

    if (pu4Plane2Ratio != NULL)
    {
        *pu4Plane2Ratio = OSD_BASE_GetPlaneDisplayRatio(OSD_PLANE_2, OSD_SC2_REG);
    }

    if (pu4Plane3Ratio != NULL)
    {
        *pu4Plane3Ratio = OSD_BASE_GetPlaneDisplayRatio(OSD_PLANE_3, OSD_SC3_REG);
    }
}

#ifndef CC_MTK_LOADER
#ifdef CC_S_PLATFORM
VOID OSD_BASE_SetHPosition(INT32 Hstart)
{
    i4Hstart = Hstart;
}
#endif
#endif

UINT32 OSD_ColormodeToBpp(OSD_COLOR_MODE_T cm)
{
    UINT32 bpp = 1;

    switch (cm)
    {
        case OSD_CM_YCBCR_CLUT2:
        case OSD_CM_YCBCR_CLUT4:
        case OSD_CM_YCBCR_CLUT8:
        case OSD_CM_RGB_CLUT2:
        case OSD_CM_RGB_CLUT4:
        case OSD_CM_RGB_CLUT8:
        case OSD_CM_ALPHA_BIT:
            bpp = 1;
            break;
        case OSD_CM_CBYCRY422_DIRECT16:
        case OSD_CM_YCBYCR422_DIRECT16:
        case OSD_CM_RGB565_DIRECT16:
        case OSD_CM_ARGB1555_DIRECT16:
        case OSD_CM_ARGB4444_DIRECT16:
            bpp = 2;
            break;
        case OSD_CM_AYCBCR8888_DIRECT32:
        case OSD_CM_ARGB8888_DIRECT32:
            bpp = 4;
            break;
        default:
            bpp = 1;
            break;
    }

    return bpp;
}

#if defined(CC_MT5893) || defined(CC_MT5895)
// if need to save original plane state:burst8/fifo?
INT32 OSD_BASE_SetOvlDLModeEnable(BOOL enable)
{
    if (enable)
    {
        // for 4k120 & 8k30, dec is default disable, so only consider l/r module
        OSD_WRITE32MSK(4, 0x04A02040, 0xFFFFFE40);
        OSD_R_WRITE32MSK(4, 0x04A02040, 0xFFFFFE40);
        OSD_PLA_SetBurst8Enable(OSD_PLANE_3, FALSE);
        OSD_PLA_SetBurst8FifoEnable(OSD_PLANE_3, FALSE);
        OSD_PLA_SetBurst8Enable(OSD_RT_PLANE_3, FALSE);
        OSD_PLA_SetBurst8FifoEnable(OSD_RT_PLANE_3, FALSE);
    }
    else
    {
        INT32 i4Osd3Region;
        UINT32 u4RgnWidth, u4RgnHeight;
        i4Osd3Region = OSD_PLA_GetFirstRegion(OSD_PLANE_3);
        OSD_WRITE32MSK(4, 0, 0x40);
        OSD_R_WRITE32MSK(4, 0, 0x40);
        OSD_RGN_Get(i4Osd3Region, OSD_RGN_BMP_W, &u4RgnWidth);
        OSD_RGN_Get(i4Osd3Region, OSD_RGN_BMP_H, &u4RgnHeight);
        OSD_PLA_SetBurst8Enable(OSD_PLANE_3, TRUE);
        OSD_PLA_SetBurst8Enable(OSD_RT_PLANE_3, TRUE);
        if ((OSD_IsDualOstgDisplay() && u4RgnWidth == 7680 && u4RgnHeight == 4320)
            || (u4RgnWidth == 3840 && u4RgnHeight == 2160))
        {
            OSD_PLA_SetBurst8FifoEnable(OSD_PLANE_3, TRUE);
            OSD_PLA_SetBurst8FifoEnable(OSD_RT_PLANE_3, TRUE);
        }
        else
        {
            OSD_PLA_SetBurst8FifoEnable(OSD_PLANE_3, FALSE);
            OSD_PLA_SetBurst8FifoEnable(OSD_RT_PLANE_3, FALSE);
        }
    }

    _OSD_BASE_SetOvlShareOsdScaler(enable);
    _OSD_UpdateReg();

    return (INT32)OSD_RET_OK;
}

BOOL OSD_IsOstg2Display(void)
{
    return ((IS_IC_5893_ES1() && PANEL_Is4K2K60()) || (OSD_Get_Dbg_Info(0x08) & 0x1));
}

BOOL OSD_IsDualOstgDisplay(void)
{
    return ((PANEL_Is4K2K120() || PANEL_Is8K4K30() || (OSD_Get_Dbg_Info(8) & 0x2)) && !IS_DISP_4K2K_TYPE3);
}

VOID OSD_SetCropingSize(INT32 i4CropSize)
{

    if (!OSD_SUPPORT_DOUBLE_CHANNEL)
    {
        OSD_LOG_E("not support ovl online mode\n");
        return;
    }

    i4OsdCropSize = i4CropSize;
}

#endif
