/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: shuo.ding $
 * $Date: 2015/08/19 $
 * $RCSfile: osd_scaler_hw.c,v $
 * $Revision: #5 $
 *
 *---------------------------------------------------------------------------*/

/** @file osd_scaler_hw.c
 *  This header file includes function definitions of OSD scaler hardware.
 */

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "osd_hw.h"
#include "osd_drvif.h"
#include "osd_debug.h"

//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

/*lint -save -e960 */
#define OSD_SC_SETGET_TMPL(NAME, FIELD) \
    INT32 _OSD_SC_Set##NAME(UINT32 u4Scaler, UINT32 u4Value) \
    { \
        OSD_VERIFY_SCALER(u4Scaler); \
        _rOsdScalerReg[u4Scaler].rField.FIELD = u4Value; \
        return (INT32)OSD_RET_OK; \
    } \
    INT32 _OSD_SC_Get##NAME(UINT32 u4Scaler, UINT32 *pu4Value) \
    { \
        OSD_VERIFY_SCALER(u4Scaler); \
        if(pu4Value == NULL) \
        { \
            return -(INT32)OSD_RET_INV_ARG; \
        } \
        *pu4Value = _rOsdScalerReg[u4Scaler].rField.FIELD; \
        return (INT32)OSD_RET_OK; \
    } \

/* above line is intendedly left blanc */
/*lint -restore */

//-----------------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------

static OSD_SC_UNION_T _rOsdScalerReg[OSD_SCALER_MAX_NUM];
#if defined(CC_DYNAMIC_IO_ADDRESS)
#ifdef __KERNEL__
#if !defined(CC_UBOOT) && !defined(CC_MTK_LOADER)
#undef IO_VIRT
#define IO_VIRT IO_VIRT_FIX
#endif
#endif
#endif
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
static volatile OSD_SC_UNION_T *_prHwOsdScalerReg[OSD_SCALER_MAX_NUM] = {NULL};
#else
static volatile OSD_SC_UNION_T *_prHwOsdScalerReg[] =
{
    (OSD_SC_UNION_T *)OSD_SC1_REG,
    (OSD_SC_UNION_T *)OSD_SC2_REG,
    (OSD_SC_UNION_T *)OSD_SC3_REG
};
#endif
#if defined(CC_DYNAMIC_IO_ADDRESS)
#ifdef __KERNEL__
#if !defined(CC_UBOOT) && !defined(CC_MTK_LOADER)
#undef IO_VIRT
#define IO_VIRT IO_VIRT_DYN
#endif
#endif
#endif

//-----------------------------------------------------------------------------
// Inter-file functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/** Brief
 *  @param
 *  @return
 */
//-----------------------------------------------------------------------------
#if defined(CC_MT5891) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5893) || defined(CC_MT5895)
void _OSD_SC_InitHwReg(void)
{
#if defined(CC_MT5895)
    _prHwOsdScalerReg[0] = (OSD_SC_UNION_T *)(OSD_SC1_REG + OFFSET_64BIT_SRE);
    _prHwOsdScalerReg[1] = (OSD_SC_UNION_T *)(OSD_SC2_REG + OFFSET_64BIT_SRE);
    _prHwOsdScalerReg[2] = (OSD_SC_UNION_T *)(OSD_SC3_REG + OFFSET_64BIT_SRE);
    _prHwOsdScalerReg[3] = (OSD_SC_UNION_T *)(OSD_RT_SC1_REG + OFFSET_64BIT_SRE);
    _prHwOsdScalerReg[4] = (OSD_SC_UNION_T *)(OSD_RT_SC2_REG + OFFSET_64BIT_SRE);
    _prHwOsdScalerReg[5] = (OSD_SC_UNION_T *)(OSD_RT_SC3_REG + OFFSET_64BIT_SRE);
#else
    _prHwOsdScalerReg[0] = (OSD_SC_UNION_T *)(OSD_SC1_REG + OFFSET_64BIT);
    _prHwOsdScalerReg[1] = (OSD_SC_UNION_T *)(OSD_SC2_REG + OFFSET_64BIT);
    _prHwOsdScalerReg[2] = (OSD_SC_UNION_T *)(OSD_SC3_REG + OFFSET_64BIT);
#if defined(CC_MT5893) || defined(CC_MT5895)
    _prHwOsdScalerReg[3] = (OSD_SC_UNION_T *)(OSD_RT_SC1_REG + OFFSET_64BIT);
    _prHwOsdScalerReg[4] = (OSD_SC_UNION_T *)(OSD_RT_SC2_REG + OFFSET_64BIT);
    _prHwOsdScalerReg[5] = (OSD_SC_UNION_T *)(OSD_RT_SC3_REG + OFFSET_64BIT);
#endif
#endif
}
#endif


//-----------------------------------------------------------------------------
/** Brief
 *  @param
 *  @return
 */
//-----------------------------------------------------------------------------
INT32 _OSD_SC_ResetCacheRegs(UINT32 u4Scaler)
{
    INT32 u4Idx = 0;
    OSD_VERIFY_SCALER(u4Scaler);

    for (; u4Idx < OSD_SC_REG_NUM; u4Idx++)
    {
        _rOsdScalerReg[u4Scaler].au4Reg[u4Idx]
            = _prHwOsdScalerReg[u4Scaler]->au4Reg[u4Idx];
    }

    return (INT32) OSD_RET_OK;
}

//-----------------------------------------------------------------------------
/** Brief
 *  @param
 *  @return
 */
//-----------------------------------------------------------------------------
INT32 _OSD_SC_UpdateHwReg(UINT32 u4Scaler)
{
    UINT32 u4Idx = 0;
#if defined(CC_MT5893) || defined(CC_MT5895)
    UINT32 u4RtScaler = OSD_SCALER_MAX_NUM;
#endif
    OSD_VERIFY_SCALER(u4Scaler);

    for (; u4Idx < OSD_SC_REG_NUM; u4Idx++)
    {
        _prHwOsdScalerReg[u4Scaler]->au4Reg[u4Idx]
            = _rOsdScalerReg[u4Scaler].au4Reg[u4Idx];
    }

#if defined(CC_MT5893) || defined(CC_MT5895)
    if (u4Scaler == OSD_SCALER_1 || u4Scaler == OSD_SCALER_2 || u4Scaler == OSD_SCALER_3)
    {
        u4RtScaler = u4Scaler + 3;
    }
    if (u4RtScaler < OSD_SCALER_MAX_NUM && OSD_IsOstg2Display())
    {
        for (u4Idx = 0; u4Idx < OSD_SC_REG_NUM; u4Idx++)
        {
            _rOsdScalerReg[u4RtScaler].au4Reg[u4Idx] = _rOsdScalerReg[u4Scaler].au4Reg[u4Idx];
            _prHwOsdScalerReg[u4RtScaler]->au4Reg[u4Idx] = _rOsdScalerReg[u4Scaler].au4Reg[u4Idx];
        }
    }
    if (u4RtScaler < OSD_SCALER_MAX_NUM && OSD_IsDualOstgDisplay())
    {
        for (u4Idx = 0; u4Idx < OSD_SC_REG_NUM; u4Idx++)
        {
            _prHwOsdScalerReg[u4RtScaler]->au4Reg[u4Idx] = _rOsdScalerReg[u4RtScaler].au4Reg[u4Idx];
        }
    }
#endif

    _OSD_UpdateReg();
    return (INT32) OSD_RET_OK;
}

//-----------------------------------------------------------------------------
/** dump osd scaler debug info
 *  @param  u4Scaler osd scaler
 *  @retval OSD_RET_OK success
 */
//-----------------------------------------------------------------------------
INT32 OSD_SC_Dump(UINT32 u4Scaler)
{
#if defined(CC_MT5893) || defined(CC_MT5895)
    UINT32 u4Enable, u4SrcW, u4SrcH, u4DstW, u4DstH, u4Is16Bpp;
#endif
    OSD_SC_UNION_T *prOsdScalerReg = NULL;
    OSD_VERIFY_SCALER(u4Scaler);

#if defined(CC_MT5893) || defined(CC_MT5895)
    if (OSD_IsDualOstgDisplay() && u4Scaler <= OSD_SCALER_3)
    {
        OSD_SC_GetScalerInfo(u4Scaler, &u4Enable, &u4SrcW, &u4SrcH, &u4DstW, &u4DstH, &u4Is16Bpp);

        LOG(0,"Sw Scaler Config[%d]:\n", u4Scaler);
        LOG(0,"Enable=%d, SrcSize=%dx%d, DstSize=%dx%d\n", u4Enable, u4SrcW, u4SrcH, u4DstW, u4DstH);
    }
#endif

    prOsdScalerReg = (OSD_SC_UNION_T *)_prHwOsdScalerReg[u4Scaler];
    LOG(0,"OSD SCALER[%d]\n", u4Scaler);
    LOG(0,"    Enable=(SCALER_EN=%d, VUSC_EN=%d, VDSC_EN=%d, HUSC_EN=%d, HDSC_EN=%d)\n",
           prOsdScalerReg->rField.fgScEn,
           prOsdScalerReg->rField.fgVuscEn,
           prOsdScalerReg->rField.fgVdscEn,
           prOsdScalerReg->rField.fgHuscEn,
           prOsdScalerReg->rField.fgHdscEn);
    LOG(0,"    SrcSize=(%d, %d), DstDize=(%d, %d), VSC_HSIZE=%d\n",
           prOsdScalerReg->rField.u4SrcHSize,
           prOsdScalerReg->rField.u4SrcVSize,
           prOsdScalerReg->rField.u4DstHSize,
           prOsdScalerReg->rField.u4DstVSize,
           prOsdScalerReg->rField.u4VscHSize);
    return (INT32) OSD_RET_OK;
}

OSD_SC_SETGET_TMPL(VdscEn, fgVdscEn)
OSD_SC_SETGET_TMPL(HdscEn, fgHdscEn)
OSD_SC_SETGET_TMPL(HdscStep, u4HdscStep)
OSD_SC_SETGET_TMPL(HdscOfst, u4HdscOfst)
OSD_SC_SETGET_TMPL(VuscEn, fgVuscEn)
OSD_SC_SETGET_TMPL(HuscEn, fgHuscEn)
OSD_SC_SETGET_TMPL(ScLpfEn, fgScLpfEn)
OSD_SC_SETGET_TMPL(ScEn, fgScEn)
OSD_SC_SETGET_TMPL(ScPtEn, fgScPtEn)
OSD_SC_SETGET_TMPL(SrcVSize, u4SrcVSize)
OSD_SC_SETGET_TMPL(SrcHSize, u4SrcHSize)
OSD_SC_SETGET_TMPL(DstVSize, u4DstVSize)
OSD_SC_SETGET_TMPL(DstHSize, u4DstHSize)
OSD_SC_SETGET_TMPL(VscHSize, u4VscHSize)
OSD_SC_SETGET_TMPL(HuscStep, u4HuscStep)
OSD_SC_SETGET_TMPL(HuscOfst, u4HuscOfst)
OSD_SC_SETGET_TMPL(VscOfstTop, u4VscOfstTop)
OSD_SC_SETGET_TMPL(VscOfstBot, u4VscOfstBot)
OSD_SC_SETGET_TMPL(VscStep, u4VscStep)
OSD_SC_SETGET_TMPL(AlphaCountEn, fgAlphaCountEn)
OSD_SC_SETGET_TMPL(ScLpfC1, u4ScLpfC1)
OSD_SC_SETGET_TMPL(ScLpfC2, u4ScLpfC2)
OSD_SC_SETGET_TMPL(ScLpfC3, u4ScLpfC3)
OSD_SC_SETGET_TMPL(AEdgeDetEnSetRGB, fgAEdgeDetEnSetRGB)
OSD_SC_SETGET_TMPL(AEdgeDetEnNear, fgAEdgeDetEnNear)
OSD_SC_SETGET_TMPL(AEdgeDetTh, u4AEdgeDetTh)
