/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/** @file pinmux.c
 *  pinmux.c contains functions to do pin selection & gpio setup.
 */


//-------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "x_bim.h"
#include "x_pdwnc.h"
#include "x_pinmux.h"
#include "x_printf.h"
#include "x_hal_5381.h"
#include "x_hal_arm.h"
#include "x_assert.h"
#include "drvcust_if.h"
#include "x_debug.h"

#ifdef DEFINE_IS_LOG
#undef DEFINE_IS_LOG
#endif
#ifdef CC_MTK_LOADER
#define DEFINE_IS_LOG(level, fmt...)   Printf(fmt)
#else /* CC_MTK_LOADER */
#define DEFINE_IS_LOG   GPIO_IsLog
#endif /* CC_MTK_LOADER */

//-------------------------------------------------------------------
// Configurations
//-------------------------------------------------------------------

//-------------------------------------------------------------------
// Constant definitions
//-------------------------------------------------------------------

//-------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------
static const UINT32 _au4PinmuxReg[TOTAL_PIN_REG+1] = {
  0, // not a pinmux register
CKGEN_PMUX0,
CKGEN_PMUX1,
CKGEN_PMUX2,
CKGEN_PMUX3,
CKGEN_PMUX4,
CKGEN_PMUX5,
CKGEN_PMUX6,
CKGEN_PMUX7,
CKGEN_PMUX8,
CKGEN_PMUX9,
CKGEN_PMUX10,
CKGEN_PMUX11,
CKGEN_PMUX12,
CKGEN_PMUX13,
CKGEN_PMUX14,
CKGEN_PMUX15,
CKGEN_PMUX16,
PDWNC_PINMUX0,
PDWNC_PINMUX1,
PDWNC_PINMUX2,
PDWNC_PINMUX3,
PDWNC_PINMUX4,
PDWNC_PINMUX5,
PDWNC_PINMUX6,
PDWNC_PINMUX7,
PDWNC_PINMUX8,
PDWNC_PINMUX9,
};
 
/** The gpio pin selection information.
 */
 
static const UINT32  _aau1GpioSelFunc[TOTAL_NORMAL_GPIO_NUM][2] =
{
	{PIN_GPIO0,	PINMUX_FUNCTION0 },//GPIO 0
	{PIN_GPIO1,	PINMUX_FUNCTION0 },//GPIO 1
	{PIN_GPIO2,	PINMUX_FUNCTION0 },//GPIO 2
	{PIN_GPIO3,	PINMUX_FUNCTION0 },//GPIO 3
	{PIN_GPIO4,	PINMUX_FUNCTION0 },//GPIO 4
	{PIN_GPIO5,	PINMUX_FUNCTION0 },//GPIO 5
	{PIN_GPIO6,	PINMUX_FUNCTION0 },//GPIO 6
	{PIN_GPIO7,	PINMUX_FUNCTION0 },//GPIO 7
	{PIN_GPIO8,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION7 - GPIO 8
	{PIN_DISEQC_OUT_DIG,	PINMUX_FUNCTION0 },//GPIO 9
	{PIN_DISEQC_IN_DIG,	PINMUX_FUNCTION0 },//GPIO 10
	{PIN_TUNER_FREEZE_SEQ,	PINMUX_FUNCTION0 },//GPIO 11
	{PIN_ASPDIF0,	PINMUX_FUNCTION0 },//GPIO 12
	{PIN_AOMCLK,	PINMUX_FUNCTION0 },//GPIO 13
	{PIN_AOLRCK,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 14
	{PIN_AOBCK,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 15
	{PIN_AOSDATA0,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 16
	{PIN_AOSDATA1,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION7 - GPIO 17
	{PIN_AOSDATA2,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 , PINMUX_FUNCTION7 - GPIO 18
	{PIN_AOSDATA3,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 - GPIO 19
	{PIN_AOSDATA4,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION6 - GPIO 20
	{PIN_OPWM0,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 , PINMUX_FUNCTION8 - GPIO 21
	{PIN_OPWM1,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION8 - GPIO 22
	{PIN_OPWM2,	PINMUX_FUNCTION0 },//GPIO 23
	{PIN_SPI_CLK,	PINMUX_FUNCTION0 },//GPIO 24
	{PIN_SPI_DI,	PINMUX_FUNCTION0 },//GPIO 25
	{PIN_SPI_DO,	PINMUX_FUNCTION0 },//GPIO 26
	{PIN_SPI_CS,	PINMUX_FUNCTION0 },//GPIO 27
	{PIN_INT,	PINMUX_FUNCTION0 },//GPIO 28
	{PIN_CIL_DEMOD0_CLK,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 - GPIO 29
	{PIN_CIL_TX_CLK,	PINMUX_FUNCTION0 },//GPIO 30
	{PIN_CIL_TX0_D0,	PINMUX_FUNCTION0 },//GPIO 31
	{PIN_CIL_TX0_D1,	PINMUX_FUNCTION0 },//GPIO 32
	{PIN_CIL_RX_CLK,	PINMUX_FUNCTION0 },//GPIO 33
	{PIN_CIL_RX0_D0,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 - GPIO 34
	{PIN_CIL_RX0_D1,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 - GPIO 35
	{PIN_CIL_DEMOD1_CLK,	PINMUX_FUNCTION0 },//GPIO 36
	{PIN_CIL_TX1_D0,	PINMUX_FUNCTION0 },//GPIO 37
	{PIN_CIL_TX1_D1,	PINMUX_FUNCTION0 },//GPIO 38
	{PIN_CIL_RX1_D0,	PINMUX_FUNCTION0 },//GPIO 39
	{PIN_CIL_RX1_D1,	PINMUX_FUNCTION0 },//GPIO 40
	{PIN_CI_RESET,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 41
	{PIN_GPIO42,	PINMUX_FUNCTION0 },//GPIO 42
	{PIN_GPIO43,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION8 - GPIO 43
	{PIN_GPIO44,	PINMUX_FUNCTION0 },//GPIO 44
	{PIN_GPIO45,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 45
	{PIN_GPIO46,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION4 - GPIO 46
	{PIN_IF_AGC,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION5 - GPIO 47
	{PIN_GPIO48,	PINMUX_FUNCTION0 },//GPIO 48
	{PIN_GPIO49,	PINMUX_FUNCTION0 },//GPIO 49
	{PIN_GPIO50,	PINMUX_FUNCTION0 },//GPIO 50
	{PIN_GPIO51,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 , PINMUX_FUNCTION6 - GPIO 51
	{PIN_GPIO52,	PINMUX_FUNCTION0 },//GPIO 52
	{PIN_GPIO53,	PINMUX_FUNCTION0 },//GPIO 53
	{PIN_GPIO54,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION6 , PINMUX_FUNCTION7 - GPIO 54
	{PIN_GPIO55,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION1 - GPIO 55
	{PIN_GPIO56,	PINMUX_FUNCTION1 },//GPIO 56
	{PIN_GPIO57,	PINMUX_FUNCTION1 },//GPIO 57
	{PIN_GPIO58,	PINMUX_FUNCTION1 },//GPIO 58
	{PIN_GPIO59,	PINMUX_FUNCTION1 },//GPIO 59
	{PIN_GPIO60,	PINMUX_FUNCTION1 },//GPIO 60
	{PIN_GPIO61,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION4 , PINMUX_FUNCTION7 - GPIO 61
	{PIN_GPIO62,	PINMUX_FUNCTION0 },//GPIO 62
	{PIN_GPIO63,	PINMUX_FUNCTION0 },//GPIO 63
	{PIN_GPIO64,	PINMUX_FUNCTION0 },//GPIO 64
	{PIN_GPIO65,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION4 - GPIO 65
	{PIN_GPIO66,	PINMUX_FUNCTION0 },//GPIO 66
	{PIN_GPIO67,	PINMUX_FUNCTION0 },//GPIO 67
	{PIN_GPIO68,	PINMUX_FUNCTION0 },//GPIO 68
	{PIN_GPIO69,	PINMUX_FUNCTION0 },//GPIO 69
	{PIN_GPIO70,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION5 - GPIO 70
	{PIN_GPIO71,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION3 , PINMUX_FUNCTION5 - GPIO 71
	{PIN_GPIO72,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION1 , PINMUX_FUNCTION3 , PINMUX_FUNCTION5 - GPIO 72
	{PIN_OSDA0,	PINMUX_FUNCTION0 },//GPIO 73
	{PIN_OSCL0,	PINMUX_FUNCTION0 },//GPIO 74
	{PIN_OSDA1,	PINMUX_FUNCTION0 },//GPIO 75
	{PIN_OSCL1,	PINMUX_FUNCTION0 },//GPIO 76
	{PIN_SF_CE,	PINMUX_FUNCTION1 },//GPIO 77
	{PIN_SF_CLK,	PINMUX_FUNCTION1 },//GPIO 78
	{PIN_SF_IN,	PINMUX_FUNCTION1 },//GPIO 79
	{PIN_SF_OUT,	PINMUX_FUNCTION1 },//GPIO 80
	{PIN_U1RX,	PINMUX_FUNCTION0 },//GPIO 81
	{PIN_U1TX,	PINMUX_FUNCTION0 },//GPIO 82
	{PIN_OSDA2,	PINMUX_FUNCTION0 },//GPIO 83
	{PIN_OSCL2,	PINMUX_FUNCTION0 },//GPIO 84
	{PIN_DEMOD_RST,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION1 , PINMUX_FUNCTION3 - GPIO 85
	{PIN_ASPDIF1,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 - GPIO 86
	{PIN_IF_AGC_S,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION4 , PINMUX_FUNCTION5 - GPIO 87
	{PIN_DEMOD_TSCLK,	PINMUX_FUNCTION0 },//GPIO 88
	{PIN_DEMOD_TSVAL,	PINMUX_FUNCTION0 },//GPIO 89
	{PIN_DEMOD_TSSYNC,	PINMUX_FUNCTION0 },//GPIO 90
	{PIN_DEMOD_TSDATA0,	PINMUX_FUNCTION0 },//GPIO 91
	{PIN_DEMOD_TSDATA1,	PINMUX_FUNCTION0 },//GPIO 92
	{PIN_DEMOD_TSDATA2,	PINMUX_FUNCTION0 },//GPIO 93
	{PIN_DEMOD_TSDATA3,	PINMUX_FUNCTION0 },//GPIO 94
	{PIN_DEMOD_TSDATA4,	PINMUX_FUNCTION0 },//GPIO 95
	{PIN_DEMOD_TSDATA5,	PINMUX_FUNCTION0 },//GPIO 96
	{PIN_DEMOD_TSDATA6,	PINMUX_FUNCTION0 },//GPIO 97
	{PIN_DEMOD_TSDATA7,	PINMUX_FUNCTION0 },//GPIO 98
	{PIN_EMMC_RST,	PINMUX_FUNCTION1 },//GPIO 99
	{PIN_EMMC_CLK,	PINMUX_FUNCTION1 },//GPIO 100
	{PIN_EMMC_DS,	PINMUX_FUNCTION1 },//GPIO 101
	{PIN_EMMC_CMD,	PINMUX_FUNCTION1 },//GPIO 102
	{PIN_EMMC_D0,	PINMUX_FUNCTION1 },//GPIO 103
	{PIN_EMMC_D1,	PINMUX_FUNCTION1 },//GPIO 104
	{PIN_EMMC_D2,	PINMUX_FUNCTION1 },//GPIO 105
	{PIN_EMMC_D3,	PINMUX_FUNCTION1 },//GPIO 106
	{PIN_EMMC_D4,	PINMUX_FUNCTION1 },//GPIO 107
	{PIN_EMMC_D5,	PINMUX_FUNCTION1 },//GPIO 108
	{PIN_EMMC_D6,	PINMUX_FUNCTION1 },//GPIO 109
	{PIN_EMMC_D7,	PINMUX_FUNCTION1 },//GPIO 110
	{PIN_ALIN,	PINMUX_FUNCTION0 },//GPIO 111
	{PIN_SD_CMD,	PINMUX_FUNCTION0 },//GPIO 112
	{PIN_SD_CLK,	PINMUX_FUNCTION0 },//GPIO 113
	{PIN_SD_D0,	PINMUX_FUNCTION0 },//GPIO 114
	{PIN_SD_D1,	PINMUX_FUNCTION0 },//GPIO 115
	{PIN_SD_D2,	PINMUX_FUNCTION0 },//GPIO 116
	{PIN_SD_D3,	PINMUX_FUNCTION0 },//PINMUX_FUNCTION5 , PINMUX_FUNCTION8 - GPIO 117
	{PIN_USB_DM_P0,	PINMUX_FUNCTION0 },//GPIO 118
	{PIN_USB_DP_P0,	PINMUX_FUNCTION0 },//GPIO 119
	{PIN_USB_DM_P1,	PINMUX_FUNCTION0 },//GPIO 120
	{PIN_USB_DP_P1,	PINMUX_FUNCTION0 },//GPIO 121
	{PIN_USB_DM_P2,	PINMUX_FUNCTION0 },//GPIO 122
	{PIN_USB_DP_P2,	PINMUX_FUNCTION0 },//GPIO 123
	{PIN_USB_DM_P3,	PINMUX_FUNCTION0 },//GPIO 124
	{PIN_USB_DP_P3,	PINMUX_FUNCTION0 },//GPIO 125
	{PIN_MCUSYS_AGPIO,	PINMUX_FUNCTION0 },//GPIO 126
 
};
 
static const UINT32   _aau1OpctrlSelFunc[TOTAL_OPCTRL_NUM][2] =
{
	{PIN_OPCTRL0,	PINMUX_FUNCTION0 },//OPCTRL(200)
	{PIN_OPCTRL1,	PINMUX_FUNCTION0 },//OPCTRL(201)
	{PIN_OPCTRL2,	PINMUX_FUNCTION0 },//OPCTRL(202)
	{PIN_OPCTRL3,	PINMUX_FUNCTION0 },//OPCTRL(203)
	{PIN_OPCTRL4,	PINMUX_FUNCTION0 },//OPCTRL(204)
	{PIN_OPCTRL5,	PINMUX_FUNCTION0 },//OPCTRL(205)
	{PIN_OPCTRL6,	PINMUX_FUNCTION0 },//OPCTRL(206)
	{PIN_OPCTRL7,	PINMUX_FUNCTION0 },//OPCTRL(207)
	{PIN_OPCTRL8,	PINMUX_FUNCTION0 },//OPCTRL(208)
	{PIN_OPCTRL9,	PINMUX_FUNCTION0 },//OPCTRL(209)
	{PIN_OPCTRL10,	PINMUX_FUNCTION0 },//OPCTRL(210)
	{PIN_OPCTRL11,	PINMUX_FUNCTION0 },//OPCTRL(211)
	{PIN_OPCTRL12,	PINMUX_FUNCTION0 },//OPCTRL(212)
	{PIN_OPCTRL13,	PINMUX_FUNCTION0 },//OPCTRL(213)
	{PIN_OPCTRL14,	PINMUX_FUNCTION0 },//OPCTRL(214)
	{PIN_OPCTRL15,	PINMUX_FUNCTION0 },//OPCTRL(215)
	{PIN_OPCTRL16,	PINMUX_FUNCTION0 },//OPCTRL(216)
	{PIN_OPCTRL17,	PINMUX_FUNCTION0 },//OPCTRL(217)
	{PIN_OPCTRL18,	PINMUX_FUNCTION0 },//OPCTRL(218)
	{PIN_OPCTRL19,	PINMUX_FUNCTION0 },//OPCTRL(219)
	{PIN_OPCTRL20,	PINMUX_FUNCTION0 },//OPCTRL(220)
	{PIN_OPCTRL21,	PINMUX_FUNCTION0 },//OPCTRL(221)
	{PIN_OPCTRL22,	PINMUX_FUNCTION0 },//OPCTRL(222)
	{PIN_OPCTRL23,	PINMUX_FUNCTION0 },//OPCTRL(223)
	{PIN_OPCTRL24,	PINMUX_FUNCTION0 },//OPCTRL(224)
	{PIN_VGA_SDA,	PINMUX_FUNCTION0 },//OPCTRL(225)
	{PIN_VGA_SCL,	PINMUX_FUNCTION0 },//OPCTRL(226)
	{PIN_STB_SDA,	PINMUX_FUNCTION0 },//OPCTRL(227)
	{PIN_STB_SCL,	PINMUX_FUNCTION0 },//OPCTRL(228)
	{PIN_U0TX,	PINMUX_FUNCTION2 },//OPCTRL(229)
	{PIN_U0RX,	PINMUX_FUNCTION2 },//OPCTRL(230)
	{PIN_OPWRSB,	PINMUX_FUNCTION1 },//OPCTRL(231)
	{PIN_OIRI,	PINMUX_FUNCTION1 },//OPCTRL(232)
	{PIN_LED_PWM0,	PINMUX_FUNCTION0 },//OPCTRL(233)
	{PIN_LED_PWM1,	PINMUX_FUNCTION0 },//OPCTRL(234)
	{PIN_LED_PWM2,	PINMUX_FUNCTION0 },//OPCTRL(235)
	{PIN_LED_PWM3,	PINMUX_FUNCTION0 },//OPCTRL(236)
	{PIN_PHYLED0,	PINMUX_FUNCTION0 },//OPCTRL(237)
	{PIN_PHYLED1,	PINMUX_FUNCTION0 },//OPCTRL(238)
	{PIN_HDMI_2_HPD,	PINMUX_FUNCTION0 },//OPCTRL(239)
	{PIN_HDMI_2_PWR5V,	PINMUX_FUNCTION0 },//OPCTRL(240)
	{PIN_HDMI_2_SDA,	PINMUX_FUNCTION0 },//OPCTRL(241)
	{PIN_HDMI_2_SCL,	PINMUX_FUNCTION0 },//OPCTRL(242)
	{PIN_HDMI_3_HPD,	PINMUX_FUNCTION0 },//OPCTRL(243)
	{PIN_HDMI_3_SDA,	PINMUX_FUNCTION0 },//OPCTRL(244)
	{PIN_HDMI_3_SCL,	PINMUX_FUNCTION0 },//OPCTRL(245)
	{PIN_HDMI_0_SDA,	PINMUX_FUNCTION0 },//OPCTRL(246)
	{PIN_HDMI_0_SCL,	PINMUX_FUNCTION0 },//OPCTRL(247)
	{PIN_HDMI_CEC,	PINMUX_FUNCTION0 },//OPCTRL(248)
	{PIN_HDMI_0_PWR5V,	PINMUX_FUNCTION0 },//OPCTRL(249)
	{PIN_HDMI_0_HPD,	PINMUX_FUNCTION0 },//OPCTRL(250)
	{PIN_HDMI_3_PWR5V,	PINMUX_FUNCTION0 },//OPCTRL(251)
	{PIN_HDMI_1_PWR5V,	PINMUX_FUNCTION0 },//OPCTRL(252)
	{PIN_HDMI_1_SDA,	PINMUX_FUNCTION0 },//OPCTRL(253)
	{PIN_HDMI_1_SCL,	PINMUX_FUNCTION0 },//OPCTRL(254)
	{PIN_HDMI_1_HPD,	PINMUX_FUNCTION0 },//OPCTRL(255)
	{PIN_ADIN1,	PINMUX_FUNCTION1 },//OPCTRL(256)
	{PIN_ADIN2,	PINMUX_FUNCTION1 },//OPCTRL(257)
	{PIN_ADIN3,	PINMUX_FUNCTION1 },//OPCTRL(258)
	{PIN_ADIN4,	PINMUX_FUNCTION1 },//OPCTRL(259)
	{PIN_ADIN5,	PINMUX_FUNCTION1 },//OPCTRL(260)
	{PIN_ADIN6,	PINMUX_FUNCTION1 },//OPCTRL(261)
	{PIN_ADIN7,	PINMUX_FUNCTION1 },//OPCTRL(262)
	{PIN_ADIN8,	PINMUX_FUNCTION1 },//OPCTRL(263)
	{PIN_ADIN9,	PINMUX_FUNCTION1 },//OPCTRL(264)
 
};;

//-----------------------------------------------------------------------------
// Static functions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------
UINT32 BSP_SpecTran(UINT32 u4Oldpin)
{
	switch(u4Oldpin)
	{
	/*
	case PIN_CI_TSVAL:
		return PIN_CI_TSVAL_ES1;
	case PIN_CI_TSSYNC:
		return PIN_CI_TSSYNC_ES1;
	case PIN_DEMOD_TSDATA5:
		return PIN_DEMOD_TSDATA5_ES1;
	case PIN_DEMOD_TSDATA6:
		return PIN_DEMOD_TSDATA6_ES1;
	case PIN_PVR_TSVAL:
		return PIN_PVR_TSVAL_ES1;
	case PIN_PVR_TSSYNC:
		return PIN_PVR_TSSYNC_ES1;
	case PIN_PVR_TSDATA0:
		return PIN_PVR_TSDATA0_ES1;
	*/
	default:
		return u4Oldpin;
	}
}

#ifdef CC_MT5895
#define PIN_TO_GPIO(X) ((X==PIN_ADIN1)?SERVO_GPIO1:((X==PIN_ADIN2)?SERVO_GPIO2:((X==PIN_ADIN3)?SERVO_GPIO3:(X==PIN_ADIN4)?SERVO_GPIO4:\
						((X==PIN_ADIN5)?SERVO_GPIO5:((X==PIN_ADIN6)?SERVO_GPIO6:((X==PIN_ADIN7)?SERVO_GPIO7:((X==PIN_ADIN8)?SERVO_GPIO8:\
						((X==PIN_ADIN9)?SERVO_GPIO9:0))))))))

//bit[31:23] control adin9~1.
#define GPIO_TO_BIT(X) 	((X==SERVO_GPIO1)?BIT23:((X==SERVO_GPIO2)?BIT24:((X==SERVO_GPIO3)?BIT25:(X==SERVO_GPIO4)?BIT26:\
							((X==SERVO_GPIO5)?BIT27:((X==SERVO_GPIO6)?BIT28:((X==SERVO_GPIO7)?BIT29:((X==SERVO_GPIO8)?BIT30:\
							((X==SERVO_GPIO9)?BIT31:0))))))))

VOID Srvad_To_Gpio_PadmodeChange(UINT32 u4PinSel, INT32 i4Func)
{
	UINT32 u4Bit;
	INT32 i4GpioNum;
	
	i4GpioNum = PIN_TO_GPIO(u4PinSel);
	if(i4GpioNum == 0)
	{
		return ;
	}
	
	u4Bit = (UINT32)GPIO_TO_BIT(i4GpioNum);
	//if(u4Bit == 0)
	//{
	//	return ;
	//}
	LOG(1, "i4GpioNum=%d, u4Bit=0x%08x\n", i4GpioNum, u4Bit);
	if(i4GpioNum >= SERVO_GPIO1 && i4GpioNum <= SERVO_GPIO9)
	{
		if(i4Func == 1)
		{
			vIO32Write4B(PDWNC_PINMUX8,u4IO32Read4B(PDWNC_PINMUX8) | u4Bit);
		}
		else if(i4Func == 0)
		{
			vIO32Write4B(PDWNC_PINMUX8,u4IO32Read4B(PDWNC_PINMUX8) & (~u4Bit));
		}
		//LOG(0, "i4PinSel = %d, i4Func = %d\n",u4PinSel,i4Func);
		//LOG(0, "PDWNC_PINMUX8 = 0x%08x\n",u4IO32Read4B(PDWNC_PINMUX8));
	}
}
#endif

//-----------------------------------------------------------------------------
/** BSP_PinSet() to set pinmux data
 *  @param u4PinSel is the bit group of pinmuxers
 *  @param i4Func is the function we want to select
 *  @retval 0 is successful, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 BSP_PinSet(UINT32 u4PinSel, INT32 i4Func)
{
    UINT32 u4HiReg, u4HiMask, u4HiBits, u4Crit;
    UINT32 u4LoReg, u4LoMask, u4LoBits;
    UINT32 u4OldRegSet;
#if 0
	//Never usage
    UINT32 u4RegAddr;
    UINT32 u4RegSetBit_Low, u4RegSetBit_Middle, u4RegSetBit_High;
    UINT32 u4RegSetBit_Count = 0;
    UINT32 u4RegValueA, u4RegValueB, u4RegValueC;
#endif
    UNUSED(u4Crit);

    if ((!IS_VALID_MUX_REG(u4PinSel)) ||
        (i4Func < 0) ||
        (i4Func > MAX_FUNC(u4PinSel)))
    {
        return -1;
    }

	if(IS_IC_5893_ES1())
	{
		u4PinSel=BSP_SpecTran(u4PinSel);
	}	
    u4HiReg = HI_MUX_REG(u4PinSel);
    u4LoReg = LO_MUX_REG(u4PinSel);
	#if 0 //fix warning
    if ((u4HiReg > TOTAL_PIN_REG) ||
        (u4LoReg > TOTAL_PIN_REG))
    {
        return -1;
    }
	#endif
    u4HiMask = 0;
    u4HiBits = 0;
    if (u4HiReg != 0)  // with high pinmux bits
    {
        u4HiReg = _au4PinmuxReg[u4HiReg];
        u4HiMask = HI_MASK(u4PinSel);
        u4HiBits = HI_BIT_OFFSET(u4PinSel);
    }
    u4LoReg = _au4PinmuxReg[u4LoReg];
    u4LoMask = LO_MASK(u4PinSel);
    u4LoBits = LO_BIT_OFFSET(u4PinSel);
#ifndef CC_UBOOT
    u4Crit = HalCriticalStart();
#endif
	#if defined(CC_MT5895) 
	Srvad_To_Gpio_PadmodeChange(u4PinSel,i4Func);
	#endif
    if (u4HiReg != 0)
    {
        u4OldRegSet = u4IO32Read4B(u4HiReg);
        vIO32Write4B(u4HiReg, (u4IO32Read4B(u4HiReg) & (~(u4HiMask << u4HiBits))) |
                     ((UINT32)((i4Func >> (MASK_TO_BITNUM(u4LoMask) + 1)) & u4HiMask) << u4HiBits));

        if (u4OldRegSet != u4IO32Read4B(u4HiReg))
        {
            LOG(5, "Change pinmux register 0x%8x change (old: 0x%8x, new: 0x%8x)\n", u4HiReg, u4OldRegSet, u4IO32Read4B(u4HiReg));
        }
    }
    u4OldRegSet = u4IO32Read4B(u4LoReg);
    vIO32Write4B(u4LoReg, (u4IO32Read4B(u4LoReg) & (~(u4LoMask << u4LoBits))) |
                 ((UINT32)(i4Func & u4LoMask) << u4LoBits));
    if (u4OldRegSet != u4IO32Read4B(u4LoReg))
    {
        LOG(5, "Change pinmux register 0x%8x (old: 0x%8x, new: 0x%8x)\n", u4LoReg, u4OldRegSet, u4IO32Read4B(u4LoReg));
    }
#ifndef CC_UBOOT
    HalCriticalEnd(u4Crit);
#endif
    return 0;
}

//-----------------------------------------------------------------------------
/** BSP_PinGet() to get pinmux data
 *  @param u4PinSel is the bit group of pinmuxers
 *  @retval the function we select now.
 */
//-----------------------------------------------------------------------------
INT32 BSP_PinGet(UINT32 u4PinSel)
{
    UINT32 u4HiReg, u4HiMask, u4HiBits, u4HiVal, u4Crit;
    UINT32 u4LoReg, u4LoMask, u4LoBits, u4LoVal, u4Func;

    UNUSED(u4Crit);
    if (!IS_VALID_MUX_REG(u4PinSel))
    {
        return -1;
    }

	if(IS_IC_5893_ES1())
	{
		u4PinSel=BSP_SpecTran(u4PinSel);
    }

    u4HiReg = HI_MUX_REG(u4PinSel);
    u4LoReg = LO_MUX_REG(u4PinSel);
	#if 0//fix warning
    if ((u4HiReg > TOTAL_PIN_REG) ||
        (u4LoReg > TOTAL_PIN_REG))
    {
        return -1;
    }
	#endif
    u4HiMask = 0;
    u4HiBits = 0;
    if (u4HiReg != 0)  // with high pinmux bits
    {
        u4HiReg = _au4PinmuxReg[u4HiReg];
        u4HiMask = HI_MASK(u4PinSel);
        u4HiBits = HI_BIT_OFFSET(u4PinSel);
    }
    u4LoReg = _au4PinmuxReg[u4LoReg];
    u4LoMask = LO_MASK(u4PinSel);
    u4LoBits = LO_BIT_OFFSET(u4PinSel);

#ifndef CC_UBOOT
    u4Crit = HalCriticalStart();
#endif
    u4LoVal = u4IO32Read4B(u4LoReg);
    u4Func = (u4LoVal >> u4LoBits) & u4LoMask;
    if (u4HiReg != 0)
    {
        u4HiVal = u4IO32Read4B(u4HiReg);
        u4Func += ((u4HiVal >> u4HiBits) & u4HiMask) << (MASK_TO_BITNUM(u4LoMask) + 1);
    }
#ifndef CC_UBOOT
    HalCriticalEnd(u4Crit);
#endif
    return (INT32)u4Func;
}

//for multi gpio, more than one function is gpio
UINT32 BSP_PinGpioCheck(UINT32 u4Pinmux)
{
    UINT32 u4Sel;

    u4Sel = BSP_PinGet(u4Pinmux);
    switch (u4Pinmux)
    {
        case PIN_OSDA2:
            if (u4Sel == PINMUX_FUNCTION3)
            {
                return 1;
            }
            break;
        default:
            break;
    }
    return 0;
}

//-----------------------------------------------------------------------------
/** BSP_PinGpioSel() to set the pin of the gpio number at gpio function.
 *  @param i4Gpio the gpio number we want to enable to gpio function.
 *  @retval 0 is successful, otherwise failed.
 */
//-----------------------------------------------------------------------------
INT32 BSP_PinGpioSel(INT32 i4Gpio)
{
    UINT32 u4GpioSel;

    if ((i4Gpio < 0) || (i4Gpio > MAX_GPIO_NUM))
    {
        return -1;
    }

    // Skip JTAG pinmux
    if ((i4Gpio >= GPIO_JTDO) && (i4Gpio <= GPIO_JTRST_))
    {
        return 0;
    }
    if ((i4Gpio >= ADIN0) && (i4Gpio <= ADIN9))//[400, 409]
    {
        UINT32 u4Idx;
        if (i4Gpio == ADIN0)
        {
#if !defined(CC_MTK_LOADER)
            LOG(2, "(%s)error servoAD 0 have no gpio function!\n", __FUNCTION__);
#endif
            return -1;
        }
        else
        {
            u4Idx = (i4Gpio - SERVO_0_ALIAS) + ((SERVO_GPIO1 - 1) - OPCTRL(0));
        }
        u4GpioSel = _aau1OpctrlSelFunc[u4Idx][0];
        if ((u4GpioSel & MULTI_GPIO_FLAG) && (BSP_PinGpioCheck(u4GpioSel)))
        {
            return 0;
        }
        if (BSP_PinGet(u4GpioSel) == _aau1OpctrlSelFunc[u4Idx][1])
        {
            return 0;
        }
        return BSP_PinSet(_aau1OpctrlSelFunc[u4Idx][0], _aau1OpctrlSelFunc[u4Idx][1]);
    }
    else if ((i4Gpio >= OPCTRL0) && (i4Gpio < (OPCTRL0 + TOTAL_OPCTRL_NUM)))//[200, 275]
    {
        u4GpioSel = _aau1OpctrlSelFunc[i4Gpio - OPCTRL0][0];

        if ((i4Gpio >= 247) && (i4Gpio <= 257))
        {
            //null, we cann't return o before call BSP_PinSet()
        }
        else
        {
            if ((u4GpioSel & MULTI_GPIO_FLAG) && (BSP_PinGpioCheck(u4GpioSel)))
            {
                return 0;
            }
            if (BSP_PinGet(u4GpioSel) == _aau1OpctrlSelFunc[i4Gpio - OPCTRL0][1])
            {
                return 0;
            }
        }
        return BSP_PinSet(_aau1OpctrlSelFunc[i4Gpio - OPCTRL0][0], _aau1OpctrlSelFunc[i4Gpio - OPCTRL0][1]);
    }
    else if (i4Gpio < TOTAL_NORMAL_GPIO_NUM)//[0, 199]
    {
        u4GpioSel = _aau1GpioSelFunc[i4Gpio][0];
        if ((u4GpioSel & MULTI_GPIO_FLAG) && (BSP_PinGpioCheck(u4GpioSel)))
        {
            return 0;
        }
        if (BSP_PinGet(u4GpioSel) == _aau1GpioSelFunc[i4Gpio][1])
        {
            return 0;
        }
        return BSP_PinSet(_aau1GpioSelFunc[i4Gpio][0], _aau1GpioSelFunc[i4Gpio][1]);
    }
    return -1;
}

//-----------------------------------------------------------------------------
/** BSP_PinGpioGet() to Get the original function of selected gpio pin.
 *  @param i4Gpio the gpio number we want to enable to gpio function.
 *  @param pi4PinSel  return of logical gpio mapping physical pin.
 *  @retval the function we select now.
 */
//-----------------------------------------------------------------------------
INT32 BSP_PinGpioGet(INT32 i4Gpio, UINT32* pu4PinSel)
{
    if ((i4Gpio < 0) || (i4Gpio > MAX_GPIO_NUM))
    {
        return -1;
    }

    if (pu4PinSel == NULL)
    {
        return -1;
    }

    if ((i4Gpio >= ADIN0) && (i4Gpio <= ADIN9))
    {
        UINT32 u4Idx;
        if (i4Gpio == ADIN0)
        {
#if !defined(CC_MTK_LOADER)
            LOG(2, "(%s)error servoAD 0 have no gpio function!\n", __FUNCTION__);
#endif
            return -1;
        }
        else
        {
            u4Idx = (i4Gpio - SERVO_0_ALIAS) + ((SERVO_GPIO1 - 1) - OPCTRL(0));
        }
        *pu4PinSel =  _aau1OpctrlSelFunc[u4Idx][0];
        return BSP_PinGet(_aau1OpctrlSelFunc[u4Idx][0]);
    }
    else if ((i4Gpio >= OPCTRL0) && (i4Gpio < (OPCTRL0 + TOTAL_OPCTRL_NUM)))
    {
        *pu4PinSel =  _aau1OpctrlSelFunc[i4Gpio - OPCTRL0][0];
        return BSP_PinGet(_aau1OpctrlSelFunc[i4Gpio - OPCTRL0][0]);
    }
    else if (i4Gpio < TOTAL_NORMAL_GPIO_NUM)
    {
        *pu4PinSel =  _aau1GpioSelFunc[i4Gpio][0];
        return BSP_PinGet(_aau1GpioSelFunc[i4Gpio][0]);
    }
    return -1;
}

