#include "download_command.h"
#include "fastboot.h"
#include "sparse_format.h"
#include <common.h>
#include <command.h>
#include <jffs2/jffs2.h>
#include <mmc.h>
#include <bootimg.h>
#ifdef CONFIG_USB_FASTBOOT

extern void *download_base;
extern unsigned download_max;
extern unsigned download_size;
extern unsigned fastboot_state;
extern struct fastboot_var *varlist;

static int critical_lock = 1;

static const unsigned char wipedata_bin[] = {
    0x62, 0x6f, 0x6f, 0x74, 0x2d, 0x72, 0x65, 0x63, 0x6f, 0x76, 0x65, 0x72, 
    0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x72, 0x65, 0x63, 0x6f, 0x76, 0x65, 0x72, 0x79, 
    0x0a, 0x2d, 0x2d, 0x77, 0x69, 0x70, 0x65, 0x5f, 0x64, 0x61, 0x74, 0x61, 
    0x0a
};

static unsigned hex2unsigned(const char *x)
{
    unsigned n = 0;

    while(*x) {
        switch(*x) {
        case '0': case '1': case '2': case '3': case '4':
        case '5': case '6': case '7': case '8': case '9':
            n = (n << 4) | (*x - '0');
            break;
        case 'a': case 'b': case 'c':
        case 'd': case 'e': case 'f':
            n = (n << 4) | (*x - 'a' + 10);
            break;
        case 'A': case 'B': case 'C':
        case 'D': case 'E': case 'F':
            n = (n << 4) | (*x - 'A' + 10);
            break;
        default:
            return n;
        }
        x++;
    }

    return n;
}

void download_value_init(void)
{
	#ifdef CC_SUPPORT_AVB
	download_base = CONFIG_SYS_MALLOC_START + CONFIG_SYS_MALLOC_LEN;
	#else
	download_base = CFG_LOAD_ADDR + 0x100000;
	#endif

	download_size = 0;
	return;
}

static int download_standard(u32 data_length)
{
	int read_length;

	printf("USB Transferring ... buffer[0x%08x], len: %d Byte.\n", download_base, data_length);

	read_length = usb_read(download_base, data_length);
	if ((read_length < 0) || ((unsigned) read_length != data_length))
	{
		printf("Read USB error, read_length = %d Byte, data_length = %d Byte.\n", read_length, data_length);
		fastboot_state = STATE_ERROR;
		return -1;
	}

	download_size = data_length;

	printf("read OK. data_length: 0x%x\n", data_length);
	return 0;
}

void do_resetBootloader(const char *arg, const char *data, unsigned int data_length)
{
	printf("rebooting the device to bootloader\n");
	fastboot_okay("");
 	do_reset(NULL,NULL,NULL,NULL);
}
void do_resetRecovery(const char *arg, const char *data, unsigned int data_length)
{

 
    unsigned char boot_cmd[32] = "boot-recovery";
    char boot_msg[64];
    memset(boot_msg, 0, 64);
    extern int emmc_write_to_partition(unsigned char* buf, char* pname, u64 offset, unsigned int size);
    //clear fastboot mode and enter recovery mode
    emmc_write_to_partition((unsigned char*)boot_msg, "misc", 512, sizeof("fastboot"));
    emmc_write_to_partition(boot_cmd, "misc", 0, sizeof(boot_cmd));
    fastboot_okay("Rebooting the device to recovery mode");
    do_reset(NULL,NULL,NULL,NULL);

}


void do_resetFastboot(const char *arg, const char *data, unsigned int data_length)
{

 
    unsigned char boot_cmd[32] = "boot-recovery";
    unsigned char boot_status[32] = "recovery\n--fastboot\n";
    char boot_msg[64];
    memset(boot_msg, 0, 64);
    extern int emmc_write_to_partition(unsigned char* buf, char* pname, u64 offset, unsigned int size);
    //clear fastboot mode and enter fastbootd mode(userspace fastboot)
    emmc_write_to_partition((unsigned char*)boot_msg, "misc", 512, sizeof("fastboot"));
    emmc_write_to_partition(boot_cmd, "misc", 0, sizeof(boot_cmd));
    emmc_write_to_partition(boot_status, "misc", 64, sizeof(boot_status));
    fastboot_okay("Rebooting the device to userspacde fastboot");
    do_reset(NULL,NULL,NULL,NULL);

}

void do_continue(const char *arg, const char *data, unsigned int data_length)
{
	printf("continue booting the device ...\n");
	fastboot_okay("");
	fastboot_state = STATE_ERROR;
}

void do_reboot(const char *arg, const char *data, unsigned int data_length)
{
	printf("rebooting the device ...\n");
	extern int emmc_write_to_partition(unsigned char* buf, char* pname, u64 offset, unsigned int size);
	emmc_write_to_partition((unsigned char*)"00000000", "misc", 512, sizeof("fastboot"));
	fastboot_okay("");
 	do_reset(NULL,NULL,NULL,NULL);
}

int do_download(const char *arg, const char *data, unsigned int data_length )
{
	char response[64];
	unsigned len = hex2unsigned(data);
	printf("do_download begin, len = %d Byte.\n", len);

	if(len == 0)
	{
		fastboot_okay("download OK");
		return 0;
	}

	sprintf(response, "DATA%08x", len);
	if (usb_write(response, strlen(response)) < 0)
	{
	    printf("cmd_download -- usb write fail\n");
		return -1;
	}

	download_standard(len);
	fastboot_okay("download OK");

    return 0;
}

#ifdef CC_SUPPORT_AVB
#define CHUNK_TYPE_FILL_BUFFER_SIZE (2*1024*1024) //2MB.
#else
#define CHUNK_TYPE_FILL_BUFFER_SIZE (64*1024)     //64KB.
#endif

#define CMD_FLASH_TO_EMMC(data,offset,sz)														\
do {																							\
    char *local_args[7];																		\
    char p0[64], p1[32], p2[32], p3[32];														\
	local_args[0] = "mmc";																		\
    local_args[1] = "write.b";																	\
    snprintf(p0, 64, "0x%lx", CONFIG_SYS_MMC_ENV_DEV);												\
    local_args[2] = p0;																			\
    snprintf(p1, 32, "0x%lx", data); 																\
    local_args[3] = p1;																			\
    snprintf(p2, 32, "0x%llx", offset); 																\
    local_args[4] = p2;																			\
    snprintf(p3, 32, "0x%lx", sz);																	\
    local_args[5] = p3;																			\
    if (do_mmcops(NULL, 0, 6, local_args)) {	fastboot_fail("mmc cops fail");	 return -1;	}	\											
} while(0)

int cmd_flash_emmc_sparse_img(char * data, unsigned long long offset,unsigned int sz)
{
	unsigned int chunk;
	unsigned int chunk_data_sz;
	sparse_header_t *sparse_header;
	chunk_header_t *chunk_header;
	unsigned int total_blocks = 0;
    char *local_args[7];
    char p0[64], p1[32], p2[32], p3[32];
	printf( "Enter cmd_flash_sparse_img()\n");


	/* Read and skip over sparse image header */
	sparse_header = (sparse_header_t *) data;
	data += sparse_header->file_hdr_sz;
	if(sparse_header->file_hdr_sz > sizeof(sparse_header_t))
	{
		/* Skip the remaining bytes in a header that is longer than we expected. */
		data += (sparse_header->file_hdr_sz - sizeof(sparse_header_t));
	}

	printf ("=== Sparse Image Header ===\n");
	printf ("magic: 0x%x\n", sparse_header->magic);
	printf ("major_version: 0x%x\n", sparse_header->major_version);
	printf ("minor_version: 0x%x\n", sparse_header->minor_version);
	printf ("file_hdr_sz: %d\n", sparse_header->file_hdr_sz);
	printf ("chunk_hdr_sz: %d\n", sparse_header->chunk_hdr_sz);
	printf ("blk_sz: %d\n", sparse_header->blk_sz);
	printf ("total_blks: %d\n", sparse_header->total_blks);
	printf ("total_chunks: %d\n", sparse_header->total_chunks);

	printf("\nWriting Flash ...\n");
	/* Start processing chunks */
	for (chunk=0; chunk<sparse_header->total_chunks; chunk++)
	{
		/* Read and skip over chunk header */
		chunk_header = (chunk_header_t *) data;
		data += sizeof(chunk_header_t);

		printf ("\n=== Chunk Header ===\n");
		printf ("chunk_type:    0x%x\n", chunk_header->chunk_type);
		printf ("chunk_data_sz: 0x%08x\n", chunk_header->chunk_sz);
		printf ("total_size:    0x%08x\n", chunk_header->total_sz);

		if(sparse_header->chunk_hdr_sz > sizeof(chunk_header_t))
		{
			/* Skip the remaining bytes in a header that is longer than we expected. */
			data += (sparse_header->chunk_hdr_sz - sizeof(chunk_header_t));
		}

		chunk_data_sz = sparse_header->blk_sz * chunk_header->chunk_sz;
		switch (chunk_header->chunk_type)
		{
		case CHUNK_TYPE_RAW:
			chunk_data_sz = chunk_header->chunk_sz * sparse_header->blk_sz;
			if(chunk_header->total_sz != (sparse_header->chunk_hdr_sz + chunk_data_sz))
			{
				fastboot_fail("Error chunk size for chunk type Raw.");
				return -1;
			}

			CMD_FLASH_TO_EMMC((unsigned long)data,(offset + ((uint64_t)total_blocks*sparse_header->blk_sz)),chunk_data_sz);

			total_blocks += chunk_header->chunk_sz;
			data += chunk_data_sz;
			break;

        case CHUNK_TYPE_FILL:
            chunk_data_sz = 4;
            if(chunk_header->total_sz != (sparse_header->chunk_hdr_sz + chunk_data_sz))
            {
                fastboot_fail("Error chunk size for chunk type Fill.");
                return -1;
            }
            else
            {
                static uint32_t *fill_data_buffer = NULL;
                uint32_t fill_data_size = chunk_header->chunk_sz * sparse_header->blk_sz;
                uint32_t fill_data = *(uint32_t *)data;
                uint32_t data_size_integer = fill_data_size/CHUNK_TYPE_FILL_BUFFER_SIZE;
                uint32_t data_size_remainder = fill_data_size%CHUNK_TYPE_FILL_BUFFER_SIZE;
                int i = 0;

                if(fill_data_buffer == NULL)
                {
                    fill_data_buffer = (uint32_t *)malloc(CHUNK_TYPE_FILL_BUFFER_SIZE);
                }
                // Fill buffer
                for(i=0; i<(CHUNK_TYPE_FILL_BUFFER_SIZE)/sizeof(uint32_t); i++)
                {
                    *(fill_data_buffer + i) = fill_data;
                }
                // Write buffer to emmc
                for(i=0; i<data_size_integer; i++)
                {
                    CMD_FLASH_TO_EMMC((unsigned long)fill_data_buffer,(offset + ((uint64_t)total_blocks*sparse_header->blk_sz) + i*CHUNK_TYPE_FILL_BUFFER_SIZE), CHUNK_TYPE_FILL_BUFFER_SIZE);
                }
                if(data_size_remainder != 0)
                {
                    CMD_FLASH_TO_EMMC((unsigned long)fill_data_buffer,(offset + ((uint64_t)total_blocks*sparse_header->blk_sz) + data_size_integer*CHUNK_TYPE_FILL_BUFFER_SIZE), data_size_remainder);
                }
            }
            total_blocks += chunk_header->chunk_sz;
            data += chunk_data_sz;
            break;

		case CHUNK_TYPE_DONT_CARE:
			chunk_data_sz = 0;
			if(chunk_header->total_sz != (sparse_header->chunk_hdr_sz + chunk_data_sz))
			{
				fastboot_fail("Error chunk size for chunk type Dont Care.");
				return -1;
			}
			total_blocks += chunk_header->chunk_sz;
			data += chunk_data_sz;
			break;

		case CHUNK_TYPE_CRC:
			chunk_data_sz = 4;
			if(chunk_header->total_sz != (sparse_header->chunk_hdr_sz + chunk_data_sz))
			{
				fastboot_fail("Error chunk size for chunk type CRC.");
				return -1;
			}
			total_blocks += chunk_header->chunk_sz;
			data += chunk_data_sz;
			break;

		default:
			fastboot_fail("Unknown chunk type");
			return -1;
		}
	}

	printf("\nWrote %d blocks, expected to write %d blocks\n",total_blocks, sparse_header->total_blks);

	if(total_blocks != sparse_header->total_blks)
	{
		fastboot_fail("sparse image write failure");
		return -1;
	}

	fastboot_okay("Write Flash OK");

	return 0;;
}

/* Ported by Zhendong Wang starts for fastboot supporting packed image */

/* Note: This function must accordate with the the u32_to_u8 function in
 * image_pack.c. Otherwise, the parsed u32 is incorrect
 */
void u8_array_to_u32( const u8 * array, u32 * num ) {
    u32 i = 0;
    *num = 0;
    for( i = 0; i < 4; i++ ) {
        *num <<= 8;
        *num += array[i];
    }
} 

int do_update_packed_images(void)
{
    int i;
    u8 *pTmp, *name;
    img_hdr *images_header;
    sparse_header_t *sparse_header;
    u32 size, partition_num, num_images, images_size, header_size;
    u64 partition_size, partition_offset;

    images_header = download_base;
    pTmp = download_base;

    u8_array_to_u32(&images_header->num_images, &num_images);
    u8_array_to_u32(&images_header->images_size, &images_size);
    u8_array_to_u32(&images_header->header_size, &header_size);
    if (num_images < 1) {
	printf("This packed images contain no image.\n");
	fastboot_fail("The packed images contain no image");
	return MULTI_IMAGE_ERROR;
    }

    pTmp += header_size;
    struct {
	u8 name[PARTITION_NAME_SIZE];
	u32 size;
    } *img_info = images_header->img_info;

    for (i = 0; i < num_images; i++) {
	name = img_info[i].name;
	u8_array_to_u32(&img_info[i].size, &size);

	if (find_partition(name, &partition_num,
		    &partition_size, &partition_offset)) {
	    printf("The partition %s is not defined.\n", name);
	    fastboot_fail("The partition is not defined");
	    return MULTI_IMAGE_ERROR;
	}
	if (size > partition_size) {
	    printf("Image size is larger than the partition size.\n");
	    fastboot_fail("Image size is larger than the partition size");
            return MULTI_IMAGE_ERROR;
	}

	sparse_header = pTmp;
	printf("Write partition %s.\n", name);

	if (sparse_header->magic != SPARSE_HEADER_MAGIC) {
	    CMD_FLASH_TO_EMMC((unsigned long)pTmp, partition_offset, size);
	    fastboot_okay("finished");
	} else {
	    if (cmd_flash_emmc_sparse_img(pTmp,
			partition_offset, size)) {
		fastboot_fail("mmc cops fail");
		return  MULTI_IMAGE_ERROR;
	    }
	    fastboot_okay("finished");
	}
	pTmp += size;
    }

    return MULTI_IMAGE_WRITE_SUCCESS;
}

/* Ported by Zhendong Wang starts for fastboot supporting packed image ends */

int do_update(const char *arg, const char *data, unsigned int data_length)
{
    const char *partition_name = data;
    const char *download_buffer = download_base;
    sparse_header_t *sparse_header = (sparse_header_t *)download_buffer;
    u32 download_length = download_size;

    u8 part_num;
    u64 part_size, part_offset;

    /* part modified by David Li starts for fastboot supporting packed image */
    if (strncmp((const char *)download_buffer, "BOOTLDR!", 8) == 0) {
        return do_update_packed_images();
    }
    /* part modified by David Li ends */

    printf("do_update begin, update size = %d Byte.\n", download_length);

#ifdef  CC_ANDROID_SECURE_BOOT
    {
        char *dev_state = getenv("devicestate");
        if (dev_state == NULL || strcmp(dev_state, "unlock") != 0)
        {
            printf("device locked, forbid to write flash.\n");
            fastboot_fail("device locked, forbid to write flash.");
            return -1;
        }
    }
#endif//CC_ANDROID_SECURE_BOOT

    if (find_partition(partition_name, &part_num, &part_size, &part_offset))
    {
        printf("the %s partition not defined at mtdparts!\n", partition_name);
        fastboot_fail("can not find the partition");
        return -1;
    }

    if (critical_lock == 1 && strncmp(partition_name, "uboot", 5) == 0)
    {
        printf("critical locked, forbid to write flash.\n");
        fastboot_fail("critical locked, forbid to write flash.");
        return -1;
    }

    printf("cmd_flash_emmc, partition: %s, size: 0x%08x.\nsparse_header->magic = 0x%x.\n", partition_name, download_length, sparse_header->magic);

    if (download_length > part_size)
    {
        printf("update abort! file size is larger than partition size.\n");
        fastboot_fail("file size is larger than partition size");
        return -1;
    }

    if (sparse_header->magic != SPARSE_HEADER_MAGIC)
    {
        CMD_FLASH_TO_EMMC((unsigned long)download_buffer, part_offset, download_length);
        fastboot_okay("Write Flash OK");
    }
    else
    {
        if (cmd_flash_emmc_sparse_img(download_buffer, part_offset, download_length))
        {
            fastboot_fail("mmc cops fail");
            return -1;
        }
        fastboot_okay("Write Flash OK");
    }
    printf("do_update Done!!\n");
    return 0;
}

int do_getvar(const char *arg, const char *data, unsigned int data_length )
{
	struct fastboot_var *var;
	char response[64];
	printf("do_getvar, var[%s].\n", data);

	if(!strcmp(data, "all")){
		for (var = varlist; var; var = var->next){
			if(var->handle)
			{
				var->handle(var->name,data,data_length);
			}
			else
			{
				sprintf(response, "\t%s: %s", var->name, var->value);
				fastboot_info(response);
			}
		}
		fastboot_okay("Done!!");
		return 0;
	}

	for (var = varlist; var; var = var->next) {
		if (!strncmp(var->name, data,strlen(var->name))) {
			if(var->handle)
			{
				var->handle(var->name,data + strlen(var->name),data_length - strlen(var->name));
			}
			else
			{
				fastboot_okay(var->value);
			}
			return 0;
		}
	}

	printf("invalid var[%s].\n", data);
	fastboot_fail("invalid var");
	return -1;
}

int do_erase_emmc(const char *arg, const char *data, unsigned int data_length )
{
    u8 part_num;
    u64 part_size, part_offset;
    u32 sz = sizeof(wipedata_bin)/sizeof(wipedata_bin[0]);
    char *dev_state = getenv("devicestate");
    struct mmc *mmc;

    mmc = find_mmc_device(CONFIG_SYS_MMC_ENV_DEV);
    if(!mmc)
    {
        printf("%s:%d:%s: Have no MMC device.\n",__FILE__,__LINE__,__FUNCTION__);
        fastboot_fail("Have no MMC device.");
        return -1;
    }
    if(mmc_init(mmc))
    {
        printf("%s:%d:%s: MMC init failed.\n",__FILE__,__LINE__,__FUNCTION__);
        fastboot_fail("MMC init failed.");
        return -1;
    }

    if (find_partition("misc", &part_num, &part_size, &part_offset))
    {
        printf("the %s partition not defined at mtdparts!\n", "misc" );
        fastboot_fail("can not find the misc partition.");
        return -1;
    }

    if (!strcmp("userdata", data)) {
        printf("begin to erase userdata, please waiting ...\n");
        data = (const char*)wipedata_bin;
        CMD_FLASH_TO_EMMC((unsigned long)data,part_offset,sz);
        printf("MMC erase userdata done.\n");
        fastboot_okay("Done!!");
        return 0;
    }
    else if(strcmp(dev_state,"unlock") == 0)
    {
        if(strcmp("uboot", data) == 0)
        {
            printf("devicestate is unlock, did not support to erase partition: %s\n",data);
            fastboot_fail("did not support to erase uboot partition.");
            return -1;
        }

        if(find_partition(data, &part_num, &part_size, &part_offset))
        {
            printf("the %s partition not defined at mtdparts!\n", data);
            fastboot_fail("can not find the target partition.");
            return -1;
        }

        printf("begin to erase %s, please waiting ...\n",data);
        if(mmc->block_dev.block_erase(CONFIG_SYS_MMC_ENV_DEV, (part_offset>>9), (part_size>>9), 2))
        {
            printf("MMC erase %s failed.\n",data);
            fastboot_fail("MMC erase failed.");
            return -1;
        }
        else
        {
            printf("MMC erase %s done.\n",data);
            fastboot_okay("Done!!");
            return 0;
        }
    }
    else // if device state is lock, only allow to erase userdata partition.
    {
        printf("devicestate is lock, did not support to erase partition: %s\n",data);
        fastboot_fail("only support erase userdata partition.");
        return -1;
    }
}

#ifdef  CC_ANDROID_SECURE_BOOT_CLASS_B
static void TrimLeft(char *pDest, const char *pSrc, int len)
{
    int i = 0;
    const char *t = pSrc;
    while (i < len && (*(t++) == ' '));
    strcpy(pDest, t - 1);
}

static void EraseUserData(void)
{
    u8 part_num;
    u64 part_size, part_offset;
    unsigned int sz = sizeof(wipedata_bin) / sizeof(wipedata_bin[0]);

    if (find_partition("misc", &part_num, &part_size, &part_offset))
    {
        printf("the %s partition not defined at mtdparts!\n", "misc" );
        return -1;
    }
    printf("begin erase userdata.\n");
    CMD_FLASH_TO_EMMC((unsigned long)wipedata_bin, part_offset, sz);
    printf("end erase userdata.\n");
}
#endif//CC_ANDROID_SECURE_BOOT_CLASS_B

int do_flashing(const char *arg, const char *data, unsigned int data_length)
{
    int ret;
#ifdef  CC_ANDROID_SECURE_BOOT_CLASS_B
    char TempData[20] = {0};
    char *dev_state = getenv("devicestate");

    ret = 0;
    TrimLeft(TempData, data, strlen(data));
    if (strcmp("get_unlock_ability", TempData) == 0)
    {
        fastboot_info(dev_state);
    }
    else if (strcmp("lock", TempData) == 0 || strcmp("unlock", TempData) == 0)
    {
        if (dev_state != NULL && strcmp(dev_state, TempData) != 0)
        {
            fastboot_info("wipe user data");
            EraseUserData();
            setenv("devicestate", TempData);
            saveenv();
        }
    }
    else if (strcmp("lock_critical", TempData) == 0)
    {
        critical_lock = 1;
    }
    else if (strcmp("unlock_critical", TempData) == 0)
    {
        fastboot_info("flashing bootloader-related partitions has its own risk");
        critical_lock = 0;
    }
    else
#endif//CC_ANDROID_SECURE_BOOT_CLASS_B
    {
        ret = -1;
    }

    if (ret == 0)
        fastboot_okay("Done!!");
    else
        fastboot_fail("invalid command");
    return ret;
}

int do_getvar_serialno(const char *arg, const char *data, unsigned int data_length)
{
     char *str;
     if(!strcmp(data, "all"))
     {
         printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
         return 0;
     }

	str = env_add_usb_serialno();

     if(str != NULL)
     {
         fastboot_okay(str);
         return 0;
     }
     else
     {
         fastboot_fail("Unknown error!!");
        return -1;
     }
}

#ifdef CC_SUPPORT_AB_UPDATE
int do_setActive(const char *arg, const char *data, unsigned int data_length)
{
    unsigned int slotNumber = 0;

    if(*data != 'a' && *data != 'b')
    {
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data -'a';

    printf("%s:%d:%s: fastboot set active of slot number %d.\n",__FILE__,__LINE__,__FUNCTION__, slotNumber);
    if(ab_setActiveSlot(slotNumber) == 0) //0 means set OK.
    {
        fastboot_okay("Done!!");
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

int do_getvar_slotSuffixes(const char *arg, const char *data, unsigned int data_length)
{
    char slotSuffix[4] = {0};

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    if(ab_appendSlotSuffix(slotSuffix,""))
    {
        fastboot_okay(slotSuffix);
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

int do_getvar_hasSlot(const char *arg, const char *data, unsigned int data_length)
{
    char partition_a[64] = {0};
    char partition_b[64] = {0};
    uint8_t pnum;
    struct mtd_device *dev;
    struct part_info *part;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    memcpy(partition_a,data,data_length);
    strncat(partition_a,"_a",strlen("_a"));
    memcpy(partition_b,data,data_length);
    strncat(partition_b,"_b", strlen("_b"));
    if((mtdparts_init() == 0)
        && (find_dev_and_part(partition_a, &dev, &pnum, &part) == 0)
        && (find_dev_and_part(partition_b, &dev, &pnum, &part) == 0))
    {
    	fastboot_okay("yes");
    }
    else
    {
    	fastboot_okay("no");
    }
    return 0;
}

int do_getvar_currentSlot(const char *arg, const char *data, unsigned int data_length)
{
    char str[32] = {0};
    int currentSlot = 0;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    currentSlot = ab_getCurrentSlotIndex();
    if(currentSlot >= 0)
    {
        snprintf(str, sizeof(str), "%c", currentSlot+'a');
        fastboot_okay(str);
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

int do_getvar_slotSuccessful(const char *arg, const char *data, unsigned int data_length)
{
    int slotNumber = 0, isSuccessful = -1;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    if(*data != 'a' && *data != 'b')
    {
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    isSuccessful = ab_getSlotSuccessful(slotNumber);
    if(isSuccessful >= 0)
    {
        if(isSuccessful)
            fastboot_okay("yes");
        else
            fastboot_okay("no");
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

int do_getvar_slotUnbootable(const char *arg, const char *data, unsigned int data_length)
{
    int slotNumber = 0, isUnbootable = -1;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    if(*data != 'a' && *data != 'b')
    {
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    isUnbootable = ab_getSlotUnbootable(slotNumber);
    if(isUnbootable >= 0)
    {
        if(isUnbootable)
            fastboot_okay("yes");
        else
            fastboot_okay("no");
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

int do_getvar_slotRetryCount(const char *arg, const char *data, unsigned int data_length)
{
    char str[32] = {0};
    int retryCount = 0;
    int slotNumber = 0;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        return 0;
    }

    if(*data != 'a' && *data != 'b')
    {
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    retryCount = ab_getSlotRetryCount(slotNumber);
    if(retryCount >= 0)
    {
        snprintf(str, sizeof(str), "%d", retryCount);
        fastboot_okay(str);
        return 0;
    }
    else
    {
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

#endif

#endif

