/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


#ifndef __USB3_PHY_H
#define __USB3_PHY_H

/* Register definitions */

//#define GENMASK(startbit, lenbit) (((1<<(lenbit))-1)<<(startbit))

/*
 * 0x100C_4000 for MAC register
 * relative to MAC base address
 */
#if defined(CC_MT5893)
#define SSUSB_MAC_BASE		(0x100c4000)
#elif defined(CC_MT5887)   //for mt5887 temp
#define SSUSB_MAC_BASE		(0x100c4000)
#elif defined(CC_MT5895)
#define SSUSB_MAC_BASE		(0x1202c000)
#endif

#if defined(CC_MT5887)
#define SSUSB_SIFSLV_IPPC_BASE		(0x100c7e00)
#elif defined(CC_MT5895)
#define SSUSB_SIFSLV_IPPC_BASE		(0x1202fe00)
#endif


#if defined(CC_MT5887) || defined(CC_MT5895)
#define SSUSB_SIF_BASE		(0x100c0300)
/*
 * 0x1129_0000 for sifslv2 register in top_ao
 *  relative to USB3_SIF2_BASE base address
 */
//#define SSUSB_SIFSLV_U2PHY_COM_BASE	(0x0000)
#define SSUSB_SIFSLV_U3PHYD_BASE_	(0x0600)
#define SSUSB_SIFSLV_U3PHYD_B2_BASE	(0x0700)
#define SSUSB_SIFSLV_U2FREQ_BASE	(0x0c00)
#define SSUSB_USB30_PHYA_SIV_B_BASE		(0x0800)
#define SSUSB_SIFSLV_U3PHYA_DA_BASE_		(0x0900)
//#define SSUSB_SIFSLV_SPLLC		(0x10000)
#endif

#define U3P_USBPHYACR0		(0x0000)
#define PA0_RG_U2INTE_EN               (0x1 << 5)
#define PA0_RG_U2PLL_FORCE_ON		(0x1 << 15)
#define PA0_RG_U2REF_EN 			(0x1 << 4)
#define PA0_RG_U2BGR_EN 			(0x1 << 0)

#define U3P_USBPHYACR2		(0x0008)
#define PA2_RG_SIF_U2PLL_FORCE_EN	(0x1 << 18)
#define U3P_USBPHYACR4		(0x0010)
#define PA4_RG_U2_FS_SR_VAL(x) (0x7 & (x))

#define PA5_RG_U2_HSTX_SRCAL_EN         (0x1 << 15)
#define PA5_RG_U2_HSTX_SRCTRL_VAL(x)	((0x7 & (x)) << 12)

#define U3P_U2PHYACR3     (0x001c)
#define RG_U2_HSTX_MODE(x)  (0x3&(x))<<24

#define P2C_RG_USB20_DP_100K_EN         (0x1 << 18)
#define P2C_RG_USB20_DM_100K_EN         (0x1 << 17)

#define U3P_PORT_OFFSET (0x1000)	/*based on port0 */
#define U3P_PHY_BASE(index) ((U3P_PORT_OFFSET) * (index))

#define U3P_IP_PW_CTRL0	(0x0000)
#define CTRL0_IP_SW_RST			(0x1<<0)

#define U3P_IP_PW_CTRL1	(0x0004)
#define CTRL1_IP_HOST_PDN		(0x1<<0)

#define U3P_IP_PW_CTRL2	(0x0008)
#define CTRL2_IP_HOST_PDN		(0x1<<0)

#define U3P_IP_PW_STS1		(0x0010)
#define STS1_U3_MAC_RST		(0x1 << 16)
#define STS1_SYS125_RST		(0x1 << 10)
#define STS1_REF_RST		(0x1 << 8)
#define STS1_SYSPLL_STABLE	(0x1 << 0)

#define U3P_IP_PW_STS2	(0x0014)
#define STS2_U2_MAC_RST	(0x1 << 0)

#define U3P_IP_XHCI_CAP	(0x24)
#define CAP_U3_PORT_NUM(p)	((p) & 0xff)
#define CAP_U2_PORT_NUM(p)	(((p) >> 8) & 0xff)

#define U3P_U3_CTRL_0P	(0x0030)
#define CTRL_U3_PORT_HOST_SEL	(0x1<<2)
#define CTRL_U3_PORT_PDN		(0x1<<1)
#define CTRL_U3_PORT_DIS		(0x1<<0)

#define U3P_U2_CTRL_0P	(0x0050)
#define CTRL_U2_PORT_HOST_SEL	(0x1<<2)
#define CTRL_U2_PORT_PDN		(0x1<<1)
#define CTRL_U2_PORT_DIS		(0x1<<0)

#define U3P_U3_CTRL(p)	(U3P_U3_CTRL_0P + ((p) * 0x08))
#define U3P_U2_CTRL(p)	(U3P_U2_CTRL_0P + ((p) * 0x08))

#define U3P_USBPHYACR5      (0x0014)
#define PA5_RG_U2_HSTX_SRCTRL			(0x7<<12)
#define PA5_RG_U2_HSTX_SRCTRL_VAL(x)	((0x7 & (x)) << 12)
#define PA5_RG_U2_HS_100U_U3_EN			(0x1<<11)

#define U3P_USBPHYACR6      (0x0018)
#define PA6_RG_U2_ISO_EN			(0x1<<31)
#define PA6_RG_U2_BC11_SW_EN		(0x1<<23)
#define PA6_RG_U2_OTG_VBUSCMP_EN	(0x1<<20)

#define U3P_U2PHYACR4       (0x0020)
#define P2C_RG_USB20_GPIO_CTL	(0x1<<9)
#define P2C_USB20_GPIO_MODE		(0x1<<8)
#define P2C_U2_GPIO_CTR_MSK  (P2C_RG_USB20_GPIO_CTL | P2C_USB20_GPIO_MODE)

#define U3P_U2PHYDTM0       (0x0068)
#define P2C_FORCE_UART_EN		(0x1<<26)
#define P2C_FORCE_DATAIN		(0x1<<23)
#define P2C_FORCE_DM_PULLDOWN	(0x1<<21)
#define P2C_FORCE_DP_PULLDOWN	(0x1<<20)
#define P2C_FORCE_XCVRSEL		(0x1<<19)
#define P2C_FORCE_SUSPENDM		(0x1<<18)
#define P2C_FORCE_TERMSEL		(0x1<<17)
#define P2C_RG_DATAIN			(0xf<<10)
#define P2C_RG_DATAIN_VAL(x)	((0xf & (x)) << 10)
#define P2C_RG_DMPULLDOWN		(0x1<<7)
#define P2C_RG_DPPULLDOWN		(0x1<<6)
#define P2C_RG_XCVRSEL			(0x3<<4)
#define P2C_RG_XCVRSEL_VAL(x)	((0x3 & (x)) << 4)
#define P2C_RG_SUSPENDM			(0x1<<3)
#define P2C_RG_TERMSEL			(0x1<<2)
#define P2C_DTM0_PART_MASK \
		(P2C_FORCE_DATAIN | P2C_FORCE_DM_PULLDOWN | \
		P2C_FORCE_DP_PULLDOWN | P2C_FORCE_XCVRSEL | \
		P2C_FORCE_TERMSEL | P2C_RG_DMPULLDOWN | \
		P2C_RG_TERMSEL)

#define U3P_U2PHYDTM1		(0x006C)
#define P2C_RG_UART_EN		(0x1<<16)
#define P2C_RG_VBUSVALID	(0x1<<5)
#define P2C_RG_SESSEND		(0x1<<4)
#define P2C_RG_AVALID		(0x1<<2)

#define U3P_U3_PHYA_REG0	(SSUSB_USB30_PHYA_SIV_B_BASE+0x0000)
#define P3A_RG_U3_VUSB10_ON			(1<<5)

#define U3P_U3_PHYA_REG6	(SSUSB_USB30_PHYA_SIV_B_BASE+0x0018)
#define P3A_RG_TX_EIDLE_CM			(0xf<<28)
#define P3A_RG_TX_EIDLE_CM_VAL(x)	((0xf & (x)) << 28)

#define U3P_U3_PHYA_REG9	(SSUSB_USB30_PHYA_SIV_B_BASE+0x0024)
#define P3A_RG_RX_DAC_MUX			(0x1f<<1)
#define P3A_RG_RX_DAC_MUX_VAL(x)	((0x1f & (x)) << 1)

#define U3P_U3PHYA_DA_REG0	(SSUSB_SIFSLV_U3PHYA_DA_BASE_  + 0x0)

#define P3A_RG_XTAL_EXT_EN_U3			(0x3<<10)
#define P3A_RG_XTAL_EXT_EN_U3_VAL(x)	((0x3 & (x)) << 10)

#define U3P_PHYD_CDR1		(SSUSB_SIFSLV_U3PHYD_BASE_+0x5c)
#define P3D_RG_CDR_BIR_LTD1				(0x1f<<24)
#define P3D_RG_CDR_BIR_LTD1_VAL(x)		((0x1f & (x)) << 24)
#define P3D_RG_CDR_BIR_LTD0				(0x1f<<8)
#define P3D_RG_CDR_BIR_LTD0_VAL(x)		((0x1f & (x)) << 8)

#define U3P_XTALCTL3		(SSUSB_SIFSLV_SPLLC + 0x18)
#define XC3_RG_U3_XTAL_RX_PWD			(0x1<<9)
#define XC3_RG_U3_FRC_XTAL_RX_PWD		(0x1<<8)

#define U3P_UX_EXIT_LFPS_PARAM	(SSUSB_USB3_MAC_CSR_BASE+0x00A0)
#define RX_UX_EXIT_REF		(0xff<<8)
#define RX_UX_EXIT_REF_VAL	(0x3 << 8)

#define U3P_REF_CLK_PARAM	(SSUSB_USB3_MAC_CSR_BASE+0x00B0)
#define REF_CLK_1000NS		(0xff << 0)
#define REF_CLK_VAL_DEF		(0xa << 0)

#define U3P_LINK_PM_TIMER	(SSUSB_USB3_SYS_CSR_BASE+0x0208)
#define PM_LC_TIMEOUT		(0xf<<0)
#define PM_LC_TIMEOUT_VAL	(0x3 << 0)

#define U3P_TIMING_PULSE_CTRL	(SSUSB_USB3_SYS_CSR_BASE+0x02B4)
#define U3T_CNT_1US			(0xff << 0)
#define U3T_CNT_1US_VAL		(0x3f << 0)	/* 62.5MHz: 63 */

#define U3P_U2_TIMING_PARAM	(SSUSB_USB2_CSR_BASE+0x0040)
#define U2T_VAL_1US			(0xff<<0)
#define U2T_VAL_1US_VAL		(0x3f << 0)	/* 62.5MHz: 63 */


#define U3P_B2_PHYD_RXDET2      (SSUSB_SIFSLV_U3PHYD_B2_BASE + 0x2c)
void setup_usb_host(void);

#endif
