/* Copyright 2021 Sony Corporation */

#ifndef _AUD_CUST_AMP_RT9119_H_
#define _AUD_CUST_AMP_RT9119_H_

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#include "x_typedef.h"
//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------
typedef enum {
    DA_RT9119_TYPE_NONE,
    DA_RT9119_TYPE_1,
    DA_RT9119_TYPE_2,
    DA_RT9119_TYPE_3,
    DA_RT9119_TYPE_4,
    DA_RT9119_TYPE_MAX
} DaRT9119Type;

#define DA_RT9119_TYPE_1_BIT_ASSIGN 0x01
#define DA_RT9119_TYPE_2_BIT_ASSIGN 0x02
#define DA_RT9119_TYPE_3_BIT_ASSIGN 0x04
#define DA_RT9119_TYPE_4_BIT_ASSIGN 0x08

// I2S_FMT
#define AMP_RT9119_REG_I2S_FMT                    0x02
#define AMP_RT9119_REG_I2S_FMT_SIZE               1

// ERR_RPT
#define AMP_RT9119_REG_ERR_RPT                    0x03
#define AMP_RT9119_REG_ERR_RPT_SIZE               1

// ERROR_MASK
#define AMP_RT9119_REG_ERROR_MASK                 0x04
#define AMP_RT9119_REG_ERROR_MASK_SIZE            1

// SYS_CTL
#define AMP_RT9119_REG_SYS_CTL                    0x05
#define AMP_RT9119_REG_SYS_CTL_SIZE               1
#define AMP_RT9119_REG_SYS_CTL_BTL_400_VAL        0x09
#define AMP_RT9119_REG_SYS_CTL_BTL_800_VAL        0x19
#define AMP_RT9119_REG_SYS_CTL_PBTL_400_VAL       0x29
#define AMP_RT9119_REG_SYS_CTL_PBTL_800_VAL       0x39
#define AMP_RT9119_REG_SYS_CTL_SHUTDOWN           0x40

// ERR_TYPE
#define AMP_RT9119_REG_ERROR_TYPE                 0x06
#define AMP_RT9119_REG_ERROR_TYPE_SIZE            1

// MS_VOL
#define AMP_RT9119_REG_MS_VOL                     0x07
#define AMP_RT9119_REG_MS_VOL_SIZE                2

// CH1_VOL
#define AMP_RT9119_REG_CH1_VOL                    0x08
#define AMP_RT9119_REG_CH1_VOL_SIZE               2
#define AMP_RT9119_REG_CH2_VOL                    0x09
#define AMP_RT9119_REG_CH2_VOL_SIZE               2

// VOL_RAMP
#define AMP_RT9119_REG_VOL_RAMP                   0x0A
#define AMP_RT9119_REG_VOL_RAMP_SIZE              1

// AUTO_RCVRY
#define AMP_RT9119_REG_AUTO_RCVRY                 0x0B
#define AMP_RT9119_REG_AUTO_RCVRY_SIZE            1

// FLTR_MISC
#define AMP_RT9119_REG_FLTR_MISC                  0x0C
#define AMP_RT9119_REG_FLTR_MISC_SIZE             1

// SDIO_SEL
#define AMP_RT9119_REG_SDIO_SEL                   0x0D
#define AMP_RT9119_REG_SDIO_SEL_SIZE              1

// SPK_GAIN
#define AMP_RT9119_REG_SPK_GAIN                   0x0E
#define AMP_RT9119_REG_SPK_GAIN_SIZE              1

// INTER_PWR_CTRL
#define AMP_RT9119_REG_INTER_PWR_CTRL             0x0F
#define AMP_RT9119_REG_INTER_PWR_CTRL_SIZE        1

// DC_PROTECT
#define AMP_RT9119_REG_DC_PROTECT                 0x10
#define AMP_RT9119_REG_DC_PROTECT_SIZE            1

// PWM_SS_OPT
#define AMP_RT9119_REG_PWM_SS_OPT                 0x11
#define AMP_RT9119_REG_PWM_SS_OPT_SIZE            1

// CH_MIX
#define AMP_RT9119_REG_CH_MIX_SIZE                4
typedef struct AMP_RT9119_ChMix {
    unsigned char mix_1[AMP_RT9119_REG_CH_MIX_SIZE];
    unsigned char mix_0[AMP_RT9119_REG_CH_MIX_SIZE];
} AMP_RT9119_ChMix;
#define AMP_RT9119_REG_CHx_MIX_SIZE               sizeof(AMP_RT9119_ChMix)

// CH1_IN_MIX
#define AMP_RT9119_REG_CH1_IN_MIX                 0x12
// CH2_IN_MIX
#define AMP_RT9119_REG_CH2_IN_MIX                 0x13
// CH1_OUT_MIX
#define AMP_RT9119_REG_CH1_OUT_MIX                0xA4
// CH2_OUT_MIX
#define AMP_RT9119_REG_CH2_OUT_MIX                0xA5

// L_BQ_BOOST
#define AMP_RT9119_REG_L_BQ_BOOST                 0x14
#define AMP_RT9119_REG_L_BQ_BOOST_SIZE            4

// R_BQ_BOOST
#define AMP_RT9119_REG_R_BQ_BOOST                 0x15
#define AMP_RT9119_REG_R_BQ_BOOST_SIZE            4

// EQ_CTRL
#define AMP_RT9119_REG_EQ_CTRL                    0x18
#define AMP_RT9119_REG_EQ_CTRL_SIZE               1

// SBQ_ALPHA
#define AMP_RT9119_REG_SBQ_ALPHA                  0x19
#define AMP_RT9119_REG_SBQ_ALPHA_SIZE             4

// SBQ_CTRL
#define AMP_RT9119_REG_SBQ_CTRL                   0x1A
#define AMP_RT9119_REG_SBQ_CTRL_SIZE              4

// DRC_CTRL1
#define AMP_RT9119_REG_DRC_CTRL1                  0x60
#define AMP_RT9119_REG_DRC_CTRL1_SIZE             1

// DRC_CTRL2
#define AMP_RT9119_REG_DRC_CTRL2                  0x61
#define AMP_RT9119_REG_DRC_CTRL2_SIZE             1

// PRE_SCALE
#define AMP_RT9119_REG_PRE_SCALE                  0xA6
#define AMP_RT9119_REG_PRE_SCALE_SIZE             4

// POST_SCALE
#define AMP_RT9119_REG_POST_SCALE                 0xA7
#define AMP_RT9119_REG_POST_SCALE_SIZE            4

// HARD_CLIP
#define AMP_RT9119_REG_HARD_CLIP                  0xB0
#define AMP_RT9119_REG_HARD_CLIP_SIZE             4
#define AMP_RT9119_REG_HARD_CLIP_BASE_SETTING     0x40000080

// GAIN_OUT
#define AMP_RT9119_REG_FINAL_GAIN_OUT             0xBB
#define AMP_RT9119_REG_FINAL_GAIN_OUT_SIZE        1

// ANA_MODULTR2
#define AMP_RT9119_REG_ANA_MODULTR2               0xCE
#define AMP_RT9119_REG_ANA_MODULTR2_SIZE          1

// PWR_SEL
#define AMP_RT9119_REG_PWR_SEL                    0xD5
#define AMP_RT9119_REG_PWR_SEL_SIZE               1

// DEPOP_GLITCH
#define AMP_RT9119_REG_DEPOP_GLITCH               0xD8
#define AMP_RT9119_REG_DEPOP_GLITCH_SIZE          1

// SERVO_TIME_OPT
#define AMP_RT9119_REG_SERVO_TIME_OPT             0xE0
#define AMP_RT9119_REG_SERVO_TIME_OPT_SIZE        1

//---------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------
/*
 * dB to Lenear Gain 0.1dB/step, -48dB ~ +48dB, center = 0x00020000
 * for Pre Scaler
 */
const unsigned int db0p1ToLenear20000forDampCtrl[] =
{
    0x00000209, //   1  :   dB
    0x00000215, //   2  :   dB
    0x0000020F, //   3  :   dB
    0x0000021C, //   4  :   dB
    0x00000222, //   5  :   dB
    0x00000228, //   6  :   dB
    0x0000022F, //   7  :   dB
    0x00000235, //   8  :   dB
    0x0000023C, //   9  :   dB
    0x00000242, //   10 :   dB
    0x00000249, //   11 :   dB
    0x00000250, //   12 :   dB
    0x00000257, //   13 :   dB
    0x0000025E, //   14 :   dB
    0x00000265, //   15 :   dB
    0x0000026C, //   16 :   dB
    0x00000273, //   17 :   dB
    0x0000027A, //   18 :   dB
    0x00000281, //   19 :   dB
    0x00000289, //   20 :   dB
    0x00000290, //   21 :   dB
    0x00000298, //   22 :   dB
    0x000002A0, //   23 :   dB
    0x000002A8, //   24 :   dB
    0x000002AF, //   25 :   dB
    0x000002B7, //   26 :   dB
    0x000002BF, //   27 :   dB
    0x000002C8, //   28 :   dB
    0x000002D0, //   29 :   dB
    0x000002D8, //   30 :   dB
    0x000002E1, //   31 :   dB
    0x000002E9, //   32 :   dB
    0x000002F2, //   33 :   dB
    0x000002FA, //   34 :   dB
    0x00000303, //   35 :   dB
    0x0000030C, //   36 :   dB
    0x00000315, //   37 :   dB
    0x0000031E, //   38 :   dB
    0x00000328, //   39 :   dB
    0x00000331, //   40 :   dB
    0x0000033B, //   41 :   dB
    0x00000344, //   42 :   dB
    0x0000034E, //   43 :   dB
    0x00000358, //   44 :   dB
    0x00000361, //   45 :   dB
    0x0000036C, //   46 :   dB
    0x00000376, //   47 :   dB
    0x00000380, //   48 :   dB
    0x0000038A, //   49 :   dB
    0x00000395, //   50 :   dB
    0x0000039F, //   51 :   dB
    0x000003AA, //   52 :   dB
    0x000003B5, //   53 :   dB
    0x000003C0, //   54 :   dB
    0x000003CB, //   55 :   dB
    0x000003D6, //   56 :   dB
    0x000003E2, //   57 :   dB
    0x000003ED, //   58 :   dB
    0x000003F9, //   59 :   dB
    0x00000405, //   60 :   dB
    0x00000411, //   61 :   dB
    0x0000041D, //   62 :   dB
    0x00000429, //   63 :   dB
    0x00000435, //   64 :   dB
    0x00000442, //   65 :   dB
    0x0000044E, //   66 :   dB
    0x0000045B, //   67 :   dB
    0x00000468, //   68 :   dB
    0x00000475, //   69 :   dB
    0x00000482, //   70 :   dB
    0x00000490, //   71 :   dB
    0x0000049D, //   72 :   dB
    0x000004AB, //   73 :   dB
    0x000004B9, //   74 :   dB
    0x000004C7, //   75 :   dB
    0x000004D5, //   76 :   dB
    0x000004E3, //   77 :   dB
    0x000004F2, //   78 :   dB
    0x00000500, //   79 :   dB
    0x0000050F, //   80 :   dB
    0x0000051E, //   81 :   dB
    0x0000052D, //   82 :   dB
    0x0000053D, //   83 :   dB
    0x0000054C, //   84 :   dB
    0x0000055C, //   85 :   dB
    0x0000056C, //   86 :   dB
    0x0000057C, //   87 :   dB
    0x0000058C, //   88 :   dB
    0x0000059D, //   89 :   dB
    0x000005AD, //   90 :   dB
    0x000005BE, //   91 :   dB
    0x000005CF, //   92 :   dB
    0x000005E0, //   93 :   dB
    0x000005F2, //   94 :   dB
    0x00000603, //   95 :   dB
    0x00000615, //   96 :   dB
    0x00000627, //   97 :   dB
    0x0000063A, //   98 :   dB
    0x0000064C, //   99 :   dB
    0x0000065F, //   100:   dB
    0x00000672, //   101:   dB
    0x00000685, //   102:   dB
    0x00000698, //   103:   dB
    0x000006AC, //   104:   dB
    0x000006BF, //   105:   dB
    0x000006D3, //   106:   dB
    0x000006E8, //   107:   dB
    0x000006FC, //   108:   dB
    0x00000711, //   109:   dB
    0x00000726, //   110:   dB
    0x0000073B, //   111:   dB
    0x00000750, //   112:   dB
    0x00000766, //   113:   dB
    0x0000077C, //   114:   dB
    0x00000792, //   115:   dB
    0x000007A9, //   116:   dB
    0x000007BF, //   117:   dB
    0x000007D6, //   118:   dB
    0x000007EE, //   119:   dB
    0x00000805, //   120:   dB
    0x0000081D, //   121:   dB
    0x00000835, //   122:   dB
    0x0000084D, //   123:   dB
    0x00000866, //   124:   dB
    0x0000087F, //   125:   dB
    0x00000898, //   126:   dB
    0x000008B1, //   127:   dB
    0x000008CB, //   128:   dB
    0x000008E5, //   129:   dB
    0x00000900, //   130:   dB
    0x0000091A, //   131:   dB
    0x00000935, //   132:   dB
    0x00000951, //   133:   dB
    0x0000096C, //   134:   dB
    0x00000988, //   135:   dB
    0x000009A4, //   136:   dB
    0x000009C1, //   137:   dB
    0x000009DE, //   138:   dB
    0x000009FB, //   139:   dB
    0x00000A19, //   140:   dB
    0x00000A37, //   141:   dB
    0x00000A55, //   142:   dB
    0x00000A74, //   143:   dB
    0x00000A93, //   144:   dB
    0x00000AB2, //   145:   dB
    0x00000AD2, //   146:   dB
    0x00000AF2, //   147:   dB
    0x00000B12, //   148:   dB
    0x00000B33, //   149:   dB
    0x00000B54, //   150:   dB
    0x00000B76, //   151:   dB
    0x00000B98, //   152:   dB
    0x00000BBA, //   153:   dB
    0x00000BDD, //   154:   dB
    0x00000C00, //   155:   dB
    0x00000C24, //   156:   dB
    0x00000C48, //   157:   dB
    0x00000C6C, //   158:   dB
    0x00000C91, //   159:   dB
    0x00000CB6, //   160:   dB
    0x00000CDC, //   161:   dB
    0x00000D02, //   162:   dB
    0x00000D29, //   163:   dB
    0x00000D50, //   164:   dB
    0x00000D77, //   165:   dB
    0x00000D9F, //   166:   dB
    0x00000DC7, //   167:   dB
    0x00000DF0, //   168:   dB
    0x00000E1A, //   169:   dB
    0x00000E43, //   170:   dB
    0x00000E6E, //   171:   dB
    0x00000E98, //   172:   dB
    0x00000EC4, //   173:   dB
    0x00000EEF, //   174:   dB
    0x00000F1C, //   175:   dB
    0x00000F49, //   176:   dB
    0x00000F76, //   177:   dB
    0x00000FA4, //   178:   dB
    0x00000FD2, //   179:   dB
    0x00001001, //   180:   dB
    0x00001030, //   181:   dB
    0x00001060, //   182:   dB
    0x00001091, //   183:   dB
    0x000010C2, //   184:   dB
    0x000010F4, //   185:   dB
    0x00001126, //   186:   dB
    0x00001159, //   187:   dB
    0x0000118C, //   188:   dB
    0x000011C0, //   189:   dB
    0x000011F5, //   190:   dB
    0x0000122A, //   191:   dB
    0x00001260, //   192:   dB
    0x00001296, //   193:   dB
    0x000012CE, //   194:   dB
    0x00001305, //   195:   dB
    0x0000133E, //   196:   dB
    0x00001377, //   197:   dB
    0x000013B0, //   198:   dB
    0x000013EB, //   199:   dB
    0x00001426, //   200:   dB
    0x00001462, //   201:   dB
    0x0000149E, //   202:   dB
    0x000014DB, //   203:   dB
    0x00001519, //   204:   dB
    0x00001557, //   205:   dB
    0x00001597, //   206:   dB
    0x000015D7, //   207:   dB
    0x00001618, //   208:   dB
    0x00001659, //   209:   dB
    0x0000169B, //   210:   dB
    0x000016DE, //   211:   dB
    0x00001722, //   212:   dB
    0x00001767, //   213:   dB
    0x000017AC, //   214:   dB
    0x000017F2, //   215:   dB
    0x00001839, //   216:   dB
    0x00001881, //   217:   dB
    0x000018CA, //   218:   dB
    0x00001913, //   219:   dB
    0x0000195D, //   220:   dB
    0x000019A9, //   221:   dB
    0x000019F5, //   222:   dB
    0x00001A42, //   223:   dB
    0x00001A90, //   224:   dB
    0x00001ADE, //   225:   dB
    0x00001B2E, //   226:   dB
    0x00001B7E, //   227:   dB
    0x00001BD0, //   228:   dB
    0x00001C22, //   229:   dB
    0x00001C76, //   230:   dB
    0x00001CCA, //   231:   dB
    0x00001D20, //   232:   dB
    0x00001D76, //   233:   dB
    0x00001DCD, //   234:   dB
    0x00001E26, //   235:   dB
    0x00001E7F, //   236:   dB
    0x00001ED9, //   237:   dB
    0x00001F35, //   238:   dB
    0x00001F91, //   239:   dB
    0x00001FEF, //   240:   dB
    0x0000204E, //   241:   dB
    0x000020AD, //   242:   dB
    0x0000210E, //   243:   dB
    0x00002170, //   244:   dB
    0x000021D3, //   245:   dB
    0x00002238, //   246:   dB
    0x0000229D, //   247:   dB
    0x00002304, //   248:   dB
    0x0000236B, //   249:   dB
    0x000023D4, //   250:   dB
    0x0000243F, //   251:   dB
    0x000024AA, //   252:   dB
    0x00002517, //   253:   dB
    0x00002585, //   254:   dB
    0x000025F4, //   255:   dB
    0x00002665, //   256:   dB
    0x000026D6, //   257:   dB
    0x00002749, //   258:   dB
    0x000027BE, //   259:   dB
    0x00002834, //   260:   dB
    0x000028AB, //   261:   dB
    0x00002923, //   262:   dB
    0x0000299D, //   263:   dB
    0x00002A19, //   264:   dB
    0x00002A96, //   265:   dB
    0x00002B14, //   266:   dB
    0x00002B94, //   267:   dB
    0x00002C15, //   268:   dB
    0x00002C97, //   269:   dB
    0x00002D1C, //   270:   dB
    0x00002DA1, //   271:   dB
    0x00002E29, //   272:   dB
    0x00002EB1, //   273:   dB
    0x00002F3C, //   274:   dB
    0x00002FC8, //   275:   dB
    0x00003055, //   276:   dB
    0x000030E5, //   277:   dB
    0x00003176, //   278:   dB
    0x00003208, //   279:   dB
    0x0000329D, //   280:   dB
    0x00003333, //   281:   dB
    0x000033CA, //   282:   dB
    0x00003464, //   283:   dB
    0x000034FF, //   284:   dB
    0x0000359C, //   285:   dB
    0x0000363B, //   286:   dB
    0x000036DC, //   287:   dB
    0x0000377F, //   288:   dB
    0x00003823, //   289:   dB
    0x000038CA, //   290:   dB
    0x00003972, //   291:   dB
    0x00003A1C, //   292:   dB
    0x00003AC9, //   293:   dB
    0x00003B77, //   294:   dB
    0x00003C27, //   295:   dB
    0x00003CD9, //   296:   dB
    0x00003D8E, //   297:   dB
    0x00003E44, //   298:   dB
    0x00003EFD, //   299:   dB
    0x00003FB8, //   300:   dB
    0x00004074, //   301:   dB
    0x00004134, //   302:   dB
    0x000041F5, //   303:   dB
    0x000042B8, //   304:   dB
    0x0000437E, //   305:   dB
    0x00004446, //   306:   dB
    0x00004511, //   307:   dB
    0x000045DD, //   308:   dB
    0x000046AC, //   309:   dB
    0x0000477E, //   310:   dB
    0x00004852, //   311:   dB
    0x00004928, //   312:   dB
    0x00004A01, //   313:   dB
    0x00004ADD, //   314:   dB
    0x00004BBA, //   315:   dB
    0x00004C9B, //   316:   dB
    0x00004D7E, //   317:   dB
    0x00004E64, //   318:   dB
    0x00004F4C, //   319:   dB
    0x00005037, //   320:   dB
    0x00005125, //   321:   dB
    0x00005216, //   322:   dB
    0x00005309, //   323:   dB
    0x000053FF, //   324:   dB
    0x000054F8, //   325:   dB
    0x000055F4, //   326:   dB
    0x000056F3, //   327:   dB
    0x000057F4, //   328:   dB
    0x000058F9, //   329:   dB
    0x00005A01, //   330:   dB
    0x00005B0C, //   331:   dB
    0x00005C1A, //   332:   dB
    0x00005D2B, //   333:   dB
    0x00005E3F, //   334:   dB
    0x00005F56, //   335:   dB
    0x00006071, //   336:   dB
    0x0000618F, //   337:   dB
    0x000062B0, //   338:   dB
    0x000063D5, //   339:   dB
    0x000064FC, //   340:   dB
    0x00006628, //   341:   dB
    0x00006757, //   342:   dB
    0x00006889, //   343:   dB
    0x000069BF, //   344:   dB
    0x00006AF8, //   345:   dB
    0x00006C35, //   346:   dB
    0x00006D76, //   347:   dB
    0x00006EBB, //   348:   dB
    0x00007003, //   349:   dB
    0x0000714F, //   350:   dB
    0x0000729F, //   351:   dB
    0x000073F3, //   352:   dB
    0x0000754A, //   353:   dB
    0x000076A6, //   354:   dB
    0x00007806, //   355:   dB
    0x0000796A, //   356:   dB
    0x00007AD1, //   357:   dB
    0x00007C3E, //   358:   dB
    0x00007DAE, //   359:   dB
    0x00007F22, //   360:   dB
    0x0000809B, //   361:   dB
    0x00008219, //   362:   dB
    0x0000839A, //   363:   dB
    0x00008520, //   364:   dB
    0x000086AB, //   365:   dB
    0x0000883A, //   366:   dB
    0x000089CE, //   367:   dB
    0x00008B66, //   368:   dB
    0x00008D04, //   369:   dB
    0x00008EA6, //   370:   dB
    0x0000904D, //   371:   dB
    0x000091F8, //   372:   dB
    0x000093A9, //   373:   dB
    0x0000955F, //   374:   dB
    0x0000971A, //   375:   dB
    0x000098DA, //   376:   dB
    0x00009A9F, //   377:   dB
    0x00009C69, //   378:   dB
    0x00009E39, //   379:   dB
    0x0000A00E, //   380:   dB
    0x0000A1E8, //   381:   dB
    0x0000A3C8, //   382:   dB
    0x0000A5AE, //   383:   dB
    0x0000A799, //   384:   dB
    0x0000A98A, //   385:   dB
    0x0000AB80, //   386:   dB
    0x0000AD7C, //   387:   dB
    0x0000AF7F, //   388:   dB
    0x0000B187, //   389:   dB
    0x0000B395, //   390:   dB
    0x0000B5AA, //   391:   dB
    0x0000B7C4, //   392:   dB
    0x0000B9E5, //   393:   dB
    0x0000BC0C, //   394:   dB
    0x0000BE39, //   395:   dB
    0x0000C06D, //   396:   dB
    0x0000C2A8, //   397:   dB
    0x0000C4E9, //   398:   dB
    0x0000C730, //   399:   dB
    0x0000C97F, //   400:   dB
    0x0000CBD4, //   401:   dB
    0x0000CE30, //   402:   dB
    0x0000D094, //   403:   dB
    0x0000D2FE, //   404:   dB
    0x0000D56F, //   405:   dB
    0x0000D7E8, //   406:   dB
    0x0000DA68, //   407:   dB
    0x0000DCF0, //   408:   dB
    0x0000DF7F, //   409:   dB
    0x0000E215, //   410:   dB
    0x0000E4B3, //   411:   dB
    0x0000E759, //   412:   dB
    0x0000EA07, //   413:   dB
    0x0000ECBD, //   414:   dB
    0x0000EF7A, //   415:   dB
    0x0000F240, //   416:   dB
    0x0000F50F, //   417:   dB
    0x0000F7E5, //   418:   dB
    0x0000FAC4, //   419:   dB
    0x0000FDAB, //   420:   dB
    0x0001009B, //   421:   dB
    0x00010394, //   422:   dB
    0x00010695, //   423:   dB
    0x000109A0, //   424:   dB
    0x00010CB3, //   425:   dB
    0x00010FD0, //   426:   dB
    0x000112F5, //   427:   dB
    0x00011624, //   428:   dB
    0x0001195D, //   429:   dB
    0x00011C9F, //   430:   dB
    0x00011FEB, //   431:   dB
    0x00012340, //   432:   dB
    0x000126A0, //   433:   dB
    0x00012A09, //   434:   dB
    0x00012D7C, //   435:   dB
    0x000130FA, //   436:   dB
    0x00013482, //   437:   dB
    0x00013815, //   438:   dB
    0x00013BB2, //   439:   dB
    0x00013F5A, //   440:   dB
    0x0001430C, //   441:   dB
    0x000146CA, //   442:   dB
    0x00014A93, //   443:   dB
    0x00014E67, //   444:   dB
    0x00015246, //   445:   dB
    0x00015631, //   446:   dB
    0x00015A27, //   447:   dB
    0x00015E29, //   448:   dB
    0x00016237, //   449:   dB
    0x00016651, //   450:   dB
    0x00016A77, //   451:   dB
    0x00016EAA, //   452:   dB
    0x000172E9, //   453:   dB
    0x00017734, //   454:   dB
    0x00017B8D, //   455:   dB
    0x00017FF2, //   456:   dB
    0x00018464, //   457:   dB
    0x000188E3, //   458:   dB
    0x00018D70, //   459:   dB
    0x0001920A, //   460:   dB
    0x000196B2, //   461:   dB
    0x00019B67, //   462:   dB
    0x0001A02B, //   463:   dB
    0x0001A4FC, //   464:   dB
    0x0001A9DC, //   465:   dB
    0x0001AECB, //   466:   dB
    0x0001B3C8, //   467:   dB
    0x0001B8D4, //   468:   dB
    0x0001BDEE, //   469:   dB
    0x0001C318, //   470:   dB
    0x0001C852, //   471:   dB
    0x0001CD9A, //   472:   dB
    0x0001D2F3, //   473:   dB
    0x0001D85B, //   474:   dB
    0x0001DDD3, //   475:   dB
    0x0001E35B, //   476:   dB
    0x0001E8F4, //   477:   dB
    0x0001EE9E, //   478:   dB
    0x0001F458, //   479:   dB
    0x0001FA23, //   480:   dB
    0x00020000, //   481:   dB
    0x000205ED, //   482:   dB
    0x00020BED, //   483:   dB
    0x000211FE, //   484:   dB
    0x00021821, //   485:   dB
    0x00021E56, //   486:   dB
    0x0002249E, //   487:   dB
    0x00022AF8, //   488:   dB
    0x00023165, //   489:   dB
    0x000237E5, //   490:   dB
    0x00023E79, //   491:   dB
    0x00024520, //   492:   dB
    0x00024BDA, //   493:   dB
    0x000252A9, //   494:   dB
    0x0002598C, //   495:   dB
    0x00026083, //   496:   dB
    0x0002678F, //   497:   dB
    0x00026EAF, //   498:   dB
    0x000275E5, //   499:   dB
    0x00027D31, //   500:   dB
    0x00028491, //   501:   dB
    0x00028C08, //   502:   dB
    0x00029395, //   503:   dB
    0x00029B38, //   504:   dB
    0x0002A2F2, //   505:   dB
    0x0002AAC3, //   506:   dB
    0x0002B2AB, //   507:   dB
    0x0002BAAA, //   508:   dB
    0x0002C2C1, //   509:   dB
    0x0002CAF0, //   510:   dB
    0x0002D338, //   511:   dB
    0x0002DB97, //   512:   dB
    0x0002E410, //   513:   dB
    0x0002ECA2, //   514:   dB
    0x0002F54D, //   515:   dB
    0x0002FE12, //   516:   dB
    0x000306F1, //   517:   dB
    0x00030FEA, //   518:   dB
    0x000318FE, //   519:   dB
    0x0003222D, //   520:   dB
    0x00032B77, //   521:   dB
    0x000334DC, //   522:   dB
    0x00033E5D, //   523:   dB
    0x000347FB, //   524:   dB
    0x000351B5, //   525:   dB
    0x00035B8C, //   526:   dB
    0x00036580, //   527:   dB
    0x00036F91, //   528:   dB
    0x000379C1, //   529:   dB
    0x0003840E, //   530:   dB
    0x00038E7A, //   531:   dB
    0x00039905, //   532:   dB
    0x0003A3AF, //   533:   dB
    0x0003AE79, //   534:   dB
    0x0003B963, //   535:   dB
    0x0003C46D, //   536:   dB
    0x0003CF98, //   537:   dB
    0x0003DAE4, //   538:   dB
    0x0003E652, //   539:   dB
    0x0003F1E1, //   540:   dB
    0x0003FD93, //   541:   dB
    0x00040967, //   542:   dB
    0x0004155E, //   543:   dB
    0x00042179, //   544:   dB
    0x00042DB8, //   545:   dB
    0x00043A1B, //   546:   dB
    0x000446A2, //   547:   dB
    0x0004534F, //   548:   dB
    0x00046022, //   549:   dB
    0x00046D1A, //   550:   dB
    0x00047A39, //   551:   dB
    0x0004877F, //   552:   dB
    0x000494EC, //   553:   dB
    0x0004A281, //   554:   dB
    0x0004B03E, //   555:   dB
    0x0004BE24, //   556:   dB
    0x0004CC33, //   557:   dB
    0x0004DA6C, //   558:   dB
    0x0004E8CF, //   559:   dB
    0x0004F75D, //   560:   dB
    0x00050615, //   561:   dB
    0x000514FA, //   562:   dB
    0x0005240A, //   563:   dB
    0x00053348, //   564:   dB
    0x000542B2, //   565:   dB
    0x0005524A, //   566:   dB
    0x00056210, //   567:   dB
    0x00057205, //   568:   dB
    0x0005822A, //   569:   dB
    0x0005927E, //   570:   dB
    0x0005A303, //   571:   dB
    0x0005B3B8, //   572:   dB
    0x0005C49F, //   573:   dB
    0x0005D5B8, //   574:   dB
    0x0005E704, //   575:   dB
    0x0005F884, //   576:   dB
    0x00060A37, //   577:   dB
    0x00061C1E, //   578:   dB
    0x00062E3B, //   579:   dB
    0x0006408D, //   580:   dB
    0x00065316, //   581:   dB
    0x000665D5, //   582:   dB
    0x000678CC, //   583:   dB
    0x00068BFB, //   584:   dB
    0x00069F64, //   585:   dB
    0x0006B305, //   586:   dB
    0x0006C6E1, //   587:   dB
    0x0006DAF8, //   588:   dB
    0x0006EF4A, //   589:   dB
    0x000703D9, //   590:   dB
    0x000718A5, //   591:   dB
    0x00072DAE, //   592:   dB
    0x000742F5, //   593:   dB
    0x0007587C, //   594:   dB
    0x00076E42, //   595:   dB
    0x00078449, //   596:   dB
    0x00079A91, //   597:   dB
    0x0007B11C, //   598:   dB
    0x0007C7E9, //   599:   dB
    0x0007DEF9, //   600:   dB
    0x0007F64F, //   601:   dB
    0x00080DE9, //   602:   dB
    0x000825C9, //   603:   dB
    0x00083DF0, //   604:   dB
    0x0008565F, //   605:   dB
    0x00086F16, //   606:   dB
    0x00088816, //   607:   dB
    0x0008A160, //   608:   dB
    0x0008BAF6, //   609:   dB
    0x0008D4D7, //   610:   dB
    0x0008EF05, //   611:   dB
    0x00090980, //   612:   dB
    0x0009244A, //   613:   dB
    0x00093F64, //   614:   dB
    0x00095ACD, //   615:   dB
    0x00097688, //   616:   dB
    0x00099296, //   617:   dB
    0x0009AEF6, //   618:   dB
    0x0009CBAA, //   619:   dB
    0x0009E8B4, //   620:   dB
    0x000A0614, //   621:   dB
    0x000A23CA, //   622:   dB
    0x000A41D9, //   623:   dB
    0x000A6041, //   624:   dB
    0x000A7F03, //   625:   dB
    0x000A9E20, //   626:   dB
    0x000ABD9A, //   627:   dB
    0x000ADD71, //   628:   dB
    0x000AFDA6, //   629:   dB
    0x000B1E3A, //   630:   dB
    0x000B3F30, //   631:   dB
    0x000B6086, //   632:   dB
    0x000B8240, //   633:   dB
    0x000BA45E, //   634:   dB
    0x000BC6E1, //   635:   dB
    0x000BE9CA, //   636:   dB
    0x000C0D1A, //   637:   dB
    0x000C30D4, //   638:   dB
    0x000C54F7, //   639:   dB
    0x000C7985, //   640:   dB
    0x000C9E80, //   641:   dB
    0x000CC3E8, //   642:   dB
    0x000CE9BF, //   643:   dB
    0x000D1007, //   644:   dB
    0x000D36C0, //   645:   dB
    0x000D5DEB, //   646:   dB
    0x000D858B, //   647:   dB
    0x000DADA0, //   648:   dB
    0x000DD62C, //   649:   dB
    0x000DFF30, //   650:   dB
    0x000E28AE, //   651:   dB
    0x000E52A7, //   652:   dB
    0x000E7D1C, //   653:   dB
    0x000EA80F, //   654:   dB
    0x000ED382, //   655:   dB
    0x000EFF75, //   656:   dB
    0x000F2BEA, //   657:   dB
    0x000F58E4, //   658:   dB
    0x000F8662, //   659:   dB
    0x000FB468, //   660:   dB
    0x000FE2F5, //   661:   dB
    0x0010120D, //   662:   dB
    0x001041B1, //   663:   dB
    0x001071E1, //   664:   dB
    0x0010A2A1, //   665:   dB
    0x0010D3F1, //   666:   dB
    0x001105D3, //   667:   dB
    0x00113849, //   668:   dB
    0x00116B55, //   669:   dB
    0x00119EF8, //   670:   dB
    0x0011D334, //   671:   dB
    0x0012080B, //   672:   dB
    0x00123D7E, //   673:   dB
    0x00127390, //   674:   dB
    0x0012AA43, //   675:   dB
    0x0012E197, //   676:   dB
    0x0013198F, //   677:   dB
    0x0013522E, //   678:   dB
    0x00138B74, //   679:   dB
    0x0013C564, //   680:   dB
    0x00140000, //   681:   dB
    0x00143B49, //   682:   dB
    0x00147742, //   683:   dB
    0x0014B3ED, //   684:   dB
    0x0014F14C, //   685:   dB
    0x00152F61, //   686:   dB
    0x00156E2D, //   687:   dB
    0x0015ADB4, //   688:   dB
    0x0015EDF7, //   689:   dB
    0x00162EF9, //   690:   dB
    0x001670BC, //   691:   dB
    0x0016B341, //   692:   dB
    0x0016F68B, //   693:   dB
    0x00173A9D, //   694:   dB
    0x00177F79, //   695:   dB
    0x0017C521, //   696:   dB
    0x00180B98, //   697:   dB
    0x001852DF, //   698:   dB
    0x00189AF9, //   699:   dB
    0x0018E3EA, //   700:   dB
    0x00192DB2, //   701:   dB
    0x00197855, //   702:   dB
    0x0019C3D6, //   703:   dB
    0x001A1036, //   704:   dB
    0x001A5D79, //   705:   dB
    0x001AABA1, //   706:   dB
    0x001AFAB0, //   707:   dB
    0x001B4AAA, //   708:   dB
    0x001B9B91, //   709:   dB
    0x001BED67, //   710:   dB
    0x001C4031, //   711:   dB
    0x001C93EF, //   712:   dB
    0x001CE8A6, //   713:   dB
    0x001D3E58, //   714:   dB
    0x001D9508, //   715:   dB
    0x001DECB9, //   716:   dB
    0x001E456E, //   717:   dB
    0x001E9F2A, //   718:   dB
    0x001EF9F0, //   719:   dB
    0x001F55C3, //   720:   dB
    0x001FB2A7, //   721:   dB
    0x0020109D, //   722:   dB
    0x00206FAA, //   723:   dB
    0x0020CFD1, //   724:   dB
    0x00213115, //   725:   dB
    0x0021937A, //   726:   dB
    0x0021F701, //   727:   dB
    0x00225BB0, //   728:   dB
    0x0022C18A, //   729:   dB
    0x00232891, //   730:   dB
    0x002390CA, //   731:   dB
    0x0023FA38, //   732:   dB
    0x002464DE, //   733:   dB
    0x0024D0C0, //   734:   dB
    0x00253DE2, //   735:   dB
    0x0025AC48, //   736:   dB
    0x00261BF5, //   737:   dB
    0x00268CED, //   738:   dB
    0x0026FF34, //   739:   dB
    0x002772CD, //   740:   dB
    0x0027E7BE, //   741:   dB
    0x00285E09, //   742:   dB
    0x0028D5B2, //   743:   dB
    0x00294EBF, //   744:   dB
    0x0029C932, //   745:   dB
    0x002A4510, //   746:   dB
    0x002AC25D, //   747:   dB
    0x002B411E, //   748:   dB
    0x002BC156, //   749:   dB
    0x002C430B, //   750:   dB
    0x002CC640, //   751:   dB
    0x002D4AFA, //   752:   dB
    0x002DD13E, //   753:   dB
    0x002E590F, //   754:   dB
    0x002EE273, //   755:   dB
    0x002F6D6E, //   756:   dB
    0x002FFA06, //   757:   dB
    0x0030883E, //   758:   dB
    0x0031181C, //   759:   dB
    0x0031A9A4, //   760:   dB
    0x00323CDB, //   761:   dB
    0x0032D1C7, //   762:   dB
    0x0033686D, //   763:   dB
    0x003400D1, //   764:   dB
    0x00349AF8, //   765:   dB
    0x003536E9, //   766:   dB
    0x0035D4A8, //   767:   dB
    0x0036743B, //   768:   dB
    0x003715A6, //   769:   dB
    0x0037B8F0, //   770:   dB
    0x00385E1E, //   771:   dB
    0x00390536, //   772:   dB
    0x0039AE3D, //   773:   dB
    0x003A5939, //   774:   dB
    0x003B0630, //   775:   dB
    0x003BB528, //   776:   dB
    0x003C6627, //   777:   dB
    0x003D1932, //   778:   dB
    0x003DCE50, //   779:   dB
    0x003E8586, //   780:   dB
    0x003F3EDC, //   781:   dB
    0x003FFA57, //   782:   dB
    0x0040B7FE, //   783:   dB
    0x004177D7, //   784:   dB
    0x004239E9, //   785:   dB
    0x0042FE3B, //   786:   dB
    0x0043C4D2, //   787:   dB
    0x00448DB5, //   788:   dB
    0x004558ED, //   789:   dB
    0x0046267E, //   790:   dB
    0x0046F672, //   791:   dB
    0x0047C8CD, //   792:   dB
    0x00489D98, //   793:   dB
    0x004974DA, //   794:   dB
    0x004A4E9A, //   795:   dB
    0x004B2AE0, //   796:   dB
    0x004C09B2, //   797:   dB
    0x004CEB19, //   798:   dB
    0x004DCF1C, //   799:   dB
    0x004EB5C3, //   800:   dB
    0x004F9F16, //   801:   dB
    0x00508B1C, //   802:   dB
    0x005179DE, //   803:   dB
    0x00526B64, //   804:   dB
    0x00535FB6, //   805:   dB
    0x005456DC, //   806:   dB
    0x005550DF, //   807:   dB
    0x00564DC6, //   808:   dB
    0x00574D9C, //   809:   dB
    0x00585068, //   810:   dB
    0x00595633, //   811:   dB
    0x005A5F06, //   812:   dB
    0x005B6AEA, //   813:   dB
    0x005C79E8, //   814:   dB
    0x005D8C0A, //   815:   dB
    0x005EA158, //   816:   dB
    0x005FB9DC, //   817:   dB
    0x0060D5A0, //   818:   dB
    0x0061F4AC, //   819:   dB
    0x0063170C, //   820:   dB
    0x00643CC9, //   821:   dB
    0x006565EC, //   822:   dB
    0x00669280, //   823:   dB
    0x0067C28F, //   824:   dB
    0x0068F624, //   825:   dB
    0x006A2D48, //   826:   dB
    0x006B6807, //   827:   dB
    0x006CA66A, //   828:   dB
    0x006DE87E, //   829:   dB
    0x006F2E4C, //   830:   dB
    0x007077E0, //   831:   dB
    0x0071C545, //   832:   dB
    0x00731686, //   833:   dB
    0x00746BAF, //   834:   dB
    0x0075C4CB, //   835:   dB
    0x007721E6, //   836:   dB
    0x0078830D, //   837:   dB
    0x0079E84A, //   838:   dB
    0x007B51AA, //   839:   dB
    0x007CBF39, //   840:   dB
    0x007E3104, //   841:   dB
    0x007FA717, //   842:   dB
    0x0081217F, //   843:   dB
    0x0082A048, //   844:   dB
    0x00842381, //   845:   dB
    0x0085AB35, //   846:   dB
    0x00873772, //   847:   dB
    0x0088C846, //   848:   dB
    0x008A5DBE, //   849:   dB
    0x008BF7E9, //   850:   dB
    0x008D96D2, //   851:   dB
    0x008F3A8A, //   852:   dB
    0x0090E31F, //   853:   dB
    0x0092909D, //   854:   dB
    0x00944315, //   855:   dB
    0x0095FA95, //   856:   dB
    0x0097B72B, //   857:   dB
    0x009978E8, //   858:   dB
    0x009B3FDA, //   859:   dB
    0x009D0C10, //   860:   dB
    0x009EDD9B, //   861:   dB
    0x00A0B489, //   862:   dB
    0x00A290EC, //   863:   dB
    0x00A472D2, //   864:   dB
    0x00A65A4E, //   865:   dB
    0x00A8476E, //   866:   dB
    0x00AA3A44, //   867:   dB
    0x00AC32E1, //   868:   dB
    0x00AE3155, //   869:   dB
    0x00B035B3, //   870:   dB
    0x00B2400C, //   871:   dB
    0x00B45071, //   872:   dB
    0x00B666F4, //   873:   dB
    0x00B883A8, //   874:   dB
    0x00BAA69E, //   875:   dB
    0x00BCCFEA, //   876:   dB
    0x00BEFF9E, //   877:   dB
    0x00C135CE, //   878:   dB
    0x00C3728B, //   879:   dB
    0x00C5B5EB, //   880:   dB
    0x00C80000, //   881:   dB
    0x00CA50DE, //   882:   dB
    0x00CCA899, //   883:   dB
    0x00CF0747, //   884:   dB
    0x00D16CFB, //   885:   dB
    0x00D3D9CA, //   886:   dB
    0x00D64DC9, //   887:   dB
    0x00D8C90E, //   888:   dB
    0x00DB4BAF, //   889:   dB
    0x00DDD5C0, //   890:   dB
    0x00E06758, //   891:   dB
    0x00E3008D, //   892:   dB
    0x00E5A177, //   893:   dB
    0x00E84A2B, //   894:   dB
    0x00EAFAC1, //   895:   dB
    0x00EDB350, //   896:   dB
    0x00F073F0, //   897:   dB
    0x00F33CB9, //   898:   dB
    0x00F60DC2, //   899:   dB
    0x00F8E725, //   900:   dB
    0x00FBC8FB, //   901:   dB
    0x00FEB35B, //   902:   dB
    0x0101A660, //   903:   dB
    0x0104A223, //   904:   dB
    0x0107A6BE, //   905:   dB
    0x010AB44C, //   906:   dB
    0x010DCAE6, //   907:   dB
    0x0110EAA8, //   908:   dB
    0x011413AC, //   909:   dB
    0x0117460F, //   910:   dB
    0x011A81EC, //   911:   dB
    0x011DC75E, //   912:   dB
    0x01211684, //   913:   dB
    0x01246F78, //   914:   dB
    0x0127D259, //   915:   dB
    0x012B3F43, //   916:   dB
    0x012EB655, //   917:   dB
    0x013237AD, //   918:   dB
    0x0135C369, //   919:   dB
    0x013959A7, //   920:   dB
    0x013CFA88, //   921:   dB
    0x0140A629, //   922:   dB
    0x01445CAD, //   923:   dB
    0x01481E32, //   924:   dB
    0x014BEAD9, //   925:   dB
    0x014FC2C4, //   926:   dB
    0x0153A613, //   927:   dB
    0x015794E8, //   928:   dB
    0x015B8F67, //   929:   dB
    0x015F95B0, //   930:   dB
    0x0163A7E7, //   931:   dB
    0x0167C630, //   932:   dB
    0x016BF0AF, //   933:   dB
    0x01702786, //   934:   dB
    0x01746ADC, //   935:   dB
    0x0178BAD5, //   936:   dB
    0x017D1796, //   937:   dB
    0x01818146, //   938:   dB
    0x0185F80B, //   939:   dB
    0x018A7C0B, //   940:   dB
    0x018F0D6E, //   941:   dB
    0x0193AC5B, //   942:   dB
    0x019858FB, //   943:   dB
    0x019D1376, //   944:   dB
    0x01A1DBF6, //   945:   dB
    0x01A6B2A3, //   946:   dB
    0x01AB97A8, //   947:   dB
    0x01B08B30, //   948:   dB
    0x01B58D65, //   949:   dB
    0x01BA9E72, //   950:   dB
    0x01BFBE85, //   951:   dB
    0x01C4EDC9, //   952:   dB
    0x01CA2C6C, //   953:   dB
    0x01CF7A9B, //   954:   dB
    0x01D4D883, //   955:   dB
    0x01DA4655, //   956:   dB
    0x01DFC43E, //   957:   dB
    0x01E5526F, //   958:   dB
    0x01EAF118, //   959:   dB
    0x01F0A06A, //   960:   dB
    0x01F66095  //   961:   dB
};

const unsigned int dbToGainIndex[] =
{
//    0xFFFFFFFF,
    0x7E560AF,
    0x7D6E289,
    0x7C87EFC,
    0x7BA35D8,
    0x7AC06ED,
    0x79DF20A,
    0x78FF6FF,
    0x782159E,
    0x7744DB7,
    0x7669F1C,
    0x759099D,
    0x74B8D0D,
    0x73E293E,
    0x730DE02,
    0x723AB2D,
    0x7169092,
    0x7098E04,
    0x6FCA357,
    0x6EFD05F,
    0x6E314F1,
    0x6D670E2,
    0x6C9E406,
    0x6BD6E33,
    0x6B10F40,
    0x6A4C700,
    0x698954D,
    0x68C79FA,
    0x68074E1,
    0x67485D7,
    0x668ACB5,
    0x65CE952,
    0x6513B86,
    0x645A32A,
    0x63A2016,
    0x62EB223,
    0x623592B,
    0x6181507,
    0x60CE590,
    0x601CAA1,
    0x5F6C415,
    0x5EBD1C5,
    0x5E0F38C,
    0x5D62946,
    0x5CB72CE,
    0x5C0D000,
    0x5B640B8,
    0x5ABC4D1,
    0x5A15C28,
    0x597069B,
    0x58CC405,
    0x5829444,
    0x5787735,
    0x56E6CB7,
    0x56474A6,
    0x55A8EE2,
    0x550BB48,
    0x546F9B8,
    0x53D4A10,
    0x533AC2F,
    0x52A1FF5,
    0x520A541,
    0x5173BF3,
    0x50DE3EC,
    0x5049D0A,
    0x4FB6730,
    0x4F2423E,
    0x4E92E14,
    0x4E02A94,
    0x4D737A0,
    0x4CE5518,
    0x4C582E0,
    0x4BCC0D8,
    0x4B40EE3,
    0x4AB6CE4,
    0x4A2DABD,
    0x49A5852,
    0x491E584,
    0x4898239,
    0x4812E53,
    0x478E9B6,
    0x470B446,
    0x4688DE7,
    0x460767D,
    0x4586DEE,
    0x450741D,
    0x44888F0,
    0x440AC4B,
    0x438DE15,
    0x4311E32,
    0x4296C89,
    0x421C8FF,
    0x41A337A,
    0x412ABE1,
    0x40B321A,
    0x403C60B,
    0x3FC679C,
    0x3F516B3,
    0x3EDD338,
    0x3E69D12,
    0x3DF7429,
    0x3D85864,
    0x3D149AB,
    0x3CA47E6,
    0x3C352FD,
    0x3BC6AD9,
    0x3B58F63,
    0x3AEC083,
    0x3A7FE21,
    0x3A14828,
    0x39A9E80,
    0x3940112,
    0x38D6FC9,
    0x386EA8E,
    0x380714A,
    0x37A03E9,
    0x373A253,
    0x36D4C74,
    0x3670236,
    0x360C383,
    0x35A9046,
    0x354686B,
    0x34E4BDC,
    0x3483A85,
    0x3423451,
    0x33C392C,
    0x3364901,
    0x33063BC,
    0x32A8949,
    0x324B995,
    0x31EF48C,
    0x3193A19,
    0x3138A2A,
    0x30DE4AB,
    0x308498A,
    0x302B8B2,
    0x2FD3212,
    0x2F7B596,
    0x2F2432C,
    0x2ECDAC2,
    0x2E77C44,
    0x2E227A2,
    0x2DCDCC8,
    0x2D79BA5,
    0x2D26427,
    0x2CD363C,
    0x2C811D2,
    0x2C2F6D9,
    0x2BDE53F,
    0x2B8DCF3,
    0x2B3DDE3,
    0x2AEE7FF,
    0x2A9FB35,
    0x2A51776,
    0x2A03CB0,
    0x29B6AD4,
    0x296A1D0,
    0x291E194,
    0x28D2A11,
    0x2887B36,
    0x283D4F4,
    0x27F373A,
    0x27AA1F9,
    0x2761522,
    0x27190A5,
    0x26D1472,
    0x268A07B,
    0x26434B1,
    0x25FD103,
    0x25B7564,
    0x25721C5,
    0x252D617,
    0x24E924B,
    0x24A5653,
    0x2462221,
    0x241F5A6,
    0x23DD0D4,
    0x239B39D,
    0x2359DF4,
    0x2318FC9,
    0x22D8910,
    0x22989BB,
    0x22591BD,
    0x221A107,
    0x21DB78C,
    0x219D540,
    0x215FA15,
    0x21225FD,
    0x20E58ED,
    0x20A92D6,
    0x206D3AD,
    0x2031B64,
    0x1FF69EF,
    0x1FBBF42,
    0x1F81B4F,
    0x1F47E0B,
    0x1F0E769,
    0x1ED575E,
    0x1E9CDDC,
    0x1E64AD8,
    0x1E2CE47,
    0x1DF581B,
    0x1DBE84B,
    0x1D87EC9,
    0x1D51B8A,
    0x1D1BE83,
    0x1CE67A9,
    0x1CB16F0,
    0x1C7CC4C,
    0x1C487B3,
    0x1C1491A,
    0x1BE1075,
    0x1BADDB9,
    0x1B7B0DD,
    0x1B489D4,
    0x1B16895,
    0x1AE4D14,
    0x1AB3748,
    0x1A82725,
    0x1A51CA1,
    0x1A217B2,
    0x19F184E,
    0x19C1E6B,
    0x19929FE,
    0x1963AFD,
    0x193515F,
    0x1906D19,
    0x18D8E22,
    0x18AB470,
    0x187DFF9,
    0x18510B4,
    0x1824698,
    0x17F8199,
    0x17CC1B1,
    0x17A06D4,
    0x17750FA,
    0x174A01A,
    0x171F429,
    0x16F4D21,
    0x16CAAF6,
    0x16A0DA1,
    0x1677519,
    0x164E154,
    0x162524A,
    0x15FC7F3,
    0x15D4245,
    0x15AC138,
    0x15844C4,
    0x155CCE1,
    0x1535984,
    0x150EAA8,
    0x14E8043,
    0x14C1A4C,
    0x149B8BD,
    0x1475B8C,
    0x14502B2,
    0x142AE27,
    0x1405DE3,
    0x13E11DF,
    0x13BCA11,
    0x1398673,
    0x13746FD,
    0x1350BA7,
    0x132D469,
    0x130A13D,
    0x12E721A,
    0x12C46FA,
    0x12A1FD5,
    0x127FCA3,
    0x125DD5E,
    0x123C1FD,
    0x121AA7B,
    0x11F96D0,
    0x11D86F4,
    0x11B7AE2,
    0x1197291,
    0x1176DFB,
    0x1156D1A,
    0x1136FE6,
    0x1117658,
    0x10F806A,
    0x10D8E16,
    0x10B9F54,
    0x109B41F,
    0x107CC6F,
    0x105E83E,
    0x1040786,
    0x1022A40,
    0x1005066,
    0xFE79F2,
    0xFCA6DE,
    0xFAD722,
    0xF90ABA,
    0xF7419F,
    0xF57BCB,
    0xF3B938,
    0xF1F9E0,
    0xF03DBD,
    0xEE84C9,
    0xECCEFF,
    0xEB1C58,
    0xE96CCF,
    0xE7C05E,
    0xE616FF,
    0xE470AD,
    0xE2CD63,
    0xE12D19,
    0xDF8FCC,
    0xDDF576,
    0xDC5E10,
    0xDAC997,
    0xD93804,
    0xD7A952,
    0xD61D7B,
    0xD4947C,
    0xD30E4D,
    0xD18AEB,
    0xD00A50,
    0xCE8C76,
    0xCD115A,
    0xCB98F5,
    0xCA2343,
    0xC8B040,
    0xC73FE5,
    0xC5D22E,
    0xC46716,
    0xC2FE99,
    0xC198B2,
    0xC0355B,
    0xBED491,
    0xBD764E,
    0xBC1A8E,
    0xBAC14D,
    0xB96A85,
    0xB81632,
    0xB6C450,
    0xB574DA,
    0xB427CC,
    0xB2DD21,
    0xB194D5,
    0xB04EE4,
    0xAF0B48,
    0xADC9FF,
    0xAC8B04,
    0xAB4E52,
    0xAA13E5,
    0xA8DBBA,
    0xA7A5CB,
    0xA67215,
    0xA54094,
    0xA41144,
    0xA2E420,
    0xA1B926,
    0xA09050,
    0x9F699A,
    0x9E4502,
    0x9D2283,
    0x9C0219,
    0x9AE3C0,
    0x99C775,
    0x98AD34,
    0x9794F9,
    0x967EC0,
    0x956A86,
    0x945846,
    0x9347FF,
    0x9239AB,
    0x912D47,
    0x9022CF,
    0x8F1A41,
    0x8E1399,
    0x8D0ED2,
    0x8C0BEA,
    0x8B0ADE,
    0x8A0BA9,
    0x890E48,
    0x8812B9,
    0x8718F7,
    0x862100,
    0x852AD0,
    0x843664,
    0x8343B8,
    0x8252CA,
    0x816396,
    0x807619,
    0x7F8A50,
    0x7EA038,
    0x7DB7CD,
    0x7CD10D,
    0x7BEBF5,
    0x7B0881,
    0x7A26AF,
    0x79467B,
    0x7867E2,
    0x778AE2,
    0x76AF78,
    0x75D5A0,
    0x74FD59,
    0x74269E,
    0x73516D,
    0x727DC4,
    0x71AB9F,
    0x70DAFC,
    0x700BD8,
    0x6F3E30,
    0x6E7202,
    0x6DA74A,
    0x6CDE06,
    0x6C1634,
    0x6B4FD0,
    0x6A8AD9,
    0x69C74B,
    0x690524,
    0x684462,
    0x678501,
    0x66C700,
    0x660A5B,
    0x654F11,
    0x64951E,
    0x63DC80,
    0x632536,
    0x626F3C,
    0x61BA90,
    0x61072F,
    0x605518,
    0x5FA448,
    0x5EF4BC,
    0x5E4672,
    0x5D9968,
    0x5CED9C,
    0x5C430B,
    0x5B99B4,
    0x5AF193,
    0x5A4AA6,
    0x59A4EC,
    0x590063,
    0x585D07,
    0x57BAD7,
    0x5719D1,
    0x5679F2,
    0x55DB39,
    0x553DA3,
    0x54A12E,
    0x5405D9,
    0x536BA0,
    0x52D283,
    0x523A7F,
    0x51A391,
    0x510DB9,
    0x5078F4,
    0x4FE540,
    0x4F529B,
    0x4EC103,
    0x4E3076,
    0x4DA0F3,
    0x4D1277,
    0x4C8500,
    0x4BF88E,
    0x4B6D1D,
    0x4AE2AC,
    0x4A5939,
    0x49D0C2,
    0x494946,
    0x48C2C2,
    0x483D36,
    0x47B89E,
    0x4734FA,
    0x46B248,
    0x463085,
    0x45AFB1,
    0x452FC9,
    0x44B0CB,
    0x4432B7,
    0x43B58A,
    0x433943,
    0x42BDE1,
    0x424360,
    0x41C9C1,
    0x415100,
    0x40D91E,
    0x406217,
    0x3FEBEB,
    0x3F7698,
    0x3F021C,
    0x3E8E76,
    0x3E1BA4,
    0x3DA9A5,
    0x3D3877,
    0x3CC819,
    0x3C5889,
    0x3BE9C6,
    0x3B7BCE,
    0x3B0EA0,
    0x3AA23A,
    0x3A369C,
    0x39CBC2,
    0x3961AD,
    0x38F85B,
    0x388FCA,
    0x3827F9,
    0x37C0E7,
    0x375A91,
    0x36F4F8,
    0x369019,
    0x362BF3,
    0x35C885,
    0x3565CE,
    0x3503CB,
    0x34A27D,
    0x3441E1,
    0x33E1F6,
    0x3382BC,
    0x332430,
    0x32C652,
    0x326920,
    0x320C99,
    0x31B0BC,
    0x315588,
    0x30FAFB,
    0x30A114,
    0x3047D3,
    0x2FEF35,
    0x2F9739,
    0x2F3FE0,
    0x2EE926,
    0x2E930C,
    0x2E3D90,
    0x2DE8B0,
    0x2D946D,
    0x2D40C4,
    0x2CEDB4,
    0x2C9B3E,
    0x2C495E,
    0x2BF815,
    0x2BA761,
    0x2B5741,
    0x2B07B4,
    0x2AB8B9,
    0x2A6A4F,
    0x2A1C75,
    0x29CF2A,
    0x29826D,
    0x29363D,
    0x28EA98,
    0x289F7E,
    0x2854EF,
    0x280AE8,
    0x27C168,
    0x277870,
    0x272FFE,
    0x26E811,
    0x26A0A7,
    0x2659C1,
    0x26135D,
    0x25CD7A,
    0x258818,
    0x254334,
    0x24FED0,
    0x24BAE8,
    0x24777E,
    0x24348F,
    0x23F21B,
    0x23B021,
    0x236EA0,
    0x232D97,
    0x22ED06,
    0x22ACEB,
    0x226D45,
    0x222E15,
    0x21EF59,
    0x21B10F,
    0x217339,
    0x2135D3,
    0x20F8DE,
    0x20BC59,
    0x208044,
    0x20449C,
    0x200962,
    0x1FCE95,
    0x1F9434,
    0x1F5A3D,
    0x1F20B2,
    0x1EE78F,
    0x1EAED6,
    0x1E7685,
    0x1E3E9B,
    0x1E0718,
    0x1DCFFA,
    0x1D9942,
    0x1D62EE,
    0x1D2CFE,
    0x1CF771,
    0x1CC247,
    0x1C8D7D,
    0x1C5915,
    0x1C250D,
    0x1BF165,
    0x1BBE1B,
    0x1B8B2F,
    0x1B58A1,
    0x1B2670,
    0x1AF49A,
    0x1AC321,
    0x1A9202,
    0x1A613D,
    0x1A30D2,
    0x1A00BF,
    0x19D105,
    0x19A1A2,
    0x197297,
    0x1943E2,
    0x191582,
    0x18E778,
    0x18B9C2,
    0x188C60,
    0x185F51,
    0x183295,
    0x18062B,
    0x17DA13,
    0x17AE4B,
    0x1782D4,
    0x1757AD,
    0x172CD5,
    0x17024B,
    0x16D810,
    0x16AE22,
    0x168481,
    0x165B2D,
    0x163224,
    0x160967,
    0x15E0F4,
    0x15B8CC,
    0x1590ED,
    0x156958,
    0x15420B,
    0x151B06,
    0x14F449,
    0x14CDD3,
    0x14A7A4,
    0x1481BB,
    0x145C17,
    0x1436B9,
    0x14119F,
    0x13ECC9,
    0x13C836,
    0x13A3E7,
    0x137FDB,
    0x135C10,
    0x133888,
    0x131540,
    0x12F23A,
    0x12CF73,
    0x12ACED,
    0x128AA6,
    0x12689D,
    0x1246D3,
    0x122548,
    0x1203F9,
    0x11E2E8,
    0x11C214,
    0x11A17C,
    0x11811F,
    0x1160FF,
    0x114119,
    0x11216D,
    0x1101FC,
    0x10E2C4,
    0x10C3C6,
    0x10A501,
    0x108674,
    0x10681F,
    0x104A02,
    0x102C1C,
    0x100E6D,
    0xFF0F4,
    0xFD3B2,
    0xFB6A5,
    0xF99CE,
    0xF7D2B,
    0xF60BE,
    0xF4484,
    0xF287E,
    0xF0CAB,
    0xEF10C,
    0xED59F,
    0xEBA65,
    0xE9F5D,
    0xE8486,
    0xE69E0,
    0xE4F6C,
    0xE3528,
    0xE1B14,
    0xE0130,
    0xDE77B,
    0xDCDF6,
    0xDB49F,
    0xD9B78,
    0xD827E,
    0xD69B2,
    0xD5113,
    0xD38A2,
    0xD205E,
    0xD0846,
    0xCF05A,
    0xCD89B,
    0xCC107,
    0xCA99E,
    0xC9260,
    0xC7B4D,
    0xC6464,
    0xC4DA5,
    0xC3710,
    0xC20A5,
    0xC0A62,
    0xBF448,
    0xBDE57,
    0xBC88F,
    0xBB2EE,
    0xB9D75,
    0xB8823,
    0xB72F9,
    0xB5DF5,
    0xB4918,
    0xB3461,
    0xB1FD0,
    0xB0B65,
    0xAF720,
    0xAE2FF,
    0xACF04,
    0xABB2D,
    0xAA77B,
    0xA93ED,
    0xA8082,
    0xA6D3C,
    0xA5A18,
    0xA4718,
    0xA343B,
    0xA2180,
    0xA0EE8,
    0x9FC72,
    0x9EA1E,
    0x9D7EB,
    0x9C5DA,
    0x9B3EA,
    0x9A21B,
    0x9906C,
    0x97EDE,
    0x96D70,
    0x95C23,
    0x94AF5,
    0x939E6,
    0x928F7,
    0x91827,
    0x90776,
    0x8F6E3,
    0x8E66F,
    0x8D619,
    0x8C5E1,
    0x8B5C7,
    0x8A5CA,
    0x895EB,
    0x88629,
    0x87683,
    0x866FB,
    0x8578F,
    0x8483F,
    0x8390B,
    0x829F4,
    0x81AF8,
    0x80C17,
    0x7FD52,
    0x7EEA8,
    0x7E019,
    0x7D1A4,
    0x7C34A,
    0x7B50B,
    0x7A6E5,
    0x798DA,
    0x78AE8,
    0x77D10,
    0x76F51,
    0x761AC,
    0x7541F,
    0x746AC,
    0x73951,
    0x72C0E,
    0x71EE4,
    0x711D3,
    0x704D9,
    0x6F7F7,
    0x6EB2C,
    0x6DE79,
    0x6D1DE,
    0x6C559,
    0x6B8EC,
    0x6AC95,
    0x6A055,
    0x6942C,
    0x68818,
    0x67C1B,
    0x67034,
    0x66463,
    0x658A7,
    0x64D01,
    0x64171,
    0x635F5,
    0x62A8F,
    0x61F3E,
    0x61401,
    0x608D9,
    0x5FDC6,
    0x5F2C6,
    0x5E7DB,
    0x5DD04,
    0x5D241,
    0x5C792,
    0x5BCF6,
    0x5B26E,
    0x5A7F9,
    0x59D98,
    0x59349,
    0x5890D,
    0x57EE4,
    0x574CE,
    0x56ACA,
    0x560D9,
    0x556FA,
    0x54D2D,
    0x54372,
    0x539C8,
    0x53031,
    0x526AB,
    0x51D37,
    0x513D4,
    0x50A82,
    0x50141,
    0x4F811,
    0x4EEF3,
    0x4E5E5,
    0x4DCE7,
    0x4D3FA,
    0x4CB1E,
    0x4C251,
    0x4B995,
    0x4B0E9,
    0x4A84D,
    0x49FC0,
    0x49743,
    0x48ED6,
    0x48679,
    0x47E2A,
    0x475EB,
    0x46DBB,
    0x4659A,
    0x45D88,
    0x45585,
    0x44D91,
    0x445AB,
    0x43DD3,
    0x4360A,
    0x42E50,
    0x426A3,
    0x41F05,
    0x41774,
    0x40FF2,
    0x4087D,
    0x40116,
    0x3F9BC,
    0x3F270,
    0x3EB32,
    0x3E400,
    0x3DCDC,
    0x3D5C5,
    0x3CEBB,
    0x3C7BE,
    0x3C0CE,
    0x3B9EA,
    0x3B313,
    0x3AC49,
    0x3A58B,
    0x39EDA,
    0x39835,
    0x3919C,
    0x38B0F,
    0x3848E,
    0x37E19,
    0x377B0,
    0x37152,
    0x36B01,
    0x364BB,
    0x35E80,
    0x35851,
    0x3522D,
    0x34C15,
    0x34608,
    0x34005,
    0x33A0E,
    0x33422,
    0x32E41,
    0x3286A,
    0x3229E,
    0x31CDD,
    0x31727,
    0x3117A,
    0x30BD9,
    0x30641,
    0x300B4,
    0x2FB31,
    0x2F5B8,
    0x2F04A,
    0x2EAE5,
    0x2E58A,
    0x2E039,
    0x2DAF2,
    0x2D5B4,
    0x2D080,
    0x2CB56,
    0x2C635,
    0x2C11D,
    0x2BC0F,
    0x2B70A,
    0x2B20E,
    0x2AD1C,
    0x2A832,
    0x2A352,
    0x29E7A,
    0x299AB,
    0x294E6,
    0x29028,
    0x28B74,
    0x286C8,
    0x28225,
    0x27D8B,
    0x278F8,
    0x2746F,
    0x26FED,
    0x26B74,
    0x26703,
    0x2629A,
    0x25E39,
    0x259E1,
    0x25590,
    0x25147,
    0x24D06,
    0x248CD,
    0x2449C,
    0x24072,
    0x23C50,
    0x23835,
    0x23423,
    0x23017,
    0x22C13,
    0x22816,
    0x22421,
    0x22033,
    0x21C4C,
    0x2186C,
    0x21494,
    0x210C2,
    0x20CF8,
    0x20934,
    0x20578,
    0x201C2,
    0x1FE13,
    0x1FA6A,
    0x1F6C9,
    0x1F32E,
    0x1EF9A,
    0x1EC0C,
    0x1E885,
    0x1E504,
    0x1E18A,
    0x1DE16,
    0x1DAA9,
    0x1D742,
    0x1D3E1,
    0x1D086,
    0x1CD31,
    0x1C9E3,
    0x1C69A,
    0x1C358,
    0x1C01B,
    0x1BCE5,
    0x1B9B4,
    0x1B68A,
    0x1B365,
    0x1B045,
    0x1AD2C,
    0x1AA18,
    0x1A70A,
    0x1A402,
    0x1A0FF,
    0x19E01,
    0x19B0A,
    0x19817,
    0x1952A,
    0x19242,
    0x18F60,
    0x18C83,
    0x189AB,
    0x186D9,
    0x1840B,
    0x18143,
    0x17E80,
    0x17BC2,
    0x17909,
    0x17655,
    0x173A6,
    0x170FB,
    0x16E56,
    0x16BB6,
    0x1691A,
    0x16683,
    0x163F1,
    0x16164,
    0x15EDB,
    0x15C57,
    0x159D8,
    0x1575D,
    0x154E7,
    0x15275,
    0x15008,
    0x14D9F,
    0x14B3B,
    0x148DB,
    0x1467F,
    0x14428,
    0x141D5,
    0x13F86,
    0x13D3C,
    0x13AF6,
    0x138B3,
    0x13676,
    0x1343C,
    0x13206,
    0x12FD4,
    0x12DA7,
    0x12B7D,
    0x12957,
    0x12735,
    0x12518,
    0x122FE,
    0x120E7,
    0x11ED5,
    0x11CC7,
    0x11ABC,
    0x118B5,
    0x116B2,
    0x114B2,
    0x112B6,
    0x110BE,
    0x10ECA,
    0x10CD8,
    0x10AEB,
    0x10901,
    0x1071B,
    0x10538,
    0x10358,
    0x1017C,
    0xFFA4,
    0xFDCE,
    0xFBFD,
    0xFA2E,
    0xF863,
    0xF69B,
    0xF4D6,
    0xF315,
    0xF157,
    0xEF9C,
    0xEDE4,
    0xEC2F,
    0xEA7E,
    0xE8CF,
    0xE724,
    0xE57C,
    0xE3D7,
    0xE234,
    0xE095,
    0xDEF9,
    0xDD60,
    0xDBC9,
    0xDA36,
    0xD8A5,
    0xD718,
    0xD58D,
    0xD405,
    0xD280,
    0xD0FD,
    0xCF7E,
    0xCE01,
    0xCC87,
    0xCB0F,
    0xC99B,
    0xC829,
    0xC6B9,
    0xC54D,
    0xC3E2,
    0xC27B,
    0xC116,
    0xBFB3,
    0xBE54,
    0xBCF6,
    0xBB9B,
    0xBA43,
    0xB8ED,
    0xB79A,
    0xB649,
    0xB4FA,
    0xB3AE,
    0xB264,
    0xB11D,
    0xAFD8,
    0xAE95,
    0xAD54,
    0xAC16,
    0xAADA,
    0xA9A1,
    0xA869,
    0xA734,
    0xA601,
    0xA4D1,
    0xA3A2,
    0xA276,
    0xA14C,
    0xA024,
    0x9EFE,
    0x9DDA,
    0x9CB8,
    0x9B98,
    0x9A7B,
    0x995F,
    0x9846,
    0x972E,
    0x9619,
    0x9505,
    0x93F4,
    0x92E4,
    0x91D6,
    0x90CB,
    0x8FC1,
    0x8EB9,
    0x8DB3,
    0x8CAF,
    0x8BAD,
    0x8AAC,
    0x89AE,
    0x88B1,
    0x87B6,
    0x86BD,
    0x85C6,
    0x84D0,
    0x83DD,
    0x82EB,
    0x81FA,
    0x810C,
    0x801F,
    0x7F34,
    0x7E4A,
    0x7D62,
    0x7C7C,
    0x7B98,
    0x7AB5,
    0x79D4,
    0x78F4,
    0x7816,
    0x773A,
    0x765F,
    0x7585,
    0x74AE,
    0x73D8,
    0x7303,
    0x7230,
    0x715E,
    0x708E,
    0x6FBF,
    0x6EF2,
    0x6E27,
    0x6D5D,
    0x6C94,
    0x6BCC,
    0x6B07,
    0x6A42,
    0x697F,
    0x68BD,
    0x67FD,
    0x673E,
    0x6681,
    0x65C5,
    0x650A,
    0x6450,
    0x6398,
    0x62E1,
    0x622C,
    0x6178,
    0x60C5,
    0x6013,
    0x5F63,
    0x5EB4,
    0x5E06,
    0x5D59,
    0x5CAE,
    0x5C04,
    0x5B5B,
    0x5AB3,
    0x5A0D,
    0x5968,
    0x58C3,
    0x5820,
    0x577F,
    0x56DE,
    0x563F,
    0x55A0,
    0x5503,
    0x5467,
    0x53CC,
    0x5332,
    0x529A,
    0x5202,
    0x516C,
    0x50D6,
    0x5042,
    0x4FAE,
    0x4F1C,
    0x4E8B,
    0x4DFB,
    0x4D6C,
    0x4CDD,
    0x4C50,
    0x4BC4,
    0x4B39,
    0x4AAF,
    0x4A26,
    0x499E,
    0x4917,
    0x4891,
    0x480B,
    0x4787,
    0x4704,
    0x4682,
    0x4600,
    0x4580,
    0x4500,
    0x4481,
    0x4404,
    0x4387,
    0x430B,
    0x4290,
    0x4216,
    0x419C,
    0x4124,
    0x40AC,
    0x4036,
    0x3FC0,
    0x3F4B,
    0x3ED7,
    0x3E63,
    0x3DF1,
    0x3D7F,
    0x3D0E,
    0x3C9E,
    0x3C2F,
    0x3BC0,
    0x3B53,
    0x3AE6,
    0x3A7A,
    0x3A0E,
    0x39A4,
    0x393A,
    0x38D1,
    0x3869,
    0x3801,
    0x379A,
    0x3734,
    0x36CF,
    0x366A,
    0x3606,
    0x35A3,
    0x3541,
    0x34DF,
    0x347E,
    0x341D,
    0x33BE,
    0x335F,
    0x3301,
    0x32A3,
    0x3246,
    0x31EA,
    0x318E,
    0x3133,
    0x30D9,
    0x307F,
    0x3026,
    0x2FCE,
    0x2F76,
    0x2F1F,
    0x2EC8,
    0x2E72,
    0x2E1D,
    0x2DC9,
    0x2D74,
    0x2D21,
    0x2CCE,
    0x2C7C,
    0x2C2A,
    0x2BD9,
    0x2B89,
    0x2B39,
    0x2AE9,
    0x2A9B,
    0x2A4C,
    0x29FF,
    0x29B2,
    0x2965,
    0x2919,
    0x28CE,
    0x2883,
    0x2838,
    0x27EF,
    0x27A5,
    0x275D,
    0x2714,
    0x26CD,
    0x2685,
    0x263F,
    0x25F8,
    0x25B3,
    0x256E,
    0x2529,
    0x24E5,
    0x24A1,
    0x245E,
    0x241B,
    0x23D9,
    0x2397,
    0x2355,
    0x2315,
    0x22D4,
    0x2294,
    0x2255,
    0x2216,
    0x21D7,
    0x2199,
    0x215B,
    0x211E,
    0x20E1,
    0x20A5,
    0x2069,
    0x202E,
    0x1FF2,
    0x1FB8,
    0x1F7E,
    0x1F44,
    0x1F0A,
    0x1ED1,
    0x1E99,
    0x1E61,
    0x1E29,
    0x1DF2,
    0x1DBB,
    0x1D84,
    0x1D4E,
    0x1D18,
    0x1CE3,
    0x1CAE,
    0x1C79,
    0x1C45,
    0x1C11,
    0x1BDD,
    0x1BAA,
    0x1B77,
    0x1B45,
    0x1B13,
    0x1AE1,
    0x1AB0,
    0x1A7F,
    0x1A4E,
    0x1A1E,
    0x19EE,
    0x19BE,
    0x198F,
    0x1960,
    0x1932,
    0x1903,
    0x18D5,
    0x18A8,
    0x187A,
    0x184E,
    0x1821,
    0x17F5,
    0x17C9,
    0x179D,
    0x1772,
    0x1747,
    0x171C,
    0x16F1,
    0x16C7,
    0x169D,
    0x1674,
    0x164B,
    0x1622,
    0x15F9,
    0x15D1,
    0x15A9,
    0x1581,
    0x155A,
    0x1532,
    0x150B,
    0x14E5,
    0x14BE,
    0x1498,
    0x1473,
    0x144D,
    0x1428,
    0x1403,
    0x13DE,
    0x13BA,
    0x1395,
    0x1371,
    0x134E,
    0x132A,
    0x1307,
    0x12E4,
    0x12C1,
    0x129F,
    0x127D,
    0x125B,
    0x1239,
    0x1218,
    0x11F6,
    0x11D5,
    0x11B5,
    0x1194,
    0x1174,
    0x1154,
    0x1134,
    0x1114,
    0x10F5,
    0x10D6,
    0x10B7,
    0x1098,
    0x107A,
    0x105C,
    0x103E,
    0x1020,
    0x1002,
    0xFE5,
    0xFC8,
    0xFAB,
    0xF8E,
    0xF71,
    0xF55,
    0xF39,
    0xF1D,
    0xF01,
    0xEE6,
    0xECA,
    0xEAF,
    0xE94,
    0xE79,
    0xE5F,
    0xE44,
    0xE2A,
    0xE10,
    0xDF6,
    0xDDD,
    0xDC3,
    0xDAA,
    0xD91,
    0xD78,
    0xD5F,
    0xD47,
    0xD2E,
    0xD16,
    0xCFE,
    0xCE6,
    0xCCF,
    0xCB7,
    0xCA0,
    0xC88,
    0xC71,
    0xC5B,
    0xC44,
    0xC2D,
    0xC17,
    0xC01,
    0xBEB,
    0xBD5,
    0xBBF,
    0xBAA,
    0xB94,
    0xB7F,
    0xB6A,
    0xB55,
    0xB40,
    0xB2B,
    0xB17,
    0xB03,
    0xAEE,
    0xADA,
    0xAC6,
    0xAB3,
    0xA9F,
    0xA8B,
    0xA78,
    0xA65,
    0xA52,
    0xA3F,
    0xA2C,
    0xA19,
    0xA07,
    0x9F4,
    0x9E2,
    0x9D0,
    0x9BE,
    0x9AC,
    0x99A,
    0x989,
    0x977,
    0x966,
    0x954,
    0x943,
    0x932,
    0x921,
    0x911,
    0x900,
    0x8EF,
    0x8DF,
    0x8CF,
    0x8BF,
    0x8AE,
    0x89F,
    0x88F,
    0x87F,
    0x86F,
    0x860,
    0x850,
    0x841,
    0x832,
    0x823,
    0x814,
    0x805,
    0x7F6,
    0x7E8,
    0x7D9,
    0x7CB,
    0x7BD,
    0x7AE,
    0x7A0,
    0x792,
    0x784,
    0x777,
    0x769,
    0x75B,
    0x74E,
    0x740,
    0x733,
    0x726,
    0x719,
    0x70C,
    0x6FF,
    0x6F2,
    0x6E5,
    0x6D8,
    0x6CC,
    0x6BF,
    0x6B3,
    0x6A7,
    0x69A,
    0x68E,
    0x682,
    0x676,
    0x66A,
    0x65F,
    0x653,
    0x647,
    0x63C,
    0x630,
    0x625,
    0x61A,
    0x60E,
    0x603,
    0x5F8,
    0x5ED,
    0x5E2,
    0x5D8,
    0x5CD,
    0x5C2,
    0x5B8,
    0x5AD,
    0x5A3,
    0x598,
    0x58E,
    0x584,
    0x57A,
    0x570,
    0x566,
    0x55C,
    0x552,
    0x548,
    0x53E,
    0x535,
    0x52B,
    0x522,
    0x518,
    0x50F,
    0x506,
    0x4FC,
    0x4F3,
    0x4EA,
    0x4E1,
    0x4D8,
    0x4CF,
    0x4C6,
    0x4BE,
    0x4B5,
    0x4AC,
    0x4A4,
    0x49B,
    0x493,
    0x48A,
    0x482,
    0x47A,
    0x471,
    0x469,
    0x461,
    0x459,
    0x451,
    0x449,
    0x441,
    0x439,
    0x432,
    0x42A,
    0x422,
    0x41B,
    0x413,
    0x40C,
    0x404,
    0x3FD,
    0x3F6,
    0x3EE,
    0x3E7,
    0x3E0,
    0x3D9,
    0x3D2,
    0x3CB,
    0x3C4,
    0x3BD,
    0x3B6,
    0x3AF,
    0x3A8,
    0x3A2,
    0x39B,
    0x394,
    0x38E,
    0x387,
    0x381,
    0x37A,
    0x374,
    0x36E,
    0x367,
    0x361,
    0x35B,
    0x355,
    0x34E,
    0x348,
    0x342,
    0x33C,
    0x336,
    0x330,
    0x32B,
    0x325,
    0x31F,
    0x319,
    0x314,
    0x30E,
    0x308,
    0x303,
    0x2FD,
    0x2F8,
    0x2F2,
    0x2ED,
    0x2E7,
    0x2E2,
    0x2DD,
    0x2D8,
    0x2D2,
    0x2CD,
    0x2C8,
    0x2C3,
    0x2BE,
    0x2B9,
    0x2B4,
    0x2AF,
    0x2AA,
    0x2A5,
    0x2A0,
    0x29B,
    0x296,
    0x292,
    0x28D,
    0x288,
    0x284,
    0x27F,
    0x27A,
    0x276,
    0x271,
    0x26D,
    0x268,
    0x264,
    0x260,
    0x25B,
    0x257,
    0x253,
    0x24E,
    0x24A,
    0x246,
    0x242,
    0x23D,
    0x239,
    0x235,
    0x231,
    0x22D,
    0x229,
    0x225,
    0x221,
    0x21D,
    0x219,
    0x216,
    0x212,
    0x20E,
    0x20A,
    0x206,
    0x203,
    0x1FF,
    0x1FB,
    0x1F8,
    0x1F4,
    0x1F0,
    0x1ED,
    0x1E9,
    0x1E6,
    0x1E2,
    0x1DF,
    0x1DB,
    0x1D8,
    0x1D5,
    0x1D1,
    0x1CE,
    0x1CA,
    0x1C7,
    0x1C4,
    0x1C1,
    0x1BD,
    0x1BA,
    0x1B7,
    0x1B4,
    0x1B1,
    0x1AE,
    0x1AB,
    0x1A7,
    0x1A4,
    0x1A1,
    0x19E,
    0x19B,
    0x198,
    0x196,
    0x193,
    0x190,
    0x18D,
    0x18A,
    0x187,
    0x184,
    0x182,
    0x17F,
    0x17C,
    0x179,
    0x177,
    0x174,
    0x171,
    0x16F,
    0x16C,
    0x169,
    0x167,
    0x164,
    0x162,
    0x15F,
    0x15C,
    0x15A,
    0x157,
    0x155,
    0x153,
    0x150,
    0x14E,
    0x14B,
    0x149,
    0x147,
    0x144,
    0x142,
    0x13F,
    0x13D,
    0x13B,
    0x139,
    0x136,
    0x134,
    0x132,
    0x130,
    0x12E,
    0x12B,
    0x129,
    0x127,
    0x125,
    0x123,
    0x121,
    0x11F,
    0x11D,
    0x11B,
    0x119,
    0x116,
    0x114,
    0x112,
    0x111,
    0x10F,
    0x10D,
    0x10B,
    0x109,
    0x107,
    0x105,
    0x103,
    0x101,
    0xFF,
    0xFD,
    0xFC,
    0xFA,
    0xF8,
    0xF6,
    0xF4,
    0xF3,
    0xF1,
    0xEF,
    0xED,
    0xEC,
    0xEA,
    0xE8,
    0xE7,
    0xE5,
    0xE3,
    0xE2,
    0xE0,
    0xDF,
    0xDD,
    0xDB,
    0xDA,
    0xD8,
    0xD7,
    0xD5,
    0xD3,
    0xD2,
    0xD0,
    0xCF,
    0xCD,
    0xCC,
    0xCA,
    0xC9,
    0xC8,
    0xC6,
    0xC5,
    0xC3,
    0xC2,
    0xC0,
    0xBF,
    0xBE,
    0xBC,
    0xBB,
    0xBA,
    0xB8,
    0xB7,
    0xB6,
    0xB4,
    0xB3,
    0xB2,
    0xB0,
    0xAF,
    0xAE,
    0xAD,
    0xAB,
    0xAA,
    0xA9,
    0xA8,
    0xA6,
    0xA5,
    0xA4,
    0xA3,
    0xA2,
    0xA1,
    0x9F,
    0x9E,
    0x9D,
    0x9C,
    0x9B,
    0x9A,
    0x99,
    0x97,
    0x96,
    0x95,
    0x94,
    0x93,
    0x92,
    0x91,
    0x90,
    0x8F,
    0x8E,
    0x8D,
    0x8C,
    0x8B,
    0x8A,
    0x89,
    0x88,
    0x87,
    0x86,
    0x85,
    0x84,
    0x83,
    0x82,
    0x81,
    0x80,
    0x7F,
    0x7E,
    0x7D,
    0x7C,
    0x7C,
    0x7B,
    0x7A,
    0x79,
    0x78,
    0x77,
    0x76,
    0x75,
    0x75,
    0x74,
    0x73,
    0x72,
    0x71,
    0x70,
    0x70,
    0x6F,
    0x6E,
    0x6D,
    0x6C,
    0x6C,
    0x6B,
    0x6A,
    0x69,
    0x68,
    0x68,
    0x67,
    0x66,
    0x65,
    0x65,
    0x64,
    0x63,
    0x63,
    0x62,
    0x61,
    0x60,
    0x60,
    0x5F,
    0x5E,
    0x5E,
    0x5D,
    0x5C,
    0x5C,
    0x5B,
    0x5A,
    0x5A,
    0x59,
    0x58,
    0x58,
    0x57,
    0x56,
    0x56,
    0x55,
    0x55,
    0x54,
    0x53,
    0x53,
    0x52,
    0x51,
    0x51,
    0x50,
    0x50,
    0x4F,
    0x4F,
    0x4E,
    0x4D,
    0x4D,
    0x4C,
    0x4C,
    0x4B,
    0x4B,
    0x4A,
    0x4A,
    0x49,
    0x48,
    0x48,
    0x47,
    0x47,
    0x46,
    0x46,
    0x45,
    0x45,
    0x44,
    0x44,
    0x43,
    0x43,
    0x42,
    0x42,
    0x41,
    0x41,
    0x40,
    0x40,
    0x3F,
    0x3F,
    0x3F,
    0x3E,
    0x3E,
    0x3D,
    0x3D,
    0x3C,
    0x3C,
    0x3B,
    0x3B,
    0x3B,
    0x3A,
    0x3A,
    0x39,
    0x39,
    0x38,
    0x38,
    0x38,
    0x37,
    0x37,
    0x36,
    0x36,
    0x36,
    0x35,
    0x35,
    0x34,
    0x34
};

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------
#endif  // _AUD_CUST_AMP_RT9119_H_
