#ifndef _DRV_MAPLE_API_H_
#define _DRV_MAPLE_API_H_

#include "general.h"
#include "util.h"
#include "hw_sw.h"
#include "hw_srs.h"
#include "video_def.h"
#include "pe_if.h"
#include "panel.h"
#include "drv_meter.h"
#include "drv_di.h"
#include "drv_dvi2.h"
#include "drv_tdtv_drvif.h"
#include "drv_scaler.h"
#include "drv_mjc.h"
#include "drv_pq_cust.h"
#include "mute_if.h"
#include "drv_mjc_sys.h"

#define DELAY_TABLE_NUM	31


//////////////////////////////////////////////////////////////////

// drv_MapleGetApiSys.c

typedef enum {
    E_CUST_INPUT_TYPE_A_RF,
    E_CUST_INPUT_TYPE_D_RF,
    E_CUST_INPUT_TYPE_COMPOSITE,
    E_CUST_INPUT_TYPE_COMPONENT,
    E_CUST_INPUT_TYPE_SCART,
    E_CUST_INPUT_TYPE_A_PC,
    E_CUST_INPUT_TYPE_HDMI,
    E_CUST_INPUT_TYPE_USB,
    E_CUST_INPUT_TYPE_DNLA,
    E_CUST_INPUT_TYPE_IPTV,
    E_CUST_INPUT_TYPE_BROWSER,
    E_CUST_INPUT_TYPE_WIFI_DISPLAY,
    E_CUST_INPUT_TYPE_UNKNOWN,
} E_CUST_INPUT_TYPE;

typedef enum {
    E_CUST_RESTRICT_COND_BE_VIDEO_PATH_0, //4k bypass to mtk
    E_CUST_RESTRICT_COND_BE_VIDEO_PATH_1, //predown to 4k to mtk
    E_CUST_RESTRICT_COND_BE_VIDEO_PATH_2, //8k div photo
    E_CUST_RESTRICT_COND_BE_VIDEO_PATH_3  //8k bypass to panel
} E_CUST_RESTRICT_COND;

typedef struct {
    D_AP_INPUT_TYPE	eMainInputType;
	D_AP_INPUT_TYPE   eSubInputType;
} CUST_INPUT_TYPE_T;

typedef enum {
    E_CUST_SIG_STA_STABLE,
    E_CUST_SIG_STA_NO_SIG,
    E_CUST_SIG_STA_UNKNOWN,
} E_CUST_SIG_STA;

typedef enum {
    E_CUST_CONTENT_TYPE_VIDEO,
    E_CUST_CONTENT_TYPE_PHOTO, // for Photo contents
    E_CUST_CONTENT_TYPE_GRAPHICS // for Analog PC, HDMI PC, Browser
} E_CUST_CONTENT_TYPE;

typedef enum {
    E_CUST_SCAN_TYPE_INTERLACE,
    E_CUST_SCAN_TYPE_PROGRESSIVE,
    E_CUST_SCAN_TYPE_UNKNOWN
} E_CUST_SCAN_TYPE;

typedef enum {
    E_CUST_COLOR_SYSTEM_NTSC_358,
    E_CUST_COLOR_SYSTEM_NTSC_443,
    E_CUST_COLOR_SYSTEM_PAL,
    E_CUST_COLOR_SYSTEM_SECAM,
    E_CUST_COLOR_SYSTEM_PAL_M,
    E_CUST_COLOR_SYSTEM_PAL_N,
    E_CUST_COLOR_SYSTEM_PAL60,
    E_CUST_COLOR_SYSTEM_UNKNOWN
} E_CUST_COLOR_SYSTEM;

typedef enum {
    E_CUST_FRAME_RATE_8,
    E_CUST_FRAME_RATE_10,
    E_CUST_FRAME_RATE_12,
    E_CUST_FRAME_RATE_15,
    E_CUST_FRAME_RATE_23_976,
    E_CUST_FRAME_RATE_24,
    E_CUST_FRAME_RATE_25,
    E_CUST_FRAME_RATE_29_97,
    E_CUST_FRAME_RATE_30,
    E_CUST_FRAME_RATE_48,
    E_CUST_FRAME_RATE_50,
    E_CUST_FRAME_RATE_59_94,
    E_CUST_FRAME_RATE_60,
    E_CUST_FRAME_RATE_96,
    E_CUST_FRAME_RATE_100,
    E_CUST_FRAME_RATE_120,
    E_CUST_FRAME_RATE_UNKNWON,
} E_CUST_FRAME_RATE;

typedef enum {
    E_CUST_VSYNC_MODE_NOT_DIRECT,
    E_CUST_VSYNC_MODE_DIRECT,
} E_CUST_VSYNC_MODE;

typedef enum {
    E_CUST_STREAMING_TYPE_NORMAL,
    E_CUST_STREAMING_TYPE_ADAPTIVE, //Adaptive Streaming
} E_CUST_STREAMING_TYPE;

typedef enum {
    E_CUST_DIMENSION_2D,
    E_CUST_DIMENSION_3D,
    E_CUST_DIMENSION_UNKNOWN,
} E_CUST_DIMENSION;

typedef enum {
    E_CUST_3D_FMT_FS, // Frame Sequential
    E_CUST_3D_FMT_FP, // Frame Packing
    E_CUST_3D_FMT_TB, // Top and Bottom
    E_CUST_3D_FMT_SS, // Side by Side
    E_CUST_3D_FMT_LI, // Line interleave
    E_CUST_3D_FMT_UNKNOWN,
} E_CUST_3D_FMT;

typedef enum {
    E_CUST_COLMT_601,
    E_CUST_COLMT_709,
    E_CUST_COLMT_2020,
    E_CUST_COLMT_UNKOWN,
} E_CUST_COLMT;

typedef enum {
    E_CUST_COL_FMT_RGB,
    E_CUST_COL_FMT_YUV_444,
    E_CUST_COL_FMT_YUV_422,
    E_CUST_COL_FMT_YUV_420,
    E_CUST_COL_FMT_UNKOWN,
} E_CUST_COL_FMT;

typedef enum {
    E_CUST_COL_DEPTH_8,
    E_CUST_COL_DEPTH_10,
    E_CUST_COL_DEPTH_12,
    E_CUST_COL_DEPTH_UNKNOWN,
} E_CUST_COL_DEPTH;

typedef enum {
    E_CUST_COL_SPACE_NORMAL,
    E_CUST_COL_SPACE_XV_COLOR,
    E_CUST_COL_SPACE_BT202C,
    E_CUST_COL_SPACE_BT2020,
    E_CUST_COL_SPACE_UNKNOWN,
} E_CUST_COL_SPACE;

typedef enum {
    E_CUST_PHOTO_COL_SPACE_SRGB,
    E_CUST_PHOTO_COL_SPACE_SYCC,
    E_CUST_PHOTO_COL_SPACE_ADOBE_RGB,
    E_CUST_PHOTO_COL_SPACE_UNKNOWN,
} E_CUST_PHOTO_COL_SPACE;

typedef enum {
    E_CUST_DRANGE_FULL,
    E_CUST_DRANGE_LIMITED,
    E_CUST_DRANGE_UNKNOWN,
} E_CUST_DRANGE;

typedef enum {
    E_CUST_ASPECT_4_3,
    E_CUST_ASPECT_16_9,
    E_CUST_ASPECT_14_9,
    E_CUST_ASPECT_UNKNOWN,
} E_CUST_ASPECT;


typedef enum {
    E_CUST_VDO_CODEC_MPEG1,
    E_CUST_VDO_CODEC_MPEG2,
    E_CUST_VDO_CODEC_MPEG4_VISUAL,
    E_CUST_VDO_CODEC_AVC,
    E_CUST_VDO_CODEC_HEVC,
    E_CUST_VDO_CODEC_AVS,
    E_CUST_VDO_CODEC_XVID,
    E_CUST_VDO_CODEC_VC1,
    E_CUST_VDO_CODEC_REAL,
    E_CUST_VDO_CODEC_VP8,
    E_CUST_VDO_CODEC_MOTION_JPEG,
    E_CUST_VDO_CODEC_JPEG,  // for photo
    E_CUST_VDO_CODEC_MPO,   // for photo
    E_CUST_VDO_CODEC_RAW,   // for photo
    E_CUST_VDO_CODEC_UNKNOWN,
} E_CUST_VDO_CODEC;

typedef enum {
E_CUST_CHROMA_SAMPLE_TYPE_0,
E_CUST_CHROMA_SAMPLE_TYPE_1,
E_CUST_CHROMA_SAMPLE_TYPE_2,
E_CUST_CHROMA_SAMPLE_TYPE_3,
E_CUST_CHROMA_SAMPLE_TYPE_4,
E_CUST_CHROMA_SAMPLE_TYPE_5,
E_CUST_CHROMA_SAMPLE_TYPE_INVALID,// Chroma sample type is invalid or color format is not YUV420
} E_CUST_CHROMA;


typedef enum {
    E_CUST_HDMI_4K_PHOTO_OFF,
    E_CUST_HDMI_4K_PHOTO_ON,
} E_CUST_HDMI_4K_PHOTO;

typedef enum {
E_CUST_HDR_TYPE_SDR,
E_CUST_HDR_TYPE_HDR1,
E_CUST_HDR_TYPE_HDR2,
E_CUST_HDR_TYPE_HDR3,
E_CUST_HDR_TYPE_UNKNOWN,
} E_CUST_HDR_TYPE;
typedef enum{
	E_CUST_DOVI_TYPE_NONE,
	E_CUST_DOVI_TYPE_HDMILL,
	E_CUST_DOVI_TYPE_HDMISTD,
	E_CUST_DOVI_TYPE_DECODER,
	E_CUST_DOVI_TYPE_UNKNOWN,
}E_CUST_DOVI_TYPE;


typedef enum{
	E_CUST_COLOR_SENSOR_OFF,
	E_CUST_COLOR_SENSOR_ON,
}E_CUST_COLOR_SENSOR;

typedef enum{
	E_CUST_COLOR_SENSOR_LIGHT_OFF,
	E_CUST_COLOR_SENSOR_LIGHT_ON,
}E_CUST_COLOR_SENSOR_LIGHT;

typedef enum{
	E_CUST_COLOR_SENSOR_COLOR_OFF,
	E_CUST_COLOR_SENSOR_COLOR_ON,
}E_CUST_COLOR_SENSOR_COLOR;

typedef enum{
	E_CUST_COLOR_SENSOR_GAMMA_OFF,
	E_CUST_COLOR_SENSOR_GAMMA_ON,
}E_CUST_COLOR_SENSOR_GAMMA;

typedef struct{
	E_CUST_COLOR_SENSOR eColorSensor;
	E_CUST_COLOR_SENSOR_LIGHT eColorSensorLight;
	E_CUST_COLOR_SENSOR_COLOR eColorSensorColor;
	E_CUST_COLOR_SENSOR_GAMMA eColorSensorGamma;
}CUST_COLOR_SENSOR_SETTING_T;

typedef struct {
    E_CUST_SIG_STA  eSigSta;
    UINT32  u4Width;
    UINT32  u4Height;
    UINT32  u4CodecWidth;
    UINT32  u4CodecHeight;
    E_CUST_CONTENT_TYPE eContentType;
    E_CUST_SCAN_TYPE    eScanType;
    E_CUST_COLOR_SYSTEM eColorSystem;
    E_CUST_FRAME_RATE   eFrameRate;
    E_CUST_STREAMING_TYPE   eStreamingType;
    E_CUST_DIMENSION    eDimension;
    E_CUST_3D_FMT   e3dFormat;
    E_CUST_COLMT    eColorimetry;
    E_CUST_COL_FMT  eColorFormat;
    E_CUST_COL_DEPTH    eColorDepth;
    E_CUST_COL_SPACE    eColorSpace;        //for video
    E_CUST_PHOTO_COL_SPACE  ePhotoColorSpace;   //for photo
    E_CUST_DRANGE   eDynamicRange;
    E_CUST_ASPECT   eAspect;
    E_CUST_VDO_CODEC    eVideoCodec;
	E_CUST_CHROMA	eCromaSample;
    E_CUST_HDMI_4K_PHOTO    eHdmi4kPhoto;
	E_CUST_HDR_TYPE	eHDRType;
	E_CUST_DOVI_TYPE eDoviType;
	UINT32 u4SrcFrameRate;
    UINT32 u4IsEPGMode;
} CUST_SIG_FMT_T;

typedef enum {
    E_CUST_WINDOW_MODE_SINGLE,
    E_CUST_WINDOW_MODE_PIP,
    E_CUST_WINDOW_MODE_PAP,
} E_CUST_WINDOW_MODE;

typedef enum {
    E_CUST_MUTE_STATE_UNMUTE,
    E_CUST_MUTE_STATE_MUTE,
} E_CUST_MUTE_STATE;

typedef struct {
    E_CUST_MUTE_STATE	eMainMuteState;
    E_CUST_MUTE_STATE	eSubMuteState;
    E_CUST_MUTE_STATE	eVideoOutputMuteState;
    E_CUST_MUTE_STATE	eGfxOutputMuteState;
} CUST_MUTE_STATE_T;

typedef struct {
    UINT32	u4X;
    UINT32	u4Y;
    UINT32	u4Width;
    UINT32	u4Height;
} CUST_OUT_POSI_T;

typedef struct {
    UINT32	u4SrcX;
    UINT32	u4SrcY;
    UINT32	u4SrcWidth;
    UINT32	u4SrcHeight;
} CUST_SRC_AREA_T;

typedef struct {
    UINT32	u4DestX;
    UINT32	u4DestY;
    UINT32	u4DestWidth;
    UINT32	u4DestHeight;
} CUST_DEST_AREA_T;

typedef enum {
    E_CUST_PATH_PQ,
    E_CUST_PATH_BYPASS,
} E_CUST_PATH;

typedef struct {
    UINT32	u4Htotal;
    UINT32	u4Vtotal;
    UINT32	u4ActX;
    UINT32	u4ActY;
    UINT32	u4ActWidth;
    UINT32	u4ActHeight;
    E_CUST_FRAME_RATE	eFrameRate;
    E_CUST_DIMENSION	eDimension;
    E_CUST_3D_FMT	    e3dFormat;
    E_CUST_COLMT	    eColorimetry;
    E_CUST_COL_FMT	    eColorFormat;
    E_CUST_COL_DEPTH	eColorDepth;
    E_CUST_COL_SPACE	eColorSpace;
    E_CUST_DRANGE	    eDynamicRange;
    E_CUST_PATH         ePath;
	UINT8 	bCtrlBitValid;
} CUST_OUT_FMT_T;

typedef struct {
    UINT32	u4Htotal;
    UINT32	u4Vtotal;
    UINT32	u4ActX;
    UINT32	u4ActY;
    UINT32	u4ActWidth;
    UINT32	u4ActHeight;
    E_CUST_FRAME_RATE	eFrameRate;
} CUST_OUT_GFX_FMT_T;

typedef struct {
    UINT8	u1NoiseLevel;
    UINT8	u1BlockNoiseLevel;
    UINT32	u4FrmDiff;
    UINT8	u1FreqHpfH;    // TBD
    UINT8	u1FreqHpfV;    // TBD
    UINT8	u1FreqBpfH;    // TBD
    UINT16	u2BlockSizeH;
	UINT16	u2BlockSizeV;
} CUST_MCVP_METER_T;

typedef enum {
    E_CUST_CADENSE_32,
    E_CUST_CADENSE_22,
    E_CUST_CADENSE_2224,
    E_CUST_CADENSE_2332,
    E_CUST_CADENSE_32322,
    E_CUST_CADENSE_44,
    E_CUST_CADENSE_55,
    E_CUST_CADENSE_64,
    E_CUST_CADENSE_66,
    E_CUST_CADENSE_87,
    E_CUST_CADENSE_UNKOWN,
} E_CUST_CADENSE;


typedef struct {
    UINT32	u4X;
    UINT32	u4Y;
    UINT32	u4Width;
    UINT32	u4Height;
    UINT16	u2HueHistL[HUE_HIST_NUM];
    UINT16	u2SatHistL[SAT_HIST_NUM];
    UINT32	u4LumaHistL[113];
    UINT8	u1ASLL;
    UINT8	u1APLL;
    UINT16	u2HueHistR[HUE_HIST_NUM];
    UINT16	u2SatHistR[SAT_HIST_NUM];
    UINT32	u4LumaHistR[113];
    UINT8	u1ASLR;
    UINT8	u1APLR;
    UINT32	u4LumaSumL;
    UINT8	u1LumaMaxL;
    UINT8	u1LumaMinL;
    UINT32	u4LumaSumR;
    UINT8	u1LumaMaxR;
    UINT8	u1LumaMinR;
} CUST_POST_PROC_METER_T;

typedef struct {
    UINT8	u1LdMaxRGB[32];
    UINT16	u2LdMaxHist[33];
    UINT16	u2LdDimInfo[32];
} CUST_LD_METER_T;

typedef struct {
    UINT32	u4MCVP;
    UINT32	u4Scaler;
    UINT32	u4MJC;
} CUST_VDO_DELAY_INFO_T;

typedef struct {
    UINT32	u4TotalOpTime;
    UINT32	u4PanelOpTime;
} CUST_HISTORY_DATA_T;

typedef enum {
    E_CUST_PANEL_STATE_OFF,
    E_CUST_PANEL_STATE_ON,
} E_CUST_PANEL_STATE;

typedef struct {
    E_CUST_PANEL_STATE	ePanelState;
} CUST_POWER_STATE_T;

/*4 band frequency detection*/
typedef struct {
   UINT16 h_filter_h_start ;
   UINT16 h_filter_h_size ;
   UINT16 h_filter_v_start ;
   UINT16 h_filter_v_size ;
} CUST_4BAND_FILTER_RANGE;

typedef struct {
   UINT8  bitshift_a0;
   UINT8  bitshift_b0 ;
   UINT8  h_filter_switch_0;
   UINT8  h_filter_band0_div;
   INT16  h_filter_band0_coef[21];
   UINT8  bitshift_a1 ;
   UINT8  bitshift_b1 ;
   UINT8  h_filter_switch_1;
   UINT8  h_filter_band1_div;
   INT8   h_filter_band1_coef[15];
   UINT8  bitshift_a2 ;
   UINT8  bitshift_b2 ;
   UINT8  h_filter_switch_2;
   UINT8  h_filter_band2_div;
   INT8   h_filter_band2_coef[15];
   UINT8  bitshift_a3 ;
   UINT8  bitshift_b3 ;
   UINT8  h_filter_switch_3;
   UINT8  h_filter_band3_div;
   INT8   h_filter_band3_coef[15];
} CUST_4BAND_H_FILTER;

typedef struct {
   UINT32 a0_hist[16] ;
   UINT32 b0_hist[16] ;
   UINT32 a1_hist[16] ;
   UINT32 b1_hist[16] ;
   UINT32 a2_hist[16] ;
   UINT32 b2_hist[16] ;
   UINT32 a3_hist[16] ;
   UINT32 b3_hist[16] ;
} CUST_4BAND_HIST_T;

// drv_MapleGetApiAp.c
typedef enum {
    E_CUST_PIC_MODE_VIVID,
    E_CUST_PIC_MODE_STANDARD,
    E_CUST_PIC_MODE_CINEMA,
	E_CUST_PIC_MODE_GAME,
	E_CUST_PIC_MODE_GRAPHICS,
    E_CUST_PIC_MODE_PHOTO,
	E_CUST_PIC_MODE_CUSTOM,
	E_CUST_PIC_MODE_USER1,
    E_CUST_PIC_MODE_USER2,
	E_CUST_PIC_MODE_DV_DARK,
	E_CUST_PIC_MODE_DV_BRIGHT,
	E_CUST_PIC_MODE_DV_VIVID,
	E_CUST_PIC_MODE_NETFLIX_DV,
	E_CUST_PIC_MODE_NETFLIX_SDR,
	E_CUST_PIC_MODE_RESERVED1,
	E_CUST_PIC_MODE_RESERVED2
} E_CUST_PIC_MODE;



typedef enum {
	E_CUST_COL_TMP_COOL,
	E_CUST_COL_TMP_NATURAL,
	E_CUST_COL_TMP_WARM1,
	E_CUST_COL_TMP_WARM2,  // Not Used
	E_CUST_COL_TMP_EXPERT1,
	E_CUST_COL_TMP_EXPERT2
} E_CUST_COL_TMP;


typedef enum {
    E_CUST_NOISE_REDUCTION_OFF,
    E_CUST_NOISE_REDUCTION_LOW,
    E_CUST_NOISE_REDUCTION_MID,
    E_CUST_NOISE_REDUCTION_HIGH,
    E_CUST_NOISE_REDUCTION_AUTO,
E_CUST_NOISE_REDUCTION_INTELLIGENT,
} E_CUST_NOISE_REDUCTION;

typedef enum {
    E_CUST_MPEG_NOISE_REDUCTION_OFF,
E_CUST_MPEG_NOISE_REDUCTION_LOW,
E_CUST_MPEG_NOISE_REDUCTION_MID,
    E_CUST_MPEG_NOISE_REDUCTION_HIGH,
E_CUST_MPEG_NOISE_REDUCTION_AUTO,
} E_CUST_MPEG_NOISE_REDUCTION;

typedef enum {
E_CUST_REALITY_CREATION_OFF,
E_CUST_REALITY_CREATION_MANUAL,
E_CUST_REALITY_CREATION_AUTO,
} E_CUST_REALITY_CREATION;

typedef enum {
E_CUST_MASTERED_IN_4K_OFF,
E_CUST_MASTERED_IN_4K_ON,
} E_CUST_MASTERED_IN_4K;

typedef enum {
    E_CUST_MOTION_FLOW_OFF,
    E_CUST_MOTION_FLOW_STANDARD,
	E_CUST_MOTION_FLOW_SMOOTH,
	E_CUST_MOTION_FLOW_CLEAR,
	E_CUST_MOTION_FLOW_IMPLUSE,
	E_CUST_MOTION_FLOW_TRUE_CINEMA,
	E_CUST_MOTION_FLOW_CUSTOM,
	E_CUST_MOTION_FLOW_AUTO,
} E_CUST_MOTION_FLOW;

typedef enum {
    E_CUST_FILM_MODE_OFF,
	E_CUST_FILM_MODE_LOW,
	E_CUST_FILM_MODE_MID,
	E_CUST_FILM_MODE_HIGH,
	E_CUST_FILM_MODE_AUTO,
} E_CUST_FILM_MODE;

typedef enum {
    E_CUST_ADV_CONTRAST_ENHANCER_OFF,
    E_CUST_ADV_CONTRAST_ENHANCER_LOW,
    E_CUST_ADV_CONTRAST_ENHANCER_MID,
    E_CUST_ADV_CONTRAST_ENHANCER_HIGH,
} E_CUST_ADV_CONTRAST_ENHANCER;

typedef enum {
    E_CUST_BLACK_CORRECTOR_OFF,
    E_CUST_BLACK_CORRECTOR_LOW,
    E_CUST_BLACK_CORRECTOR_MID,
    E_CUST_BLACK_CORRECTOR_HIGH,
} E_CUST_BLACK_CORRECTOR;

typedef enum {
E_CUST_LED_DYNAMIC_CTRL_OFF,
E_CUST_LED_DYNAMIC_CTRL_LOW,
E_CUST_LED_DYNAMIC_CTRL_MID,
E_CUST_LED_DYNAMIC_CTRL_HIGH,
} E_CUST_LED_DYNAMIC_CTRL;


typedef enum {
    E_CUST_LIVE_COLOR_OFF,
    E_CUST_LIVE_COLOR_LOW,
    E_CUST_LIVE_COLOR_MID,
    E_CUST_LIVE_COLOR_HIGH,
} E_CUST_LIVE_COLOR;

typedef enum {
E_CUST_SPEED_PREFERENCE_OFF,
E_CUST_SPEED_PREFERENCE_ON,
} E_CUST_SPEED_PREFERENCE;

typedef enum {
E_CUST_LED_MOTION_MODE_OFF,
E_CUST_LED_MOTION_MODE_ON,
} E_CUST_LED_MOTION_MODE;

typedef enum {
E_CUST_XDR_OFF,
E_CUST_XDR_LOW,
E_CUST_XDR_MID,
E_CUST_XDR_HIGH,
} E_CUST_XDR;

typedef enum {
E_CUST_DRANGE_UI_AUTO,
E_CUST_DRANGE_UI_FULL,
E_CUST_DRANGE_UI_LIMITED,
} E_CUST_DRANGE_UI;

typedef enum {
E_CUST_SMOOTH_GRADATION_OFF,
E_CUST_SMOOTH_GRADATION_LOW,
E_CUST_SMOOTH_GRADATION_MID,
E_CUST_SMOOTH_GRADATION_HIGH,
E_CUST_SMOOTH_GRADATION_AUTO,
} E_CUST_SMOOTH_GRADATION_UI;

typedef enum {
	E_CUST_TRANSFER_FUNC_UI_AUTO,
	E_CUST_TRANSFER_FUNC_UI_OFF,
	E_CUST_TRANSFER_FUNC_UI_HDR10,
	E_CUST_TRANSFER_FUNC_UI_HLG,
	E_CUST_TRANSFER_FUNC_UI_HDR4,
} E_CUST_TRANSFER_FUNC_UI;
typedef enum{
	E_CUST_WB_OFFSET_20P_DIS,
	E_CUST_WB_OFFSET_20P_EN,
}E_CUST_WB_OFFSET_20P;

typedef enum {
E_CUST_COL_SPACE_UI_AUTO,
E_CUST_COL_SPACE_UI_SRGB_709,
E_CUST_COL_SPACE_UI_DCI,
E_CUST_COL_SPACE_UI_ADOBE_RGB,
E_CUST_COL_SPACE_UI_2020,
} E_CUST_COL_SPACE_UI;

typedef enum{
E_CUST_TONEMAP_OFF,
E_CUST_TONEMAP_LOW,
E_CUST_TONEMAP_MID,
E_CUST_TONEMAP_HIGH,
} E_CUST_TONEMAP_UI;

typedef struct {
    E_CUST_PIC_MODE	ePicMode;
    UINT8	u1Backlight;
    UINT8	u1Contrast;
    UINT8	u1Brightness;
    UINT8	u1Color;
    UINT8	u1Hue;
    UINT8	u1Sharpness;
    E_CUST_COL_TMP	eColTmp;
    E_CUST_NOISE_REDUCTION	eNoiseReduction;
    E_CUST_MPEG_NOISE_REDUCTION	eMpegNoiseReduction;
	E_CUST_REALITY_CREATION	eRealityCreation;
	E_CUST_MASTERED_IN_4K	eMasteredIn4K;
	INT16	i2Resolution;
	INT16	i2NoiseFiltering;
    E_CUST_MOTION_FLOW	eMotionFlow;
	INT16	i2MotionFlowSmoothness;
	INT16	i2MotionFlowClearness;
    E_CUST_FILM_MODE	eFilmMode;
    E_CUST_ADV_CONTRAST_ENHANCER    eAdvContrastEnhancer;
    E_CUST_BLACK_CORRECTOR	eBlackCorrector;
    INT16	i2Gamma;
	E_CUST_LED_DYNAMIC_CTRL	eLedDynamicCtrl;
    E_CUST_LIVE_COLOR	eLiveColor;
    INT16	i2RGain;
    INT16	i2GGain;
    INT16	i2BGain;
    INT16	i2RBias;
    INT16	i2GBias;
    INT16	i2BBias;
	E_CUST_SPEED_PREFERENCE	eSpeedPreference;
	E_CUST_LED_MOTION_MODE	eLedMotionMode;
	E_CUST_XDR				eXdr;
	E_CUST_DRANGE_UI		eDrange;
	E_CUST_COL_SPACE_UI		eColorSpace;
	E_CUST_SMOOTH_GRADATION_UI eSmoothGradation;
	E_CUST_TRANSFER_FUNC_UI eTransFunc;
	//E_CUST_WB_OFFSET_20P eWbOfst20pEn;
#ifdef CC_DRV_CUST_TONEMAP_TO_KO
    E_CUST_TONEMAP_UI eToneMapInfo;
#endif
} CUST_PIC_SETTING_T;


typedef enum {
    E_CUST_WIDE_MODE_WIDE_ZOOM,
    E_CUST_WIDE_MODE_NORMAL,
    E_CUST_WIDE_MODE_FULL,
    E_CUST_WIDE_MODE_ZOOM,
    E_CUST_WIDE_MODE_14_9,
    E_CUST_WIDE_MODE_UNKNOWN,
} E_CUST_WIDE_MODE;

typedef enum {
    E_CUST_DISPLAY_AREA_FULL, //+1
    E_CUST_DISPLAY_AREA_NORMAL,
    E_CUST_DISPLAY_AREA_MINUS_1,
} E_CUST_DISPLAY_AREA;

typedef enum {
    E_CUST_ZOOM_1X,
    E_CUST_ZOOM_2X,
    E_CUST_ZOOM_4X,
    E_CUST_ZOOM_FULL,
    E_CUST_ZOOM_UNKNOWN,
} E_CUST_ZOOM;

typedef enum {
    E_CUST_PC_WIDE_MODE_NORMAL,
    E_CUST_PC_WIDE_MODE_FULL1,
    E_CUST_PC_WIDE_MODE_FULL2,
    E_CUST_PC_WIDE_MODE_UNKNOWN,
} E_CUST_PC_WIDE_MODE;

typedef struct {
    E_CUST_WIDE_MODE	eWideMode;
    E_CUST_DISPLAY_AREA	eDisplayArea;
    INT16	i2HCenter;
    INT16	i2VCenter;
    INT16	i2VerticalSize;
    E_CUST_ZOOM eZoom; // For USB and IPTV contents
    E_CUST_PC_WIDE_MODE	ePcWIdeMode; // For PC
    INT16	i2PcPhase; // For PC
    INT16	i2PcPitch; // For PC
    INT16	i2PcHCenter; // For PC
    INT16	i2PcVCenter; // For PC
} CUST_SCREEN_SETTING_T;

typedef enum {
    E_CUST_3D_DISP_OFF,
    E_CUST_3D_DISP_SBS,
    E_CUST_3D_DISP_TAB,
    E_CUST_3D_DISP_ON,
} E_CUST_3D_DISPLAY;

typedef enum {
E_CUST_3D_GLASS_BRIGHTNESS_LOW,
E_CUST_3D_GLASS_BRIGHTNESS_MID,
E_CUST_3D_GLASS_BRIGHTNESS_HIGH,
} E_CUST_3D_GLASS_BRIGHTNESS;

typedef struct {
    E_CUST_3D_DISPLAY	e3dDisplay;
    INT16	i23dDepthAdjustment;
    E_CUST_3D_GLASS_BRIGHTNESS	e3dGlassBrightness;
} CUST_3D_SETTING_T;

typedef enum {
    E_CUST_POWER_SAVING_OFF,
    E_CUST_POWER_SAVING_LOW,
    E_CUST_POWER_SAVING_MID,
    E_CUST_POWER_SAVING_HIGH,
    E_CUST_POWER_SAVING_PICTURE_OFF,
} E_CUST_POWER_SAVING;

typedef enum {
    E_CUST_LIGHT_SENSOR_OFF,
    E_CUST_LIGHT_SENSOR_ON,
} E_CUST_LIGHT_SENSOR;

typedef struct {
    E_CUST_POWER_SAVING	ePowerSaving;
    E_CUST_LIGHT_SENSOR	eLightSensor;
} CUST_ECO_SETTING_T;

typedef struct {
    UINT32 u4TBD;
} CUST_PHOTO_SETTING_T;

typedef enum {
	E_CUST_DEMO_MODE_OFF,
	E_CUST_DEMO_MODE_EFFECT_OFF,
	E_CUST_DEMO_MODE_X_REALITY_PRO,
	E_CUST_DEMO_MODE_X_REALITY_PRO2,
	E_CUST_DEMO_MODE_MOTION_FLOW,
	E_CUST_DEMO_MODE_MOTION_FLOW2,
	E_CUST_DEMO_MODE_THRILUMINOUS,
	E_CUST_DEMO_MODE_XDR,
	E_CUST_DEMO_MODE_XDR_BL_IMAGE,
	E_CUST_DEMO_MODE_NO_09,
	E_CUST_DEMO_MODE_NO_10,
	E_CUST_DEMO_MODE_NO_11,
	E_CUST_DEMO_MODE_NO_12,
	E_CUST_DEMO_MODE_NO_13,
	E_CUST_DEMO_MODE_NO_14,
	E_CUST_DEMO_MODE_NO_15,
	E_CUST_DEMO_MODE_NO_16,
	E_CUST_DEMO_MODE_NO_17,
	E_CUST_DEMO_MODE_NO_18,
	E_CUST_DEMO_MODE_NO_19,
	E_CUST_DEMO_MODE_NO_20,
	E_CUST_DEMO_MODE_NO_21,
	E_CUST_DEMO_MODE_NO_22,
	E_CUST_DEMO_MODE_NO_23,
	E_CUST_DEMO_MODE_NO_24,
	E_CUST_DEMO_MODE_NO_25,
	E_CUST_DEMO_MODE_NO_26,
	E_CUST_DEMO_MODE_NO_27,
	E_CUST_DEMO_MODE_NO_28,
	E_CUST_DEMO_MODE_NO_29,
	E_CUST_DEMO_MODE_NO_30,
	E_CUST_DEMO_MODE_NO_31,

} E_CUST_DEMO_MODE;//add for spec IDM_0130


typedef enum {
	E_CUST_DEMO_PATTERN_NONE,
	E_CUST_DEMO_PATTERN_SPLIT1_L_ON,
	E_CUST_DEMO_PATTERN_SPLIT1_R_ON,
	E_CUST_DEMO_PATTERN_SPLIT2,
	E_CUST_DEMO_PATTERN_ACT_AREA1,
	E_CUST_DEMO_PATTERN_ACT_AREA2,
	E_CUST_DEMO_PATTERN_ACT_AREA3,
	E_CUST_DEMO_PATTERN_FULL_ON,
	E_CUST_DEMO_PATTERN_FULL_OFF,
	E_CUST_DEMO_PATTERN_NO_09,
	E_CUST_DEMO_PATTERN_NO_10,
	E_CUST_DEMO_PATTERN_NO_11,
	E_CUST_DEMO_PATTERN_NO_12,
	E_CUST_DEMO_PATTERN_NO_13,
	E_CUST_DEMO_PATTERN_NO_14,
	E_CUST_DEMO_PATTERN_NO_15,
	E_CUST_DEMO_PATTERN_NO_16,
	E_CUST_DEMO_PATTERN_NO_17,
	E_CUST_DEMO_PATTERN_NO_18,
	E_CUST_DEMO_PATTERN_NO_19,
	E_CUST_DEMO_PATTERN_NO_20,
	E_CUST_DEMO_PATTERN_NO_21,
	E_CUST_DEMO_PATTERN_NO_22,
	E_CUST_DEMO_PATTERN_NO_23,
	E_CUST_DEMO_PATTERN_NO_24,
	E_CUST_DEMO_PATTERN_NO_25,
	E_CUST_DEMO_PATTERN_NO_26,
	E_CUST_DEMO_PATTERN_NO_27,
	E_CUST_DEMO_PATTERN_NO_28,
	E_CUST_DEMO_PATTERN_NO_29,
	E_CUST_DEMO_PATTERN_NO_30,
	E_CUST_DEMO_PATTERN_NO_31,

} E_CUST_DEMO_PATTERN;//add for spec IDM_0130

typedef struct {
    E_CUST_DEMO_MODE eDemoMode;
	E_CUST_DEMO_PATTERN eDemoPattern;
	UINT32	u4DemoOption;
} CUST_DEMO_MODE_T;

typedef struct {
    UINT16 u2X;
    UINT16 u2Y;
    UINT16 u2Width;
    UINT16 u2Height;
} CUST_RECT_AREA_T;

typedef struct {
	UINT8	u1Num;
	CUST_RECT_AREA_T	rectArea[8];
	UINT8	b4KCoordinate;
} CUST_DEMO_OSD_MASK_T;

typedef struct {
    UINT16	u2RGain;
    UINT16	u2GGain;
    UINT16	u2BGain;
    UINT16	u2ROffset;
    UINT16	u2GOffset;
    UINT16	u2BOffset;
} CUST_SERVICE_SETTING_T;

typedef enum {
	E_CUST_SERVICE_TRANS_SOC_TO_TCON,
	E_CUST_SERVICE_TRANS_TCON_TO_SOC,
	E_CUST_SERVICE_TRANS_INITIALIZE,
	E_CUST_SERVICE_TRANS_NONE,
} E_CUST_SERVICE_TRANS;

typedef struct {
    E_CUST_SERVICE_TRANS	eWBTransferMode;
    E_CUST_SERVICE_TRANS	eMuraTransferMode;
    E_CUST_SERVICE_TRANS	eCUCTransferMode;
} CUST_SERVICE_TRANS_T;

typedef struct {
    UINT8	bMode;
    UINT16	u2Data;
} CUST_SERVICE_VCOM_T;

typedef struct {
    UINT16	i2ROffset1;
    UINT16	i2ROffset2;
	UINT16	i2ROffset3;
	UINT16	i2ROffset4;
	UINT16	i2ROffset5;
	UINT16	i2ROffset6;
	UINT16	i2ROffset7;
	UINT16	i2ROffset8;
	UINT16	i2ROffset9;
	UINT16	i2ROffset10;

	UINT16	i2ROffset11;
	UINT16	i2ROffset12;
	UINT16	i2ROffset13;
	UINT16	i2ROffset14;
	UINT16	i2ROffset15;
	UINT16	i2ROffset16;
	UINT16	i2ROffset17;
	UINT16	i2ROffset18;
	UINT16	i2ROffset19;
	UINT16	i2ROffset20;

	UINT16	i2GOffset1;
    UINT16	i2GOffset2;
	UINT16	i2GOffset3;
	UINT16	i2GOffset4;
	UINT16	i2GOffset5;
	UINT16	i2GOffset6;
	UINT16	i2GOffset7;
	UINT16	i2GOffset8;
	UINT16	i2GOffset9;
	UINT16	i2GOffset10;

	UINT16	i2GOffset11;
	UINT16	i2GOffset12;
	UINT16	i2GOffset13;
	UINT16	i2GOffset14;
	UINT16	i2GOffset15;
	UINT16	i2GOffset16;
	UINT16	i2GOffset17;
	UINT16	i2GOffset18;
	UINT16	i2GOffset19;
	UINT16	i2GOffset20;

	UINT16	i2BOffset1;
    UINT16	i2BOffset2;
	UINT16	i2BOffset3;
	UINT16	i2BOffset4;
	UINT16	i2BOffset5;
	UINT16	i2BOffset6;
	UINT16	i2BOffset7;
	UINT16	i2BOffset8;
	UINT16	i2BOffset9;
	UINT16	i2BOffset10;

	UINT16	i2BOffset11;
	UINT16	i2BOffset12;
	UINT16	i2BOffset13;
	UINT16	i2BOffset14;
	UINT16	i2BOffset15;
	UINT16	i2BOffset16;
	UINT16	i2BOffset17;
	UINT16	i2BOffset18;
	UINT16	i2BOffset19;
	UINT16	i2BOffset20;

} CUST_WB_OFFSET_T;

typedef struct {
    INT16	i2RHue;
    INT16	i2RSaturation;
	INT16	i2RBrightness;

	INT16	i2YeHue;
    INT16	i2YeSaturation;
	INT16	i2YeBrightness;

	INT16	i2GHue;
    INT16	i2GSaturation;
	INT16	i2GBrightness;

	INT16	i2CyHue;
    INT16	i2CySaturation;
	INT16	i2CyBrightness;

	INT16	i2BHue;
    INT16	i2BSaturation;
	INT16	i2BBrightness;

	INT16	i2MgHue;
    INT16	i2MgSaturation;
	INT16	i2MgBrightness;


} CUST_COLOR_ADJUST_T;

typedef struct {
	E_CUST_WB_OFFSET_20P eWbOfst20pEn;
} CUST_WB_OFFSET_20P_EN_T;


typedef enum{
    E_CUST_DELAY_MODE_OFF,
    E_CUST_DELAY_MODE_ON,
    E_CUST_DELAY_MODE_AUTO,
}E_CUST_DELAY_MODE;
typedef struct {
    E_CUST_DELAY_MODE   eDelayMode;
} CUST_BT_SETTING_T;


typedef struct {
    UINT16	u2RGain;
    UINT16	u2GGain;
    UINT16	u2BGain;
    UINT16	u2ROffset;
    UINT16	u2GOffset;
    UINT16	u2BOffset;
} CUST_RGB_GAIN_OFST_T;

typedef struct
{
	UINT8  dv_en;
    UINT8  dv_gd_en;
    UINT32 gd_bl_duty;
}CUST_GLOBAL_DIMMING_INFO_T ;


typedef enum {
    E_CUST_RESET_REBOOT,
    E_CUST_RESET_SHUTDOWN,
} E_CUST_RESET;
#if 0
typedef enum {
E_CUST_BE_SCL_TYPE_PQ,
E_CUST_BE_SCL_TYPE_VIDEO,
E_CUST_BE_SCL_TYPE_GFX,
E_CUST_BE_SCL_TYPE_INVALID,
} E_CUST_BE_SCL_TYPE;
#endif
typedef enum {
E_CUST_RESULT_OK,
E_CUST_RESULT_ERROR_NOT_READY,
E_CUST_RESULT_ERROR_INSUFFICIENT_RESOURCES,
E_CUST_RESULT_ERROR_RESOURCES_PREEMPTED,
E_CUST_RESULT_ERROR_BAD_PARAMETER,
E_CUST_RESULT_MAX,
} E_CUST_RESULT;

typedef enum {
E_CUST_UI_INPUT_TARGET_DEFAULT,
E_CUST_UI_INPUT_TARGET_PHOTO_APP,
E_CUST_UI_INPUT_TARGET_VIDEO_APP,
E_CUST_UI_INPUT_TARGET_GEN_APP,
E_CUST_UI_INPUT_TARGET_WIFI_DISPLAY,
E_CUST_UI_INPUT_TARGET_VIDEO_CHAT,
E_CUST_UI_INPUT_TARGET_GAME_APP,
E_CUST_UI_INPUT_TARGET_UNKNOWN,
E_CUST_UI_INPUT_TARGET_OTT1 // netflix
}E_CUST_UI_INPUT_TARGET;

typedef enum {
E_CUST_PIC_CONTROL_DISABLE,
E_CUST_PIC_CONTROL_ENABLE,
}E_CUST_PIC_CONTROL_STATE;


typedef struct {
UINT8  u1PlaneOrder;
UINT32 u4SrcX;
UINT32 u4SrcY;
UINT32 u4SrcWidth;
UINT32 u4SrcHeight;
UINT32 u4DestX;
UINT32 u4DestY;
UINT32 u4DestWidth;
UINT32 u4DestHeight;
} CUST_BE_SCL_CONFIG_T;

typedef struct SELF_DIAG_FUNC_T
{
    void (*self_diag_MainPowerError_func)(void);
    void (*self_diag_DcAlertError_func)(void);
    void (*self_diag_AudError_func)(void);
    void (*self_diag_TuDemodError_func)(void);
    void (*self_diag_LdError_func)(void);
    void (*self_diag_VledError_func)(void);
    void (*self_diag_TconError_func)(void);
    void (*self_diag_PanelIdError_func)(void);
    void (*self_diag_BacklightError_func)(void);
    void (*self_diag_TemperatureError_func)(void);
    void (*self_diag_SWError_func)(void);
    void (*self_diag_4kBeOutError_func)(void);
#ifdef CC_SUPPORT_BOSTON_SWITCH
    void (*self_diag_HdmiEqError_func)(void);
#endif
    void (*self_diag_FRCTCI2CError_func)(void);
    void (*self_diag_AudI2CError_func)(void);
    void (*self_diag_4KPQError_func)(void);
    void (*self_diag_4KPQI2CError_func)(void);
	//new add
    void (*self_diag_TempI2CError_func)(void);
    void (*self_diag_TuDemodI2CError_func)(void);
    void (*self_diag_FrCTCError_func)(void);
    void (*self_diag_EArcError_func)(void);
    void (*self_diag_EArcI2CError_func)(void);
	void (*self_diag_8KBEComError_func)(void);
	void (*self_diag_FanError_func)(void);
	//reserve
    void (*self_diag_8KSHVError_func)(void);
    void (*self_diag_AmbI2CError_func)(void);
    void (*self_diag_RSV13Error_func)(void);
    void (*self_diag_RSV14Error_func)(void);
    void (*self_diag_RSV15Error_func)(void);
    void (*self_diag_RSV16Error_func)(void);
    void (*self_diag_ALLError_func)(void);
} SELF_DIAG_FUNC_T;


typedef struct _SONY_PQ_FUNC_T
{
    void (*pq_post_out_vsync_isr_func)(void);                        // DRVCUST_PostOutVSyncISR(void)
    void (*pq_post_de_end_isr_func)(void);                           // DRVCUST_PostDEEndISR(void)
    void (*pq_out_vsync_isr_func)(void);                             // DRVCUST_OutVSyncISR(void)
    void (*pq_vdo_in_isr_func)(void);                                // DRVCUST_VdoInISR(void)
    void (*pq_video_main_loop_func)(void);                           // DRVCUST_VideoMainloop(void)
    void (*pq_vdo_mode_chg_done_func)(void);                         // DRVCUST_VdoModeChgDone(UINT8 bPath)
    void (*pq_send_event_func)(E_CUST_EVENT, UINT8);                 // Trigger Event
    void (*pq_associate_path)(UINT8 , UINT8);                         // Notify logical path ID for Physical path ID
    void (*pq_dissociate_path)(UINT8);
    void (*pq_send_ecs_func)(UINT8* , UINT32);                       // ECS Command
} SONY_PQ_FUNC_T;


typedef struct _SONY_BE_FUNC_T
{
    void (*be_send_event_func)(E_CUST_EVENT, UINT8);
    void (*be_receive_from_pq)(UINT8* , UINT32);
    void (*be_send_ecs_func)(UINT8* , UINT32);
    void (*be_panel_power_func)(BOOL);
    E_CUST_RESULT (*be_allocate_resource)(UINT8 , E_CUST_BE_SCL_TYPE , CUST_BE_SCL_CONFIG_T* , UINT8*);
    E_CUST_RESULT (*be_free_resource)(UINT8);
    E_CUST_RESULT (*be_associate_path)(UINT8 , UINT8);
    E_CUST_RESULT (*be_dissociate_path)(UINT8 , UINT8);
} SONY_BE_FUNC_T;

typedef struct _SONY_BE_GEN_FUNC_T
{
   void (*be_set_general_data)(UINT8*, UINT8);
   void (*be_get_general_data)(UINT8*, UINT8*) ; //Maximum size is 255. Please allocate 255byte data area before calling get API.
   void (*be_set_color_pickup_data)(INT16, INT16);
   void (*be_get_color_pickup_data)(INT16*, INT16*, INT16*);
   void (*be_set_general_param)(UINT32 type, UINT8[255], UINT8 size);
   void (*be_get_general_param)(UINT32 type, UINT8* data, UINT8* size);//Maximum size is 255. Please allocate 255byte data area before calling get API.
} SONY_BE_GEN_FUNC_T;

typedef struct _SONY_PQ_GEN_FUNC_T
{
	void (*pq_set_general_param)(UINT32 type, UINT8[255], UINT8 size);
	void (*pq_get_general_param)(UINT32 type, UINT8* data, UINT8* size);//Maximum size is 255. Please allocate 255byte data area before calling get API.
}SONY_PQ_GEN_FUNC_T;

EXTERN UINT8* SRS_Table_A;
EXTERN UINT8* SRS_Table_B;
EXTERN UINT8* SRS_Table;

typedef enum {
	E_CUST_PART_DDRI,
	E_CUST_PART_MLFW,
	E_CUST_PART_LDAT,
	E_CUST_PART_MAFW,
	E_CUST_PART_DFW1,
	E_CUST_PART_DFW2,
	E_CUST_PART_PDAT,
	E_CUST_PART_BDAT,
	E_CUST_PART_NDAT,
	E_CUST_PART_FDAT,
	E_CUST_PART_UDAT,
	E_CUST_PART_BDIX,
	E_CUST_PART_MINF,
	E_CUST_PART_BEDAT,
	E_CUST_PART_RSV1,
	E_CUST_PART_RSV2,
	E_CUST_PART_RSV3,
	E_CUST_PART_RSV4,
	E_CUST_PART_MAX,
} E_CUST_BE_PART;

typedef enum {
E_CUST_PART_SPQD,
E_CUST_PART_SPQC,
E_CUST_PART_SPMAX,
} E_CUST_SPQ_PART;

typedef enum {
    PANEL_INFO,
    BE_SW_VERSION,
 }E_BE_DEVICE;

typedef enum {
    E_CUST_TCON_ERR = 1,
    E_CUST_P_ID_ERR = 1 << 1,
    E_CUST_VLED_ERR = 1 << 2,
    E_CUST_LD_ERR_ADC = 1 << 3,
    E_CUST_BACKLIGHT_ERR = 1 << 4,
    E_CUST_SOFTWARE_ERR = 1 << 5,
    E_CUST_EXT_FRC_ERR = 1 << 6, //2K Model Ext. FRC Error
    E_CUST_4KPQ_I2C_ERROR = 1 << 7,
    E_CUST_8KBE_COM_ERR = 1 << 8,
    E_CUST_FAN_ERR = 1 << 9,
    E_CUST_8KSHV_ERR = 1 << 10,   // for 8KDEC communication error
    E_CUST_ALL = 1 << 31,
 }E_BE_ERROR_DETECT;


typedef struct {
	UINT16 u2LDDimTable[33];
	UINT16 u2LDGainTable[41];
	UINT16 u2LDGainLimitTable[33];
} CUST_LD_HW_TABLE_T;

typedef struct {
	UINT16 u2LDGainLimitTable_LOW_BND[33];
	UINT16 u2LDGainLimitTable_UP_BND[33];
} CUST_LD_SW_TABLE_T;

typedef struct {
	UINT16  u2max_pxl_th;
    UINT8   u1dim_wet_0;
    UINT8   u1dim_wet_1;
    UINT8   u1dim_wet_2;
    UINT8   u1dim_wet_3;
    UINT8   u1dim_wet_4;
    UINT8   u1rgb_weight;
    UINT8   u1rgb_weight2;
    UINT8   u1lpf_weight;
    UINT8   u1blkinfo_weight;
    UINT8   u1maxinfo_lpf_mode;
    UINT16  u2max_dim;
    UINT16  u2min_dim;
    UINT8   u1dy_dim_slope;
    UINT8   u1max_up_dim_step;
    UINT8   u1max_dn_dim_step;
	UINT16	u2max_diff_thd;
	UINT16	u2total_diff_thd;
	UINT16	u2avg_diff_thd;
} CUST_LD_HW_PARAM_T;

typedef struct {
	UINT8   u1lcdim_speed_ctrl_en;
    UINT8   u1osd_detect;
    UINT8   u1motion_th;
    UINT16  u2max_diff_thd2;
    UINT16  u2total_diff_thd2;
    UINT16  u2avg_diff_thd2;
    UINT8   u1scg_wait_th;
    UINT8   u1motion_penalty;
    UINT8   u1gamma_penalty;
    UINT8   u1static_step;
    UINT8   u1maxrgb_step_low;
    UINT8   u1maxrgb_step_mid_dn;
    UINT8   u1maxrgb_step_mid_up;
    UINT8   u1maxrgb_step_high;
    UINT8   u1predet_apl_th_low;
    UINT8   u1predet_apl_th_high;
    UINT8   u1predet_apl_th_ostg;
    UINT8   u1predet_step;
    UINT8   u1predet_frame_th;
    UINT8   u1fadein_det_dark_th;
    UINT8   u1fadein_det_inc_th;
    UINT8   u1fadein_step;
    UINT8   u1confidence_th0;
    UINT8   u1confidence_th1;
    UINT8   u1confidence_th2;
    UINT8   u1confidence_th3;
    UINT8   u1confidence_thdft;
    UINT8   u1lcdim_max_up_dim_step_low;
    UINT8   u1lcdim_max_up_dim_step_mid_dn;
    UINT8   u1lcdim_max_up_dim_step_mid_up;
    UINT8   u1lcdim_max_up_dim_step_high;
    UINT8   u1lcdim_max_dn_dim_step_low;
    UINT8   u1lcdim_max_dn_dim_step_mid_dn;
    UINT8   u1lcdim_max_dn_dim_step_mid_up;
    UINT8   u1lcdim_max_dn_dim_step_high;
    UINT8   u1lcdim_dy_dim_slope_low;
    UINT8   u1lcdim_dy_dim_slope_mid_dn;
    UINT8   u1lcdim_dy_dim_slope_mid_up;
    UINT8   u1lcdim_dy_dim_slope_high;
    UINT8   u1lcdim_adap_mindim;
    UINT8   u1dist_th;
    UINT8   u1apl_th;
    UINT8   u1adap_weight_0_l;
    UINT8   u1adap_weight_1_l;
    UINT8   u1adap_weight_2_l;
    UINT8   u1adap_weight_3_l;
    UINT8   u1adap_weight_4_l;
    UINT8   u1adap_weight_0_h;
    UINT8   u1adap_weight_1_h;
    UINT8   u1adap_weight_2_h;
    UINT8   u1adap_weight_3_h;
    UINT8   u1adap_weight_4_h;
    UINT8   u1adv_ps_en;
    UINT8   u1adv_ps_ratio;
    UINT8   u1scg_iir_en;
    UINT8   u1iir_sel;
    UINT8   u1downtrend_dim_th;
    UINT8   u1scg_enable;
    UINT8   u1boost_0;
    UINT8   u1boost_1;
    UINT8   u1boost_2;
} CUST_LD_SW_PARAM_T;


typedef struct {
    UINT8 u1GainH[3];
    UINT8 u1GainV[3];
    UINT8 u1GainD[3];
    UINT8 u1CorzH[3];
    UINT8 u1CorzV[3];
    UINT8 u1CorzD[3];
	UINT8 u1HSHPGainLow[2];
    UINT8 u1HSHPGainMid[2];
    UINT8 u1HSHPGainHigh[2];
    UINT8 u1HSHPCor[3];
	UINT8 u1Edf1Th[3];
    UINT8 u1EdgeGain[3];
    UINT8 u1DetailGain[3];
	UINT8 u1Edf2Th[4];
	UINT8 u1Edf2FGain[3];
	UINT8 u1Edf2DGain[3];
	UINT8 u1Edf2EGain[6];
	UINT8 u1LTIGain[3];
    UINT8 u1LTICorz[3];
    UINT8 u1TapAdap[4];
    UINT8 u1Ibse[2];
} CUST_SHARP_OFST_T;

typedef enum {
	E_CUST_ADC_PORT_SCART_FS,
	E_CUST_ADC_PORT_KEY,
	E_CUST_ADC_PORT_LIGHT_SENSOR,
	E_CUST_ADC_PORT_PANEL_PWR_DET,
	E_CUST_ADC_PORT_BL_ERR_DET,
	E_CUST_ADC_PORT_USB_OC2_DET,
	E_CUST_ADC_PORT_RF_AGCI,
	E_CUST_ADC_PORT_X_SYSTEM_RST,
	E_CUST_ADC_PORT_12V_DET,
	E_CUST_ADC_PORT_5V_DET,
	E_CUST_ADC_PORT_ADIN5_SRV,
	E_CUST_ADC_PORT_ADIN7_SRV,
} E_CUST_ADC_PORT;

typedef enum {
	E_CUST_SCREEN_SAVER_MODE_0,
	E_CUST_SCREEN_SAVER_MODE_1,
	E_CUST_SCREEN_SAVER_MODE_2,
	E_CUST_SCREEN_SAVER_MODE_3,
	E_CUST_SCREEN_SAVER_MODE_4,
} E_CUST_SCREEN_SAVER_MODE;

typedef enum {
	CUST_SCR_MIRROR_PROTOCOL_UDP,
	CUST_SCR_MIRROR_PROTOCOL_TCP,
	CUST_SCR_MIRROR_PROTOCOL_UNKNOWN,
} E_CUST_SCR_MIRROR_PROTOCOL;


typedef struct {
	UINT32 u4LBoxLowFinal;
	UINT32 u4LBoxUpFinal;
	UINT32 u4LBoxRightFinal;
	UINT32 u4LBoxLeftFinal;
}CUST_LETTER_BOX_INFO_T;


typedef struct {
	E_CUST_SCREEN_SAVER_MODE eScreenSaverMode;
} CUST_SCREEN_SAVER_T;

typedef struct {
	UINT8 bAgingMode;
} CUST_AGING_MODE_T;

typedef struct {
	E_CUST_DIMENSION eDimension;
} CUST_TV_OUT_FMT_T;

typedef struct{
    UINT32 u4Width;
    UINT32 u4Height;
}CUST_AREA_T;

typedef struct{
	CUST_AREA_T   t_DownScalerInput ;
	CUST_AREA_T   t_DownScalerOutput;
	CUST_AREA_T   t_ScalerInput;
	CUST_AREA_T   t_ScalerOutput;
	CUST_AREA_T   t_ScalerTiming;//newly add
	E_CUST_FRAME_RATE eFrameRate;
}
CUST_SCL_AREA_T;

typedef struct{
    E_CUST_SCR_MIRROR_PROTOCOL eProtocol;
}CUST_SCR_MIRROR_CONFIG_T;

typedef struct{
	UINT16 u2DataSize;
	UINT8 u1DataBody[65533];
}CUST_PQ_META_DATA_T;

typedef struct{
	UINT16 u2X;
	UINT16 u2Y;
}CUST_CAP_POS_T;

typedef struct{
	UINT16 u2Cr_R;
	UINT16 u2Y_G;
	UINT16 u2Cb_B;
}CUST_CAP_DATA_T;

typedef struct{
	CUST_CAP_DATA_T SceInput;
	CUST_CAP_DATA_T SceOutput;
}CUST_SCE_CAP_DATA_T;

typedef struct
{
	INT8 s1QpPPS;
	INT8 s1QpPictrue;
	INT8 s1QpMB;
	UINT8 u1QpMin;
	UINT8 u1QpMax;
    BOOL    b1Valid;
    UINT8   u1PictureCodingType;
    UINT32  u4GeneratedBit;
    UINT8   u1ScalingListIntra[64];
    UINT8   u1ScalingListInter[64];
    UINT8   u1OddEven;
    UINT8   u1color_primaries;
    UINT8   u1transfer_characteristics;
    UINT8   u1matrix_coefficients;
    UINT8   u1UserData[128];
}CUST_DECODER_METER_T;

//============================below for native matrix setting=====================================

typedef enum{
	E_THROUGH,          // 0
	E_601_LIMITED,	    // 1
	E_601_FULL,         // 2
	E_709_LIMITED,      // 3
	E_709_FULL,         // 4
	E_2020_LIMITED,     // 5
	E_2020_FULL,        // 6
	E_2020YC_LIMITED,   // 7
	E_2020YC_FULL,      // 8
	E_RGB_LIMITED,      // 9
	E_RGB_FULL,         // 10
	E_YCBR_LIMITED,     // 11
	E_YCBR_FULL,        // 12
	E_RGB2020_LIMITED,  // 13
	E_RGB2020_FULL,     // 14
	E_RESERVED3,        // 15
	E_RESERVED4,        // 16
	E_RESERVED5,        // 17
	E_RESERVED6,        // 18

}E_COL_FORMAT;

typedef enum{
	E_NOT_MODIFY,
	E_MODIFY_BY_CONVERT_PATTERN,
	E_MODIFY_BY_MATRIX_COEFFICIENT,

}E_COL_TYPE;

typedef enum{
	E_DITHER_MODE_DISABLE=0,
	E_DITHER_MODE_ENABLE_12BIT,
	E_DITHER_MODE_ENABLE_10BIT,
	E_DITHER_MODE_ENABLE_8BIT,
	E_DITHER_MODE_ENABLE_6BIT,
	E_DITHER_MODE_MAX,

}E_CUST_DITHER_MODE;
typedef enum{
	E_CUST_DITHER_BLOCK_CCIR0=0,
	E_CUST_DITHER_BLOCK_CCIR1,
	E_CUST_DITHER_BLOCK_OSTG0,
	E_CUST_DITHER_BLOCK_OSTG1,
	E_CUST_DITHER_BLOCK_OSTG_POST0,
	E_CUST_DITHER_BLOCK_OSTG_POST1,
	E_CUST_DITHER_BLOCK_MAX,

}E_CUST_DITHER_BLOCK;

typedef enum{
	E_CUST_DEMURA_CHANNEL_A=0,
	E_CUST_DEMURA_CHANNEL_B,
}E_CUST_DEMURA_CHANNEL;

typedef struct{
	UINT32 onoff;
	E_CUST_DITHER_MODE type;
}CUST_DITHER_T;

typedef struct{
	UINT32 block;
	E_COL_FORMAT src;
	E_COL_FORMAT dst;
}CUST_COL_CONV_CAPABILITY_PATTERN_T;
typedef struct{
	UINT8 u1BlendRatioBlky;
	UINT8 u1BlendRatioDe;
	UINT8 u1BlendRatioTxtr;
	UINT8 u1BlendRatioMv;
	UINT8 u1FltStrMax;

}CUST_TNR_BLEND_RATIO;

static const CUST_COL_CONV_CAPABILITY_PATTERN_T supported_pattern_list[] = {
	//PRE
	{0,0,0}, 	 //PRE through setting but RGB/YCC is different, So using 0 1 1  or 0 10 10, sugest not using
	{0,1,1}, 	 //PRE 601L-->601L
	{0,2,1}, 	 //PRE 601F-->601L
	{0,3,1}, 	 //PRE 709L-->601L
	{0,4,1}, 	 //PRE 709F-->601L
	{0,5,1},	 //PRE 2020L-->601L
	{0,6,1},	 //PRE 2020F-->601L
	{0,9,1},	 //PRE RGBL-->601L
	{0,10,1},	 //PRE RGBF-->601L
	{0,13,1},	 //PRE RGB2020L-->601L
	{0,14,1},	 //PRE RGB2020F-->601L

	{0,9,3},	 //PRE RGBl-->709l
	{0,10,3},	 //PRE RGBF-->709l
	{0,1,3}, 	 //PRE 601L-->709L
	{0,2,3}, 	 //PRE 601F-->709L

	{0,5,5},	 //PRE 2020l-->2020l-->601L-->601L
	{0,6,5},	 //PRE 2020F-->2020l-->601F-->601L
	{0,13,5},	 //PRE RGB2020l-->2020l-->RGB601L-->601L
	{0,14,5},	 //PRE RGB2020F-->2020l-->RGB601F-->601L
	{0,7,7}, 	 //PRE 2020Cl-->2020Cl-->601L-->601L
	{0,8,7}, 	 //PRE 2020CF-->2020Cl-->601F-->601L

	{0,1,10}, 	 //PRE 601L-->RGBF
	{0,2,10}, 	 //PRE 601F-->RGBF
	{0,3,10}, 	 //PRE 709L-->RGBF
	{0,4,10}, 	 //PRE 709F-->RGBF
	{0,9,10},	 //PRE RGBL-->RGBF
	{0,10,10},	 //PRE RGBF-->RGBF

	//OS
	{1,0,0},     //OS through
	{1,10,10},   //OS RGBF-->RGBF
	{1,1,10},    //OS 601L-->RGBF
	{1,5,10},    //OS 2020L-->RGBF

	{1,1,9},     //OS 601L-->RGBL
	{1,5,9},     //OS 2020L-->RGBL

	//xv1
	{2,0,0},     //xv1 through

	//r2y
	{3,0,0},     //r2y through
	{3,10,1},	 //r2y RGBF-->601L
	{3,10,2},    //r2y RGBF-->601F
	{3,10,3},    //r2y RGBF-->709L
	{3,10,4},    //r2y RGBF-->709F
};

typedef struct {
	E_COL_FORMAT input;
	E_COL_TYPE type;
	UINT32 block;
	E_COL_FORMAT src;
	E_COL_FORMAT dst;
	INT32 matrix[15];
}CUST_COL_CONV_TABLE_PARAM_T;

typedef struct _SONY_BE_EXT_FUNC_T{
    void (*be_ext_edid_write)(void);
}SONY_BE_EXT_FUNC_T;

typedef struct{
	UINT8  ysh_3d_hist_cen;
	UINT8  ysh_3d_nonlinear_norm;
	UINT16 ysh_3d_norm_variance;
	UINT16 ysh_3d_y_interval[2];
	UINT16 ysh_3d_s_interval[7];
	UINT16 ysh_3d_h_interval[11];
}CUST_TDHIST_CONFIG_PARAM_T;
enum
{
	E_MTX_INPUT,
	E_MTX_PRE,
	E_MTX_OS,
	E_MTX_XV1,
	E_MTX_R2Y,
	E_MTX_MAX,

};


enum
{
	E_INPUT_THROUGH,		  // 0
	E_INPUT_601L,
	E_INPUT_601F,
	E_INPUT_709L,
	E_INPUT_709F,
	E_INPUT_2020L,
	E_INPUT_2020F,
	E_INPUT_2020YCL,
	E_INPUT_2020YCF,
	E_INPUT_RGBL,
	E_INPUT_RGBF,
	E_INPUT_YCBR_LIMITED,   // 11
	E_INPUT_YCBR_FULL,      // 12
	E_INPUT_RGB2020_LIMITED,      // 13
	E_INPUT_RGB2020_FULL,         // 14
	E_INPUT_RESERVED3,      // 15
	E_INPUT_RESERVED4,      // 16
	E_INPUT_RESERVED5,      // 17
	E_INPUT_RESERVED6,      // 18
	E_INPUT_MAX,

};
typedef enum{
	XVYCC_BLOCK_1,
	XVYCC_BLOCK_2,
	XVYCC_BLOCK_MAX,
}E_XVYCC_BLOCK;
typedef enum{
	XVYCC_GAMMA_GM,
	XVYCC_GAMMA_AGM,
	XVYCC_GAMMA_MAX,
}E_XVYCC_GAMMA;
typedef struct{
	E_XVYCC_BLOCK block_no;    //xvYCC1/xvYCC2
	E_XVYCC_GAMMA gamma_type;  //GM/AGM
}CUST_XVYCC_DEST_T;
typedef struct{
	UINT8 xvycc1_gm;		//set to mtk reg: reg_xvycc_gm_table_sel
	UINT8 xvycc1_agm;		//set to mtk reg: reg_xvycc_agm_table_sel
	UINT8 xvycc2_gm;		//set to mtk reg: reg_xvs_gm_table_sel
	UINT8 xvycc2_agm;		//set to mtk reg: reg_xvs_agm_table_sel
}CUST_XVYCC_GAMMA_TABLE_SEL_T;

// enum definitions of RGB sensor type
typedef enum
{
    E_RGB_SENSOR_TYPE_0,  // Vishay VEML3328SL
    E_RGB_SENSOR_TYPE_NUM
} E_CUST_RGB_SENSOR_TYPE;

// configuration parameters for E_RGB_SENSOR_TYPE_0
typedef struct
{
    UINT8 it;
    UINT8 gain1;
    UINT8 gain2;
    UINT8 hd;
} CUST_RGB_SENSOR_CONFIG_TYPE_0;

// configuration parameters for RGB sensor
typedef struct
{
    E_CUST_RGB_SENSOR_TYPE eRgbSensorType;
    union
    {
        CUST_RGB_SENSOR_CONFIG_TYPE_0  type0;
        UINT8 config_data[16];
    } RgbSensorConfig;
} CUST_RGB_SENSOR_PARAM_T;

typedef struct
{
    UINT8  device_error;
    UINT16 r_level;
    UINT16 g_level;
    UINT16 b_level;
    UINT16 c_level;
    UINT16 ir_level;
} CUST_RGB_SENSOR_LEVEL_T;

typedef struct sony_lsad_ {
	UINT16 LSAD00;
	UINT16 LSAD01;
	UINT16 LSAD02;
	UINT16 LSAD03;
	UINT16 LSAD04;
	UINT16 LSAD05;
	UINT16 LSAD06;
	UINT16 LSAD07;
	UINT16 LSAD08;
	UINT16 LSAD09;
	UINT16 LSAD10;
	UINT16 LSAD11;
	UINT16 LSAD12;
	UINT16 LSAD13;
	UINT16 LSAD14;
	UINT16 LSAD15;
} sony_lsad;

typedef struct sony_lgmv_ {
	UINT16 LGMV00;
	UINT16 LGMV01;
	UINT16 LGMV02;
	UINT16 LGMV03;
	UINT16 LGMV04;
	UINT16 LGMV05;
	UINT16 LGMV06;
	UINT16 LGMV07;
	UINT16 LGMV08;
	UINT16 LGMV09;
	UINT16 LGMV10;
	UINT16 LGMV11;
	UINT16 LGMV12;
	UINT16 LGMV13;
	UINT16 LGMV14;
	UINT16 LGMV15;
} sony_lgmv;

typedef struct sony_params_ {
	sony_lsad s_lsad;
	sony_lgmv s_lgmv;
	UINT16 gsad;
	UINT8 h_gmv;
	UINT8 v_gmv;
	UINT8 cadence_result;
	UINT8 system_update;
	UINT8 chaos_level;
	UINT8 scene_change;
	UINT8 is_hpan;
	UINT8 is_vpan;
	UINT8 is_diapan;
	UINT8 reserve;
} sony_params;

typedef struct {
  UINT16	pTopPos;	// Result value of top position of black-belt detection
  UINT16	pBotPos;	// Result value of bottom position of black-belt detection
  UINT8	pDarkSceneFlag;	// Result of detection for dark scene
} CUST_MEMC_BLACK_BELT_T;

typedef struct
{
    sony_params stSony;
	CUST_MEMC_BLACK_BELT_T stMEMCBlackBelt;
    UINT8 EndCreditRollFlag;
    UINT8 GetTelopFlag;

} mtk_params;

typedef struct{
	UINT16 u2X_Start; //(MTK Req) skin luma hist x start
	UINT16 u2X_End;   //(MTK Req) skin luma hist x end
	UINT16 u2Y_Start; //(MTK Req) skin luma hist y start
	UINT16 u2Y_End;   //(MTK Req) skin luma hist y end
}CUST_SKIN_LUMA_POS_T;

typedef struct{
    UINT8     y2r_en;
    UINT8     y2r_round_en;
    UINT8     y2r_input_format;
    UINT16    y2r_c00;
    UINT16    y2r_c01;
    UINT16    y2r_c02;
    UINT16    y2r_c10;
    UINT16    y2r_c11;
    UINT16    y2r_c12;
    UINT16    y2r_c20;
    UINT16    y2r_c21;
    UINT16    y2r_c22;
    UINT16    y2r_c21_sp;
    UINT16    y2r_c02_sp;
    UINT16    y2r_pre_add_0;
    UINT16    y2r_pre_add_1;
    UINT16    y2r_pre_add_2;
    UINT16    y2r_post_add_0;
    UINT16    y2r_post_add_1;
    UINT16    y2r_post_add_2;
    UINT8     r2y_en;
    UINT8     r2y_round_en;
    UINT8     r2y_input_format;
    UINT16    r2y_c00;
    UINT16    r2y_c01;
    UINT16    r2y_c02;
    UINT16    r2y_c10;
    UINT16    r2y_c11;
    UINT16    r2y_c12;
    UINT16    r2y_c20;
    UINT16    r2y_c21;
    UINT16    r2y_c22;
    UINT16    r2y_pre_add_0;
    UINT16    r2y_pre_add_1;
    UINT16    r2y_pre_add_2;
    UINT16    r2y_post_add_0;
    UINT16    r2y_post_add_1;
    UINT16    r2y_post_add_2;
}CUST_OPENHDR_COLORIMETRY_T;

typedef struct{
    UINT8     gamut_matrix_en;
    UINT32    c00;
    UINT32    c01;
    UINT32    c02;
    UINT32    c10;
    UINT32    c11;
    UINT32    c12;
    UINT32    c20;
    UINT32    c21;
    UINT32    c22;
    UINT32    pre_add_0;
    UINT32    pre_add_1;
    UINT32    pre_add_2;
    UINT32    post_add_0;
    UINT32    post_add_1;
    UINT32    post_add_2;
}CUST_OPENHDR_CSC_T;

typedef struct{
    UINT8    reg_luma_gain_en;
    UINT8    tone_map_en;
    UINT8    c_hdr2sdr_main_bypass_0;
}CUST_OPENHDR_TONEMAP_REG_T;

typedef struct{
    UINT32    reg_p1;
    UINT32    reg_p2;
    UINT32    reg_p3;
    UINT32    reg_p4;
    UINT32    reg_p5;
    UINT32    reg_p6;
    UINT32    reg_p7;
    UINT32    reg_p8;
    UINT32    reg_p9;
    UINT32    reg_p10;
    UINT32    reg_p11;
    UINT32    reg_p12;
    UINT32    reg_p13;
    UINT32    reg_p14;
    UINT32    reg_p15;
    UINT32    reg_p16;
    UINT32    reg_slope0;
    UINT32    reg_slope1;
    UINT32    reg_slope2;
    UINT32    reg_slope3;
    UINT32    reg_slope4;
    UINT32    reg_slope5;
    UINT32    reg_slope6;
    UINT32    reg_slope7;
    UINT32    reg_slope8;
    UINT32    reg_slope9;
    UINT32    reg_slope10;
    UINT32    reg_slope11;
    UINT32    reg_slope12;
    UINT32    reg_slope13;
    UINT32    reg_slope14;
    UINT32    reg_slope15;
    UINT16    act_panel_nit;
    UINT16    hw_src_nit;
    UINT8    system_gamma_sel;
}CUST_OPENHDR_TONEMAP_SW_T;

typedef enum{
    OPENHDR_GAMMA_EOTF,
    OPENHDR_GAMMA_OETF,
    OPENHDR_GAMMA_MAX,
}E_CUST_OPENHDR_GAMMA_DEST;

// drv_MapleGetApiSys.c
E_CUST_INPUT_TYPE DRVCUST_GetInputType(UINT8 bPath);
void DRVCUST_GetSignalFormat(UINT8 bPath, CUST_SIG_FMT_T* pSignalFormat);
UINT8 DRVCUST_GetAnalogDecNoiseLevel(UINT8 bPath);
UINT32 DRVCUST_GetDecoderAvgBitRate(UINT8 ucEsId);
E_CUST_WINDOW_MODE DRVCUST_GetWindowMode(void);
E_CUST_MUTE_STATE DRVCUST_GetPlaneMuteState(UINT8 bPath);
E_CUST_MUTE_STATE DRVCUST_GetOutputMuteState(UINT8 u1TxPath);
void DRVCUST_GetOutputPosition(UINT8 bPath, CUST_OUT_POSI_T* pOutPosition);
void DRVCUST_GetSourceArea(UINT8 u1BeHandle, CUST_SRC_AREA_T* pSclReq);
void DRVCUST_GetScalingRequest(UINT8 u1BeHandle, CUST_DEST_AREA_T* pSclReq);
void DRVCUST_GetOutputFormat(UINT8 u1TxPath, CUST_OUT_FMT_T* pOutFmt);
UINT8 DRVCUST_GetBorderPlaneRequest(UINT8 u1BeHandle);
UINT8 DRVCUST_GetBlurRequest(UINT8 u1BeHandle);
void DRVCUST_GetMCVPMeter(CUST_MCVP_METER_T* pMcvpMeter);
E_CUST_CADENSE DRVCUST_GetCadense(void);
E_CUST_FRAME_RATE DRVCUST_GetMJCInputFrequency(void);
void DRVCUST_GetPostProcMeter(CUST_POST_PROC_METER_T* pPostProcMeter);
void DRVCUST_GetPostProcTdHistMeter(UINT8* pData);
void DRVCUST_GetLdMeter(CUST_LD_METER_T* pLdMeter);
UINT8 DRVCUST_GetOpticalSensorLevel(void);
UINT8 DRVCUST_GetTemperatureSensorLevel(void);
UINT8 DRVCUST_GetBLPowerLimit(void);
UINT8 DRVCUST_GetPwmSetting(void);
void DRVCUST_GetVideoFrameDelay(CUST_VDO_DELAY_INFO_T* pVdoDelayInfo);
void DRVCUST_GetHistoryData(CUST_HISTORY_DATA_T* pHistoryData);
void DRVCUST_GetPowerState(CUST_POWER_STATE_T* pPowerState);
UINT32 DRVCUST_BePartitionRead(E_CUST_BE_PART ePartId, UINT32 u4Offset, VOID *pvBuf, UINT32 u4BufSize);
UINT32 DRVCUST_SopqPartitionRead(E_CUST_SPQ_PART ePartId, UINT32 u4Offset, VOID *pvBuf, UINT32 u4BufSize);
UINT16 DRVCUST_GetAdcValue(E_CUST_ADC_PORT eAdcPort);
void DRVCUST_GetScreenSaver(CUST_SCREEN_SAVER_T* pScreenSaver);
void DRVCUST_GetAgingMode(CUST_AGING_MODE_T* pAgingMode);
void DRVCUST_GetTvOutputFormat(CUST_TV_OUT_FMT_T* pTvOutFmt);
void DRVCUST_GetMuffinScalerInfo(UINT8 bPath, CUST_SCL_AREA_T* pScalerArea);
void DRVCUST_GetOsdState(BOOL* bOsdState);
void DRVCUST_GetModelVariationInfoStartAddress(UINT32* pAddress);
void DRVCUST_GetLetterBoxDetection(CUST_LETTER_BOX_INFO_T* pLetterBoxInfo);
void DRVCUST_GetScreenMirroringConfig(UINT8 bPath, CUST_SCR_MIRROR_CONFIG_T* pScrMirrorConfig);
void DRVCUST_GetScePixMonData(CUST_SCE_CAP_DATA_T* pData);
void DRVCUST_GetOstgPixMonData(CUST_CAP_DATA_T* pData);
void DRVCUST_GetDecoderMeter(UINT8 bPath, CUST_DECODER_METER_T* pDecoderMeter);
UPTR DRVCUST_GetVirtual2PhysicalAddress(UPTR* pVirtualAddress);
UPTR DRVCUST_GetPhysical2VirtualAddress(UPTR* pPhysicalAddress);

// drv_MapleGetApiAp.c
void DRVCUST_GetPictureSetting(CUST_PIC_SETTING_T* pPictureSetting);
void DRVCUST_GetScreenSetting(CUST_SCREEN_SETTING_T* pScreenSetting);
void DRVCUST_Get3DSetting(CUST_3D_SETTING_T* p3dSetting);
void DRVCUST_GetEcoSetting(CUST_ECO_SETTING_T* pEcoSetting);
void DRVCUST_GetPhotoSetting(CUST_PHOTO_SETTING_T* pPhotoSetting);
void DRVCUST_GetDemoMode(CUST_DEMO_MODE_T* pDemoMode);
void DRVCUST_GetDemoOsdMaskArea(CUST_DEMO_OSD_MASK_T* pOsdMask);
void DRVCUST_GetServiceSetting(CUST_SERVICE_SETTING_T* pServiceSetting);
void DRVCUST_GetServiceTransferSetting(CUST_SERVICE_TRANS_T* pServiceTrans);
void DRVCUST_GetServiceVcomAdjust(CUST_SERVICE_VCOM_T* pServiceVcom);
E_CUST_UI_INPUT_TARGET DRVCUST_GetUiInputTarget(void);
void DRVCUST_GetWhiteBalanceOffset(CUST_WB_OFFSET_T* pWbOffset);
void DRVCUST_GetBTSetting(CUST_BT_SETTING_T* pBtSetting);
void DRVCUST_GetColorAdjust(CUST_COLOR_ADJUST_T* pColorAdjust);
void DRVCUST_GetWhiteBalanceOffset20pEnable(CUST_WB_OFFSET_20P_EN_T* pWbOffset20pEn);
void DRVCUST_Get4bandFreqHist(CUST_4BAND_HIST_T* pData);
void DRVCUST_GetPicControlState(E_CUST_PIC_CONTROL_STATE *pPicControlState);
UINT32 DRVCUST_GetOsdPixelCount(void);


// drv_MapleSetApi.c
void DRVCUST_SetExclusiveReg(UINT8 u1Enable);
UINT8 DRVCUST_CheckSetExclusiveReg(UINT8 u1Enable);
void DRVCUST_SetLumaCurve(const UINT16* waLArray, const UINT16* waSkinLArray);
void DRVCUST_GetADLLumaCurveHw(UINT16 * waLArray);
void DRVCUST_GetADLLumaCurve65Hw(UINT16 * waLArray);
void DRVCUST_GetADLSecondLumaCurveHw(UINT16 * waSLArray);
void DRVCUST_GetADLSecondLumaCurve65Hw(UINT16 * waSLArray);
void DRVCUST_GetLumaCurve(UINT16* waLArray, UINT16* waSkinLArray);
void DRVCUST_SetContrast(UINT16 u2Value);
UINT16 DRVCUST_GetContrast(void);
void DRVCUST_SetColor(UINT16 u2Value);
UINT16 DRVCUST_GetColor(void);
void DRVCUST_SetHue(UINT16 u2Value);
UINT16 DRVCUST_GetHue(void);
void DRVCUST_SetPqWBGainOffset(CUST_RGB_GAIN_OFST_T* pstRgbGainOfst);
void DRVCUST_GetPqWBGainOffset(CUST_RGB_GAIN_OFST_T* pstRgbGainOfst);
void DRVCUST_SetSharpnessOffset(CUST_SHARP_OFST_T* pSharpOfst);
void DRVCUST_SetSRSLut(UINT8* pTableA, UINT8* pTableB);
UINT8 DRVCUST_CheckSetSRSLut(void);
UINT8 DRVCUST_FlushPqSetting(void);
UINT8 DRVCUST_CheckFlushPqSettingFlag(void);
void DRVCUST_EnNoiseLevel(UINT8 u1Enable);
UINT8 DRVCUST_GetNoiseLevelEn(void);
void DRVCUST_SetNoiseLevel(UINT8 u1NoiseLevel);
UINT8 DRVCUST_GetNoiseLevel(void);
void DRVCUST_EnNoiseLevel2(UINT8 u1Enable);
UINT8 DRVCUST_GetNoiseLevel2En(void);
void DRVCUST_SetNoiseLevel2(UINT8 u1NoiseLevel);
UINT8 DRVCUST_GetNoiseLevel2(void);
void DRVCUST_SetPostSharpYLev(UINT8* pYLev);
void DRVCUST_SetIbseYLev(UINT8* pYLev);

void DRVCUST_GetPostSharpYLev(UINT8* pYLev);
void DRVCUST_GetIbseYLevTable(UINT8* pYLev);

void DRVCUST_GetYLevTable(UINT8* pYLev);
void DRVCUST_UpdateTimingGroup(UINT8* pCustSrcTimingTbl, UINT8* pCustSrcGroup);
void DRVCUST_PqSendDataToBe(UINT8* pData, UINT32 u4Size);
void DRVCUST_RegisterPqFunc(SONY_PQ_FUNC_T *pq_func);
void DRVCUST_DeregisterPqFunc(void);
void DRVCUST_RegisterBeFunc(SONY_BE_FUNC_T *be_func);
void DRVCUST_DeregisterBeFunc(void);
void DRVCUST_RegisterBeGeneralFunc (SONY_BE_GEN_FUNC_T* be_gen_func);
void DRVCUST_RegisterPQGeneralFunc(SONY_PQ_GEN_FUNC_T* pq_gen_func);
void DRVCUST_DeRegisterPQGeneralFunc(void);
void DRVCUST_DeregisterBeGeneralFunc(void);
void DRVCUST_ResetSystem(E_CUST_RESET eReset);
void DRVCUST_SetEcsCommandReply(UINT8* pData, UINT32 u4Size);
UINT32 DRVCUST_BePartitionWrite(E_CUST_BE_PART ePartId, UINT32 u4Offset, VOID *pvBuf, UINT32 u4BufSize);
UINT32 DRVCUST_SopqPartitionWrite(E_CUST_SPQ_PART ePartId, UINT32 u4Offset, VOID *pvBuf, UINT32 u4BufSize);
void DRVCUST_SetBeDeviceErrorDetect(E_BE_ERROR_DETECT Efactor);
void DRV_CUST_SetBEDeviceVersion(UINT8 u1Number, UINT8* pVersion);
void DRVCUST_SetBeDeviceInformation(E_BE_DEVICE device, VOID* pParam, UINT32 size);
void DRVCUST_SetPqColMatrixParam(UINT8 u1Upof_ext, UINT8 u1Ucof_ext,UINT8 u1Ubof_ext );
void DRVCUST_GetPqColMatrixParam(UINT8 *u1Cont_ext, UINT8 *u1Color_ext,UINT8 *u1Bri_ext );
void DRVCUST_SetPqNewPartialS(UINT8* u1PointL, UINT8* u1PointM, UINT8* u1PointH);
void DRVCUST_SetPqPartialY(UINT8* u1Point);
void DRVCUST_SetScalerCoefLumaH(UINT32* u4Array);
void DRVCUST_SetScalerCoefChromaH(UINT32* u4Array);
void DRVCUST_SetScalerCoefLumaV(UINT32* u4Array);
void DRVCUST_SetScalerCoefChromaV(UINT32* u4Array);
void DRVCUST_SetLdHwParam(CUST_LD_HW_PARAM_T* pLDHWParam);
void DRVCUST_SetLdSwParam(CUST_LD_SW_PARAM_T* pLDSWParam);
void DRVCUST_SetLdHwTable(CUST_LD_HW_TABLE_T* pLDHWTable);
void DRVCUST_SetLdSwTable(CUST_LD_SW_TABLE_T* pLDSWTable);
void DRVCUST_SetPqSwVersion(UINT8* pData);
void DRVCUST_SetPqMetaData(CUST_PQ_META_DATA_T* pPqMetaData);
void TranMetadataToDram(UINT8 *pDest, UINT8 *pSrc, UINT16 u4Size);
void DRVCUST_SetScePixMonCapEnable(BOOL en);
CUST_SCE_CAP_DATA_T* DRVCUST_SetScePixMonPos(CUST_CAP_POS_T* pPos);
void DRVCUST_SetScePixMonInkEnable(BOOL en);
void DRVCUST_SetScePixMonInkColor(CUST_CAP_DATA_T* pData);
void DRVCUST_SetOstgPixMonCapEnable(BOOL en);
CUST_CAP_DATA_T* DRVCUST_SetOstgPixMonPos(CUST_CAP_POS_T* pPos);
void DRVCUST_SetOstgPixMonCurseColor(CUST_CAP_DATA_T* pColor);
void DRVCUST_Set4BandFilterRang(CUST_4BAND_FILTER_RANGE *pData);
void DRVCUST_Set4BandHFilter(CUST_4BAND_H_FILTER *pData);
void DRVCUST_EnSetTnr(UINT8 u1Enable);
void DRVCUST_SetTnr(CUST_TNR_BLEND_RATIO* pTnrSet);
void DRVCUST_SetDemuraTransferSetting(E_CUST_DEMURA_CHANNEL eChannel, BOOL en );
void DRVCUST_SetPqTdHistConfig(CUST_TDHIST_CONFIG_PARAM_T* pData);
void DRVCUST_SetPqNewPartialSposition(UINT8* u1Point1, UINT8* u1Point2);
void DRVCUST_SetXvyccRamTable(CUST_XVYCC_DEST_T* pData, UINT16*pRamData, UINT16 u2Size);
void DRVCUST_SetXvyccGammaTableSel(CUST_XVYCC_GAMMA_TABLE_SEL_T *pData);
UINT8 DRVCUST_SetRGBSensorParam(CUST_RGB_SENSOR_PARAM_T *pGlobalDimmingInfo);
void DRVCUST_SetPqSkinLumaHistDetPosition(CUST_SKIN_LUMA_POS_T* pPos);
void DRVCUST_SetPqSkinLumaHistDetThreshold(UINT16 u2Threshold);
void DRVCUST_GetPqFaceDetInfo(UINT8* u1Num);
void DRVCUST_GetPqSkinLumaHistMeter(UINT32* u4LumaHist);
void DRVCUST_SetDV_GDLog(UINT8 u1OnOff);



// backlight control

UINT32 DRVCUST_GetBLPwmFrequency(UINT8 u1Channel);
UINT16 DRVCUST_GetBLPwmDuty(UINT8 u1Channel);
void DRVCUST_GetBLScanPwmNumberOfPulses(UINT16* pulseNumber);
void DRVCUST_GetBLScanPwmDuty(UINT16* ratio);
void DRVCUST_GetBLScanPwmPhase(UINT32* commonPhase, INT32* phase);
void DRVCUST_InitBLScanPWM(UINT16 pulseNumber, UINT16 ratio, UINT32 commonPhase, INT32 phase);
UINT8 DRVCUST_GetBLPwmMode(UINT8 channel);
UINT8 DRVCUST_GetBLControlState(void);
void DRVCUST_SetBLPwmFrequency(UINT8 u1Channel, UINT32 u4TimeBase);
void DRVCUST_SetBLPwmDuty(UINT8 u1Channel, UINT16 u2Ratio);
void DRVCUST_SetBLScanPwmNumberOfPulses(UINT16* pulseNumber);
void DRVCUST_SetBLScanPwmDuty(UINT16* ratio);
void DRVCUST_SetBLScanPwmPhase(UINT32* commonPhase, INT32* phase);
void DRVCUST_SetBLPwmMode(UINT8 channel, UINT8 mode);
UINT8 DRVCUST_SetBLHeaderFooter(UINT32 u4CmdHeader, UINT32 u4CmdFooter);
UINT16 DRVCUST_GetCurrentScanPWMPulseNum(UINT8 u1Src);
UINT8 DRVCUST_GetBLHeaderFooter(UINT32 *u4CmdHeader, UINT32 *u4CmdFooter);

// Gamma table
void DRVCUST_GetWBGammaTable(UINT8 u1Index, UINT16 * pData);
void DRVCUST_SetWBGammaTable(UINT8 u1Index, UINT16 * pData);

void DRVCUST_SetWhiteBalanceOffset20pEnable(CUST_WB_OFFSET_20P_EN_T* pWbOffset20pEn);

UINT8 DRVCUST_SuperGameMode(UINT32 u4VdpId);
void DRVCUST_SetDemoMode(E_CUST_DEMO_MODE eDemoMode, E_CUST_DEMO_PATTERN eDemoPattern, UINT32 u4DemoOption);
void DRVCUST_SetOsdPixCountAlphaThresh(UINT8 u1Thresh);
UINT8 DRVCUST_GetOsdPixCountAlphaThresh(void);
void DRVCUST_GetColorSensorSetting(CUST_COLOR_SENSOR_SETTING_T *pColorSensorSetting);



INT32 DRVCUST_SetColorimetryConvTable(CUST_COL_CONV_TABLE_PARAM_T* pColConvTableParam);
void DRVCUST_GetColorimetryConvCapability(UINT32* pSize, CUST_COL_CONV_CAPABILITY_PATTERN_T* pColConvCapabilityPattern);

INT32 DRVCUST_SetDolbyConfigData(UPTR u4Offset, UINT32 u4Size);
//add for AUTO TEST
void DRVCUST_GetLdHwParam(CUST_LD_HW_PARAM_T* pLDHWParam);
void DRVCUST_GetLdSwParam(CUST_LD_SW_PARAM_T* pLDSWParam);
void DRVCUST_GetLdSwTable(UINT16 *u2LDGainLimitTable_LOW_BND, UINT16 *u2LDGainLimitTable_UP_BND);

void DRVCUST_RegisterBeExtFunc(SONY_BE_EXT_FUNC_T* pfnBeCb);
void DRVCUST_ReadEDID(UINT8 u1Type, UINT8* pBuff, UINT32* pOffset);
UINT8 DRVCUST_GetHDMIPortNumber(void);

void DRVCUST_SetAddtionalRestrictionCondition(E_CUST_RESTRICT_COND eAddCondition);
UINT8 DRVCUST_GetAddtionalRestrictionCondition(void);
void DRVCUST_GetDolbyVisionGlobalDimming(CUST_GLOBAL_DIMMING_INFO_T *pGlobalDimmingInfo);
void DRVCUST_TestWatchdogReset(UINT8 u1WatchdogID);
BOOL DRVCUST_IsMaintenanceReset(void);
void DRVCUST_GetDOVIType(UINT8 bPath, E_CUST_DOVI_TYPE* pSignalFormat);
void DRVCUST_GetRGBSensorLevel(CUST_RGB_SENSOR_LEVEL_T *pGlobalDimmingInfo);
UINT8 DRVCUST_GetRGBSensorID(void);

//mStar LD ctrol
void DRVCUST_SetGDStrength(UINT8 u1Str);
void DRVCUST_SetLDStrength(UINT8 u1Str);
void DRVCUST_SetLdCompensationBlend(UINT8 u1Alpha);
void DRVCUST_SetLdMeanMaxAlpha(UINT8 u1Alpha);
void DRVCUST_SetLdBacklightInGamma(UINT8 u1Type);
void DRVCUST_SetLdBacklightRemap(UINT8 u1Type);
void DRVCUST_SetLdMinClamp(UINT8 u1MinVal);
void DRVCUST_SetLdSpetialFilterStrength1(UINT8 u1Str);
void DRVCUST_SetLdSpetialFilterStrength2(UINT8 u1Str);
void DRVCUST_SetLdSpetialFilterStrength3(UINT8 u1Str);
void DRVCUST_SetLdSpetialFilterStrength4(UINT8 u1Str);
void DRVCUST_SetLdSpetialFilterStrength5(UINT8 u1Str);
void DRVCUST_SetLdTemporalFilterStrengthDn(UINT8 u1Str);
void DRVCUST_SetLdTemporalFilterStrengthUp(UINT8 u1Str);
void DRVCUST_SetLdTemporalFilterLowTh(UINT8 u1Th);
void DRVCUST_SetLdTemporalFilterHighTh(UINT8 u1Th);
void DRVCUST_SetLdTemporalFilterMaxSpeed(UINT8 u1Spd);
void DRVCUST_SetLDEn(BOOL bEn);
void DRVCUST_SetLdSaturationProtection(BOOL bEn, UINT8 u1StartPoint, UINT8 u1FinalGain);
void DRVCUST_SetLdGroupingStart(BOOL bEn);
void DRVCUST_SetColorSensorSetting(CUST_COLOR_SENSOR_SETTING_T *pColorSensorSetting);
void DRVCUST_SetLdDynamicAlphaEn(UINT8 u1DynAlphEn);
void DRVCUST_SetLdfBlendAlpha(UINT8 u1BlendAlph);
void DRVCUST_SetLdDynamicAlphaLUT(UINT8 u1Idx, UINT8 u1Val);

//void DRVCUST_SetLdGroupingEnd(BOOL bEn);
void DRVCUST_MAPI_LD_Set_SPI_Output_En(BOOL bEn);

#ifdef CC_SUPPORT_PHOTO8K

typedef struct {
    UINT32 u4X;
    UINT32 u4Y;
    UINT32 u4Width;
    UINT32 u4Height;
    E_CUST_COL_DEPTH eColorDepth;
    E_CUST_COL_SPACE eColorSpace;
    E_CUST_PHOTO_COL_SPACE ePhotoColorSpace;
    E_CUST_VDO_CODEC eVideoCodec;
    E_CUST_HDR_TYPE eHDRType;
} CUST_DIVPHOTO_META_INFO_T;

typedef struct {
  UINT32 (*GetVB1Mode)(void);
  void (*ReplyVB1ModeChange)(BOOL bResult);
  UINT32 (*GetState)(void);
  void (*ReplyStateChange)(UINT32 u4State, BOOL bResult);
  void (*GetMetaInfo)(CUST_DIVPHOTO_META_INFO_T *pMetaInfo);
  UINT32 (*GetFadeDuration)(void);
} CUST_DIVPHOTO_FUNC_TBL_T;

typedef enum {
    E_CUST_NORMAL_PHOTO_MODE,
    E_CUST_DIVIDE_PHOTO_MODE,
    E_CUST_UNKNOWN_PHOTO_MODE,
} E_CUST_B2R_DIVPHOTO_MODE;

typedef enum {
    E_CUST_DIV_IDLE,
    E_CUST_DIV_LEFT_TOP,
    E_CUST_DIV_RIGHT_TOP,
    E_CUST_DIV_LEFT_BOTTOM,
    E_CUST_DIV_RIGHT_BOTTOM,
    E_CUST_DIV_UNKNOWN,
} E_CUST_B2R_DIVPHOTO_STA;

typedef struct {
  UINT32 u4HRegion;
  UINT32 u4VRegion;
} CUST_DIVPHOTO_REGION_T;

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When the timing of ¡°addon¡± initialize, this function will called from ¡°addon¡±.
 *   And when the timing of deinitialize, this function will called with NULL parameter.
 *
 *  @param  pDivPhotoFuncTbl     Callback function table for DivPhoto addon
 *  @retval   void
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_RegisterDivPhotoFunc(CUST_DIVPHOTO_FUNC_TBL_T *pDivPhotoFuncTbl);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *  Get current VB1 transfer mode.
 *
 *  @param  none
 *  @retval 0: Normal transfer mode (default)
 *  @retval 1: 4Divided 8K photo transfer mode
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 DRVCUST_GetVB1Mode(void);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When this function is called from sony_bectrl.ko module,
 *   call DivPhotoFuncTbl->ReplyVB1ModeChange() with received arguments.
 *
 *  @param  bResult        result of VB1 mode changing  (TRUE: success, FALSE: failed)
 *  @retval   none
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_ReplyVB1ModeChange(BOOL bResult);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When this function is called from sony_bectrl.ko module,
 *   call DivPhotoFuncTbl->GetState() and please return its return value.
 *
 *  @param  none
 *  @retval   Current DivPhoto sequencer state
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 DRVCUST_GetDivPhotoState(void);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When this function is called from sony_bectrl.ko module,
 *   call DivPhotoFuncTbl->ReplyStateChange() with received arguments.
 *
 *  @param  u4State   Indicates current state value (for error detection).
 *  @param  bResult   Result of processing which defined by each state.
 *  @retval   none
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_ReplyDivPhotoStateChange( UINT32 u4State, BOOL  bResult);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When this function is called from sony_bectrl.ko module,
 *   call DivPhotoFuncTbl->GetMetaInfo() with receiveid arguments.
 *
 *  @param  pDivPhotoMetainfo        photo meta information
 *  @retval   none
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_GetDivPhotoMetaInfo(CUST_DIVPHOTO_META_INFO_T *pDivPhotoMetainfo);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When this function is called from sony_bectrl.ko module,
 *   call DivPhotoFuncTbl->GetFadeDuration() and please return its return value.
 *
 *  @param  none
 *  @retval   Fade duration value
 */
//-----------------------------------------------------------------------------
EXTERN UINT32  DRVCUST_GetDivPhotoFadeDuration(void);


//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   When initialization of "addon" is performed,
 *   this function is called from "addon" with u4Mode is other than 0.
 *   In this case, please change VB1 transfer mode to
 *   8K divided photo mode. For more detail about "divided photo mode"
 *
 *  @param  u4Mode
        0: normal video transfer (default)
        1: divided photo mode (main video path)
 *  @retval   none
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_SetB2RPhotoMode(UINT32 u4Mode);

//-----------------------------------------------------------------------------
/** Driver customization interface function.
 *   This function is used in order to specify position of divided photo.
 *
 *  @param  u4DivPhotoState  Divided photo state (means position)
 *      0: idle (default)
 *      1: left top transfer
 *      2: right top transfer
 *      3: left bottom transfer
 *      4: right bottom transfer
 *  @retval   none
 */
//-----------------------------------------------------------------------------
EXTERN void DRVCUST_SetB2RDivPhotoState(UINT32 u4DivPhotoState);

#endif

// MEMC interfaces
EXTERN void DRVCUST_MEMC_GetSonyParams(MailBoxCb callback, UINT32 req_id);
EXTERN void DRVCUST_MEMC_GetBlackBeltPosition(MailBoxCb callback, UINT32 req_id);
EXTERN void DRVCUST_MEMC_GetEndCreditRollFlag(MailBoxCb callback, UINT32 req_id);
EXTERN void DRVCUST_MEMC_GetTelopFlag(MailBoxCb callback, UINT32 req_id);
EXTERN void DRVCUST_MEMC_VideoDelayAdjustment(MailBoxCb callback, UINT32 req_id, UINT16 u2AdditionalDelay);
EXTERN void DRVCUST_MEMC_SetVideoFormat(UINT8 u1InputWidth,UINT8 u1OutputWidth,UINT8 u1InputFrequency,UINT8 u1OutputFrequency);
EXTERN void DRVCUST_MEMC_SetVideoMode(UINT8 u1Mode);
EXTERN void DRVCUST_MEMC_SetOsdBypassWin(MailBoxCb callback, UINT32 req_id, UINT8 u1WinID,UINT8 u1WinEn,UINT16 u2WinStartX,UINT16 u2WinStartY,UINT16 u2WinWidth,UINT16 u2WinHeight);
EXTERN void DRVCUST_MEMC_SetMfcDemo(MailBoxCb callback, UINT32 req_id, UINT8 u1DemoMode,UINT8 u1BoarderLineWidth,UINT8 u1BoarderColor);
EXTERN void DRVCUST_MEMC_FilmCadanceControl(MailBoxCb callback, UINT32 req_id, UINT16 u2CadenceEnable);
EXTERN void DRVCUST_MEMC_DeflickerControl(MailBoxCb callback, UINT32 req_id, UINT8 u1DeflickerLevel,UINT8 u1AutoThLUp,UINT8 u1AutoThMUp,UINT8 u1AutoThHUp,UINT8 u1AutoThLDown,UINT8 u1AutoThMDown,UINT8 u1AutoThHDown);
EXTERN void DRVCUST_MEMC_Set_RelPosRatioIndex(MailBoxCb callback, UINT32 req_id, UINT8 u1Index);
EXTERN void DRVCUST_MEMC_SetChaosLevelKeep(MailBoxCb callback, UINT32 req_id, UINT8 u1KeepValue);
EXTERN void DRVCUST_MEMC_SetBlackBeltSel(MailBoxCb callback, UINT32 req_id, UINT8 u1Sel);
EXTERN void DRVCUST_MEMC_FunctionEnable(MailBoxCb callback, UINT32 req_id, UINT8 u1FunctionSel,UINT8 u1Enable,UINT8 u1Parameter1);
EXTERN void DRVCUST_MEMC_OriInputResolutionInfo(MailBoxCb callback, UINT32 req_id, UINT16 u2Width,UINT16 u2Height);
EXTERN void DRVCUST_MEMC_SetModelName(MailBoxCb callback, UINT32 req_id, UINT8 u1ModelName,UINT8 u1PanelSize,UINT8 u1BackLightType);
EXTERN void DRVCUST_MEMC_PicMode(UINT8 u1PictureMode);
EXTERN void DRVCUST_MEMC_ImpremetationFrame(MailBoxCb callback, UINT32 req_id, UINT8 u1Enable,UINT8 u1Index);
EXTERN void DRVCUST_MEMC_PositionRatioTable(MailBoxCb callback, UINT32 req_id, UINT8 u1Enable);
EXTERN void DRVCUST_MEMC_ShowfileCadenceErrorEnable(MailBoxCb callback, UINT32 req_id, UINT8 u1Enable);

EXTERN void DRVCUST_RequestSpecialPostMute(BOOL bEnable);
EXTERN void DRVCUST_SetVDECSignalPath(UINT32 num,UINT32 path);
EXTERN UINT32 DRVCUST_GetVDECSignalPath(UINT32 num);
EXTERN void DRVCUST_SetBacklightMute(UINT8 u1OnOff);

EXTERN BOOL DRVCUST_MEMC_SetMode(UINT8 u1MEMCMode, UINT8 u1ColorMode);
EXTERN void DRVCUST_MEMC_GetMode(UINT8 *u1MEMCMode, UINT8 *u1ColorMode);

EXTERN void DRVCUST_MEMC_SetInfo(MEMC_MODE_INFO_TYPE *pMemcInfo);
EXTERN void DRVCUST_MEMC_DumpInfo(void);

EXTERN void DRVCUST_GetVRRStatus(E_CUST_VSYNC_MODE* pVsyncMode);
EXTERN INT32 DRVCUST_SetVRRMode(E_CUST_VSYNC_MODE eVsyncMode);
EXTERN void DRVCUST_SetHDMISignalPath(UINT32 u4Num, UINT32 u4SigPath);
EXTERN UINT32 DRVCUST_GetHDMISignalPath(void);
EXTERN void DRVCUST_MEMC_SetColorFormat(UINT32 u4ClrFmt);

EXTERN void DRVCUST_SetOpenHdrColorimetryParam(CUST_OPENHDR_COLORIMETRY_T* pHwData);
EXTERN void DRVCUST_SetOpenHdrCscParam(CUST_OPENHDR_CSC_T* pHwData);
EXTERN void DRVCUST_SetOpenHdrToneMapParam(CUST_OPENHDR_TONEMAP_REG_T* pHwData, CUST_OPENHDR_TONEMAP_SW_T* pSwData);
EXTERN void DRVCUST_SetOpenHdrGammaRamTable(E_CUST_OPENHDR_GAMMA_DEST dest, UINT32* pRamData, UINT16 size);

#endif //_DRV_MAPLE_API_H_

