/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: fbm_pool_config.c,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

/** @file fbm_pool_config.c
 *  Brief of file fbm_pool_config.c.
 *  FBM Pool Management
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "x_lint.h"
#include "fbm_pool.h"
#include "fbm_pool_config.h"
#include "fbm_debug.h"
#include"fbm_drvif.h"
#include "vdo_misc.h"
#include "../tve/tve_if.h"
#include "b2r_if.h"

#ifdef CC_SRM_ON
#include "srm_drvif.h"
#endif

LINT_EXT_HEADER_BEGIN
#include "source_table.h"
#include "vdp_drvif.h"
#include "x_assert.h"
#include "x_hal_5381.h"
#include "c_model.h"
#include "drvcust_if.h"
#include "x_mid.h"
LINT_EXT_HEADER_END

#include "drv_display.h"
#include "drv_tdtv_drvif.h"

#undef min
#define min(x, y) ((x < y) ? x : y)

extern UINT32 SRM_GetScposMode(UINT32 u4VdpId);
extern UINT32 SRM_GetMddiMode(UINT32 u4VdpId);
extern UINT32 SRM_GetTDCMode(UINT32 u4VdpId);
extern UINT32 SRM_GetScposHPDSize(UINT32 u4VdpId);

#if defined(CC_EDB_FBM_SHARE_MEM)
extern UINT32 _gu4EDBSize;
extern UINT32 _gu4EDBAddr;
#endif
#if defined(CC_TRUSTZONE_SUPPORT) && defined(CC_SVP_SUPPORT)
extern FBM_POOL_T _arPoolTZ[];
#endif
//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------

static FBM_POOL_TYPE_T _eCurrentScposMain = FBM_POOL_TYPE_SCPOS_MAIN;
static FBM_POOL_TYPE_T _eCurrentScposPip = FBM_POOL_TYPE_SCPOS_PIP;
static FBM_POOL_TYPE_T _eCurrentMddiMain = FBM_POOL_TYPE_MDDI_MAIN;
static FBM_POOL_TYPE_T _eCurrentMddiPip = FBM_POOL_TYPE_MDDI_PIP;
static FBM_POOL_TYPE_T _eCurrentMjc = FBM_POOL_TYPE_MJC_1;
static FBM_POOL_TYPE_T _eCurrentTdc = FBM_POOL_TYPE_TDC;
static FBM_POOL_TYPE_T _eCurrentVbi = FBM_POOL_TYPE_VBI;

// NPTV dynamically-allocated FBM
static UINT32 _fbm_u4NptvBeAddr;
#if defined(CC_MT5890)||defined(CC_MT5891) || defined(CC_MT5893)|| defined(CC_MT5886) || defined(CC_MT5895)
static UINT32 _fbm_u4NptvSubAddr;
#endif
#if defined(CC_MT5882)||defined(CC_MT5865) || defined(CC_MT5863)
static UINT32 _fbm_u4NptvSubPopAddr;
static UINT32 _fbm_u4NptvSubPipAddr;
#endif
typedef enum
{
    FBM_MODE_1080I,
    FBM_MODE_576I,
    FBM_MODE_1080P_422,
    FBM_MODE_1080P_444
} FBM_POOL_SCENARIO_T;

typedef struct
{
    UINT8 u1fgPIP;
    UINT8 u1fgPOP;
    UINT8 u1fgSingle;
    UINT8 u1mode[2];
    UINT8 u1Interlace[2];
    UINT8 u1IsMPEG[2];
    UINT8 u1IsTDC[2];
    UINT32 u4Resolution[2];
    UINT32 u4InputWidth[2];
    UINT32 u4InputHeight[2];
} FBM_AUTO_INC_ENV_T;

typedef struct
{
    UINT32 u4MainPipMaxSize;
    UINT32 u4MainPopMaxSize;
    UINT32 u4SubPipMaxSize;
    UINT32 u4SubPopMaxSize;
    UINT32 u4MainPipMaxEnd;
    UINT32 u4MainPopMaxEnd;
    UINT32 u4SubPipMaxEnd;
    UINT32 u4SubPopMaxEnd;
    UINT8 u1MainPipMaxMode;
    UINT8 u1MainPopMaxMode;
    UINT8 u1SubPipMaxMode;
    UINT8 u1SubPopMaxMode;
} FBM_VIDEO_MAX_T;

// FBM_POOL_T _FBM_NPTV_VIDEO_CONF[FBM_TYPE_ENV_MAX][FBM_VIDEO_AGENT_MAX];
FBM_POOL_T (* _FBM_NPTV_VIDEO_CONF)[FBM_VIDEO_AGENT_MAX] = NULL;

/* get the max range of main and sub video agents */
FBM_VIDEO_MAX_T _FBM_NPTV_VideoRange;

//For ATVPVR to make scaler dram address fixed.
static UINT8  fgFBMFixedScalerDram = FALSE;

// for Pscan testing
BOOL fgFBM_PscanTest=0;
BOOL fgFBM_PscanTestSub=0;

void FBM_SetPscanTestMode(BOOL fgIsTestMode)
{
    //Printf("---------FBM_SetPscanTestMode: %x----------\n",fgIsTestMode);
    fgFBM_PscanTest=fgIsTestMode;
}

void FBM_SetPscanTestModeSubPIP(BOOL fgIsTestMode)
{
    //Printf("---------FBM_SetPscanTestMode: %x----------\n",fgIsTestMode);
    fgFBM_PscanTestSub=fgIsTestMode;
}

void FBM_FixedScalerDram(UINT8 fgOnOff)
{
    fgFBMFixedScalerDram = fgOnOff;
}

UINT8 FBM_GetFixedScalerDram(void)
{
    return fgFBMFixedScalerDram;
}

static FBM_AUTO_INC_ENV_T* _FbmGetEnv(FBM_AUTO_INC_ENV_T *env)
{
    UINT8 u1fgPIP = SRM_IsPipVideo();
    UINT8 u1fgPOP = SRM_IsPopVideo();
    UINT8 u1fgSingle = !(u1fgPIP || u1fgPOP);
    UINT8 u1mode[2];
    UINT8 u1Interlace[2];
    UINT8 u1IsMPEG[2] = {0, 0};
    UINT8 u1IsTDC[2] = {0, 0};
    SRM_VDP_INFO_T* prVdpInfo[2];
    UINT32 u4Resolution[2];

    prVdpInfo[VDP_1] = SRM_GetVdpInfo(VDP_1);
    prVdpInfo[VDP_2] = SRM_GetVdpInfo(VDP_2);

    u4Resolution[VDP_1] = prVdpInfo[VDP_1]->u4Resolution;
    if (prVdpInfo[VDP_1]->u4Interlace == 1)
    {
        u1Interlace[VDP_1] = 1;
        if (prVdpInfo[VDP_1]->u4Resolution < SRM_VDP_720HD_RESOLUTION)
        {
            u1mode[VDP_1] = FBM_MODE_576I;
        }
        else
        {
            u1mode[VDP_1] = FBM_MODE_1080I;
        }
    }
    else
    {
        u1Interlace[VDP_1] = 0;
        if (prVdpInfo[VDP_1]->u4VgaTiming == 1)
        {
            u1mode[VDP_1] = FBM_MODE_1080P_444;
        }
        else
        {
            u1mode[VDP_1] = FBM_MODE_1080P_422;
        }
    }

    u4Resolution[VDP_2] = prVdpInfo[VDP_2]->u4Resolution;
    if (prVdpInfo[VDP_2]->u4Interlace == 1)
    {
        u1Interlace[VDP_2] = 1;
        if (prVdpInfo[VDP_2]->u4Resolution < SRM_VDP_720HD_RESOLUTION)
        {
            u1mode[VDP_2] = FBM_MODE_576I;
        }
        else
        {
            u1mode[VDP_2] = FBM_MODE_1080I;
        }
    }
    else
    {
        u1Interlace[VDP_2] = 0;
        if (prVdpInfo[VDP_2]->u4VgaTiming == 1)
        {
            u1mode[VDP_2] = FBM_MODE_1080P_444;
        }

        else
        {
            u1mode[VDP_2] = FBM_MODE_1080P_422;
        }
    }

    /**
     * manual scan mode workaround: during manual scan, these scpos events
     * will confuse the fbm configuration.
     */
    if (SRM_IsScanMode(VDP_1))
    {
        u1mode[VDP_1] = FBM_MODE_576I;
    }
    if (SRM_IsScanMode(VDP_2))
    {
        u1mode[VDP_2] = FBM_MODE_576I;
    }

    if (prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_DTV)
    {
        u1IsMPEG[VDP_1] = 1;
    }
    if (prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_DTV)
    {
        u1IsMPEG[VDP_2] = 1;
    }

    if ((prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_ATV) ||
            (prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_ATD) ||
            (prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_SV) ||
            (prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_CVBS) ||
            (prVdpInfo[VDP_1]->u4Source == (UINT32)VSS_SCART))
    {
        u1IsTDC[VDP_1] = 1;
    }
    if ((prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_ATV) ||
            (prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_ATD) ||
            (prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_SV) ||
            (prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_CVBS) ||
            (prVdpInfo[VDP_2]->u4Source == (UINT32)VSS_SCART))
    {
        u1IsTDC[VDP_2] = 1;
    }

    env->u1fgPIP = u1fgPIP;
    env->u1fgPOP = u1fgPOP;
    env->u1fgSingle = u1fgSingle;
    env->u1mode[0] = u1mode[0];
    env->u1mode[1] = u1mode[1];
    env->u1Interlace[0] = u1Interlace[0];
    env->u1Interlace[1] = u1Interlace[1];
    env->u1IsMPEG[0] = u1IsMPEG[0];
    env->u1IsMPEG[1] = u1IsMPEG[1];
    env->u1IsTDC[0] = u1IsTDC[0];
    env->u1IsTDC[1] = u1IsTDC[1];
    env->u4Resolution[0] = u4Resolution[0];
    env->u4Resolution[1] = u4Resolution[1];
    env->u4InputWidth[0]  = prVdpInfo[0]->u4InputWidth;
    env->u4InputWidth[1]  = prVdpInfo[1]->u4InputWidth;
    env->u4InputHeight[0] = prVdpInfo[0]->u4InputHeight;
    env->u4InputHeight[1] = prVdpInfo[1]->u4InputHeight;
    return env;
}


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------
#if defined(CC_DYNAMIC_FBMSRM_CONF)
static UINT32 _srmfbm_conf_value = FBM_MEM_CFG_MT5890_3DTV_4K;
static UINT32 _srmfbm_chb_conf_value = FBM_MEM_CFG_MT5890_3DTV_4K_CHB;
static UINT32 _srmfbm_chc_conf_value = FBM_MEM_CFG_MT5890_3DTV_4K_CHC;

void SRMFBM_SetConf(UINT32 conf)
{
    _srmfbm_conf_value = conf;
}
UINT32 SRMFBM_GetConf(void)
{
    return _srmfbm_conf_value;
}
void SRMFBM_CHB_SetConf(UINT32 conf)
{
    _srmfbm_chb_conf_value = conf;
}
UINT32 SRMFBM_CHB_GetConf(void)
{
    return _srmfbm_chb_conf_value;
}
void SRMFBM_CHC_SetConf(UINT32 conf)
{
    _srmfbm_chc_conf_value = conf;
}
UINT32 SRMFBM_CHC_GetConf(void)
{
    return _srmfbm_chc_conf_value;
}


#else
UINT32 SRMFBM_GetConf(void)
{
    return DRVCUST_OptGet(eFbmMemSize);
}
#endif

#if defined(CC_MT5882)

#if !defined(CC_TVP_SUPPORT)
static VOID _FbmConfigurePool_MT5882_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMX | MPEG | PIP | MDDI | DSP

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = FBM_POOL_TYPE_TOTAL;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((0x300000), FBM_DMX_SIZE_ALIGMENT);
#endif
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = FBM_POOL_TYPE_DMX;


    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = FBM_POOL_TYPE_SEAML1;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FORCE_DRAM256M)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit =  FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 =  FBM_POOL_TYPE_JPG_VDP;
    u4AheadType =  FBM_POOL_TYPE_MPEG;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = FBM_POOL_TYPE_MPEG;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = FBM_POOL_TYPE_SCPOS_MAIN;

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = 0;
#endif
    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif
#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif
#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }
#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if (defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)) && (!defined(CC_FORCE_DRAM256M))
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FORCE_DRAM256M)
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif
#if defined(CC_FBM_PQ_3DC)
    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;
#endif
    u4BehindType = FBM_POOL_TYPE_NS;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4BehindType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4BehindType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;

#if !defined(CC_FORCE_DRAM256M)
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4BehindType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4BehindType = FBM_POOL_TYPE_DSP;
#endif

#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_OD;
#endif
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_DSP_BIN;
#endif

#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize * 2;
#endif

    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_SCPOS_MAIN;

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif

    // JPEG (Between Start & OSD1)
#if (!defined(CC_FORCE_DRAM256M))
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
#endif

#else
    // JPEG + B2R share DMX + MPEG
    // JPEG
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = FBM_POOL_TYPE_TOTAL;

    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = FBM_POOL_TYPE_JPEG;
#endif
}

#else
static VOID _FbmConfigurePool_MT5882_TVP_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    //==================NoSecure FBM===================

    u4AheadType = FBM_POOL_TYPE_TOTAL;

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }

#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)
#if (!defined(CC_FBM_CAST_TV_STRINK)) && (!defined(CC_LMP_CUST))
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
#if !defined(CC_FBM_CAST_TV_STRINK)
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4AheadType = FBM_POOL_TYPE_DSP;

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif
#if defined(CC_FBM_PQ_3DC)
    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;
#endif
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER;


    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
#ifdef CC_FBM_PQ_3DC
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_PQ_3DC;
#else
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_FEEDER;
#endif

    if(prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize >= prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize)
        u4AheadType = FBM_POOL_TYPE_FEEDER;
    else
        u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;

#if (!defined(CC_FBM_CAST_TV_STRINK)) && (!defined(CC_LMP_CUST))
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD1;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif

#else
    // JPEG
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    //==================Secure FBM=====================

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif


#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = 0;
#endif
    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif


#ifndef CC_OSD_USE_FBM
    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_B2R;
#endif


    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((0x300000), FBM_DMX_SIZE_ALIGMENT);
#endif
    u4AheadType = FBM_POOL_TYPE_DMX;

#if !defined(CC_SUPPORT_NPTV_SEAMLESS)
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;
#endif

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_CAST_TV_STRINK) || defined(CC_LMP_CUST)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit =  FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 =  FBM_POOL_TYPE_JPG_VDP;
    u4AheadType =  FBM_POOL_TYPE_MPEG;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif


#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;


}
#endif //!defined(CC_TVP_SUPPORT)

#endif//defined(CC_MT5882)


#if defined(CC_MT5883)
#if !defined(CC_TVP_SUPPORT)
static VOID _FbmConfigurePool_MT5883_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMX | MPEG | PIP | MDDI | DSP

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = FBM_POOL_TYPE_TOTAL;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((0x300000), FBM_DMX_SIZE_ALIGMENT);
#endif
    u4AheadType = FBM_POOL_TYPE_DMX;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;


    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#endif

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_MPEG;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif


#if defined(CC_SUPPORT_3D_IMGRZ_SEAMLESS)


    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif

    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER;

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if 0 //lemur disable feeder2
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif
#endif

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }
#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

    u4BehindType = FBM_POOL_TYPE_NS;
#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_OD;
#endif
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4BehindType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif

#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize * 2;
#endif

    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)


    // JPEG (Between Start & OSD1)
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_FEEDER;
#else
    // JPEG + B2R share DMX + MPEG
    // JPEG
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = FBM_POOL_TYPE_TOTAL;

    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = FBM_POOL_TYPE_JPEG;
#endif
}

#else
static VOID _FbmConfigurePool_MT5883_TVP_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    //==================NoSecure FBM===================

    u4AheadType = FBM_POOL_TYPE_TOTAL;

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
#if !defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) && !defined(CC_VOMX_TV_COEXIST) //close 3d ,dsp  will align again
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4AheadType = FBM_POOL_TYPE_DSP;


    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }

#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER;


    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_PQ_3DC;

    if(prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize >= prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize)
        u4AheadType = FBM_POOL_TYPE_FEEDER;
    else
        u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;

    // JPEG (Between Start & OSD1)
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#else
    // JPEG
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if 0  //lemur disable feeder2
#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif
#endif

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    //==================Secure FBM=====================

    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;
    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#endif

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_MPEG;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif

#if defined(CC_SUPPORT_3D_IMGRZ_SEAMLESS)


    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;
#endif
#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif


#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = 0;
#endif
    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif


#ifndef CC_OSD_USE_FBM
    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_B2R;
#endif


    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((0x300000), FBM_DMX_SIZE_ALIGMENT);
#endif
    u4AheadType = FBM_POOL_TYPE_DMX;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif


#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

}
#endif //!defined(CC_TVP_SUPPORT)

#endif//defined(CC_MT5883)


#if  defined(CC_MT5399) ||defined(CC_MT5890)||defined(CC_MT5891)
static VOID _FbmConfigurePool_MT5890_2K80_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_720P_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif
#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;

#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel
    u4AheadType = FBM_POOL_TYPE_TOTAL2;
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif

#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif

#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG1_2K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG2_2K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    // handle Image Viewer Pool
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif

#endif
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;
#else
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = FBM_POOL_TYPE_TOTAL3;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG1_2K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG2_2K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_2K_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif
    prPoolList[FBM_POOL_TYPE_REST_MEM].u4PoolSize = FBM_REST_MEM_POOL_SIZE_2K;
    prPoolList[FBM_POOL_TYPE_REST_MEM].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REST_MEM;
#endif
}

static VOID _FbmConfigurePool_MT5890_FHD_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif
#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif
#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif
    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel
    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = FBM_POOL_TYPE_TOTAL2;
    u4AheadType = FBM_POOL_TYPE_DMX;

    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_DMX2_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG1_2K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG2_2K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_2K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_2K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif

#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif

#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_2K_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;


    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG1_2K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG2_2K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    // handle Image Viewer Pool
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif

#endif
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;
#else
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = FBM_POOL_TYPE_TOTAL3;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif
#endif
}

static VOID _FbmConfigurePool_MT5890_4K_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_4K_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif


#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = FBM_POOL_TYPE_TOTAL2;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_TOTAL2;

    u4AheadType = FBM_POOL_TYPE_FEEDER;

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif
    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_DMX2_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX3_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG1_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG2_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML3_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;
#endif

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG3_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

#if defined(CC_SUPPORT_W2DR2R)
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = FBM_POOL_TYPE_MPEG;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML4_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
#endif
    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG4_4K_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

#endif

    // MPEG_C|MJC|OD|PNG|JPEG|VSS|FEEDER|DRM|PQ_3DC|SWDMX
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;
#else
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = FBM_POOL_TYPE_TOTAL3;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG1_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML1_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;


#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
#endif
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML2_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG2_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;


    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG3_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML3_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG4_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML4_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_720P_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

    prPoolList[FBM_POOL_TYPE_REST_MEM].u4PoolSize = FBM_REST_MEM_POOL_SIZE_4K;
    prPoolList[FBM_POOL_TYPE_REST_MEM].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REST_MEM;

#endif
}

static VOID _FbmConfigurePool_MT5890_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif
#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif
#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = FBM_POOL_TYPE_TOTAL2;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_TOTAL2;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif


#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    // handle Image Viewer Pool
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
#if 0       //
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif
#endif
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif
#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

#endif


    // MPEG_C|MJC|OD|PNG|JPEG|VSS|FEEDER|DRM|PQ_3DC|SWDMX
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;
#else
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = FBM_POOL_TYPE_TOTAL3;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif
    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)

    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#endif
}

static VOID _FbmConfigurePool_MT5861_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif
#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif


    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;
#endif


#if !defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL2;

#if defined(CC_TVP_SUPPORT)
#ifndef CC_BYPASS_MJC
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif
#endif


#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;


#if !defined(CC_TVP_SUPPORT)
#ifndef CC_BYPASS_MJC
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif
#endif

#endif
}

static VOID _FbmConfigurePool_MT5891_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif
#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif
    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;


#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL2;

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#endif

#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif


#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    // handle Image Viewer Pool
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
#if 0       //
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif
#endif
    // secure buffer begin
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B;


#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_SCPOS_MAIN;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif


#endif


    // MPEG_C|MJC|OD|PNG|JPEG|VSS|FEEDER|DRM|PQ_3DC|SWDMX
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C;
#else
    u4AheadType = FBM_POOL_TYPE_TOTAL3;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C;

    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif


#if defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#endif

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
#endif
    // secure buffer end
#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#endif
}

static VOID _FbmConfigurePool_MT5891_4K_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------
    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_4K_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;


#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_SCPOS_MAIN;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;


#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif


#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;

#if defined(CC_S_PLATFORM) && (defined(CC_WK_CUST) || defined(CC_HK_CUST))
    u4BehindType = FBM_POOL_TYPE_NS_A;// must align channelA tail
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_RAMOOPS;
#endif

    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = FBM_POOL_TYPE_TOTAL2;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_TOTAL2;

    u4AheadType = FBM_POOL_TYPE_FEEDER;

    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif

#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif
    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_DMX2_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX3_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

#if defined(CC_FBM_SUPPORT_VENC_OUT)
    prPoolList[FBM_POOL_TYPE_VENC_OUT].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC_OUT].u4PoolSize = FBM_VENC_OUT_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC_OUT].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC_OUT].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC_OUT;
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_4K_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG1_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;


    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML2_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;


    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG2_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML3_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG3_4K_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML4_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG4_4K_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B;

#endif

    // MPEG_C|MJC|OD|PNG|JPEG|VSS|FEEDER|DRM|PQ_3DC|SWDMX
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel
#ifdef CC_BYPASS_MJC
    u4AheadType = FBM_POOL_TYPE_TOTAL3;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C;
#else
    u4AheadType = FBM_POOL_TYPE_TOTAL3;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C;

    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG1_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML1_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML2_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG2_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;

    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML3_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG3_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML4_C_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG4_4K_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_720P_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
#endif

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
#endif
#endif

#if defined(CC_SUPPORT_VENC)
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_4K_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

    prPoolList[FBM_POOL_TYPE_REST_MEM].u4PoolSize = FBM_REST_MEM_POOL_SIZE_4K;
    prPoolList[FBM_POOL_TYPE_REST_MEM].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REST_MEM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REST_MEM;

#endif
}

static VOID _FbmConfigurePool_MT5891_2CH_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)

{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

    //==================Secure FBM===================== begin


#if defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
#endif


    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

#if !defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif
    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;

    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL2;

#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif
    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
#endif

#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

#ifndef CC_BYPASS_MJC
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize = FBM_MJC_RSV_POOL_SIZE;
    LOG(3, "FBMCFG CHANEL B: prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize :%d \n", prPoolList[FBM_POOL_TYPE_MJC_1].u4PoolSize);
    prPoolList[FBM_POOL_TYPE_MJC_1].u4AddrAlign = FBM_MJC_ADDR_ALIGMENT;
    prPoolList[FBM_POOL_TYPE_MJC_1].u4Mode= FBM_POOL_MODE_MJC_Y10_C10_422_1RDC_3FRM;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MJC_1;
#endif


    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;

    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif
#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif


    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif
    //==================Secure FBM===================== end
    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B;


#endif
}
#endif

#if defined(CC_MT5886)
static VOID _FbmConfigurePool_MT5886_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    //==========================================

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;


    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

#if 0
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_SCPOS_MAIN;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;
    //u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif
#endif

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    //#ifdef __MODEL_slt__
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    //#endif
    u4AheadType = FBM_POOL_TYPE_DSP;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    //u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;


#if defined(CC_S_PLATFORM) && (defined(CC_WK_CUST) || defined(CC_HK_CUST))
    u4BehindType = FBM_POOL_TYPE_NS_A;// must align channelA tail
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_RAMOOPS;
#endif


    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL2;

#if defined(CC_FBM_FB_IN_CHANNELB)
    prPoolList[FBM_POOL_TYPE_MTFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTFB].u4PoolSize = FBM_MTFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTFB;
#endif

#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif

#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    // handle Image Viewer Pool
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
#if 0       //
#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif
#endif
    // secure buffer begin
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;


#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX2;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
#endif

    //-----Vdec lax3-----
    prPoolList[FBM_POOL_TYPE_LAX3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX3].u4PoolSize = FBM_LAX3_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX3;


#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax4-----
    prPoolList[FBM_POOL_TYPE_LAX4].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX4].u4PoolSize = FBM_LAX4_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX4;
    u4AheadType = FBM_POOL_TYPE_LAX3;

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

#endif

#if !defined(CC_BYPASS_MJC)
    //-----MJC1-----
    prPoolList[FBM_POOL_TYPE_MJC1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1].u4PoolSize = FBM_POOL_MJC1_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1;

    //-----MJC2-----
    prPoolList[FBM_POOL_TYPE_MJC2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2].u4PoolSize = FBM_POOL_MJC2_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2;

    //-----MJC3-----
    prPoolList[FBM_POOL_TYPE_MJC3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3].u4PoolSize = FBM_POOL_MJC3_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3;

    //-----MJC1_C-----
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4PoolSize = FBM_POOL_MJC1_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1_C;

    //-----MJC2_C-----
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4PoolSize = FBM_POOL_MJC2_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2_C;

    //-----MJC3_C-----
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4PoolSize = FBM_POOL_MJC3_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3_C;

    prPoolList[FBM_POOL_TYPE_DYN_MPEG].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_DYN_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DYN_MPEG].eMode = FBM_POOL_ARRANGE_SHARE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DYN_MPEG].ePool1 = FBM_POOL_TYPE_MJC1;
    //u4AheadType = FBM_POOL_TYPE_DYN_MPEG;

    prPoolList[FBM_POOL_TYPE_DYN_MPEG2].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_DYN_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DYN_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DYN_MPEG2].ePool1 = FBM_POOL_TYPE_DYN_MPEG;
    //u4AheadType = FBM_POOL_TYPE_DYN_MPEG2;
#endif


#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    prPoolList[FBM_POOL_TYPE_PAD].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_PAD].u4PoolSize = FBM_PAD_SIZE; //dynamic size
    prPoolArrangeInfo[FBM_POOL_TYPE_PAD].eMode = FBM_POOL_ARRANGE_BETWEEN;
    prPoolArrangeInfo[FBM_POOL_TYPE_PAD].ePool1 = u4AheadType;
    //u4AheadType = FBM_POOL_TYPE_PAD;

    u4BehindType = FBM_POOL_TYPE_NS;// must align channelB tail
    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].ePool1 = u4BehindType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4BehindType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B;

#endif


    // MPEG_C|MJC|OD|PNG|JPEG|VSS|FEEDER|DRM|PQ_3DC|SWDMX
#if defined(CC_FBM_3ND_CHANNEL) // 3 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL3;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_C;

#if !defined(CC_BYPASS_MJC)
    //-----MJCMV-----
    prPoolList[FBM_POOL_TYPE_MJCMV].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJCMV].u4PoolSize = FBM_POOL_MJCMV_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJCMV;
#endif

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
#endif

    //============secure buffer end==============================

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif


#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_FBM_DFB_IN_CHANNELC)
    prPoolList[FBM_POOL_TYPE_MTDFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTDFB].u4PoolSize = FBM_MTDFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTDFB;
#endif

#endif
}

static VOID _FbmConfigurePool_MT5886_2CH_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)

{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;
#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

    //==================Secure FBM===================== begin


#if defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

    //-----MJC1-----
    prPoolList[FBM_POOL_TYPE_MJC1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1].u4PoolSize = FBM_POOL_MJC1_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1;

    //-----MJC2-----
    prPoolList[FBM_POOL_TYPE_MJC2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2].u4PoolSize = FBM_POOL_MJC2_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2;

    //-----MJC3-----
    prPoolList[FBM_POOL_TYPE_MJC3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3].u4PoolSize = FBM_POOL_MJC3_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3;

    //-----MJCMV-----
    prPoolList[FBM_POOL_TYPE_MJCMV].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJCMV].u4PoolSize = FBM_POOL_MJCMV_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJCMV;


#if !defined(CC_TVP_SUPPORT)
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    //u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;

    //channel B
    //DMX|MPEG|SCPOS_MAIN
#if defined(CC_FBM_2ND_CHANNEL) // 2 Channel

    u4AheadType = FBM_POOL_TYPE_TOTAL2;

    //-----MJC1_C-----
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4PoolSize = FBM_POOL_MJC1_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1_C;

    //-----MJC2_C-----
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4PoolSize = FBM_POOL_MJC2_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2_C;

    //-----MJC3_C-----
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4PoolSize = FBM_POOL_MJC3_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3_C;


#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
#endif
    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;

    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX2;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;

    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;

    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif
#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif


    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif
    //==================Secure FBM===================== end
    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_B;


#endif
}


#endif

#if defined(CC_MT5863) || defined(CC_MT5865)

#if !defined(CC_TVP_SUPPORT)
static VOID _FbmConfigurePool_MT5863_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMX | MPEG | PIP | MDDI | DSP

    u4AheadType = FBM_POOL_TYPE_TOTAL;

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = 0x0;
#endif
    u4AheadType = FBM_POOL_TYPE_DMX;

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    //==========================================

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX2;
    u4AheadType = FBM_POOL_TYPE_LAX1;
#endif

#if defined(CC_FBM_SUPPORT_4K2K)
    //----- MPEG C-----
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- MPEG Y-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y1_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#else
    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#endif

#if defined(CC_FORCE_DRAM256M)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit =  FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 =  FBM_POOL_TYPE_JPG_VDP;
    u4AheadType =  FBM_POOL_TYPE_MPEG;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = FBM_POOL_TYPE_MPEG;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif

    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER;

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = 0;
#endif
    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

#if defined(CC_FBM_SUPPORT_4K2K)
    //----- MPEG2 C-----
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
    //----- MPEG2 Y-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif

#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif
#endif
#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif
#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }
#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if (defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)) && (!defined(CC_FORCE_DRAM256M))
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FORCE_DRAM256M)
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4AheadType = FBM_POOL_TYPE_DSP;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif
#if defined(CC_FBM_PQ_3DC)
    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;
#endif
    u4BehindType = FBM_POOL_TYPE_NS;

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4BehindType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4BehindType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;

#if !defined(CC_FORCE_DRAM256M)
    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4BehindType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4BehindType = FBM_POOL_TYPE_DSP;
#endif

#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_OD;
#endif
#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_DSP_BIN;
#endif

#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize * 2;
#endif

    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_FEEDER;

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = FBM_POOL_TYPE_FEEDER_MMP;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = FBM_POOL_TYPE_JPEG_OSD1;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = FBM_POOL_TYPE_JPEG_OSD2;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = FBM_POOL_TYPE_JPEG_OSD3;

    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#else
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#endif
#else
    u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;
#endif

    // JPEG (Between Start & OSD1)
#if (!defined(CC_FORCE_DRAM256M))
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
#endif

#else
    // JPEG + B2R share DMX + MPEG
    // JPEG
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = FBM_POOL_TYPE_TOTAL;

    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = FBM_POOL_TYPE_JPEG;
#endif
}

#else
static VOID _FbmConfigurePool_MT5863_TVP_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    //==================NoSecure FBM===================

    u4AheadType = FBM_POOL_TYPE_TOTAL;

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    //==========================================

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif


    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE; // A3 needs 8MB for OD dump

    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_PQ_TOOL;
    }

#if defined(CC_53XX_FACE_DETECTION)
    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif
#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif
#if defined(CC_FBM_3D_OSD)||defined(CC_MHP_SUPPORT)
#if (!defined(CC_FBM_CAST_TV_STRINK)) && (!defined(CC_LMP_CUST))
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif
#endif
#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif
#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
#if (!defined(CC_FBM_CAST_TV_STRINK) && !defined(ANDROID))
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
#if defined(CC_SUPPORT_5_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x400000;
#elif defined(CC_SUPPORT_2_SEC_PTS_PCR_OFFSET)
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE + 0x1A0000;
#else
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
#endif
    u4AheadType = FBM_POOL_TYPE_DSP;

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

#if defined(SUPPORT_DIIVA)
    prPoolList[FBM_POOL_TYPE_DIVA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DIVA].u4PoolSize = FBM_SCPOS_8BIT_SIZE_FLIP(1920, 1080, 4);
    prPoolList[FBM_POOL_TYPE_DIVA].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_DIVA].u4Height = 1080;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DIVA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DIVA;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_DUAL_3D_BROADCASTING)
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_IMGRZ_3D_KR].u4PoolSize = FBM_IMGRZ_3D_KR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_IMGRZ_3D_KR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_IMGRZ_3D_KR;
#endif

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_FBM_SUPPORT_MUXER)
    prPoolList[FBM_POOL_TYPE_MUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MUXER].u4PoolSize = FBM_MUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MUXER;
#endif
#if defined(CC_FBM_PQ_3DC)
    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;
#endif
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER;


    // handle Image Viewer Pool
#ifdef CC_OSD_USE_FBM
    // JPEG | OSD1 | OSD2 | OSD3 | OSD4 (Share all except DSP)
    //
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
#if 0//defined(CC_FBM_INT_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize * 2;
#endif
#ifdef CC_FBM_PQ_3DC
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_PQ_3DC;
#else
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = FBM_POOL_TYPE_FEEDER;
#endif

    if(prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize >= prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize)
        u4AheadType = FBM_POOL_TYPE_FEEDER;
    else
        u4AheadType = FBM_POOL_TYPE_FEEDER_MMP;

    // JPEG (Between Start & OSD1)
#if (!defined(CC_FBM_CAST_TV_STRINK)) && (!defined(CC_LMP_CUST))
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#ifndef CC_FBM_3D_OSD
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD1].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD1;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD2].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD2;
#if !defined(CC_REMOVE_JPEG_OSD34)
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD3].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD3;

    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4PoolSize = FBM_OSD_POOL_SIZE(1920, 1088, 1, 4); // FHD, 32bpp, x1
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Width = 1920;
    prPoolList[FBM_POOL_TYPE_JPEG_OSD4].u4Height = 1088;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG_OSD4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG_OSD4;
#endif
#endif

#else
    // JPEG
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

    //==================Secure FBM=====================

#if defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;

    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_DMX_H264_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;

    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = 0x1780000;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;
#endif


#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    if (u4VdoBehindType == FBM_POOL_TYPE_NS)
    {
        u4VdoBehindType = FBM_POOL_TYPE_DMX2;
    }
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    {
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);

#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
        prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = 0;
#endif
    }
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;

#if defined(CC_FBM_SUPPORT_4K2K)
    //----- MPEG2 C-----
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
    //----- MPEG2 Y-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
#if defined(CC_VOMX_TV_COEXIST)
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK);
    //prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = 0x2F00000; // 23.5Mx2
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#else
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_2_SIZE; // Jason rewrite
#endif
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;
#endif

#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif


#ifndef CC_OSD_USE_FBM
    // B2R
    prPoolList[FBM_POOL_TYPE_B2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_B2R].u4PoolSize = FBM_B2R_POOL_SIZE(1920, 1088, 2, 2); // FHD, 422, x2
    prPoolList[FBM_POOL_TYPE_B2R].u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_2FB;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_B2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_B2R;
#endif


    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_DMX1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
#if defined(CC_TRUSTZONE_SUPPORT)&& defined(CC_SVP_SUPPORT)
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = 0x0;
#endif
    u4AheadType = FBM_POOL_TYPE_DMX;

#if !defined(CC_SUPPORT_NPTV_SEAMLESS)
    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;
#endif

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

#if defined(CC_FBM_TWO_FBP_SHARED_WITH_DFB) || defined(CC_VOMX_TV_COEXIST)
    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX2;
    u4AheadType = FBM_POOL_TYPE_LAX1;
#endif

#if defined(CC_FBM_SUPPORT_4K2K)
    //----- MPEG C-----
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C1_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- MPEG Y-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y1_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#else
    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;
#endif

#if defined(CC_FBM_CAST_TV_STRINK) || defined(CC_LMP_CUST)
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit =  FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 =  FBM_POOL_TYPE_JPG_VDP;
    u4AheadType =  FBM_POOL_TYPE_MPEG;
#endif

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
#if defined(CC_SUPPORT_SCART_OUT_IN_MMP)
    // allocate 7MB for sub modules, no consider flip/mirror
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2F00000;
#endif

    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    }
    else // for ic-verification
    {
        prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2400000 + 0x80000; /* add extra 500K to avoid sub buffer overlapped with dsp in SLT environment */
    }

    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
#if defined(CC_IC_VERIFY_FBM_MODE)
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = 0x2b00000;
#endif
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

#if defined(CC_SUPPORT_MONITOR_OUT_TVE)
    // ----TVE----
    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;
#endif


#ifdef CC_SUPPORT_OD    //if (DRVCUST_OptGet(eFlagSupportOD) == TRUE)
    prPoolList[FBM_POOL_TYPE_OD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_OD].u4PoolSize = FBM_OD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_OD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_OD;
#endif

    //-----hot boot dram calibration-----
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A].u4PoolSize = FBM_DRAMHOTBOOTCALIBR_SIZE;
    u4AheadType = FBM_POOL_TYPE_DRAMHOTBOOTCALIBR_A;


}
#endif //!defined(CC_TVP_SUPPORT)

#endif//defined(CC_MT5863)

#if defined(CC_MT5893)
static VOID _FbmConfigurePool_MT5893_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;

#if defined(CC_FBM_FB_IN_CHANNELB)
    prPoolList[FBM_POOL_TYPE_MTFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTFB].u4PoolSize = FBM_MTFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTFB;
#endif

#if defined(CC_FBM_DFB_IN_CHANNELC)
    prPoolList[FBM_POOL_TYPE_MTDFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTDFB].u4PoolSize = FBM_MTDFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTDFB;
#endif

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    //==========================================


#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;


    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    //#ifdef __MODEL_slt__
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    //#endif
    u4AheadType = FBM_POOL_TYPE_DSP;


#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif


#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif
#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;


#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

    // secure buffer begin



    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;


#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    //u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

#ifdef CC_SUPPORT_PHOTO8K
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#endif

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
#ifdef CC_SUPPORT_PHOTO8K
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif
#endif
    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

    prPoolList[FBM_POOL_TYPE_DTCPIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DTCPIP].u4PoolSize = FBM_DTCPIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DTCPIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DTCPIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DTCPIP;

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;


#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX2;
    u4AheadType = FBM_POOL_TYPE_LAX1;
#endif

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;
#endif

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
#endif

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    //-----Vdec lax3-----
    prPoolList[FBM_POOL_TYPE_LAX3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX3].u4PoolSize = FBM_LAX3_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX3;


#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax4-----
    prPoolList[FBM_POOL_TYPE_LAX4].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX4].u4PoolSize = FBM_LAX4_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX4;
    u4AheadType = FBM_POOL_TYPE_LAX3;
#endif
#endif

#ifdef CC_FBM_DYNMJC
    if(SRMFBM_GetConf() != FBM_MEM_CFG_MT5893_3DTV_NOMJC)
#endif
    {
        //-----MJC1-----
        prPoolList[FBM_POOL_TYPE_MJC1].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC1].u4PoolSize = FBM_POOL_MJC1_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC1;

        //-----MJC2-----
        prPoolList[FBM_POOL_TYPE_MJC2].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC2].u4PoolSize = FBM_POOL_MJC2_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC2;
 
        //-----MJC3-----
        prPoolList[FBM_POOL_TYPE_MJC3].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC3].u4PoolSize = FBM_POOL_MJC3_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC3;

        //-----MJC1_C-----
        prPoolList[FBM_POOL_TYPE_MJC1_C].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC1_C].u4PoolSize = FBM_POOL_MJC1_C_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC1_C;

        //-----MJC2_C-----
        prPoolList[FBM_POOL_TYPE_MJC2_C].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC2_C].u4PoolSize = FBM_POOL_MJC2_C_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC2_C;

        //-----MJC3_C-----
        prPoolList[FBM_POOL_TYPE_MJC3_C].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJC3_C].u4PoolSize = FBM_POOL_MJC3_C_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJC3_C;

        //-----MJCMV-----
        prPoolList[FBM_POOL_TYPE_MJCMV].u4Inherit = FBM_POOL_ROOT;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].eMode = FBM_POOL_ARRANGE_AFTER;
        prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].ePool1 = u4AheadType;
        prPoolList[FBM_POOL_TYPE_MJCMV].u4PoolSize = FBM_POOL_MJCMV_SIZE;
        u4AheadType = FBM_POOL_TYPE_MJCMV;
    }

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;
#ifndef CC_SUPPORT_PHOTO8K
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#endif
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;

#ifndef CC_SUPPORT_PHOTO8K
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
#endif
#if defined(CC_RAMOOPS_SUPPORT)
    u4BehindType = FBM_POOL_TYPE_NS_A;// must align FBM tail
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_RAMOOPS;
#endif
    //============secure buffer end==============================
}

static VOID _FbmConfigurePool_MT5893_4KMainOnly_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;

#if defined(CC_FBM_FB_IN_CHANNELB)
    prPoolList[FBM_POOL_TYPE_MTFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTFB].u4PoolSize = FBM_MTFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTFB;
#endif

#if defined(CC_FBM_DFB_IN_CHANNELC)
    prPoolList[FBM_POOL_TYPE_MTDFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTDFB].u4PoolSize = FBM_MTDFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTDFB;
#endif

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    //==========================================


#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;

#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CHA].u4PoolSize = FBM_W2DR2R_POOL_CHA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CHA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CHA;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    //#ifdef __MODEL_slt__
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    //#endif
    u4AheadType = FBM_POOL_TYPE_DSP;


#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif


#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
#endif
    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;

#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif


#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

    // secure buffer begin
    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;

    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;

    //-----MJC1-----
    prPoolList[FBM_POOL_TYPE_MJC1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1].u4PoolSize = FBM_POOL_MJC1_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1;

    //-----MJC2-----
    prPoolList[FBM_POOL_TYPE_MJC2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2].u4PoolSize = FBM_POOL_MJC2_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2;

    //-----MJC3-----
    prPoolList[FBM_POOL_TYPE_MJC3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3].u4PoolSize = FBM_POOL_MJC3_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3;

    //-----MJC1_C-----
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC1_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC1_C].u4PoolSize = FBM_POOL_MJC1_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC1_C;

    //-----MJC2_C-----
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC2_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC2_C].u4PoolSize = FBM_POOL_MJC2_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC2_C;

    //-----MJC3_C-----
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC3_C].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJC3_C].u4PoolSize = FBM_POOL_MJC3_C_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJC3_C;


#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

    //-----MJCMV-----
    prPoolList[FBM_POOL_TYPE_MJCMV].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCMV].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJCMV].u4PoolSize = FBM_POOL_MJCMV_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJCMV;

    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)

    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

#if defined(CC_RAMOOPS_SUPPORT)
    u4BehindType = FBM_POOL_TYPE_NS_A;// must align FBM tail
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_RAMOOPS;
#endif
    //============secure buffer end==============================
}
#endif

#if defined(CC_MT5895)
static VOID _FbmConfigurePool_MT5895_MJC_SCPOS_MAIN(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
    FBM_POOL_TYPE_T u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN, u4BehindType, u4VdoBehindType = FBM_POOL_TYPE_NS;
    UNUSED(u4AheadType);
    UNUSED(u4BehindType);
    UNUSED(u4VdoBehindType);
    // arrange root pool ---------------------------------------------------

    // DMXPID |DSP|VBI|PVR|VENC|SCPOS_SUB|

    u4AheadType = FBM_POOL_TYPE_TOTAL;

#if defined(CC_FBM_FB_IN_CHANNELB)
    prPoolList[FBM_POOL_TYPE_MTFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTFB].u4PoolSize = FBM_MTFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTFB;
#endif

#if defined(CC_FBM_DFB_IN_CHANNELC)
    prPoolList[FBM_POOL_TYPE_MTDFB].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MTDFB].u4PoolSize = FBM_MTDFB_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MTDFB].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MTDFB;
#endif

    //share memory layout====================
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMEWS].u4PoolSize = FBM_SHMEWS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMEWS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMEWS;

    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMTSRQUE].u4PoolSize = FBM_SHMTSRQUE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMTSRQUE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMTSRQUE;

    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMISDBCC].u4PoolSize = FBM_SHMISDBCC_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMISDBCC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMISDBCC;

    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMSSMA].u4PoolSize = FBM_SHMSSMA_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMSSMA].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMSSMA;

    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMVBIF].u4PoolSize = FBM_SHMVBIF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMVBIF].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMVBIF;

    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMATVTS].u4PoolSize = FBM_SHMATVTS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMATVTS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMATVTS;

    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SHMMUXER].u4PoolSize = FBM_SHMMUXER_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SHMMUXER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SHMMUXER;

    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_THUMBNAIL].u4PoolSize = FBM_THUMBNAIL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_THUMBNAIL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_THUMBNAIL;

    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUNDCLIP].u4PoolSize = FBM_SOUNDCLIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUNDCLIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUNDCLIP;

    prPoolList[FBM_POOL_TYPE_HDMI_SHM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HDMI_SHM].u4PoolSize = FBM_HDMI_SHM_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HDMI_SHM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HDMI_SHM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HDMI_SHM;

    prPoolList[FBM_POOL_TYPE_SOUND_MODEL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SOUND_MODEL].u4PoolSize = FBM_SOUND_MODEL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUND_MODEL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SOUND_MODEL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SOUND_MODEL;

    //==========================================

#if defined(CC_FBM_SUPPORT_ATSC30)
    prPoolList[FBM_POOL_TYPE_ATSC30ALP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_ATSC30ALP].u4PoolSize = FBM_ATSC30ALP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_ATSC30ALP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_ATSC30ALP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_ATSC30ALP;
#endif

#if defined(CC_FBM_SUPPORT_DMXPID)
    prPoolList[FBM_POOL_TYPE_DMXPID].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMXPID].u4PoolSize = FBM_DMX_PID_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMXPID].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMXPID;
#endif

    prPoolList[FBM_POOL_TYPE_FACEDET].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FACEDET].u4PoolSize = FBM_FD_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FACEDET].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FACEDET;

    // vbi
    prPoolList[FBM_POOL_TYPE_VBI].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VBI].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VBI;

#if defined(CC_FBM_HW_DEMOD)
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_HW_DEMOD].u4PoolSize = FBM_HW_DEMOD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_HW_DEMOD].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_HW_DEMOD;
#endif

#ifdef CC_SUPPORT_RECORD_AV
    prPoolList[FBM_POOL_TYPE_TDC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TDC].u4PoolSize = FBM_TDCAV_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TDC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TDC;
#endif

    prPoolList[FBM_POOL_TYPE_TVE].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_TVE].u4PoolSize = FBM_TVE_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_TVE].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_TVE;


    //-----PQ_3DC-----
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_3DC].u4PoolSize = FBM_PQ_3DC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_3DC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_3DC;

#if defined(CC_FBM_SUPPORT_SWDMX)||defined(CC_FBM_SUPPORT_LITE_SWDMX)
    prPoolList[FBM_POOL_TYPE_SWDMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SWDMX].u4PoolSize = FBM_SWDMX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SWDMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SWDMX;
#endif

#if !defined(__MODEL_slt__)
    prPoolList[FBM_POOL_TYPE_FEEDER].u4PoolSize = FBM_FEEDER1_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER].ePool1 = u4AheadType;

    prPoolList[FBM_POOL_TYPE_FEEDER_MMP].u4PoolSize = FBM_FEEDER_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER_MMP].ePool1 = u4AheadType;

    u4AheadType = FBM_POOL_TYPE_FEEDER;
#if defined(CC_FBM_TWO_FBP)
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4PoolSize = FBM_FEEDER2_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER2].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER2;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4PoolSize = FBM_FEEDER3_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER3;

    prPoolList[FBM_POOL_TYPE_FEEDER4].u4PoolSize = FBM_FEEDER4_SIZE;
    prPoolList[FBM_POOL_TYPE_FEEDER4].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FEEDER4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FEEDER4;
#endif

#endif


    prPoolList[FBM_POOL_TYPE_DRM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DRM].u4PoolSize = FBM_DRM_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DRM].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DRM;

#if defined(CC_SUPPORT_VENC)
    prPoolList[FBM_POOL_TYPE_VENC].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VENC].u4PoolSize = FBM_VENC_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VENC].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VENC;
#endif

#ifdef ADSP_BIN_SUPPORT
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DSP_BIN].u4PoolSize = FBM_DSP_BIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP_BIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DSP_BIN;
#endif

    //-----DSP-----
    prPoolList[FBM_POOL_TYPE_DSP].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DSP].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_DSP].u4PoolSize = FBM_DSP_POOL_SIZE;
    //#ifdef __MODEL_slt__
    prPoolList[FBM_POOL_TYPE_DSP].u4AddrAlign= (0x1000000-1);
    //#endif
    u4AheadType = FBM_POOL_TYPE_DSP;


#ifdef __MODEL_slt__
    //-----SLT_GFX-----
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SLT_GFX].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_SLT_GFX].u4PoolSize = FBM_SLT_GFX_POOL_SIZE;
    u4AheadType = FBM_POOL_TYPE_SLT_GFX;
#endif


#if defined(CC_FBM_SUPPORT_GFX)
    prPoolList[FBM_POOL_TYPE_GFX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_GFX].u4PoolSize = FBM_GFX_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_GFX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_GFX;
#endif

    prPoolList[FBM_POOL_TYPE_FILM_GRAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_FILM_GRAIN].u4PoolSize = FBM_FILM_GRAIN_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_FILM_GRAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_FILM_GRAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_FILM_GRAIN;

#if defined(CC_SUPPORT_CIPLUS14)
    //----- Remux-----
    prPoolList[FBM_POOL_TYPE_REMUX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_REMUX].u4PoolSize = FBM_REMUX_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_REMUX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_REMUX;
#endif
#if defined(CC_FBM_SUPPORT_SRS)
    //----- SRS-----
    prPoolList[FBM_POOL_TYPE_SRS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SRS].u4PoolSize = FBM_SRS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SRS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SRS;
#endif
#if defined(CC_FBM_3D_OSD) || defined(CC_MHP_SUPPORT) //MHP use jpeg_vdp buffer
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPG_VDP].u4PoolSize = FBM_JPEVDP_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPG_VDP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPG_VDP;
#endif

    // pq tool
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PQ_TOOL].u4PoolSize = FBM_PQ_TOOL_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PQ_TOOL].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PQ_TOOL;


#if 1
    prPoolList[FBM_POOL_TYPE_JPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_JPEG].u4PoolSize = FBM_JPEG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_JPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_JPEG;
#endif

#if defined(CC_FBM_SUPPORT_PNG)
    prPoolList[FBM_POOL_TYPE_PNG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PNG].u4PoolSize = FBM_PNG_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PNG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PNG;
#endif

#if defined(CC_SUPPORT_W2DR2R)
    //----- W2D R2R-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R].u4PoolSize = FBM_W2DR2R_V2_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R;
    //----- W2D R2R Y-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_Y].u4PoolSize = FBM_W2DR2R_POOL_V2_Y_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_Y].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_Y;
    //----- W2D R2R CR-----
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_W2D_R2R_CR].u4PoolSize = FBM_W2DR2R_POOL_V2_CR_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_W2D_R2R_CR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_W2D_R2R_CR;
#endif

#if defined(CC_FBM_SUPPORT_PVR)
    prPoolList[FBM_POOL_TYPE_PVR].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_PVR].u4PoolSize = FBM_PVR_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_PVR].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_PVR;
#endif

#if defined(CC_SUPPORT_VSS)
    prPoolList[FBM_POOL_TYPE_VSS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_VSS].u4PoolSize = FBM_VSS_POOL_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_VSS].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_VSS;
#endif

    // secure buffer begin



    //----- SCPOS MAIN-----
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_MAIN].u4PoolSize = FBM_SCPOS_MAIN_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_MAIN].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_MAIN;

    //----- SCPOS PIP-----
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SCPOS_PIP].u4PoolSize = FBM_SCPOS_SUB_SIZE; //main 1080i 422 0xe9c0000*2+sub 0x880000
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SCPOS_PIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SCPOS_PIP;


#ifdef CC_S_PLATFORM
#if defined(CC_FBM_SUPPORT_BEIC)
    prPoolList[FBM_POOL_TYPE_BEIC].u4Inherit = FBM_POOL_INHERIT;
    prPoolList[FBM_POOL_TYPE_BEIC].u4PoolSize = FBM_BEIC_BUF_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_BEIC].ePool1 = u4AheadType;
    //u4AheadType = FBM_POOL_TYPE_BEIC;
#endif
#endif

    //----- MPEG-----
    prPoolList[FBM_POOL_TYPE_MPEG].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG].u4PoolSize = FBM_MPEG_Y_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG;

    prPoolList[FBM_POOL_TYPE_MPEG_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG_C].u4PoolSize = FBM_MPEG_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG_C;

    //----- MPEG2-----
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4PoolSize = FBM_MPEG_Y2_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG2].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG2_C].u4PoolSize = FBM_MPEG_C2_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG2_C;
#endif

    //----- DMX-----
    prPoolList[FBM_POOL_TYPE_DMX].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX1_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX;
#if defined(CC_FBM_TWO_FBP)
    //----- DMX2-----
    prPoolList[FBM_POOL_TYPE_DMX2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX2].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX2_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX2;
#endif
#if defined(CC_FBM_TWO_4K2K)|| defined(CC_FBM_FOUR_INST_SUPPORT)
    //----- DMX3-----
    prPoolList[FBM_POOL_TYPE_DMX3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX3].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX3_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX3;
    //----- DMX4-----
    prPoolList[FBM_POOL_TYPE_DMX4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DMX4].u4PoolSize = FBM_ALIGN_MASK((FBM_DMX4_SIZE), FBM_DMX_SIZE_ALIGMENT);
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DMX4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DMX4;
#endif

    prPoolList[FBM_POOL_TYPE_DTCPIP].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_DTCPIP].u4PoolSize = FBM_DTCPIP_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_DTCPIP].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_DTCPIP].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_DTCPIP;

    //----- SEAMLESS1-----
    prPoolList[FBM_POOL_TYPE_SEAML1].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1;

    //-----Vdec lax1-----
    prPoolList[FBM_POOL_TYPE_LAX1].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX1].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX1].u4PoolSize = FBM_LAX1_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX1;


#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax2-----
    prPoolList[FBM_POOL_TYPE_LAX2].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX2].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX2].u4PoolSize = FBM_LAX2_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX2;
    u4AheadType = FBM_POOL_TYPE_LAX1;
#endif

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2-----
    prPoolList[FBM_POOL_TYPE_SEAML2].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2;
#endif
#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
#ifdef CC_FBM_DYNMJC
    if(SRMFBM_GetConf() != FBM_MEM_CFG_MT5895_3DTV_MJC)
#endif
    {
#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS3-----
    prPoolList[FBM_POOL_TYPE_SEAML3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3;
#endif

    //----- MPEG3-----
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4PoolSize = FBM_MPEG_Y3_SIZE;
    prPoolList[FBM_POOL_TYPE_MPEG3].u4Mode = (FBM_POOL_MODE_MPEG_DBK | FBM_POOL_MODE_SHARE);
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3;

#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS4-----
    prPoolList[FBM_POOL_TYPE_SEAML4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4].u4PoolSize = FBM_SEAML_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4;
#endif

    // -----MPEG4-----
    prPoolList[FBM_POOL_TYPE_MPEG4].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4].u4PoolSize = FBM_MPEG_Y4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4;

    //-----Vdec lax3-----
    prPoolList[FBM_POOL_TYPE_LAX3].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX3].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX3].u4PoolSize = FBM_LAX3_SIZE;
    u4AheadType = FBM_POOL_TYPE_LAX3;

#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //-----Vdec lax4-----
    prPoolList[FBM_POOL_TYPE_LAX4].u4Inherit = FBM_POOL_INHERIT;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].eMode = FBM_POOL_ARRANGE_ALLIGN_END;
    prPoolArrangeInfo[FBM_POOL_TYPE_LAX4].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_LAX4].u4PoolSize = FBM_LAX4_SHARE_SIZE;
    //u4AheadType = FBM_POOL_TYPE_LAX4;
    u4AheadType = FBM_POOL_TYPE_LAX3;
#endif
    }
#endif

#if defined(CC_FBM_SECURE_FEEDER)
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4PoolSize = FBM_SECURE_FEEDER_SIZE;
    prPoolList[FBM_POOL_TYPE_SECURE_FEEDER].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SECURE_FEEDER].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SECURE_FEEDER;
#endif

#ifdef CC_FBM_DYNMJC
    if(SRMFBM_GetConf() != FBM_MEM_CFG_MT5895_3DTV_NOMJC)
#endif
    {
    //-----MJCALL-----
    prPoolList[FBM_POOL_TYPE_MJCALL].u4Inherit = FBM_POOL_ROOT;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCALL].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJCALL].ePool1 = u4AheadType;
    prPoolList[FBM_POOL_TYPE_MJCALL].u4PoolSize = FBM_POOL_MJCALL_SIZE;
    u4AheadType = FBM_POOL_TYPE_MJCALL;
    }
    //----- SEAMLESS1_C-----
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML1_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML1_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML1_C;
#if defined(CC_FBM_TWO_FBP)||defined(CC_FBM_SUPPORT_4K2K)
    //----- SEAMLESS2_C-----
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML2_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML2_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML2_C;
#endif

#if defined(CC_FBM_TWO_4K2K)||defined(CC_FBM_FOUR_INST_SUPPORT)
#ifdef CC_FBM_DYNMJC
    if(SRMFBM_GetConf() != FBM_MEM_CFG_MT5895_3DTV_MJC)
#endif
    {
#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS3_C-----
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML3_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML3_C;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG3_C].u4PoolSize = FBM_MPEG_C3_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG3_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG3_C;

#if !defined(CC_SUPPORT_DOLBY_VISION)
    //----- SEAMLESS4_C-----
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_SEAML4_C].u4PoolSize = FBM_SEAML_C_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_SEAML4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_SEAML4_C;
#endif

    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MPEG4_C].u4PoolSize = FBM_MPEG_C4_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].eMode = FBM_POOL_ARRANGE_AFTER;
    prPoolArrangeInfo[FBM_POOL_TYPE_MPEG4_C].ePool1 = u4AheadType;
    u4AheadType = FBM_POOL_TYPE_MPEG4_C;
    }
#endif

#ifdef CC_FBM_DYNMJC
    if(SRMFBM_GetConf() != FBM_MEM_CFG_MT5895_3DTV_NOMJC)
#endif
    {
#if defined(CC_RAMOOPS_SUPPORT)
    u4BehindType = FBM_POOL_TYPE_MJC_R20;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
#endif
    //============secure buffer end==============================
    //============R2 buffer =====================================
    //
    // Note:
    // R2 buffer should be the end of FBM, near the TZ buffer
    // because we should known the R20/R21 start address in loader
    //
    u4BehindType = FBM_POOL_TYPE_MJC_R21;
    prPoolList[FBM_POOL_TYPE_MJC_R20].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_R20].u4PoolSize = FBM_MJC_R20_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R20].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R20].ePool1 = u4BehindType;

    u4BehindType = FBM_POOL_TYPE_LD;
    prPoolList[FBM_POOL_TYPE_MJC_R21].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_R21].u4PoolSize = FBM_MJC_R21_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R21].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R21].ePool1 = u4BehindType;

    u4BehindType = FBM_POOL_TYPE_3DLUT;
    prPoolList[FBM_POOL_TYPE_LD].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_LD].u4PoolSize = FBM_LD_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_LD].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_LD].ePool1 = u4BehindType;

    u4BehindType = FBM_POOL_TYPE_MJC_R20_SHM;
    prPoolList[FBM_POOL_TYPE_3DLUT].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_3DLUT].u4PoolSize = FBM_3DLUT_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_3DLUT].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_3DLUT].ePool1 = u4BehindType;

    u4BehindType = FBM_POOL_TYPE_NS_A; // must align FBM tail
    prPoolList[FBM_POOL_TYPE_MJC_R20_SHM].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_MJC_R20_SHM].u4PoolSize = FBM_MJC_R20_SHM_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R20_SHM].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_MJC_R20_SHM].ePool1 = u4BehindType;
    //============R2 buffer end =================================
    }
#ifdef CC_FBM_DYNMJC
    else
    {
    #if defined(CC_RAMOOPS_SUPPORT)
    u4BehindType = FBM_POOL_TYPE_NS_A;// must align FBM tail
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4Inherit = FBM_POOL_ROOT;
    prPoolList[FBM_POOL_TYPE_RAMOOPS].u4PoolSize = FBM_RAMOOPS_SIZE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].eMode = FBM_POOL_ARRANGE_BEFORE;
    prPoolArrangeInfo[FBM_POOL_TYPE_RAMOOPS].ePool1 = u4BehindType;
    u4BehindType = FBM_POOL_TYPE_RAMOOPS;
    #endif
    }
#endif
}
#endif



#define UPDATE_PRPOOL(type, module) \
    do {\
        u4NextDramAddr = FBM_ALIGN_MASK(u4NextDramAddr, FBM_ ## module ## _ADDR_ALIGMENT);\
        prPool = FBM_GetPoolInfo(type);\
        prPool->u4Addr = u4NextDramAddr;\
        prPool->u4Size = u4Size;\
        prPool->u4Width = u4Width;\
        prPool->u4Height = u4Height;\
        prPool->u4Mode = u4Mode;\
        prPool->u4Inherit = FBM_POOL_INHERIT;\
        u4NextDramAddr = prPool->u4Addr + prPool->u4Size;\
    } while (0)

#define UPDATE_POP_WIDTH() \
    do {\
        if (env->u1fgPOP)\
        {\
            u4Width = MIN(u4Width,u4HPDSize/2);\
        }\
    } while (0)

#if defined(CC_MT5891) || defined(CC_MT5886)|| defined(CC_MT5893) || defined(CC_MT5895)
#define UPDATE_DI_FIFO_WIDTH(u4VdpId) \
    do {\
        if (fgFBM_MDDI_FIFO_LIMIT(u4VdpId))\
        {\
            if (env->u1fgPOP)\
            {\
                u4DIFIFO = FBM_DI_POP_FIFO_WIDTH;\
            }\
            else if (VDP_1 == u4VdpId)\
            {\
                u4DIFIFO = 3840;\
                if (E_TD_IN_LI_P == TD_MIB_IN(u4DIMode))\
                {\
                    u4DIFIFO = MIB_LI_INPUT_UPPER_BOUND;\
                }\
            }\
            else\
            {\
                u4DIFIFO = 1920;\
            }\
        }\
        else\
        {\
            u4DIFIFO = 65535;\
        }\
    } while (0)
#else
#define UPDATE_DI_FIFO_WIDTH(u4VdpId) \
    do {\
        if (fgFBM_MDDI_FIFO_LIMIT(u4VdpId))\
        {\
            if (env->u1fgPOP)\
            {\
                u4DIFIFO = FBM_DI_POP_FIFO_WIDTH;\
            }\
            else if ( (u4ScalerMode & FBM_POOL_MODE_SCPOS_HYBRID_DISP) ||\
                    (u4ScalerMode & FBM_POOL_MODE_SCPOS_DUAL_HYBRID_DISP) )\
            {\
                u4DIFIFO = FBM_DI_HYBRID_WIDTH;\
            }\
            else\
            {\
                u4DIFIFO = FBM_DI_PIP_FIFO_WIDTH;\
            }\
        }\
        else\
        {\
            u4DIFIFO = 65535;\
        }\
    } while (0)
#endif
#define UPDATE_POP_HEIGHT() \
    do {\
        if (env->u1fgPOP)\
        {\
            u4Height /= 2;\
        }\
    } while (0)

//Check Panel Display Size
#define UPDATE_HPD_WIDTH()\
    do {\
        if (u4VdpId == VDP_1)\
        {\
            if(u4HPDSize != 0)\
            {\
                u4Width = MIN(u4HPDSize,u4Width);\
            }\
        }\
    } while (0)

#define CALCULATE_PSCAN_NR_NUM()\
    do {\
        if (u4MddiMode & FBM_POOL_MODE_MDDI_DISP)\
        u1NRNum = 3;\
        else if ((u4MddiMode & FBM_POOL_MODE_MDDI_HYBRID) || (u4MddiMode & FBM_POOL_MODE_MDDI_DUAL_HYBRID))\
        {\
            u1NRNum = FBM_POOL_HYBRID_DRAM_FRAME_NUM;\
            if (u4FlipModule & FLIP_BY_SCALER)\
            u1NRNum += 1;\
        }\
        else if (u4MddiMode & FBM_POOL_MODE_MDDI_NR_Y_C)\
        u1NRNum = 1;\
        else\
        u1NRNum = 0;\
        if(E_TDTV_DI_DRAM_DOUBLE==u4PscanModeEx)\
        u1NRNum *= 2;\
        if(E_TDTV_DI_DRAM_0FB==u4PscanModeEx)\
        u1NRNum = 0;\
        if(fgFBM_PscanTest)\
        u1NRNum = 10;\
    }while(0)

#define CALCULATE_PROG_DISP_444_SIZE()\
    do {\
        CALCULATE_PSCAN_NR_NUM();\
        if(u4MddiMode & FBM_POOL_MODE_MDDI_MAIN_UFO)\
        {\
            u4Size = ((u4Width * u4Height)/16 + 1) * 4 * u1NRNum * 128 /8;\
            u4Size += ((u4Width /2048 + 1) * u4Height * 4 * u1NRNum * 128 /8);\
        }\
        else\
        {\
            if (u4MddiMode & FBM_POOL_MODE_10BIT)\
            {\
                u4Size = FBM_MDDI_BI_NR_10BIT_444_SIZE(u4Width, u4Height, u1NRNum);\
            }\
            else\
            {\
                u4Size = FBM_MDDI_BI_NR_8BIT_444_SIZE(u4Width, u4Height, u1NRNum);\
            }\
        }\
    } while (0)


#define CALCULATE_PSCAN_NR_YC_SIZE_3D_422() \
    do {\
        CALCULATE_PSCAN_NR_NUM();\
        if (u4MddiMode & FBM_POOL_MODE_10BIT)\
        {\
            u4Size = FBM_MDDI_BI_NR_10BIT_SIZE(u4Width, u4Height, u1NRNum);\
        }\
        else\
        {\
            u4Size = FBM_MDDI_BI_NR_8BIT_SIZE(u4Width, u4Height, u1NRNum);\
        }\
    } while (0)

#define CALCULATE_PSCAN_NR_YC_SIZE_3D_420() \
    do {\
        CALCULATE_PSCAN_NR_NUM();\
        if (u4MddiMode & FBM_POOL_MODE_10BIT)\
        {\
            if(u4MddiMode & FBM_POOL_MODE_MDDI_NR_OFF)\
            u4Size = FBM_MDDI_BI_NR_OFF_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);\
            else \
            u4Size = FBM_MDDI_BI_NR_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);\
        }\
        else\
        {\
            if(u4MddiMode & FBM_POOL_MODE_MDDI_NR_OFF)\
            u4Size = FBM_MDDI_BI_NR_OFF_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);\
            else \
            u4Size = FBM_MDDI_BI_NR_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);\
        }\
    } while (0)

#define CALCULATE_PSCAN_NR_YC_SIZE_3D()\
    do {\
        if (env->u1IsMPEG[u4VdpId]&&(!(u4MddiMode & FBM_POOL_MODE_MDDI_HYBRID))&&(!(u4MddiMode & FBM_POOL_MODE_MDDI_DUAL_HYBRID)))\
        {\
            CALCULATE_PSCAN_NR_YC_SIZE_3D_420();\
        }\
        else\
        {\
            CALCULATE_PSCAN_NR_YC_SIZE_3D_422();\
        }\
        if(0==u1NRNum)\
        u4Size = 0;\
    }while (0)

#define CALCULATE_SCALER_444_PARAMETERS() \
    do {\
        u1ScalerNum = 3;\
        if (u4FlipModule & FLIP_BY_SCALER)\
        {\
            u1ScalerNum += 1;\
        }\
        if (u4FlipModule & FLIP_BY_3D)\
        {\
            u1ScalerNum += 3;\
        }\
        if (u4ScalerMode & FBM_POOL_MODE_10BIT)\
        {\
            u4Size = FBM_SCPOS_444_10BIT_POOL_SIZE_FLIP(u4Width, u4Height, u1ScalerNum);\
        }\
        else\
        {\
            u4Size = FBM_SCPOS_444_8BIT_POOL_SIZE_FLIP(u4Width, u4Height, u1ScalerNum);\
        }\
        u4Mode = FBM_POOL_MODE_444_FB | FBM_POOL_MODE_SCPOS_3FB;\
        if (u4ScalerMode & FBM_POOL_MODE_10BIT)\
        {\
            u4Mode |= FBM_POOL_MODE_10BIT;\
        }\
        else\
        {\
            u4Mode |= FBM_POOL_MODE_8BIT;\
        }\
    } while (0)
#define CALCULATE_SCALER_422_PARAMETERS() \
    do {\
        u1ScalerNum = 3;\
        if (u4FlipModule & FLIP_BY_SCALER)\
        {\
            u1ScalerNum += 1;\
        }\
        if (u4FlipModule & FLIP_BY_3D)\
        {\
            u1ScalerNum += 3;\
        }\
        if (u4ScalerMode & FBM_POOL_MODE_10BIT)\
        {\
            u4Size = FBM_SCPOS_10BIT_SIZE_FLIP(u4Width, u4Height, u1ScalerNum);\
        }\
        else\
        {\
            u4Size = FBM_SCPOS_8BIT_SIZE_FLIP(u4Width, u4Height, u1ScalerNum);\
        }\
        u4Mode = FBM_POOL_MODE_422_FB | FBM_POOL_MODE_SCPOS_3FB;\
        if (u4ScalerMode & FBM_POOL_MODE_10BIT)\
        {\
            u4Mode |= FBM_POOL_MODE_10BIT;\
        }\
        else\
        {\
            u4Mode |= FBM_POOL_MODE_8BIT;\
        }\
    } while (0)

//4K2K EPG Mode dram always config 3840X2160
#if 0
#if defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5891)
#define UPDATE_EPG_WxH(u4VdpId, width, height)\
    do {\
        if (u4VdpId == VDP_1)\
        {\
            if (SRM_IsEpgMode() && !fgIs4k2kNativeTimingOut(VDP_1))\
            {\
                u4EPGWidth = MAX(u4EPGWidth,width);\
                u4EPGHeight = MAX(u4EPGHeight,height);\
                u4Width = MIN(u4EPGWidth,u4Width);\
                u4Height = MIN(u4EPGHeight,u4Height);\
            }\
        }\
    } while (0)
#else
#define UPDATE_EPG_WxH(u4VdpId, width, height)\
    do {\
        if (u4VdpId == VDP_1)\
        {\
            if (SRM_IsEpgMode())\
            {\
                u4Width = MIN(width,u4Width);\
                u4Height = MIN(height,u4Height);\
            }\
        }\
    } while (0)
#endif
#endif

#define UPDATE_WxH_BY_RESOLUTION(u4VdpId) \
    do {\
        if((env->u4Resolution[u4VdpId] == 0)||(SRM_IsScanMode(u4VdpId)))\
        {\
            u4Width = 720;\
            u4Height = 576;\
        }\
        else\
        {\
            u4Width = env->u4InputWidth[u4VdpId] ;\
            u4Height = env->u4InputHeight[u4VdpId] ;\
        }\
        u4Width = MIN(u4Width, u4DIFIFO);\
        UPDATE_POP_WIDTH();\
        UPDATE_HPD_WIDTH();\
    } while (0)

#define CHECK_PSCAN_SIZE_TO_0()\
    do {\
        UINT32 ModeFlags = 0;\
        ModeFlags = (ModeFlags | FBM_POOL_MODE_MDDI_DISP | FBM_POOL_MODE_MDDI_HYBRID | FBM_POOL_MODE_MDDI_DUAL_HYBRID);\
        if (((u4MddiMode & ModeFlags) == 0) && (u4MddiMode & FBM_POOL_MODE_MDDI_BOB) && (u4MddiMode & FBM_POOL_MODE_MDDI_NR_OFF))\
        {\
            u4Size = 0;\
        }\
    } while (0)
#define CHECK_SCALER_SIZE_TO_0()\
    do {\
        if ((u4ScalerMode & FBM_POOL_MODE_SCPOS_PSCAN_DISP) || (u4ScalerMode & FBM_POOL_MODE_SCPOS_HYBRID_DISP)|| (u4ScalerMode & FBM_POOL_MODE_SCPOS_DUAL_HYBRID_DISP)||(u4ScalerMode == FBM_POOL_MODE_UNSTABLE))\
        {\
            u4Size = 0;\
        }\
    } while (0)

static UINT32 FBM_FlipModule[VDP_NS];

static inline void v3DWxHAlign(const UINT32 u4DIMode, UINT32* pu4Width, UINT32* pu4Height)
{
    UINT32 u4Width = *pu4Width;
    UINT32 u4Height = *pu4Height;
    E_TD_OUT eDIOUT = TD_MIB_OUT(u4DIMode);

#if defined(CC_MT5882)||defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5883)||defined(CC_MT5891)|| defined(CC_MT5893) || defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5895)
    UINT32 u4DisplayH = PANEL_GetPanelHeight();
    E_TD_IN eDIIn = TD_MIB_IN(u4DIMode);


#if defined(CC_MT5883)
    if((E_TD_IN_FS_P == eDIIn) || (E_TD_IN_FS_I== eDIIn))
    {
        u4Width = MIN(960 , u4Width);
    }
#endif

    if(E_TD_OUT_3D_LI == eDIOUT)
    {
        if((E_TD_IN_SBS_P == eDIIn) || (E_TD_IN_SBS_I == eDIIn) ||
                (E_TD_IN_FS_P == eDIIn) || (E_TD_IN_FS_I == eDIIn))
        {
            //According to Predown rule, it will be predown to half of panel height in these conditions.
            u4Height = MIN(u4DisplayH/2 , u4Height);
        }
        if((E_TD_IN_TB_I == eDIIn) || (E_TD_IN_TB_P == eDIIn) ||
                (E_TD_IN_LI_P== eDIIn))
        {
            u4Height = MIN(u4DisplayH , u4Height);
        }
    }

    if(E_TD_OUT_NATIVE == eDIOUT &&(E_TD_IN_LI_P== eDIIn))
    {
        u4Height = MIN(u4DisplayH , u4Height);
    }

#endif

    if(E_TD_OUT_NATIVE != eDIOUT)
    {
        u4Width = (u4Width+511)/512*512;
    }

    *pu4Width = u4Width;
    *pu4Height= u4Height;
}

//CR : DTV00716642 2880*240P60HZ; dot by dot
UINT32 fgAdjustFramebuffer(VOID)
{
    UINT32 u4PanelHeight;
    UINT16 u2Height;

    u4PanelHeight = PANEL_GetPanelHeight();
    u2Height = wDrvVideoInputHeight(VDP_1);

    LOG(2, "fgAdjustFramebuffer = %d = %d \n", u2Height, u4PanelHeight * 2 / 5);

    if(((UINT32)u2Height) < (u4PanelHeight * 2 / 5))
    {
        return 1;
    }

    return 0;
}

static inline void vCalcInterlaceFldNum(UINT32 u4VdpId,
        UINT32 u4MddiMode, UINT32 u4flip, UINT8* u1Ynum, UINT8* u1Cnum,
        UINT32 u4PscanModeEx)
{
    if (u4MddiMode & FBM_POOL_MODE_MDDI_FULL)
    {
        *u1Ynum = 4;
        *u1Cnum = 4;
    }
    else if (u4MddiMode & FBM_POOL_MODE_MDDI_CBOB)
    {
        *u1Ynum = 4;
        *u1Cnum = 3;
    }
    else /* if (u4MddiMode & FBM_POOL_MODE_MDDI_BOB) */
    {
        //one more buffer for Pscan display mode BOB to solve tear issue
        *u1Ynum = 2;
        *u1Cnum = 2;
    }

    if ((u4MddiMode & FBM_POOL_MODE_MDDI_DISP))
    {
        // 1 frame for MIB frc
        // 1 frame for flilp mirror or epg
        *u1Ynum += 2;
        *u1Cnum += 2;
        if ((u4MddiMode & FBM_POOL_MODE_MDDI_BOB))
        {
            // when output sync bob mode and pr mode,A field have not line buffer,so it need to use the C field. Y/C num +1
            if(E_TD_OUT_3D_LI == TD_MIB_OUT(u4DrvTDTVDIModeQuery()))
            {
                *u1Ynum += 2;
                *u1Cnum += 2;
            }
        }
        /*
           if ((u4flip & FLIP_BY_PSCAN) || SRM_IsEpgMode() || fgAdjustFramebuffer())
           {
         *u1Ynum += 1;
         *u1Cnum += 1;
         }
         */
    }

    if(E_TDTV_DI_DRAM_DOUBLE==u4PscanModeEx)
    {
        *u1Ynum *=2;
        *u1Cnum *=2;
    }
    else if(E_TDTV_DI_DRAM_0FB==u4PscanModeEx)
    {
        *u1Ynum=0;
        *u1Cnum=0;
    }

    if(fgFBM_PscanTest)
    {
        *u1Ynum = 10;
        *u1Cnum = 10;
    }
}



inline UINT32 u4AllocMIBSizeProgressive(const UINT32 u4VdpId,
        const UINT32 u4MddiMode,const UINT32 u4FlipModule,
        const UINT32 u4Width,const UINT32 u4Height,const UINT32 u4PscanModeEx, const UINT8 fgIsMpeg)
{
    UINT32 u4Size;
    UINT8 u1NRNum;
    UINT32 u4DIMode = u4DrvTDTVDIModeQuery();

    CALCULATE_PSCAN_NR_NUM();
#if 0
    if (u4Width >= 500 && u4Width <= 2000)
    {
        u1NRNum = 16;//phoenix frame delay
    }
#endif

    //No 420 Support for Line interleave Src
    if((u4MddiMode & FBM_POOL_MODE_MDDI_NR_OFF)||(E_TDTV_DI_DRAM_DOUBLE==u4PscanModeEx))
    {
        if(fgIsMpeg&&!(E_TD_IN_LI_P==TD_MIB_IN(u4DIMode))&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                u4Size = FBM_MDDI_NR_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);
            }
            else
            {
                u4Size = FBM_MDDI_NR_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);
            }
        }
        else
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                u4Size = FBM_MDDI_NR_10BIT_SIZE(u4Width, u4Height, u1NRNum);
            }
            else
            {
                u4Size = FBM_MDDI_NR_8BIT_SIZE(u4Width, u4Height, u1NRNum);
            }
        }
    }
    else
    {
        if(fgIsMpeg&&!(E_TD_IN_LI_P==TD_MIB_IN(u4DIMode))&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                u4Size = FBM_MDDI_BI_NR_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);
            }
            else
            {
                u4Size = FBM_MDDI_BI_NR_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);
            }
        }
        else
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                u4Size = FBM_MDDI_BI_NR_10BIT_SIZE(u4Width, u4Height, u1NRNum);
            }
            else
            {
                u4Size = FBM_MDDI_BI_NR_8BIT_SIZE(u4Width, u4Height, u1NRNum);
            }
        }
    }

    if(0==u1NRNum)
        u4Size = 0;

    return u4Size;
}

static inline UINT32 u4AllocMIBSizeInterlace(const UINT32 u4VdpId,
        const UINT32 u4MddiMode,const UINT32 u4FlipModule,
        const UINT32 u4Width,const UINT32 u4Height,const UINT32 u4PscanModeEx, const UINT8 fgIsMpeg)
{
    UINT8 u1Ynum, u1Cnum;
    UINT32 u4Size = 0;

    vCalcInterlaceFldNum(u4VdpId, u4MddiMode, u4FlipModule, &u1Ynum, &u1Cnum,u4PscanModeEx);
#if 0
    if (u4Width >= 500 && u4Width <= 2000)
    {
        u1Ynum = u1Cnum = 16;//phoenix frame delay
    }
#endif

    if(VDP_1==u4VdpId)
    {
#if defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5882)||defined(CC_MT5891) || defined(CC_MT5893)|| defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5895)
        // Only applied in MT5399 because it's One of worst condition of MT5399,
        // keep untouched for saving QA effort,
        // This Definition check should be removed in next generation for unify system flow

        if(E_TDTV_DI_DRAM_DOUBLE==u4PscanModeEx)
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                if (fgIsMpeg)
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_10BIT_420_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
                else
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_10BIT_422_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
            }
            else // if (u4MddiMode & FBM_POOL_MODE_8BIT)
            {
                if (fgIsMpeg)
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_8BIT_420_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
                else
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_8BIT_422_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
            }
        }
        else
#endif
        {
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {
                if (fgIsMpeg)
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_BI_10BIT_420_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
                else
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_BI_10BIT_422_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
            }
            else // if (u4MddiMode & FBM_POOL_MODE_8BIT)
            {
                if (fgIsMpeg)
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_BI_8BIT_420_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
                else
                {
                    u4Size = FBM_ALIGN_MASK(FBM_MDDI_BI_8BIT_422_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
                }
            }
        }
    }
    else
    {
        u4Size = FBM_ALIGN_MASK(FBM_MDDI_SUB_BI_8BIT_422_SIZE(u4Width, u4Height, u1Ynum, u1Cnum), FBM_MDDI_SIZE_ALIGMENT);
    }

    CHECK_PSCAN_SIZE_TO_0();

    return u4Size;
}

UINT32 u4QueryFlipModule(UINT32 u4VdpId)
{
    return FBM_FlipModule[u4VdpId];
}

static inline UINT32 u4DecideFlipModule(UINT32 u4VdpId, UINT32 u4ScalerMode, UINT8 u1IsMPEG, FBM_AUTO_INC_ENV_T* env)
{
    UINT32 u4FlipModule = FLIP_NONE;
    UINT8 ucTVEEnable;
#ifdef CC_SUPPORT_NPTV_SEAMLESS
    VDP_SEAMLESS_INFO_T tb2rVrmInfo;
#endif

    TVE_GetEnable(TVE_1, &ucTVEEnable);
#if (defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5891)||defined(CC_MT5882)||defined(CC_MT5883) ||defined(CC_MT5863) ) && !defined(CC_BYPASS_MJC)
    if (TRUE == fgIsMJCToOSTG())
    {
        u4FlipModule = FLIP_BY_MJC;
    }
    else
#endif
    {
        if(u4ScalerMode & FBM_POOL_MODE_SCPOS_HYBRID_DISP)
        {
            u4FlipModule = FLIP_BY_SCALER;
        }
        else if(u4ScalerMode & FBM_POOL_MODE_SCPOS_DUAL_HYBRID_DISP)
        {
            u4FlipModule = FLIP_BY_PSCAN;
        }

#if defined(CC_MT5891)|| defined(CC_MT5893) || defined(CC_MT5895)
        else if((u1IsMPEG) && (u4VdpId == VDP_1) && !(IS_IC_5891_ES1())
#ifdef CC_SUPPORT_NPTV_SEAMLESS
                //nptv seamless b2r can not to do flip&mirror
                && (VDP_SET_ERROR == VDP_GetSeamlessInfo(u4VdpId, &tb2rVrmInfo))
#endif
               )
        {
            u4FlipModule = FLIP_BY_B2R;
        }
#elif !defined(CC_MT5883) && !defined(CC_MT5882) && !defined(CC_MT5863) && !defined(CC_MT5865)

        else if((u1IsMPEG) && (u4VdpId == VDP_1))
        {
            u4FlipModule = FLIP_BY_B2R;
        }

#endif
        else if (u4VdpId == VDP_2)
        {
            u4FlipModule = FLIP_BY_SCALER;
        }
        else
        {
            if (u4ScalerMode & FBM_POOL_MODE_SCPOS_FULL)
            {
                u4FlipModule = FLIP_BY_SCALER;
            }
            else if ( (u4ScalerMode & FBM_POOL_MODE_SCPOS_PSCAN_DISP)||(u4ScalerMode & FBM_POOL_MODE_SCPOS_LINESYNC_DISP))
            {
                u4FlipModule = FLIP_BY_PSCAN;
            }
        }

        //The follow case is that sub path don't do flip/mirror for scart out
        //1.non dtv source
        if((ucTVEEnable)&&(!env->u1IsMPEG[VDP_1])&&(!env->u1IsMPEG[VDP_2])&&(u4VdpId==VDP_2))
        {
            u4FlipModule = FLIP_NONE;
        }
        // 2.New B2R has issue doing flip/mirror
#if defined(CC_MT5882) || defined(CC_MT5883) || defined(CC_MT5863) || defined(CC_MT5865)
        if((ucTVEEnable) && (u4VdpId==VDP_2))
        {
            u4FlipModule = FLIP_NONE;
        }
#endif
    }
#ifdef CC_FLIP_MIRROR_SUPPORT
    if(u1GetFlipMirrorConfig() == 0)
        u4FlipModule = FLIP_NONE;
#endif
    FBM_FlipModule[u4VdpId] = u4FlipModule;

    return u4FlipModule;
}

UINT32 u4SizeOfCurrent[2];

BOOL fgFBM_MDDI_FIFO_LIMIT(UINT32 u4VdpId)
{
    UINT32 u4MddiMode = SRM_GetMddiMode(u4VdpId);
    UINT32 u4ScposMode = SRM_GetScposMode(u4VdpId);
    FBM_AUTO_INC_ENV_T env;

    _FbmGetEnv(&env);

    if ( !(u4ScposMode & FBM_POOL_MODE_SCPOS_FULL)     ||
            (env.u1Interlace[u4VdpId])                                     ||
            !(u4MddiMode & FBM_POOL_MODE_MDDI_NR_OFF)    )
    {
        return TRUE;
    }

    return FALSE;
}



void ExpandFBM4SinglexPOP(UINT32 u4VdpId, FBM_AUTO_INC_ENV_T* env, UINT32 u4BaseDramAddr)
{
    UINT32 u4NextDramAddr, u4MddiMode, u4ScalerMode, u4Width, u4Height, u4Size, u4Mode, u4HPDSize;
    UINT8 u1ScalerNum, u1NRNum;
    UINT32 u4FlipModule;
    UINT32 u4PscanModeEx;
    FBM_POOL_T* prPool;
#if defined(CC_MT5893) || defined(CC_MT5895)
    FBM_POOL_T* prRootPool;
#endif

    //UINT32 u4EPGWidth;
    //UINT32 u4EPGHeight;
    UINT32 u4PanelHeight;
    UINT32 u4DIMode = u4DrvTDTVDIModeQuery();
    static UINT32 u4PreMainScalerWidth = 0, u4PreMainScalerHeight = 0;
    UINT32 u4DIFIFO = 65535;
#if defined(CC_MT5893) || defined(CC_MT5895)
    UINT32 u4NewAddrModeAlign = (0xF<<11);
#endif

#ifdef CC_SUPPORT_DOLBY_VISION
    env->u1IsMPEG[u4VdpId] = 0;
#endif

    //should same as SRM_IsEpgMode  srm_if.c
    //u4EPGWidth = PANEL_GetPanelWidth()/3 + 20;
    //u4EPGHeight = PANEL_GetPanelHeight()*2/5 + 20;
    u4PanelHeight = PANEL_GetPanelHeight();

    u4PscanModeEx = u4DrvTDTVForceDIDramModeQueryEx(u4VdpId);
    u4MddiMode = SRM_GetMddiMode(u4VdpId);
    u4ScalerMode = SRM_GetScposMode(u4VdpId);
    u4HPDSize = SRM_GetScposHPDSize(u4VdpId); //Miss Nameing, it's Panel Display Size
    u4FlipModule = u4DecideFlipModule(u4VdpId, u4ScalerMode, env->u1IsMPEG[u4VdpId], env);

    u4NextDramAddr = u4BaseDramAddr;
    UPDATE_DI_FIFO_WIDTH(u4VdpId);

    // reset value
#if defined(CC_MT5890)||defined(CC_MT5891)|| defined(CC_MT5893) || defined(CC_MT5895)
    if (u4VdpId == VDP_1 && bDrvVideoIsDecDualMode())
    {
        u4PscanModeEx = E_TDTV_DI_DRAM_DOUBLE;
    }
#endif

    if ((E_TD_IN_SBS_P == TD_MIB_IN(u4DIMode) || E_TD_IN_SBS_I == TD_MIB_IN(u4DIMode)) &&
            E_TD_OUT_NATIVE != TD_MIB_OUT(u4DIMode) &&
            VDP_1 == u4VdpId)
    {
        u4HPDSize <<= 1;
        u4DIFIFO   <<= 1;
    }

    UPDATE_WxH_BY_RESOLUTION(u4VdpId);
#if defined(CC_MT5882) || defined(CC_MT5863)
#ifndef CC_FBM_SUPPORT_HDMI_4K2K30
    if(env->u4Resolution[u4VdpId] > SRM_VDP_1080HD_RESOLUTION)
    {
        u4Height = MIN(u4PanelHeight,env->u4InputHeight[u4VdpId]);
    }
#endif
#endif
    u4Size = 0;
    u4Mode = 0;

    if (u4VdpId == VDP_1)
    {
        UPDATE_PRPOOL(FBM_POOL_TYPE_TNR_DYN_MAIN, TNR);
        UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
        UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_MAIN, SCPOS);
    }
    else
    {
        UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
        UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
    }

    if (env->u1IsTDC[u4VdpId])
    {
        UPDATE_PRPOOL(FBM_POOL_TYPE_TDC_DYN, TDC);
    }

    if ((env->u1mode[u4VdpId] == FBM_MODE_1080I) || (env->u1mode[u4VdpId] == FBM_MODE_576I))
    {
        // scaler
        if((u4VdpId == VDP_2)&&(env->u1mode[u4VdpId] == FBM_MODE_1080I))
        {
#if defined(CC_FBM_SUB_SUPPORT_FHD)
            if(env->u1mode[u4VdpId] == FBM_MODE_1080I)
            {
                u4Width = 1920;
                u4Height = 1080;
            }
            else
            {
                u4Width = 960;
                u4Height = 540;
            }
#else
            u4Width = 960;
            u4Height = 1080;
#endif
        }

        if (u4VdpId == VDP_1)
        {
            //For ATV PVR, it's to keep Scaler dram address.
            if (fgFBMFixedScalerDram)
            {
                u4Width  = u4PreMainScalerWidth;
                u4Height = u4PreMainScalerHeight;
            }
            else
            {
                u4PreMainScalerWidth  = u4Width;
                u4PreMainScalerHeight = u4Height;
            }
        }

        //UPDATE_EPG_WxH(u4VdpId, 720, 576);

        CALCULATE_SCALER_422_PARAMETERS();
        CHECK_SCALER_SIZE_TO_0();

        if (u4VdpId == VDP_1)
        {
            UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_MAIN, SCPOS);
        }
        else
        {
            UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
        }

        // pscan
        UPDATE_WxH_BY_RESOLUTION(u4VdpId);

        //UPDATE_EPG_WxH(u4VdpId, 720, 1082);

        v3DWxHAlign(u4DIMode,&u4Width,&u4Height);

        u4Size = u4AllocMIBSizeInterlace(u4VdpId,u4MddiMode,u4FlipModule
                ,u4Width,u4Height,u4PscanModeEx,env->u1IsMPEG[u4VdpId]);

        if (u4VdpId == VDP_1)
        {
            UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
        }
        else
        {
            UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
        }

#ifndef CC_SUPPORT_RECORD_AV
        // tdc
        if(env->u1IsTDC[u4VdpId])
        {
            u4Width = 780;
            u4Height = 578;
            u4Mode = FBM_POOL_MODE_10BIT;
            u4Size = FBM_TDC_POOL_SIZE;
            UPDATE_PRPOOL(FBM_POOL_TYPE_TDC_DYN, TDC);
        }
#endif
    }
    else if (env->u1mode[u4VdpId] == FBM_MODE_1080P_422)
    {
        //UPDATE_EPG_WxH(u4VdpId, 720, 576);
        v3DWxHAlign(u4DIMode,&u4Width,&u4Height);
#if defined(CC_MT5886) || defined(CC_MT5893)  || defined(CC_MT5895)
        if (u4MddiMode & FBM_POOL_MODE_MDDI_TNR)
        {
            CALCULATE_PSCAN_NR_NUM();
            if(u4MddiMode & FBM_POOL_MODE_MDDI_TNR_UFO)
            {
                //UFO max data size
                u4Width = (u4Width + 256 -1) / 256 * 256;
                u4Size = ((u4Width * u4Height)/16 + 1) * 4 * u1NRNum * 128 /8;
                //UFO max EU_size size
                u4Size += ((u4Width /2048 + 1) * u4Height * 4 * u1NRNum * 128 /8);
            }
            else
            {
                if (u4MddiMode & FBM_POOL_MODE_10BIT)
                {
                    if(env->u1IsMPEG[u4VdpId]&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
                    {
                        u4Size = FBM_MDDI_NR_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);
                    }
                    else
                    {
                        u4Size = FBM_MDDI_NR_10BIT_SIZE(u4Width, u4Height, u1NRNum);
                    }
                }
                else
                {
                    if(env->u1IsMPEG[u4VdpId]&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
                    {
                        u4Size = FBM_MDDI_NR_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);
                    }
                    else
                    {
                        u4Size = FBM_MDDI_NR_8BIT_SIZE(u4Width, u4Height, u1NRNum);
                    }
                }
            }

            #if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
            #endif

            UPDATE_PRPOOL(FBM_POOL_TYPE_TNR_DYN_MAIN, TNR);
            // MDDI 720x480 when TNR is on
            u4Width = 720;
            u4Height = 480;
            if (u4MddiMode & FBM_POOL_MODE_10BIT)
            {//don't need mv flag if tnr is on
                if (env->u1IsMPEG[u4VdpId]&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
                {
                    u4Size = FBM_MDDI_BI_NR_10BIT_420_SIZE(u4Width, u4Height, u1NRNum);
                }
                else
                {
                    u4Size = FBM_MDDI_BI_NR_10BIT_SIZE(u4Width, u4Height, u1NRNum);
                }
            }
            else
            {
                if (env->u1IsMPEG[u4VdpId]&&(FBM_GetVideoCodecType(u4VdpId)!=FBM_VDEC_JPEG))
                {
                    u4Size = FBM_MDDI_BI_NR_8BIT_420_SIZE(u4Width, u4Height, u1NRNum);
                }
                else
                {
                    u4Size = FBM_MDDI_BI_NR_8BIT_SIZE(u4Width, u4Height, u1NRNum);
                }
            }

#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

            UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
        }
        else
#endif
            if (u4MddiMode & FBM_POOL_MODE_MDDI_DISP)
            {
#if defined(CC_MT5399)||defined(CC_MT5890) ||defined(CC_MT5882)||defined(CC_MT5891) || defined(CC_MT5893)|| defined(CC_MT5886) || defined(CC_MT5863) || defined(CC_MT5895)
                // Only applied in MT5399 because it's One of worst condition of MT5399,
                // keep untouched for saving QA effort,
                // This Definition check should be removed in next generation for unify system flow
                u4Size = u4AllocMIBSizeProgressive(u4VdpId,u4MddiMode,u4FlipModule
                        ,u4Width,u4Height,u4PscanModeEx,env->u1IsMPEG[u4VdpId]);
#else
                //No 420 Support for Line interleave Src
                if (E_TD_IN_LI_P==TD_MIB_IN(u4DIMode))
                {
                    CALCULATE_PSCAN_NR_YC_SIZE_3D_422();
                }
                else
                {
                    CALCULATE_PSCAN_NR_YC_SIZE_3D();
                }
#endif
#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

                if (u4VdpId == VDP_1)
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
                }
                else
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
                }
            }
            else
            {
                if (u4VdpId == VDP_1)
                {
                    //For ATV PVR, it's to keep Scaler dram address.
                    if (fgFBMFixedScalerDram)
                    {
                        u4Width  = u4PreMainScalerWidth;
                        u4Height = u4PreMainScalerHeight;
                    }
                    else
                    {
                        u4PreMainScalerWidth  = u4Width;
                        u4PreMainScalerHeight = u4Height;
                    }
                }
                else
                {
                    u4Width = MIN(u4Width, 1920);
                    u4Height = MIN(u4Height, 1080);
                }

                CALCULATE_SCALER_422_PARAMETERS();
                CHECK_SCALER_SIZE_TO_0();

#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

                if (u4VdpId == VDP_1)
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_MAIN, SCPOS);
                }
                else
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
                }


            CALCULATE_PSCAN_NR_YC_SIZE_3D();
            CHECK_PSCAN_SIZE_TO_0();
#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

                if (u4VdpId == VDP_1)
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
                }
                else
                {
                    UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
                }
            }
    }
    else if (env->u1mode[u4VdpId] == FBM_MODE_1080P_444)
    {
        //UPDATE_EPG_WxH(u4VdpId, 720, 576);

        if(u4MddiMode & FBM_POOL_MODE_MDDI_DISP)
        {
            v3DWxHAlign(u4DIMode,&u4Width,&u4Height);

            CALCULATE_PROG_DISP_444_SIZE();
#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

            if (u4VdpId == VDP_1)
            {
                UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_MAIN, MDDI);
            }
            else
            {
                UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
            }
        }
        else
        {
            if(u4VdpId == VDP_2)
            {
                u4Width = MIN(u4Width, 1920);
                u4Height = MIN(u4Height, 1080);
            }
            CALCULATE_SCALER_444_PARAMETERS();
            CHECK_SCALER_SIZE_TO_0();
#if defined(CC_MT5893) || defined(CC_MT5895)
            u4Size = u4Size | u4NewAddrModeAlign;
#endif

            if (u4VdpId == VDP_1)
            {
                UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_MAIN, SCPOS);
            }
            else
            {
                UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
            }
        }
    }

    u4SizeOfCurrent[u4VdpId] = u4NextDramAddr-u4BaseDramAddr;

#if defined(CC_MT5893) || defined(CC_MT5895)
    if(u4VdpId == VDP_1)
    {
        prRootPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
    }
    else
    {
        prRootPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_PIP);
    }
    if((prRootPool != NULL) && (prRootPool->u4Size < u4SizeOfCurrent[u4VdpId]))
    {
        LOG(0,"NPTV FBM root buffer for path(%d) is too small!!! FIX ME\n",u4VdpId);
    }
#endif
    LOG(3, "vdp:%d, occupied size: 0x%x\n", u4VdpId, u4SizeOfCurrent[u4VdpId]);
}

void ExpandFBM4SubPIP(UINT32 u4VdpId, FBM_AUTO_INC_ENV_T* env, UINT32 u4BaseDramAddr)
{
    UINT32 u4NextDramAddr, u4MddiMode, u4ScalerMode,u4Width, u4Height, u4Size, u4Mode;
    UINT32 u4FlipModule;
    FBM_POOL_T* prPool;
    UINT8 u1ScalerNum;
#if defined(CC_MT5893) || defined(CC_MT5895)
    UINT32 u4NewAddrModeAlign = (0xF<<11);
    FBM_POOL_T* prRootPool;
#endif

    u4NextDramAddr = u4BaseDramAddr;
    u4MddiMode = SRM_GetMddiMode(u4VdpId);
    u4ScalerMode = SRM_GetScposMode(u4VdpId);

    u4FlipModule = u4DecideFlipModule(u4VdpId, u4ScalerMode, env->u1IsMPEG[u4VdpId], env);

    // reset value
    u4Size = 0;
    u4Width = 720;
    u4Height = 576;
    u4Mode = 0;

#if defined(CC_FBM_SUB_SUPPORT_FHD)
    u4Width = env->u4InputWidth[u4VdpId] ;
    u4Height = env->u4InputHeight[u4VdpId] ;
#endif
    UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);
    UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
    if (env->u1IsTDC[u4VdpId])
    {
        UPDATE_PRPOOL(FBM_POOL_TYPE_TDC_DYN, TDC);
    }

    if ((env->u1mode[u4VdpId] == FBM_MODE_1080I) || (env->u1mode[u4VdpId] == FBM_MODE_576I))
    {
        // pscan
#if !defined(CC_FBM_SUB_SUPPORT_FHD)
        if (env->u1mode[u4VdpId] == FBM_MODE_1080I)
        {
#ifdef CC_MT5882
            if(IS_IC_5885())
            {
#ifdef CC_FORCE_DRAM256M
                u4Width = 720;
#else
                u4Width = 1280;
#endif
            }
            else if(IS_IC_5882_ES1())
            {
                u4Width = 720;
            }
            else
            {
                u4Width = 960;
            }
#else
            u4Width = 720;
#endif
            u4Height = 1082;
        }
        else
        {
            u4Width = 780;
            u4Height = 576;
        }
#endif

        u4Size = u4AllocMIBSizeInterlace(u4VdpId,u4MddiMode,u4FlipModule
                ,u4Width,u4Height,E_TDTV_DI_DRAM_NORMAL,env->u1IsMPEG[u4VdpId]);

#if defined(CC_MT5893) || defined(CC_MT5895)
        u4Size = u4Size | u4NewAddrModeAlign;
#endif

        UPDATE_PRPOOL(FBM_POOL_TYPE_MDDI_DYN_SUB, MDDI);

        // scaler
#if !defined(CC_FBM_SUB_SUPPORT_FHD)
        u4Width = 720;
        u4Height = 576;

        // DTV00627049  FBM->u4 Height < u4DispHeight in 4k2k
#ifdef CC_SUPPORT_4K2K
        if (PANEL_IsSupport4K2KDD())
        {
            u4Height = 1080;
        }
#endif

#endif
        CALCULATE_SCALER_422_PARAMETERS();
#if defined(CC_MT5893) || defined(CC_MT5895)
        u4Size = u4Size | u4NewAddrModeAlign;
#endif

        UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);

        //tdc
        if(env->u1IsTDC[u4VdpId])
        {
            u4Width = 780;
            u4Height = 578;
            u4Mode = FBM_POOL_MODE_10BIT;
            u4Size = FBM_TDC_POOL_SIZE;
            UPDATE_PRPOOL(FBM_POOL_TYPE_TDC_DYN, TDC);
        }
    }
    else if ((env->u1mode[u4VdpId] == FBM_MODE_1080P_422) || (env->u1mode[u4VdpId] == FBM_MODE_1080P_444))
    {
        // scaler
        u4NextDramAddr = u4BaseDramAddr;
#if !defined(CC_FBM_SUB_SUPPORT_FHD)
        u4Width = 780;
        u4Height = 576;

        // DTV00627049  FBM->u4 Height < u4DispHeight in 4k2k
#ifdef CC_SUPPORT_4K2K
        if (PANEL_IsSupport4K2KDD())
        {
            u4Height = 1080;
        }
#endif

#endif
        u4Width = MIN(u4Width,1920);
        u4Height = MIN(u4Height,1080);

        CALCULATE_SCALER_444_PARAMETERS();
#if defined(CC_MT5893) || defined(CC_MT5895)
        u4Size = u4Size | u4NewAddrModeAlign;
#endif
        UPDATE_PRPOOL(FBM_POOL_TYPE_SCPOS_DYN_SUB, SCPOS);
    }

#if defined(CC_MT5893) || defined(CC_MT5895)
    if(u4VdpId == VDP_1)
    {
        prRootPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
    }
    else
    {
        prRootPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_PIP);
    }
    if((prRootPool != NULL) && (prRootPool->u4Size < u4SizeOfCurrent[u4VdpId]))
    {
        LOG(0,"NPTV FBM root buffer for path(%d) is too small!!! FIX ME\n",u4VdpId);
    }
#endif

    u4SizeOfCurrent[u4VdpId] = u4NextDramAddr-u4BaseDramAddr;
    LOG(3, "vdp:%d, occupied size: 0x%x\n", u4VdpId, u4SizeOfCurrent[u4VdpId]);
}


#if defined(CC_FBM_SPLIT_MPEG_FREE_PART)
UINT32 _FbmGetDyamicMPEG_NptvMaxSize(void)
{
    return 0x34a000; // 720P
}

UINT32 _FbmGetDyamicMPEG_MPEGMaxSize(void)
{
    return 0x750000; // RMVB 720P
}
#endif

#if defined(CC_MT5882)
static FBM_POOL_T* _FbmGetPoolInfoAuto_MT5882_1GB(UCHAR ucPoolType, const FBM_POOL_MODULE_INFO_T* prInfo)
{
    FBM_POOL_T* prPool=NULL;
    static UINT8 fgInited = FALSE;
    //FBM_POOL_T* prPrevPool;
    FBM_AUTO_INC_ENV_T env;
    UINT8 IsNoSub = FALSE;
    UINT8 u1CheckDSP = 1;
    UNUSED(u1CheckDSP);

    if (!fgInited) // calculate max size of main, sub path in pip/pop
    {
        fgInited = TRUE;

        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
        _fbm_u4NptvBeAddr = prPool->u4Addr;
        //prPrevPool = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP);
        //_fbm_u4NptvSubPopAddr = (_fbm_u4NptvBeAddr + (prPrevPool->u4Addr))/2;
#ifdef CC_FORCE_DRAM256M
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvBeAddr + (prPool->u4Size);
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvSubPopAddr - 0x700000;
#else
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvBeAddr + (prPool->u4Size)/2;
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvSubPopAddr +0x200000;
#endif
        /* set _fbm_u4NptvSubPipAddr in the following case */

        if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
        {
#if defined(CC_FBM_SUPPORT_HDMI_4K2K30)
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1e80000 + 0x2E00000;
#else
#ifdef CC_FORCE_DRAM256M
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + (prPool->u4Size);
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvSubPipAddr - 0x700000;
#else
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1a00000;
#endif
#endif
        }
        else
        {
#if defined(CC_FLIP_MIRROR_SUPPORT)
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1310000;
#else
            // MT5365_128Mx2
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1180000;
            //_fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1c00000;
#endif
        }

        //#if defined(CC_IC_VERIFY_FBM_MODE)
#if 0
        _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x2000000;
#endif

        // reset value
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#ifndef CC_SUPPORT_RECORD_AV
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#endif

    }

    _FbmGetEnv(&env);

#if defined(CC_FBM_NO_SUBPATH)
    IsNoSub = TRUE;
#endif

    if (IsNoSub)
    {
        if (env.u1IsMPEG[VDP_1])
        {
#if defined(CC_DYNAMIC_MPEG_SIZE)
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
#else
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
#endif
            _fbm_u4NptvBeAddr = prPool->u4Addr;
#if defined(CC_DYNAMIC_MPEG_SIZE) // unmark
            _fbm_u4NptvBeAddr = prPool->u4Addr + SRM_GetMMSizeMode(VDP_1);
#endif
        }
        else
        {
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
            _fbm_u4NptvBeAddr = prPool->u4Addr;
        }

        if (env.u1fgPIP || env.u1fgPOP)
        {
            LOG(0, "This model does NOT support PIP/POP\n");
            //ASSERT(0);
        }
    }

    ExpandFBM4SinglexPOP(VDP_1, &env, _fbm_u4NptvBeAddr);
#if !defined(CC_FBM_NO_SUBPATH)
    if (env.u1fgPIP || env.u1fgPOP)
    {
        if (env.u1fgPOP)
        {
            // u4NextDramAddr = _fbm_u4NptvSubPopAddr; // todo
            ExpandFBM4SinglexPOP(VDP_2, &env, _fbm_u4NptvSubPopAddr);
        }
        else
        {
            // u4NextDramAddr = _fbm_u4NptvSubPipAddr; // todo
            ExpandFBM4SubPIP(VDP_2, &env, _fbm_u4NptvSubPipAddr);
        }
    }
#endif
    if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP))
    {
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC)
        {
            _eCurrentTdc = FBM_POOL_TYPE_TDC_DYN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN)
        {
            _eCurrentMddiMain = FBM_POOL_TYPE_MDDI_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN)
        {
            _eCurrentScposMain = FBM_POOL_TYPE_SCPOS_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP)
        {
            _eCurrentMddiPip = FBM_POOL_TYPE_MDDI_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP)
        {
            _eCurrentScposPip = FBM_POOL_TYPE_SCPOS_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        }
        // sub pip error checking
        // sub pop error checking
#if !defined(CC_FBM_NO_SUBPATH)
        if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
                (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN))
        {
            if (env.u1fgPIP)
            {
                if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPipAddr)
                {
                    LOG(0, "FBM main overwrite subpip\n");
                }
            }
            else if (env.u1fgPOP)
            {
                if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPopAddr)
                {
                    LOG(0, "FBM main overwrite subpop\n");
                }
            }
        }
#endif

        /*
           if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_DDR2x2)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5396_DDR3x4)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_3DTVCHB_NOMJC))
           {
        // only SCPOS_MAIN in channel 2, so it need not to check
        u1CheckDSP = 0;
        }
        */
        // dsp error checking
#if !defined(CC_TVP_SUPPORT)
        if (u1CheckDSP)
        {
            if ((prPool->u4Addr + prPool->u4Size) > (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr))
            {
                LOG(0, "FBM nptv overlap with DSP: 0x%x\n", ucPoolType);
                LOG(0, "occupied range: 0x%x--0x%x\n", prPool->u4Addr, (prPool->u4Addr + prPool->u4Size));
                LOG(0, "valid range: 0x%x--0x%x\n", _fbm_u4NptvBeAddr, (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr));
#ifndef CC_TVP_SUPPORT
                VERIFY(0);
#endif
            }
        }
#endif
        return prPool;
    }

#if defined(CC_FBM_SPLIT_MPEG_FREE_PART)
    prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG_FREE);
    prPool->u4Addr = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG)->u4Addr;
    prPool->u4Addr += _FbmGetDyamicMPEG_NptvMaxSize();
    prPool->u4Addr += _FbmGetDyamicMPEG_MPEGMaxSize();
    prPool->u4Size = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr - prPool->u4Addr;
#endif

    if(ucPoolType >= FBM_POOL_TYPE_NS)
    {
        LOG(0, "input type error (%d)\n",ucPoolType);
        return (FBM_GetPoolInfo(FBM_POOL_TYPE_TOTAL));
    }
    prPool = FBM_GetPoolInfo(ucPoolType);

    return prPool;
}

#endif

#if defined(CC_MT5863) || defined(CC_MT5865)
static FBM_POOL_T* _FbmGetPoolInfoAuto_MT5863_1GB(UCHAR ucPoolType, const FBM_POOL_MODULE_INFO_T* prInfo)
{
    FBM_POOL_T* prPool=NULL;
    static UINT8 fgInited = FALSE;
    //FBM_POOL_T* prPrevPool;
    FBM_AUTO_INC_ENV_T env;
    UINT8 IsNoSub = FALSE;
    UINT8 u1CheckDSP = 1;
    UNUSED(u1CheckDSP);

    if (!fgInited) // calculate max size of main, sub path in pip/pop
    {
        fgInited = TRUE;

        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
        _fbm_u4NptvBeAddr = prPool->u4Addr;
        //prPrevPool = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP);
        //_fbm_u4NptvSubPopAddr = (_fbm_u4NptvBeAddr + (prPrevPool->u4Addr))/2;
#ifdef CC_FORCE_DRAM256M
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvBeAddr + (prPool->u4Size);
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvSubPopAddr - 0x700000;
#else
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvBeAddr + (prPool->u4Size)/2;
        _fbm_u4NptvSubPopAddr = _fbm_u4NptvSubPopAddr +0x200000;
#endif
        /* set _fbm_u4NptvSubPipAddr in the following case */

        if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
        {
#if defined(CC_FBM_SUPPORT_HDMI_4K2K30)
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1e80000 + 0x2E00000;
#else
#ifdef CC_FORCE_DRAM256M
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + (prPool->u4Size);
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvSubPipAddr - 0x700000;
#else
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1a00000;
#endif
#endif
        }
        else
        {
#if defined(CC_FLIP_MIRROR_SUPPORT)
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1310000;
#else
            // MT5365_128Mx2
            _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1180000;
            //_fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x1c00000;
#endif
        }

        //#if defined(CC_IC_VERIFY_FBM_MODE)
#if 0
        _fbm_u4NptvSubPipAddr = _fbm_u4NptvBeAddr + 0x2000000;
#endif

        // reset value
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#ifndef CC_SUPPORT_RECORD_AV
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#endif

    }

    _FbmGetEnv(&env);

#if defined(CC_FBM_NO_SUBPATH)
    IsNoSub = TRUE;
#endif

    if (IsNoSub)
    {
        if (env.u1IsMPEG[VDP_1])
        {
#if defined(CC_DYNAMIC_MPEG_SIZE)
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
#else
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
#endif
            _fbm_u4NptvBeAddr = prPool->u4Addr;
#if defined(CC_DYNAMIC_MPEG_SIZE) // unmark
            _fbm_u4NptvBeAddr = prPool->u4Addr + SRM_GetMMSizeMode(VDP_1);
#endif
        }
        else
        {
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
            _fbm_u4NptvBeAddr = prPool->u4Addr;
        }

        if (env.u1fgPIP || env.u1fgPOP)
        {
            LOG(0, "This model does NOT support PIP/POP\n");
            //ASSERT(0);
        }
    }

    ExpandFBM4SinglexPOP(VDP_1, &env, _fbm_u4NptvBeAddr);
#if !defined(CC_FBM_NO_SUBPATH)
    if (env.u1fgPIP || env.u1fgPOP)
    {
        if (env.u1fgPOP)
        {
            // u4NextDramAddr = _fbm_u4NptvSubPopAddr; // todo
            ExpandFBM4SinglexPOP(VDP_2, &env, _fbm_u4NptvSubPopAddr);
        }
        else
        {
            // u4NextDramAddr = _fbm_u4NptvSubPipAddr; // todo
            ExpandFBM4SubPIP(VDP_2, &env, _fbm_u4NptvSubPipAddr);
        }
    }
#endif
    if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP))
    {
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC)
        {
            _eCurrentTdc = FBM_POOL_TYPE_TDC_DYN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN)
        {
            _eCurrentMddiMain = FBM_POOL_TYPE_MDDI_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN)
        {
            _eCurrentScposMain = FBM_POOL_TYPE_SCPOS_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP)
        {
            _eCurrentMddiPip = FBM_POOL_TYPE_MDDI_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP)
        {
            _eCurrentScposPip = FBM_POOL_TYPE_SCPOS_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        }
        // sub pip error checking
        // sub pop error checking
#if !defined(CC_FBM_NO_SUBPATH)
        if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
                (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN))
        {
            if (env.u1fgPIP)
            {
                if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPipAddr)
                {
                    LOG(0, "FBM main overwrite subpip\n");
                }
            }
            else if (env.u1fgPOP)
            {
                if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPopAddr)
                {
                    LOG(0, "FBM main overwrite subpop\n");
                }
            }
        }
#endif

        /*
           if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_DDR2x2)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5396_DDR3x4)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_3DTVCHB_NOMJC))
           {
        // only SCPOS_MAIN in channel 2, so it need not to check
        u1CheckDSP = 0;
        }
        */
        // dsp error checking
#if !defined(CC_TVP_SUPPORT)
        if (u1CheckDSP)
        {
            if ((prPool->u4Addr + prPool->u4Size) > (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr))
            {
                LOG(0, "FBM nptv overlap with DSP: 0x%x\n", ucPoolType);
                LOG(0, "occupied range: 0x%x--0x%x\n", prPool->u4Addr, (prPool->u4Addr + prPool->u4Size));
                LOG(0, "valid range: 0x%x--0x%x\n", _fbm_u4NptvBeAddr, (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr));
#ifndef CC_TVP_SUPPORT
                VERIFY(0);
#endif
            }
        }
#endif
        return prPool;
    }

#if defined(CC_FBM_SPLIT_MPEG_FREE_PART)
    prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG_FREE);
    prPool->u4Addr = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG)->u4Addr;
    prPool->u4Addr += _FbmGetDyamicMPEG_NptvMaxSize();
    prPool->u4Addr += _FbmGetDyamicMPEG_MPEGMaxSize();
    prPool->u4Size = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr - prPool->u4Addr;
#endif

    if(ucPoolType >= FBM_POOL_TYPE_NS)
    {
        LOG(0, "input type error (%d)\n",ucPoolType);
        return (FBM_GetPoolInfo(FBM_POOL_TYPE_TOTAL));
    }
    prPool = FBM_GetPoolInfo(ucPoolType);

    return prPool;
}

#endif


#if defined(CC_MT5883)
static FBM_POOL_T* _FbmGetPoolInfoAuto_MT5883_1G(UCHAR ucPoolType, const FBM_POOL_MODULE_INFO_T* prInfo)
{
    FBM_POOL_T* prPool=NULL;
    static UINT8 fgInited = FALSE;
    //FBM_POOL_T* prPrevPool;
    FBM_AUTO_INC_ENV_T env;
    UINT8 IsNoSub = FALSE;
    UINT8 u1CheckDSP = 1;
    UNUSED(u1CheckDSP);

    if (!fgInited) // calculate max size of main, sub path in pip/pop
    {
        fgInited = TRUE;

        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
        _fbm_u4NptvBeAddr = prPool->u4Addr;

        // reset value
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#ifndef CC_SUPPORT_RECORD_AV
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#endif

    }

    _FbmGetEnv(&env);

    IsNoSub = ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV));

#if defined(CC_FBM_NO_SUBPATH)
    IsNoSub = TRUE;
#endif

    if (IsNoSub)
    {
        if (env.u1IsMPEG[VDP_1])
        {
#if defined(CC_DYNAMIC_MPEG_SIZE)
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
#else
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
#endif
            _fbm_u4NptvBeAddr = prPool->u4Addr;
#if defined(CC_DYNAMIC_MPEG_SIZE) // unmark
            _fbm_u4NptvBeAddr = prPool->u4Addr + SRM_GetMMSizeMode(VDP_1);
#endif
        }
        else
        {
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
            _fbm_u4NptvBeAddr = prPool->u4Addr;
        }

        if (env.u1fgPIP || env.u1fgPOP)
        {
            LOG(0, "This model does NOT support PIP/POP\n");
            //ASSERT(0);
        }
    }

    ExpandFBM4SinglexPOP(VDP_1, &env, _fbm_u4NptvBeAddr);
#if !defined(CC_FBM_NO_SUBPATH)
    if (env.u1fgPIP || env.u1fgPOP)
    {
        if (env.u1fgPOP)
        {
            // u4NextDramAddr = _fbm_u4NptvSubPopAddr; // todo
            ExpandFBM4SinglexPOP(VDP_2, &env, _fbm_u4NptvSubPopAddr);
        }
        else
        {
            // u4NextDramAddr = _fbm_u4NptvSubPipAddr; // todo
            ExpandFBM4SubPIP(VDP_2, &env, _fbm_u4NptvSubPipAddr);
        }
    }
#endif
    if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP))
    {
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC)
        {
            _eCurrentTdc = FBM_POOL_TYPE_TDC_DYN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN)
        {
            _eCurrentMddiMain = FBM_POOL_TYPE_MDDI_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN)
        {
            _eCurrentScposMain = FBM_POOL_TYPE_SCPOS_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP)
        {
            _eCurrentMddiPip = FBM_POOL_TYPE_MDDI_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP)
        {
            _eCurrentScposPip = FBM_POOL_TYPE_SCPOS_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        }
        // sub pip error checking
        // sub pop error checking
        if (!IsNoSub)
        {
#if !defined(CC_FBM_NO_SUBPATH)
            if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
                    (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN))
            {
                if (env.u1fgPIP)
                {
                    if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPipAddr)
                    {
                        LOG(0, "FBM main overwrite subpip\n");
                    }
                }
                else if (env.u1fgPOP)
                {
                    if ((prPool->u4Addr + prPool->u4Size) > _fbm_u4NptvSubPopAddr)
                    {
                        LOG(0, "FBM main overwrite subpop\n");
                    }
                }
            }
#endif
        }

        /*
           if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_DDR2x2)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5396_DDR3x4)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_3DTVCHB_NOMJC))
           {
        // only SCPOS_MAIN in channel 2, so it need not to check
        u1CheckDSP = 0;
        }
        */
        // dsp error checking
        /* mark dsp check,because dsp is move to fbm pool front
           if (u1CheckDSP)
           {
           if ((prPool->u4Addr + prPool->u4Size) > (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr))
           {
           Printf("FBM nptv overlap with DSP: 0x%x\n", ucPoolType);
           Printf("occupied range: 0x%x--0x%x\n", prPool->u4Addr, (prPool->u4Addr + prPool->u4Size));
           Printf("valid range: 0x%x--0x%x\n", _fbm_u4NptvBeAddr, (FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr));
#ifndef CC_TVP_SUPPORT
VERIFY(0);
#endif
}
}
*/
return prPool;
}

#if defined(CC_FBM_SPLIT_MPEG_FREE_PART)
prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG_FREE);
prPool->u4Addr = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG)->u4Addr;
prPool->u4Addr += _FbmGetDyamicMPEG_NptvMaxSize();
prPool->u4Addr += _FbmGetDyamicMPEG_MPEGMaxSize();
prPool->u4Size = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr - prPool->u4Addr;
#endif

if(ucPoolType >= FBM_POOL_TYPE_NS)
{
    LOG(0, "input type error (%d)\n",ucPoolType);
    return (FBM_GetPoolInfo(FBM_POOL_TYPE_TOTAL));
}
prPool = FBM_GetPoolInfo(ucPoolType);

return prPool;
}
#endif


static FBM_POOL_T* _FbmGetPoolInfoAuto_MT5890_1GB(UCHAR ucPoolType, const FBM_POOL_MODULE_INFO_T* prInfo)
{
    FBM_POOL_T* prPool=NULL;
    static UINT8 fgInited = FALSE;
    //FBM_POOL_T* prPrevPool;
    FBM_AUTO_INC_ENV_T env;
#if defined(CC_FBM_NO_SUBPATH)
    UINT8 IsNoSub = FALSE;
#endif
    UINT8 u1CheckDSP = 1;
    UNUSED(u1CheckDSP);

    if (!fgInited) // calculate max size of main, sub path in pip/pop
    {
        fgInited = TRUE;

        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
        _fbm_u4NptvBeAddr = prPool->u4Addr;

        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_PIP);
        _fbm_u4NptvSubAddr = prPool->u4Addr + 0x80000;

        // reset value
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_MDDI_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TNR_DYN_MAIN);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_TNR_ADDR_ALIGMENT);
#ifndef CC_SUPPORT_RECORD_AV
        prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC);
        prPool->u4Addr = FBM_ALIGN_MASK(_fbm_u4NptvBeAddr, FBM_SCPOS_ADDR_ALIGMENT);
#endif

    }

    _FbmGetEnv(&env);
#if defined(CC_FBM_NO_SUBPATH)
    IsNoSub = TRUE;


    if (IsNoSub)
    {
        if (env.u1IsMPEG[VDP_1])
        {
#if defined(CC_DYNAMIC_MPEG_SIZE)
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG);
#else
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_FEEDER);
#endif
            _fbm_u4NptvBeAddr = prPool->u4Addr + prPool->u4Size;
#if defined(CC_DYNAMIC_MPEG_SIZE) // unmark
            _fbm_u4NptvBeAddr = prPool->u4Addr + SRM_GetMMSizeMode(VDP_1);
#endif
        }
        else
        {
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_MAIN);
            _fbm_u4NptvBeAddr = prPool->u4Addr;
        }

        if (env.u1fgPIP || env.u1fgPOP)
        {
            LOG(0, "This model does NOT support PIP/POP\n");
            //ASSERT(0);
        }
    }
#endif


    ExpandFBM4SinglexPOP(VDP_1, &env, _fbm_u4NptvBeAddr);
#if !defined(CC_FBM_NO_SUBPATH)
    if (env.u1fgPIP || env.u1fgPOP)
    {
        if (env.u1fgPOP)
        {
            // u4NextDramAddr = _fbm_u4NptvSubPopAddr; // todo
            ExpandFBM4SinglexPOP(VDP_2, &env, _fbm_u4NptvSubAddr);
        }
        else
        {
            // u4NextDramAddr = _fbm_u4NptvSubPipAddr; // todo
            ExpandFBM4SubPIP(VDP_2, &env, _fbm_u4NptvSubAddr);
        }
    }
#endif
    if ((ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_TNR_DYN_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP) ||
            (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP))
    {
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_TDC)
        {
            _eCurrentTdc = FBM_POOL_TYPE_TDC_DYN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TDC_DYN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_MAIN)
        {
            _eCurrentMddiMain = FBM_POOL_TYPE_MDDI_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_TNR_DYN_MAIN)
        {
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_TNR_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN)
        {
            _eCurrentScposMain = FBM_POOL_TYPE_SCPOS_DYN_MAIN;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_MAIN);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MDDI_PIP)
        {
            _eCurrentMddiPip = FBM_POOL_TYPE_MDDI_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MDDI_DYN_SUB);
        }
        if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_PIP)
        {
            _eCurrentScposPip = FBM_POOL_TYPE_SCPOS_DYN_SUB;
            prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_SCPOS_DYN_SUB);
        }

        /*
           if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_DDR2x2)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5396_DDR3x4)
           || (SRMFBM_GetConf() == FBM_MEM_CFG_MT5395_3DTVCHB_NOMJC))
           {
        // only SCPOS_MAIN in channel 2, so it need not to check
        u1CheckDSP = 0;
        }
        */
        // dsp error checking

        return prPool;
    }

#if defined(CC_FBM_SPLIT_MPEG_FREE_PART)
    prPool = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG_FREE);
    prPool->u4Addr = FBM_GetPoolInfo(FBM_POOL_TYPE_MPEG)->u4Addr;
    prPool->u4Addr += _FbmGetDyamicMPEG_NptvMaxSize();
    prPool->u4Addr += _FbmGetDyamicMPEG_MPEGMaxSize();
    prPool->u4Size = FBM_GetPoolInfo(FBM_POOL_TYPE_DSP)->u4Addr - prPool->u4Addr;
#endif

    if(ucPoolType >= FBM_POOL_TYPE_NS)
    {
        LOG(0, "input type error (%d)\n",ucPoolType);
        return (FBM_GetPoolInfo(FBM_POOL_TYPE_TOTAL));
    }
    prPool = FBM_GetPoolInfo(ucPoolType);

    return prPool;
}

//-------------------------------------------------------------------------
/** FBM_GetPoolInfoAuto
 *  Get Pool Info
 *  @param  ucPoolType
 *  @param  prInfo
 *  @return NONE.
 */
//-------------------------------------------------------------------------
FBM_POOL_T* FBM_GetPoolInfoAuto(UCHAR ucPoolType, const FBM_POOL_MODULE_INFO_T* prInfo)
{
    FBM_POOL_MODULE_INFO_T rInfo;
    if (prInfo == NULL)
    {
        SRM_VDP_INFO_T* prVdpInfo;
        prVdpInfo = SRM_GetVdpInfo(VDP_1);
        x_memset((VOID*)&rInfo, 0, sizeof(FBM_POOL_MODULE_INFO_T));
        rInfo.u4SourceType = prVdpInfo->u4Source;
        rInfo.u4Resolution = prVdpInfo->u4Resolution;
        rInfo.u4Interlace = prVdpInfo->u4Interlace;
        rInfo.u4VgaTiming = prVdpInfo->u4VgaTiming;
        rInfo.u4ColorFormat444 = prVdpInfo->u4ColorFormat444;

        if (SRM_IsPopVideo())
        {
            rInfo.u4PipPop = SRM_TV_MODE_TYPE_POP;
        }
        else if (SRM_IsPipVideo())
        {
            rInfo.u4PipPop = SRM_TV_MODE_TYPE_PIP;
        }
        prInfo = &rInfo;
    }
    else
    {
        x_memcpy((VOID*)&rInfo, (VOID*)prInfo, sizeof(FBM_POOL_MODULE_INFO_T));
        rInfo.u4Resolution = prInfo->u4Width * prInfo->u4Height;
        prInfo = &rInfo;
    }

    LOG(3, "FBM Get Pool (%s) s(%d) w(%d) h(%d), p(%d), i(%d), r(%d)\n",
            FBM_GetPoolName(ucPoolType),
            prInfo->u4SourceType,
            prInfo->u4Width,
            prInfo->u4Height,
            prInfo->u4PipPop,
            prInfo->u4Interlace,
            prInfo->u4Resolution);

#if defined(CC_MT5895)
    if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV)
#ifdef CC_FBM_DYNMJC
         ||(SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_MJC)
         ||(SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_NOMJC)
#endif
       )
    {
        return _FbmGetPoolInfoAuto_MT5890_1GB(ucPoolType, prInfo);
    }
#endif

#if defined(CC_MT5893)
    if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV)
#ifdef CC_FBM_DYNMJC
            ||(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_NOMJC)
#endif
            ||(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_4K_MAINONLY)
       )
    {
        return _FbmGetPoolInfoAuto_MT5890_1GB(ucPoolType, prInfo);
    }
#endif

#if defined(CC_MT5886)
    if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV)
            ||(SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV_2CH)
       )
    {
        return _FbmGetPoolInfoAuto_MT5890_1GB(ucPoolType, prInfo);
    }
#endif

#if defined(CC_MT5882)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
        return _FbmGetPoolInfoAuto_MT5882_1GB(ucPoolType, prInfo);
    }
#endif
#if defined(CC_MT5863) || defined(CC_MT5865)
    if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
        return _FbmGetPoolInfoAuto_MT5863_1GB(ucPoolType, prInfo);
    }
#endif
#if defined(CC_MT5883)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
        return _FbmGetPoolInfoAuto_MT5883_1G(ucPoolType, prInfo);
    }
#endif

    else
    {
        LOG(0, "no fbm table fit \n");
        VERIFY(0);
    }
    return FBM_GetPoolInfo(ucPoolType);
}


//-------------------------------------------------------------------------
/** FBM_GetPoolCurrent
 *  Get Pool Current Use
 *  @param ucPoolType
 *  @return ucPoolType.
 */
//-------------------------------------------------------------------------

UCHAR FBM_GetPoolCurrent(UCHAR ucPoolType)
{
    if (ucPoolType == (UCHAR) FBM_POOL_TYPE_SCPOS_MAIN)
    {
        return (UCHAR) _eCurrentScposMain;
    }
    else if (ucPoolType == (UCHAR) FBM_POOL_TYPE_MJC_1)
    {
        return (UCHAR) _eCurrentMjc;
    }

    return ucPoolType;
}


//-------------------------------------------------------------------------
/** FBM_PrintPoolInfo
 *  Print FBM pool status
 *  @return NONE
 */
//-------------------------------------------------------------------------
#if defined(CC_MT5882)||defined(CC_MT5865)||defined(CC_MT5863)||defined(CC_MT5893)||defined(CC_MT5895)
VOID FBM_PrintPoolInfo(FBM_POOL_T* prPool, POOL_LIST_T* prPoolList)
{
    UCHAR ucIdx, ucLoop;

    for (ucLoop = 0; ucLoop < 3; ucLoop++)
    {
        for (ucIdx = 0; ucIdx < (UINT32)FBM_POOL_TYPE_NS; ucIdx++)
        {
            if ((((ucLoop == 0) && (prPool[ucIdx].u4Size > 0))&&(prPool[ucIdx].u4Inherit!= FBM_POOL_ROOT)) ||
                    (((ucLoop == 1) && (prPool[ucIdx].u4Size > 0))&&(prPool[ucIdx].u4Inherit == FBM_POOL_ROOT))||
                    ((ucLoop == 2) && (prPool[ucIdx].u4Size == 0)))
            {
#ifdef FBM_PRINT_STATUS_TOTAL
                LOG(0, "Addr(0x%09lx) Size(0x%08x) Total (0x%09lx) %s %s\n",
                            prPool[ucIdx].u4Addr,
                            prPool[ucIdx].u4Size,
                            (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                            prPoolList[ucIdx].szName,
                            prPool[ucIdx].fgAlloced ? "(Alloc)" : "");
#else
                if (prPool[ucIdx].u4Size > 0)
                {
                    LOG(0, "Addr(0x%09lx) Size(0x%08x(%d.%dM)) End(0x%09lx) Mode(%c, 0x%08x, %04d, %04d) %s %s\n",
                                prPool[ucIdx].u4Addr,
                                prPool[ucIdx].u4Size,
                                (prPool[ucIdx].u4Size/1024/1024),((prPool[ucIdx].u4Size*10)/(0x100000)%10),
                                (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                                (prPool[ucIdx].u4Inherit == FBM_POOL_ROOT)?'R':'I',
                                prPool[ucIdx].u4Mode,
                                prPool[ucIdx].u4Width,
                                prPool[ucIdx].u4Height,
                                prPoolList[ucIdx].szName,
                                prPool[ucIdx].fgAlloced ? "(Alloc)" : "");
                }
                else
                {
                    LOG(8, "Addr(0x%09lx) Size(0x%08x) End(0x%09lx) Mode(%c, 0x%08x, %04d, %04d) %s\n",
                                prPool[ucIdx].u4Addr,
                                prPool[ucIdx].u4Size,
                                (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                                (prPool[ucIdx].u4Inherit == FBM_POOL_ROOT)?'R':'I',
                                prPool[ucIdx].u4Mode,
                                prPool[ucIdx].u4Width,
                                prPool[ucIdx].u4Height,
                                prPoolList[ucIdx].szName);
                }
#endif
            }
        }
    }
#if defined(CC_TRUSTZONE_SUPPORT) && defined(CC_SVP_SUPPORT)
    for (ucIdx = 0; ucIdx < (UINT32)FBM_POOL_TYPE_TZ_NS; ucIdx++)
    {
        if (_arPoolTZ[ucIdx].u4Size > 0)
        {
            LOG(3, "[TrustZone]Addr(0x%09lx) Size(0x%08x) End(0x%09lx) %s %s\n",
                    _arPoolTZ[ucIdx].u4Addr,
                    _arPoolTZ[ucIdx].u4Size,
                    (_arPoolTZ[ucIdx].u4Addr + _arPoolTZ[ucIdx].u4Size),
                    (ucIdx == 0) ? "FBM_POOL_TYPE_TZ_DMX" : "FBM_POOL_TYPE_TZ_DMX2",
                    _arPoolTZ[ucIdx].fgAlloced ? "(Alloc)" : "");

        }
    }
#endif
    LOG(1, "Current pool (SCPOS-M=%s, SCPOS-P=%s, MDDI-M=%s, MDDI-P=%s, TDC=%s)\n",
            FBM_GetPoolName((UCHAR)_eCurrentScposMain),
            FBM_GetPoolName((UCHAR)_eCurrentScposPip),
            FBM_GetPoolName((UCHAR)_eCurrentMddiMain),
            FBM_GetPoolName((UCHAR)_eCurrentMddiPip),
            FBM_GetPoolName((UCHAR)_eCurrentTdc));

    LOG(1, "Current pool (MJC=%s, VBI=%s)\n",
            FBM_GetPoolName((UCHAR)_eCurrentMjc),
            FBM_GetPoolName((UCHAR)_eCurrentVbi));

#if defined(CC_MT5882)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        LOG(0, "FBM_MEM_CFG_MT5882_TVP_3DTV Model\n");
#else
        LOG(0, "FBM_MEM_CFG_MT5882_3DTV Model\n");
#endif
    }
#endif
#if defined(CC_MT5863)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        LOG(0, "FBM_MEM_CFG_MT5863_TVP_3DTV Model\n");
#else
        LOG(0, "FBM_MEM_CFG_MT5863_3DTV Model\n");
#endif
    }
#endif
#if defined(CC_MT5883)
    if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        LOG(0, "FBM_MEM_CFG_MT5883_TVP_3DTV Model\n");
#else
        LOG(0, "FBM_MEM_CFG_MT5883_3DTV Model\n");
#endif
    }
#endif

#if defined(CC_MT5895)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV Model\n");
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_MJC)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV_MJC Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_NOMJC)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV_NOMJC Model\n");
    }
#endif
#endif

#if defined(CC_MT5893)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV Model\n");
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_NOMJC)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV_NOMJC Model\n");
    }
#endif
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_4K_MAINONLY)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV_4K_Main_Only Model\n");
    }
#endif

    LOG(9, "0x%08x = FBM_POOL_MODE_8BIT\n", FBM_POOL_MODE_8BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_10BIT\n", FBM_POOL_MODE_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_C_10BIT\n", FBM_POOL_MODE_C_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_420_FB\n", FBM_POOL_MODE_420_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_422_FB\n", FBM_POOL_MODE_422_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_444_FB\n", FBM_POOL_MODE_444_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SD\n", FBM_POOL_MODE_SD);
    LOG(9, "0x%08x = FBM_POOL_MODE_720HD\n", FBM_POOL_MODE_720HD);
    LOG(9, "0x%08x = FBM_POOL_MODE_1080HD\n", FBM_POOL_MODE_1080HD);
    LOG(9, "0x%08x = FBM_POOL_MODE_SHARE\n", FBM_POOL_MODE_SHARE);
    LOG(9, "0x%08x = FBM_POOL_MODE_TOGGLE_TRIGGER\n", FBM_POOL_MODE_TOGGLE_TRIGGER);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_NO_COMP\n", FBM_POOL_MODE_MPEG_NO_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_COMP_75\n", FBM_POOL_MODE_MPEG_COMP_75);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_COMP_50\n", FBM_POOL_MODE_MPEG_COMP_50);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_DBK\n", FBM_POOL_MODE_MPEG_DBK);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_DROP\n", FBM_POOL_MODE_MJC_DROP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_NO_DROP\n", FBM_POOL_MODE_MJC_NO_DROP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_NO_COMP\n", FBM_POOL_MODE_MJC_NO_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_Y_COMP4\n", FBM_POOL_MODE_MJC_Y_COMP4);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_Y_COMP3\n", FBM_POOL_MODE_MJC_Y_COMP3);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_3D_Y_C\n", FBM_POOL_MODE_NR_3D_Y_C);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_3D_Y_ONLY\n", FBM_POOL_MODE_NR_3D_Y_ONLY);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_2D\n", FBM_POOL_MODE_NR_2D);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_BYPASS\n", FBM_POOL_MODE_NR_BYPASS);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_1FB\n", FBM_POOL_MODE_SCPOS_1FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_2FB\n", FBM_POOL_MODE_SCPOS_2FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_3FB\n", FBM_POOL_MODE_SCPOS_3FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_DISP\n", FBM_POOL_MODE_SCPOS_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_FULL\n", FBM_POOL_MODE_SCPOS_FULL);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_PSCAN_DISP\n", FBM_POOL_MODE_SCPOS_PSCAN_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_MJC_DISP\n", FBM_POOL_MODE_SCPOS_MJC_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_LINESYNC_DISP\n", FBM_POOL_MODE_SCPOS_LINESYNC_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_BOB\n", FBM_POOL_MODE_MDDI_BOB);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_FULL\n", FBM_POOL_MODE_MDDI_FULL);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_CBOB\n", FBM_POOL_MODE_MDDI_CBOB);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_Y_C\n", FBM_POOL_MODE_MDDI_NR_Y_C);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_Y_ONLY\n", FBM_POOL_MODE_MDDI_NR_Y_ONLY);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_OFF\n", FBM_POOL_MODE_MDDI_NR_OFF);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_10BIT\n", FBM_POOL_MODE_MDDI_NR_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_COMP\n", FBM_POOL_MODE_MDDI_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_DISP\n", FBM_POOL_MODE_MDDI_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_P2I\n", FBM_POOL_MODE_MDDI_P2I);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_NORMAL\n", FBM_TV_MODE_TYPE_NORMAL);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_PIP\n", FBM_TV_MODE_TYPE_PIP);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_POP\n", FBM_TV_MODE_TYPE_POP);

    LOG(5, "NPTV Main Begin Addr:          0x%08x\n", _fbm_u4NptvBeAddr);
}

#else
VOID FBM_PrintPoolInfo(FBM_POOL_T* prPool, POOL_LIST_T* prPoolList)
{
    UCHAR ucIdx, ucLoop,bchannelA=0,bchannelB=0,bchannelc=0;

    for (ucLoop = 0; ucLoop < 3; ucLoop++)
    {
        for (ucIdx = 0; ucIdx < (UINT32)FBM_POOL_TYPE_NS; ucIdx++)
        {
            if ((((ucLoop == 0) && (prPool[ucIdx].u4Addr < DRAM_CHB_START_ADDR)) ||
                        ((ucLoop == 1) &&((prPool[ucIdx].u4Addr >= DRAM_CHB_START_ADDR)&&(prPool[ucIdx].u4Addr < DRAM_CHC_START_ADDR)))
                        ||((ucLoop == 2)&&(prPool[ucIdx].u4Addr >= DRAM_CHC_START_ADDR))
                )
               )
            {
                if((ucLoop == 0) && (bchannelA==0)&&((prPool[ucIdx].u4Addr > 0)&&(prPool[ucIdx].u4Addr < DRAM_CHB_START_ADDR)))
                {
                    LOG(0,"-------------CHANNEL A ----------------\n");
                    bchannelA =1;
                }
                else if((ucLoop == 1) && (bchannelB==0)&&((prPool[ucIdx].u4Addr >= DRAM_CHB_START_ADDR)&&(prPool[ucIdx].u4Addr < DRAM_CHC_START_ADDR)))
                {
                    LOG(0,"-------------CHANNEL B ---------------- \n");
                    bchannelB =1;
                }
                else if((ucLoop == 2) && (bchannelc==0)&&(prPool[ucIdx].u4Addr >= DRAM_CHC_START_ADDR)&&!IS_IC_5861())
                {
                    LOG(0,"-------------CHANNEL C ---------------- \n");
                    bchannelc =1;
                }
#ifdef FBM_PRINT_STATUS_TOTAL
                LOG(0, "Addr(0x%08x) Size(0x%08x) Total (0x%08x) %s %s\n",
                        prPool[ucIdx].u4Addr,
                        prPool[ucIdx].u4Size,
                        (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                        prPoolList[ucIdx].szName,
                        prPool[ucIdx].fgAlloced ? "(Alloc)" : "");
#else
                if (prPool[ucIdx].u4Size > 0)
                {
                    LOG(0, "Addr(0x%08x) Size(0x%08x(%d.%dM)) End(0x%08x) Mode(%c, 0x%08x, %04d, %04d) %s %s\n",
                            prPool[ucIdx].u4Addr,
                            prPool[ucIdx].u4Size,
                            (prPool[ucIdx].u4Size/1024/1024),((prPool[ucIdx].u4Size*10)/(0x100000)%10),
                            (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                            (prPool[ucIdx].u4Inherit == FBM_POOL_ROOT)?'R':'I',
                            prPool[ucIdx].u4Mode,
                            prPool[ucIdx].u4Width,
                            prPool[ucIdx].u4Height,
                            prPoolList[ucIdx].szName,
                            prPool[ucIdx].fgAlloced ? "(Alloc)" : "");
                }
                else
                {
                    LOG(8, "Addr(0x%08x) Size(0x%08x) End(0x%08x) Mode(%c, 0x%08x, %04d, %04d) %s\n",
                            prPool[ucIdx].u4Addr,
                            prPool[ucIdx].u4Size,
                            (prPool[ucIdx].u4Addr + prPool[ucIdx].u4Size),
                            (prPool[ucIdx].u4Inherit == FBM_POOL_ROOT)?'R':'I',
                            prPool[ucIdx].u4Mode,
                            prPool[ucIdx].u4Width,
                            prPool[ucIdx].u4Height,
                            prPoolList[ucIdx].szName);
                }

            }
#endif
        }
    }
#if defined(CC_TRUSTZONE_SUPPORT) && defined(CC_SVP_SUPPORT)
    for (ucIdx = 0; ucIdx < (UINT32)FBM_POOL_TYPE_TZ_NS; ucIdx++)
    {
        if (_arPoolTZ[ucIdx].u4Size > 0)
        {
            LOG(3, "[TrustZone]Addr(0x%08x) Size(0x%08x) End(0x%08x) %s %s\n",
                    _arPoolTZ[ucIdx].u4Addr,
                    _arPoolTZ[ucIdx].u4Size,
                    (_arPoolTZ[ucIdx].u4Addr + _arPoolTZ[ucIdx].u4Size),
                    (ucIdx == 0) ? "FBM_POOL_TYPE_TZ_DMX" : "FBM_POOL_TYPE_TZ_DMX2",
                    _arPoolTZ[ucIdx].fgAlloced ? "(Alloc)" : "");

        }
    }
#endif
    LOG(0, "Current pool (SCPOS-M=%s, SCPOS-P=%s, MDDI-M=%s, MDDI-P=%s, TDC=%s)\n",
            FBM_GetPoolName((UCHAR)_eCurrentScposMain),
            FBM_GetPoolName((UCHAR)_eCurrentScposPip),
            FBM_GetPoolName((UCHAR)_eCurrentMddiMain),
            FBM_GetPoolName((UCHAR)_eCurrentMddiPip),
            FBM_GetPoolName((UCHAR)_eCurrentTdc));

    LOG(0, "Current pool (MJC=%s, VBI=%s)\n",
            FBM_GetPoolName((UCHAR)_eCurrentMjc),
            FBM_GetPoolName((UCHAR)_eCurrentVbi));

#if defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5891)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5890_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_4K)
    {
        LOG(0, "FBM_MEM_CFG_MT5890_3DTV_4K Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_FHD)
    {
        LOG(0, "FBM_MEM_CFG_MT5890_3DTV_FHD Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5861_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5861_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_2K80)
    {
        LOG(0, "FBM_MEM_CFG_MT5890_3DTV_2K80 Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5891_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV_4K)
    {
        LOG(0, "FBM_MEM_CFG_MT5891_3DTV_4K Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV_2CH)
    {
        LOG(0, "FBM_MEM_CFG_MT5891_3DTV_2CH Model\n");
    }
#endif

#if defined(CC_MT5895)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV Model\n");
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_MJC)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV_MJC Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_NOMJC)
    {
        LOG(0, "FBM_MEM_CFG_MT5895_3DTV_NOMJC Model\n");
    }
#endif
#endif

#if defined(CC_MT5893)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV Model\n");
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV_NOMJC Model\n");
    }
#endif
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_4K_MAINONLY)
    {
        LOG(0, "FBM_MEM_CFG_MT5893_3DTV_4K_Main_Only Model\n");
    }
#endif

#if defined(CC_MT5886)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5886_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV_2CH)
    {
        LOG(0, "FBM_MEM_CFG_MT5886_3DTV_2CH Model\n");
    }
#endif

#if defined(CC_MT5882)||defined(CC_MT5883) || defined(CC_MT5863)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5882_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5883_3DTV Model\n");
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
        LOG(0, "FBM_MEM_CFG_MT5863_3DTV Model\n");
    }
#endif

    LOG(9, "0x%08x = FBM_POOL_MODE_8BIT\n", FBM_POOL_MODE_8BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_10BIT\n", FBM_POOL_MODE_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_C_10BIT\n", FBM_POOL_MODE_C_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_420_FB\n", FBM_POOL_MODE_420_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_422_FB\n", FBM_POOL_MODE_422_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_444_FB\n", FBM_POOL_MODE_444_FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SD\n", FBM_POOL_MODE_SD);
    LOG(9, "0x%08x = FBM_POOL_MODE_720HD\n", FBM_POOL_MODE_720HD);
    LOG(9, "0x%08x = FBM_POOL_MODE_1080HD\n", FBM_POOL_MODE_1080HD);
    LOG(9, "0x%08x = FBM_POOL_MODE_SHARE\n", FBM_POOL_MODE_SHARE);
    LOG(9, "0x%08x = FBM_POOL_MODE_TOGGLE_TRIGGER\n", FBM_POOL_MODE_TOGGLE_TRIGGER);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_NO_COMP\n", FBM_POOL_MODE_MPEG_NO_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_COMP_75\n", FBM_POOL_MODE_MPEG_COMP_75);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_COMP_50\n", FBM_POOL_MODE_MPEG_COMP_50);
    LOG(9, "0x%08x = FBM_POOL_MODE_MPEG_DBK\n", FBM_POOL_MODE_MPEG_DBK);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_DROP\n", FBM_POOL_MODE_MJC_DROP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_NO_DROP\n", FBM_POOL_MODE_MJC_NO_DROP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_NO_COMP\n", FBM_POOL_MODE_MJC_NO_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_Y_COMP4\n", FBM_POOL_MODE_MJC_Y_COMP4);
    LOG(9, "0x%08x = FBM_POOL_MODE_MJC_Y_COMP3\n", FBM_POOL_MODE_MJC_Y_COMP3);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_3D_Y_C\n", FBM_POOL_MODE_NR_3D_Y_C);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_3D_Y_ONLY\n", FBM_POOL_MODE_NR_3D_Y_ONLY);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_2D\n", FBM_POOL_MODE_NR_2D);
    LOG(9, "0x%08x = FBM_POOL_MODE_NR_BYPASS\n", FBM_POOL_MODE_NR_BYPASS);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_1FB\n", FBM_POOL_MODE_SCPOS_1FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_2FB\n", FBM_POOL_MODE_SCPOS_2FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_3FB\n", FBM_POOL_MODE_SCPOS_3FB);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_DISP\n", FBM_POOL_MODE_SCPOS_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_FULL\n", FBM_POOL_MODE_SCPOS_FULL);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_PSCAN_DISP\n", FBM_POOL_MODE_SCPOS_PSCAN_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_MJC_DISP\n", FBM_POOL_MODE_SCPOS_MJC_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_SCPOS_LINESYNC_DISP\n", FBM_POOL_MODE_SCPOS_LINESYNC_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_BOB\n", FBM_POOL_MODE_MDDI_BOB);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_FULL\n", FBM_POOL_MODE_MDDI_FULL);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_CBOB\n", FBM_POOL_MODE_MDDI_CBOB);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_Y_C\n", FBM_POOL_MODE_MDDI_NR_Y_C);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_Y_ONLY\n", FBM_POOL_MODE_MDDI_NR_Y_ONLY);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_OFF\n", FBM_POOL_MODE_MDDI_NR_OFF);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_NR_10BIT\n", FBM_POOL_MODE_MDDI_NR_10BIT);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_COMP\n", FBM_POOL_MODE_MDDI_COMP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_DISP\n", FBM_POOL_MODE_MDDI_DISP);
    LOG(9, "0x%08x = FBM_POOL_MODE_MDDI_P2I\n", FBM_POOL_MODE_MDDI_P2I);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_NORMAL\n", FBM_TV_MODE_TYPE_NORMAL);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_PIP\n", FBM_TV_MODE_TYPE_PIP);
    LOG(9, "0x%08x = FBM_TV_MODE_TYPE_POP\n", FBM_TV_MODE_TYPE_POP);
#if !defined(CC_MT5881) && !defined(CC_MT5883)
    LOG(5, "Main pip: (EndAddr,Size,Mode): 0x%08x, 0x%08x, 0x%08x\n", _FBM_NPTV_VideoRange.u4MainPipMaxEnd, _FBM_NPTV_VideoRange.u4MainPipMaxSize, _FBM_NPTV_VideoRange.u1MainPipMaxMode);
    LOG(5, "Sub  pip: (EndAddr,Size,Mode): 0x%08x, 0x%08x, 0x%08x\n", _FBM_NPTV_VideoRange.u4SubPipMaxEnd, _FBM_NPTV_VideoRange.u4SubPipMaxSize, _FBM_NPTV_VideoRange.u1SubPipMaxMode);
    LOG(5, "Main pop: (EndAddr,Size,Mode): 0x%08x, 0x%08x, 0x%08x\n", _FBM_NPTV_VideoRange.u4MainPopMaxEnd, _FBM_NPTV_VideoRange.u4MainPopMaxSize, _FBM_NPTV_VideoRange.u1MainPopMaxMode);
    LOG(5, "Sub  pop: (EndAddr,Size,Mode): 0x%08x, 0x%08x, 0x%08x\n", _FBM_NPTV_VideoRange.u4SubPopMaxEnd, _FBM_NPTV_VideoRange.u4SubPopMaxSize, _FBM_NPTV_VideoRange.u1SubPopMaxMode);
#endif
    LOG(5, "NPTV Main Begin Addr:          0x%08x\n", _fbm_u4NptvBeAddr);
}
#endif
//-------------------------------------------------------------------------
/** FBM_ConfigurePool
 *  Initialize this FBM pool
 *  @retval NONE
 */
//-------------------------------------------------------------------------
VOID FBM_ConfigurePool(POOL_LIST_T* prPoolList, POOL_ARRANGE_INFO_T* prPoolArrangeInfo)
{
#if defined(CC_MT5399)||defined(CC_MT5890)||defined(CC_MT5891)
    if ((SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV)
       )
    {
        _FbmConfigurePool_MT5890_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_4K)
    {
        _FbmConfigurePool_MT5890_4K_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_FHD)
    {
        _FbmConfigurePool_MT5890_FHD_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5890_3DTV_2K80)
    {
        _FbmConfigurePool_MT5890_2K80_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5861_3DTV)
    {
        _FbmConfigurePool_MT5861_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV)
    {
        _FbmConfigurePool_MT5891_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV_4K)
    {
        _FbmConfigurePool_MT5891_4K_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5891_3DTV_2CH)
    {
        _FbmConfigurePool_MT5891_2CH_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }

#endif

#if defined(CC_MT5895)
    if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV)
    {
        _FbmConfigurePool_MT5895_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_MJC)
    {
        _FbmConfigurePool_MT5895_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5895_3DTV_NOMJC)
    {
        _FbmConfigurePool_MT5895_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
#endif
#endif

#if defined(CC_MT5893)
    if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV)
    {
        _FbmConfigurePool_MT5893_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
#ifdef CC_FBM_DYNMJC
    else if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_NOMJC)
    {
        _FbmConfigurePool_MT5893_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
#endif
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5893_3DTV_4K_MAINONLY)
    {
        _FbmConfigurePool_MT5893_4KMainOnly_MAIN(prPoolList, prPoolArrangeInfo);
    }
#endif

#if defined(CC_MT5886)
    if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV)
    {
        _FbmConfigurePool_MT5886_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
    else if(SRMFBM_GetConf() == FBM_MEM_CFG_MT5886_3DTV_2CH)
    {
        _FbmConfigurePool_MT5886_2CH_MJC_SCPOS_MAIN(prPoolList, prPoolArrangeInfo);
    }
#endif

#if defined(CC_MT5882)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5882_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        _FbmConfigurePool_MT5882_TVP_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#else
        _FbmConfigurePool_MT5882_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#endif
    }
#endif
#if defined(CC_MT5863) || defined(CC_MT5865)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5863_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        _FbmConfigurePool_MT5863_TVP_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#else
        _FbmConfigurePool_MT5863_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#endif
    }
#endif
#if defined(CC_MT5883)
    if (SRMFBM_GetConf() == FBM_MEM_CFG_MT5883_3DTV)
    {
#if defined(CC_TVP_SUPPORT)
        _FbmConfigurePool_MT5883_TVP_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#else
        _FbmConfigurePool_MT5883_SCPOS_MAIN( prPoolList,prPoolArrangeInfo);
#endif
    }
#endif
    else
    {
        VERIFY(0);
    }
}

